#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

replace_file() {
    # Following awk program read each string and replace each environment variable
    # ($... or ${...}) with its value if defined.
    # If not defined, the environment variable expression is not changed.
    cat $1 | awk '
        {
            finished="false"
            input=$0
            output=""
            while (finished == "false") {
                # Find environment variable expression: $... or ${...}
                # var_name will contain the matching regexp in [0]
                # and just the variable name in [1]
                match(input, "\\${?(\\w+)}?", var_name)
                
                # if variable expression was found
                if ( RLENGTH != -1 ) {
                    # try to get value from environment
                    var_content=ENVIRON[var_name[1]]
                    # if exist in environemt
                    if ( length(var_content) > 0 ) {
                        # copy string before variable
                        output=output""substr(input, 1, RSTART-1)
                        # replace variable with its value
                        output=output""var_content
                    } else {
                        # if variable is not defined, copy string including variable
                        output=output""substr(input, 1, RSTART+RLENGTH-1)
                    }
                    
                    # set input as remaining of the string
                    input=substr(input, RSTART+RLENGTH)
                    
                } else {
                    # no variable expression found; copy remaining of the string and end loop
                    output=output""input
                    finished="true"
                }
            }
            
            # print resulting string
            print output
        }
    ' > $2
}


# Complete cdmw_env with current path at configure.sh time then source it
sed s/@date@/"\`date\`"/ cdmw_env >cdmw_env.tmp
sed s/@CDMW_HOME@/"\\\$PWD\\/delivery"/ cdmw_env.tmp >cdmw_env
rm -f cdmw_env.tmp
. cdmw_env

export LOGNAME=_
export TMPDIR=

        
# find files with environment variables expressions
for i in `grep -Il "\\(\\${\\w\\w*}\\)\\|\\(\\$\\w\\w*\\)" delivery/share/*`; do
    tmpfile=`mktemp`
    replace_file $i $tmpfile
    mv -f $tmpfile $i
done


