#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

min_cpu_num=2
min_mem_size=512
min_mem_swap_size=1024
                                        
check_host_cpu_number()
{
   mb_host_cpu_numbers=`cat /proc/cpuinfo | grep 'processor' | wc -l`
   if [ -z "${mb_host_cpu_numbers}" ]
   then
      echo "Number of Processors not found"
   else
      echo "Number of Processors: ${mb_host_cpu_numbers}"
      result=`expr ${min_cpu_num} \> ${mb_host_cpu_numbers}`
      if [ "${result}" = "1" ]
      then
         echo "############################################################################################"
         echo "Warning: the number of processors should be at least equal to ${min_cpu_num}"
         echo "############################################################################################"
      fi
   fi
}

check_host_cpu_speed()
{
   mb_host_cpu_speed=`cat /proc/cpuinfo | grep 'cpu MHz' |sed -r -e 's/\s*cpu\s+MHz\s*:\s*//g' | sort | uniq`
   if [ -z "${mb_host_cpu_speed}" ]
   then
      echo "CPU Speed not found"
   else
      echo "CPU Speed: ${mb_host_cpu_speed} MHz"
   fi
}

check_host_mem_size()
{
   mb_host_mem_size=`echo $(($(cat /proc/meminfo | grep 'MemTotal:' |sed -r -e 's/\s*MemTotal\s*:\s*//g' |sed -r -e 's/\s*kB//g' ) / 1024))`
   if [ -z "${mb_host_mem_size}" ]
   then
      echo "MEM Size not found"
   else
      echo "MEM Size: ${mb_host_mem_size} MB"
      result=`expr ${min_mem_size} \> ${mb_host_mem_size}`
      if [ "${result}" = "1" ]
      then
         echo "############################################################################################"
         echo "Warning: the total amount of physical memory in the system should be at least equal to ${min_mem_size} MB"
         echo "############################################################################################"
      fi
   fi
}

check_host_mem_swap_size()
{
   mb_host_mem_swap_size=`echo $(($(cat /proc/meminfo | grep 'SwapTotal:' |sed -r -e 's/\s*SwapTotal\s*:\s*//g' |sed -r -e 's/\s*kB//g' ) / 1024))`
   if [ -z "${mb_host_mem_swap_size}" ]
   then
      echo "SWAP Size not found"
   else
      echo "SWAP Size: ${mb_host_mem_swap_size} MB"
      result=`expr ${min_mem_swap_size} \> ${mb_host_mem_swap_size}`
      if [ "${result}" = "1" ]
      then
         echo "############################################################################################"
         echo "Warning: the total amount of swap memory in the system should be at least equal to ${min_mem_swap_size} MB"
         echo "############################################################################################"
      fi
   fi
}


check_host_disk_info()
{
   mb_host_disk_info=` df -hl -B MB . | grep -vE '^Filesystem'|awk '{ print $4 }' `
   if [ -z "${mb_host_disk_info}" ]
   then
      echo "Available disk space not found"
   else
      echo "Available disk space in current partition: ${mb_host_disk_info}"
   fi
}


#MAIN
echo
echo
check_host_cpu_number
check_host_cpu_speed
check_host_mem_size
check_host_mem_swap_size
check_host_disk_info
