#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

check_ace() {
    # if an argument is provided, use it as ACE_ROOT
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$ACE_ROOT
    fi
    
    echo "checking ACE_ROOT directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$ACE_ROOT is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "ACE_ROOT=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $ACE_ROOT/lib/libACE.so exist
    if [ ! -f ${TO_CHECK}/lib/libACE.so ]; then
        echo "ACE_ROOT=\"${TO_CHECK}\" doesn't seems to contain ACE"
        echo "(${TO_CHECK}/lib/libACE.so not found)"
        return 1
    fi
    
    # check if $ACE_ROOT/lib/libTAO.so exist
    if [ ! -f ${TO_CHECK}/lib/libTAO.so ]; then
        echo "ACE_ROOT=\"${TO_CHECK}\" doesn't seems to contain ACE"
        echo "(${TO_CHECK}/lib/libTAO.so not found)"
        return 1
    fi

    # check if $ACE_ROOT/bin/tao_idl exist
    if [ ! -f ${TO_CHECK}/bin/tao_idl ]; then
        echo "ACE_ROOT=\"${TO_CHECK}\" doesn't seems to contain ACE"
        echo "(${TO_CHECK}/bin/tao_idl not found)"
        return 1
    fi
    
    # check if $ACE_ROOT/bin/mpc.pl exist
    if [ ! -f ${TO_CHECK}/bin/mpc.pl ]; then
        echo "ACE_ROOT=\"${TO_CHECK}\" doesn't seems to contain ACE"
        echo "(${TO_CHECK}/bin/mpc.pl not found)"
        return 1
    fi
}


check_tao() {
    # if an argument is provided, use it as TAO_ROOT
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$TAO_ROOT
    fi
    
    echo "checking TAO_ROOT directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$TAO_ROOT is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "TAO_ROOT=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $TAO_ROOT/idl/tao/orb.idl exist
    if [ ! -f ${TO_CHECK}/idl/tao/orb.idl ]; then
         echo "TAO_ROOT=\"${TO_CHECK}\" doesn't seems to contain TAO"
        echo "(${TO_CHECK}/idl/tao/orb.idl not found)"
        return 1
    fi
    
    # check if $TAO_ROOT/idl/orbsvcs/ exist
    if [ ! -d ${TO_CHECK}/idl/orbsvcs ]; then
         echo "TAO_ROOT=\"${TO_CHECK}\" doesn't seems to contain TAO"
         echo "(${TO_CHECK}/idl/orbsvcs/ not found)"
        return 1
    fi
}


check_ant() {
    # if an argument is provided, use it as ANT_HOME
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$ANT_HOME
    fi
    
    echo "checking ANT_HOME directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$ANT_HOME is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "ANT_HOME=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $ANT_HOME/bin/ant exist
    if [ ! -f ${TO_CHECK}/bin/ant ]; then
        echo "ANT_HOME=\"${TO_CHECK}\" doesn't seems to contain Ant"
        echo "(${TO_CHECK}/bin/ant not found)"
        return 1
    fi
    
    # check if $ANT_HOME/lib/ant.jar exist
    if [ ! -f ${TO_CHECK}/lib/ant.jar ]; then
        echo "ANT_HOME=\"${TO_CHECK}\" doesn't seems to contain Ant"
        echo "(${TO_CHECK}/bin/ant.jar not found)"
        return 1
    fi
    
    # check if $ANT_HOME/lib/ant-contrib-1.0b3.jar exist
    if [ ! -f ${TO_CHECK}/lib/ant-contrib-1.0b3.jar ]; then
        echo "ANT_HOME=\"${TO_CHECK}\" doesn't seems to contain Ant Contrib"
        echo "(${TO_CHECK}/bin/ant-contrib-1.0b3.jar not found)"
        return 1
    fi
}


check_java() {
    # if an argument is provided, use it as JAVA_HOME
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$JAVA_HOME
    fi
    
    echo "checking JAVA_HOME directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$JAVA_HOME is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "JAVA_HOME=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $JAVA_HOME/include/jni.h exist (required for JNI)
    if [ ! -f ${TO_CHECK}/include/jni.h ]; then
        echo "JAVA_HOME=\"${TO_CHECK}\" doesn't seems to contain Java"
        echo "(${TO_CHECK}/include/jni.h not found)"
        return 1
    fi
    
    # check if $JAVA_HOME/include/linux/ exist (required for JNI)
    if [ ! -d ${TO_CHECK}/include/linux ]; then
        echo "JAVA_HOME=\"${TO_CHECK}\" doesn't seems to contain Java"
        echo "(${TO_CHECK}/include/linux/ not found)"
        return 1
    fi
}


check_jacorb() {
    # if an argument is provided, use it as JACORB_HOME
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$JACORB_HOME
    fi
    
    echo "checking JACORB_HOME directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$JACORB_HOME is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "JACORB_HOME=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $JACORB_HOME/lib/endorsed/jacorb.jar exist
    if [ ! -f ${TO_CHECK}/lib/endorsed/jacorb.jar ]; then
        echo "JACORB_HOME=\"${TO_CHECK}\" doesn't seems to contain JacORB"
        echo "(${TO_CHECK}/lib/endorsed/jacorb.jar not found)"
        return 1
    fi
    
    # check if $JACORB_HOME/bin/jaco exist
    if [ ! -f ${TO_CHECK}/bin/jaco ]; then
        echo "JACORB_HOME=\"${TO_CHECK}\" doesn't seems to contain JacORB"
        echo "(${TO_CHECK}/bin/jaco not found)"
        return 1
    fi
}


check_cppunit() {
    # if an argument is provided, use it as CPPUNIT_ROOT
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$CPPUNIT_ROOT
    fi
    
    echo "checking CPPUNIT_ROOT directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$CPPUNIT_ROOT is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "CPPUNIT_ROOT=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $CPPUNIT_ROOT/lib/libcppunit.so exist
    if [ ! -f ${TO_CHECK}/lib/libcppunit.so ]; then
        echo "CPPUNIT_ROOT=\"${TO_CHECK}\" doesn't seems to contain CPPUnit"
        echo "(${TO_CHECK}/lib/libcppunit.so not found)"
        return 1
    fi
    
    # check if $CPPUNIT_ROOT/include/cppunit/ exist
    if [ ! -d ${TO_CHECK}/include/cppunit ]; then
        echo "CPPUNIT_ROOT=\"${TO_CHECK}\" doesn't seems to contain CPPUnit"
        echo "(${TO_CHECK}/include/cppunit/ not found)"
        return 1
    fi
}


check_junit() {
    # if an argument is provided, use it as JUNIT_HOME
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$JUNIT_HOME
    fi
    
    echo "checking JUNIT_HOME directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$JUNIT_HOME is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "JUNIT_HOME=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $JUNIT_HOME/junit-4.1.jar exist
    if [ ! -f ${TO_CHECK}/junit-4.1.jar ]; then
        echo "JUNIT_HOME=\"${TO_CHECK}\" doesn't seems to contain JUnit"
        echo "(${TO_CHECK}/junit-4.1.jar not found)"
        return 1
    fi
}


check_saxon() {
    # if an argument is provided, use it as SAXON_HOME
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$SAXON_HOME
    fi
    
    echo "checking SAXON_HOME directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$SAXON_HOME is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "SAXON_HOME=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $SAXON_HOME/saxon.jar exist
    if [ ! -f ${TO_CHECK}/saxon.jar ]; then
        echo "SAXON_HOME=\"${TO_CHECK}\" doesn't seems to contain Saxon"
        echo "(${TO_CHECK}/saxon.jar not found)"
        return 1
    fi
}


check_xercesc() {
    # if an argument is provided, use it as XERCESCROOT
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$XERCESCROOT
    fi
    
    echo "checking XERCESCROOT directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$XERCESCROOT is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "\XERCESCROOT=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $XERCESCROOT/lib/libxerces-c.so exist
    if [ ! -f ${TO_CHECK}/lib/libxerces-c.so ]; then
        echo "XERCESCROOT=\"${TO_CHECK}\" doesn't seems to contain XercesC"
        echo "(${TO_CHECK}/lib/libxerces-c.so not found)"
        return 1
    fi
    
    # check if $XERCESCROOT/include/xercesc/ exist
    if [ ! -d ${TO_CHECK}/include/xercesc ]; then
        echo "XERCESCROOT=\"${TO_CHECK}\" doesn't seems to contain XercesC"
        echo "(${TO_CHECK}/include/xercesc/ not found)"
        return 1
    fi
}

check_emma() {
    # if an argument is provided, use it as EMMA_HOME
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$EMMA_HOME
    fi
    
    echo "checking EMMA_HOME directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$EMMA_HOME is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "\EMMA_HOME=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $EMMA_HOME/lib/emma_ant.jar exist
    if [ ! -f ${TO_CHECK}/lib/emma_ant.jar ]; then
        echo "EMMA_HOME=\"${TO_CHECK}\" doesn't seems to contain Emma"
        echo "(${TO_CHECK}/lib/emma_ant.jar not found)"
        return 1
    fi
    
    # check if $EMMA_HOME/lib/emma.jar exist
    if [ ! -f ${TO_CHECK}/lib/emma.jar ]; then
        echo "EMMA_HOME=\"${TO_CHECK}\" doesn't seems to contain Emma"
        echo "(${TO_CHECK}/lib/emma.jar not found)"
        return 1
    fi
}


check_codegenerator() {
    # if an argument is provided, use it as EMMA_HOME
    if [ -n "$1" ]; then
        TO_CHECK=$1
    else
        TO_CHECK=$CODEGENERATOR_HOME
    fi
    
    echo "checking CODEGENERATOR_HOME directory: $TO_CHECK"
    
    if [ -z "$TO_CHECK" ]; then
        echo "\$CODEGENERATOR_HOME is not defined"
        return 1
    fi
    if [ ! -d "$TO_CHECK" ]; then
        echo "\CODEGENERATOR_HOME=\"${TO_CHECK}\" is not a directory"
        return 1
    fi
    
    # check if $EMMA_HOME/lib/emma_ant.jar exist
     #FIXME set the right jar 
    if [ ! -f ${TO_CHECK}/com.thalesgroup.cdmw.codegenerator.jar ]; then
        echo "\$CODEGENERATOR_HOME=\"${TO_CHECK}\" doesn't seems to contain Codegenerator" 
        echo "(${TO_CHECK}/com.thalesgroup.cdmw.codegenerator.jar not found)"
        return 1
    fi
}

