#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

SCRIPT_NAME=`basename $0`
SCRIPT_PATH=`dirname $0`

# compute CDMW_HOME from $SCRIPT_PATH
CDMW_HOME=`cd $SCRIPT_PATH/../..; pwd`

# Configuration file (previous if exist will be renamed)
CDMW_CONF_FILE=$CDMW_HOME/cdmw_env

print_usage() {
    echo "usage: $SCRIPT_NAME [-Dproperty=value] [previous_conf_file]"
    echo "   or: $SCRIPT_NAME [--help]"
}


for i in $*
do
  if [ "${i:0:1}" != "-" ]
  then 
    if [ ! -f $i ]; then
        echo "ERROR: $i is not a valid file"
        print_usage
        exit 1
    fi
    
    CDMW_PREVIOUS_CONF=$i  
  fi
done


# If a previous configuration file was specified, load it
if [ -n "$CDMW_PREVIOUS_CONF" ]; then
    echo "load previous configuration file: $CDMW_PREVIOUS_CONF"
    . $CDMW_PREVIOUS_CONF
else
    # If a configuration file already exists, load it
    if [ -f $CDMW_CONF_FILE ]; then
        echo "load previous configuration file: $CDMW_CONF_FILE"
        . $CDMW_CONF_FILE
    fi
fi

# Default values
CDMW_CPP_GCOV=0
CDMW_JAVA_EMMA=0


while getopts D:-: option
do
  case "$option" in     
  D)
     property=`expr "$OPTARG" : '\([^=]*\)=.*'`
     value=`expr "$OPTARG" : '.*=\([^=]*\)'`
     case "$property" in
     generate.gcov.infos)
       case "$value" in
            true)
                export CDMW_CPP_GCOV=1
                ;;
            *)
                export CDMW_CPP_GCOV=0
                ;;
        esac
    ;;
     generate.emma.infos)
       case "$value" in
            true)
                export CDMW_JAVA_EMMA=1
                ;;
            *)
                export CDMW_JAVA_EMMA=0
                ;;
        esac
    ;;
      esac
      ;; 
     
  
  -)
     if [ "$OPTARG" != "help" ]
     then
       echo "ERROR invalid argument --$OPTARG"
     fi
     print_usage
     exit 1 
     ;;
     
  *)
     print_usage
     exit 1 
     ;;
  esac
done


# compute CDMW_HOME again (in case it was redefine from loaded previous config)
export CDMW_HOME=`cd $SCRIPT_PATH/../..; pwd`


#
# Before any question, check required environment variables
#

# load functions for tools checking
. $SCRIPT_PATH/check_tools.sh


##
## Question about Java not asked but check anyway
## (variable is supposed to be already set in environment)
## variable: JAVA_HOME
##
check_java $JAVA_HOME
if [ $? != 0 ]; then
    echo "\$JAVA_HOME must be correctly set"
    exit 1
fi

##
## Question about ANT not asked but check anyway
## (variable is supposed to be already set in environment)
## variable: ANT_HOME
##
check_ant $ANT_HOME
if [ $? != 0 ]; then
    echo "\$ANT_HOME must be correctly set"
    exit 1
fi


# Function to ask a directory path and set result in $ASKED_DIRECTORY_PATH
# First argument (optional) is a default directory to be choosen if user just press 'return' key.
# This operation will loop until directory path is acceptable (i.e. directory exist)
ask_directory_path() {
    export ASKED_DIRECTORY_PATH=
    DEFAULT_DIR=$1
    INPUT=none
    while [ "$INPUT" = "none" ]; do
        echo -n "> " ; read INPUT
        if [ -z "$INPUT" ]; then
            if [ -z "$DEFAULT_DIR" ]; then
                echo "There was no previous selection. Please, make a first one!"
                INPUT=none
            fi
        else
            if [ -d "$INPUT" ]; then
                # compute absolute path
                DEFAULT_DIR=`cd $INPUT; pwd`
            else
                echo "'$INPUT' is not a valid directory. Please choose another one."
                INPUT=none
            fi
        fi
    done
    export ASKED_DIRECTORY_PATH=$DEFAULT_DIR
}



##
## Here was suppressed a question from old Build:
##  "Which CSCIs are available on your system ?"
##


##
## Question about packages to be build
## variable: CDMW_PACKAGES_LIST
##
echo ""
echo "Which packages do you want to build ?"
echo "List of package(s):"
echo "    1) CORE_FT - Foundation,SystemMngt,TraceAndPerf,Repository,LifeCycle,ConfAndPlug,CodeGen,Event,FaultTolerance"
echo "    2) DEFAULT - Foundation,SystemMngt,TraceAndPerf,Repository,XMLServer,LifeCycle,ConfAndPlug,CodeGen,Event,CCMContainer,CCMDeployment,Time,LoadBalancing"
echo "    3) DEFAULT_FT - Foundation,SystemMngt,TraceAndPerf,Repository,XMLServer,LifeCycle,ConfAndPlug,CodeGen,Event,CCMContainer,CCMDeployment,Time,LoadBalancing,FaultTolerance"
if [ -n "$CDMW_PACKAGES_LIST" ]; then
    echo "  (Your previous selection was [$CDMW_PACKAGES_LIST]. Just type 'return' to keep this one)"
fi

INPUT=none
while [ "$INPUT" = "none" ]; do
    echo "Enter the number of the package you want:"
    echo -n "> " ; read INPUT
    if [ -z "$INPUT" ]; then
        if [ -z "$CDMW_PACKAGES_LIST" ]; then
            echo "There was no previous selection. Please, make a first one!"
            INPUT=none
        fi
    else
        case "$INPUT" in
            1)
                export CDMW_PACKAGES_LIST=CORE_FT
                ;;
            2)
                export CDMW_PACKAGES_LIST=DEFAULT
                ;;
            3)
                export CDMW_PACKAGES_LIST=DEFAULT_FT
                ;;
            *)
                echo "Wrong selection: $INPUT"
                INPUT=none
                ;;
        esac
    fi
done



##
## Question about debug flag for C++ compilation
## variable: CDMW_CPP_DEBUG
##
echo ""
echo "Do you want to compile the C++ interface in DEBUG mode ? (y,n)" 
if [ -n "$CDMW_CPP_DEBUG" ]; then
    if [ "$CDMW_CPP_DEBUG" = "1" ]; then
        echo "  (previous selection: [y])"
    else
        echo "  (previous selection: [n])"
    fi
fi
INPUT=none
while [ "$INPUT" = "none" ]; do
    echo -n "> " ; read INPUT
    if [ -z "$INPUT" ]; then
        if [ -z "$CDMW_CPP_DEBUG" ]; then
            echo "There was no previous selection. Please, make a first one!"
            INPUT=none
        fi
    else
        case "$INPUT" in
            y | Y | yes | YES)
                export CDMW_CPP_DEBUG=1
                ;;
            n | N | no | NO)
                export CDMW_CPP_DEBUG=0
                ;;
            *)
                echo "Wrong selection: $INPUT"
                INPUT=none
                ;;
        esac
    fi
done


##
## Question about optimization flag for C++ compilation
## variable: CDMW_CPP_OPTIM
##
echo ""
echo "Do you want to compile with the optimisation flags ? (y,n)" 
if [ -n "$CDMW_CPP_OPTIM" ]; then
    if [ "$CDMW_CPP_OPTIM" = "1" ]; then
        echo "  (previous selection: [y])"
    else
        echo "  (previous selection: [n])"
    fi
fi
INPUT=none
while [ "$INPUT" = "none" ]; do
    echo -n "> " ; read INPUT
    if [ -z "$INPUT" ]; then
        if [ -z "$CDMW_CPP_OPTIM" ]; then
            echo "There was no previous selection. Please, make a first one!"
            INPUT=none
        fi
    else
        case "$INPUT" in
            y | Y | yes | YES)
                export CDMW_CPP_OPTIM=1
                ;;
            n | N | no | NO)
                export CDMW_CPP_OPTIM=0
                ;;
            *)
                echo "Wrong selection: $INPUT"
                INPUT=none
                ;;
        esac
    fi
done


##
## Question about trace level
## variable: CDMW_TRACE_LEVEL
##
echo ""
echo "Enter the trace level you want for your CARDAMOM System" 
echo "    0) No trace"
echo "    1) Fatal traces only"
echo "    2) Error traces + selection 1)"
echo "    3) Warning traces + selection 2)"
echo "    4) Info traces + selection 3)"
echo "    5) Debug traces + selection 4)"
echo "    6) All : Additionnal CARDAMOM traces + selection 5)"
echo "Enter your choice:(0,1,2,3,4,5,6)"

if [ -n "$CDMW_TRACE_LEVEL" ]; then
    echo "  (previous selection: [$CDMW_TRACE_LEVEL])"
fi
INPUT=none
while [ "$INPUT" = "none" ]; do
    echo -n "> " ; read INPUT
    if [ -z "$INPUT" ]; then
        if [ -z "$CDMW_TRACE_LEVEL" ]; then
            echo "There was no previous selection. Please, make a first one!"
            INPUT=none
        fi
    else
        case "$INPUT" in
            0 | 1 | 2 | 3 | 4 | 5 | 6 ) 
                export CDMW_TRACE_LEVEL=$INPUT
                ;;
            *)
                echo "Wrong selection: $INPUT"
                INPUT=none
                ;;
        esac
    fi
done


##
## Question about call-traces in Cardamom processes
## variable: CDMW_CALL_TRACES
##
echo ""
echo "Should the Call-Traces messages from the CARDAMOM services be activated ? (y,n)" 
if [ -n "$CDMW_CALL_TRACES" ]; then
    if [ "$CDMW_CALL_TRACES" = "1" ]; then
        echo "  (previous selection: [y])"
    else
        echo "  (previous selection: [n])"
    fi
fi
INPUT=none
while [ "$INPUT" = "none" ]; do
    echo -n "> " ; read INPUT
    if [ -z "$INPUT" ]; then
        if [ -z "$CDMW_CALL_TRACES" ]; then
            echo "There was no previous selection. Please, make a first one!"
            INPUT=none
        fi
    else
        case "$INPUT" in
            y | Y | yes | YES)
                export CDMW_CALL_TRACES=1
                ;;
            n | N | no | NO)
                export CDMW_CALL_TRACES=0
                ;;
            *)
                echo "Wrong selection: $INPUT"
                INPUT=none
                ;;
        esac
    fi
done


##
## Question about installation directory
## variable: CDMW_INSTALL_DIR
##
echo ""
echo "Where should this product be installed ?" 
if [ -n "$CDMW_INSTALL_DIR" ]; then
    echo "  (previous selection: [$CDMW_INSTALL_DIR])"
fi
INPUT=none
while [ "$INPUT" = "none" ]; do
    echo -n "> " ; read INPUT
    if [ -z "$INPUT" ]; then
        if [ -z "$CDMW_INSTALL_DIR" ]; then
            echo "There was no previous selection. Please, make a first one!"
            INPUT=none
        fi
    else
        if [ -e "$INPUT" ]; then
            if [ -d "$INPUT" ]; then
                # compute absolute path
                export CDMW_INSTALL_DIR=`cd $INPUT; pwd`
                if [ "$CDMW_INSTALL_DIR" = "$CDMW_HOME" ]; then
                    echo "Cannot install in same directory than sources. Please choose another one."
                    INPUT=none
                fi
            else
                echo "'$INPUT' is not a valid directory. Please choose another one."
                INPUT=none
            fi
        else
            mkdir -p "$INPUT"
        # compute absolute path     
            export CDMW_INSTALL_DIR=`cd $INPUT; pwd`
        if [ ! -e "$INPUT" ]; then
            echo "Unable to create $INPUT"
            INPUT=none
        else
             echo "$CDMW_INSTALL_DIR directory created."
        fi
    fi  
    fi
done
### TODO: check disk space as in old Build


##
## Here was suppressed a question from old Build:
##   "Enter the path of Cardamom sources"
## Now this path is computed from this script's directory

### TODO: check disk space as in old Build



### TODO: check CSCIs versions & patch levels

echo ""
echo "Starting the tool configuration..."

##
## Here was suppressed a question from old Build:
##   "The configurator cannot determine which of these tools {ORBacusCPP=ORBacus C++, TAO=The ACE ORB, TAOOpenFusion=The TAO OpenFusion} to use for the CodeGen CSCI..."
##   "List of conflicting tools for the CodeGen CSCI:"
##   "   1) ORBacusCPP - ORBacus C++"
##   "   2) TAO - The ACE ORB"
##   "   3) TAOOpenFusion - The TAO OpenFusion"
##   ""
##   "Which of them do you want to use ? (1,2,3)"
##


##
## Question about ACE/TAO installation directory
## variable: ACE_ROOT, TAO_ROOT
##
RESULT_OK=false
while [ "$RESULT_OK" = "false" ]; do
    echo ""
    echo "Enter the ACE/TAO installation directory (\$ACE_ROOT)" 
    if [ -n "$ACE_ROOT" ]; then
        echo "  (previous selection: [$ACE_ROOT])"
    fi
    ask_directory_path $ACE_ROOT
    check_ace $ASKED_DIRECTORY_PATH && check_tao $ASKED_DIRECTORY_PATH
    if [ $? = 0 ]; then
        export ACE_ROOT=$ASKED_DIRECTORY_PATH
        export TAO_ROOT=$ACE_ROOT
        RESULT_OK=true
    fi
done
  

##
## Question about JACORB installation directory
## variable: JACORB_HOME
##
RESULT_OK=false
while [ "$RESULT_OK" = "false" ]; do
    echo ""
    echo "Enter the JacORB installation directory" 
    if [ -n "$JACORB_HOME" ]; then
        echo "  (previous selection: [$JACORB_HOME])"
    fi
    ask_directory_path $JACORB_HOME
    check_jacorb $ASKED_DIRECTORY_PATH
    if [ $? = 0 ]; then
        export JACORB_HOME=$ASKED_DIRECTORY_PATH
        RESULT_OK=true
    fi
done



##
## Question about CPPUNIT installation directory
## variable: CPPUNIT_ROOT
##
RESULT_OK=false
while [ "$RESULT_OK" = "false" ]; do
    echo ""
    echo "Enter the CPPUnit installation directory" 
    if [ -n "$CPPUNIT_ROOT" ]; then
        echo "  (previous selection: [$CPPUNIT_ROOT])"
    fi
    ask_directory_path $CPPUNIT_ROOT
    check_cppunit $ASKED_DIRECTORY_PATH
    if [ $? = 0 ]; then
        export CPPUNIT_ROOT=$ASKED_DIRECTORY_PATH
        RESULT_OK=true
    fi
done



##
## Question about JUNIT installation directory
## variable: JUNIT_HOME
##
RESULT_OK=false
while [ "$RESULT_OK" = "false" ]; do
    echo ""
    echo "Enter the JUnit installation directory" 
    if [ -n "$JUNIT_HOME" ]; then
        echo "  (previous selection: [$JUNIT_HOME])"
    fi
    ask_directory_path $JUNIT_HOME
    check_junit $ASKED_DIRECTORY_PATH
    if [ $? = 0 ]; then
        export JUNIT_HOME=$ASKED_DIRECTORY_PATH
        RESULT_OK=true
    fi
done



##
## Question about SAXON installation directory
## variable: SAXON_HOME
##
RESULT_OK=false
while [ "$RESULT_OK" = "false" ]; do
    echo ""
    echo "Enter the Saxon installation directory" 
    if [ -n "$SAXON_HOME" ]; then
        echo "  (previous selection: [$SAXON_HOME])"
    fi
    ask_directory_path $SAXON_HOME
    check_saxon $ASKED_DIRECTORY_PATH
    if [ $? = 0 ]; then
        export SAXON_HOME=$ASKED_DIRECTORY_PATH
        RESULT_OK=true
    fi
done



##
## Question about XERCESC installation directory
## variable: XERCESCROOT
##
RESULT_OK=false
while [ "$RESULT_OK" = "false" ]; do
    echo ""
    echo "Enter the XercesC installation directory" 
    if [ -n "$XERCESCROOT" ]; then
        echo "  (previous selection: [$XERCESCROOT])"
    fi
    ask_directory_path $XERCESCROOT
    check_xercesc $ASKED_DIRECTORY_PATH
    if [ $? = 0 ]; then
        export XERCESCROOT=$ASKED_DIRECTORY_PATH
        RESULT_OK=true
    fi
done


if [ "$CDMW_JAVA_EMMA" = "1" ]
then
    ##
    ## Question about EMMA installation directory
    ## variable: EMMA_HOME
    ##
    RESULT_OK=false
    while [ "$RESULT_OK" = "false" ]; do
        echo ""
        echo "Enter the Emma installation directory" 
        if [ -n "$EMMA_HOME" ]; then
            echo "  (previous selection: [$EMMA_HOME])"
        fi
        ask_directory_path $EMMA_HOME
        check_emma $ASKED_DIRECTORY_PATH
        if [ $? = 0 ]; then
            export EMMA_HOME=$ASKED_DIRECTORY_PATH
            RESULT_OK=true
        fi
    done
fi


##
## Questions about CODEGENERATOR installation directory
## variable: CODEGENERATOR_HOME
##
RESULT_OK=false
while [ "$RESULT_OK" = "false" ]; do
    echo ""
    echo "Enter the CodeGenerator installation directory" 
    if [ -n "$CODEGENERATOR_HOME" ]; then
        echo "  (previous selection: [$CODEGENERATOR_HOME])"
    fi
    ask_directory_path $CODEGENERATOR_HOME
    check_codegenerator $ASKED_DIRECTORY_PATH
    if [ $? = 0 ]; then
        export CODEGENERATOR_HOME=$ASKED_DIRECTORY_PATH
        RESULT_OK=true
    fi
done


##
## Here was suppressed a question from old Build:
##   "Do you want to use Splice ? (y/n)"
##



##
## Write configuration file
##
echo ""
echo "Writting configuration file $CDMW_CONF_FILE"

if [ -f $CDMW_CONF_FILE ]; then
    DATE=`date +%G_%m_%d-%H_%M_%S`
    OLD_CONF_FILE=${CDMW_CONF_FILE}.BACKUP_${DATE}
    echo "   but first rename previous one as $OLD_CONF_FILE"
    mv $CDMW_CONF_FILE $OLD_CONF_FILE
fi

$CDMW_HOME/Build/scripts/create_cdmw_env.sh --compilationversion $CDMW_HOME

echo "  File created."
echo ""
