#! /bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

########################################################################
# Environment variables
########################################################################
if [ -z "$CDMW_HOME" ];then
    echo "Error install-binary.sh : CDMW_HOME is not intialised" 
    exit 1
fi

# Set the Path for the package to construct
script_name=`basename $0`
#script_path=`dirname \`which $script_name\``
script_path="$CDMW_HOME/Build/scripts"
CDMW_Version=`grep -w CDMW $CDMW_HOME/Build/data/ReleaseProperties.xml|
                                                                  cut -d'"' -f4`
[ -z "$CDMW_Version" ] && exit 1
install_bin_tmp="$script_path/CDMW-$CDMW_Version"
Product_basename=`basename $install_bin_tmp`

########################################################################
# Functions
########################################################################

clean_exit() {
    # Abnormal termination
    echo "ERROR install-binary.sh : Abnormal termination"
    echo "ERROR install-binary.sh : Clean exit..."
    
    # Go to the install-binary.sh Workspace
    [ "`pwd`" != "$script_path" ] && cd $script_path
    # Clean the temporar directory : CDMW-$CDMW_Version
    [ -d "$Product_basename" ] && rm -rf $Product_basename
    [ -f "$CDMW_HOME/$Product_basename.tar.gz" ] && rm -f $CDMW_HOME/$Product_basename.tar.gz
    exit 1
}

usage() {
cat <<EOF
install-binary.sh [-h]
    Recompose a Cardamom environment from the local one exportable elsewhere                      
EOF
    exit 0
}

########################################################################
# Main Program
########################################################################

# On a trap EXIT exit as clean as possible
trap clean_exit 1 2 3 6

# Parse the options.
while getopts "h" ARG; do
    case $ARG in
    h)    usage;
          exit 0;;
    \?)   usage;
          exit 1;;
    esac
done
shift `expr ${OPTIND} - 1`

 
# --------------------------------------1--------------------------------------
# In a <tmp>/delivery directory copy bin, idl, include, share, demos
echo "install-binary.sh : Filling $Product_basename/delivery/ i.e..."
mkdir -p $install_bin_tmp/delivery
for dir in bin lib idl include share
do
    echo "        -> + $dir/"
    cp -frL $CDMW_HOME/$dir        $install_bin_tmp/delivery 
done
echo "        -> + demos/"
cp -frL $CDMW_HOME/ProdTests/demos $install_bin_tmp/delivery

# --------------------------------------2--------------------------------------
# In a <tmp>/tools directory copy a defined list of tools i.e: J2SDK, 
# TAOOpenFusion, XercesCPP, Saxon, JacORB, CPPUnit, JUnit
echo "install-binary.sh : Filling $Product_basename/tools/ i.e ..."
mkdir -p $install_bin_tmp/tools

# J2SDK :
JAVA_dir_name=`basename $JAVA_HOME`
echo "        -> + `basename $JAVA_HOME`/"
cd `dirname $JAVA_HOME` 2>&1 >/dev/null
tar cpf - --exclude man --exclude sample --exclude demo $JAVA_dir_name |
                            tar xpvf - -C $install_bin_tmp/tools 2>&1 >/dev/null
cd - 2>&1 >/dev/null

# TAOOpenFusion :
TAO_dir=${TAO_ROOT}
TAO_dir_name=`basename $TAO_dir`
echo "        -> + $TAO_dir_name/"
cd $TAO_dir/.. 2>&1 >/dev/null
tar cpf - --exclude tests --exclude examples --exclude performance-tests \
          --exclude .shobj $TAO_dir_name |
                            tar xpvf - -C $install_bin_tmp/tools 2>&1 >/dev/null
cd - 2>&1 >/dev/null

# XercesCPP
XERCES_dir_name=`basename $XERCESCROOT`
echo "        -> + $XERCES_dir_name/"
cp -r $XERCESCROOT    $install_bin_tmp/tools

# Saxon
SAXON_dir_name=`basename $SAXON_HOME`
echo "        -> + $SAXON_dir_name/"
cp -r $SAXON_HOME     $install_bin_tmp/tools

# JacORB
JacORB_dir=`dirname $JACORB_HOME`
JacORB_dir_name=`basename $JacORB_dir`
echo "        -> + $JacORB_dir_name/"
cp -r $JacORB_dir     $install_bin_tmp/tools

# CPPUnit
CPPUNIT_dir_name=`basename $CPPUNIT_ROOT`
echo "        -> + $CPPUNIT_dir_name/"
cp -r $CPPUNIT_ROOT   $install_bin_tmp/tools

# JUnit
JUNIT_dir_name=`basename $JUNIT_HOME`
echo "        -> + $JUNIT_dir_name/"
cp -r $JUNIT_HOME     $install_bin_tmp/tools


# --------------------------------------3--------------------------------------
# Build the <tmp>/configure.sh that is needed for extraction,installation

echo "install-binary.sh : Preparing $Product_basename/configure.sh ..."
configure_sh_file=$install_bin_tmp/configure.sh

cp $CDMW_HOME/Build/scripts/binary-install-configure.sh $configure_sh_file

# Make the script executable
chmod a+x $configure_sh_file

# --------------------------------------4--------------------------------------
# Build the <tmp>/cdmw_env limitated to above tools for a local extraction
# In <tmp>/cdmw_env : @<strings> will be translated by <tmp>/configure.sh

echo "install-binary.sh : Preparing $Product_basename/cdmw_env ..."
cdmw_env_file=$install_bin_tmp/cdmw_env

cat <<EOF >$cdmw_env_file
#
# Environment file generated at: @date@
#

export CDMW_HOME=@CDMW_HOME@

export ACE_ROOT=\$CDMW_HOME/../tools/$TAO_dir_name
export TAO_ROOT=\$CDMW_HOME/../tools/$TAO_dir_name
export CIAO_ROOT=\$CDMW_HOME/../tools/$TAO_dir_name
export XERCESCROOT=\$CDMW_HOME/../tools/$XERCES_dir_name
export CPPUNIT_ROOT=\$CDMW_HOME/../tools/$CPPUNIT_dir_name

export JAVA_HOME=\$CDMW_HOME/../tools/$JAVA_dir_name
export ANT_HOME=\$CDMW_HOME/../tools/exec/ant-1.7.0/
export JACORB_HOME=\$CDMW_HOME/../tools/$JacORB_dir_name
export JUNIT_HOME=\$CDMW_HOME/../tools/$JUNIT_dir_name
export SAXON_HOME=\$CDMW_HOME/../tools/$SAXON_dir_name

export PATH=\$CDMW_HOME/bin:\$CDMW_HOME/../tools/$TAO_dir_name/bin:\$ANT_HOME/bin:\$PATH
export LD_LIBRARY_PATH=\$CDMW_HOME/lib/c++:\$CDMW_HOME/../tools/$TAO_dir_name/lib:\$CDMW_HOME/../tools/$XERCES_dir_name/lib:\$CDMW_HOME/../tools/$CPPUNIT_dir_name/lib:\$LD_LIBRARY_PATH


# Compilation options
export CDMW_INSTALL_DIR=\$CDMW_HOME
export CDMW_TRACE_LEVEL=6

EOF

# --------------------------------------5--------------------------------------
# Make the tar -czf CDMW-<Version>.tar.gz of the above
[ "`pwd`" != "$script_path" ] && cd $script_path 2>&1 >/dev/null
echo "install-binary.sh : Building $Product_basename.tar.gz ..."
tar -czf $CDMW_HOME/$Product_basename.tar.gz $Product_basename
rm -rf $Product_basename 
