#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

if [ -z "$CDMW_HOME" ]; then
    echo "ERROR: \$CDMW_HOME must be defined"
    exit 1
fi

if [ -z "$CDMW_INSTALL_DIR" ]; then
    echo "ERROR: \$CDMW_INSTALL_DIR must be defined"
    exit 1
fi



#
# Copy all Cardamom release files 
#
copy_release() {
    echo "Install release files ..."
    ## install bin
    echo "   Install $CDMW_INSTALL_DIR/bin ..."
    cp -frL $CDMW_HOME/bin $CDMW_INSTALL_DIR/
    
    ## install lib
    echo "   Install $CDMW_INSTALL_DIR/lib ..."
    cp -frL $CDMW_HOME/lib $CDMW_INSTALL_DIR/
    
    ## install idl
    echo "   Install $CDMW_INSTALL_DIR/idl ..."
    cp -frL $CDMW_HOME/idl $CDMW_INSTALL_DIR/
    
    ## install include
    echo "   Install $CDMW_INSTALL_DIR/include ..."
    cp -frL $CDMW_HOME/include $CDMW_INSTALL_DIR/
    
    ## install share
    echo "   Install $CDMW_INSTALL_DIR/share ..."
    cp -frL $CDMW_HOME/share $CDMW_INSTALL_DIR
    
    ## install demos
    echo "   Install $CDMW_INSTALL_DIR/demos ..."
    # copy demos directory
    cp -frL $CDMW_HOME/ProdTests/demos $CDMW_INSTALL_DIR/
    # find all files with .taoopenfusion extension and copy it without extension
    for i in `find $CDMW_INSTALL_DIR/demos -name "*.taoopenfusion"`; do
        cp $i `dirname $i`/`basename $i .taoopenfusion`
    done
    # find all files with .jacorb extension and copy it without extension
    for i in `find $CDMW_INSTALL_DIR/demos -name "*.jacorb"`; do
        cp $i `dirname $i`/`basename $i .jacorb`
    done
}

#
# Copy all validation tests files 
#
copy_validation() {
    echo "Install validation files ..."
    # copy all $SRC_CDMW_HOME/validation content
    cp -frL $CDMW_HOME/validation/* $CDMW_INSTALL_DIR/

    # and ProdTests/perftests shall also be copied...
    cp -frL $CDMW_HOME/ProdTests/perftests $CDMW_INSTALL_DIR/
}


#
# Copy all qualification tests files 
#
copy_qualification() {
    echo "Install qualification files ..."
    # copy all $SRC_CDMW_HOME/qualification content
    cp -frL $CDMW_HOME/qualification/* $CDMW_INSTALL_DIR/

    # and ProdTests/perftests shall also be copied...
    cp -frL $CDMW_HOME/ProdTests/perftests $CDMW_INSTALL_DIR/
}


#
# Copy all required tools, and change environment variables to reflect
# new tools pathes.
#
copy_tools() {
    echo "Install tools ..."
    TOOLS_DIR=${CDMW_INSTALL_DIR}/../tools
    mkdir -p $TOOLS_DIR
    cd $TOOLS_DIR
    TOOLS_DIR=$PWD
    cd - 2>&1 >/dev/null
    
    # J2SDK :
    JAVA_dir_name=`basename $JAVA_HOME`
    echo "   Copy $TOOLS_DIR/$JAVA_dir_name..."
    cd `dirname $JAVA_HOME` 2>&1 >/dev/null
    tar cpf - --exclude man --exclude sample --exclude demo $JAVA_dir_name |
        tar xpvf - -C $TOOLS_DIR 2>&1 >/dev/null
    cd - 2>&1 >/dev/null
    export JAVA_HOME=$TOOLS_DIR/$JAVA_dir_name
    
    # TAOOpenFusion :
    ACE_dir=${ACE_ROOT}
    ACE_dir_name=`basename $ACE_dir`
    echo "   Copy $TOOLS_DIR/$ACE_dir_name..."
    cd $ACE_dir/.. 2>&1 >/dev/null
    tar cpf - --exclude tests --exclude examples --exclude performance-tests \
              --exclude .shobj $ACE_dir_name |
        tar xpvf - -C $TOOLS_DIR 2>&1 >/dev/null
    cd - 2>&1 >/dev/null
    old_tools_dir=`dirname $ACE_dir`
    export ACE_ROOT=${ACE_ROOT/#$old_tools_dir/$TOOLS_DIR}
    export TAO_ROOT=$ACE_ROOT
    
    # XercesCPP
    XERCES_dir_name=`basename $XERCESCROOT`
    echo "   Copy $TOOLS_DIR/$XERCES_dir_name..."
    cp -r $XERCESCROOT    $TOOLS_DIR
    export XERCESCROOT=$TOOLS_DIR/$XERCES_dir_name
    
    # Saxon
    SAXON_dir_name=`basename $SAXON_HOME`
    echo "   Copy $TOOLS_DIR/$SAXON_dir_name..."
    cp -r $SAXON_HOME     $TOOLS_DIR
    export SAXON_HOME=$TOOLS_DIR/$SAXON_dir_name
    
    # JacORB
    JacORB_dir=${JACORB_HOME}
    JacORB_dir_name=`basename $JacORB_dir`
    echo "   Copy $TOOLS_DIR/$JacORB_dir_name..."
    cp -r $JacORB_dir     $TOOLS_DIR
    old_tools_dir=`dirname $JacORB_dir`
    export JACORB_HOME=${JACORB_HOME/#$old_tools_dir/$TOOLS_DIR}
    
    # CPPUnit
    CPPUNIT_dir_name=`basename $CPPUNIT_ROOT`
    echo "   Copy $TOOLS_DIR/$CPPUNIT_dir_name..."
    cp -r $CPPUNIT_ROOT   $TOOLS_DIR
    export CPPUNIT_ROOT=$TOOLS_DIR/$CPPUNIT_dir_name
    
    # JUnit
    JUNIT_dir_name=`basename $JUNIT_HOME`
    echo "   Copy $TOOLS_DIR/$JUNIT_dir_name..."
    cp -r $JUNIT_HOME     $TOOLS_DIR
    export JUNIT_HOME=$TOOLS_DIR/$JUNIT_dir_name
    
    # Emma
    if [ -n "$EMMA_HOME" ]; then
        EMMA_dir_name=`basename $EMMA_HOME`
        echo "   Copy $TOOLS_DIR/$EMMA_dir_name..."
        cp -r $EMMA_HOME   $TOOLS_DIR
        export EMMA_HOME=$TOOLS_DIR/$EMMA_dir_name
    fi
}


#
# Replace each environment variable expressions in share and demos files  
# (only if variable is defined)
#
replace_environment_variables() {
    echo "Replace environment variables in share files ..."
    # ${CDMW_HOME} has to be replaces with futur CDMW_HOME=$CDMW_INSTALL_DIR
    # So save actual one before
    export ACTUAL_CDMW_HOME=$CDMW_HOME
    export CDMW_HOME=$CDMW_INSTALL_DIR
    # ${LOGNAME} has to be replaced with '_' in all share files
    export LOGNAME=_
    # ${TMPDIR} has not to be replaced (used in ExportHelpers.xslt)
    export TMPDIR=
    $ACTUAL_CDMW_HOME/Build/scripts/replace_environment.sh $CDMW_INSTALL_DIR/share
    
    # Find all site.mk and replace environment variables inside
    # Following variables are used in site.mk
    export CDMW_CPP_ORB="TAOOpenFusion"
    export CDMW_CPP_ORB_VERSION="1.5.1"
    export CDMW_JAVA_ORB="JacORB"
    for i in `find $CDMW_INSTALL_DIR/demos -name "site.mk"`; do
        $ACTUAL_CDMW_HOME/Build/scripts/replace_environment.sh $i
    done
    
    # Re-export CDMW_HOME
    export CDMW_HOME=$ACTUAL_CDMW_HOME
}

#
# Remove all possibly copied .svn directories
#
clean_svn_dirs() {
    find $CDMW_INSTALL_DIR -type d -name .svn | xargs rm -fr
}


#
# Remote copy of $CDMW_INSTALL_DIR on specified hosts
#
remote_copy() {
    echo "Remote copy of installed files to $*"
    HOSTS=$*
    # remove "localhost" and $HOSTNAME from hosts list
    HOSTS=${HOSTS/localhost/}
    HOSTS=${HOSTS/$HOSTNAME/}
    
    # if some hosts are remaining in list, perform remote copy
    if [ -n "${HOSTS// /}" ]; then
        # create temporary tgz
        echo "   Create temporary tar file"
        tmpfile=`mktemp -p $CDMW_INSTALL_DIR/.. tmp_tgz_for_install.XXXXX`
        tar czf $tmpfile $CDMW_INSTALL_DIR

        # install on each host
        for i in $HOSTS; do
            echo "   Installing $CDMW_INSTALL_DIR on $i"
            rcp $tmpfile $i:/tmp/
            tmpfilename=`basename $tmpfile`
            rsh $i "cd / && tar xzf /tmp/$tmpfilename; rm -f /tmp/$tmpfilename"
        done
        
        # remove temporary tgz
        echo "   Remove temporary tar file"
        #rm -f $tmpfile
    fi
}

#
# Write library version
#
write_lib_ver() {
   export CDMW_VER=`$CDMW_HOME/Build/scripts/version.sh only_cdmw_ver`
   for i in `find $CDMW_INSTALL_DIR -name '*.so'`; do
       mv $i $i.$CDMW_VER
       ln -s $i.$CDMW_VER $i
   done
   for i in `find $CDMW_INSTALL_DIR -name '*.jar'`; do
       mv $i $i.$CDMW_VER
       ln -s $i.$CDMW_VER $i
   done
}



#
# startCdmw.sh install
#
install() {
    # create installation dir
    mkdir -p $CDMW_INSTALL_DIR
    # copy release files
    copy_release
    # replace environment variables in appropriate files
    replace_environment_variables
    # rename shared libraries
    write_lib_ver
    # remove .svn dirs
    clean_svn_dirs
    # create cdmw_env
    $CDMW_HOME/Build/scripts/create_cdmw_env.sh $CDMW_INSTALL_DIR
}


#
# startCdmw.sh install-valid ($* are hostnames)
#
install_valid() {
    # Change install directory to be <install_dir>-valid
    export CDMW_INSTALL_DIR=${CDMW_INSTALL_DIR}-valid
    # create installation dir
    mkdir -p $CDMW_INSTALL_DIR
    # if only validation tests have to be installed, don't copy release files
    if [ "$INSTALL_ONLY_VALID" != "true" ]; then
        # option not set, copy release files
        copy_release
    fi
    # copy validation files
    copy_validation
    # replace environment variables in appropriate files
    replace_environment_variables
    # rename shared libraries
    write_lib_ver
    # remove .svn dirs
    clean_svn_dirs
    # create cdmw_env
    $CDMW_HOME/Build/scripts/create_cdmw_env.sh --testversion $CDMW_INSTALL_DIR
    # copy install on remote hosts
    remote_copy $*
}


#
# startCdmw.sh install-qualif ($* are hostnames)
#
install_qualif() {
    # Change install directory to be <install_dir>-qualif/delivery
    export CDMW_INSTALL_DIR=${CDMW_INSTALL_DIR}-qualif/delivery
    # create installation dir
    mkdir -p $CDMW_INSTALL_DIR
    # if only qualification tests have to be installed, don't copy release files
    if [ "$INSTALL_ONLY_QUALIF" != "true" -a "$INSTALL_ONLY_TOOLS" != "true" ]; then
        # option not set, copy release files
        copy_release
    fi
    # copy qualification files
    if [ "$INSTALL_ONLY_TOOLS" != "true" ]; then
        copy_qualification
    fi
    # remove .svn dirs
    clean_svn_dirs
    # create binary version
    if [ "$INSTALL_WITHOUT_BINARY" != "true" -a "$INSTALL_ONLY_QUALIF" != "true" -a "$INSTALL_ONLY_TOOLS" != "true" ]; then
        $CDMW_HOME/Build/scripts/install-binary.sh
        mv $CDMW_HOME/CDMW-*.tar.gz $CDMW_INSTALL_DIR/share
    fi
    # copy tools
    if [ "$INSTALL_WITHOUT_TOOLS" != "true" -a "$INSTALL_ONLY_QUALIF" != "true" ]; then
        copy_tools
    fi
    # replace environment variables in appropriate files
    replace_environment_variables
    # rename shared libraries
    write_lib_ver
    # create cdmw_env
    $CDMW_HOME/Build/scripts/create_cdmw_env.sh --testversion $CDMW_INSTALL_DIR
    # change CDMW_INSTALL_DIR to also have tools/ copied (not only delivery/)
    export CDMW_INSTALL_DIR=${CDMW_INSTALL_DIR%%/delivery}
    # copy install on remote hosts
    remote_copy $*
}



