#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

echo ""
echo "...install $PWD in $1"

# Install test from current directory in installation directory specified as first argument.
# (actually only create symbolic links)
# Binaries to be installed have to be specified as remaining arguments
#  - arguments "lib*.so" will be installed in $INSTALL_DIR/lib/c++
#  - arguments "*.jar" will be installed in $INSTALL_DIR/lib/java
#  - others arguments will be installed in $INSTALL_DIR/bin
# Then script search in current directory and install followings:
#  - all "*.sh" will be installed in $INSTALL_DIR/bin
#  - all "*.xml" except CSCPRoperties.xml, Conditions.xml and build.xml  
#    will be installed in $INSTALL_DIR/share

if [ -z "$1" ]; then 
    echo "$0: ERROR. no installation directory specified"
    exit 1
fi


INSTALL_DIR=$1
shift

get_full_path() {
    file=`basename $1`
    dir=`dirname $1`
    cd $dir
    path=$PWD/$file
    echo $path
}

# make directories 'in case not already done)
mkdir -p $INSTALL_DIR/bin
mkdir -p $INSTALL_DIR/lib/c++
mkdir -p $INSTALL_DIR/lib/java
mkdir -p $INSTALL_DIR/share

# install files specified as arguments:
if [ -n "$*" ]; then
    for i in $*; do
        if [ -f "$i" ]; then
            filepath=`get_full_path $i`
        
            if [ -z "${i/lib*.so/}" ]; then
                # it is a lib
                echo ln -sf $filepath $INSTALL_DIR/lib/c++
                ln -sf $filepath $INSTALL_DIR/lib/c++
            else
                if [ -z "${i/*.jar/}" ]; then
                    # it is a jar
                    echo ln -sf $filepath $INSTALL_DIR/lib/java
                    ln -sf $filepath $INSTALL_DIR/lib/java
                else        
                    # it is a bin
                    echo ln -sf $filepath $INSTALL_DIR/bin
                    ln -sf $filepath $INSTALL_DIR/bin
                fi
            fi
        fi
    done
fi

# install *.sh
for i in `ls *.sh 2>/dev/null`; do
    echo ln -sf $PWD/$i $INSTALL_DIR/bin
    ln -sf $PWD/$i $INSTALL_DIR/bin
done

# install *.xml
for i in `ls *.xml 2>/dev/null | grep -v CSCProperties.xml | grep -v Conditions.xml | grep -v build.xml`; do
    echo ln -sf $PWD/$i $INSTALL_DIR/share
    ln -sf $PWD/$i $INSTALL_DIR/share
done

