#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================




CSCI_LIST="CCMContainer     \
           CCMDeployment    \
           CodeGen          \
           ConfAndPlug      \
           Event            \
           FaultTolerance   \
           Foundation       \
           LifeCycle        \
           LoadBalancing    \
           ProdTests        \
           Repository       \
           SystemMngt       \
           Time             \
           TraceAndPerf     \
           XMLServer"


if [ -z "$CDMW_HOME" ]; then
    echo "ERROR: \$CDMW_HOME is not defined"
    exit 1
fi

if [ ! -d $CDMW_HOME ]; then
    echo "ERROR: \$CDMW_HOME ($CDMW_HOME) is not a valid directory"
    exit 1
fi
    


# Check if link could be created (not overwriting existing file) and create it.
create_link() {
    file_to_link=$1
    output_dir=$2
    filename=`basename $file_to_link`
    result_link=$output_dir/$filename
    
    # if file exists...
    if [ -e $result_link ]; then
        # ...and is not a link
        if [ ! -L $result_link ]; then
            echo "WARNING: $result_link already exists and is not a link." 
            echo "    Cannot overwrite with a link to $file_to_link"
            exit 1
        # ...and is a link
        else
            target=`ls -l $result_link | awk '{print $NF}'`
            # check if target of link is the same than file_to_link
            if [ "$target" != "$file_to_link" ]; then
                echo "WARNING: 2 different files with the same name:"
                echo "    $target"
                echo "    $file_to_link"
                echo "  Cannot create link to the second one."
                ###exit 1
            fi
        fi
    else
        # link can be created
        ln -sf $file_to_link $output_dir/
    fi
}


## This script perform following copies (actually create symbolic links):
##   - all idl files from <CSCI>/idl into $CDMW_HOME/idl
##   - all scripts from <CSCI>/bin into $CDMW_HOME/bin
##   - all files from <CSCI>/share into $CDMW_HOME/share
##   - all <CSCI>/c++/include/<CSCI> directories into $CDMW_HOME/include/c++
prepare_normal() {
    mkdir -p $CDMW_HOME/idl
    mkdir -p $CDMW_HOME/bin
    mkdir -p $CDMW_HOME/share
    mkdir -p $CDMW_HOME/include/c++
   
   # create link for code generation script
  create_link $CDMW_HOME/Build/scripts/cdmw_code_generator.sh $CDMW_HOME/bin
 
    for i in $CSCI_LIST; do
        csci=$CDMW_HOME/$i
        echo "Create links for files from $i:"    
    
        if [ ! -d $csci ]; then
            echo "ERROR: $csci is not a valid directory"
        fi
    
        if [ -d $csci/idl ]; then
            echo "   links to idl files in $CDMW_HOME/idl"    
            for file in `ls $csci/idl`; do
                create_link $csci/idl/$file $CDMW_HOME/idl
            done
        fi
    
        if [ -d $csci/bin ]; then
            echo "   links to .sh files in $CDMW_HOME/bin"    
            for file in `ls $csci/bin`; do
                create_link $csci/bin/$file $CDMW_HOME/bin
            done
        fi
    
        if [ -d $csci/share ]; then
            echo "   links to data files in $CDMW_HOME/share"    
            for file in `ls $csci/share`; do
                create_link $csci/share/$file $CDMW_HOME/share
            done
        fi
        
        scsi_name=`basename $csci`
        if [ -d $csci/c++/include/$scsi_name ]; then
            echo "   links to .hpp files in $CDMW_HOME/include/c++"    
            ln -sf $csci/c++/include/$scsi_name $CDMW_HOME/include/c++
        fi
    done
}


## This script perform following copies (actually create symbolic links):
##   - all scripts from <CSCI>/valid/bin into $CDMW_HOME/validation/bin
##   - all files from <CSCI>/share into $CDMW_HOME/validation/share
prepare_validation() {
    mkdir -p $CDMW_HOME/validation/bin
    mkdir -p $CDMW_HOME/validation/share
    
    for i in $CSCI_LIST; do
        csci=$CDMW_HOME/$i
        echo "Create links for files from $i/valid:"    
    
        if [ ! -d $csci ]; then
            echo "ERROR: $csci is not a valid directory"
        fi
    
        if [ -d $csci/valid/bin ]; then
            echo "   links to .sh files in $CDMW_HOME/validation/bin"    
            for file in `ls $csci/valid/bin`; do
                create_link $csci/valid/bin/$file $CDMW_HOME/validation/bin
            done
        fi
    
        if [ -d $csci/valid/share ]; then
            echo "   links to data files in $CDMW_HOME/validation/share"    
            for file in `ls $csci/valid/share`; do
                create_link $csci/valid/share/$file $CDMW_HOME/validation/share
            done
        fi
    done
}


## This script perform following copies (actually create symbolic links):
##   - all scripts from ProdTests/qualif/bin into $CDMW_HOME/qualification/bin
##   - all files from ProsTests/qualif/share into $CDMW_HOME/qualification/share
prepare_qualification() {
    mkdir -p $CDMW_HOME/qualification/bin
    mkdir -p $CDMW_HOME/qualification/share
    
    csci="$CDMW_HOME/ProdTests/qualif"
    echo "Create links for files from ProdTests/qualif:"    
    
    if [ ! -d $csci ]; then
        echo "ERROR: $csci is not a valid directory"
    fi
    
    echo "   links to .sh files in $CDMW_HOME/qualification/bin"    
    for file in `ls $csci/bin`; do
        create_link $csci/bin/$file $CDMW_HOME/qualification/bin
    done

    echo "   links to data files in $CDMW_HOME/qualification/share"    
    for file in `ls $csci/share`; do
        create_link $csci/share/$file $CDMW_HOME/qualification/share
    done
}



if [ -z "$1" ]; then
    prepare_normal
    
else

    case "$1" in
        validation )
            prepare_validation
            ;;
        
        qualification )
            prepare_qualification
            ;;
            
        * )
            echo "ERROR: unknown option: $1"
            ;;
    esac
fi

