#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

print_usage() {
    echo "Usage:" 
    echo "  $0 <input_file> [<output_file>]"
    echo "    Replace all environment variables with their values in <input_file>"
    echo "    and write result in <output_file> (if not specified, overwirte <input_file>)."
    echo "  $0 <directory>"
    echo "    Find all non-binary files with environment variables expression"
    echo "    in directory, and replace those variables with their values."
    echo "  In both case, if variable is not set, it is not replaced"
}

replace_file() {
    # Following awk program read each string and replace each environment variable
    # ($... or ${...}) with its value if defined.
    # If not defined, the environment variable expression is not changed.
    cat $1 | awk '
        {
            finished="false"
            input=$0
            output=""
            while (finished == "false") {
                # Find environment variable expression: $... or ${...}
                # var_name will contain the matching regexp in [0]
                # and just the variable name in [1]
                # Note: [[:digit:]] is added because of a bug in Awk 3.1.3.
                #       \\w is sufficient with Awk 3.1.5.
                match(input, "\\${?(([[:digit:]]|\\w)+)}?", var_name)
                
                # if variable expression was found
                if ( RLENGTH != -1 ) {
                    # try to get value from environment
                    var_content=ENVIRON[var_name[1]]
                    # if exist in environemt
                    if ( length(var_content) > 0 ) {
                        # copy string before variable
                        output=output""substr(input, 1, RSTART-1)
                        # replace variable with its value
                        output=output""var_content
                    } else {
                        # if variable is not defined, copy string including variable
                        output=output""substr(input, 1, RSTART+RLENGTH-1)
                    }
                    
                    # set input as remaining of the string
                    input=substr(input, RSTART+RLENGTH)
                    
                } else {
                    # no variable expression found; copy remaining of the string and end loop
                    output=output""input
                    finished="true"
                }
            }
            
            # print resulting string
            print output
        }
    ' > $2
}



if [ -z "$1" ]; then
    echo "Error: no input file or directory"
    print_usage
    exit 1
fi
export HOSTNAME=`hostname`
# First case of usage: $0 <inputfile> [<outputfile>]
if [ -f "$1" ]; then
    if [ -n "$2" ]; then
        replace_file $1 $2
    else
        tmpfile=`mktemp`
        replace_file $1 $tmpfile
        mv -f $tmpfile $1
    fi

else

    # Second case of usage: $0 <directory>
    if [ -d "$1" ]; then
        if [ -n "$2" ]; then
        echo "Error: first argument is a directory; no need of a second argument"
        print_usage
        exit 1
        fi
        
        # find files with environment variables expressions
        for i in `grep -RIl "\\(\\${\\w\\w*}\\)\\|\\(\\$\\w\\w*\\)" $1/*`; do
            tmpfile=`mktemp`
            replace_file $i $tmpfile
            mv -f $tmpfile $i
        done
    
    else
        echo "Error: first argument is not a file nor a directory"
        print_usage
        exit 1
    fi
    
fi


