#! /bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

########################################################################
# Environment variables
########################################################################
if [ -z "$CDMW_HOME" ];then
    echo "Error runtest.sh : CDMW_HOME is not intialised" 
    exit 1
fi

# time in seconds at script startup
Start_time=`date "+%s"`
# SELF-TESTING REPORT to construct while tests are in progress
runtest_log=/tmp/runtest_log.$Start_time
#list of all run tests
tests_list=""

########################################################################
# Functions
########################################################################

clean_exit() {
    rm -f $runtest_log
    exit 0
}

display_delta_time() {
    Start_time=$1
    End_time=$2
    echo $Start_time $End_time | awk '{ dureeHeures=int(($2-$1)/3600)
                                  dureeMinutes=int(((($2-$1)/3600)-dureeHeures)*60)
                                  dureeSeconds=int(((((($2-$1)/3600)-dureeHeures)*60)-dureeMinutes)*60)
                                  printf "%.2d hour(s) %.2d minutes %.2d seconds\n",
                                  dureeHeures,dureeMinutes,dureeSeconds}'
}

launch_test() {
    # perl_test is the perl test set by the caller
    echo "$perl_test :"
    cd `dirname $perl_test`
    ./`basename $perl_test`
    cat runtest/`basename $perl_test .pl`/results/result.txt
    cd - >/dev/null
}
update_self_testing_report() {
    perl_test=$1
    Test_id=`echo $perl_test |awk -F'/' '{print $NF}'|cut -d'.' -f1`
    Test_path=`dirname $perl_test`
    Result_file="`dirname $perl_test`/runtest/`basename $perl_test .pl`/results/result.txt"
    Result_path=`dirname $Result_file`
    
    # Start displaying self test reporting
cat <<EOF
[TEST] $Test_path
ID : $Test_id
EOF

    # Check if the executable of the test is present or not
    eval `awk ' $1 == "$test_exec" {print $0}' $perl_test| tr -d '$ '`
    if [ "$TARGET_LANGUAGE" = "c++" ] && [ ! -x "$Test_path/$test_exec" ];then
cat <<EOF
+ 
|   ### NO EXECUTABLE FOUND. PLEASE COMPILE THE CSC ###
EOF
    fi

    # Is there at least one core dump file ?
    Found_core=`find $Test_path/ -type f -name "core*"`
    if [ -n "$Found_core" ];then
cat <<EOF
+ 
|   ### TEST FAILED ###
|   ### THE TEST HAS PRODUCED CORE DUMP FILE(S) ###
EOF
    else
        # No core file found, look for test success or failure
        grep "Test OK" $Result_file 2>&1 >/dev/null
        if [ $? -eq 0 ];then
            Test_result="TEST OK"
        else
            if [ -f "$Result_file" ];then
                 Test_result="### TEST FAILED ###"
         else
             Test_result="### TEST FAILED : missing $Result_file ###" 
         fi
        fi
        # Display the test result summary
cat <<EOF
+
|   $Test_result
EOF
        if  [ "$Test_result" != "TEST OK" ];then
            # set the nbtestable number of tests from the perl test script itself
            eval `grep '$nbtestable' $perl_test|grep -v TESTABLE | tr -d '$ '`
             if [ -f $Result_file ] && [ -f $Result_path/output.txt ];then 
             nbsucessful=`grep TESTABLE_OK $Result_path/{result,output}.txt 2>/dev/null |wc -l`
         else
             nbsucessful="?"
         fi
      # How find number of startable tests and number of succeeded
cat <<EOF
|       Testable objects to check            : $nbtestable
|       Testable objects sucessfully checked : $nbsucessful
EOF
        fi 
    fi  
        echo "+--------------------------------------------------------------------+"
    
    if [ "$VALGRIND_CHECK" = "1" ]
    then
        valgrind_log_file=""
        for i in `find $Test_path/runtest/$Test_id/ -name '*_??h??.log' | sort -r`
     do
        valgrind_log_file=$i
     done
     if [ -n "$valgrind_log_file" ]
     then
         lost=`grep -E "==[0-9]+==    definitely lost:" $valgrind_log_file | sed -r -e 's/==[0-9]+==    definitely lost: //g' | sed -r -e 's/ byte[s]* in [0-9,]+ blocks.//g'`
cat <<EOF
|   LEAK SUMMARY:
|   definitely lost: $lost
EOF
            if [ "$lost" -eq "0" ]
         then
cat <<EOF
|   LEAK DETECTION OK
+--------------------------------------------------------------------+
EOF
         else
cat <<EOF
|   ### LEAK DETECTION NOK ###
+--------------------------------------------------------------------+
EOF
         fi
     fi
     
    
    fi
     
}


usage() {
cat <<EOF
runtest.sh   [-o <CSCI>|<CSCI_PATH>|<CSC>|<CSC_PATH> ]
             [-h]
             [-l c++|java ] 
                       
EOF
    exit 0
}

########################################################################
# Main Program
########################################################################

# On a trap EXIT exit as clean as possible
trap clean_exit 1 2 3 6

# Parse the options.
CSCI_LIST=""
TARGET_LANGUAGE=""
while getopts "o:l:h" ARG; do
    case $ARG in
    o)    CSCI_LIST="${OPTARG} $CSCI_LIST";;
    l)    TARGET_LANGUAGE="${OPTARG}";;
    h)    usage;;
    \?)   usage;;
    esac
done
shift `expr ${OPTIND} - 1`

echo "############################################################"
echo "#???? : UNIT TESTS EXECUTION"
echo "############################################################"
echo "START DATE : `date`"

# If -o otion is not used, runtest are launched on all CSCI
[ -z "$CSCI_LIST" ] && CSCI_LIST="$CDMW_HOME/"

for CSCI in $CSCI_LIST
do
    unset CSC_PATH CSC_PATH_cpp CSC_PATH_java 
    # Locate the CSCI root directory to parse U-???-????.pl tests from
    if [ "$CSCI" = "`basename $CSCI`" ];then
        # CSCI or CSC name given (not a Path)
        CSC_PATH_cpp=`find $CDMW_HOME/* -type d -name $CSCI|grep "/tests/c++"`
        CSC_PATH_java=`find $CDMW_HOME/ $* -type d -name $CSCI|
                                                           grep "/tests/java"`
        CSC_PATH_java="`find $CDMW_HOME/ $* -type d -name ${CSCI}data` $CSC_PATH_java"
        # If both variables are not set consider that a CSCI name was given
        if [ -z "$CSC_PATH_cpp" ] && [ -z "$CSC_PATH_java" ];then
            CSC_PATH="$CDMW_HOME/$CSCI"
        fi
    else
        # Path was given
        CSC_PATH="$CSCI"
    fi

    # Running C++ tests 
    if [ "$TARGET_LANGUAGE" = "c++" ] || [ "$TARGET_LANGUAGE" = "" ];then
        # do something if a path to search for perl test is set
        if [ -n "$CSC_PATH_cpp" ] || [ -n "$CSC_PATH" ];then
            CSC_PATH_cpp=${CSC_PATH_cpp:=$CSC_PATH}
         # Remove the core files below CSC_PATH_cpp before running the test
         find $CSC_PATH_cpp/ -type f -name "core*" -exec rm {} \;
         # Identify the perl test entry then run it
            for perl_test in `find $CSC_PATH_cpp -name "U-???-0???.pl"|
                                                                    grep /c++/`
            do
                launch_test $perl_test
          update_self_testing_report $perl_test >>$runtest_log
          tests_list=$perl_test" "$tests_list
            done
        fi 
    fi

    # Running java tests
    if [ "$TARGET_LANGUAGE" = "java" ] || [ "$TARGET_LANGUAGE" = "" ];then
        # do something if a path to search for perl test is set 
        if [ -n "$CSC_PATH_java" ] || [ -n "$CSC_PATH" ];then
            CSC_PATH_java=${CSC_PATH_java:=$CSC_PATH}
            for perl_test in `find $CSC_PATH_java -name "U-???-1???.pl"|
                                                                   grep /java/`
            do
                launch_test $perl_test
          update_self_testing_report $perl_test >>$runtest_log
          tests_list=$perl_test" "$tests_list
            done
        fi
    fi
        
done

# SELF-TESTING REPORT display 
echo "+--------------------------------------------------------------------+"
echo " SELF-TESTING REPORT"
echo "+--------------------------------------------------------------------+"
cat $runtest_log
# Clean out runtest_log
rm $runtest_log

# time in seconds at script startup
End_time=`date "+%s"`

echo "Total time:  `display_delta_time $Start_time $End_time`"
 
 
echo "============================================================"
echo "#???? : UNIT TESTS EXECUTION"
echo "============================================================"
echo "END DATE : `date`"

${CDMW_HOME}/Build/scripts/generateHTML.sh $tests_list
echo "${CDMW_HOME}/Build/scripts/generateHTML.sh $tests_list"
echo "Done"
