#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

if [ -z "$CDMW_HOME" ]; then
    echo "ERROR: \$CDMW_HOME must be defined"
    exit 1
fi

if [ ! -f "$CDMW_HOME/cdmw_env" ]
then
    echo "ERROR: Cardamom not configured"
    exit 1
fi

# Parse $CDMW_HOME/Build/data/ReleaseProperties.xml to find Cardamom version
while read line
do
   if [ "${line:0:20}" = "<product name=\"CDMW\"" ]
   then
       CDMW_ver=`expr "$line" : '.*version=\"\([^\"]*\)'`
   fi
done < $CDMW_HOME/Build/data/ReleaseProperties.xml

if [ "$1" = "only_cdmw_ver" ]
then
   echo $CDMW_ver
   exit 0
fi


CSCI_name=(CCMContainer CCMDeployment CodeGen ConfAndPlug Event FaultTolerance Foundation LifeCycle LoadBalancing ProdTests Repository SystemMngt Time TraceAndPerf XMLServer)


echo ===================================================


# Parse all CSCIs CSCIProperties.xml to find each CSCI version
for i in ${!CSCI_name[*]}
do
    while read line
    do
       if [ "${line:0:5}" = "<csci" ]
       then
           CSCI_ver[$i]=`expr "$line" : '.*version=\"\([^\"]*\)'`
       fi
    done < $CDMW_HOME/${CSCI_name[$i]}/data/CSCIProperties.xml
done

Patch_Level=00

# Find bigest patch version in all CSCIs
for i in ${!CSCI_name[*]}
do
    patch_version=`expr "${CSCI_ver[$i]}" : '.*p\(.*\)'`    
    if [ -n "$patch_version" ] && [ "$patch_version" -gt "$Patch_Level" ]
    then
        Patch_Level=$patch_version
    fi    
done

# Parse $CDMW_HOME/cdmw_env to find Configuration time and Packages list
while read line
do
   if [ "${line:0:18}" = "# Environment file" ]
   then
       Config_time=`expr "$line" : '.*at:\(.*\)'`
   fi
   if [ "${line:0:25}" = "export CDMW_PACKAGES_LIST" ]
   then
       Packages=`expr "$line" : '.*=\(.*\)'`
   fi
   
done < $CDMW_HOME/cdmw_env

echo "CARDAMOM version V$CDMW_ver Patch Level $Patch_Level"
echo "(Platfrom :`uname`-`uname -m`-`uname -r`  --  Configuration time: $Config_time)"
echo "Configuration:"
echo "    Package(s)   : $Packages"
echo ""
echo ""
echo "CSCIs versions"

# Display all CSCIs versions
for i in ${!CSCI_name[*]}
do
    echo "    CSCI ${CSCI_name[$i]} version:"
    echo "                                  ${CSCI_ver[$i]}"
done
