#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX Sistemi Integrati. It is derivative work based on PERCO 
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
# All rights reserved.
# Copyright (C) THALES 2008-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

ulimit -c unlimited

StartDateSec=`date "+%s"`

SCRIPTNAME=`basename $0`
SCRIPTPATH=`dirname $0`
CDMW_HOME=`cd $SCRIPTPATH/..; pwd`

# expected configuration file (created by configure target)
CONFIGURATION_FILE=$CDMW_HOME/cdmw_env

TARGET_OPTS_LIST="TARGET_OPT1;TARGET_OPT2;..."
GENERIC_TRGT_OPTS="TARGET"
GENERIC_TRGT_OPTS="${GENERIC_TRGT_OPTS} [OPTIONS]..."


# Initialize the variables to their default values.
TARGET=
GENERATE_GCOV="-Dgenerate.gcov.infos=true"
GENERATE_EMMA="-Dgenerate.emma.infos=true"
CFG_OPTS=

CDMW_HOSTNAME_1=
CDMW_HOSTNAME_2=
CDMW_HOSTNAME_3=
CDMW_HOSTNAME_4=
CDMW_HOSTNAME_5=
OPT_WITHOUTUNITTESTS=
OPT_TESTS_TO_RUN=
OPT_MAKE=

print_usage() {
    echo "Usage: ${SCRIPTNAME} ${GENERIC_TRGT_OPTS}"
    exit 0
}

# Display the usage if the script has no command-line arguments.
if [ $# -eq 0 ]; then
    print_usage
fi

print_help() {
    S4="-O cscis=CSCI_NAME1,CSCI_NAME2,..."
    S5="-O \"host1=name host2=name\""
    S6="-O \"host3=name host4=name host5=name\""
    S7="-O c++; java; ada; timescale=n; CSCI; CSCI/CSC"
    S8="-O withoutbinaryversion"
    S9="-O onlyvalidationtest"
    S11="-O onlyqualificationtest"
    S12="-O withouttools"
    S13="-O onlytools"
    S15="-O generategcov"
    S16="-O withoutunittests"
    S17="-O generateemma"
    S18="-O makeOption=-j<N>"
    S19="-O makeOption=-k"

    
    echo "usage: $SCRIPTNAME $GENERIC_TRGT_OPTS"
    echo
    echo "General options"
    echo "---------------"
    echo "    -t, --target TARGET      equivalent to TARGET. see list of targets."
    echo "    -f, --cfgfile CFG_FILE   use a specific configuration file"
    echo "    -h, --help               help screen"
    echo "    -O, --targetopts OPTION  "
    echo
    echo "    -O,--targetopts  <loosecheckmode|strictcheckmode> check mode for the CSCIs version"
    echo
    echo "List of targets"
    echo "---------------"
    echo "* get the list of needed tools:"
    echo "    needed-tools"
    echo
    echo "* return the version of the product and the version of each CSCIs:"
    echo "    version"
    echo
    echo "* get a summary ot the host hardware resources:"
    echo "    check-host-resources"
    echo
    echo "* configure of the build environment:"
    echo "    configure [$S15] [$S17]"
    echo "    reconfigure"
    echo
    echo "* compile the source files:"
    echo "    compile [${S16}] [${S4}] [${S18}] [${S19}]"
    echo "    compile-valid [${S4}] [${S18}] [${S19}]"
    echo "    compile-qualif [${S4}] [${S18}] [${S19}]"
    echo
    echo "* install the product:"
    echo "    install [$S4]"
    echo "    install-valid $S5 [$S6] [$S4] [$S9]"
    echo "    install-qualif $S5 [$S6] [$S4] [$S11] [$S8] [$S12] [$S13]"
    echo "    install-binary [S14]"
    echo
    echo "* run the test applications:"
    echo "    runtest [$S7]"
    echo
    echo "* debug the test application:"
    echo "    debugtest <name-of-test>"
    echo
    echo "* check the test application:"
    echo "    checktest [<name-of-test>]"
    echo
    echo "    inspect-valid $S4"
    echo "    inspect-qualif $S4"
    echo
    echo "* generate the API documentation:"
    echo "    genpublicdoc"
    echo "    genfulldoc"
    echo
    echo "* combo targets:"
    echo "    compile-and-install"
    echo
    echo "* clean the CSCI directories"
    echo "    clean"
    echo
    echo "* thoroughly clean the CSCI directories"
    echo "    distclean"
    echo
    echo "* print a summary of the configuration files"
    echo "    configsum"
    echo
    echo "* create a new CSC in an existing CSCI"
    echo "    addcsc"
    exit 0
}

display_delta_time() {
    Start_time=$1
    End_time=$2
    echo
    echo -n "Total time : "
    echo $Start_time $End_time | awk '{ dureeHeures=int(($2-$1)/3600)
                                  dureeMinutes=int(((($2-$1)/3600)-dureeHeures)*60)
                                  dureeSeconds=int(((((($2-$1)/3600)-dureeHeures)*60)-dureeMinutes)*60)
                                  printf "%.2d hour(s) %.2d minutes %.2d seconds\n",
                                  dureeHeures,dureeMinutes,dureeSeconds}'
}

print_msg() {
    echo "$SCRIPTNAME: $1"
    exit 1
}

target_already_set() {
    if [ -n "$TARGET" ]; then
        print_msg "only one target can be set at once"
    fi
}

check_target() {
    if [ "$1" = "needed-tools" -o \
         "$1" = "version" -o \
         "$1" = "check-host-resources" -o \
         "$1" = "configure" -o \
         "$1" = "reconfigure" -o \
         "$1" = "compile" -o \
         "$1" = "compile-valid" -o \
         "$1" = "compile-qualif" -o \
         "$1" = "install" -o \
         "$1" = "install-valid" -o \
         "$1" = "install-qualif" -o \
         "$1" = "install-binary" -o \
         "$1" = "runtest" -o \
         "$1" = "debugtest" -o \
         "$1" = "checktest" -o \
         "$1" = "genpublicdoc" -o \
         "$1" = "genfulldoc" -o \
         "$1" = "compile-and-install" -o \
         "$1" = "clean" -o \
         "$1" = "distclean" -o \
         "$1" = "configsum" -o \
         "$1" = "inspect-valid" -o \
         "$1" = "inspect-qualif" -o \
         "$1" = "testdescription" -o \
         "$1" = "addcsc" \
        ]; then

        TARGET=$1
    else
        print_msg "unknown target \"$1\""
    fi
}

parse_target_options() {
    for o in `echo $1 | sed -e 's:;: :g'`; do
        case $o in
        
            generategcov)
                if [ -z "`echo  $CFG_OPTS | grep 'generate.gcov.infos'`" ]; then
                    CFG_OPTS="$CFG_OPTS $GENERATE_GCOV"
                fi
                ;;
           
            generateemma)
                if [ -z "`echo  $CFG_OPTS | grep 'generate.emma.infos'`" ]; then
                    CFG_OPTS="$CFG_OPTS $GENERATE_EMMA"
                fi
                ;;

            loosecheckmode)
                echo "option '$o' not yet supported"
                exit 1
                ## TODO
                if [ -z "`echo $GEN_OPTS | grep 'csci.check.mode'`" ]; then
                    GEN_OPTS="$GEN_OPTS $LOOSE_CHECK_MODE"
                fi
                ;;

            strictcheckmode)
                echo "option '$o' not yet supported"
                exit 1
                ## TODO
                if [ -z "`echo $GEN_OPTS | grep 'csci.check.mode'`" ]; then
                    GEN_OPTS="$GEN_OPTS $STRICT_CHECK_MODE"
                fi
                ;;

            withoutbinaryversion)
                # export variable used in install-functions.sh
                export INSTALL_WITHOUT_BINARY="true"
                ;;

            onlyvalidationtest)
                # export variable used in install-functions.sh
                export INSTALL_ONLY_VALID="true"
                ;;

            onlyqualificationtest)
                # export variable used in install-functions.sh
                export INSTALL_ONLY_QUALIF="true"
                ;;

            withouttools)
                # export variable used in install-functions.sh
                export INSTALL_WITHOUT_TOOLS="true"
                ;;

            onlytools)
                # export variable used in install-functions.sh
                export INSTALL_ONLY_TOOLS="true"
                ;;

            withoutunittests)
                OPT_WITHOUTUNITTESTS="true"
                ;;

            host1=*)
                # export variable used in install-functions.sh
                export CDMW_HOSTNAME_1=`echo $o | cut -d'=' -f2`
                ;;

            host2=*)
                # export variable used in install-functions.sh
                export CDMW_HOSTNAME_2=`echo $o | cut -d'=' -f2`
                ;;

            host3=*)
                # export variable used in install-functions.sh
                export CDMW_HOSTNAME_3=`echo $o | cut -d'=' -f2`
                ;;

            host4=*)
                # export variable used in install-functions.sh
                export CDMW_HOSTNAME_4=`echo $o | cut -d'=' -f2`
                ;;

            host5=*)
                # export variable used in install-functions.sh
                export CDMW_HOSTNAME_5=`echo $o | cut -d'=' -f2`
                ;;

            cscis=*)
                echo "option '$o' not yet supported"
                exit 1
                ## TODO
                CPL_CSCIS="-Dconfig.sel.min.cscis=`echo $1 | cut -d= -f2 | \
                          sed -e 's: ::g' -e 's:,: :g'`"
                ;;

            makeOption=*)
                opt=`echo $o | cut -d'=' -f2`
                OPT_MAKE="${OPT_MAKE} $opt"
                ;;
                
            *)
                if [ "$o" = "java" -o "$o" = "c++" ]; then
                    OPT_TESTS_TO_RUN="${OPT_TESTS_TO_RUN} -l $o"
                else
                    OPT_TESTS_TO_RUN="${OPT_TESTS_TO_RUN} -o $o";
                fi
                ;;
        esac
    done
}


load_configuration_file() {
    # check if configuration file exists
    if [ ! -f $CDMW_HOME/cdmw_env ]; then
        echo "ERROR: 'configure' was not performed ($CDMW_HOME/cdmw_env doesn't exist)"
        exit 1
    fi
    # load created configuration
    . $CDMW_HOME/cdmw_env
}


# Parse the command-line arguments.
while [ -n "$*" ]; do
    case "$1" in
        -h | --help)
            print_help
            ;;

        -f | --cfgfile)
            if [ -n "$2" -a -z "`echo $2 | grep '^-'`" ]; then
                DIRNAME=`dirname $2`
                DIRNAME=`cd $DIRNAME; pwd`
                SELECT_CONFIG="$DIRNAME/`basename $2`"
            else
                print_msg "missing configuration file name"
            fi
            shift
            ;;

        -t | --target)
            if [ -n "$2" -a -z "`echo $2 | grep '^-'`" ]; then
                check_target $2
            else
                print_msg "missing target name"
            fi
            shift
            ;;

        -O | --targetopts)
            if [ -n "$2" -a -z "`echo $2 | grep '^-'`" ]; then
                parse_target_options "$2"
            else
                print_msg "missing target options"
            fi
            shift
            ;;

        -*)
            print_msg "unknown option \"$1\""
            ;;

        *)
            check_target $1
            ;;
    esac
    shift
done

# Check if the target is set.
if [ -z "$TARGET" ]; then
    print_msg "you must specify one target"
fi

ALL_TARGET_OPTIONS=
case "$TARGET" in
    configure )
        # create configuration
        $CDMW_HOME/Build/scripts/create_configuration.sh $CFG_OPTS $SELECT_CONFIG
        # load created configuration
        load_configuration_file
        # create required links
        $CDMW_HOME/Build/scripts/prepare_directories.sh
        # generate makefiles
        cd $CDMW_HOME/Build/MPC
        ./gen_makefiles.sh
        ;;
   
    reconfigure )
        # load created configuration
        load_configuration_file
        # create required links
        $CDMW_HOME/Build/scripts/prepare_directories.sh
        # generate makefiles
        cd $CDMW_HOME/Build/MPC
        ./gen_makefiles.sh
        ;;
   
    check-host-resources )
        $CDMW_HOME/Build/scripts/check-host-resources.sh
        ;;

    compile )
        # load created configuration
        load_configuration_file
        # compile Java (require C++ part of code generator to be compiled before)
        echo "Compile Java..."
        cd $CDMW_HOME/CodeGen
        compileCSC
        cd $CDMW_HOME/Build/Ant
        ant compile
        if [ $? != 0 ]; then
            echo "Java compilation failed !"
            exit 1
        fi
        # compile C++ (require Java part of code generator to be compiled before)
        cd $CDMW_HOME/Build/MPC
        if [ -z "$OPT_WITHOUTUNITTESTS" ]; then
            echo "Compile C++ (with tests)..."
            make ${OPT_MAKE}
        else
            echo "Compile C++ (without tests)..."
            make ${OPT_MAKE} -f GNUmakefile.withoutunittests
        fi
        if [ $? != 0 ]; then
            echo "C++ compilation failed !"        
            exit 1
        fi
        if [ -z "$OPT_WITHOUTUNITTESTS" ]; then
            # compile Java tests (require C++ part of code generator to be compiled before)
            echo "Compile Java tests..."
            cd $CDMW_HOME/Build/Ant
            ant compile-tests
            if [ $? != 0 ]; then
                echo "Java tests compilation failed !"
                exit 1
            fi
        fi
        ;;
    
        
    compile-valid )
        # load created configuration
        load_configuration_file
        # compile C++ validation tests
        cd $CDMW_HOME/Build/MPC
        echo "Compile C++ validation tests..."
        make ${OPT_MAKE} -f GNUmakefile.validation
        if [ $? != 0 ]; then
            echo "C++ validation tests compilation failed !"        
            exit 1
        fi
        # compile Java validation tests
        echo "Compile Java validation tests..."
        cd $CDMW_HOME/Build/Ant
        ant compile-valid-tests
        if [ $? != 0 ]; then
            echo "Java validation tests compilation failed !"
            exit 1
        fi
        ;;

    compile-qualif )
        # load created configuration
        load_configuration_file
        # compile C++ qualification tests
        cd $CDMW_HOME/Build/MPC
        echo "Compile C++ qualification tests..."
        make ${OPT_MAKE} -f GNUmakefile.qualification
        if [ $? != 0 ]; then
            echo "C++ qualification tests compilation failed !"        
            exit 1
        fi
        # compile Java qualification tests
        echo "Compile Java qualification tests..."
        cd $CDMW_HOME/Build/Ant
        ant compile-qualif-tests
        if [ $? != 0 ]; then
            echo "Java qualification tests compilation failed !"
            exit 1
        fi
        ;;

    clean )
        # load created configuration
        load_configuration_file
        # clean Java
        cd $CDMW_HOME/Build/Ant
        ant clean-all
        # clean C++
        cd $CDMW_HOME/Build/MPC
        make realclean
    make -f GNUmakefile.validation realclean
    make -f GNUmakefile.qualification realclean
        ;;
        
    distclean )
        # load created configuration
        load_configuration_file
        # clean Java
        cd $CDMW_HOME/Build/Ant
        ant clean-all
        # clean C++
        cd $CDMW_HOME/Build/MPC
        make realclean
    make -f GNUmakefile.validation realclean
    make -f GNUmakefile.qualification realclean
        # complete cleaning
        echo "Remove generated directories..."
        find $CDMW_HOME -type d -name "generated" | xargs rm -fr
        echo "Remove makefiles..."
        find $CDMW_HOME -type f -name "GNUmakefile*" | xargs rm -fr
        find $CDMW_HOME -type f -name ".depend*" | xargs rm -fr
        echo "Remove links"
        rm -fr $CDMW_HOME/bin $CDMW_HOME/lib $CDMW_HOME/idl $CDMW_HOME/include $CDMW_HOME/share $CDMW_HOME/tmp $CDMW_HOME/validation $CDMW_HOME/qualification
        ;;
        
        
    install)
        # load created configuration
        load_configuration_file
        # source script for installations
        . $CDMW_HOME/Build/scripts/install-functions.sh
        # call install()
        install
        ;;
        
    install-qualif)
        # check that at least 3 hosts have been specified
        if [ -z "$CDMW_HOSTNAME_1" -o -z "$CDMW_HOSTNAME_2" -o -z "$CDMW_HOSTNAME_3" ]; then
            echo "ERROR: at least 'host1' 'host2' and 'host3' options have to be set for install-qualif"
            exit 1
        fi
        # load created configuration
        load_configuration_file
        # prepare required links
        $CDMW_HOME/Build/scripts/prepare_directories.sh qualification
        # source script for installations
        . $CDMW_HOME/Build/scripts/install-functions.sh
        # call install_qualif()
        install_qualif $CDMW_HOSTNAME_1 $CDMW_HOSTNAME_2 $CDMW_HOSTNAME_3 $CDMW_HOSTNAME_4 $CDMW_HOSTNAME_5
        ;;
        
    install-valid)
        # check that at least 2 hosts have been specified
        if [ -z "$CDMW_HOSTNAME_1" -o -z "$CDMW_HOSTNAME_2" ]; then
            echo "ERROR: at least 'host1' and 'host2' options have to be set for install-valid"
            exit 1
        fi
        # load created configuration
        load_configuration_file
        # prepare required links
        $CDMW_HOME/Build/scripts/prepare_directories.sh validation
        # source script for installations
        . $CDMW_HOME/Build/scripts/install-functions.sh
        # call install_valid()
        install_valid $CDMW_HOSTNAME_1 $CDMW_HOSTNAME_2 $CDMW_HOSTNAME_3 $CDMW_HOSTNAME_4 $CDMW_HOSTNAME_5
        ;;
        
    install-binary)
        # Create a compressed archive containing the binary version of CARDAMOM
        $CDMW_HOME/Build/scripts/install-binary.sh
        ;;
    
    compile-and-install)
        # Run compile and intall targets
        $CDMW_HOME/Build/startCdmw.sh compile && \
        $CDMW_HOME/Build/startCdmw.sh install    
        ;;

    runtest)
        # load created configuration
        load_configuration_file
        # start tests
        ${CDMW_HOME}/Build/scripts/runtest.sh $OPT_TESTS_TO_RUN
        ;;

    debugtest)
        echo "Target '$TARGET' not yet supported"
        exit 1
        ## TODO
        if [ -z "`echo ${RUN_OPTS}`" ]; then
             print_msg "missing mandatory options, check the usage"
        fi
        ANT_LOGGER="-logger com.thalesgroup.tools.ant.listener.XDefaultLogger"
        VERBOSE_MODE="-emacs"
        ALL_TARGET_OPTIONS=$RUN_OPTS
        CLASSPATH=$PWD/delivery/lib/libcdmwtasks.jar:$CLASSPATH
        export CLASSPATH
        ;;

    checktest)
        # load created configuration
        load_configuration_file
        # start tests
    export VALGRIND_CHECK=1
        ${CDMW_HOME}/Build/scripts/runtest.sh $OPT_TESTS_TO_RUN
        ;;


     test)
        echo "Target '$TARGET' not yet supported"
        exit 1
        ## TODO
        ANT_LOGGER="-logger com.thalesgroup.tools.ant.listener.XDefaultLogger"
        VERBOSE_MODE="-emacs"
        ALL_TARGET_OPTIONS=$RUN_OPTS
        CLASSPATH=$PWD/delivery/lib/libcdmwtasks.jar:$CLASSPATH
        export CLASSPATH
        ;;
    
    needed-tools)
        # Display needed tools
        cat $CDMW_HOME/Build/scripts/needed-tools.txt
        ;;
    
    configsum)
        # print a summary of the configuration files
        load_configuration_file
        $CDMW_HOME/Build/scripts/configsum.sh
        ;;

    inspect-valid)
        $CDMW_HOME/Build/scripts/inspect-valid.sh
        ;;

    inspect-qualif)
        $CDMW_HOME/Build/scripts/inspect-qualif.sh
        ;;
    
    addcsc)
        $CDMW_HOME/Build/scripts/addcsc.sh
        ;;
        
    version)
        $CDMW_HOME/Build/scripts/version.sh
        ;;   
        
    genpublicdoc)
        $CDMW_HOME/Build/scripts/gen-docs.sh genpublicdoc
        ;;   
        
    genfulldoc)
        $CDMW_HOME/Build/scripts/gen-docs.sh genfulldoc
        ;;      
esac

EndDate=`date "+%s"`

display_delta_time $StartDateSec $EndDate 

