# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================
package TestUtils;

use strict;
use Cwd;
use File::Basename;
use File::Copy;
use File::Path;
use Config;
use XML::DOM;

#################################################################################

sub checkXML
{
    my $file = shift;

    if (!-e $file)
    {
        return "ERROR: $file file does not exist \n";
    }
    
    my $result = "";

    my $parser = new XML::DOM::Parser;
    
    -e $file or return;
    
    my $doc = $parser->parsefile($file);

    my $failure_type;
    my $failure_message;
    
    for my $node ($doc->getElementsByTagName("FailedTest"))
    {
        for my $child ($node->getChildNodes())
        {
            if ($child->getNodeType == ELEMENT_NODE)
            {
                my $child_name = $child->getNodeName();
            
                if ($child_name eq "FailureType")
                {
                    $failure_type = $child->getFirstChild()->getData();
                    $failure_type = convertFailureType($failure_type);
                }

                if ($child_name eq "Message")
                {
                    $failure_message = $child->getFirstChild()->getData();
                    $failure_message =~ s/\n/ /
                }        
            }
        }
        $result = $failure_type, ": ", $failure_message, "\n";
    }

    $doc->dispose;
    
    return $result;
}


sub convertFailureType($)
{
    my $failure_type = shift;
    
    if (($failure_type eq "error") or ($failure_type eq "Error"))
    {
        $failure_type = "ERROR";
    }

    if (($failure_type eq "fail") or ($failure_type eq "Fail"))
    {
        $failure_type = "FAIL";
    }

    if (($failure_type eq "failed") or ($failure_type eq "Failed"))
    {
        $failure_type = "FAILED";
    }

    if (($failure_type eq "exception") or ($failure_type eq "Exception"))
    {
        $failure_type = "EXCEPTION";
    }

    if (($failure_type eq "assertion") or ($failure_type eq "ASSERTION"))
    {
        $failure_type = "Assertion";
    }
    
    return $failure_type;    
}


sub checkForMessage
{
    my $file = shift;
    my $message = shift;
	
    my $count = 0;
    open(DAT, $file) || die("Could not open file - $file!");
    $count = 0;
    while ( <DAT> ) {
        if (/$message/)
        {   
            $count++;   
        }
    }
    close(DAT);

    return $count;
}

sub checkForErrorMessages
{
    my $file = shift;
    
    my $count = 0;
    
    #$count += checkForMessage($file, "ERROR");
    $count += checkForMessage($file, "Test FAILED");
    
    return $count;
}


sub moveFileToResults
{
    my $file = shift;

    if (-e $file)
    {
        move $file, "results/".$file;
    }
}

sub prepareDirectoryStructureForUnitTest
{

    my $test_folder = shift;
    
    if (! -e "runtest")
    {
        mkpath "runtest/".$test_folder."/results";
    }
    else
    {
        rmtree (["runtest/".$test_folder], 0, 1);
        mkpath "runtest/".$test_folder."/results";
    }
}


sub replace
{
    my $file = shift;
    my $placeholder = shift;
    my $value = shift;    
    my $temp_file = $file.".tmp";
    open(FILE, $file) || die("Could not open file - $file!");
    open(TEMPFILE, ">$temp_file") || die("Could not open file - $temp_file!");

    while ( my $line = <FILE> ) {
        $line =~ s/$placeholder/$value/;
	print TEMPFILE $line;
    }
    close(FILE);
    close(TEMPFILE);
  
    move $temp_file, $file;
}

1;
