/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_CIF_CCMCONTEXT_IMPL_HPP
#define INCL_CCM_CIF_CCMCONTEXT_IMPL_HPP


#include <Foundation/idllib/Components.skel.hpp>

#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include "Foundation/osthreads/MutexGuard.hpp"
#include <string>
#include <map>


namespace Cdmw {

namespace CCM {

namespace CIF {


/**
*Purpose:
*<p>    [class description]
*    This class implement the Components::CCMContext interface.
*    It will be inherited by all components context implementation.
*
*/

//
// IDL:omg.org/Components/CCMContext:1.0
//
class CCMContext_impl : virtual public Components::CCMContext,
                        virtual public OrbSupport::RefCountLocalObject
{
public:

    CCMContext_impl
    (
     Components::CCMContext_ptr ctx,
     const ::std::string & component_id,
     CdmwLogging::LoggerFactory_ptr logger_factory
     );
     ~CCMContext_impl();

    //
    // IDL:omg.org/Components/CCMContext/get_caller_principal:1.0
    //
    virtual Components::Principal_ptr get_caller_principal()
        throw(CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/get_CCM_home:1.0
    //
    virtual Components::CCMHome_ptr get_CCM_home()
        throw(CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/get_rollback_only:1.0
    //
    virtual CORBA::Boolean get_rollback_only()
        throw(Components::IllegalState,
              CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/get_user_transaction:1.0
    //
    virtual Components::Transaction::UserTransaction_ptr get_user_transaction()
        throw(Components::IllegalState,
              CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/is_caller_in_role:1.0
    //
    virtual CORBA::Boolean is_caller_in_role(const char* role)
        throw(CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/set_rollback_only:1.0
    //
    virtual void set_rollback_only()
        throw(Components::IllegalState,
              CORBA::SystemException);

    //
    // IDL:omg.org/Components/CCMContext/resolve_service_reference:1.0
    //
    virtual CORBA::Object_ptr resolve_service_reference(const char* service_id) 
        throw(Components::CCMException,
              CORBA::SystemException);

protected:
    OsSupport::Mutex m_mtx;
    // map of installed services
    typedef std::map<std::string, CORBA::Object_var> services_map_t;
    static services_map_t m_services_map;
    typedef services_map_t::iterator services_map_iterator;
            
              
private:
    CCMContext_impl();
    CCMContext_impl(const CCMContext_impl&);
    void operator=(const CCMContext_impl&);
     
    Components::CCMContext_var m_context;   // container context
    
    const std::string m_component_id;

    // logger to be used in CCMContext
    CdmwLogging::Logger_var m_logger;

};  // End CCMContext_impl class

} // End namespace CIF

} // End namespace CCM

} // End namespce Cdmw


#endif  // INCL_CCM_CIF_CCMCONTEXT_IMPL_HPP

