/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_CIF_EXTENSIONCONTEXT_IMPL_HPP
#define INCL_CCM_CIF_EXTENSIONCONTEXT_IMPL_HPP

#include <Foundation/idllib/Components.skel.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <CCMContainer/ccmcif/CCMContext_impl.hpp>
#include <CCMContainer/ccmcommon/Cookie_impl.hpp>

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace CIF {


/**
*Purpose:
*<p>    [class description]
*    This class implement the Components::SessionContext interface.
*    It will be inherited by all session components context implementation.
*
*/

//
// IDL:omg.org/Components/ExtensionContext:1.0
//
class ExtensionContext_impl : public CCMContext_impl,
                              virtual public Components::ExtensionContext,
                              virtual public OrbSupport::RefCountLocalObject
                          
{
public:

    ExtensionContext_impl
    (
     Components::CCMContext_ptr ctx,
     const ::std::string & component_id,
     CdmwLogging::LoggerFactory_ptr logger_factory
     )
        throw (CORBA::SystemException);

    ~ExtensionContext_impl();

    virtual Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_ptr get_client_interceptor_registration ()
        throw(::CORBA::SystemException,::Components::CCMException);


    virtual Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_ptr get_server_interceptor_registration ()
        throw(::CORBA::SystemException,::Components::CCMException);

    virtual Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_ptr get_stub_interceptor_registration ()
        throw(::CORBA::SystemException,::Components::CCMException);

    virtual Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_ptr get_servant_interceptor_registration ()
        throw(::CORBA::SystemException,::Components::CCMException);

    virtual Components::Cookie * install_service_reference (
                                                            const char * service_id,
                                                            ::CORBA::Object_ptr objref
                                                            )
        throw(::CORBA::SystemException,::Components::CCMException);

    virtual CORBA::Object_ptr uninstall_service_reference (
                                                           ::Components::Cookie * ck
                                                           )
        throw(::CORBA::SystemException,::Components::CCMException);

private:
    ExtensionContext_impl();
    ExtensionContext_impl(const ExtensionContext_impl&);
    void operator=(const ExtensionContext_impl&);

    Components::ExtensionContext_var m_extension_context;

    // logger to be used in CCMContext
    CdmwLogging::Logger_var m_logger;

};  // End ExtensionContext_impl class


} // End namespace CIF

} // End namespace CCM

} // End namespce Cdmw

#endif // INCL_CCM_CIF_EXTENSIONCONTEXT_IMPL_HPP


