/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef APPLICATIONELEMENTSCONTEXTSERVICE_HPP_
#define APPLICATIONELEMENTSCONTEXTSERVICE_HPP_

#include <string>
#include <vector>
#include <map>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/logging/Logger.hpp>
#include "CCMContainer/idllib/CdmwCcmContainer.stub.hpp"
#include "Foundation/idllib/Components.stub.hpp"
#include "AbstractApplicationElementContextFactory.hpp"

namespace Cdmw
{
namespace CCM
{
namespace Container
{

class ApplicationElementsContextService: public Cdmw::OrbSupport::RefCountLocalObject,
        public CdmwCcmContainer::ApplicationElementContextService,
        public AbstractApplicationElementContextFactory
{
public:
    static const std::string ApplicationElementsContextServiceId;

protected:
    Cdmw::Logging::Logger* m_logger;

private:
    //forward declaration of nested classes
    class ComponentElementContextFactory;
    class HomeElementContextFactory;

private:
    CdmwCcmContainer::LifeCycleInterceptorRegistration_ptr m_registration;

    ComponentElementContextFactory* m_compFactory;
    HomeElementContextFactory* m_homeFactory;

    CdmwCcmContainer::ComponentServerContext* m_componentServerContext;

    std::vector<AbstractApplicationElementContextFactory*> m_contextsFactories;

    ///Ctor && Dtor
public:
    ApplicationElementsContextService(const std::string& componentServerId, const Components::ConfigValues& values);

    virtual ~ApplicationElementsContextService();

public:
    CdmwCcmContainer::ComponentElementContext* find_component_context_by_id(const std::string& id) const
            throw(Components::CCMException);

    CdmwCcmContainer::HomeElementContext* find_home_context_by_id(const std::string& id) const
            throw(Components::CCMException);

    CdmwCcmContainer::ComponentServerContext*
    get_component_server_context() const throw(Components::CCMException);

    CdmwCcmContainer::ApplicationElementContext* create_element_context(const std::string& id,
            CORBA::Object_ptr ref, const Components::ConfigValues& configValues);

    CdmwCcmContainer::ApplicationElementContext* find_element_context_by_id(const std::string& id) const;

    void remove_element_context(const std::string& id);

    void clear();

};

}
}
}
#endif /* APPLICATIONELEMENTSCONTEXTSERVICE_HPP_ */
