/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_CCM_SERVICEREGISTRY_IMPL_HPP
#define INCL_CDMW_CCM_SERVICEREGISTRY_IMPL_HPP

#include <CCMContainer/idllib/CdmwCcmContainer.skel.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include "Foundation/osthreads/MutexGuard.hpp"

#include <map>

namespace
{

} // End anonymous space

namespace Cdmw
{
namespace CCM
{
namespace Container
{

class ServiceRegistry: public virtual CdmwCcmContainer::ServiceRegistry,
                            public virtual OrbSupport::RefCountLocalObject
{
public:

    static ServiceRegistry* get_instance();
    static void destroy_instance();

    virtual CdmwCcmContainer::ServicesIds* list() throw (::CORBA::SystemException );

    virtual ::CORBA::Object_ptr lookup(const char * service_id) throw (::CORBA::SystemException,
                                                               ::Components::CCMException );

    virtual
    void bind(const char * service_id, ::CORBA::Object_ptr service_ref) throw (::CORBA::SystemException,
                                                                       ::Components::CCMException );

    virtual
    void rebind(const char * service_id, ::CORBA::Object_ptr service_ref) throw (::CORBA::SystemException,
                                                                         ::Components::CCMException );

    virtual ::CORBA::Object_ptr unbind(const char * service_id) throw (::CORBA::SystemException,
                                                               ::Components::CCMException );

    virtual
    void clear() throw (::CORBA::SystemException );

protected:
    typedef std::map<std::string, CORBA::Object_ptr> services_map_t;
    services_map_t m_services_map;

    static OsSupport::Mutex m_mtx;

private:
    // Constructor
    ServiceRegistry(const std::string & name);
    ServiceRegistry& operator =(const ServiceRegistry& other);
    ServiceRegistry(const ServiceRegistry& other);
    // Destructor
    ~ServiceRegistry(void);

    static ServiceRegistry * m_instance;
    std::string m_name;
};

}
; // End namespace Container
}
; // End namespace CCM
}
; // End namespace Cdmw

#endif // INCL_CDMW_CCM_SERVICEREGISTRY_IMPL_HPP
