/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_CCM_COPI_COPICURRENT_IMPL_HPP
#define CDMW_CCM_COPI_COPICURRENT_IMPL_HPP

#include <string>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>


namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {

    class  COPICurrent_impl
        : public virtual CdmwCcmCopi::COPICurrent,
          public virtual OrbSupport::RefCountLocalObject
    {


    public:
        static const std::string COPI_CURRENT_ID;

        // Constructor
        COPICurrent_impl(
                         int origin_ID_SlotID, 
                         int target_ID_SlotID,
                         int originFeatureName_SlotID, 
                         int targetFeatureName_SlotID,
                         int originCustomSlotItemSeq_SlotID,
                         int targetCustomSlotItemSeq_SlotID, 
                         CdmwCcmCopi::COPIRegistrar_ptr reg
                         );


        // Destructor
        virtual ~COPICurrent_impl (void);

        virtual
        ::PortableInterceptor::SlotId get_slot_id (
                                                   const char * slot_name
                                                   )
            throw(
                  ::CORBA::SystemException,
                  ::PortableInterceptor::InvalidSlot
                  );
        virtual
        ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_ptr get_client_interceptor_registration ()
            throw (
                   ::CORBA::SystemException,
                   ::Components::CCMException
                   );

        virtual
        ::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_ptr get_server_interceptor_registration ()
            throw (
                   ::CORBA::SystemException,
                   ::Components::CCMException
                   );

        virtual
        ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_ptr get_stub_interceptor_registration ()
            throw (
                   ::CORBA::SystemException,
                   ::Components::CCMException
                   );

        virtual
        ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_ptr get_servant_interceptor_registration ()
            throw (
                   ::CORBA::SystemException,
                   ::Components::CCMException
                   );

        virtual
        ::CdmwCcmCopi::RegisteredCOPICollection_ptr get_registered_copi_collection ()
            throw (
                   ::CORBA::SystemException
                   );


    

    private:
        COPICurrent_impl&  operator = (const COPICurrent_impl& other);
        COPICurrent_impl(const COPICurrent_impl& other);    

        /**
         * The registration interface of COPIs
         */
        CdmwCcmCopi::COPIRegistrar_ptr m_container_interceptor_registration;


        /**
         * The needed slots identifiers
         */
        int m_origin_ID_SlotID, m_target_ID_SlotID, m_originFeatureName_SlotID,
            m_targetFeatureName_SlotID, m_originCustomSlotItemSeq_SlotID,
            m_targetCustomSlotItemSeq_SlotID;
    };

   
} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

#endif // CDMW_CCM_COPI_COPICURRENT_IMPL_HPP

