/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <Foundation/idllib/Components.skel.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include "Foundation/osthreads/MutexGuard.hpp"
#include "Foundation/osthreads/Condition.hpp"
#include <orbsupport/ExceptionMinorCodes.hpp>
#include <orbsupport/CORBASystemExceptionWrapper.hpp>
#include "CCMContainer/ccmcopi/COPICookie_impl.hpp"

#include <map>
#include <vector>
#include <sstream>

#ifndef CDMW_CCM_COPI_COPIREGISTRATION_IMPL_HPP
#define CDMW_CCM_COPI_COPIREGISTRATION_IMPL_HPP

namespace
{

} // End anonymous space


namespace Cdmw
{

namespace CCM
{

namespace COPI
{

class COPIRegistration_impl;

// class to wrap a cookie pointer. This class it to serve as the key type in a map
// where one would want to have a cookie ptr as key.

// Note: This class has an ordering which is determined by the cookies it wraps.
// Cookies with highest priorities come first. If 2 cookies have the same priority
// the order is the order of the cookie superclass.

// Note: This class does not take ownership
class CookieKey_impl
{
    friend class COPIRegistration_impl;

public:

    CookieKey_impl(COPICookie_impl * _cookie) :
        cookie(_cookie)
    {
        if (cookie == 0)
        {
            throw CORBA::BAD_PARAM();
        }
    }
    ;

    bool operator<(const CookieKey_impl& right) const throw ()
    {
        if (cookie->priority == right.cookie->priority)
        {
            return *cookie < *(right.cookie);
        }
        else
        {
            return cookie->priority > right.cookie->priority;
        }
    }

    bool operator==(const CookieKey_impl& right) const throw ()
    {
        return *cookie == *(right.cookie);
    }

private:
    COPICookie_impl * cookie;

};

template<typename T>
class InterceptorTrait
{
    //See COPIResgistration_impl.inl for specialisations
    //No body here to fail at compile time if used with wrong type
};

template<>
class InterceptorTrait<
        Components::ContainerPortableInterceptor::ClientContainerInterceptor>
{
public:
    typedef ::CdmwCcmCopi::ClientContainerInterceptorSeq seq_t;
    typedef std::map<CookieKey_impl,
            ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>
            cookie_to_copy_t;
    typedef std::map<std::string, cookie_to_copy_t *> map_t;
};

template<>
class InterceptorTrait<
        Components::ContainerPortableInterceptor::ServerContainerInterceptor>
{
public:
    typedef ::CdmwCcmCopi::ServerContainerInterceptorSeq seq_t;
    typedef std::map<CookieKey_impl,
            ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr>
            cookie_to_copy_t;
    typedef std::map<std::string, cookie_to_copy_t *> map_t;
};

template<>
class InterceptorTrait<
        Components::ContainerPortableInterceptor::ServantContainerInterceptor>
{
public:
    typedef ::CdmwCcmCopi::ServantContainerInterceptorSeq seq_t;
    typedef std::map<CookieKey_impl,
            ::Components::ContainerPortableInterceptor::ServantContainerInterceptor_ptr>
            cookie_to_copy_t;
    typedef std::map<std::string, cookie_to_copy_t *> map_t;
};

template<>
class InterceptorTrait<
        Components::ContainerPortableInterceptor::StubContainerInterceptor>
{
public:
    typedef ::CdmwCcmCopi::StubContainerInterceptorSeq seq_t;
    typedef std::map<CookieKey_impl,
            ::Components::ContainerPortableInterceptor::StubContainerInterceptor_ptr>
            cookie_to_copy_t;
    typedef std::map<std::string, cookie_to_copy_t *> map_t;
};

template<>
class InterceptorTrait<
        ::CdmwCcmContainer::HomeLifeCycleInterceptor>
{
public:
    typedef ::CdmwCcmCopi::HomeLifeCycleInterceptorSeq seq_t;
    typedef std::map<CookieKey_impl,
            ::CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>
            cookie_to_copy_t;
    typedef std::map<std::string, cookie_to_copy_t *> map_t;
};

template<>
class InterceptorTrait<
        ::CdmwCcmContainer::ComponentLifeCycleInterceptor>
{
public:
    typedef ::CdmwCcmCopi::ComponentLifeCycleInterceptorSeq seq_t;
    typedef std::map<CookieKey_impl,
            ::CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>
            cookie_to_copy_t;
    typedef std::map<std::string, cookie_to_copy_t *> map_t;
};

class COPIRegistration_impl: public virtual CdmwCcmCopi::COPIRegistrar,
                             public virtual OrbSupport::RefCountLocalObject

{
public:
    static const char * COPIRegistration_impl::M_LifeCycleInterceptorRegistrationServiceId;

    static COPIRegistration_impl * Get_Instance();

    static void Initialise(int client_slot_id, int server_slot_id);

    // Constructor private

    // Destructor
    virtual ~COPIRegistration_impl(void);

    virtual ::Components::Cookie *
    register_client_interceptor(
            ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr
    unregister_client_interceptor(::Components::Cookie * ck) throw (::CORBA::SystemException,
            ::Components::ContainerPortableInterceptor::InvalidRegistration);

    virtual ::Components::Cookie *
    register_server_interceptor(
            ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr
    unregister_server_interceptor(::Components::Cookie * ck) throw (::CORBA::SystemException,
            ::Components::ContainerPortableInterceptor::InvalidRegistration);

    virtual ::Components::Cookie *
    register_stub_interceptor(
            ::Components::ContainerPortableInterceptor::StubContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual ::Components::ContainerPortableInterceptor::StubContainerInterceptor_ptr
    unregister_stub_interceptor(::Components::Cookie * ck) throw (::CORBA::SystemException,
            ::Components::ContainerPortableInterceptor::InvalidRegistration);

    virtual ::Components::Cookie *
    register_servant_interceptor(
            ::Components::ContainerPortableInterceptor::ServantContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual ::Components::ContainerPortableInterceptor::ServantContainerInterceptor_ptr
    unregister_servant_interceptor(::Components::Cookie * ck) throw (::CORBA::SystemException,
            ::Components::ContainerPortableInterceptor::InvalidRegistration);

    virtual ::Components::Cookie *
    register_home_lifecycle_interceptor (::CdmwCcmContainer::HomeLifeCycleInterceptor_ptr ci) 
            throw (::CORBA::SystemException);
    
    virtual ::CdmwCcmContainer::HomeLifeCycleInterceptor_ptr
    unregister_home_lifecycle_interceptor (::Components::Cookie * ck)
            throw (::CORBA::SystemException,
                   ::Components::ContainerPortableInterceptor::InvalidRegistration);
    
    virtual ::Components::Cookie *
    register_component_lifecycle_interceptor (::CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr ci)
            throw (::CORBA::SystemException);
  
    virtual ::CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr
    unregister_component_lifecycle_interceptor (::Components::Cookie * ck)
            throw (::CORBA::SystemException,
                   ::Components::ContainerPortableInterceptor::InvalidRegistration);

    virtual ::CdmwCcmCopi::ClientContainerInterceptorSeq *
    getClientInterceptorByIPOrderedByPriority(
            const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip)
            throw (::CORBA::SystemException);

    virtual ::CdmwCcmCopi::ServerContainerInterceptorSeq *
    getServerInterceptorByIPOrderedByPriority(
            const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip)
            throw (::CORBA::SystemException);

    virtual ::CdmwCcmCopi::StubContainerInterceptorSeq *
    getStubInterceptorByIPOrderedByPriority(
            const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip)
            throw (::CORBA::SystemException);

    virtual ::CdmwCcmCopi::ServantContainerInterceptorSeq *
    getServantInterceptorByIPOrderedByPriority(
            const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip)
            throw (::CORBA::SystemException);

    virtual ::CdmwCcmCopi::HomeLifeCycleInterceptorSeq *
    getHomeLifeCycleInterceptorByIPOrderedByPriority(
            const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip)
            throw (::CORBA::SystemException);
  
    virtual ::CdmwCcmCopi::ComponentLifeCycleInterceptorSeq *
    getComponentLifeCycleInterceptorByIPOrderedByPriority (
            const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip)
            throw (::CORBA::SystemException);

    virtual CORBA::Short addClientCOPIRef(
            ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual CORBA::Short addServerCOPIRef(
            ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual CORBA::Short addStubCOPIRef(::Components::ContainerPortableInterceptor::StubContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual CORBA::Short addServantCOPIRef(
            ::Components::ContainerPortableInterceptor::ServantContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual ::CORBA::Short addHomeLifeCycleCOPIRef (
            ::CdmwCcmContainer::HomeLifeCycleInterceptor_ptr ci)
            throw (::CORBA::SystemException);
  
    virtual ::CORBA::Short addComponentLifeCycleCOPIRef (
            ::CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual CORBA::Short removeClientCOPIRef(
            ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual CORBA::Short removeServerCOPIRef(
            ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual CORBA::Short removeStubCOPIRef(
            ::Components::ContainerPortableInterceptor::StubContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual CORBA::Short removeServantCOPIRef(
            ::Components::ContainerPortableInterceptor::ServantContainerInterceptor_ptr ci)
            throw (::CORBA::SystemException);

    virtual ::CORBA::Short removeHomeLifeCycleCOPIRef(
            ::CdmwCcmContainer::HomeLifeCycleInterceptor_ptr ci)
            throw (::CORBA::SystemException);
  
    virtual ::CORBA::Short removeComponentLifeCycleCOPIRef (
            ::CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr ci)
            throw (::CORBA::SystemException);

protected:
    class ObjectRefKey_impl;
    class RefCounter_impl;
    template <typename T>class COPIKindStorage;

    typedef ::Components::ContainerPortableInterceptor::ContainerInterceptor_ptr
            container_interceptor_ptr_t;
    typedef ::Components::ContainerPortableInterceptor::ContainerInterceptor_var
            container_interceptor_var_t;

    //        typedef std::map< CookieKey_impl, ::Components::ContainerPortableInterceptor::ContainerInterceptor_var > cookie_to_copy_t;
    //        typedef std::map< std::string, cookie_to_copy_t * > COPI_map_t;
    typedef std::map<ObjectRefKey_impl, RefCounter_impl *> ref_count_t;

    typedef InterceptorTrait<Components::ContainerPortableInterceptor::ClientContainerInterceptor>::map_t Client_COPI_map_t;
    typedef InterceptorTrait<Components::ContainerPortableInterceptor::ServerContainerInterceptor>::map_t Server_COPI_map_t;
    typedef InterceptorTrait<Components::ContainerPortableInterceptor::StubContainerInterceptor>::map_t Stub_COPI_map_t;
    typedef InterceptorTrait<Components::ContainerPortableInterceptor::ServantContainerInterceptor>::map_t Servant_COPI_map_t;

    // must be called under mtx protection
    template<typename Interceptor_T>
    COPICookie_impl *
    register_container_interceptor
    (
     typename Interceptor_T::_ptr_type copi,
     const int & slot_id,
     COPIKindStorage<Interceptor_T> & storage
     )
        throw (::CORBA::SystemException);

    // must be called under mtx protection
    // mtx which protects the call must belong to cond
    template<typename Interceptor_T>
    typename Interceptor_T::_ptr_type
    unregister_container_interceptor
    (
     ::Components::Cookie * ck,
     COPIKindStorage<Interceptor_T> & storage
     );

    // must be called under mtx protection
    template<typename Interceptor_T>
    typename InterceptorTrait<Interceptor_T>::seq_t*
    getContainerPortableInterceptorsByIPOrderedByPriority(
            const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip,
            COPIKindStorage<Interceptor_T> & storage);

    template<typename Interceptor_T>
    typename InterceptorTrait<Interceptor_T>::seq_t*
    COPIRegistration_impl::getAllContainerInterceptors
    (
     COPIKindStorage<Interceptor_T> & storage
     ) 
        throw (
               ::CORBA::SystemException
               );

    template<typename Interceptor_T>
    void destroy_copi_storage(COPIKindStorage<Interceptor_T> & storage);


    virtual std::string get_copi_key(
            ::Components::ContainerPortableInterceptor::ContainerInterceptor_ptr copi);

    virtual std::string create_the_copismap_key(const std::string & port,
            const std::string & operation);

    // checks the cookie, and returns a casted version
    // throws BAD_PARAM in case Cookie invalid
    virtual COPICookie_impl * get_checked_cookie(::Components::Cookie * ck)
            throw (CORBA::BAD_PARAM);


    /*
    virtual CORBA::Short COPIRegistration_impl::add_COPI_ref(
            COPIRegistration_impl::container_interceptor_ptr_t copi, OsSupport::Mutex & lock,
            COPIRegistration_impl::ref_count_t & ref_count);
    */


    template<typename Interceptor_T>  
    CORBA::Short COPIRegistration_impl::add_COPI_ref
    (
     COPIRegistration_impl::container_interceptor_ptr_t copi, 
     COPIKindStorage<Interceptor_T> & storage
     );
    

    template<typename Interceptor_T>
    CORBA::Short remove_COPI_ref
    (
     COPIRegistration_impl::container_interceptor_ptr_t copi,
     COPIKindStorage<Interceptor_T> & storage
     );


    class RefCounter_impl
    {
        friend class COPIRegistration_impl;

    public:
        // ctor
        RefCounter_impl(Cdmw::CCM::COPI::COPICookie_impl * ck) :
            m_ref_count(0), m_ck(ck)
        {
        }

        void add_ref()
        {
            CDMW_MUTEX_GUARD(m_mtx);
            m_ref_count++;
        }

        void remove_ref()
        {
            CDMW_MUTEX_GUARD(m_mtx);
            m_ref_count--;

            if (m_ref_count < 0)
            {
                // Ups, negative ref count
                throw CORBA::BAD_PARAM();
            }
        }

        bool is_still_used()
        {
            CDMW_MUTEX_GUARD(m_mtx);
            return m_ref_count == 0 ? false : true;
        }

    private:
        int m_ref_count;
        Cdmw::CCM::COPI::COPICookie_impl * m_ck;
        OsSupport::Mutex m_mtx;

    };

    class ObjectRefKey_impl
    {
        friend class COPIRegistration_impl;
    public:

        ObjectRefKey_impl(::CORBA::Object_ptr _obj) :
            m_obj(::CORBA::Object::_duplicate(_obj))
        {
            if (true == ::CORBA::is_nil(_obj))
            {
                throw CORBA::BAD_PARAM();
            }
        }
        ;

        bool operator<(const ObjectRefKey_impl & right) const throw ()
        {
            // order of object refs is determined by adress
            // this is not very clean though since ther is no
            // reqirement that a _ptr is indeed a pointer...
            return m_obj.in() < right.m_obj.in();
        }

        bool operator==(const ObjectRefKey_impl & right) const throw ()
        {
            return m_obj.in()->_is_equivalent(right.m_obj.in());
        }

    private:
        ::CORBA::Object_var m_obj;
    };


    template<typename T>
    class COPIKindStorage {

    public:
        COPIKindStorage<T>::COPIKindStorage() : m_still_used_cond(m_lock) {};

        typename InterceptorTrait<T>::map_t m_copi_map;
        
        // This keeps track for each client interceptor, the number of requests
        // in which it is invloved
        ref_count_t m_copi_ref_count;
        
        OsSupport::Mutex m_lock;
        OsSupport::Condition m_still_used_cond;
    };


private:

    // ctor
    COPIRegistration_impl();

    COPIRegistration_impl(const COPIRegistration_impl&);

    COPIRegistration_impl& operator=(const COPIRegistration_impl&);

    // copi storage

    COPIKindStorage< Components::ContainerPortableInterceptor::ClientContainerInterceptor > 
    m_client_copi_storage;
    
    COPIKindStorage< Components::ContainerPortableInterceptor::ServerContainerInterceptor >
    m_server_copi_storage;
    
    COPIKindStorage< Components::ContainerPortableInterceptor::StubContainerInterceptor >
    m_stub_copi_storage;
    
    COPIKindStorage< Components::ContainerPortableInterceptor::ServantContainerInterceptor >
    m_servant_copi_storage;
    
    COPIKindStorage< CdmwCcmContainer::HomeLifeCycleInterceptor >
    m_home_lc_copi_storage;
    
    COPIKindStorage< CdmwCcmContainer::ComponentLifeCycleInterceptor >
    m_component_lc_copi_storage;

    // Slot ids
    
    static int M_client_slot_id;
    static int M_server_slot_id;

    // singleton

    static bool M_initialised;
    static COPIRegistration_impl * The_Instance; 
};

} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

#include "COPIRegistration_impl.inl"

#endif // CDMW_CCM_COPI_COPIREGISTRATION_IMPL_HPP
