/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef COPIREGISTRATION_IMPL_INL_
#define COPIREGISTRATION_IMPL_INL_

#include "COPIRegistration_impl.hpp"
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

namespace
{

} // End anonymous space


namespace Cdmw
{

namespace CCM
{

namespace COPI
{

template<typename Interceptor_T>
COPICookie_impl *
COPIRegistration_impl::register_container_interceptor
(
    typename Interceptor_T::_ptr_type copi,
    const int & slot_id,
    COPIKindStorage<Interceptor_T> & storage
)
    throw (::CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    if (copi == 0)
    {
        CORBA::BAD_PARAM ex = CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMBADCookie,
                CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    copi->set_slot_id(slot_id);

    COPICookie_impl * ck = new COPICookie_impl(copi);
    COPICookie_impl * s_ck = new COPICookie_impl(copi);

    std::string key = get_copi_key(static_cast< ::Components::ContainerPortableInterceptor::ContainerInterceptor_ptr>(copi));

    typename InterceptorTrait<Interceptor_T>::map_t::iterator it = storage.m_copi_map.find(key);

    if (it != storage.m_copi_map.end())
    {
        // check whether copi has been registered already
        for (typename InterceptorTrait<Interceptor_T>::cookie_to_copy_t::iterator cit = it->second->begin(); cit
                != it->second->end(); ++cit)
            if (true == cit->second->_is_equivalent(copi))
            {
                delete ck;
                delete s_ck;
                CORBA::BAD_PARAM ex = CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMCOPIAlreadyExist,
                        CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);

                throw ex;
            }



        it->second->insert(std::make_pair(CookieKey_impl(s_ck), copi));

    }
    else
    {

        typename InterceptorTrait<Interceptor_T>::cookie_to_copy_t * mm =
                new typename InterceptorTrait<Interceptor_T>::cookie_to_copy_t();
        mm->insert(std::make_pair(CookieKey_impl(s_ck), copi));
        storage.m_copi_map.insert(std::make_pair(key, mm));
    }

    // initialize the ref counting of the interceptor
    storage.m_copi_ref_count.insert(std::make_pair(ObjectRefKey_impl(copi), new RefCounter_impl(s_ck)));


    return ck;
}

template<typename Interceptor_T>
typename Interceptor_T::_ptr_type COPIRegistration_impl::unregister_container_interceptor
(
    ::Components::Cookie * ck, 
    COPIKindStorage<Interceptor_T> & storage
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    COPICookie_impl * ck_impl = get_checked_cookie(ck);

    try
    {
        bool flag = false;

        for (ref_count_t::iterator ref_it = storage.m_copi_ref_count.begin(); ref_it != storage.m_copi_ref_count.end(); ++ref_it)
        {
            if (ck_impl && ref_it->second->m_ck && ck_impl->equals(ref_it->second->m_ck))
            {
                flag = true;
                // stl iterators are stable, except deletion
                while (ref_it->second->is_still_used() == true)
                {
                    storage.m_still_used_cond.wait();
                }
            }
        }

        if (flag == false)
        {
            CORBA::BAD_PARAM ex = CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMBADCookie,
                    CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

    }
    catch (::Components::ContainerPortableInterceptor::InvalidRegistration & ir)
    {
        throw ;
    }

    typename Interceptor_T::_ptr_type removed = 0;

    std::string key =
    create_the_copismap_key(ck_impl->port, ck_impl->operation);

    // remove copi

    typename InterceptorTrait<Interceptor_T>::map_t::iterator it = storage.m_copi_map.find(key);

    if(it != storage.m_copi_map.end())
    {
        typename InterceptorTrait<Interceptor_T>::cookie_to_copy_t::iterator cit =
        it->second->find(CookieKey_impl(ck_impl));

        if( cit != it->second->end() )
        {
            removed = cit->second;
            // delete the cookie
            delete cit->first.cookie;
            it->second->erase(cit);
        }
    }

    if(removed)
    {

        // remove RefCounter for the removed copi
        ref_count_t::iterator rCit = storage.m_copi_ref_count.find(removed);
        if(rCit != storage.m_copi_ref_count.end())
        {
            // delete RefCounter_impl
            delete rCit->second;
            storage.m_copi_ref_count.erase(rCit);
        }
        else
        {
            CORBA::BAD_PARAM ex = CORBA::BAD_PARAM (Cdmw::OrbSupport::BAD_PARAMBADCookie, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

        return Interceptor_T::_duplicate(removed); //FIXME should we check for null pointer?
    }
    else
    {
        ::Components::ContainerPortableInterceptor::InvalidRegistration ex =
        ::Components::ContainerPortableInterceptor::InvalidRegistration();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

// call under mtx protection 

template<typename Intercept_T>
typename InterceptorTrait<Intercept_T>::seq_t *
COPIRegistration_impl::getAllContainerInterceptors 
(
   COPIKindStorage<Intercept_T> & storage
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    typename InterceptorTrait<Intercept_T>::seq_t * retSeq =
    new typename InterceptorTrait<Intercept_T>::seq_t;

    unsigned int len = 0;

    for(typename InterceptorTrait<Intercept_T>::map_t::iterator it = storage.m_copi_map.begin(); it != storage.m_copi_map.end(); ++it)
    for (typename InterceptorTrait<Intercept_T>::cookie_to_copy_t::iterator cit = it->second->begin(); cit != it->second->end(); ++cit)
    {
        retSeq->length(++len);
        (*retSeq)[len-1] = Intercept_T::_duplicate(cit->second);
        if((*retSeq)[len-1] == 0)//FIXME is there possible case with a null pointer here?

        {
            CORBA::INTERNAL ex = CORBA::INTERNAL (Cdmw::OrbSupport::INTERNALBug, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }

    return retSeq;
}

template<typename Intercept_T>
typename InterceptorTrait<Intercept_T>::seq_t*
COPIRegistration_impl::getContainerPortableInterceptorsByIPOrderedByPriority
(
        const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip,
        COPIKindStorage<Intercept_T> & storage
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    std::string port;
    std::string operation;

    port = std::string(ip.port.in());
    operation = std::string(ip.operation.in());

    // return all copies if port/ip have value of ""

    std::string key1 = create_the_copismap_key(port, operation);
    typename InterceptorTrait<Intercept_T>::cookie_to_copy_t map;

    typename InterceptorTrait<Intercept_T>::map_t::iterator it = storage.m_copi_map.find(key1);

    if(it != storage.m_copi_map.end())
    {
        for (typename InterceptorTrait<Intercept_T>::cookie_to_copy_t::iterator cit = it->second->begin();
                cit != it->second->end(); ++cit)
        {
            map.insert(std::make_pair(cit->first, cit->second));
        }
    }

    // To add all the COPIs intercepting the given port whatever the
    // operation
    std::string key2 = create_the_copismap_key("", operation);

    if(key1 != key2)
    {
        it = storage.m_copi_map.find(key2);
        if (it != storage.m_copi_map.end())
        {
            for (typename InterceptorTrait<Intercept_T>::cookie_to_copy_t::iterator cit = it->second->begin();
                    cit != it->second->end(); ++cit)
            {
                map.insert(std::make_pair(cit->first, cit->second));
            }
        }
    }

    // To add all the COPIs intercepting any operation of any port (all
    // the operations)
    key2 = create_the_copismap_key("", "");

    if(key1 != key2)
    {
        it = storage.m_copi_map.find(key2);
        if (it != storage.m_copi_map.end())
        {
            for (typename InterceptorTrait<Intercept_T>::cookie_to_copy_t::iterator cit = it->second->begin();
                    cit != it->second->end(); ++cit)
            {
                map.insert(std::make_pair(cit->first, cit->second));
            }
        }
    }

    typename InterceptorTrait<Intercept_T>::seq_t * retSeq =
    new typename InterceptorTrait<Intercept_T>::seq_t();
    int len = 0;

    //putting all the copis in one map will reorder them
    for (typename InterceptorTrait<Intercept_T>::cookie_to_copy_t::iterator cit = map.begin(); cit != map.end(); ++cit)
    {
        retSeq->length(++len);
        (*retSeq)[len-1] = Intercept_T::_duplicate(cit->second);
    }
    return retSeq;
}

template<typename Interceptor_T>
void COPIRegistration_impl::destroy_copi_storage
(    
    COPIKindStorage<Interceptor_T> & storage
)
{

    for
    (
        typename InterceptorTrait<Interceptor_T>::map_t::iterator it = storage.m_copi_map.begin();
        it != storage.m_copi_map.end(); 
        ++it
    )
    {

        // delete cookies
        for (typename InterceptorTrait<Interceptor_T>::cookie_to_copy_t::iterator cit = it->second->begin();
                cit != it->second->end();
                ++cit)
        {
            delete cit->first.cookie;
        }

        delete it->second;
    }
}

template<typename Interceptor_T>
CORBA::Short COPIRegistration_impl::add_COPI_ref
(
        COPIRegistration_impl::container_interceptor_ptr_t copi,
        COPIKindStorage<Interceptor_T> & storage
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(storage.m_lock);

    ref_count_t::iterator it =
    storage.m_copi_ref_count.find(ObjectRefKey_impl(copi));

    if(it != storage.m_copi_ref_count.end())
    {
        it->second->add_ref();
    }
    else
    {
        CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "COPI Not Found. It was probably unregistered !");
        return 1;
    }

    return 0;
}

template<typename Interceptor_T>
CORBA::Short COPIRegistration_impl::remove_COPI_ref
(
        COPIRegistration_impl::container_interceptor_ptr_t copi,
        COPIKindStorage<Interceptor_T> & storage
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(storage.m_lock);

    ref_count_t::iterator it =
    storage.m_copi_ref_count.find(ObjectRefKey_impl(copi));

    if(it != storage.m_copi_ref_count.end())
    {
        it->second->remove_ref();
        storage.m_still_used_cond.broadcast();
    }
    else
    {
        CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "COPI Not Found. It was probably unregister !");
        return 1;
    }

    return 0;
}


} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

#endif /* COPIREGISTRATION_IMPL_INL_ */
