/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_CCM_COPI_COPISERVICEINIT_IMPL_HPP
#define CDMW_CCM_COPI_COPISERVICEINIT_IMPL_HPP

#include <string>
#include <orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <CCMContainer/ccmcopi/COPIMainORBInitializer.hpp>

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {


class COPIServiceInit {

public:
    /**
     * Purpose:
     * <p> Initialises the copi mechanism : it creates server
     * request interceptor and client request interceptor
     *
     *@param argc The number of parameter received of the command line
     *@param argv The list of parameter received of the command line
     *
     *@return none
     *
     *@exception CORBA::INTERNAL  The CDMW <B>INTERNAL</B> minor code indicates an
     * internal error occured within Cdmw Commmon library.
     *@exception CORBA::SystemException Any CORBA system exception that
     * may be raised by the underlying ORB.
     */
    static void
    Init (int& argc, char** argv)
        throw (CORBA::SystemException);

    static void
    Post_init(CORBA::ORB_ptr orb) 
        throw (CORBA::SystemException); 

    static void
    Destroy() 
        throw (CORBA::SystemException); 

    static bool 
    Is_initialized();
    
    static void 
    Enable_read_write_requests();
   
    static void 
    Disable_read_write_requests();

    static bool 
    Is_COPI_requests_read_write();
      

private:
    /*    
          private final static String ORBINITIALIZER_PROPERTY_NAME="org.omg.PortableInterceptor.ORBInitializerClass.COPI";
          private final static String ORBINITIALIZER_CLASS_NAME="cdmw.ccm.ccmcopi.COPIMainORBInitializer";
    
          private static Properties COPIServiceProps;
    */
    
    static bool M_initialized;


public:


    /**
     * The needed slots identifiers
     */
    const static std::string M_origin_ID_SlotNAME;
    const static std::string M_target_ID_SlotNAME;
    const static std::string M_originFeatureName_SlotNAME;
    const static std::string M_targetFeatureName_SlotNAME;
    const static std::string M_originCustomSlotItemSeq_SlotNAME;
    const static std::string M_targetCustomSlotItemSeq_SlotNAME;
 
    /** 
     * The COPI Service Context ID
     * TODO: to remove this ID and use the standard one  
     */
    static const unsigned int CDMW_COPI_SERVICE_CONTEXT_ID;
    
    /**
     * The Basic client copi service context Id
     * It is used to vehicle the request Id
     */
    static const int CDMW_CLIENT_COPI_SERVICE_CONTEXT_ID;
    
    /**
     * the name of the property that indicates whether the copis can modify the request
     * elements (parameters, return, target, ...) or not.
     */
    static const std::string READ_WRITE_REQUEST_PROPERTY_NAME;


private:
    static OsSupport::Mutex M_mtx;
    static Cdmw::CCM::COPI::COPIMainORBInitializer * M_initialiser;
    
};

} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

#endif // CDMW_CCM_COPI_COPISERVICEINIT_IMPL_HPP
