/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_CCM_COPI_CLIENT_REQUEST_INTERCEPTOR_ITERATOR_IMPL_HPP
#define INCL_CDMW_CCM_COPI_CLIENT_REQUEST_INTERCEPTOR_ITERATOR_IMPL_HPP

#include <map>
#include <vector>
#include <string>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <CCMContainer/ccmcopi/CopisStackHolder_impl.hpp>
#include <CCMContainer/ccmcopi/ContainerClientRequestInfo_impl.hpp>


namespace {


} // End anonymous space


namespace Cdmw
{
namespace CCM
{
namespace COPI
{

/**
*Purpose:
*<p>    This class implements the ClientRequestInterceptor class.
*
*Features:
*<p> Thread safe
*
*/
class ClientRequestInterceptorIterator_impl : public PortableInterceptor::ClientRequestInterceptor,
                                      virtual public Cdmw::OrbSupport::RefCountLocalObject
{
public:
    /**
     * Purpose:
     * <p> Constructor
     * outgoing request.
     *
     */ 
    ClientRequestInterceptorIterator_impl
    (
     int origin_id, 
     int target_id,
     int origin_feature_name_id, 
     int custom_slot_item_seq_id,  
     ::CdmwCcmCopi::RegisteredCOPICollection_ptr registration
     );



    /**
     * Purpose:
     * <p> Return the name of the interceptor
     *
     */
    virtual char*
    name() throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> Destroy the interceptor
     *
     */
    virtual void
    destroy() throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query request
     * information and modify the service context before the request is 
     * sent to the server. 
     * Here, the date of the outgoing request is stored here.
     * @param ri Client request information
     *
     */
    virtual void
    send_request(
        PortableInterceptor::ClientRequestInfo_ptr info)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    /**
     * Purpose:
     * <p>  This interception point allows an Interceptor to query information
     * during a Time-Independent Invocation (TII) polling get reply sequence.
     * @param ri Client request information
     *
     */
    virtual void
    send_poll(PortableInterceptor::ClientRequestInfo_ptr info)
        throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query the 
     * information on a reply after it is returned from the server and 
     * before control is returned to the client.
     * Here the date of the outgoing request is reset and T0 of incoming 
     * request updated to remove the duration of the outgoing request.
     * @param ri Client request information
     */
    virtual void
    receive_reply(PortableInterceptor::ClientRequestInfo_ptr info)
        throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> When an exception occurs, this interception point is called. 
     * It allows an Interceptor to query the exception's information before
     * it is raised to the client.
     * @param ri Client request information
     */
    virtual void
    receive_exception(PortableInterceptor::ClientRequestInfo_ptr info)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query the 
     * information available when a request results in something other 
     * than a normal reply or an exception. For example, a request could 
     * result in a retry (for example, a GIOP Reply with a LOCATION_FORWARD
     * status was received); or on asynchronous calls, the reply does not 
     * immediately follow the request, but control shall return to the client
     * and an ending interception point shall be called.
     * @param ri Client request information
     */
    virtual void
    receive_other(PortableInterceptor::ClientRequestInfo_ptr info)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

private:
    
    ContainerClientRequestInfo_impl *
    build_container_client_request_info
    (
     ::PortableInterceptor::ClientRequestInfo_ptr info
     );

    ContainerClientRequestInfo_impl *
    build_container_client_request_info
    (
     ::PortableInterceptor::ClientRequestInfo_ptr info,
     CORBA::OctetSeq * target_id
     );

    void 
    build_request_service_context
    (
     ::PortableInterceptor::ClientRequestInfo_ptr info
     );
    
    void 
    build_request_service_context
    (
     ::PortableInterceptor::ClientRequestInfo_ptr info,
     const CORBA::ULong & request_id
     );

    CORBA::OctetSeq *
    extract_from_reply_service_context
    (
     ::PortableInterceptor::ClientRequestInfo_ptr info
     );

    CopisStackHolder_impl< 
        ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr> *
    get_copis_stack
    (
     unsigned long request_id
     );

    void
    remove_copis_stack
    (
     unsigned long request_id
     );

    void
    set_copis_stack
    (
     unsigned long request_id,
     CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr> * copis_stack
     );

    void 
    update_state_after_send
    (
     CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr> * req_dat,
     CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>::copis_list_t * safe_interceptors,
     PortableInterceptor::ClientRequestInfo_ptr info
     );

    enum sending_ip
        {
            SEND_REQUEST,
            SEND_POLL
        };
     
    void
    do_sending_ip
    (
     PortableInterceptor::ClientRequestInfo_ptr info, sending_ip sip
     )
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    enum receiving_ip
        {
            RECEIVE_REPLY,
            RECEIVE_EXCEPTION,
            RECEIVE_OTHER
        };

    void
    do_receiving_ip
    (
     PortableInterceptor::ClientRequestInfo_ptr info, receiving_ip rip
     )
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);


    CORBA::ULong
    extract_req_id_from_reply_service_context
    (
     ::PortableInterceptor::ClientRequestInfo_ptr info
     );

    std::string octet_seq_2_std_string(const CORBA::OctetSeq * oct_seq);

    CopisStackHolderTrait< 
        ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>::copis_list_t *
    get_safe_client_interceptors
    (
     const ::CdmwCcmCopi::ClientContainerInterceptorSeq & interceptors
     );

    /**
     * needed slots ids for the COPI service :
     *  1- The client component identity
     *  2- The server component identity
     *  3- The client component port name (receptacle or source)
     *  4- The Custom slots     
     */
    const int m_origin_id_slot_id;
    const int m_target_id_slot_id;
    const int m_origin_feature_name_slot_id;
    const int m_custom_slot_item_seq_slot_id;
        
    /**
     * 
     * Request specific context. Each request has its own context 
     */
    typedef std::map< ::CORBA::ULong, CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr> *> request_context_t;
    request_context_t m_request_context;  

    ::CdmwCcmCopi::RegisteredCOPICollection_var m_registration;

    static CORBA::ULong Get_request_id();   
    static const std::string M_name;
    static CORBA::ULong M_request_id;
    static OsSupport::Mutex M_mtx;
    
    ClientRequestInterceptorIterator_impl(const ClientRequestInterceptorIterator_impl&);
    ClientRequestInterceptorIterator_impl& operator=(const ClientRequestInterceptorIterator_impl&);

};

} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw

#endif // INCL_CDMW_CCM_COPI_CLIENT_REQUEST_INTERCEPTOR_ITERATOR_IMPL_HPP

