/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_CCM_COPI_CONTAINEREXTENDEDCOPISREQUEST_IMPL_HPP
#define CDMW_CCM_COPI_CONTAINEREXTENDEDCOPISREQUEST_IMPL_HPP

#include <string>


#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <CCMContainer/ccmcif/Context.hpp>

#include <CCMContainer/idllib/CdmwCcmCopi.stub.hpp>
#include <CCMContainer/ccmcopi/ContainerRequest_impl.hpp>
#include <CCMContainer/ccmcopi/ContainerStubRequestInfo_impl.hpp>
#include <CCMContainer/ccmcopi/StubContainerInterceptorIterator_impl.hpp>

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {



/**
 * represents an outcoming request
 */
template<typename T>
class ContainerExtendedCopisRequest_impl : public virtual ContainerRequest_impl {
    

public:    
    
    ContainerExtendedCopisRequest_impl
    (
     ::Cdmw::CCM::CIF::Context * ctx
    );


    virtual
    ~ContainerExtendedCopisRequest_impl
    ();
    
    // need initialise because ctor can't call virtual 
    // functions
    void initialise
    (
     const char * name, 
     const char * component_id,
     ::CORBA::Object_ptr target
     );
            
    void add_in_args (const ::CORBA::Any & result);
    
    ::CORBA::Any * get_arg (unsigned int index);

    ::CORBA::Any * get_arg (const ::Dynamic::ParameterList & arguments, unsigned int index);
            
    void add_out_args (const ::CORBA::Any & out_arg);
    
    void add_inout_args (const ::CORBA::Any & inout_arg);
    
    void add_result(const ::CORBA::Any & res);
    
    void add_exception(const ::CORBA::Any &  ex);
        
    ::CORBA::Any * the_exception ();
 
    void add_system_exception(::CORBA::SystemException & se);
    
    ::CORBA::SystemException * the_systemException();
    
    ::CORBA::Any * result ();
    


    ::CORBA::Boolean m_proceed_call;

protected:
    Cdmw::Logging::Logger * m_logger;

    T * m_request_info;

    /**
     * used to set the slots
     */
    ::PortableInterceptor::Current_var m_pi_current;

    /**
     * used to retrieve the slots ids
     */
    ::CdmwCcmCopi::COPICurrent_var m_copi_current;

    ::Cdmw::CCM::CIF::Context * m_context;    
    
private:
    ContainerExtendedCopisRequest_impl(const ContainerExtendedCopisRequest_impl &);
    ContainerExtendedCopisRequest_impl & operator=(const ContainerExtendedCopisRequest_impl &);

    void do_add_args
    (
     const ::CORBA::Any & arg,
     ::CORBA::ParameterMode mode
     ); 

};


} // End namespace Cdmw

} // End namespace CCM

} // End namespce COPI

#endif // CDMW_CCM_COPI_CONTAINEREXTENDEDCOPISREQUEST_IMPL_HPP
