/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_CCM_COPI_CONTAINERREQUEST_IMPL_HPP
#define CDMW_CCM_COPI_CONTAINERREQUEST_IMPL_HPP

#include <vector>
#include <Foundation/idllib/Components.stub.hpp>


namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {

class ContainerRequest_impl {
    
public:
    
    // ctor
    ContainerRequest_impl
    ();


    // dtor
    virtual
    ~ContainerRequest_impl
    ();
    
    /**
     * implemented by generated code (the typed request object) to return the operation name
     * @return the operation name
     */
    virtual
    char * operation() = 0;
    
    /**
     * checks the READ-WRITE-ability of the request; whether the COPIs can modify the request parameters or not
     * @return true if the request is RW 
     */
    bool is_read_write();

    /**
     * adds a given exception Type Code. This TC is used to build a DII request
     * @param tc the exception type code
     */
    virtual
    void add_exception_tc(::CORBA::TypeCode_ptr tc);


protected:
       
    virtual
    const ::CORBA::Any * get_arg 
    (
     const ::Dynamic::ParameterList & arguments,
     unsigned int index
     );

    std::vector< ::CORBA::TypeCode_var > m_exceptions_type_codes;

private:   
    ContainerRequest_impl(const ContainerRequest_impl &);
    ContainerRequest_impl & operator=(const ContainerRequest_impl&);
};



} // End namespace Cdmw

} // End namespace CCM

} // End namespce COPI

#endif // CDMW_CCM_COPI_CONTAINERREQUEST_IMPL_HPP

