/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_CCM_COPI_COPIS_STACK_HOLDER_HPP
#define CDMW_CCM_COPI_COPIS_STACK_HOLDER_HPP

#include <Foundation/idllib/Components.skel.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {

template<typename T>
class CopisStackHolderTrait
{
    //No body here to fail at compile time if used with wrong type
};

template<>
class CopisStackHolderTrait<
        ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>
{
public:
    typedef std::vector< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr > copis_list_t;
};

template<>
class CopisStackHolderTrait<
        ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr>
{
public:
    typedef std::vector< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr > copis_list_t;
};

template<>
class CopisStackHolderTrait<
        CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>
{
public:
    typedef std::vector< CdmwCcmContainer::HomeLifeCycleInterceptor_ptr > copis_list_t;
};

template<>
class CopisStackHolderTrait< CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr
        >
{
public:
    typedef std::vector< CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr > copis_list_t;
};


template<typename T>
class CopisStackHolder_impl {

public:
    CopisStackHolder_impl() :
        m_RECEIVING_state_interceptor_index(-1),
        m_SENDING_state_interceptor_index(-1),
        m_copis_list(0)
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    };

    ~CopisStackHolder_impl() {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        // empty
    };

    int get_recv_idx() {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        return this->m_RECEIVING_state_interceptor_index;
    };
       
    void set_recv_idx(int idx) {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        if(idx < -1) {
            CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "bad index");
            CORBA::INTERNAL ex = CORBA::INTERNAL (Cdmw::OrbSupport::INTERNALBug, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        this->m_RECEIVING_state_interceptor_index = idx;
    };

    int get_send_idx() {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        return this->m_SENDING_state_interceptor_index;
    };
       
    void set_send_idx(int idx) {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        if(idx < -1) {
            CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "bad index");
            CORBA::INTERNAL ex = CORBA::INTERNAL (Cdmw::OrbSupport::INTERNALBug, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        this->m_SENDING_state_interceptor_index = idx;
    };

    void set_copis(typename CopisStackHolderTrait<T>::copis_list_t * copis) {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        if(this->m_copis_list) {
            delete m_copis_list;
            m_copis_list = 0;
        }
        this->m_copis_list = copis;
    };

    
    typename CopisStackHolderTrait<T>::copis_list_t *
    get_copis() {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        if(0 == m_copis_list) {
            m_copis_list =
                new typename CopisStackHolderTrait<T>::copis_list_t();
        }

        return this->m_copis_list;           
    };


        
protected:
    CopisStackHolder_impl(const CopisStackHolder_impl&);
    CopisStackHolder_impl& operator=(const CopisStackHolder_impl&);

    // NOTE: value -1 means that the indices haven't been set yet

    /**
     * A Client COPI enters the RECEIVING state whenever it process one of the
     * receive_* operations successfully.
     * it gives the index (in the client COPI list) of the last Client COPI that
     * processes a receiving end point
     * successfully. All greater indexes are in the RECEIVING state; due to the
     * fact that receiving end points are called in reverse order
     */
    int m_RECEIVING_state_interceptor_index;

    /**
     * A Client COPI enters the SENDING state whenever it processes one the
     * send_* operations successfully.
     * it gives the index (in the client COPI
     * list) of the last Client COPI that processes send_* successfully. All
     * lesser indexes are in the SENDING state.
     */
    int m_SENDING_state_interceptor_index;

    typename CopisStackHolderTrait<T>::copis_list_t * m_copis_list;
};
    


} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

#endif // CDMW_CCM_COPI_COPIS_STACK_HOLDER_HPP
