/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_CCM_COPI_SERVANT_CONTAINER_INTERCEPTOR_ITERATOR_IMPL_HPP
#define INCL_CDMW_CCM_COPI_SERVANT_CONTAINER_INTERCEPTOR_ITERATOR_IMPL_HPP

#include <vector>
#include <Foundation/idllib/Components.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>

namespace {


} // End anonymous space


namespace Cdmw
{
namespace CCM
{
namespace COPI
{

typedef std::vector< ::Components::ContainerPortableInterceptor::ServantContainerInterceptor_ptr > servant_copis_list_t;

class ServantContainerInterceptorIterator_impl {
public:

    ServantContainerInterceptorIterator_impl 
    (
     ::CdmwCcmCopi::ServantContainerInterceptorSeq *servant_interceptors, 
     ::CdmwCcmCopi::RegisteredCOPICollection_ptr registration_util
     );


    virtual
    ~ServantContainerInterceptorIterator_impl();

    virtual
    void servant_receive_request 
    (
     ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
     ::CORBA::Boolean & proceed_call
     )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               );

    virtual
    void servant_send_reply 
    (
     ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
     ::CORBA::Boolean & proceed_call
     )
        throw (
               ::CORBA::SystemException
               );

    virtual
    void servant_send_exception 
    (
     ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
     ::CORBA::Boolean & proceed_call
     )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               );

    virtual
    void servant_send_other 
    (
     ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info
     )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               );
    
    
private:
    ServantContainerInterceptorIterator_impl(const ServantContainerInterceptorIterator_impl&);
    ServantContainerInterceptorIterator_impl& operator=(const ServantContainerInterceptorIterator_impl&);
    
    /**
     * The list of registered stub interceptors
     */   
    ::CdmwCcmCopi::ServantContainerInterceptorSeq_var m_servant_interceptors;
    ::CdmwCcmCopi::RegisteredCOPICollection_var m_copis_list;
    
    
    /**
     * A Stub COPI enters the SENDING state whenever it processes
     * stub_send_reuqest operations successfully. it gives the index (in the stub COPI
     * list) of the last Server COPI that has already processed a sending end
     * point. All lesser indexes are in the SENDING state
     */
    int m_sending_state_interceptor_index;
    
    /**
     * A stub COPI enters the RECEIVING state whenever it processes one of the
     * stub_receive_* operations successfully. it gives the
     * index (in the server COPI list) of the last Server COPI that processes
     * stub_receive_* successfully.
     */
    int m_receiving_state_interceptor_index;
    
    servant_copis_list_t * m_safe_interceptors;

    servant_copis_list_t *
    get_safe_servant_interceptors
    (
     const CdmwCcmCopi::ServantContainerInterceptorSeq & interceptors
     );
    
};



} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw

#endif // INCL_CDMW_CCM_COPI_SERVANT_CONTAINER_INTERCEPTOR_ITERATOR_IMPL_HPP
