/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_CCM_COPI_SERVER_REQUEST_INTERCEPTOR_ITERATOR_IMPL_HPP
#define INCL_CDMW_CCM_COPI_SERVER_REQUEST_INTERCEPTOR_ITERATOR_IMPL_HPP

// Cdmw Files

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/CdmwServerRequestInterceptor.hpp>
#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <CCMContainer/ccmcopi/CopisStackHolder_impl.hpp>
#include <CCMContainer/ccmcopi/ContainerServerRequestInfo_impl.hpp>

namespace {


} // End anonymous space


namespace Cdmw
{
namespace CCM
{
namespace COPI
{

/**
*Purpose:
*<p>    This class implements the ServerRequestInterceptor class.
*
*Features:
*<p> Thread safe
*
*/
class ServerRequestInterceptorIterator_impl : public Cdmw::OrbSupport::CdmwServerRequestInterceptor
{
public:

    /**
     * Purpose:
     * <p> Constructor
     * incoming request.
     *
     */ 
    ServerRequestInterceptorIterator_impl
    (
     int origin_id, 
     int target_id,
     int target_feature_name_id, 
     int custom_slot_item_seq_id,
     ::CdmwCcmCopi::RegisteredCOPICollection_ptr registration
     );

    /**
     * Purpose:
     * <p> Return the name of the interceptor
     *
     */
    virtual char*
    name() throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> Destroy the interceptor
     *
     */
    virtual void
    destroy() throw (CORBA::SystemException);

    

    /**
     * Purpose:
     * <p> This is a -specific operation to enable returning previous
     * logged replies. This operation is used in an ftrequestlogging CSC 
     * and then the code shall be located here if we want to set a slot.
     *
     */ 
    void
    cdmw_ft_interception_point(
      PortableInterceptor::ServerRequestInfo_ptr ri,
      CORBA::OctetSeq_out ocs)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);


    /**
     * Purpose:
     * <p> At this interception point, Interceptors must get their service
     * context information from the incoming request and transfer it to 
     * PortableInterceptor::Current's slots.
     * @param ri Server request information
     */
    virtual void
    receive_request_service_contexts(
        PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query request 
     * information after all the information, including operation parameters,
     * are available.
     * @param ri Server request information
     */
    virtual void
    receive_request(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query reply 
     * information and modify the reply service context after the target
     * operation has been invoked and before the reply is returned to the
     * client.
     * @param ri Server request information
     */
    virtual void
    send_reply(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException);

    /**
     * Purpose:
     * <p> This interception point allows an Interceptor to query the 
     * information available when a request results in something other
     * than a normal reply or an exception. A request could result in 
     * a retry (for example, a GIOP Reply with a LOCATION_FORWARD status
     * was received).
     * @param ri Server request information
     */
    virtual void
    send_other(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

    /**
     * Purpose:
     * <p> When an exception occurs, this interception point is called. 
     * It allows an Interceptor to query the exception information and 
     * modify the reply service context before the exception is raised 
     * to the client.
     * @param ri Server request information
     */
    virtual void
    send_exception(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);

private:

    ContainerServerRequestInfo_impl *
    build_container_server_request_info
    (
     ::PortableInterceptor::ServerRequestInfo_ptr info
     );
    
    void
    extract_from_request_service_context
    (
     ::PortableInterceptor::ServerRequestInfo_ptr info
     );

    int 
    extract_req_id_from_request_service_context
    (
     ::PortableInterceptor::ServerRequestInfo_ptr info,
     CORBA::ULong * request_id 
     );

    void 
    build_reply_service_context
    (
     ::PortableInterceptor::ServerRequestInfo_ptr info
     );

    void 
    extract_feature_name
    (
     ::PortableInterceptor::ServerRequestInfo_ptr info
     );

    CopisStackHolder_impl< 
        ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr > *
    get_copis_stack
    (
     CORBA::ULong request_id
     );

    void
    remove_copis_stack
    (
     CORBA::ULong request_id
     );

    void
    set_copis_stack
    (
     CORBA::ULong request_id,
     CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr> * copis_stack
     );

    void 
    update_state_after_send
    (
     CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr > * req_dat,
     CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr >::copis_list_t * safe_interceptors,
     PortableInterceptor::ServerRequestInfo_ptr info
     );

    CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr >::copis_list_t *
    get_safe_server_interceptors
    (
     const ::CdmwCcmCopi::ServerContainerInterceptorSeq * interceptors
     );

    enum sending_ip
        {
            SEND_REPLY,
            SEND_EXCEPTION,
            SEND_OTHER
        };
     
    void
    do_sending_ip
    (
     PortableInterceptor::ServerRequestInfo_ptr info, sending_ip sip
     )
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest);


    /**
     * needed slots ids for the COPI service :
     *  1- The client component identity
     *  2- The server component identity
     *  3- The client component port name (receptacle or source)
     *  4- The Custom slots     
     */
    int m_origin_id_slot_id;
    int m_target_id_slot_id;
    int m_target_feature_name_slot_id;
    int m_custom_slot_item_seq_slot_id;

    /**
     * 
     * Request specific context. Each request has its own context 
     */
    typedef std::map<CORBA::ULong, 
                     CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr> *> request_context_t;
    request_context_t m_request_context;  


    ::CdmwCcmCopi::RegisteredCOPICollection_var m_registration;

    static const std::string M_name;
    static OsSupport::Mutex M_mtx;

    ServerRequestInterceptorIterator_impl(
          const ServerRequestInterceptorIterator_impl&);

    ServerRequestInterceptorIterator_impl& operator=(
          const ServerRequestInterceptorIterator_impl&);
};


} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw

#endif // INCL_CDMW_CCM_COPI_SERVER_REQUEST_INTERCEPTOR_ITERATOR_IMPL_HPP

