/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "CCMContainer/ccmcontainer/ApplicationElementsContextService.hpp"
#include "ComponentElementContext_impl.hpp"
#include "HomeElementContext_impl.hpp"


#ifndef APPLICATIONELEMENTCONTEXTSERVICEFACTORIES_HPP_
#define APPLICATIONELEMENTCONTEXTSERVICEFACTORIES_HPP_

namespace Cdmw
{
namespace CCM
{
namespace Container
{

class ApplicationElementsContextService::ComponentElementContextFactory: public Cdmw::OrbSupport::RefCountLocalObject,
                                                                         public CdmwCcmContainer::ComponentLifeCycleInterceptor,
                                                                         public AbstractApplicationElementContextFactory
{
private:
    Cdmw::Logging::Logger* m_logger;
    CdmwCcmContainer::LifeCycleInterceptorRegistration_ptr m_registration;

    typedef std::map<std::string, ComponentElementContext_impl*> ComponentElementContextMap;
    ComponentElementContextMap m_componentElementContextsMap;

    Components::Cookie* m_ck;
public:
    explicit ComponentElementContextFactory(CdmwCcmContainer::LifeCycleInterceptorRegistration_ptr reg);

    virtual ~ComponentElementContextFactory();

    CdmwCcmContainer::ApplicationElementContext* create_element_context(const std::string& id,
                CORBA::Object_ptr ref,  const Components::ConfigValues& configValues);

    virtual CdmwCcmContainer::ApplicationElementContext* find_element_context_by_id(const std::string& id) const;

    void remove_element_context(const std::string& id);

    void clear();

    void after_component_activate(const char* id) throw (CORBA::SystemException);
    void after_component_connect(const char* id) throw (CORBA::SystemException);
    void after_component_create(const char* id, CORBA::Object_ptr ref) throw (CORBA::SystemException);
    void after_component_passivate(const char* arg0) throw (CORBA::SystemException);
    void after_component_remove(const char* id) throw (CORBA::SystemException);

    void before_component_activate(const char* id) throw (CORBA::SystemException);
    void before_component_connect(const char* id, const char* port, CORBA::Object_ptr ref) throw (CORBA::SystemException);
    void before_component_create(const char* id, Components::ConfigValues& configuration) throw (CORBA::SystemException);
    void before_component_passivate(const char* id) throw (CORBA::SystemException);
    void before_component_remove(const char* id) throw (CORBA::SystemException);

    void destroy() throw (::CORBA::SystemException);
    char* name() throw (::CORBA::SystemException);
    CORBA::UShort priority() throw (::CORBA::SystemException);
    Components::ContainerPortableInterceptor::IntegrationPoint* registration_info() throw (::CORBA::SystemException);
    void set_slot_id(::PortableInterceptor::SlotId slot_id) throw (::CORBA::SystemException);
};

class ApplicationElementsContextService::HomeElementContextFactory: public Cdmw::OrbSupport::RefCountLocalObject,
                                                                    public CdmwCcmContainer::HomeLifeCycleInterceptor,
                                                                    public AbstractApplicationElementContextFactory
{
private:
    Cdmw::Logging::Logger* m_logger;
    CdmwCcmContainer::LifeCycleInterceptorRegistration_ptr m_registration;

    typedef std::map<std::string, HomeElementContext_impl*> HomeElementContextMap;
    HomeElementContextMap m_homeElementContextsMap;

    Components::Cookie* m_ck;

public:
    explicit HomeElementContextFactory(CdmwCcmContainer::LifeCycleInterceptorRegistration_ptr reg);

    virtual ~HomeElementContextFactory();


    CdmwCcmContainer::ApplicationElementContext* create_element_context(const std::string& id,
                CORBA::Object_ptr ref,  const Components::ConfigValues& configValues);

    CdmwCcmContainer::ApplicationElementContext* find_element_context_by_id(const std::string& id) const;

    void remove_element_context(const std::string& id);

    void clear();

    void after_home_create(const char* id, CORBA::Object_ptr ref) throw (CORBA::SystemException);
    void after_home_remove(const char* id) throw (CORBA::SystemException);
    void before_home_create(const char* id, Components::ConfigValues& configuration, const char* entrypoint) throw (CORBA::SystemException);
    void before_home_remove(const char* id) throw (CORBA::SystemException);

    void destroy() throw (::CORBA::SystemException);
    char* name() throw (::CORBA::SystemException);
    CORBA::UShort priority() throw (::CORBA::SystemException);
    Components::ContainerPortableInterceptor::IntegrationPoint* registration_info() throw (::CORBA::SystemException);
    void set_slot_id(::PortableInterceptor::SlotId slot_id) throw (::CORBA::SystemException);
};

}
}
}
#endif /* APPLICATIONELEMENTCONTEXTSERVICEFACTORIES_HPP_ */
