/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>
#include "ComponentLifeCycleInterceptorIterator.hpp"
#include "Foundation/logging/FunctionLogger.hpp"
#include "CCMContainer/ccmcommon/CCMLoggers.hpp"

namespace Cdmw
{
namespace CCM
{
namespace Container
{

void ComponentLifeCycleInterceptorIterator::invoke_ip
(
 CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr interceptor,
 InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t arg
) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    switch(arg.m_ip) {
    case AFTER_COMPONENT_ACTIVATE: {
        interceptor->after_component_activate(arg.m_compId);  
        break;
    }
    case AFTER_COMPONENT_CONNECT: {
        interceptor->after_component_connect(arg.m_compId);  
        break;
    }
    case AFTER_COMPONENT_CREATE: {
        interceptor->after_component_create(arg.m_compId, arg.m_compRef);  
        break;
    }
    case AFTER_COMPONENT_PASSIVATE: {
        interceptor->after_component_passivate(arg.m_compId);  
        break;
    }
    case AFTER_COMPONENT_REMOVE: {
        interceptor->after_component_remove(arg.m_compId);  
        break;
    }
    case BEFORE_COMPONENT_ACTIVATE: {
        interceptor->before_component_activate(arg.m_compId);  
        break;
    }
    case BEFORE_COMPONENT_CONNECT: {
        interceptor->before_component_connect(arg.m_compId, arg.m_featureName, arg.m_compRef);  
        break;
    }
    case BEFORE_COMPONENT_CREATE: {
        CDMW_ASSERT(arg.m_values);
        interceptor->before_component_create(arg.m_compId, *(arg.m_values));  
        break;
    }
    case BEFORE_COMPONENT_PASSIVATE: {
        interceptor->before_component_passivate(arg.m_compId);  
        break;
    }
    case BEFORE_COMPONENT_REMOVE: {
        interceptor->before_component_remove(arg.m_compId);  
        break;
    }           
    default: {
        CORBA::INTERNAL ex = CORBA::INTERNAL (Cdmw::OrbSupport::INTERNALBug, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }                                                                  
    }   
}

InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::interceptor_seq_t *
ComponentLifeCycleInterceptorIterator::get_interceptors() {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    ::Components::ContainerPortableInterceptor::IntegrationPoint ip;
    ip.operation = ::CORBA::string_dup("");
    ip.port = ::CORBA::string_dup("");
    
    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::interceptor_seq_var_t  interceptors = 
          m_registration->getComponentLifeCycleInterceptorByIPOrderedByPriority(ip);

    return interceptors._retn();
}

::CORBA::UShort
ComponentLifeCycleInterceptorIterator::add_interceptor_ref(CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr interceptor) {   
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_registration->addComponentLifeCycleCOPIRef(interceptor);
}

::CORBA::UShort
ComponentLifeCycleInterceptorIterator::remove_interceptor_ref(CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr interceptor) {   
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_registration->removeComponentLifeCycleCOPIRef(interceptor);
}


ComponentLifeCycleInterceptorIterator::ComponentLifeCycleInterceptorIterator(Cdmw::CCM::CIF::CCMHome_impl* ref) :
    m_logger(CCMLoggers::Get_ccmcontainer_logger()), m_compIdentityCreator(ref)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
}

void ComponentLifeCycleInterceptorIterator::after_component_activate(const char* compId)
{
    CDMW_LOG_FUNCTION(m_logger, "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t 
        arg(AFTER_COMPONENT_ACTIVATE, compId);
    
    do_ending_ip(arg);
}

void ComponentLifeCycleInterceptorIterator::after_component_connect(const char* compId)
{
    CDMW_LOG_FUNCTION(m_logger, "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t 
        arg(AFTER_COMPONENT_CONNECT, compId);
    
    do_ending_ip(arg);
}

void ComponentLifeCycleInterceptorIterator::after_component_create(const char* compId, CORBA::Object_ptr compRef)
{
    CDMW_LOG_FUNCTION(m_logger, "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t 
        arg(AFTER_COMPONENT_CREATE, compId, compRef);
    
    do_ending_ip(arg);
}

void ComponentLifeCycleInterceptorIterator::after_component_passivate(const char* compId)
{
    CDMW_LOG_FUNCTION(m_logger, "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t 
        arg(AFTER_COMPONENT_PASSIVATE, compId);
    
    do_ending_ip(arg);
}

void ComponentLifeCycleInterceptorIterator::after_component_remove(const char* compId)
{
    CDMW_LOG_FUNCTION(m_logger, "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t 
        arg(AFTER_COMPONENT_REMOVE, compId);
    
    do_ending_ip(arg);
}

void ComponentLifeCycleInterceptorIterator::before_component_activate(const char* compId)
{
    CDMW_LOG_FUNCTION(m_logger, "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t 
        arg(BEFORE_COMPONENT_ACTIVATE, compId);
    
    do_starting_ip(arg);
}

void ComponentLifeCycleInterceptorIterator::before_component_connect(const char* compId, const char* featureName,
        CORBA::Object_ptr compRef)
{
    CDMW_LOG_FUNCTION(m_logger, "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t 
        arg(BEFORE_COMPONENT_CONNECT, compId, featureName, compRef);
    
    do_starting_ip(arg);
}

void ComponentLifeCycleInterceptorIterator::before_component_create(const char* compId,
        Components::ConfigValues& compConfigVals)
{
    CDMW_LOG_FUNCTION(m_logger, "");
    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t 
        arg(BEFORE_COMPONENT_CREATE, compId, compConfigVals);
    
    do_starting_ip(arg);
}

void ComponentLifeCycleInterceptorIterator::before_component_passivate(const char* compId)
{
    CDMW_LOG_FUNCTION(m_logger, "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t 
        arg(BEFORE_COMPONENT_PASSIVATE, compId);
    
    do_starting_ip(arg);
}

void ComponentLifeCycleInterceptorIterator::before_component_remove(const char* compId)
{
    CDMW_LOG_FUNCTION(m_logger, "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t 
        arg(BEFORE_COMPONENT_REMOVE, compId);
    
    do_starting_ip(arg);
}

void ComponentLifeCycleInterceptorIterator::destroy() throw (::CORBA::SystemException)
{

}

char* ComponentLifeCycleInterceptorIterator::name() throw (::CORBA::SystemException)
{
    return ::CORBA::string_dup("ComponentLifecycleIterator");
}

CORBA::UShort ComponentLifeCycleInterceptorIterator::ComponentLifeCycleInterceptorIterator::priority()
                                                                                                       throw (::CORBA::SystemException)
{
    return 0;
}

Components::ContainerPortableInterceptor::IntegrationPoint* ComponentLifeCycleInterceptorIterator::registration_info()
                                                                                                                       throw (::CORBA::SystemException)
{
    return 0;
}

void
ComponentLifeCycleInterceptorIterator::set_slot_id(::PortableInterceptor::SlotId slot_id) throw (::CORBA::SystemException)
{

}

} // End of namespace CIF
} // End of namespace CCM
} // End of namespace Cdmw

