/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef COMPONENTLIFECYCLEINTERCEPTORITERATOR_HPP_
#define COMPONENTLIFECYCLEINTERCEPTORITERATOR_HPP_

#include <vector>
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/idllib/Components.stub.hpp"
#include "Foundation/logging/Logger.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "CCMContainer/idllib/CdmwCcmContainer.stub.hpp"
#include "CCMContainer/ccmcif/CCMHome_impl.hpp"
#include "InterceptorIteratorStackSupport_impl.hpp"

namespace Cdmw
{
namespace CCM
{
namespace Container
{


/**
 * call operations on registered lifecycle interceptors before/after operations on components
 */
//FIXME ContainerInterceptor? ComponentLifeCycleInterceptorOperations?
class ComponentLifeCycleInterceptorIterator: 
        public Components::ContainerPortableInterceptor::ContainerInterceptor,
        public virtual InterceptorIteratorStackSupport_impl<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>
{
private:
    typedef std::vector<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr> InterceptorList;

    Cdmw::Logging::Logger* m_logger;

    InterceptorList m_interceptors;

    Cdmw::CCM::CIF::CCMHome_impl* m_compIdentityCreator;

public:
    ///Ctor
    ComponentLifeCycleInterceptorIterator(Cdmw::CCM::CIF::CCMHome_impl* ref);

    void after_component_activate(const char* compId);
    void after_component_connect(const char* compId);

    void after_component_create(const char* compId, CORBA::Object_ptr compRef);

    void after_component_passivate(const char* compId);

    void after_component_remove(const char* compId);
    void before_component_activate(const char* compId);

    void before_component_connect(const char* compId, const char* featureName, CORBA::Object_ptr compRef);

    //FIXME ConfigValues? ConfigValuesHolder?
    void before_component_create(const char* compId, Components::ConfigValues& compConfigVals);

    void before_component_passivate(const char* compId);
    void before_component_remove(const char* compId);

    void destroy() throw (::CORBA::SystemException);
    char* name() throw (::CORBA::SystemException);
    CORBA::UShort priority() throw (::CORBA::SystemException);
    Components::ContainerPortableInterceptor::IntegrationPoint* registration_info() throw (::CORBA::SystemException);
    void set_slot_id(::PortableInterceptor::SlotId slot_id) throw (::CORBA::SystemException);

protected:
    void invoke_ip
    (
     CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr interceptor,
     InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::arg_t arg
     );

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>::interceptor_seq_t *
    get_interceptors();

    ::CORBA::UShort
    add_interceptor_ref(CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr interceptor);
    
    ::CORBA::UShort
    remove_interceptor_ref(CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr interceptor);
};

}
}
}

#endif /* COMPONENTLIFECYCLEINTERCEPTORITERATOR_HPP_ */
