/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "ComponentServerElementContext_impl.hpp"
#include "CCMContainer/ccmcontainer/ServiceRegistry.hpp"
#include "Foundation/idllib/Components.stub.hpp"
#include "Foundation/logging/Logger.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "CCMContainer/ccmcommon/CCMLoggers.hpp"
#include "CCMContainer/ccmcommon/CCMId.hpp"

namespace Cdmw
{
namespace CCM
{
namespace Container
{

Components::NameList*
ComponentServerElementContext_impl::get_installed_services() throw (::CORBA::SystemException )
{
    //CdmwCcmContainer::ServicesIds* and Components::NameList* provide the same implementation and both inherit from TAO_Unbounded_String_Sequence
    return reinterpret_cast<Components::NameList*> (ServiceRegistry::get_instance()->list());
}

CORBA::Object_ptr ComponentServerElementContext_impl::ref() throw (CORBA::SystemException )
{
    Components::Deployment::ComponentServer_ptr csRef = 0;
    try
    {
        CORBA::Object_ptr obj = ServiceRegistry::get_instance()->lookup(
                CCM::Common::CCMId::COMPONENT_SERVER_REF_SERVICE_PROVIDER);
        csRef = Components::Deployment::ComponentServer::_narrow(obj);

    }
    catch (Components::CCMException ex)
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcontainer_logger(), "Failed to retrieve the ComponentServer reference");
    }
    if (0 == csRef)
    {
        CORBA::INTERNAL ex = CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_YES);
        throw ex;
    }
    return csRef;
}

}// End namespace Container
}// End namespace CCM
}// End namespace Cdmw
