/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>
#include "HomeLifeCycleInterceptorIterator.hpp"


namespace Cdmw
{
namespace CCM
{
namespace Container
{

void HomeLifeCycleInterceptorIterator::invoke_ip
(
 CdmwCcmContainer::HomeLifeCycleInterceptor_ptr interceptor,
 InterceptorIteratorStackSupportTrait<CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>::arg_t arg
) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    switch(arg.m_ip) {
    case BEFORE_HOME_CREATE: {
        CDMW_ASSERT(arg.m_values);
        interceptor->before_home_create(arg.m_homeId, *(arg.m_values), arg.m_entryPoint);  
        break;
    }
    case AFTER_HOME_CREATE: {
        interceptor->after_home_create(arg.m_homeId, arg.m_homeRef);
        break;
    }
    case BEFORE_HOME_REMOVE: {
        interceptor->before_home_remove(arg.m_homeId);  
        break;
    }
    case AFTER_HOME_REMOVE: {
        interceptor->after_home_remove(arg.m_homeId);
        break;
    }        
    default: {
        CORBA::INTERNAL ex = CORBA::INTERNAL (Cdmw::OrbSupport::INTERNALBug, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }                                                                  
    }   
}

InterceptorIteratorStackSupportTrait<CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>::interceptor_seq_t *
HomeLifeCycleInterceptorIterator::get_interceptors() {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    ::Components::ContainerPortableInterceptor::IntegrationPoint ip;
    ip.operation = ::CORBA::string_dup("");
    ip.port = ::CORBA::string_dup("");
    
    ::CdmwCcmCopi::HomeLifeCycleInterceptorSeq_var interceptors = 
          m_registration->getHomeLifeCycleInterceptorByIPOrderedByPriority(ip);

    return interceptors._retn();
}

::CORBA::UShort
HomeLifeCycleInterceptorIterator::add_interceptor_ref(CdmwCcmContainer::HomeLifeCycleInterceptor_ptr interceptor) {   
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_registration->addHomeLifeCycleCOPIRef(interceptor);
}

::CORBA::UShort
HomeLifeCycleInterceptorIterator::remove_interceptor_ref(CdmwCcmContainer::HomeLifeCycleInterceptor_ptr interceptor) {   
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_registration->removeHomeLifeCycleCOPIRef(interceptor);
}


void HomeLifeCycleInterceptorIterator::before_home_create(const char* homeId, Components::ConfigValues& values, const char* entryPoint)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>::arg_t 
        arg(BEFORE_HOME_CREATE, homeId, values, entryPoint);
    
    do_starting_ip(arg);
}

void HomeLifeCycleInterceptorIterator::after_home_create(const char* homeId, CORBA::Object_ptr homeRef)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>::arg_t 
        arg(AFTER_HOME_CREATE, homeId, homeRef);

    do_ending_ip(arg);
}


void HomeLifeCycleInterceptorIterator::before_home_remove(const char* homeId)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>::arg_t 
        arg(BEFORE_HOME_REMOVE, homeId);
    
    do_starting_ip(arg);
}


void HomeLifeCycleInterceptorIterator::after_home_remove(const char* homeId)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
   
    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>::arg_t 
        arg(AFTER_HOME_REMOVE, homeId);

    do_ending_ip(arg);
}

void HomeLifeCycleInterceptorIterator::destroy() throw (::CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    //Nothing to do
}

char* HomeLifeCycleInterceptorIterator::name() throw (::CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return ::CORBA::string_dup("HomeLifecycleIterator");
}

CORBA::UShort HomeLifeCycleInterceptorIterator::priority() throw (::CORBA::SystemException)
{
    return 0;
}

Components::ContainerPortableInterceptor::IntegrationPoint* HomeLifeCycleInterceptorIterator::registration_info() throw (::CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return 0;
}

void HomeLifeCycleInterceptorIterator::set_slot_id(::PortableInterceptor::SlotId) throw (::CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    //Nothing to do
}

}
}
}
