/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <vector>
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/idllib/Components.stub.hpp"
#include "CCMContainer/idllib/CdmwCcmContainer.stub.hpp"
#include "CCMContainer/ccmcopi/COPIRegistration_impl.hpp"
#include "CCMContainer/ccmcopi/CopisStackHolder_impl.hpp"



#ifndef INTERCEPTOR_ITERATOR_STACK_SUPPOER_IMPL_HPP
#define INTERCEPTOR_ITERATOR_STACK_SUPPOER_IMPL_HPP

namespace Cdmw
{
namespace CCM
{
namespace Container
{

template<typename T>
class InterceptorIteratorStackSupportTrait
{
    //No body here to fail at compile time if used with wrong type
};


enum home_lc_ip {
    BEFORE_HOME_CREATE,
    AFTER_HOME_CREATE,
    BEFORE_HOME_REMOVE,
    AFTER_HOME_REMOVE
};

class ArgsForHomeLCI {

public:
    ArgsForHomeLCI
    (
     home_lc_ip ip, const char* homeId, Components::ConfigValues& values, const char* entryPoint
     ) : 
        m_ip(ip), 
        m_homeId (homeId), 
        m_values(&values), 
        m_entryPoint(entryPoint), 
        m_homeRef(0), 
        m_exception_occured(homeId ? false : true) 
    {};

      
    ArgsForHomeLCI
    (
     home_lc_ip ip, const char* homeId, CORBA::Object_ptr homeRef
     ) : 
        m_ip(ip), 
        m_homeId (homeId), 
        m_values(0), 
        m_entryPoint(0), 
        m_homeRef(homeRef),
        m_exception_occured(homeId ? false : true)
    {};


    ArgsForHomeLCI
    (
     home_lc_ip ip, const char* homeId
     ) : 
        m_ip(ip), 
        m_homeId (homeId), 
        m_entryPoint(0), 
        m_homeRef(0), 
        m_exception_occured(homeId ? false : true) 
    {};

    home_lc_ip m_ip;
    const char * m_homeId;
    Components::ConfigValues * m_values;
    const char * m_entryPoint;
    ::CORBA::Object_ptr m_homeRef;

    bool m_exception_occured;
};

enum component_lc_ip {
    AFTER_COMPONENT_ACTIVATE,
    AFTER_COMPONENT_CONNECT,
    AFTER_COMPONENT_CREATE,
    AFTER_COMPONENT_PASSIVATE,
    AFTER_COMPONENT_REMOVE,
    BEFORE_COMPONENT_ACTIVATE,
    BEFORE_COMPONENT_CONNECT,
    BEFORE_COMPONENT_CREATE,
    BEFORE_COMPONENT_PASSIVATE,
    BEFORE_COMPONENT_REMOVE
};


class ArgsForComponentLCI {

public:
    ArgsForComponentLCI
    (
     component_lc_ip ip, const char* compId
     ) : 
        m_ip(ip),
        m_compId(compId),
        m_values(0), 
        m_featureName(0),
        m_compRef(0),
        m_exception_occured(compId ? false : true) 
    {};


    ArgsForComponentLCI
    (
     component_lc_ip ip, const char* compId, CORBA::Object_ptr compRef
     ) : 
        m_ip(ip),
        m_compId(compId),
        m_values(0), 
        m_featureName(0),
        m_compRef(compRef),
        m_exception_occured(compId ? false : true) 
    {};

    ArgsForComponentLCI
    (
     component_lc_ip ip, const char* compId, const char* featureName, CORBA::Object_ptr compRef
     ) : 
        m_ip(ip),
        m_compId(compId),
        m_values(0), 
        m_featureName(featureName),
        m_compRef(compRef),
        m_exception_occured(compId ? false : true) 
    {};

    ArgsForComponentLCI
    (
     component_lc_ip ip,
     const char* compId,
     Components::ConfigValues & compConfigVals
     ) : 
        m_ip(ip),
        m_compId(compId),
        m_values(&compConfigVals),
        m_featureName(0),
        m_compRef(0),
        m_exception_occured(compId ? false : true) 
    {};
      
    component_lc_ip m_ip;
    const char * m_compId;
    Components::ConfigValues * m_values;
    const char * m_featureName;
    CORBA::Object_ptr m_compRef;       
    bool m_exception_occured;
};



template<>
class InterceptorIteratorStackSupportTrait<
    CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>
{
public:
    typedef ArgsForHomeLCI arg_t;
    typedef ::CdmwCcmCopi::HomeLifeCycleInterceptorSeq interceptor_seq_t;
    typedef ::CdmwCcmCopi::HomeLifeCycleInterceptorSeq_var interceptor_seq_var_t;
};


template<>
class InterceptorIteratorStackSupportTrait<
    CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr>
{
public:
    typedef ArgsForComponentLCI arg_t;
    typedef ::CdmwCcmCopi::ComponentLifeCycleInterceptorSeq interceptor_seq_t;
    typedef ::CdmwCcmCopi::ComponentLifeCycleInterceptorSeq_var interceptor_seq_var_t;
};



template<typename T>
class InterceptorIteratorStackSupport_impl
{

public:
    ///Ctor
    InterceptorIteratorStackSupport_impl() : m_stack(0)
    {
        m_registration =
              ::Cdmw::CCM::COPI::COPIRegistration_impl::Get_Instance();
    };


protected:
    typename ::Cdmw::CCM::COPI::CopisStackHolderTrait<T>::copis_list_t *
    get_safe_interceptors() {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

        typename ::Cdmw::CCM::COPI::CopisStackHolderTrait<T>::copis_list_t * ret_list =
              new typename ::Cdmw::CCM::COPI::CopisStackHolderTrait<T>::copis_list_t();
                   
        typename InterceptorIteratorStackSupportTrait<T>::interceptor_seq_var_t interceptors =
            get_interceptors();

        for(unsigned int i = 0; i < interceptors->length(); ++i) {
            if(0 == add_interceptor_ref(interceptors[i])) {
                ret_list->push_back(interceptors[i]); 
            }
        }

        return ret_list;
    };

        

    void do_starting_ip
    ( 
     typename InterceptorIteratorStackSupportTrait<T>::arg_t arg
     )
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

        typename ::Cdmw::CCM::COPI::CopisStackHolderTrait<T>::copis_list_t *
              safe_interceptors = get_safe_interceptors();

        if(0 == safe_interceptors->size()) {
            delete safe_interceptors;
            return;
        }
        
        m_stack =
            new ::Cdmw::CCM::COPI::CopisStackHolder_impl<T>();
        
        m_stack->set_copis(safe_interceptors);

        try {
            for (unsigned int i = 0; i < safe_interceptors->size(); ++i) {
                // invoke interception point
                invoke_ip((*safe_interceptors)[i], arg);
                // update stack
                m_stack->set_send_idx (i);
                m_stack->set_recv_idx(m_stack->get_send_idx() + 1);
            }
        } catch (...) {        
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(), "A COPI raised an exception during before_home_create");
      
            throw;
        }
    }


    void do_ending_ip
    ( 
     typename InterceptorIteratorStackSupportTrait<T>::arg_t arg
     )
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

        if(0 == m_stack) {
            return;
        }

        typename ::Cdmw::CCM::COPI::CopisStackHolderTrait<T>::copis_list_t * interceptors =
              m_stack->get_copis();

        unsigned int i = 0;
    
        try {
            int begin;

            if(arg.m_exception_occured) {
                // homeId set -> no exception occured
                begin = m_stack->get_send_idx();
            } else {
                begin = m_stack->get_recv_idx() - 1;
            }
                 

            for (i = begin;
                 i < interceptors->size() && i >= 0;
                 i--) 
                {
                    if(arg.m_exception_occured && !((int)i <= m_stack->get_send_idx())) {
                        // don't see how this can ever happen, check java part 
                        continue;
                    }           
         
                    m_stack->set_recv_idx(i);

                    invoke_ip((*interceptors)[i], arg);
                   
                    remove_interceptor_ref((*interceptors)[i]);
                }

            delete m_stack;
            m_stack = 0;
        } catch (...) {
            if(arg.m_exception_occured && (i < interceptors->size() && i >= 0)) {

                remove_interceptor_ref((*interceptors)[i]);

            } else {
                if( i < interceptors->size() && i >= 0 && (int)i <= m_stack->get_send_idx() ) {

                    remove_interceptor_ref((*interceptors)[i]);

                }
            }

            // note: we don't delete stack here, because we expect the caller of a
            // end ip call the same ip one more time signalling an exception (passing 0 values)

            // note2: this can be done easily for extendened + lc copis, basic copis
            // are different breed since the orb won't be calling an ending ips on
            // the portable interceptor which's starting ip raised an exception
        }
    }


    // purely virtual 
    virtual
    void invoke_ip(T interceptor, typename InterceptorIteratorStackSupportTrait<T>::arg_t arg) = 0;

    virtual
    typename InterceptorIteratorStackSupportTrait<T>::interceptor_seq_t *
        get_interceptors() = 0;

    virtual
    ::CORBA::UShort add_interceptor_ref(T interceptor) = 0;

    virtual
    ::CORBA::UShort remove_interceptor_ref(T interceptor) = 0;   

    // data

    ::Cdmw::CCM::COPI::CopisStackHolder_impl<T> * m_stack;

    ::Cdmw::CCM::COPI::COPIRegistration_impl * m_registration;
};


}
}
}

#endif /* INTERCEPTOR_ITERATOR_STACK_SUPPOER_IMPL_HPP */
