/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_CCM_CONTAINER_SESSION_2_EXTENTIONCONTEXT_IMPL_HPP
#define INCL_CDMW_CCM_CONTAINER_SESSION_2_EXTENTIONCONTEXT_IMPL_HPP

#include <string>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/idllib/Components.stub.hpp>
#include <Foundation/idllib/CdmwDeployment.stub.hpp>
#include "CCMContainer/idllib/CdmwCcmContainer.skel.hpp"
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcommon/Cookie_impl.hpp>
#include <CCMContainer/ccmcontainer/ComponentServantProvider.hpp>
#include "CCMContainer/ccmcopi/COPICurrent_impl.hpp"
#include <CCMContainer/ccmcif/CCMContext_impl.hpp>
#include "Event/idllib/CdmwEvent.stub.hpp"
#include "Session2Context_impl.hpp"

namespace Cdmw
{
namespace CCM
{
namespace Container
{

class Session2ExtensionContext_impl: public virtual CdmwCcmContainer::Session2ExtensionContext,
                                     public virtual Session2Context_impl,
                                     public virtual Cdmw::OrbSupport::RefCountLocalObject
{
public:
    /// ctor
    Session2ExtensionContext_impl(CORBA::ORB_ptr orb, PortableServer::POA_ptr parent, Components::CCMHome_ptr home,
            const std::string & home_id, ComponentServantProvider* servant_provider,
            CdmwDeployment::ServantLifetimeValue servant_lifetime,
            CdmwDeployment::ThreadingPolicyValue threading_policy, Components::HomeRegistration_ptr home_registration,
            CdmwEvent::EventChannelFactory_ptr event_channel_factory, const std::string & process_name,
            const std::string & application_name, const ValueTypeDependencyList & valuetype_dependencies);

    /// dtor
    virtual ~Session2ExtensionContext_impl(void) throw();

    virtual ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_ptr
            get_client_interceptor_registration() throw (::CORBA::SystemException, ::Components::CCMException );

    virtual ::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_ptr
            get_server_interceptor_registration() throw (::CORBA::SystemException, ::Components::CCMException );

    virtual ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_ptr
            get_stub_interceptor_registration() throw (::CORBA::SystemException, ::Components::CCMException );

    virtual ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_ptr
            get_servant_interceptor_registration() throw (::CORBA::SystemException, ::Components::CCMException );

    //ok
    virtual ::Components::Cookie * install_service_reference(const char * service_id, ::CORBA::Object_ptr objref)
            throw (::CORBA::SystemException, ::Components::CCMException );

    //ok
    virtual ::CORBA::Object_ptr uninstall_service_reference(::Components::Cookie * ck) throw (::CORBA::SystemException,
            ::Components::CCMException );
  
private:
    class Cookie_impl: public virtual Cdmw::CCM::Common::Cookie_impl
    {

    public:
        // ctor
        Cookie_impl(const std::string & cookie_value)
        {
            CORBA::OctetSeq_var _cookie_value = Cdmw::CCM::Common::string_to_OctetSeq(cookie_value.c_str());
            cookieValue(_cookie_value.in());
        }

        std::string get_cookie_string()
        {
            char * c_str_value = Cdmw::CCM::Common::OctetSeq_to_string(cookieValue());
            return std::string(c_str_value);
        }
    };

    Cdmw::Logging::Logger*              m_logger;

    CdmwCcmCopi::COPICurrent_ptr        m_copi_current;

    bool m_is_copi_current_initialised;

    Components::ExtensionContext_ptr    m_context;

};

} // End namespace Container
} // End namespace CCM
} // End namespace Cdmw

#endif // INCL_CDMW_CCM_CONTAINER_SESSION_2_EXTENTIONCONTEXT_IMPL_HPP
