/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcopi/COPICurrent_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>


namespace Cdmw {

namespace CCM {

namespace COPI {

// Initialise static data

const std::string COPICurrent_impl::COPI_CURRENT_ID = "COPICurrent";


// Implementation skeleton constructor
COPICurrent_impl::COPICurrent_impl
(
 int origin_ID_SlotID, 
 int target_ID_SlotID,
 int originFeatureName_SlotID, 
 int targetFeatureName_SlotID,
 int originCustomSlotItemSeq_SlotID,
 int targetCustomSlotItemSeq_SlotID, 
 CdmwCcmCopi::COPIRegistrar_ptr reg
 ) : m_container_interceptor_registration(reg),
     m_origin_ID_SlotID (origin_ID_SlotID),
     m_target_ID_SlotID (target_ID_SlotID),
     m_originFeatureName_SlotID (originFeatureName_SlotID),
     m_targetFeatureName_SlotID (targetFeatureName_SlotID),
     m_originCustomSlotItemSeq_SlotID (originCustomSlotItemSeq_SlotID),
     m_targetCustomSlotItemSeq_SlotID (targetCustomSlotItemSeq_SlotID)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    if(CORBA::is_nil(m_container_interceptor_registration) == true) {
        CORBA::BAD_PARAM ex(Cdmw::OrbSupport::BAD_PARAMNilObjectReference, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;            
    }
};


// Implementation skeleton destructor
COPICurrent_impl::~COPICurrent_impl (void)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
}

::PortableInterceptor::SlotId 
COPICurrent_impl::get_slot_id (const char * slot_name)
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::InvalidSlot
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    
    if (strcmp(COPIServiceInit::M_origin_ID_SlotNAME.c_str(), slot_name) == 0)
        return m_origin_ID_SlotID;

    if (strcmp(COPIServiceInit::M_target_ID_SlotNAME.c_str(), slot_name) == 0)
        return m_target_ID_SlotID;

    if (strcmp(COPIServiceInit::M_originFeatureName_SlotNAME.c_str(), slot_name) == 0)
        return m_originFeatureName_SlotID;

    if (strcmp(COPIServiceInit::M_targetFeatureName_SlotNAME.c_str(), slot_name) == 0)
        return m_targetFeatureName_SlotID;

    if (strcmp(COPIServiceInit::M_originCustomSlotItemSeq_SlotNAME.c_str(), slot_name) == 0)
        return m_originCustomSlotItemSeq_SlotID;

    if (strcmp(COPIServiceInit::M_targetCustomSlotItemSeq_SlotNAME.c_str(), slot_name) == 0)
        return m_targetCustomSlotItemSeq_SlotID;

    PortableInterceptor::InvalidSlot ex;
    CDMW_LOG_FUNCTION_EXCEPTION(ex);
    throw ex;    
}

::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_ptr 
COPICurrent_impl::get_client_interceptor_registration ()
  throw (
    ::CORBA::SystemException,
    ::Components::CCMException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_var              
          clientCOPIRegistration =
          ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration::_nil();

        try {
            clientCOPIRegistration = 
                ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration::_narrow(m_container_interceptor_registration);
        } catch (CORBA::BAD_PARAM & bp) {
            ::Components::CCMException ex;
            ex.reason = ::Components::REGISTRATION_ERROR;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;               
        }

        if(CORBA::is_nil(clientCOPIRegistration) == true) {
            ::Components::CCMException ex;
            ex.reason = ::Components::REGISTRATION_ERROR;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;               
        }

        return clientCOPIRegistration._retn();
}

::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_ptr 
COPICurrent_impl::get_server_interceptor_registration ()
  throw (
    ::CORBA::SystemException,
    ::Components::CCMException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_var
          serverCOPIRegistration =
          ::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration::_nil();

        try {
            serverCOPIRegistration = 
                ::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration::_narrow(m_container_interceptor_registration);
        } catch (CORBA::BAD_PARAM & bp) {
            ::Components::CCMException ex;
            ex.reason = ::Components::REGISTRATION_ERROR;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;               
        }

        if(CORBA::is_nil(serverCOPIRegistration) == true) {
            ::Components::CCMException ex;
            ex.reason = ::Components::REGISTRATION_ERROR;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;               
        }

        return serverCOPIRegistration._retn();
}

::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_ptr 
COPICurrent_impl::get_stub_interceptor_registration ()
  throw (
    ::CORBA::SystemException,
    ::Components::CCMException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_var
          stubCOPIRegistration =
          ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration::_nil();

        try {
            stubCOPIRegistration = 
                ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration::_narrow(m_container_interceptor_registration);
        } catch (CORBA::BAD_PARAM & bp) {
            ::Components::CCMException ex;
            ex.reason = ::Components::REGISTRATION_ERROR;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;               
        }

        if(CORBA::is_nil(stubCOPIRegistration) == true) {
            ::Components::CCMException ex;
            ex.reason = ::Components::REGISTRATION_ERROR;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;               
        }

        return stubCOPIRegistration._retn();    
}

::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_ptr 
COPICurrent_impl::get_servant_interceptor_registration ()
  throw (
    ::CORBA::SystemException,
    ::Components::CCMException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_var
          servantCOPIRegistration =
          ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration::_nil();

        try {
            servantCOPIRegistration = 
                ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration::_narrow(m_container_interceptor_registration);
        } catch (CORBA::BAD_PARAM & bp) {
            ::Components::CCMException ex;
            ex.reason = ::Components::REGISTRATION_ERROR;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;               
        }

        if(CORBA::is_nil(servantCOPIRegistration) == true) {
            ::Components::CCMException ex;
            ex.reason = ::Components::REGISTRATION_ERROR;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;               
        }

        return servantCOPIRegistration._retn();
}

::CdmwCcmCopi::RegisteredCOPICollection_ptr 
COPICurrent_impl::get_registered_copi_collection ()
  throw (
    ::CORBA::SystemException
  )
{
    return
        ::CdmwCcmCopi::RegisteredCOPICollection::_narrow(m_container_interceptor_registration);
}

} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

