/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tao/CodecFactory/CodecFactory.h"
#include "tao/Codeset/Codeset.h"

// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

// Local Files
#include <CCMContainer/ccmcopi/COPIMainORBInitializer.hpp>
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>
#include <CCMContainer/ccmcopi/COPICurrent_impl.hpp>
#include <CCMContainer/ccmcopi/ClientRequestInterceptorIterator_impl.hpp>
#include <CCMContainer/ccmcopi/ServerRequestInterceptorIterator_impl.hpp>


namespace {


} // End anonymous space


namespace Cdmw
{
namespace CCM
{
namespace COPI
{


// constructor
COPIMainORBInitializer::COPIMainORBInitializer()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
}

// destructor
COPIMainORBInitializer::~COPIMainORBInitializer()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");  
}

void
COPIMainORBInitializer::pre_init(PortableInterceptor::ORBInitInfo_ptr info) 
  throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    // empty
}

    
//
// Note: the interceptors are registered in post_init()
//       if their constructors require
//       ORBInitInfo::resolve_initial_reference(), which 
//       cannot be called in pre_init().
//
void
COPIMainORBInitializer::post_init(PortableInterceptor::ORBInitInfo_ptr info) 
   throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "post_init => allocate needed slot_ids");

    int origin_id = info->allocate_slot_id();
    int target_id = info->allocate_slot_id();
    int origin_feature_name_id = info->allocate_slot_id();
    int target_feature_name_id = info->allocate_slot_id();
    int origin_custom_slot_item_seq_id = info->allocate_slot_id();
    int target_custom_slot_item_seq_id = info->allocate_slot_id();

    try {            
        //creating the registration interface

        COPIRegistration_impl::Initialise
            (
             origin_custom_slot_item_seq_id,
             target_custom_slot_item_seq_id
             );

        
        ::CdmwCcmCopi::COPIRegistrar_ptr registration_interface =
              COPIRegistration_impl::Get_Instance();
       
        // create and register the COPICurrent object
        ::CdmwCcmCopi::COPICurrent_var copi_current =
              new COPICurrent_impl
              (
               origin_id,
               target_id, 
               origin_feature_name_id,
               target_feature_name_id,
               origin_custom_slot_item_seq_id,
               target_custom_slot_item_seq_id,
               registration_interface
               );
            
        info->register_initial_reference(COPICurrent_impl::COPI_CURRENT_ID.c_str(), copi_current.in());

        // registering the main client and server interceptors
        PortableInterceptor::ClientRequestInterceptor_var cri =
            new ClientRequestInterceptorIterator_impl 
            (
             origin_id,
             target_id,
             origin_feature_name_id,
             origin_custom_slot_item_seq_id, 
             ::CdmwCcmCopi::RegisteredCOPICollection::_narrow(registration_interface)
             );

        
        PortableInterceptor::ServerRequestInterceptor_var sri =
            new ServerRequestInterceptorIterator_impl 
            (
             origin_id,
             target_id,
             target_feature_name_id,
             target_custom_slot_item_seq_id,
             ::CdmwCcmCopi::RegisteredCOPICollection::_narrow(registration_interface)
             );

      
        info->add_client_request_interceptor(cri.in());
        info->add_server_request_interceptor(sri.in());
    }
    catch(const ::PortableInterceptor::ORBInitInfo::InvalidName & in) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Failed to get PICurrent reference: " << in);
        throw;
    }
    catch(const ::PortableInterceptor::ORBInitInfo::DuplicateName & dn) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Caught PortableInterceptor::ORBInitInfo::DuplicateName during COPI initilisation: " << dn);
        throw;
    }
    catch(const ::IOP::CodecFactory::UnknownEncoding & ue) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Unsupported Encoding format: " << ue);
        throw;
    }
    catch(const IOP::CodecFactory::UnsupportedCodeset & uc) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "UnsupportedCodeset: " << uc);
        throw;
    }
}

} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw




