/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>

namespace Cdmw
{

namespace CCM
{

namespace COPI
{

/*
 TODOs:
 -not clear how to shutdown. There should maybe be a dedicated shutdown operation
 which should maybe block if copies are still used
 */

// statics

const char* COPIRegistration_impl::M_LifeCycleInterceptorRegistrationServiceId = "CDMW_LifeCycleInterceptorRegistrationService";

int COPIRegistration_impl::M_client_slot_id = 0;
int COPIRegistration_impl::M_server_slot_id = 0;
bool COPIRegistration_impl::M_initialised = false;
COPIRegistration_impl * COPIRegistration_impl::The_Instance = 0;

COPIRegistration_impl * COPIRegistration_impl::Get_Instance() {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    if(false == M_initialised) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "COPI Registration not initialised" );
        ::CORBA::INTERNAL e(Cdmw::OrbSupport::INTERNAL, ::CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    };

    if(0 == The_Instance) {
        The_Instance = new COPIRegistration_impl();
    }

    return The_Instance;
}

void COPIRegistration_impl::Initialise(int client_slot_id, int server_slot_id) {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    if(true == M_initialised) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "COPI Registration already initialised" );
        ::CORBA::INTERNAL e(Cdmw::OrbSupport::INTERNAL, ::CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    };

    COPIRegistration_impl::M_client_slot_id = client_slot_id;
    COPIRegistration_impl::M_server_slot_id = server_slot_id;
    COPIRegistration_impl::M_initialised = true;
}


// Implementation skeleton constructor
COPIRegistration_impl::COPIRegistration_impl()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");    
}

// Implementation skeleton destructor
COPIRegistration_impl::~COPIRegistration_impl(void)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    try
    {
        destroy_copi_storage<Components::ContainerPortableInterceptor::ClientContainerInterceptor>(m_client_copi_storage);
        destroy_copi_storage<Components::ContainerPortableInterceptor::ServerContainerInterceptor>(m_server_copi_storage);
        destroy_copi_storage<Components::ContainerPortableInterceptor::StubContainerInterceptor>(m_stub_copi_storage);
        destroy_copi_storage<Components::ContainerPortableInterceptor::ServantContainerInterceptor>(m_servant_copi_storage);
    }
    catch (...)
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "Caught exception in dtor")
        ;
    }
}

// Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration
::Components::Cookie *
COPIRegistration_impl::register_client_interceptor(
        ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr ci)
        throw (::CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_client_copi_storage.m_lock);

    return register_container_interceptor<Components::ContainerPortableInterceptor::ClientContainerInterceptor>
        (
         ::Components::ContainerPortableInterceptor::ClientContainerInterceptor::_duplicate(ci),
         M_client_slot_id,
         m_client_copi_storage
         );

}

::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr COPIRegistration_impl::unregister_client_interceptor(
        ::Components::Cookie * ck) throw (::CORBA::SystemException,
        ::Components::ContainerPortableInterceptor::InvalidRegistration)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_client_copi_storage.m_lock);

    return unregister_container_interceptor<
            Components::ContainerPortableInterceptor::ClientContainerInterceptor> 
        (
         ck,
         m_client_copi_storage
         );
}

// Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration
::Components::Cookie *
COPIRegistration_impl::register_server_interceptor(
        ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr ci)
        throw (::CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_server_copi_storage.m_lock);

    return register_container_interceptor<Components::ContainerPortableInterceptor::ServerContainerInterceptor>
        (
         ::Components::ContainerPortableInterceptor::ServerContainerInterceptor::_duplicate(ci),
         M_server_slot_id,
         m_server_copi_storage
         );
}

::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr COPIRegistration_impl::unregister_server_interceptor(
        ::Components::Cookie * ck) throw (::CORBA::SystemException,
        ::Components::ContainerPortableInterceptor::InvalidRegistration)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_server_copi_storage.m_lock);

    return unregister_container_interceptor<
            Components::ContainerPortableInterceptor::ServerContainerInterceptor> 
        (ck,
         m_server_copi_storage
         );
}

// Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration
::Components::Cookie *
COPIRegistration_impl::register_stub_interceptor(
        ::Components::ContainerPortableInterceptor::StubContainerInterceptor_ptr ci)
        throw (::CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_stub_copi_storage.m_lock);

    return register_container_interceptor<
        Components::ContainerPortableInterceptor::StubContainerInterceptor> 
        (
         ::Components::ContainerPortableInterceptor::StubContainerInterceptor::_duplicate(ci),
         M_client_slot_id,
         m_stub_copi_storage
         );
}

::Components::ContainerPortableInterceptor::StubContainerInterceptor_ptr COPIRegistration_impl::unregister_stub_interceptor(
        ::Components::Cookie * ck) throw (::CORBA::SystemException,
        ::Components::ContainerPortableInterceptor::InvalidRegistration)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_stub_copi_storage.m_lock);

    return unregister_container_interceptor<
            Components::ContainerPortableInterceptor::StubContainerInterceptor> 
        (
         ck,
         m_stub_copi_storage
         );
}

// Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration
::Components::Cookie *
COPIRegistration_impl::register_servant_interceptor(
        ::Components::ContainerPortableInterceptor::ServantContainerInterceptor_ptr ci)
        throw (::CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_servant_copi_storage.m_lock);

    return register_container_interceptor<
            Components::ContainerPortableInterceptor::ServantContainerInterceptor> 
        (
         ::Components::ContainerPortableInterceptor::ServantContainerInterceptor::_duplicate(ci),
         M_server_slot_id, 
         m_servant_copi_storage
         );
}

::Components::ContainerPortableInterceptor::ServantContainerInterceptor_ptr
COPIRegistration_impl::unregister_servant_interceptor(
        ::Components::Cookie * ck) throw (::CORBA::SystemException,
        ::Components::ContainerPortableInterceptor::InvalidRegistration)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_servant_copi_storage.m_lock);

    return unregister_container_interceptor<Components::ContainerPortableInterceptor::ServantContainerInterceptor>
        (
         ck, 
         m_servant_copi_storage
         );
}

::Components::Cookie *
COPIRegistration_impl::register_home_lifecycle_interceptor 
(
 ::CdmwCcmContainer::HomeLifeCycleInterceptor_ptr ci
 ) 
    throw (::CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_home_lc_copi_storage.m_lock);

    return register_container_interceptor<
        ::CdmwCcmContainer::HomeLifeCycleInterceptor> 
        (
         ::CdmwCcmContainer::HomeLifeCycleInterceptor::_duplicate(ci),
         M_server_slot_id, 
         m_home_lc_copi_storage
         );
}
    
::CdmwCcmContainer::HomeLifeCycleInterceptor_ptr
COPIRegistration_impl::unregister_home_lifecycle_interceptor (::Components::Cookie * ck)
            throw (::CORBA::SystemException,
                   ::Components::ContainerPortableInterceptor::InvalidRegistration)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_home_lc_copi_storage.m_lock);

    return unregister_container_interceptor< CdmwCcmContainer::HomeLifeCycleInterceptor >
        (
         ck, 
         m_home_lc_copi_storage
         );
}
    
::Components::Cookie *
COPIRegistration_impl::register_component_lifecycle_interceptor 
(
 ::CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr ci
 )
    throw (::CORBA::SystemException) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_component_lc_copi_storage.m_lock);

    return register_container_interceptor<
        ::CdmwCcmContainer::ComponentLifeCycleInterceptor> 
        (
         ::CdmwCcmContainer::ComponentLifeCycleInterceptor::_duplicate(ci),
         M_server_slot_id, 
         m_component_lc_copi_storage
         );
}
  
::CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr
COPIRegistration_impl::unregister_component_lifecycle_interceptor (::Components::Cookie * ck)
    throw (::CORBA::SystemException,
           ::Components::ContainerPortableInterceptor::InvalidRegistration)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_component_lc_copi_storage.m_lock);

    return unregister_container_interceptor< CdmwCcmContainer::ComponentLifeCycleInterceptor >
        (
         ck, 
         m_component_lc_copi_storage
         );
}

::CdmwCcmCopi::ClientContainerInterceptorSeq *
COPIRegistration_impl::getClientInterceptorByIPOrderedByPriority
(
        const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_MUTEX_GUARD(m_client_copi_storage.m_lock);

    return getContainerPortableInterceptorsByIPOrderedByPriority<Components::ContainerPortableInterceptor::ClientContainerInterceptor>
        (
         ip, 
         m_client_copi_storage
         );
}

::CdmwCcmCopi::StubContainerInterceptorSeq *
COPIRegistration_impl::getStubInterceptorByIPOrderedByPriority
(
        const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_MUTEX_GUARD(m_stub_copi_storage.m_lock);

    return getContainerPortableInterceptorsByIPOrderedByPriority<Components::ContainerPortableInterceptor::StubContainerInterceptor>
        (
         ip,
         m_stub_copi_storage
         );
}

::CdmwCcmCopi::ServerContainerInterceptorSeq *
COPIRegistration_impl::getServerInterceptorByIPOrderedByPriority
(
        const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_MUTEX_GUARD(m_server_copi_storage.m_lock);

    return getContainerPortableInterceptorsByIPOrderedByPriority<Components::ContainerPortableInterceptor::ServerContainerInterceptor>
        (
         ip, 
         m_server_copi_storage
         );

}

::CdmwCcmCopi::ServantContainerInterceptorSeq *
COPIRegistration_impl::getServantInterceptorByIPOrderedByPriority
(
        const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_MUTEX_GUARD(m_servant_copi_storage.m_lock);

    return getContainerPortableInterceptorsByIPOrderedByPriority<Components::ContainerPortableInterceptor::ServantContainerInterceptor>
        (
         ip, 
         m_servant_copi_storage
         );
}

::CdmwCcmCopi::HomeLifeCycleInterceptorSeq *
COPIRegistration_impl::getHomeLifeCycleInterceptorByIPOrderedByPriority
(
 const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip
 )
    throw 
(
 ::CORBA::SystemException
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_MUTEX_GUARD(m_home_lc_copi_storage.m_lock);
    
    // ignore ip for the moment

    return getAllContainerInterceptors<CdmwCcmContainer::HomeLifeCycleInterceptor>
        (         
         m_home_lc_copi_storage
         );
}
  
::CdmwCcmCopi::ComponentLifeCycleInterceptorSeq *
COPIRegistration_impl::getComponentLifeCycleInterceptorByIPOrderedByPriority 
(
 const ::Components::ContainerPortableInterceptor::IntegrationPoint & ip
 )
    throw 
(
 ::CORBA::SystemException
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_MUTEX_GUARD(m_component_lc_copi_storage.m_lock);
    
    // ignore ip for the moment

    return getAllContainerInterceptors<CdmwCcmContainer::ComponentLifeCycleInterceptor>
        (         
         m_component_lc_copi_storage
         );
}

CORBA::Short
COPIRegistration_impl::addClientCOPIRef
(
        ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr ci
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return add_COPI_ref(ci, m_client_copi_storage);
}

CORBA::Short
COPIRegistration_impl::addServerCOPIRef
(
        ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr ci
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return add_COPI_ref(ci, m_server_copi_storage);
}

CORBA::Short
COPIRegistration_impl::addStubCOPIRef
(
        ::Components::ContainerPortableInterceptor::StubContainerInterceptor_ptr ci
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return add_COPI_ref(ci, m_stub_copi_storage);
}

CORBA::Short
COPIRegistration_impl::addServantCOPIRef
(
        ::Components::ContainerPortableInterceptor::ServantContainerInterceptor_ptr ci
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return add_COPI_ref(ci, m_servant_copi_storage);
}

::CORBA::Short 
COPIRegistration_impl::addHomeLifeCycleCOPIRef 
(
 ::CdmwCcmContainer::HomeLifeCycleInterceptor_ptr ci
 )
    throw (
           ::CORBA::SystemException
           ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return add_COPI_ref(ci, m_home_lc_copi_storage);
}

::CORBA::Short
COPIRegistration_impl::addComponentLifeCycleCOPIRef 
(
 ::CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr ci
 )
    throw (
           ::CORBA::SystemException
           )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return add_COPI_ref(ci, m_component_lc_copi_storage);
}

CORBA::Short
COPIRegistration_impl::removeClientCOPIRef
(
        ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr ci
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return remove_COPI_ref(ci, m_client_copi_storage);
}

CORBA::Short
COPIRegistration_impl::removeServerCOPIRef
(
        ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr ci
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return remove_COPI_ref(ci, m_server_copi_storage);
}

CORBA::Short
COPIRegistration_impl::removeStubCOPIRef
(
        ::Components::ContainerPortableInterceptor::StubContainerInterceptor_ptr ci
)
throw (
        ::CORBA::SystemException
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return remove_COPI_ref(ci, m_stub_copi_storage);
}

CORBA::Short
COPIRegistration_impl::removeServantCOPIRef
(
        ::Components::ContainerPortableInterceptor::ServantContainerInterceptor_ptr ci
)
throw (
        ::CORBA::SystemException
        )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return remove_COPI_ref(ci, m_servant_copi_storage);
}


::CORBA::Short
COPIRegistration_impl::removeHomeLifeCycleCOPIRef
(
 ::CdmwCcmContainer::HomeLifeCycleInterceptor_ptr ci)
    throw (
           ::CORBA::SystemException
           ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return remove_COPI_ref(ci, m_home_lc_copi_storage);
}
  
::CORBA::Short
COPIRegistration_impl::removeComponentLifeCycleCOPIRef
(
 ::CdmwCcmContainer::ComponentLifeCycleInterceptor_ptr ci
 )
    throw 
(
 ::CORBA::SystemException
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return remove_COPI_ref(ci, m_component_lc_copi_storage);
}


std::string
COPIRegistration_impl::get_copi_key
(
        ::Components::ContainerPortableInterceptor::ContainerInterceptor_ptr copi
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    std::string key = "/";
    if(copi->registration_info() != 0)
    {
        std::stringstream ostr;
        ostr << create_the_copismap_key(copi->registration_info()->port.in(), copi->registration_info()->operation.in());
        key = ostr.str();
    }

    return key;
}

std::string
COPIRegistration_impl::create_the_copismap_key
(
        const std::string & port, const std::string & operation
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return (port + "/" + operation);
}

COPICookie_impl *
COPIRegistration_impl::get_checked_cookie
(
        ::Components::Cookie * ck
)
throw(CORBA::BAD_PARAM)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    if(ck == 0)
    {
        CORBA::BAD_PARAM ex = CORBA::BAD_PARAM (Cdmw::OrbSupport::BAD_PARAMBADCookie, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    COPICookie_impl * ck_impl = 0;

    try
    {
        ck_impl = dynamic_cast<COPICookie_impl *>(ck);
        if(ck_impl == 0)
        {
            CORBA::BAD_PARAM ex = CORBA::BAD_PARAM (Cdmw::OrbSupport::BAD_PARAMBADCookie, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }
    catch (std::bad_cast & bd)
    {
        CORBA::BAD_PARAM ex = CORBA::BAD_PARAM (Cdmw::OrbSupport::BAD_PARAMBADCookie, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    return ck_impl;
}

} // End namespace CIF

} // End namespace CCM

} // End namespce COPI
