/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "tao/CodecFactory/CodecFactory.h"
#include "tao/Codeset/Codeset.h"

#if CDMW_ORB_VDR == tao && CDMW_ORB_VER > 13
#include <tao/ORBInitializer_Registry.h>
#endif

#include <orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {

    Cdmw::CCM::COPI::COPIMainORBInitializer * COPIServiceInit::M_initialiser = 0;

    void COPIServiceInit::Init(int& argc, char** argv) 
        throw (CORBA::SystemException)
    {
        CDMW_MUTEX_GUARD(M_mtx);
        // Init the Logging library
        Cdmw::Logging::LogManager::Init(argc, argv);

        M_initialiser = new Cdmw::CCM::COPI::COPIMainORBInitializer();
        
        PortableInterceptor::register_orb_initializer(M_initialiser);

        M_initialized = true;
    };

    
    void COPIServiceInit::Post_init(CORBA::ORB_ptr orb) 
        throw (CORBA::SystemException)
    {
        CDMW_MUTEX_GUARD(M_mtx);

        CORBA::Object_var obj =     
            orb->resolve_initial_references ("CodecFactory");
 
        IOP::CodecFactory_var codec_factory =
            IOP::CodecFactory::_narrow (obj.in ());
   
        IOP::Encoding encoding;
        encoding.format = IOP::ENCODING_CDR_ENCAPS;
        encoding.major_version = 1;
        encoding.minor_version = 2;

        // Obtain the CDR encapsulation Codec.
        IOP::Codec_var codec =
            codec_factory->create_codec (encoding);
        

        Cdmw::CCM::COPI::COPIUtils_impl::Set_codec(codec.in());

        
        M_initialized = true;
    };
    

    void COPIServiceInit::Destroy() 
        throw (CORBA::SystemException)
    {
        CDMW_MUTEX_GUARD(M_mtx);
        M_initialized = false;       
    }
    

    bool COPIServiceInit::Is_initialized() {
        CDMW_MUTEX_GUARD(M_mtx);
        return M_initialized;
    }
    
    void COPIServiceInit::Enable_read_write_requests() {
        CDMW_MUTEX_GUARD(M_mtx);
        ::Cdmw::OsSupport::SystemProperties::Set_Property
              (
               READ_WRITE_REQUEST_PROPERTY_NAME,
               "yes"
               );
    }
   
    void COPIServiceInit::Disable_read_write_requests() {
        CDMW_MUTEX_GUARD(M_mtx);
        ::Cdmw::OsSupport::SystemProperties::Set_Property
              (
               READ_WRITE_REQUEST_PROPERTY_NAME,
               "no"
               );
    }

    bool COPIServiceInit::Is_COPI_requests_read_write() {
        CDMW_MUTEX_GUARD(M_mtx);

        std::string prop_value = 
            ::Cdmw::OsSupport::SystemProperties::Get_Property
            (
             READ_WRITE_REQUEST_PROPERTY_NAME, 
             "no"
             );


        return "yes" == prop_value ? true : false;
    }
      

    bool COPIServiceInit::M_initialized = false;
    const std::string COPIServiceInit::M_origin_ID_SlotNAME = "Origin_ID_SlotID";
    const std::string COPIServiceInit::M_target_ID_SlotNAME = "Target_ID_SlotID";
    const std::string COPIServiceInit::M_originFeatureName_SlotNAME = "OriginFeatureName_SlotID";
    const std::string COPIServiceInit::M_targetFeatureName_SlotNAME = "TargetFeatureName_SlotID";
    const std::string COPIServiceInit::M_originCustomSlotItemSeq_SlotNAME = "OriginCustomSlotItemSeq_SlotID";
    const std::string COPIServiceInit::M_targetCustomSlotItemSeq_SlotNAME = "TargetCustomSlotItemSeq_SlotID";
 
    /** 
     * The COPI Service Context ID
     * TODO: to remove this ID and use the standard one  
     */
    const unsigned int COPIServiceInit::CDMW_COPI_SERVICE_CONTEXT_ID = 10000;
    
    /**
     * The Basic client copi service context Id
     * It is used to vehicle the request Id
     */
    const int COPIServiceInit::CDMW_CLIENT_COPI_SERVICE_CONTEXT_ID = 2 | Cdmw::OrbSupport::CDMW_VMCID;
    
    /**
     * the name of the property that indicates whether the copis can modify the request
     * elements (parameters, return, target, ...) or not.
     */
    const std::string COPIServiceInit::READ_WRITE_REQUEST_PROPERTY_NAME = "Cdmw.CCMContainer.Copi.RWRequest";


    OsSupport::Mutex COPIServiceInit::COPIServiceInit::M_mtx;
    
} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

