/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <string>

// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

// Local Files
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>

/** TODO: When this class was ported 99% of the Java class was commented out the following
    Comment was left:

    // 2009/04/29 FLE Remove this line and uncomment next one when PCR-2154 is fixed.

    -> need to review this PORT
*/


#include <iostream>

namespace {


} // End anonymous space


namespace Cdmw
{
namespace CCM
{
namespace COPI
{

    ::IOP::Codec_ptr COPIUtils_impl::M_codec = 0;
    const std::string COPIUtils_impl::M_ENCODING = "ISO-8859-1";
    const std::string COPIUtils_impl::M_item_sep = "<customslotitemsep>";
    const std::string COPIUtils_impl::M_item_seq_sep = "<customslotitemseqsep>";
    const std::string COPIUtils_impl::M_copi_ctx_sep = "<copictxsep>";

    
    ::Components::ContainerPortableInterceptor::COPIServiceContext *
    COPIUtils_impl::Service_to_copi_context
    (
     const ::IOP::ServiceContext & svc_ctx
     )
        throw(::IOP::Codec::FormatMismatch, ::Cdmw::BadParameterException) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

        if(svc_ctx.context_id != COPIServiceInit::CDMW_COPI_SERVICE_CONTEXT_ID) {
            CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "bad context id");
            CDMW_ASSERT(false);
            return 0;
        }

        CORBA::Any_var decoded_data;
        decoded_data = COPIUtils_impl::Get_codec()->decode (svc_ctx.context_data);
            
        ::Components::ContainerPortableInterceptor::COPIServiceContext *
              copi_ctx = 0;

        if (!((*decoded_data) >>= copi_ctx)) {
            CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "bad data in copi service context");
            CDMW_ASSERT(false);
            return 0;
        }
        
        ::Components::ContainerPortableInterceptor::COPIServiceContext_var cc = 
              new ::Components::ContainerPortableInterceptor::COPIServiceContext(*copi_ctx);

        return cc._retn();
    }
     

    ::IOP::ServiceContext *
    COPIUtils_impl::Copi_to_service_context
    (
     ::Components::ContainerPortableInterceptor::COPIServiceContext * copi_ctx
     ) 
        throw(::IOP::Codec::InvalidTypeForEncoding) {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

        ::IOP::ServiceContext * service_context = 0;
        try {
   
            CORBA::Any data;
            data <<= copi_ctx;
    
            CORBA::OctetSeq_var encoded_data =
                COPIUtils_impl::Get_codec()->encode(data);

            service_context =
                new ::IOP::ServiceContext();

            service_context->context_id = COPIServiceInit::CDMW_COPI_SERVICE_CONTEXT_ID;
            service_context->context_data = encoded_data;

        }
        catch(CORBA::SystemException & ex) {
            CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Exception during context decoding " << ex);
            throw;
        }

        return service_context;                
    }

    ::IOP::Codec_ptr
    COPIUtils_impl::Get_codec() {
        CDMW_ASSERT(0 != M_codec);
        return ::IOP::Codec::_duplicate(M_codec);
    }
       
    void
    COPIUtils_impl::Set_codec(::IOP::Codec_ptr codec) {
        M_codec = ::IOP::Codec::_duplicate(codec);
    }
    
    int
    COPIUtils_impl::Get_ARG_MODE(const ::CORBA::ParameterMode *& mode) {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        throw CORBA::NO_IMPLEMENT();
        return 0;
    }


    
    std::string 
    COPIUtils_impl::Timestamp_to_string
    (
     const Cdmw::OsSupport::OS::Timeval & tv, 
     const char * format
     )
    {
        // format date
        char time_buf[128];
        Cdmw::OsSupport::OS::get_fmt_time 
            (
             time_buf, sizeof(time_buf),
             tv.seconds,
             const_cast<char*>(format), Cdmw::OsSupport::OS::LOCAL_TIME
             );
        
        return std::string(time_buf);
    }


} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw

