/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// standard files


// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>

// Local Files
#include <CCMContainer/ccmcopi/ClientRequestInterceptorIterator_impl.hpp>
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>

namespace 
{
    const PortableInterceptor::ORBInitInfo::ObjectId COPI_CLIENT_REQUEST_INTERCEPTOR_ITERATOR_NAME
            = "f0c06d79-b0a8-4832-b6c0-6bdb1c1f845b";
    
} // end of anonymous namespace


namespace Cdmw
{
namespace CCM
{
namespace COPI
{

// static members / methods

const std::string ClientRequestInterceptorIterator_impl::M_name = "b20c8fe9-a750-4eef-acab-c5d3515f31e2";

CORBA::ULong ClientRequestInterceptorIterator_impl::M_request_id = 0;

OsSupport::Mutex ClientRequestInterceptorIterator_impl::M_mtx;

CORBA::ULong ClientRequestInterceptorIterator_impl::Get_request_id() {
    CDMW_MUTEX_GUARD(M_mtx);
    return M_request_id++;
}


ClientRequestInterceptorIterator_impl::ClientRequestInterceptorIterator_impl
(
 int origin_id, 
 int target_id,
 int origin_feature_name_id, 
 int custom_slot_item_seq_id,  
 ::CdmwCcmCopi::RegisteredCOPICollection_ptr registration
 )
    : m_origin_id_slot_id(origin_id),
      m_target_id_slot_id(target_id),
      m_origin_feature_name_slot_id(origin_feature_name_id),
      m_custom_slot_item_seq_slot_id(custom_slot_item_seq_id),
      m_registration(::CdmwCcmCopi::RegisteredCOPICollection::_duplicate(registration))
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
}


char *
ClientRequestInterceptorIterator_impl::name() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    
    return ::CORBA::string_dup(M_name.c_str());
}

void
ClientRequestInterceptorIterator_impl::destroy() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    throw CORBA::NO_IMPLEMENT();    
}


void
ClientRequestInterceptorIterator_impl::send_request
(
 PortableInterceptor::ClientRequestInfo_ptr info
 )
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    do_sending_ip(info, SEND_REQUEST);    
}


void
ClientRequestInterceptorIterator_impl::send_poll
(
 PortableInterceptor::ClientRequestInfo_ptr info
 )
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    do_sending_ip(info, SEND_POLL);
}


void
ClientRequestInterceptorIterator_impl::receive_reply
(
 PortableInterceptor::ClientRequestInfo_ptr info
 )
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    do_receiving_ip(info, RECEIVE_REPLY);
}


void
ClientRequestInterceptorIterator_impl::receive_exception
(
 PortableInterceptor::ClientRequestInfo_ptr info
 )
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    do_receiving_ip(info, RECEIVE_EXCEPTION);
}


void
ClientRequestInterceptorIterator_impl::receive_other
(
 PortableInterceptor::ClientRequestInfo_ptr info
 )
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    do_receiving_ip(info, RECEIVE_OTHER); 
}



ContainerClientRequestInfo_impl *
ClientRequestInterceptorIterator_impl::build_container_client_request_info
(
 ::PortableInterceptor::ClientRequestInfo_ptr info
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    ContainerClientRequestInfo_impl * ccri = 0;

    CORBA::OctetSeq * _origin_id = 0;
    CORBA::OctetSeq * _target_id = 0;
    const char * _feature_name = 0;

    try {
        // in case of interception of ordinary client / server there won't be a orgin_id etc., we juts need 
        // to carry without reporting errors
        // Note: the slots, must be allocated in any case, slots are allocated during copi service
        // initilisation
        CORBA::Any_var any = info->get_slot(m_origin_id_slot_id);
        if (any != 0 && any->type()->kind() != CORBA::tk_null) {
            any.in() >>= _origin_id;
        }

        any = info->get_slot(m_target_id_slot_id);
        if (any != 0 && any->type()->kind() != CORBA::tk_null) {
            any.in() >>= _target_id;             
        }
                        
        any = info->get_slot(m_origin_feature_name_slot_id);
        if (any != 0 && any->type()->kind() != CORBA::tk_null) {
            any.in() >>= _feature_name; 
        }

        ccri =
            new ContainerClientRequestInfo_impl
            (
             _feature_name,
             _origin_id,
             _target_id,
             info
             );

    } catch (::PortableInterceptor::InvalidSlot & is) {
        // This can nly happen if there is a Bug within COPIMainORBInitializer,
        // i.e. the slots have not been allocated and/or the slot ids haven't been passed
        // correctly to the ctor of this class
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Failed to get slot." << is);
        throw;
    }


    return ccri;    
}


ContainerClientRequestInfo_impl *
ClientRequestInterceptorIterator_impl::build_container_client_request_info
(
 ::PortableInterceptor::ClientRequestInfo_ptr info,
 CORBA::OctetSeq * target_id
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
           
    ContainerClientRequestInfo_impl * ccri = 0;
        
    CORBA::OctetSeq * _origin_id = 0;
    CORBA::OctetSeq * _target_id = 0;
    const char * _feature_name = 0;

    try {
        CORBA::Any_var any = info->get_slot(m_origin_id_slot_id);
        if (any != 0 && any->type()->kind() != CORBA::tk_null) {
            any.in() >>= _origin_id;
        }

        any = info->get_slot(m_origin_feature_name_slot_id);
        if (any != 0 && any->type()->kind() != CORBA::tk_null) {
            any.in() >>= _feature_name;
        }        

        any = info->get_slot(m_target_id_slot_id);
        if (any != 0 && any->type()->kind() != CORBA::tk_null) {
            any.in() >>= _target_id;
        }
        
        if(0 != _target_id  && 0 != target_id &&  *_target_id != *target_id) {        
            std::string local_t_id =
                _target_id != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*_target_id) : "NA" ;
            std::string received_t_id =
                target_id != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*target_id) : "NA";
                                 
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(), "the local target component id (" << 
                      local_t_id <<  ") is different than " << "that sent by the target component itself (" << 
                      received_t_id << ") !");
        }

        ccri =
            new ContainerClientRequestInfo_impl
            (
             _feature_name,
             _origin_id,
             target_id,
             info
             );       

        return ccri;

    } catch (::PortableInterceptor::InvalidSlot &) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Failed to get the slots");
    }

    return ccri;    
}


void 
ClientRequestInterceptorIterator_impl::build_request_service_context
(
 ::PortableInterceptor::ClientRequestInfo_ptr info
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    // This function should only be called from a context where all COPI
    // specific slots (such as target id etc.) are available. If this is not
    // the case this function will treat this an error condition

    ::Components::ContainerPortableInterceptor::CustomSlotItemSeq *
          csi = 0;

    CORBA::Any_var any = info->get_slot(m_custom_slot_item_seq_slot_id);
    if (any != 0 && any->type()->kind() != CORBA::tk_null) {
        if(!(any.in() >>= csi)) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "no data in CustomSlotItemSeq slot");
        }   
    }

    CORBA::OctetSeq * origin_id = 0;
    CORBA::OctetSeq * target_id = 0;

    any = info->get_slot(m_origin_id_slot_id);
    if (any != 0 && any->type()->kind() != CORBA::tk_null) {               
        if(!(any.in() >>= origin_id)) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "no data in origin_id slot");
        }
    }

    any = info->get_slot(m_target_id_slot_id);
    if (any != 0 && any->type()->kind() != CORBA::tk_null) {
        if(!(any.in() >>= target_id)) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "bad data in target_id slot");
        }
    }

    ::Components::ContainerPortableInterceptor::COPIServiceContext * copi_ctx_to_send =
          new ::Components::ContainerPortableInterceptor::COPIServiceContext();


    if(origin_id) {
        copi_ctx_to_send->origin_id = *origin_id;
    } else {
        ::CORBA::OctetSeq_var os = new ::CORBA::OctetSeq();
        os->length(0);
        copi_ctx_to_send->origin_id = os;
    } 

    if(target_id) { 
        copi_ctx_to_send->target_id = *target_id;            
    } else {  
        ::CORBA::OctetSeq_var os = new ::CORBA::OctetSeq();
        os->length(0);
        copi_ctx_to_send->origin_id = os;
    } 

    if(csi) {
        copi_ctx_to_send->slot_info = *csi;
    } else {
        ::Components::ContainerPortableInterceptor::CustomSlotItemSeq_var csi_var
              = new ::Components::ContainerPortableInterceptor::CustomSlotItemSeq();
        csi_var->length(0);
        copi_ctx_to_send->slot_info = csi_var;
    }

    ::IOP::ServiceContext_var svc_ctx_to_send =
          COPIUtils_impl::Copi_to_service_context(copi_ctx_to_send);
                             
    info->add_request_service_context(svc_ctx_to_send, true);
}

    
void 
ClientRequestInterceptorIterator_impl::build_request_service_context
(
 ::PortableInterceptor::ClientRequestInfo_ptr info,
 const CORBA::ULong & request_id
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    this->build_request_service_context(info);
    ::IOP::ServiceContext_var svc_ctx = new ::IOP::ServiceContext();

    CORBA::Any data;
    data <<= request_id;

    CORBA::OctetSeq_var encoded_data =
        COPIUtils_impl::Get_codec()->encode(data);

    svc_ctx->context_id = COPIServiceInit::CDMW_CLIENT_COPI_SERVICE_CONTEXT_ID;
    svc_ctx->context_data = encoded_data;

    info->add_request_service_context(*svc_ctx, true);    
}


CORBA::OctetSeq *
ClientRequestInterceptorIterator_impl::extract_from_reply_service_context
(
 ::PortableInterceptor::ClientRequestInfo_ptr info
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
            
    try {
        ::IOP::ServiceContext_var svc_ctx_received =
              info->get_reply_service_context(COPIServiceInit::CDMW_COPI_SERVICE_CONTEXT_ID);

        ::Components::ContainerPortableInterceptor::COPIServiceContext_var copi_ctx_received =
              COPIUtils_impl::Service_to_copi_context(svc_ctx_received);        

        CORBA::OctetSeq * ret_val = 
            new CORBA::OctetSeq();
        
        ret_val->length(copi_ctx_received->target_id.length());

        for(unsigned long q = 0; q < copi_ctx_received->target_id.length(); ++q) {
            (*ret_val)[q] = copi_ctx_received->target_id[q];
        }
               
        return ret_val;

    } catch (CORBA::BAD_PARAM & bp) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Failed to decode reply service context: " << bp);
        throw;
    } catch (::IOP::Codec::FormatMismatch & fm) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Failed to decode the reply service context" << fm);           
        throw;
    } catch (::Cdmw::BadParameterException & bp) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Failed to decode the reply service context" << bp);            
        throw;
    }
                
    return 0;    
}


CopisStackHolder_impl < 
    ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr> *
ClientRequestInterceptorIterator_impl::get_copis_stack
(
 unsigned long request_id
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_MUTEX_GUARD(M_mtx);

    request_context_t::iterator it =
        m_request_context.find(request_id);

    if(it == m_request_context.end()) {
        return 0;
    }
        
    if(it->second == 0) {
        CORBA::INTERNAL ex = CORBA::INTERNAL (Cdmw::OrbSupport::INTERNALBug, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    return it->second;
}


void 
ClientRequestInterceptorIterator_impl::remove_copis_stack
(
 unsigned long request_id
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    // clean up data which is assosiated with 
    // the request
    CDMW_MUTEX_GUARD(M_mtx);

    request_context_t::iterator it =
        m_request_context.find(request_id);

    if(it == m_request_context.end()) {
        return;
    }

    delete it->second;
    m_request_context.erase(it);
}


void 
ClientRequestInterceptorIterator_impl::set_copis_stack
(
 unsigned long request_id,
 CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr> * copis_stack
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_MUTEX_GUARD(M_mtx);
    m_request_context.insert(std::make_pair(request_id, copis_stack));
}

void 
ClientRequestInterceptorIterator_impl::update_state_after_send
(
 CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr> * req_dat,
 CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>::copis_list_t * safe_interceptors,
 PortableInterceptor::ClientRequestInfo_ptr info
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    if (safe_interceptors->size() > 0) {
        // No interceptor is in the RECEIVING state yet
        req_dat->set_recv_idx (req_dat->get_send_idx() + 1);
    }

    req_dat->set_copis (safe_interceptors);

    // building the copi service context
    CORBA::ULong req_id = Get_request_id();
    
    build_request_service_context (info, req_id);
    
    CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "Setting copi stack for request id " << req_id);
    set_copis_stack(req_id, req_dat);
}


void
ClientRequestInterceptorIterator_impl::do_sending_ip
(
 PortableInterceptor::ClientRequestInfo_ptr info,
 sending_ip sip
 )
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    ContainerClientRequestInfo_impl *
        ccri = build_container_client_request_info(info);

    ::CORBA::OctetSeq_var origin_id = ccri->origin_id();
    ::CORBA::OctetSeq_var target_id = ccri->target_id();
    ::CORBA::String_var name = ccri->name();

    // TODO if debug...
    ::CORBA::String_var origin_id_str =
        origin_id->length() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(origin_id.in()) : ::CORBA::string_dup("NA");

    ::CORBA::String_var target_id_str =
        target_id->length() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(target_id.in()) : ::CORBA::string_dup("NA");

    ::CORBA::String_var empty_name = CORBA::string_dup("");

    if (0 == strcmp(empty_name.in(), name.in())) {    
        name = CORBA::string_dup("NA");
    }

    
    CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(),  
              "Component " 
               << origin_id_str << " has sent a request to component "
               << target_id_str << " on its port " << name << std::endl);

    ::Components::ContainerPortableInterceptor::IntegrationPoint ip;

    ip.port = ccri->name();
    ip.operation = info->operation();

    ::CdmwCcmCopi::ClientContainerInterceptorSeq_var interceptors =
          m_registration->getClientInterceptorByIPOrderedByPriority(ip);

    if(interceptors->length() > 32767) {

        // since we will be later iterating / looping using ints, we
        // need to limit the maximum number of copis

        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "copi registry returned too many copis (more than 32767)");
        ccri->_remove_ref();
        return;
    }

    // When reaching here the interceptors sequence might be invalid since clients can 
    // unregister interceptors anytime
    
     CopisStackHolderTrait< 
         ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>::copis_list_t * safe_interceptors =
        get_safe_client_interceptors(interceptors);

    CopisStackHolder_impl< 
        ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr> * req_dat =
        new CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>();    
    
    try {
        for (unsigned int i = 0; i < safe_interceptors->size(); ++i) {
            switch(sip) {
            case SEND_REQUEST: {
                (*safe_interceptors)[i]->send_request(ccri);
                break;
            }
            case SEND_POLL: {
                (*safe_interceptors)[i]->send_poll(ccri);
                break;
            }
            default: {
                CORBA::INTERNAL ex = CORBA::INTERNAL (Cdmw::OrbSupport::INTERNALBug, CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }   
            }
            
            req_dat->set_send_idx (i);
        }
    } catch (...) {        
        CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(), "A COPI raised an exception during sending ip");
        update_state_after_send (req_dat, safe_interceptors, info);
        ccri->_remove_ref();

        throw;
    }

    update_state_after_send (req_dat, safe_interceptors, info);
    ccri->_remove_ref();
}


void
ClientRequestInterceptorIterator_impl::do_receiving_ip
(
 PortableInterceptor::ClientRequestInfo_ptr info, receiving_ip rip
 ) 
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{

    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CORBA::ULong req_id;

    // accessing context which is associted with the current request
    try {        
        req_id =
            extract_req_id_from_reply_service_context(info);
    }
    catch(CORBA::BAD_PARAM & ex) {  
        CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), 
                   "reply service context did not contain the request Id." 
                   << " The request target is not CDMW-compliant server !");
        return;
    }

    CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "Trying to access copi stack for request id " << req_id);

    CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr> * req_dat =
        get_copis_stack(req_id);

    if(0 == req_dat) {
        // bad request identifier
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "bad request identifier");
        CORBA::BAD_PARAM ex = CORBA::BAD_PARAM (Cdmw::OrbSupport::BAD_PARAMBADRequestId, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
                
    // extracting the target_id from the reply
    CORBA::OctetSeq * target_id =
        extract_from_reply_service_context(info);

    // building the ContainerRequestInfo
    ContainerClientRequestInfo_impl * ccri =
        build_container_client_request_info(info, target_id);

     CopisStackHolderTrait< 
         ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>::copis_list_t * interceptors =
        req_dat->get_copis();

    unsigned int i = 0;
        
    try {
        if(rip == RECEIVE_REPLY) {
            // RECEIVE_REPLY
            for (i = req_dat->get_send_idx();
                 i < interceptors->size() && i >= 0;
                 i--) 
                {
                    req_dat->set_recv_idx(i);
                    (*interceptors)[i]->receive_reply(ccri);
                    m_registration->removeClientCOPIRef((*interceptors)[i]);
                }
        } else {
            // RECEIVE_Execption / RECEIVE_OTHER
            for (i = req_dat->get_recv_idx() - 1;
                 i < interceptors->size() && i >= 0 && (int)i <= req_dat->get_send_idx();
                 i--)                 
                {
                    req_dat->set_recv_idx(i);
                    switch(rip) {
                    case RECEIVE_EXCEPTION: {
                        (*interceptors)[i]->receive_exception(ccri);
                        break;
                    }
                    case RECEIVE_OTHER: {
                        (*interceptors)[i]->receive_other(ccri);
                        break;
                    }
                    default: {
                        CORBA::INTERNAL ex = CORBA::INTERNAL (Cdmw::OrbSupport::INTERNALBug, CORBA::COMPLETED_NO);
                        CDMW_LOG_FUNCTION_EXCEPTION(ex);
                        throw ex;
                    }                                              
                    }    

                    m_registration->removeClientCOPIRef((*interceptors)[i]);
                }
        }

        remove_copis_stack(req_id);
        CORBA::OctetSeq_var free_target_id(target_id);
        ccri->_remove_ref();        
    } 
    catch(...) {
        if(rip == RECEIVE_REPLY) {
            // RECEIVE_REPLY
            if(i < interceptors->size() && i >= 0) {
                m_registration->removeClientCOPIRef((*interceptors)[i]);
            }
        } else {
            // RECEIVE_EXCEPTION / RECEIVE_OTHER
            if(i < interceptors->size() && i >= 0 && (int)i <= req_dat->get_send_idx()) {
                m_registration->removeClientCOPIRef((*interceptors)[i]);
            }

        }
    
        ccri->_remove_ref();
        CORBA::OctetSeq_var free_target_id(target_id);
        CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(), "COPI raised exception on receive_reply");
        throw;
    }    
}


CORBA::ULong
ClientRequestInterceptorIterator_impl::extract_req_id_from_reply_service_context
(
 ::PortableInterceptor::ClientRequestInfo_ptr info
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CORBA::ULong id = 0;
    ::IOP::ServiceContext_var svc_ctx =
          info->get_reply_service_context(COPIServiceInit::CDMW_CLIENT_COPI_SERVICE_CONTEXT_ID);
    
     CORBA::Any_var decoded_data;
     decoded_data = COPIUtils_impl::Get_codec()->decode (svc_ctx->context_data);

     if (!((*decoded_data) >>= id)) {
         CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "bad data in in reply service context");
         CDMW_ASSERT(false);
         return 0;
     }

    
    return id;    
}

// caller must delete return value

CopisStackHolderTrait< 
    ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>::copis_list_t *
ClientRequestInterceptorIterator_impl::get_safe_client_interceptors
(
 const ::CdmwCcmCopi::ClientContainerInterceptorSeq & interceptors
 ) 
{
    CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>::copis_list_t *
        safe_interceptors =
        new  CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr>::copis_list_t();

    // try to increment ref for each interceptor, if this fails then the 
    // interceptor was unregistered, i.e MUST not be used 
    for (unsigned int i = 0; i < interceptors.length(); ++i) {
        if(0 == m_registration->addClientCOPIRef(interceptors[i])) {
            safe_interceptors->push_back(interceptors[i]);
        }
    }  

    return safe_interceptors;
}


} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw

