/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// standard files


// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

// Local Files
#include <CCMContainer/ccmcopi/ContainerExtendedCopisRequestInfoData_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>

namespace 
{
    
} // end of anonymous namespace


namespace Cdmw
{
namespace CCM
{
namespace COPI
{

template <typename T>
ContainerExtendedCopisRequestInfoData_impl<T>::ContainerExtendedCopisRequestInfoData_impl
(
 const char * operation
 ) :
     m_arguments(new ::Dynamic::ParameterList()),
     m_operation(::CORBA::string_dup(operation)),
     m_system_exception_ptr(0)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    m_arguments->length(0);
};


template <typename T>
ContainerExtendedCopisRequestInfoData_impl<T>::ContainerExtendedCopisRequestInfoData_impl
(
 ::Dynamic::ParameterList * arguments,
 ::CORBA::Any & result,
 T target,
 ::CORBA::Any & the_exception,
 const char * operation
 ) :
    m_arguments(arguments),
    m_operation(operation),
    m_result(result),
    m_target(0),
    m_the_exception(the_exception),
    m_system_exception_ptr(0)
{
}


// Implementation skeleton destructor
template <typename T>
ContainerExtendedCopisRequestInfoData_impl<T>::~ContainerExtendedCopisRequestInfoData_impl (void)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        if(m_system_exception_ptr) {
        delete m_system_exception_ptr;
        m_system_exception_ptr = 0;
    } 
}

template <typename T>
::Dynamic::ParameterList *
ContainerExtendedCopisRequestInfoData_impl<T>::arguments ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    if(false == COPIServiceInit::Is_COPI_requests_read_write()) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "A Copi tried to access arguments within the info structure when requests_read_write was deactivated.");
        CORBA::BAD_CONTEXT ex = 
            CORBA::BAD_CONTEXT();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    ::Dynamic::ParameterList_var param_list
          = new ::Dynamic::ParameterList();
    
    param_list->length(m_arguments->length());

    for(CORBA::ULong i = 0; i < m_arguments->length(); i++) {
        param_list[i] = m_arguments[i];
    }

    return param_list._retn();        
}

template <typename T>
void ContainerExtendedCopisRequestInfoData_impl<T>::arguments (
    const ::Dynamic::ParameterList & arguments
  )
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    if(false == COPIServiceInit::Is_COPI_requests_read_write()) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "A Copi tried to access arguments within the info structure when requests_read_write was deactivated.");
        CORBA::BAD_CONTEXT ex = 
            CORBA::BAD_CONTEXT();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    m_arguments->length(arguments.length());

    for(::CORBA::ULong i = 0; i < arguments.length(); ++i) {
        m_arguments[i] = arguments[i];
    }
}

template <typename T>
char * ContainerExtendedCopisRequestInfoData_impl<T>::operation ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    
    return ::CORBA::string_dup(m_operation);
}

template <typename T>
::CORBA::Any * ContainerExtendedCopisRequestInfoData_impl<T>::result ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ::CORBA::Any * any = 
          new ::CORBA::Any(m_result);
    return any;
}

template <typename T>
void ContainerExtendedCopisRequestInfoData_impl<T>::result (
    const ::CORBA::Any & result
  )
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    m_result = result;
}

template <typename T>
T ContainerExtendedCopisRequestInfoData_impl<T>::target() {
    return ::CORBA::Object::_duplicate(m_target);
}

template <typename T>
void ContainerExtendedCopisRequestInfoData_impl<T>::target(T target) {
    m_target = ::CORBA::Object::_duplicate(target);
}

template <>
Components::EnterpriseComponent_ptr ContainerExtendedCopisRequestInfoData_impl<Components::EnterpriseComponent_ptr>::target() {
    return ::Components::EnterpriseComponent::_duplicate(m_target);
}

template <>
void ContainerExtendedCopisRequestInfoData_impl<Components::EnterpriseComponent_ptr>::target(Components::EnterpriseComponent_ptr target) {
    m_target = Components::EnterpriseComponent::_duplicate(target);
}

template <typename T>
::CORBA::Any * ContainerExtendedCopisRequestInfoData_impl<T>::the_exception ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        ::CORBA::Any * any = 
          new ::CORBA::Any(m_the_exception);

    return any;
}

template <typename T>
void ContainerExtendedCopisRequestInfoData_impl<T>::the_exception (
    const ::CORBA::Any & the_exception
  )
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    m_the_exception = the_exception;
}

template <typename T>
::CORBA::SystemException *
ContainerExtendedCopisRequestInfoData_impl<T>::get_system_exception_ptr() {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_system_exception_ptr;
}

template <typename T>
void
ContainerExtendedCopisRequestInfoData_impl<T>::set_system_exception_ptr(::CORBA::SystemException * se) {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    if(m_system_exception_ptr) {
        delete m_system_exception_ptr;
        m_system_exception_ptr = 0;
    } 

    m_system_exception_ptr = se;
}

template <typename T>
void
ContainerExtendedCopisRequestInfoData_impl<T>::throw_system_exception() {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    if(m_system_exception_ptr) {
        m_system_exception_ptr->_raise();
    }
}

template class ContainerExtendedCopisRequestInfoData_impl < ::CORBA::Object_ptr >;
template class ContainerExtendedCopisRequestInfoData_impl < ::Components::EnterpriseComponent_ptr >;


} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw
