/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tao/AnyTypeCode/Any_SystemException.h"
#include "tao/DynamicInterface/DII_CORBA_methods.h"
#include "tao/AnyTypeCode/ExceptionA.h"
#include "tao/AnyTypeCode/SystemExceptionA.h"
#include "tao/AnyTypeCode/AnyTypeCode_methods.h"
#include "tao/DynamicInterface/Request.h"
#include "tao/DynamicInterface/Unknown_User_Exception.h"

#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcif/CCMUtils.hpp>


#include <CCMContainer/idllib/CdmwCcmCopi.stub.hpp>
#include <CCMContainer/ccmcopi/ContainerStubRequest_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <CCMContainer/ccmcopi/COPICurrent_impl.hpp>

#include <CCMContainer/ccmcopi/ContainerExtendedCopisRequest_impl.hpp>
#include <CCMContainer/ccmcopi/ContainerStubRequestInfo_impl.hpp>
#include <CCMContainer/ccmcopi/ContainerServantRequestInfo_impl.hpp>

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {

template <typename T>
ContainerExtendedCopisRequest_impl<T>::ContainerExtendedCopisRequest_impl
(
 ::Cdmw::CCM::CIF::Context * ctx
) : m_proceed_call(true), m_logger(CCMLoggers::Get_ccmcopi_logger()), m_request_info(0), m_context(ctx)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    try {
        m_pi_current = ::PortableInterceptor::Current::_narrow
            (
             ctx->get_ccm_context()->resolve_initial_references("PICurrent")
             );
        
        m_copi_current = ::CdmwCcmCopi::COPICurrent::_narrow
            (
             ctx->get_ccm_context()->resolve_initial_references(COPICurrent_impl::COPI_CURRENT_ID.c_str())
             );
            
        
    } catch (CORBA::ORB::InvalidName & e) {
        // TODO: manage failing resolve_init_ref - (comment taken as it was found in Java)
        
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        CDMW_ERROR(m_logger, "Failed to obatin pi or copi current due to: " << e);
        throw;
    }
}

template <>
void ContainerExtendedCopisRequest_impl<ContainerStubRequestInfo_impl>::initialise
(
 const char * name, 
 const char * component_id,
 ::CORBA::Object_ptr target
 ) {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    char * s_target_id = 0; 

    try {
        ::CORBA::String_var oid = m_context->extract_oid_str_from_ref(target);
        ::Cdmw::CCM::CIF::CCMUtils::NameVector names =
              ::Cdmw::CCM::CIF::CCMUtils::Read_oid(oid.in());
        
        CDMW_ASSERT(false == names.empty());
        
        s_target_id = strdup(names[0].c_str());

        // NOTE: extract_oid_str_from_ref uses poa's ref2id operation,
        // which will fail if the obj ref was not created by the poa what is
        // definitely the case if the target is not collocated. Hence we can't rely
        // on this but it would make a good optimisation.
        // Therefore, we catch all exceptions
    } catch(::Components::BadComponentReference & bcr){
        CDMW_DEBUG(m_logger, "Failed to read the target oid: " << bcr);
    } catch (::CORBA::SystemException & se) {
        CDMW_DEBUG(m_logger, "Failed to read the target oid. Caught " << se);
    } catch (...) {
        CDMW_DEBUG(m_logger, "Failed to read the target oid. Caught unknown exception.");
    }
    
    ::CORBA::OctetSeq * origin_id = 
          0 != component_id ? ::Cdmw::CCM::Common::string_to_OctetSeq(component_id) : 0;

    ::CORBA::OctetSeq * target_id =
          0 != s_target_id ? ::Cdmw::CCM::Common::string_to_OctetSeq(s_target_id) : 0;

    // opreation implemented by subclass
    ::CORBA::String_var op = operation();

    m_request_info =
        new ContainerStubRequestInfo_impl
        (
         name,
         origin_id, 
         target_id,
         op.in()
         );

    // free origin_id / target_id
    if(origin_id) {
        ::CORBA::OctetSeq_var v = origin_id;
    }
    if(target_id){
        ::CORBA::OctetSeq_var v = target_id;
    }

    m_request_info->target(target);
    ::CORBA::Any any;
    m_request_info->result(any);    
}

template <>
void ContainerExtendedCopisRequest_impl<ContainerServantRequestInfo_impl>::initialise
(
 const char * name, 
 const char * component_id,
 ::CORBA::Object_ptr target
 ) {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");

    ::CORBA::OctetSeq * target_id =
          0 != component_id ? ::Cdmw::CCM::Common::string_to_OctetSeq(component_id) : 0;

    // operation called by subclass
    ::CORBA::String_var op = operation();
           
    m_request_info =
        new ContainerServantRequestInfo_impl
        (
         name,
         0, 
         target_id,
         op.in()
         );

    if(target_id) {
        ::CORBA::OctetSeq_var v = target_id;
    }
    
    ::Components::EnterpriseComponent_var ec_target =
          ::Components::EnterpriseComponent::_narrow(target);

    m_request_info->target(ec_target.in());    
}



template <typename T>
ContainerExtendedCopisRequest_impl<T>::~ContainerExtendedCopisRequest_impl
()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    m_request_info->_remove_ref();
}



template <typename T>
void 
ContainerExtendedCopisRequest_impl<T>::add_in_args 
(
 const ::CORBA::Any & in_arg
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");    
    do_add_args(in_arg, CORBA::PARAM_IN);
}

template <typename T>
::CORBA::Any *
ContainerExtendedCopisRequest_impl<T>::get_arg 
(
 unsigned int index
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    ::Dynamic::ParameterList_var args = m_request_info->arguments();
    return get_arg(args.in(), index);
}

template <typename T>
::CORBA::Any *
ContainerExtendedCopisRequest_impl<T>::get_arg (const ::Dynamic::ParameterList & arguments, unsigned int index) {
        
    if(index > arguments.length()) {
        return 0; //TODO : throw an exception
    }
        
    ::CORBA::Any * any =
          new ::CORBA::Any(arguments[index].argument);
        
    return any;
}


template <typename T>
void 
ContainerExtendedCopisRequest_impl<T>::add_out_args 
(
 const ::CORBA::Any & out_arg
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    do_add_args(out_arg, CORBA::PARAM_OUT);
}
    
template <typename T>
void 
ContainerExtendedCopisRequest_impl<T>::add_inout_args 
(
 const ::CORBA::Any & inout_arg
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    do_add_args(inout_arg, CORBA::PARAM_INOUT);
}

    
template <typename T>
void 
ContainerExtendedCopisRequest_impl<T>::add_result
(
 const ::CORBA::Any & res
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    m_request_info->result(res);
}


template <typename T>
void 
ContainerExtendedCopisRequest_impl<T>::add_exception
(
 const ::CORBA::Any & ex
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");    
    m_request_info->the_exception(ex);
}

        
template <typename T>
::CORBA::Any *
ContainerExtendedCopisRequest_impl<T>::the_exception ()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    return m_request_info->the_exception();   
}
 

template <typename T>
::CORBA::SystemException *
ContainerExtendedCopisRequest_impl<T>::the_systemException()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    return m_request_info->get_system_exception_ptr();
}

template <typename T>
::CORBA::Any *
ContainerExtendedCopisRequest_impl<T>::result () {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    return m_request_info->result();
}

    
template <typename T>
void
ContainerExtendedCopisRequest_impl<T>::add_system_exception
(
 ::CORBA::SystemException & se
)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");    

    // we use the obscure "_tao_duplicate()" method here
    // since there is no chance to ever get an SysEx copied
    // (unless one is brave enough to give memcopy a try)
    // Hopefully, a heap'd copy is returned, and hopefully future
    // versions of TAO won't break this
    ::CORBA::Exception * ex_heaped 
          = se._tao_duplicate();

    ::CORBA::SystemException * se_heaped 
          = ::CORBA::SystemException::_downcast(ex_heaped);

    m_request_info->set_system_exception_ptr(se_heaped);
    
    // Insert any, but we will have a hard time extraction
    // the sys ex unless we want handle every single possible
    // sys ex! We need this because we need to chec via the_exception whether
    // an exception occured at all. 
    ::CORBA::Any any;
    any <<= se;
    m_request_info->the_exception(any);

    // Note: we could have used: CORBAExceptionHelper::Extract_SystemException()
    // but this function relies on TAO internal calls as well. Furthermore, it might not
    // handle all possible SystemExceptions
}


template <typename T>
void
ContainerExtendedCopisRequest_impl<T>::do_add_args
(
 const ::CORBA::Any & arg,
 ::CORBA::ParameterMode mode
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    ::Dynamic::Parameter_var param =
          new ::Dynamic::Parameter ();

    param->argument = arg;
    param->mode = mode;

    ::Dynamic::ParameterList_var args =
          m_request_info->arguments();   

    if(0 != args) {
        args->length(args->length() + 1);

        args[args->length() - 1]
            = param;        
            
    } else {
        args = new ::Dynamic::ParameterList();
        args->length(1);
        ::CORBA::ULong indx = 0;
        args[indx] = param;
    }

    m_request_info->arguments(args.in());
}


template class ContainerExtendedCopisRequest_impl < ContainerStubRequestInfo_impl >;
template class ContainerExtendedCopisRequest_impl < ContainerServantRequestInfo_impl >;


} // End namespace Cdmw

} // End namespace CCM

} // End namespce COPI

