/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

#include <CCMContainer/ccmcopi/ContainerRequestInfo_impl.hpp>


namespace Cdmw {

namespace CCM {

namespace COPI {

// Implementation skeleton constructor
ContainerRequestInfo_impl::ContainerRequestInfo_impl 
(
 const char * name,
 CORBA::OctetSeq * origin_id,
 CORBA::OctetSeq * target_id
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    if(0 == name) {
        m_name = ::CORBA::string_dup("");
    } else {
        m_name = ::CORBA::string_dup(name);
    }

    ::CORBA::ULong origin_id_len = 0 == origin_id ? 0 : origin_id->length();
    ::CORBA::ULong target_id_len = 0 == target_id ? 0 : target_id->length();

    m_origin_id = new CORBA::OctetSeq();
    m_target_id = new CORBA::OctetSeq();
    m_origin_id->length(origin_id_len);
    m_target_id->length(target_id_len);

    for(::CORBA::ULong i = 0; i < origin_id_len; ++i) {
        m_origin_id[i] = (*origin_id)[i];
    }

    for(::CORBA::ULong i = 0; i < target_id_len; ++i) {
        m_target_id[i] = (*target_id)[i];
    }


}

// Implementation skeleton destructor
ContainerRequestInfo_impl::~ContainerRequestInfo_impl (void)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    // empty
}

::CORBA::OctetSeq *
ContainerRequestInfo_impl::origin_id ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ::CORBA::OctetSeq_var origin_id 
          = new ::CORBA::OctetSeq();    
    origin_id->length(m_origin_id->length());

    for(::CORBA::ULong i = 0; i < m_origin_id->length(); ++i) {
        origin_id[i] = m_origin_id[i];
    }
    
    return origin_id._retn();
}

::CORBA::OctetSeq *
ContainerRequestInfo_impl::target_id ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    ::CORBA::OctetSeq_var target_id 
          = new ::CORBA::OctetSeq();
    target_id->length(m_target_id->length());
    for(::CORBA::ULong i = 0; i < m_target_id->length(); ++i) {
        target_id[i] = m_target_id[i];
    }
    
    return target_id._retn();
}

char *
ContainerRequestInfo_impl::name ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return ::CORBA::string_dup(m_name);
}

   
void
ContainerRequestInfo_impl::set_origin_id(::CORBA::OctetSeq * id) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ::CORBA::OctetSeq_var _id
          = new ::CORBA::OctetSeq();

    if(id) {    
        _id->length((id->length()));
        for(::CORBA::ULong i = 0; i < id->length(); ++i) {
            _id[i] = (*id)[i];
        }

    } else {
        _id->length(0);
    }

    m_origin_id = _id;
}

void 
ContainerRequestInfo_impl::set_target_id(::CORBA::OctetSeq * id) {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ::CORBA::OctetSeq_var _id
          = new ::CORBA::OctetSeq();

    if(id) {
        _id->length((id->length()));
        for(::CORBA::ULong i = 0; i < id->length(); ++i) {
            _id[i] = (*id)[i];
        }
    } else {
        _id->length(0);
    }

    m_target_id = _id;
}


} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

