/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// standard files


// Cdmw Files
#include <Foundation/idllib/Components.skel.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

// Local Files
#include <CCMContainer/ccmcopi/ContainerServantRequestInfo_impl.hpp>
#include <CCMContainer/ccmcopi/ContainerRequestInfo_impl.hpp>

namespace 
{
    
} // end of anonymous namespace


namespace Cdmw
{
namespace CCM
{
namespace COPI
{

// Implementation skeleton constructor
ContainerServantRequestInfo_impl::ContainerServantRequestInfo_impl 
(
 const char * name,
 ::CORBA::OctetSeq * origin_id,
 ::CORBA::OctetSeq * target_id,
 const char * operation
 ) : ContainerRequestInfo_impl(name, origin_id, target_id),
     m_data(operation)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
}

ContainerServantRequestInfo_impl::ContainerServantRequestInfo_impl 
(
 const char * name,
 ::CORBA::OctetSeq * origin_id,
 ::CORBA::OctetSeq * target_id,
 ::Dynamic::ParameterList * arguments,
 ::CORBA::Any & result,
 ::Components::EnterpriseComponent_ptr target,
 ::CORBA::Any & the_exception,
 const char * operation
 ) : ContainerRequestInfo_impl(name, origin_id, target_id),
     m_data(arguments, result, target, the_exception, operation)
{
}


// Implementation skeleton destructor
ContainerServantRequestInfo_impl::~ContainerServantRequestInfo_impl (void)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    // empty
}

::Dynamic::ParameterList * ContainerServantRequestInfo_impl::arguments ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_data.arguments ();
}

void ContainerServantRequestInfo_impl::arguments (
    const ::Dynamic::ParameterList & arguments
  )
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    m_data.arguments(arguments);
}

char * ContainerServantRequestInfo_impl::operation ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_data.operation();
}

::CORBA::Any * ContainerServantRequestInfo_impl::result ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_data.result();
}

void ContainerServantRequestInfo_impl::result (
    const ::CORBA::Any & result
    )
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    m_data.result(result);
}

::Components::EnterpriseComponent_ptr ContainerServantRequestInfo_impl::target ()
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_data.target();
}

void ContainerServantRequestInfo_impl::target (
    ::Components::EnterpriseComponent_ptr target
    )
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    m_data.target(target);
}

::CORBA::Any * ContainerServantRequestInfo_impl::the_exception ()
  throw (
    ::CORBA::SystemException
  )
{    
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_data.the_exception();
}

void ContainerServantRequestInfo_impl::the_exception (
    const ::CORBA::Any & the_exception
  )
  throw (
    ::CORBA::SystemException
  )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    m_data.the_exception(the_exception);
}

::CORBA::SystemException *
ContainerServantRequestInfo_impl::get_system_exception_ptr() {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return m_data.get_system_exception_ptr();
}

void
ContainerServantRequestInfo_impl::set_system_exception_ptr(::CORBA::SystemException * se) {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    m_data.set_system_exception_ptr(se);
}

void
ContainerServantRequestInfo_impl::throw_system_exception() {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    m_data.throw_system_exception();
}


} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw
