/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "tao/AnyTypeCode/ExceptionA.h"

#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcif/CCMUtils.hpp>

#include <CCMContainer/idllib/CdmwCcmCopi.stub.hpp>
#include <CCMContainer/ccmcopi/ContainerServantRequest_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <CCMContainer/ccmcopi/COPICurrent_impl.hpp>

namespace {

} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {



ContainerServantRequest_impl::ContainerServantRequest_impl
(
 ::Cdmw::CCM::CIF::Context * ctx
 ) : ContainerExtendedCopisRequest_impl<ContainerServantRequestInfo_impl>(ctx),
    m_servant_copi_iterator(0)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
}


ContainerServantRequest_impl::~ContainerServantRequest_impl()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    delete m_servant_copi_iterator;
    m_servant_copi_iterator = 0;
}

            
void 
ContainerServantRequest_impl::pre_invoke()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    pre_invoke_extended_copi();
}


void 
ContainerServantRequest_impl::post_invoke() 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    post_invoke_basic_copi();
    post_invoke_extended_copi();
}
    
    
void 
ContainerServantRequest_impl::pre_invoke_extended_copi()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    // TODO: not needed verify this
    // bool proceed_call = true;

    m_proceed_call = true;
        
    // getting the origin_id from the thread context 
    try {
        ::CORBA::Any_var any_origin_id =
              m_pi_current->get_slot
              (
               m_copi_current->get_slot_id(COPIServiceInit::M_origin_ID_SlotNAME.c_str())
               );

        ::CORBA::OctetSeq * origin_id = 0;
        if(0 != any_origin_id && any_origin_id->type()->kind() != CORBA::tk_null) {
            any_origin_id.in() >>= origin_id;
        }

        // origin id not set, this might happen if
        // a non cdmw client or a cdmw client with deactivated copi support
        // is calling
        if(0 == origin_id) {
            CDMW_WARN(CCMLoggers::Get_ccmcopi_logger(), "origin_id not set in slot "
                    << COPIServiceInit::M_origin_ID_SlotNAME);
        }

        m_request_info->set_origin_id(origin_id);

    } catch (::PortableInterceptor::InvalidSlot & e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcopi_logger(), "pre_invoke_extended_copi caught InvalidSlot, copi service not initialised properly"
                   << e);
        // TODO: java mosic just caries on and print the stacktrace
        // this is rather to be considered as a error condition since slots should be allocated
        // during initialisation of the copi service
        throw;                   
    }

    ::CdmwCcmCopi::RegisteredCOPICollection_ptr reg =
          m_copi_current->get_registered_copi_collection();

    ::Components::ContainerPortableInterceptor::IntegrationPoint ip;
    ::CORBA::String_var name = m_request_info->name();
    ::CORBA::String_var operation = m_request_info->operation();

    ip.port = name;
    ip.operation = operation;
      
    ::CdmwCcmCopi::ServantContainerInterceptorSeq_var interceptors =
          reg->getServantInterceptorByIPOrderedByPriority(ip);
        
    CDMW_DEBUG(CCMLoggers::Get_ccmcopi_logger(), 
               "Iterating over " << interceptors->length() << " SERVANT interceptors");
        
    m_servant_copi_iterator =
        new ServantContainerInterceptorIterator_impl 
        (
         interceptors, 
         m_copi_current->get_registered_copi_collection()
         );
        
    m_servant_copi_iterator->servant_receive_request(m_request_info, m_proceed_call);
}


void 
ContainerServantRequest_impl::post_invoke_basic_copi() 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");    
    // nothing to do for now
}

    
void 
ContainerServantRequest_impl::post_invoke_extended_copi ()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");

    if(!m_proceed_call){
        m_servant_copi_iterator->servant_send_other(m_request_info);
        return;
    }
        
    // checking the request outcome
    
    ::CORBA::Any_var the_exception 
          = m_request_info->the_exception();

    if(::CORBA::tk_null == the_exception->type()->kind()) {
        m_servant_copi_iterator->servant_send_reply
            (
             m_request_info,
             m_proceed_call
             );
    } else {
        m_servant_copi_iterator->servant_send_exception
            (
             m_request_info,
             m_proceed_call
             ); 
    }
}

} // End namespace Cdmw

} // End namespace CCM

} // End namespce COPI
