/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "tao/AnyTypeCode/Any_SystemException.h"
#include "tao/DynamicInterface/DII_CORBA_methods.h"
#include "tao/AnyTypeCode/ExceptionA.h"
#include "tao/AnyTypeCode/SystemExceptionA.h"
#include "tao/AnyTypeCode/AnyTypeCode_methods.h"
#include "tao/DynamicInterface/Request.h"
#include "tao/DynamicInterface/Unknown_User_Exception.h"

#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcif/CCMUtils.hpp>


#include <CCMContainer/idllib/CdmwCcmCopi.stub.hpp>
#include <CCMContainer/ccmcopi/ContainerStubRequest_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <CCMContainer/ccmcopi/COPICurrent_impl.hpp>

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {



/**
 * represents an outcoming request
 */
    
ContainerStubRequest_impl::ContainerStubRequest_impl
(
 ::Cdmw::CCM::CIF::Context * ctx
) : ContainerExtendedCopisRequest_impl<ContainerStubRequestInfo_impl>(ctx),
    m_stub_copi_iterator(0)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
}


ContainerStubRequest_impl::~ContainerStubRequest_impl()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    delete m_stub_copi_iterator;
}

               
void
ContainerStubRequest_impl::invoke() 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");

    ::CORBA::Request_var r;

    ::CORBA::Any_var the_exception 
          = m_request_info->the_exception();
               
    // return if the call has not to be done or an exception has been raised 
    if (!m_proceed_call || ::CORBA::tk_null != the_exception->type()->kind())
        return;
        
    CDMW_DEBUG(CCMLoggers::Get_ccmcopi_logger(), "Performing a DII invocation of operation " 
               << this->m_request_info->operation());        

    try {
        ::CORBA::String_var op = m_request_info->operation();

        r = m_request_info->target()->_request(op.in());
                     
        ::CORBA::NVList_ptr arg_list = r->arguments();

        ::Dynamic::ParameterList_var param_list =
              m_request_info->arguments();

        ::CORBA::ULong flags;
        for(unsigned i = 0; i <  param_list->length(); ++i) {

            switch(param_list[i].mode) {
            case ::CORBA::PARAM_IN: {
                flags = ::CORBA::ARG_IN;
                break;
            }
            case ::CORBA::PARAM_OUT: {
                flags = ::CORBA::ARG_OUT;
                break;
            }
            case ::CORBA::PARAM_INOUT: {
                flags = ::CORBA::ARG_INOUT;
                break;
            }

            default: {
                CDMW_ASSERT(0);
            }
           }
  
            arg_list->add_value
                (
                 "", 
                 param_list[i].argument,
                 flags
                 );
        }
        
        // set return tk
        ::CORBA::Any_var result_any
              = m_request_info->result();

        r->set_return_type(result_any->type());
                        
        // add exception tc
        
        ::CORBA::ExceptionList_ptr ex_list =
              r->exceptions();

        for(std::vector< ::CORBA::TypeCode_var >::iterator it = m_exceptions_type_codes.begin();
            it != m_exceptions_type_codes.end();
            ++it)
            {
                ex_list->add((*it).in());
            }
       
            
        // invoke request
         r->invoke();


        // NOTE: we do not set return value in case
        // of exception (JAVA MOSIC does this though)
                  
         m_request_info->result(r->return_value());

    } catch (::Components::CreateFailure & cf) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcopi_logger(), "Failed to create the CORBA request "
                   << this->operation() << ": " 
                   << cf);
        ::CORBA::Any any;
        any <<= cf;
        m_request_info->the_exception(any);
    } catch (CORBA::UnknownUserException & user_ex) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcopi_logger(), "Failed to create the CORBA request "
                   << this->operation() << ": " 
                   << user_ex);

        m_request_info->the_exception(user_ex.exception());
    } catch (CORBA::SystemException & se) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcopi_logger(), "Failed to create the CORBA request "
                   << this->operation() << ": " << se);

        this->add_system_exception(se);
    }
}
    
    
void 
ContainerStubRequest_impl::pre_invoke()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    pre_invoke_extended_copi();
    pre_invoke_basic_copi();
}


void 
ContainerStubRequest_impl::post_invoke() 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    post_invoke_basic_copi();
    post_invoke_extended_copi();
}
    

void 
ContainerStubRequest_impl::pre_invoke_basic_copi() 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    try {
            
        // setting the origin_id
        {
            ::CORBA::Any any;
            any <<= m_request_info->origin_id();

            m_pi_current->set_slot
                (
                 m_copi_current->get_slot_id(COPIServiceInit::M_origin_ID_SlotNAME.c_str()),
                 any
                 );
        }

        // setting the target_id            
        {
            ::CORBA::Any any;
            any <<= m_request_info->target_id();
            m_pi_current->set_slot
                (
                 m_copi_current->get_slot_id(COPIServiceInit::M_target_ID_SlotNAME.c_str()),
                 any
                 );
        }
            
        // setting the origin feature name
        {
            ::CORBA::Any any;
            ::CORBA::String_var name = m_request_info->name();
            any <<= name.in();

            m_pi_current->set_slot
                (
                 m_copi_current->get_slot_id(COPIServiceInit::M_originFeatureName_SlotNAME.c_str()),
                 any
                 );
        }

        // initilise CustomSlotItemSeq
        {
            ::CORBA::Any any;
            ::Components::ContainerPortableInterceptor::CustomSlotItemSeq * custom_slot_item_seq =
                  new ::Components::ContainerPortableInterceptor::CustomSlotItemSeq();

            custom_slot_item_seq->length(0);
            
            any <<= custom_slot_item_seq;

            m_pi_current->set_slot
                (
                 m_copi_current->get_slot_id(COPIServiceInit::M_originCustomSlotItemSeq_SlotNAME.c_str()),
                 any
                 );
        }

        
    } catch (::PortableInterceptor::InvalidSlot & is) {
        CDMW_ERROR(CCMLoggers::Get_ccmcopi_logger(), "pre_invoke_basic_copi failed, copi service not initilised or the needed slots have not been allocated properly: "
                   << is);
        throw;                   
    }
}
       
    
void 
ContainerStubRequest_impl::pre_invoke_extended_copi()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    // TODO: not needed verify this
    // bool proceed_call = true;
        
    try {
        // Copy content of slot identified by TargetCustomSlotItem_Seq_SlotID
        // to slot identified by OriginCustomSlotItem_Seq_SlotID
        // for chaining purpose
        ::CORBA::Any_var any =
              m_pi_current->get_slot
              (
               m_copi_current->get_slot_id(COPIServiceInit::M_targetCustomSlotItemSeq_SlotNAME.c_str())
               );


        m_pi_current->set_slot
            (
             m_copi_current->get_slot_id(COPIServiceInit::M_originCustomSlotItemSeq_SlotNAME.c_str()),
             any.in()
             );

    } catch (::PortableInterceptor::InvalidSlot & is) {
        CDMW_ERROR(CCMLoggers::Get_ccmcopi_logger(), "pre_invoke_extended_copi failed, copi service not initilised or the needed slots have not been allocated properly: "
                   << is);
        throw;                   
    }


    // call extended copis

    ::CdmwCcmCopi::RegisteredCOPICollection_ptr reg =
          m_copi_current->get_registered_copi_collection();

    ::Components::ContainerPortableInterceptor::IntegrationPoint ip;

    
    ::CORBA::String_var name = m_request_info->name();
    ::CORBA::String_var op = m_request_info->operation();

    ip.port = name;
    ip.operation = op;

    ::CdmwCcmCopi::StubContainerInterceptorSeq_var interceptors =
          reg->getStubInterceptorByIPOrderedByPriority(ip);

        
    CDMW_DEBUG(CCMLoggers::Get_ccmcopi_logger(), "Iterating over " 
               << interceptors->length() << " STUB interceptors");
        
    m_stub_copi_iterator =
        new StubContainerInterceptorIterator_impl
        (
         interceptors,
         m_copi_current->get_registered_copi_collection()
         );
        
    m_stub_copi_iterator->stub_send_request
        (
         m_request_info,
         m_proceed_call
         );
}


void 
ContainerStubRequest_impl::post_invoke_basic_copi() 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");
    // may be unsetting the slots ?!
}

    
void 
ContainerStubRequest_impl::post_invoke_extended_copi ()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcopi_logger(), "");

    if(!m_proceed_call) {
        m_stub_copi_iterator->stub_receive_other(m_request_info);
        return;
    }
            
    // checking the request outcome

    ::CORBA::Any_var the_exception 
          = m_request_info->the_exception();

    if(::CORBA::tk_null == the_exception->type()->kind()) {
        m_stub_copi_iterator->stub_receive_reply
            (
             m_request_info,
             m_proceed_call
             );
    } else { 
        m_stub_copi_iterator->stub_receive_exception
            (
             m_request_info,
             m_proceed_call
             ); 
    }
}




} // End namespace Cdmw

} // End namespace CCM

} // End namespce COPI


