/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcopi/PushContainerStubRequest_impl.hpp>



namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {

    
PushContainerStubRequest_impl::PushContainerStubRequest_impl
(
 const char * source_name,
 const char * component_id, 
 ::CosEventComm::PushConsumer_ptr proxy_consumer,
 ::Cdmw::CCM::CIF::Context * ctx
 ) : Cdmw::CCM::COPI::ContainerExtendedCopisRequest_impl<Cdmw::CCM::COPI::ContainerStubRequestInfo_impl>(ctx),
     ContainerStubRequest_impl(ctx), 
     m_source_event_port_name(::CORBA::string_dup(source_name)),
     m_proxy_consumer(::CosEventComm::PushConsumer::_duplicate(proxy_consumer))
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");   
    this->initialise(source_name, component_id, proxy_consumer);
}


PushContainerStubRequest_impl::~PushContainerStubRequest_impl() {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    // empty
}

char * PushContainerStubRequest_impl::operation() {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    return ::CORBA::string_dup("push");
}

void PushContainerStubRequest_impl::push(::CORBA::Any event) {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    if(true == this->is_read_write()){
        ::CORBA::Any arg;
        arg <<= event;
        this->add_in_args(arg);
        this->add_exception_tc(::CosEventComm::_tc_Disconnected);   
        this->pre_invoke();
        this->invoke();
        this->post_invoke();
    } else {
        try {
            if(true == this->m_proceed_call && ::CORBA::tk_null != m_request_info->the_exception()->type()->kind()) {
                m_proxy_consumer->push(event);
            }                
        } catch (::CosEventComm::Disconnected & d) {
            CDMW_LOG_FUNCTION_EXCEPTION(d);
            this->add_exception_disconnected(d);
              
        } catch(::CORBA::SystemException & se){
            CDMW_LOG_FUNCTION_EXCEPTION(se);
            this->add_system_exception(se);
        }
    }
        
    throw_exception();
}

void PushContainerStubRequest_impl::add_exception_disconnected(CosEventComm::Disconnected & d) {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ::CORBA::Any d_any;
    d_any <<= d;
    this->add_exception(d_any);
}

void PushContainerStubRequest_impl::throw_exception() throw (::CORBA::SystemException) {
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ::CORBA::Any_var ex_any = m_request_info->the_exception();
    if(::CORBA::tk_null == ex_any->type()->kind()) {
        return;
    }

    if(ex_any->type()->equal(::CosEventComm::_tc_Disconnected)) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Cannot send event with " << 
                   m_source_event_port_name << " port since it's not connected");
        CORBA::INTERNAL e(Cdmw::OrbSupport::INTERNAL, ::CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    } else {
        ::CORBA::SystemException * e = this->the_systemException();
        if(0 != e) {
            CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Failed to send event with " << 
                       m_source_event_port_name << " port");
            CDMW_LOG_FUNCTION_EXCEPTION(*e);
            e->_raise();
        }
    }            
}
            

} // End namespace Cdmw

} // End namespace CCM

} // End namespce COPI


