/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import java.util.HashMap;
import java.util.Properties;

import org.omg.Components.ContainerPortableInterceptor.COPIServiceContext;
import org.omg.IOP.ServiceContext;
import org.omg.PortableInterceptor.Current;
import org.omg.PortableInterceptor.ClientRequestInterceptor;
import org.omg.PortableInterceptor.ServerRequestInterceptor;

import cdmw.orbsupport.ExceptionMinorCodes;

import com.thalesgroup.CdmwCcmCopi.COPICurrent;



/**
 * This class initializes the basic and the extended interception capabilities of the container 
 */
public class COPIServiceInit {
    
    private final static String ORBINITIALIZER_PROPERTY_NAME="org.omg.PortableInterceptor.ORBInitializerClass.COPI";
    private final static String ORBINITIALIZER_CLASS_NAME="cdmw.ccm.ccmcopi.COPIMainORBInitializer";
    
    private static Properties COPIServiceProps;
    
    private static boolean initialized = false;

    /**
     * The needed slots identifiers
     */
    final static String Origin_ID_SlotNAME ="Origin_ID_SlotID",
                        Target_ID_SlotNAME ="Target_ID_SlotID",
                        OriginFeatureName_SlotNAME ="OriginFeatureName_SlotID",
                        TargetFeatureName_SlotNAME ="TargetFeatureName_SlotID",
                        OriginCustomSlotItemSeq_SlotNAME ="OriginCustomSlotItemSeq_SlotID",
                        TargetCustomSlotItemSeq_SlotNAME ="TargetCustomSlotItemSeq_SlotID";
 
    /** 
     * The COPI Service Context ID
     * TODO: to remove this ID and use the standard one  
     */
    final static int CDMW_COPI_SERVICE_CONTEXT_ID = 10000;
    
    /**
     * The Basic client copi service context Id
     * It is used to vehicle the request Id
     */
    static final int CDMW_CLIENT_COPI_SERVICE_CONTEXT_ID = 2 | ExceptionMinorCodes.CDMW_VMCID;
    
    /**
     * the name of the property that indicates whether the copis can modify the request
     * elements (parameters, return, target, ...) or not.
     */
    static final String READ_WRITE_REQUEST_PROPERTY_NAME = "Cdmw.CCMContainer.Copi.RWRequest";
    
    /**
     * Allows to enable the COPIs interception.
     * To enable the basic COPIs, this method must be called before the orb init.
     * @param args not used for now
     */
    public static void init(String[] args){
        //registering the COPI ORBInitializer 
        System.setProperty(ORBINITIALIZER_PROPERTY_NAME, ORBINITIALIZER_CLASS_NAME);
        COPIServiceProps = System.getProperties();
        initialized = true;
    }
    
    public static boolean isInitialized(){
        return initialized;
    }
    
    public static void enableReadWriteRequests(){
        COPIServiceProps.setProperty(READ_WRITE_REQUEST_PROPERTY_NAME, "yes");
    }
    
    public static void disableReadWriteRequests(){
        COPIServiceProps.setProperty(READ_WRITE_REQUEST_PROPERTY_NAME, "no");
    }
    
   
    public static boolean isCOPIRequestsReadWrite(){
       return (COPIServiceProps.getProperty(COPIServiceInit.READ_WRITE_REQUEST_PROPERTY_NAME)!=null && 
               "yes".equals(COPIServiceProps.getProperty(COPIServiceInit.READ_WRITE_REQUEST_PROPERTY_NAME)));
   }
}
