/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;


import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import java.lang.reflect.Array;
import java.util.Properties;

import org.omg.CORBA.ARG_IN;
import org.omg.CORBA.ARG_INOUT;
import org.omg.CORBA.ARG_OUT;
import org.omg.CORBA.Any;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.ORB;
import org.omg.CORBA.ParameterMode;
import org.omg.Components.ContainerPortableInterceptor.COPIServiceContext;
import org.omg.Components.ContainerPortableInterceptor.COPIServiceContextHelper;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItem;
import org.omg.IOP.Codec;
import org.omg.IOP.ServiceContext;
import org.omg.IOP.CodecPackage.FormatMismatch;
import org.omg.IOP.CodecPackage.InvalidTypeForEncoding;

import cdmw.ccm.common.CCMLoggers;
import cdmw.common.Assert;
import cdmw.common.BadParameterException;
import cdmw.orbsupport.ExceptionMinorCodes;

/**
 *
 */
public class COPIUtils {
    
    private static Codec codec;
    private final static String ENCODING = "ISO-8859-1";
    static final String itemSep = "<customslotitemsep>",
                        itemSeqSep = "<customslotitemseqsep>",
                        copiCtxSep = "<copictxsep>";
    
    // 2009/04/29 FLE Remove this line and uncomment next one when PCR-2154 is fixed.
    // 2009/04/29 FLE Remove this line and uncomment next one when PCR-2154 is fixed.
    static ORB orb = ORB.init((String[])null, (Properties)null);
    // static ORB orb = ORB.init();
    
//     /**
//      * encodes a string into a byte sequence using the ISO-8859-1 charset
//      * @param str the string to encode
//      * @return the coded byte array
//      */
//     public static byte[] stringToBytes(String str){
        
//         byte[] bytes;
//         try{
//             bytes = str.getBytes(ENCODING);
//         }catch (java.io.UnsupportedEncodingException e) {
//             CDMW_WARN(CCMLoggers.GetCcmcommonLogger(),
//                     "WARNING: UnsupportedEncodingException for "+ ENCODING,
//                     new Throwable());
//                 e.printStackTrace();
//                 bytes= str.getBytes(); //default encoding
//         }
        
//         return bytes;
//     }
        
//     /**
//      * decodes a bytes array into a String using the ISO-8859-1 charset
//      * @param bytes the bytes sequence to decode
//      * @return the resultant string
//      */
//     public static String bytesToString(byte[] bytes){
//         String str;
//         try{
//             str = new String(bytes, ENCODING);
//         }catch (java.io.UnsupportedEncodingException e) {
//             CDMW_WARN(CCMLoggers.GetCcmcommonLogger(),
//                     "WARNING: UnsupportedEncodingException for "+ ENCODING,
//                     new Throwable());
//                 e.printStackTrace();
//                 str = new String(bytes); //default encoding
//         }
                        
//         return str;
//     }
    
//     /**
//      * encodes a COPIServiceContext into a sequence of bytes
//      * @param ctx
//      * @return
//      * @throws InvalidTypeForEncoding
//      */
//     public static byte[] copiContextToOctetSeq (COPIServiceContext ctx) throws InvalidTypeForEncoding{
//         String context_data=null;
    
//         if(ctx==null) return null;
//         CDMW_LOG_FUNCTION(CCMLoggers.GetCcmCopiLogger(),"");
    
//         // serialized format = origin_id '<copictxsep>' target_id '<copictxsep>' <customSlotItemSeq_Str> (without spaces)
//         // customSlotItemSeq_Str = (CustomSlotItem.identifier'<customslotseqsep>'CustomSlotItem.content['<customslotitemseqsep>'])*
//         String origin_id = ctx.origin_id!=null ? new String(ctx.origin_id):"",
//                 target_id = ctx.target_id!=null ? new String(ctx.target_id):"",
//                 customSlotItemSeq_Str = ctx.slot_info!=null ? customSlotItemSeqToString(ctx.slot_info):"";
        
//         context_data = origin_id + copiCtxSep
//         + target_id + copiCtxSep
//         + customSlotItemSeq_Str;
        
//         CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmCopiLogger());
            
//         return stringToBytes(context_data);
//     }
        
//     /**
//      * decodes a byte array to build a COPIServiceContext
//      * @param context_data
//      * @return
//      * @throws FormatMismatch
//      * @throws BadParameterException
//      */
//     public static COPIServiceContext octetSeqToCOPIContext (byte[] context_data)
//                         throws FormatMismatch, BadParameterException{
    
//         if(context_data==null) return null;
//         CDMW_LOG_FUNCTION(CCMLoggers.GetCcmCopiLogger(),"");
        
//         return stringToCOPIContext(bytesToString(context_data));
    
//     }
        
//     /**
//      * constructs a COPIServiceContext starting from a string
//      * @param context_data
//      * @return
//      * @throws BadParameterException
//      * @throws FormatMismatch
//      */
//     public static COPIServiceContext stringToCOPIContext(String context_data)
//                         throws BadParameterException, FormatMismatch{
//         COPIServiceContext ctx = null;
//         CDMW_LOG_FUNCTION(CCMLoggers.GetCcmCopiLogger(), "context_data = "+context_data);
        
//         String[] parts =context_data.split(copiCtxSep);

//         if(!context_data.contains(copiCtxSep) || parts.length > 3 ) {
//             CDMW_WARN(CCMLoggers.GetCcmCopiLogger(), "Invalid Service Context string format: "+context_data);
//             BadParameterException ex = new BadParameterException("Invalid Service Context string format");
//             throw ex;
//         }
        
//         byte [] origin_id = (parts.length > 0)? stringToBytes(parts[0]):null;
//         byte [] target_id = (parts.length > 1)? stringToBytes(parts[1]):null;
//         CustomSlotItem[] slots = (parts.length > 2)?stringToCustomSlotItemSeq(parts[2]):null;
    
//         ctx = new COPIServiceContext(origin_id, target_id, slots);

//         CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmCopiLogger());

//         return ctx;
//     }

//     /**
//      * stringify a CustomSlotItem sequence
//      * @param slots
//      * @return
//      * @throws InvalidTypeForEncoding
//      */
//     public static String customSlotItemSeqToString (CustomSlotItem[] slots) throws InvalidTypeForEncoding{
//         CDMW_LOG_FUNCTION(CCMLoggers.GetCcmCopiLogger(), "");

//         String str = "";
//             for (int i = 0; i < slots.length; i++) {
//                 String s = new String(slots[i].identifier);
//                 CDMW_DEBUG(CCMLoggers.GetCcmCopiLogger(), "slot id: "+s);
//                 str += s;
//                 str += itemSep;
//                 s = bytesToString(COPIUtils.getCodec().encode(slots[i].content));
//                 CDMW_DEBUG(CCMLoggers.GetCcmCopiLogger(),"slot content: "+s);
//                 str += s;
//                 str += (i==slots.length-1?"":itemSeqSep);
//             }

//         CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmCopiLogger(), str);
//         return str;
//     }

//     /**
//      * constructs a sequence of CustomSlotItem starting from a string
//      * @param str
//      * @return
//      * @throws FormatMismatch
//      */
//     public static CustomSlotItem[] stringToCustomSlotItemSeq (String str) throws FormatMismatch{
//         CustomSlotItem[] slots = null;
//         CDMW_LOG_FUNCTION(CCMLoggers.GetCcmCopiLogger(), "str= "+str);

//         String[] parts = str.split(itemSeqSep);
//         slots = new CustomSlotItem[parts.length];

//         for(int i = 0; i < parts.length; i++){
//             String part = parts[i];
//             String[] sub_parts = part.split(itemSep);

//             if(!part.contains(itemSep) || sub_parts.length > 2){
//                 CDMW_WARN(CCMLoggers.GetCcmCopiLogger(), "Invalid Custom Slots string format of the Service Context : "
//                         +str +". invalid slots won't be part of the Service Context");
//                 continue;
//             }
//             String identifier = sub_parts.length>=1 ? sub_parts[0]:null;
//             CDMW_DEBUG(CCMLoggers.GetCcmCopiLogger()," slot id = "+ identifier);
//             Any content = sub_parts.length==2 ? COPIUtils.getCodec().decode(stringToBytes(sub_parts[1])):null;
//             CDMW_DEBUG(CCMLoggers.GetCcmCopiLogger(), "slot content = "+ (sub_parts.length==2 ?sub_parts[1]:null));
//             CustomSlotItem slot =new CustomSlotItem(identifier, content);
//             slots[i] = slot;
//         }

//         CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmCopiLogger());
//         return slots;
//     }

    /**
     * decodes a COPIServiceContext from a ServiceContext
     * @param svc_ctx
     * @return
     * @throws FormatMismatch
     * @throws BadParameterException
     */
    public static COPIServiceContext serviceToCopiContext(ServiceContext svc_ctx) 
                    throws FormatMismatch, BadParameterException {
        COPIServiceContext copi_ctx = null;
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmCopiLogger(),"");
        
        Assert.check(svc_ctx.context_id == COPIServiceInit.CDMW_COPI_SERVICE_CONTEXT_ID);
//         copi_ctx = octetSeqToCOPIContext(svc_ctx.context_data);
//         CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmCopiLogger());
        
//         return copi_ctx;

        Any any = COPIUtils.getCodec().decode(svc_ctx.context_data);
        CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmCopiLogger());
        return COPIServiceContextHelper.extract(any);

    }

    /**
     * encodes a COPIServiceContext in ServiceContext
     * @param copi_ctx
     * @return
     * @throws InvalidTypeForEncoding
     */
    public static ServiceContext copiToServiceContext(
            COPIServiceContext copi_ctx) throws InvalidTypeForEncoding {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmCopiLogger(),"");
        
//         return  new ServiceContext(COPIServiceInit.CDMW_COPI_SERVICE_CONTEXT_ID, // TODO: using the standard service ID
//                 COPIUtils.copiContextToOctetSeq(copi_ctx));

        Any any = COPIUtils.createAny();
        COPIServiceContextHelper.insert(any, copi_ctx);

        CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmCopiLogger());
        return  new ServiceContext(COPIServiceInit.CDMW_COPI_SERVICE_CONTEXT_ID, // TODO: using the standard service ID 
                                   COPIUtils.getCodec().encode(any));
    }
    
    
    

    public static Codec getCodec() {
        return codec;
    }

    public static void setCodec(Codec codec) {
        COPIUtils.codec = codec;
    }


    public static Any createAny(){
        return orb.create_any();
    }

    public static int get_ARG_MODE(ParameterMode mode) {

        switch(mode.value()){
        case ParameterMode._PARAM_IN : return ARG_IN.value;
        case ParameterMode._PARAM_OUT : return ARG_OUT.value;
        case ParameterMode._PARAM_INOUT : return ARG_INOUT.value;
        }
       throw new org.omg.CORBA.BAD_PARAM("Unsupported ParameterMode "+mode.value(),
               ExceptionMinorCodes.BAD_PARAM, CompletionStatus.COMPLETED_NO); 
    }

    /**
     * Copies the given array into a new array without a given element 
     * @param <T> the array type
     * @param source the array to copy
     * @param index the element's index to exclude
     * @return a copy of the source array without the excluded element 
     */
    public static <T> T[] removeArrayElement(T[] source, int index){
        Class type = source.getClass().getComponentType();
        T[] target = (T[])Array.newInstance(type, source.length-1);
        if(index>0) {
            System.arraycopy(source, 0, target, 0, index);
        }
        System.arraycopy(source, index+1, target, index, source.length-index-1);
        return target; 
    }
}
