/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import java.util.ArrayList;

import org.omg.CORBA.Any;
import org.omg.CORBA.TypeCode;
import org.omg.Dynamic.Parameter;


/**
 * we may put common code of ContainerStubRequest and ContainerServantRequest classes here
 */
abstract public class ContainerRequest {
    
    protected ArrayList<TypeCode> exceptionsTypeCodes = new ArrayList<TypeCode>();
    
    /**
     * implemented by generated code (the typed request object) to return the operation name
     * @return the operation name
     */
    public abstract String operation();
    
    /**
     * checks the READ-WRITE-ability of the request; whether the COPIs can modify the request parameters or not
     * @return true if the request is RW 
     */
    public boolean isReadWrite(){
       return COPIServiceInit.isCOPIRequestsReadWrite();
    }
    
    public Any createAny(){
        return COPIUtils.createAny();
    }
    
    protected Any get_arg (Parameter[] arguments, int index) {
        Any param = null;
        
        if(index > arguments.length) return null; //TODO : throw an exception
        
        return arguments[index].argument;
    }
    
    /**
     * adds a given exception Type Code. This TC is used to build a DII request
     * @param tc the exception type code
     */
    public void add_exception(TypeCode tc){
        this.exceptionsTypeCodes.add(tc);
    }
    
}
