/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import static cdmw.logging.FunctionLogger.*;
import static cdmw.logging.Logger.*;

import org.omg.CORBA.Any;
import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.ORB;
import org.omg.CORBA.OctetSeqHelper;
import org.omg.CORBA.ParameterMode;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.SystemExceptionHelper;
import org.omg.CORBA.TCKind;
import org.omg.Components.ContainerPortableInterceptor.ContainerServantRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.Components.ContainerPortableInterceptor.ServantContainerInterceptor;
import org.omg.Dynamic.Parameter;
import org.omg.PortableInterceptor.Current;
import org.omg.PortableInterceptor.CurrentHelper;
import org.omg.PortableInterceptor.InvalidSlot;

import cdmw.ccm.cif.Context;

import com.thalesgroup.CdmwCcmContainer.CCM2ContextPackage.InvalidName;
import com.thalesgroup.CdmwCcmCopi.COPICurrent;
import com.thalesgroup.CdmwCcmCopi.COPICurrentHelper;
import com.thalesgroup.CdmwCcmCopi.RegisteredCOPICollection;

/**
 *
 */
abstract public class ContainerServantRequest extends ContainerRequest{

    private ContainerServantRequestInfo request_info;

    /**
     * Logger for logging
     */
    protected cdmw.logging.Logger logger;

    /**
     * used to set the slots
     */
    protected Current piCurrent;

    /**
     * used to retrieve the slots ids
     */
    protected COPICurrent copiCurrent;
    
    /**
     * 
     */
    public BooleanHolder proceed_call;
    
    private ServantContainerInterceptorIterator servantCOPIIterator;

 

    public ContainerServantRequest(String facet_name, String component_id,
            org.omg.CORBA.Object target_object, Context ctx) {
        CDMW_LOG_FUNCTION(cdmw.ccm.common.CCMLoggers.GetCcmCopiLogger(), ""); 
        this.logger = cdmw.ccm.common.CCMLoggers.GetCcmCopiLogger();
        this.proceed_call = new BooleanHolder(true);

        try {
            this.piCurrent = CurrentHelper.narrow(ctx.get_ccm_context()
                    .resolve_initial_references("PICurrent"));
            this.copiCurrent = COPICurrentHelper
                    .narrow(ctx.get_ccm_context().resolve_initial_references(
                            COPICurrentImpl.COPI_CURRENT_ID));
        } catch (InvalidName e) {
            // TODO: manage failing resolve_init_ref
            e.printStackTrace();
        }
        
        byte[] origin_id = new byte[0],
                target_id = (component_id!=null?component_id.getBytes():new byte[0]);
        
        this.request_info= new ContainerServantRequestInfoImpl(facet_name, origin_id, 
                target_id, this.operation() );
        request_info.target((org.omg.Components.EnterpriseComponent)target_object);
        CDMW_LOG_FUNCTION_RETURN(logger); 
    }
    
    public void add_in_args (Any in_arg){
        Parameter in = new Parameter (in_arg, ParameterMode.PARAM_IN);
        Parameter[] args = this.request_info.arguments();
        if(args != null){
            Parameter[] new_args = new Parameter[args.length+1];
            System.arraycopy(args, 0, new_args, 0, args.length);
            new_args[args.length] = in;
            args = new_args;
            
        } else {
            args = new Parameter[]{in};
        }
        this.request_info.arguments(args);
    }
    
    public void add_out_args (Any out_arg){
        Parameter out = new Parameter(out_arg, ParameterMode.PARAM_OUT);
        Parameter[] args = this.request_info.arguments();
        if(args != null){
            Parameter[] new_args = new Parameter[args.length+1];
            System.arraycopy(args, 0, new_args, 0, args.length);
            new_args[args.length] = out;
            args = new_args;
            
        } else {
            args = new Parameter[]{out};
        }
        this.request_info.arguments(args);
    }
    
    public void add_inout_args (Any inout_arg){
        Parameter inout = new Parameter(inout_arg, ParameterMode.PARAM_INOUT);
        Parameter[] args = this.request_info.arguments();
        if(args != null){
            Parameter[] new_args = new Parameter[args.length+1];
            System.arraycopy(args, 0, new_args, 0, args.length);
            new_args[args.length] = inout;
            args = new_args;
            
        } else {
            args = new Parameter[]{inout};
        }
        this.request_info.arguments(args);
    }

    public Any get_arg (int index){
        return get_arg(this.request_info.arguments(), index);        
    }
    
    public void add_result(Any res){
        this.request_info.result(res);
    }
    
    public void add_exception(Any ex){
        this.request_info.the_exception(ex);
    }
    
    public Any the_exception (){
        return this.request_info.the_exception();
     }
    
    public void add_SystemException(SystemException se){
        Any any = COPIUtils.createAny();
        SystemExceptionHelper.insert(any, se);
        this.request_info.the_exception(any);
    }
    
    public SystemException the_systemException(){
        Any any = this.request_info.the_exception();
        return SystemExceptionHelper.extract(any);
    }
     
     public Any result (){
        return this.request_info.result(); 
     }
    
    /**
     * This is called by the typed facet implementation before invoking the
     * operation on the user implementation of the facet
     * 
     */

    public void preInvoke() {
        preInvokeExtendedCOPI();
    }
    
    
    protected void preInvokeExtendedCOPI () {
        CDMW_LOG_FUNCTION(logger, "");
        proceed_call.value = true;
        
        // getting the origin_id from the thread context 
        try {
            Any Any_origin_id = piCurrent.get_slot(copiCurrent.get_slot_id(COPIServiceInit.Origin_ID_SlotNAME));
            byte[] origin_id = (Any_origin_id != null && !Any_origin_id.type().kind().equals(TCKind.tk_null) ? OctetSeqHelper.extract(Any_origin_id):null);            
            ((ContainerRequestInfoImpl)request_info).setOrigin_id(origin_id);
        } catch (InvalidSlot e) {
            e.printStackTrace();
        }

        RegisteredCOPICollection reg = copiCurrent.get_registered_copi_collection();
        
        ServantContainerInterceptor[] interceptors = reg.getServantInterceptorByIPOrderedByPriority(
                        new IntegrationPoint(this.request_info.name(), this.operation()));
        
        CDMW_DEBUG(logger, "Iterating over "+interceptors.length+" SERVANT interceptors");
        
        servantCOPIIterator = new ServantContainerInterceptorIterator (interceptors, 
                copiCurrent.get_registered_copi_collection());
        
        servantCOPIIterator.servant_receive_request(request_info, proceed_call);
        CDMW_LOG_FUNCTION_RETURN(logger);
    }


    public void postInvoke() {
        
        postInvokeBasicCOPI();
        postInvokeExtendedCOPI();

    }

    protected void postInvokeBasicCOPI(){
        // nothing to do for now
    }
    
    protected void postInvokeExtendedCOPI () {
        CDMW_LOG_FUNCTION(logger, ""); 
        if(!proceed_call.value){
            servantCOPIIterator.servant_send_other(request_info);
            return;
        }
        
        // checking the request outcome
        if(request_info.the_exception() == null) servantCOPIIterator.servant_send_reply(request_info, proceed_call);
            else servantCOPIIterator.servant_send_exception(request_info, proceed_call); 
        CDMW_LOG_FUNCTION_RETURN(logger);
    }
    
}
