/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION;
import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION_RETURN;
import static cdmw.logging.Logger.CDMW_DEBUG;
import static cdmw.logging.Logger.CDMW_WARN;

import org.omg.CORBA.Any;
import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.ORB;
import org.omg.CORBA.OctetSeqHelper;
import org.omg.CORBA.ParameterMode;
import org.omg.CORBA.Request;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.SystemExceptionHelper;
import org.omg.CORBA.TypeCode;

import org.omg.CORBA.TCKind;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItem;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItemSeqHelper;
import cdmw.ccm.qosenablers.requestfollowup.ExtendedNDCExecutorImpl;

import org.omg.Components.BadComponentReference;
import org.omg.Components.ContainerPortableInterceptor.ContainerStubRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptor;
import org.omg.Dynamic.Parameter;
import org.omg.PortableInterceptor.Current;
import org.omg.PortableInterceptor.CurrentHelper;
import org.omg.PortableInterceptor.InvalidSlot;

import cdmw.ccm.cif.CCMUtils;
import cdmw.ccm.cif.Context;
import cdmw.common.Assert;

import com.sun.org.apache.xerces.internal.parsers.IntegratedParserConfiguration;
import com.thalesgroup.CdmwCcmContainer.CCM2ContextPackage.InvalidName;
import com.thalesgroup.CdmwCcmCopi.COPICurrent;
import com.thalesgroup.CdmwCcmCopi.COPICurrentHelper;
import com.thalesgroup.CdmwCcmCopi.RegisteredCOPICollection;

/**
 * represents an outcoming request
 */
public abstract class ContainerStubRequest extends ContainerRequest{
    
    private ContainerStubRequestInfo request_info;

    /**
     * Logger for logging
     */
    protected cdmw.logging.Logger logger;

//     /**
//      * the singleton orb
//      */
//     protected org.omg.CORBA.ORB orb = ORB.init();

    /**
     * used to set the slots
     */
    protected Current piCurrent;

    /**
     * used to retrieve the slots ids
     */
    protected COPICurrent copiCurrent;
    
    /**
     * 
     */
    public BooleanHolder proceed_call;
    
    
    private StubContainerInterceptorIterator stubCOPIIterator;
    
    private Context context;
    
    
    public ContainerStubRequest(String receptacle_name, String component_id,
            org.omg.CORBA.Object target, Context ctx) {
        CDMW_LOG_FUNCTION(cdmw.ccm.common.CCMLoggers.GetCcmCopiLogger(), "");
        this.logger = cdmw.ccm.common.CCMLoggers.GetCcmCopiLogger();
        this.context = ctx;
        
        this.proceed_call = new BooleanHolder(true);

        try {
            this.piCurrent = CurrentHelper.narrow(ctx.get_ccm_context()
                    .resolve_initial_references("PICurrent"));
            this.copiCurrent = COPICurrentHelper
                    .narrow(ctx.get_ccm_context().resolve_initial_references(
                            COPICurrentImpl.COPI_CURRENT_ID));
        } catch (InvalidName e) {
            // TODO: manage failing resolve_init_ref
            e.printStackTrace();
        }
        
        //setting the target_id
        String s_target_id = ""; 
        try{
        String oid = ctx.extract_oid_str_from_ref(target);
        java.util.Vector names = CCMUtils.readOid(oid);
        Assert.check(!names.isEmpty());
        s_target_id = ((String) names.elementAt(0));
        }
        catch(BadComponentReference bcr){
            CDMW_DEBUG(logger, "Failed to read the target oid: "+bcr.reason, bcr);
            
        } catch (Throwable e) {
            CDMW_DEBUG(logger, "Failed to read the target oid: "+e.getMessage(), e);
        }
        byte[] origin_id = (component_id!=null ? component_id.getBytes():new byte[0]),
                target_id = (s_target_id!=null ? s_target_id.getBytes():new byte[0]);
        
        this.request_info = new ContainerStubRequestInfoImpl(receptacle_name, origin_id, 
                target_id, operation() );
        this.request_info.target(target);
        this.request_info.result(createAny());

    }
    
    public abstract String operation();
    
    
    public void add_in_args (Any in_arg){
        Parameter in = new Parameter (in_arg, ParameterMode.PARAM_IN);
        Parameter[] args = this.request_info.arguments();
        if(args != null){
            Parameter[] new_args = new Parameter[args.length+1];
            System.arraycopy(args, 0, new_args, 0, args.length);
            new_args[args.length] = in;
            args = new_args;
            
        } else {
            args = new Parameter[]{in};
        }
        this.request_info.arguments(args);
    }
    
    public Any get_arg (int index){
        return get_arg(this.request_info.arguments(), index);        
    }
    
    public void add_out_args (Any out_arg){
        Parameter out = new Parameter(out_arg, ParameterMode.PARAM_OUT);
        Parameter[] args = this.request_info.arguments();
        if(args != null){
            Parameter[] new_args = new Parameter[args.length+1];
            System.arraycopy(args, 0, new_args, 0, args.length);
            new_args[args.length] = out;
            args = new_args;
            
        } else {
            args = new Parameter[]{out};
        }
        this.request_info.arguments(args);
    }
    
    public void add_inout_args (Any inout_arg){
        Parameter inout = new Parameter(inout_arg, ParameterMode.PARAM_INOUT);
        Parameter[] args = this.request_info.arguments();
        if(args != null){
            Parameter[] new_args = new Parameter[args.length+1];
            System.arraycopy(args, 0, new_args, 0, args.length);
            new_args[args.length] = inout;
            args = new_args;
            
        } else {
            args = new Parameter[]{inout};
        }
        this.request_info.arguments(args);
    }
    
    public void add_result(Any res){
        this.request_info.result(res);
    }
    
    public void add_exception(Any ex){
        this.request_info.the_exception(ex);
    }
    
    
    public Any the_exception (){
       return this.request_info.the_exception();
    }
 
    public void add_SystemException(SystemException se){
        Any any = COPIUtils.createAny();
        SystemExceptionHelper.insert(any, se);
        this.request_info.the_exception(any);
    }
    
    public SystemException the_systemException(){
        SystemException e = null;
        Any any = this.request_info.the_exception();
        // Bug Jacorb: SystemExceptionHelper does not extract MARSHALL exceptions
        if (any.type().equal(org.omg.CORBA.MARSHALHelper.type())) { 
            e = org.omg.CORBA.MARSHALHelper.extract(any); 
         } else {
             e = SystemExceptionHelper.extract(any);
         }
         return e;
    }
    
    public Any result (){
       return this.request_info.result(); 
    }
    
    public void invoke() {
        CDMW_LOG_FUNCTION(logger, "");
        Request r = null;
        
        // return if the call has not to be done or an exception has been raised 
        if (!this.proceed_call.value ||  this.request_info.the_exception() != null )
            return;
        
        CDMW_DEBUG(logger, "Performing a DII invocation of operation "+ this.request_info.operation());        
        try {
            r = this.context.create_request(
                    this.request_info.operation(), this.request_info.arguments(), this.request_info
                            .target(), this.request_info.result() );
            org.omg.CORBA.ExceptionList exList = r.exceptions();
            for(TypeCode tc : this.exceptionsTypeCodes)
                exList.add(tc);
            
            r.invoke();
       } catch (org.omg.Components.CreateFailure cf) {
            CDMW_DEBUG(logger, "Failed to create the CORBA request"
                    + this.operation() + ": " + cf.getMessage());
            Any any = COPIUtils.createAny();
            org.omg.Components.CreateFailureHelper.insert(any, cf);
            this.request_info.the_exception(any);
            return;
        }

        Exception ex = r.env().exception();
        if (ex != null) {
            if (ex instanceof org.omg.CORBA.SystemException) add_SystemException((org.omg.CORBA.SystemException)ex);
            else this.request_info.the_exception(((org.omg.CORBA.UnknownUserException) ex).except);
        }

        this.request_info.result(r.return_value());
        CDMW_LOG_FUNCTION_RETURN(logger);
    }
    
    
    /**
     * This is called by the typed receptacle objects before invoking the
     * operation on the remote facet
     * 
     * @param arguments
     * @param operation
     * @param result
     * @param the_exception
     */

    public void preInvoke() {

        preInvokeExtendedCOPI();
        preInvokeBasicCOPI();
        
    }
    
    protected void preInvokeBasicCOPI(){
        CDMW_LOG_FUNCTION(logger, "");
        try {
            
            // setting the origin_id
            Any any = createAny();
            OctetSeqHelper.insert(any, this.request_info.origin_id());
            piCurrent.set_slot(copiCurrent
                    .get_slot_id(COPIServiceInit.Origin_ID_SlotNAME), any);
            
            any = createAny();
            OctetSeqHelper.insert(any, this.request_info.target_id());
            piCurrent.set_slot(copiCurrent
                    .get_slot_id(COPIServiceInit.Target_ID_SlotNAME), any);
            
            // setting the origin feature name
            any = createAny();
            any.insert_string(this.request_info.name());
            piCurrent.set_slot(copiCurrent
                    .get_slot_id(COPIServiceInit.OriginFeatureName_SlotNAME),
                    any);

        } catch (InvalidSlot is) {
            is.printStackTrace();

        }
        CDMW_LOG_FUNCTION_RETURN(logger);
    }
    
    
    protected void preInvokeExtendedCOPI () {
        CDMW_LOG_FUNCTION(logger, "");
        proceed_call.value = true;

        try {
            // Copy content of slot identified by TargetCustomSlotItem_Seq_SlotID
            // to slot identified by OriginCustomSlotItem_Seq_SlotID
            // for chaining purpose
            Any any = 
                piCurrent.get_slot(
                    copiCurrent.get_slot_id(
                        COPIServiceInit.TargetCustomSlotItemSeq_SlotNAME));

            if( any != null && ! any.type().kind().equals(TCKind.tk_null)) {
                CDMW_DEBUG(logger, "Some information to transfer");
                CustomSlotItem[] csi = CustomSlotItemSeqHelper.extract(any);
                    
                // Search the CustomSmotItem corresponding to the RFU Service
                for( int idx = 0 ; idx <  csi.length ; idx++ ) {
                    if(csi[idx].identifier.equals(ExtendedNDCExecutorImpl.RFU_SERVICE_NAME)) {
                        CDMW_DEBUG(logger, "RFU Found");
                    }
                }
            }

            piCurrent.set_slot(
                copiCurrent.get_slot_id(
                    COPIServiceInit.OriginCustomSlotItemSeq_SlotNAME),
                any);
        } catch (InvalidSlot is) {
            is.printStackTrace();
        }

        try {
            // Copy content of slot identified by TargetCustomSlotItem_Seq_SlotID
            // to slot identified by OriginCustomSlotItem_Seq_SlotID
            // for chaining purpose
            Any any = 
                piCurrent.get_slot(
                    copiCurrent.get_slot_id(
                        COPIServiceInit.TargetCustomSlotItemSeq_SlotNAME));

            if( any != null && ! any.type().kind().equals(TCKind.tk_null)) {
                CDMW_DEBUG(logger, "Some information to transfer");
                CustomSlotItem[] csi = CustomSlotItemSeqHelper.extract(any);
                    
                // Search the CustomSmotItem corresponding to the RFU Service
                for( int idx = 0 ; idx <  csi.length ; idx++ ) {
                    if(csi[idx].identifier.equals(ExtendedNDCExecutorImpl.RFU_SERVICE_NAME)) {
                        CDMW_DEBUG(logger, "RFU Found");
                    }
                }
            }

            piCurrent.set_slot(
                copiCurrent.get_slot_id(
                    COPIServiceInit.OriginCustomSlotItemSeq_SlotNAME),
                any);
        } catch (InvalidSlot is) {
            is.printStackTrace();
        }

        RegisteredCOPICollection reg = copiCurrent.get_registered_copi_collection();
        
        StubContainerInterceptor[] interceptors = reg.getStubInterceptorByIPOrderedByPriority(
                        new IntegrationPoint(this.request_info.name(), this.operation()));
        
        CDMW_DEBUG(logger, "Iterating over "+interceptors.length+" STUB interceptors");
        
        stubCOPIIterator = new StubContainerInterceptorIterator (interceptors, 
                copiCurrent.get_registered_copi_collection());
        
        stubCOPIIterator.stub_send_request(request_info, proceed_call);
        CDMW_LOG_FUNCTION_RETURN(logger);
    }

    /**
     * This is called by the typed receptacle objects after invoking the
     * operation on the remote facet
     * 
     * @param arguments
     * @param operation
     * @param result
     * @param the_exception
     */
    public void postInvoke() {
        
        postInvokeBasicCOPI();
        postInvokeExtendedCOPI();

    }

    protected void postInvokeBasicCOPI(){
        // may be unsetting the slots ?!
    }
    
    protected void postInvokeExtendedCOPI () {
        CDMW_LOG_FUNCTION(logger, "");
        if(!proceed_call.value) {
            stubCOPIIterator.stub_receive_other(request_info);
            return;
        }
            
        // checking the request outcome
        if(request_info.the_exception() == null) stubCOPIIterator.stub_receive_reply(request_info, proceed_call);
            else stubCOPIIterator.stub_receive_exception(request_info, proceed_call); 
        CDMW_LOG_FUNCTION_RETURN(logger);
    }

 
 
}
