/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import static cdmw.logging.FunctionLogger.*;
import static cdmw.logging.Logger.*;

import org.omg.CORBA.Any;
import org.omg.CORBA.Object;
import org.omg.CosEventComm.Disconnected;
import org.omg.CosEventComm.DisconnectedHelper;

import cdmw.ccm.cif.Context;
import cdmw.orbsupport.ExceptionMinorCodes;

/**
 * 
 *
 */
public class PushContainerStubRequest extends ContainerStubRequest {

    private String sourceEventPortName;
    private org.omg.CORBA.Object proxyConsumer;
    
    
    public PushContainerStubRequest(String source_name,
            String component_id, Object proxyConsumer, Context ctx) {
        super(source_name, component_id, proxyConsumer, ctx);
        CDMW_LOG_FUNCTION(logger, "");
        this.sourceEventPortName = source_name;
        this.proxyConsumer = proxyConsumer;
        CDMW_LOG_FUNCTION_RETURN(logger);
    }

    public String operation() {
        return "push";
    }
    
    public void push(Any event){
        CDMW_LOG_FUNCTION(logger, "");
        Exception thrownEx;
        
        if(this.isReadWrite()){
            Any arg = this.createAny();
            arg.insert_any(event);
            this.add_in_args(arg);
            this.add_exception(DisconnectedHelper.type());
            this.preInvoke();
            this.invoke();
            this.postInvoke();            
        } else {
            try {
                if((this.proceed_call.value &&  this.the_exception() == null))
                    ((org.omg.CosEventComm.PushConsumer)this.proxyConsumer).push(event);
                
           } catch (org.omg.CosEventComm.Disconnected d) {
               d.printStackTrace();
              this.add_exception_Disconnected(d);
              
           } catch(org.omg.CORBA.SystemException se){
               se.printStackTrace();
               this.add_SystemException(se);  
           }
        }
               
        throw_exception();
               
        CDMW_LOG_FUNCTION_RETURN(logger);
        
    }
    
    private void add_exception_Disconnected(Disconnected d) {
        Any d_any = this.createAny();
        DisconnectedHelper.insert(d_any, d);
        this.add_exception(d_any);
    }

    
    public void throw_exception() throws org.omg.CORBA.SystemException{
        
        if(this.the_exception()==null) return;
        
        org.omg.CORBA.Any ex = this.the_exception();
        if(ex.type().equal(org.omg.CosEventComm.DisconnectedHelper.type())){
            CDMW_ERROR(logger, "Cannot send event with "+ this.sourceEventPortName +" port since it's not connected");
            org.omg.CORBA.INTERNAL e = new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, e);
            throw e;
        } else {
            CDMW_ERROR(logger, "Failed to send event with "+ this.sourceEventPortName +" port");
            org.omg.CORBA.SystemException e = this.the_systemException();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, e);
            throw e;
        }
            
    }

}
