/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;



public final class DefaultCCMObjectStates
{
    /**
     * INITIAL CCMObject's state.
     */
    public static CCMObjectState CCM_OBJECT_STATE_INITIAL = 
        new CCMObjectStateInitial();
    
    /**
     * FULLY_ACTIVATED CCMObject's state.
     */
    public static CCMObjectState CCM_OBJECT_STATE_FULLY_ACTIVATED = 
        new CCMObjectStateFullyActivated();
    
    /**
     * REMOVED CCMObject's state.
     */
    public static CCMObjectState CCM_OBJECT_STATE_REMOVED = 
        new CCMObjectStateRemoved();
}



//
// Implementation of INITIAL CCMObject's state
//
final class CCMObjectStateInitial
    extends CCMObjectState
{
    /*
     * In case of configuration_complete:
     *   - call ccm_activate
     *   - change state to FULLY_ACTIVATED
     */
    public CCMObjectState configuration_complete(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        // call ccm_activate
        ccmObject.ccm_activate();
        
        // change state to FULLY_ACTIVATED
        return DefaultCCMObjectStates.CCM_OBJECT_STATE_FULLY_ACTIVATED;
    }
    
    /*
     * In case of remove:
     *   - call ccm_remove
     *   - change state to REMOVED
     */
    public CCMObjectState remove(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        // call ccm_remove
        ccmObject.ccm_remove();
    
        // change state to REMOVED
        return DefaultCCMObjectStates.CCM_OBJECT_STATE_REMOVED;
    }
        
    /*
     * In case of activate:
     *   - failed assertion, since this transition 
     *     doesn't exist for default case
     */
    public CCMObjectState activate(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        System.err.println(
            "'activate' transition not authorized from CCM_OBJECT_STATE_INITIAL");
        cdmw.common.Assert.check(false);
    
        // for compiler
        return null;
    }
    
    /*
     * In case of deactivate:
     *   - failed assertion, since this transition 
     *     doesn't exist for default case
     */
    public CCMObjectState deactivate(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        System.err.println(
            "'deactivate' transition not authorized from CCM_OBJECT_STATE_INITIAL");
        cdmw.common.Assert.check(false);
    
        // for compiler
        return null;
    }
}


//
// Implementation of FULLY_ACTIVATED CCMObject's state
//
final class CCMObjectStateFullyActivated
    extends CCMObjectState
{
    /*
     * In case of configuration_complete:
     *   - already configured; ignore
     */
    public CCMObjectState configuration_complete(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        // ignore: just return same state
        return this;
    }
    
    /*
     * In case of remove:
     *   - call ccm_passivate
     *   - call ccm_remove
     *   - change state to REMOVED
     */
    public CCMObjectState remove(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        // call ccm_passivate
        ccmObject.ccm_passivate();
        
        // call ccm_remove
        ccmObject.ccm_remove();
    
        // change state to REMOVED
        return DefaultCCMObjectStates.CCM_OBJECT_STATE_REMOVED;
    }
        
    /*
     * In case of activate:
     *   - failed assertion, since this transition 
     *     doesn't exist for default case
     */
    public CCMObjectState activate(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        System.err.println(
            "'activate' transition not authorized from CCM_OBJECT_STATE_FULLY_ACTIVATED");
        cdmw.common.Assert.check(false);
    
        // for compiler
        return null;
    }
    
    /*
     * In case of deactivate:
     *   - failed assertion, since this transition 
     *     doesn't exist for default case
     */
    public CCMObjectState deactivate(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        System.err.println(
            "'deactivate' transition not authorized from CCM_OBJECT_STATE_FULLY_ACTIVATED");
        cdmw.common.Assert.check(false);
    
        // for compiler
        return null;
    }
}


//
// Implementation of REMOVED CCMObject's state
//
final class CCMObjectStateRemoved
    extends CCMObjectState
{
    /*
     * In case of configuration_complete:
     *   - failed assertion, since component is removed
     */
    public CCMObjectState configuration_complete(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        System.err.println(
            "'configuration_complete' transition not authorized from CCM_OBJECT_STATE_REMOVED");
        cdmw.common.Assert.check(false);
    
        // for compiler
        return null;
    }
    
    /*
     * In case of remove:
     *   - already removed; ignore
     */
    public CCMObjectState remove(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        // ignore: just return same state
        return this;
    }
        
    /*
     * In case of activate:
     *   - failed assertion, since this transition 
     *     doesn't exist for default case
     */
    public CCMObjectState activate(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        System.err.println(
            "'activate' transition not authorized from CCM_OBJECT_STATE_REMOVED");
        cdmw.common.Assert.check(false);
    
        // for compiler
        return null;
    }
    
    /*
     * In case of deactivate:
     *   - failed assertion, since this transition 
     *     doesn't exist for default case
     */
    public CCMObjectState deactivate(CCMObjectImpl ccmObject)
        throws org.omg.Components.CCMException
    {
        System.err.println(
            "'deactivate' transition not authorized from CCM_OBJECT_STATE_REMOVED");
        cdmw.common.Assert.check(false);
    
        // for compiler
        return null;
    }
}


