/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.cif;

import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION;

import org.omg.Components.CCMException;
import org.omg.Components.Cookie;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.ServantContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.ServerContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptorRegistration;


import cdmw.ccm.common.CCMLoggers;

import com.thalesgroup.CdmwCcmContainer.CCM2Context;
import com.thalesgroup.CdmwCcmContainer.Session2Context;
import com.thalesgroup.CdmwCcmContainer.Session2ExtensionContext;
import com.thalesgroup.CdmwCcmContainer.Session2ExtensionContextHelper;



public class ExtensionContext extends SessionContext {
    

    public ExtensionContext(Session2ExtensionContext context) {
        super((Session2Context)context);
    }

    public ClientContainerInterceptorRegistration get_client_interceptor_registration() throws CCMException {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.getExtensionContext().get_client_interceptor_registration();
    }

    public ServantContainerInterceptorRegistration get_servant_interceptor_registration() throws CCMException {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.getExtensionContext().get_servant_interceptor_registration();
    }

    public ServerContainerInterceptorRegistration get_server_interceptor_registration() throws CCMException {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.getExtensionContext().get_server_interceptor_registration();
    }

    public StubContainerInterceptorRegistration get_stub_interceptor_registration() throws CCMException {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.getExtensionContext().get_stub_interceptor_registration();
    }

    public Cookie install_service_reference(String service_id, org.omg.CORBA.Object objref) throws CCMException {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.getExtensionContext().install_service_reference(service_id, objref);
    }

    public org.omg.CORBA.Object uninstall_service_reference(Cookie ck) throws CCMException {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.getExtensionContext().uninstall_service_reference(ck);
    }

    /**
     * returns the extension context
     */
    protected Session2ExtensionContext getExtensionContext()
        throws org.omg.CORBA.SystemException
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        CCM2Context context = this.get_ccm_context();
        Session2ExtensionContext extension_context = Session2ExtensionContextHelper.narrow(context);
        return extension_context;
    }    

}
