/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.cif;

import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION;
import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION_EXCEPTION;
import static cdmw.logging.Logger.CDMW_ERROR;
import static cdmw.logging.Logger.CDMW_WARN;

import org.omg.Components.CCMContext;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.ExtensionContextHelper;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.ServantContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.ServerContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptorRegistration;

import cdmw.orbsupport.ExceptionMinorCodes;

/**
 * 
 * This class is the base class of all the Extension components contexts
 * 
 */

public class ExtensionContextImpl extends CCMContextImpl implements
        org.omg.Components.ExtensionContext {

    /**
     * The container context
     */
    private org.omg.Components.ExtensionContext extension_context;

    /**
     * Logger for logging
     */
    private com.thalesgroup.CdmwLogging.Logger logger;
    
    
    /**
     * List of forbidden service identifiers
     * override the base CCMContextImpl.getUnavailableServices implementation
     * to return a different list and allows QoSEnablers to access to the
     * PICurrent
     */
    protected String[] getUnavailableServices(){
        return new String[]{"RootPOA", 
                            "POACurrent", 
                            "ORBPolicyManager",
                            "PolicyCurrent"};
    }
    

    public ExtensionContextImpl(CCMContext ctx,
            String componentId,
            com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory) {

        super(ctx, componentId, loggerFactory);

        // create logger with domain for ccmcif
        this.logger = loggerFactory
                .get_logger(cdmw.ccm.common.CCMLoggers.CCMCIF_LOG_DOMAIN);

        CDMW_LOG_FUNCTION(logger, "");

        try {
            this.extension_context = ExtensionContextHelper.narrow(ctx);
        } catch (org.omg.CORBA.BAD_PARAM e) {
            CDMW_ERROR(logger, "Context is not a ExtensionContext",
                    new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMWrongContextType,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    }

    // Specific Cookie implementation to ease Service search
    private class CookieImpl extends org.omg.Components.Cookie {
        public CookieImpl(String cookieValue) {
            this.cookieValue = cookieValue.getBytes();
        }

        public String getCookieString() {
            return new String(this.cookieValue);
        }
    }
    
    //
    // IDL:omg.org/Components/ExtensionContext/install_service_reference:1.0
    //
    public org.omg.Components.Cookie install_service_reference(
            String service_id, org.omg.CORBA.Object objref) throws CCMException {

        CDMW_LOG_FUNCTION(logger, "service_id=" + service_id);
        
        return this.extension_context.install_service_reference(service_id, objref);        
    }

    //
    // IDL:omg.org/Components/ExtensionContext/uninstall_service_reference:1.0
    //
    public org.omg.CORBA.Object uninstall_service_reference(
            org.omg.Components.Cookie ck) throws CCMException {
        CDMW_LOG_FUNCTION(logger, "");
        return this.extension_context.uninstall_service_reference(ck);
    }

    public ClientContainerInterceptorRegistration get_client_interceptor_registration()
            throws CCMException {
        CDMW_LOG_FUNCTION(logger, "");
        return this.extension_context.get_client_interceptor_registration();
    }

    public ServerContainerInterceptorRegistration get_server_interceptor_registration()
            throws CCMException {
        return this.extension_context.get_server_interceptor_registration();
    }

    public StubContainerInterceptorRegistration get_stub_interceptor_registration()
            throws CCMException {
        return this.extension_context.get_stub_interceptor_registration();
    }

    public ServantContainerInterceptorRegistration get_servant_interceptor_registration()
            throws CCMException {
        return this.extension_context.get_servant_interceptor_registration();
    }


}
