/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;

import org.omg.CORBA.OctetSeqHelper;

import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

//
// IDL:omg.org/Components/HomeConfiguration:1.0
//
public class PushConsumerImpl
    extends com.thalesgroup.CdmwCcmCif.PushConsumerPOA {

    /**
     * non cdmw consumer
     */
    private org.omg.Components.EventConsumerBase ccmConsumer;

    /**
     * LifeCycleObjectBase object used for delegation
     */
    private LifeCycleObjectBase lifecycleObjectBase;

    /**
     * constructor
     * .
     * @param ctx
     * @param ccmConsumer
     */
    public PushConsumerImpl(
        Context ctx,
        org.omg.Components.EventConsumerBase ccmConsumer) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.lifecycleObjectBase = new LifeCycleObjectBase(ctx);
        this.ccmConsumer = ccmConsumer;
    }

    //
    // IDL:omg.org/CosEventComm/PushConsumer/disconnect_push_consumer:1.0
    //
    public void disconnect_push_consumer() {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        // do nothing
    }

    //
    // IDL:omg.org/CosEventComm/PushConsumer/push:1.0
    //
    public void push(org.omg.CORBA.Any data) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
                           
        org.omg.Components.EventBase evt;
        
        try {
            if(OctetSeqHelper.type().equal(data.type())) {
                // data has been encoded
                cdmw.orbsupport.Codec codec = new cdmw.orbsupport.Codec();
                evt =
                    (org.omg.Components.EventBase) codec.decode(data);
            } else if(cdmw.ccm.ccmcopi.COPIServiceInit.isInitialized() &&
                      cdmw.ccm.ccmcopi.COPIServiceInit.isCOPIRequestsReadWrite()) {
                // data contains a tk which is not OctSeq, i.e. data has not been encoded using a codec,
                // hence we treat it as an ordinary any
                java.io.Serializable valueType =
                    org.omg.CORBA.ValueBaseHelper.extract(data);
                evt =
                    (org.omg.Components.EventBase) valueType;                                       
            } else {
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                        "PushConsumerImpl push got any wich was not produced be encoding while COPIs ReadWrite deactivated or COPIs not initilised",
                        new Throwable());
                org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMBadEventType,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;                
            }
            
        } catch (ClassCastException e) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                    "PushConsumerImpl push a non event type!",
                    new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMBadEventType,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }

        if (evt == null) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "PushConsumerImpl push nil event!",
                new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMBadEventType,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }

        try {
            this.ccmConsumer.push_event(evt);
        } catch (org.omg.Components.BadEventType bet) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "PushConsumerImpl push bad event type!",
                new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMBadEventType,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }

    }

    //
    // IDL:omg.org/CosLifeCycle/LifeCycleObject/copy:1.0
    //
    public org.omg.CosLifeCycle.LifeCycleObject copy(
        org.omg.CosLifeCycle.FactoryFinder there,
        org.omg.CosLifeCycle.NVP[] the_criteria)
        throws
            org.omg.CosLifeCycle.NoFactory,
            org.omg.CosLifeCycle.NotCopyable,
            org.omg.CosLifeCycle.InvalidCriteria,
            org.omg.CosLifeCycle.CannotMeetCriteria 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.lifecycleObjectBase.copy(there, the_criteria);
    }

    //
    // IDL:omg.org/CosLifeCycle/LifeCycleObject/move:1.0
    //
    public void move(
        org.omg.CosLifeCycle.FactoryFinder there,
        org.omg.CosLifeCycle.NVP[] the_criteria)
        throws
            org.omg.CosLifeCycle.NoFactory,
            org.omg.CosLifeCycle.NotMovable,
            org.omg.CosLifeCycle.InvalidCriteria,
            org.omg.CosLifeCycle.CannotMeetCriteria 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.lifecycleObjectBase.move(there, the_criteria);
    }

    //
    // IDL:omg.org/CosLifeCycle/LifeCycleObject/remove:1.0
    //
    public void remove() throws org.omg.CosLifeCycle.NotRemovable {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.lifecycleObjectBase.remove();
    }

}
