/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.container;

import org.omg.CORBA.Object;
import org.omg.Components.ConfigValuesHolder;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;

import com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor;
import com.thalesgroup.CdmwCcmContainer.HomeLifeCycleInterceptor;
import com.thalesgroup.CdmwCcmContainer.HomeLifeCycleInterceptorOperations;

/**
 * call operations on registered lifecycle interceptors before/after operations on home   
 */
public class HomeLifeCycleInterceptorIterator implements
        HomeLifeCycleInterceptorOperations {

    HomeLifeCycleInterceptor[] interceptors;
    
    public HomeLifeCycleInterceptorIterator(){
        interceptors = LifeCycleInterceptorRegistration.getInstance().listHomeLifeCycleInterceptor();
    }

    public void after_home_create(String homeId, Object homeRef) {
        
        for(HomeLifeCycleInterceptor copi: interceptors){
            copi.after_home_create(homeId, homeRef);
        }
    }

    public void after_home_remove(String homeId) {
        
        for(HomeLifeCycleInterceptor copi: interceptors){
            copi.after_home_remove(homeId);
        }
    }

    public void before_home_create(String homeId, ConfigValuesHolder values,
            String entryPoint) {
        
        for(HomeLifeCycleInterceptor copi: interceptors){
            copi.before_home_create(homeId, values, entryPoint);
        }

    }

    public void before_home_remove(String homeId) {

        for(HomeLifeCycleInterceptor copi: interceptors){
            copi.before_home_remove(homeId);
        }

    }


    public void destroy() {
    }


    public String name() {
        return "HomeLifecycleIterator";
    }


    public short priority() {
        return 0;
    }


    public IntegrationPoint registration_info() {
        return null;
    }


    public void set_slot_id(int arg0) {
    }

}
