/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.container;

import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION;
import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION_EXCEPTION;
import static cdmw.logging.Logger.CDMW_DEBUG;
import static cdmw.logging.Logger.CDMW_ERROR;

import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.ServantContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.ServerContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptorRegistration;

import cdmw.ccm.ccmcopi.COPICurrentImpl;
import cdmw.ccm.ccmcopi.COPIServiceInit;
import cdmw.ccm.common.CCMLoggers;

import com.thalesgroup.CdmwCcmCopi.COPICurrent;
import com.thalesgroup.CdmwCcmCopi.COPICurrentHelper;

public class Session2ExtensionContextImpl extends Session2ContextImpl implements
        com.thalesgroup.CdmwCcmContainer.Session2ExtensionContext {
    /*
     * logger to be used in CCMContext
     */
    protected cdmw.logging.Logger logger = CCMLoggers.GetCcmcontainerLogger();

    /*
     * container context
     */
    private org.omg.Components.ExtensionContext context;

    /*
     * Services map
     */
    //private java.util.Map<String, org.omg.CORBA.Object> servicesMap;
    
    
    /**
     * gives access to the orb specific data related to Container Interceptors
     */
    protected COPICurrent copiCurrent;
    

    public Session2ExtensionContextImpl(
            org.omg.CORBA.ORB orb,
            org.omg.PortableServer.POA parent,
            org.omg.Components.CCMHome home,
            String homeId,
            ComponentServantProviderInterface servantProvider,
            com.thalesgroup.CdmwDeployment.ServantLifetimeValue servantLifetime,
            com.thalesgroup.CdmwDeployment.ThreadingPolicyValue threadingPolicy,
            org.omg.Components.HomeRegistration homeRegistration,
            com.thalesgroup.CdmwEvent.EventChannelFactory eventChannelFactory,
            String processName, String applicationName) {
        super(orb, parent, home, homeId, servantProvider, servantLifetime,
                threadingPolicy, homeRegistration, eventChannelFactory,
                processName, applicationName);

        CDMW_LOG_FUNCTION(logger, "");
        try {
        
            if(COPIServiceInit.isInitialized()) 
                this.copiCurrent = COPICurrentHelper.narrow(orb.resolve_initial_references(COPICurrentImpl.COPI_CURRENT_ID));
        
        } catch (InvalidName e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    // Specific Cookie implementation to ease Service search
    private class CookieImpl extends org.omg.Components.Cookie {
        public CookieImpl(String cookieValue) {
            this.cookieValue = cookieValue.getBytes();
        }

        public String getCookieString() {
            return new String(this.cookieValue);
        }
    }

    //
    // IDL:omg.org/Components/ExtensionContext/install_service_reference:1.0
    //
    public org.omg.Components.Cookie install_service_reference(
            String service_id, org.omg.CORBA.Object objref) throws CCMException {
        CDMW_LOG_FUNCTION(logger, "service_id=" + service_id);
        
       /* synchronized (servicesMap) {
            // check if service is already installed
            if (servicesMap.containsKey(service_id)) {
                // service already installed
                CDMW_ERROR(logger, "Service " + service_id
                        + " is already installed.", new Throwable());
                org.omg.Components.CCMException ex = new org.omg.Components.CCMException(
                        org.omg.Components.CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }

            // add service to ServicesMap
            servicesMap.put(service_id, objref);
            */
        try {
            serviceRegistry.bind(service_id, objref);
        } catch (CCMException e) {
            CDMW_ERROR(logger, "Service " + service_id
                    + " is already installed.", new Throwable());
            org.omg.Components.CCMException ex = new org.omg.Components.CCMException(
                    org.omg.Components.CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

            // return Cookie with service_id inside
        return new CookieImpl(service_id);
        //}
    }

    //
    // IDL:omg.org/Components/ExtensionContext/uninstall_service_reference:1.0
    //
    public org.omg.CORBA.Object uninstall_service_reference(
            org.omg.Components.Cookie ck) throws CCMException {
        CDMW_LOG_FUNCTION(logger, "");
       /* synchronized (servicesMap) {
            // extract service_id from Cookie
            String service_id = ((CookieImpl) ck).getCookieString();

            // check if service is installed
            if (!servicesMap.containsKey(service_id)) {
                // service not installed
                CDMW_ERROR(logger, "Service " + service_id
                        + " is not installed", new Throwable());
                org.omg.Components.CCMException ex = new org.omg.Components.CCMException(
                        org.omg.Components.CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }*/
            
        String service_id = ((CookieImpl) ck).getCookieString();
        try {
            return serviceRegistry.unbind(service_id);
        } catch (CCMException e) {
            // service not installed
            CDMW_ERROR(logger, "Service " + service_id + " is not installed",
                    new Throwable());
            org.omg.Components.CCMException ex = new org.omg.Components.CCMException(
                    org.omg.Components.CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
            // remove service from ServicesMap
            //return servicesMap.remove(service_id);
        //}
    }

    public ClientContainerInterceptorRegistration get_client_interceptor_registration()
            throws CCMException {
        if(copiCurrent == null) {
            throw new CCMException("COPI Service not Initialized", 
                                CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
        }
        return copiCurrent.get_client_interceptor_registration();
    }

    public ServerContainerInterceptorRegistration get_server_interceptor_registration()
            throws CCMException {
        if(copiCurrent == null) {
            throw new CCMException("COPI Service not Initialized", 
                                CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
        }
        return copiCurrent.get_server_interceptor_registration();
        
    }

    public StubContainerInterceptorRegistration get_stub_interceptor_registration()
            throws CCMException {
        if(copiCurrent == null) {
            throw new CCMException("COPI Service not Initialized", 
                                CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
        }
        return copiCurrent.get_stub_interceptor_registration();
    }

    public ServantContainerInterceptorRegistration get_servant_interceptor_registration()
            throws CCMException {
        if(copiCurrent == null) {
            throw new CCMException("COPI Service not Initialized", 
                                CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
        }
        return copiCurrent.get_servant_interceptor_registration();
    }

}
