/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.qosenablers.dds;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.util.ArrayList;

import org.omg.CORBA.StringHolder;
import org.omg.CORBA.StringSeqHolder;

/**
 * Generic wrapper for the CORBA "<T>[Seq]Holder" type
 */
public class DataSeqHolder<D, H extends Object> {

    private H corbaHolder;
    private Field values;
    Class valueClass;

    public DataSeqHolder(H holder, Class<D> cl) throws SecurityException,
            NoSuchFieldException, IllegalArgumentException,
            IllegalAccessException {
        this.corbaHolder = holder;
        this.valueClass = cl;
        if (values == null && corbaHolder != null) {
            Class<? extends Object> c = corbaHolder.getClass();
            values = c.getField("value");

        }
    }

    public DataSeqHolder(Class<D[]> cl, H holder) throws SecurityException,
            NoSuchFieldException, IllegalArgumentException,
            IllegalAccessException {
        this.corbaHolder = holder;
        this.valueClass = cl;

        if (values == null && corbaHolder != null) {
            Class<? extends Object> c = corbaHolder.getClass();
            values = c.getField("value");

        }
    }

    /**
     * setting the sequence holder of <D>
     *
     * @param data
     * @throws SecurityException
     * @throws NoSuchFieldException
     * @throws IllegalArgumentException
     * @throws IllegalAccessException
     */
    public void setDataValueSeq(D[] data) throws SecurityException,
            NoSuchFieldException, IllegalArgumentException,
            IllegalAccessException {
        values.set(corbaHolder, data);
    }

    public void setDataValueSeq_o(Object[] data)
            throws IllegalArgumentException, IllegalAccessException {

        if (data != null) {
            // NOTA: data.getClass().getComponentType() does not return the
            // runtime type
            Object a = Array.newInstance(valueClass.getComponentType(), data.length);
            for (int i = 0; i < data.length; i++) {
                Array.set(a, i, (D) data[i]);
            }
            values.set(corbaHolder, a);
        } else
            values.set(corbaHolder, data);

    }

    /**
     * setting a non sequence holder of <D>
     *
     * @param data
     * @throws SecurityException
     * @throws NoSuchFieldException
     * @throws IllegalArgumentException
     * @throws IllegalAccessException
     */
    public void setDataValue(D data) throws SecurityException,
            NoSuchFieldException, IllegalArgumentException,
            IllegalAccessException {

        values.set(corbaHolder, data);
    }

    public D[] seqValue() throws IllegalArgumentException,
            IllegalAccessException {
        return (D[]) values.get(corbaHolder);
    }

    public D value() throws IllegalArgumentException, IllegalAccessException {
        return (D) values.get(corbaHolder);
    }

    public void init(Class<D> valueClass) throws InstantiationException,
            IllegalAccessException, SecurityException,
            IllegalArgumentException, NoSuchFieldException {
        D d = valueClass.newInstance();
        setDataValue(d);
    }

    public void initSeq(Class<D[]> valueClass) throws InstantiationException,
            IllegalAccessException, SecurityException,
            IllegalArgumentException, NoSuchFieldException {
        Object a = Array.newInstance(valueClass.getComponentType(), 0);
        values.set(corbaHolder, a);
    }

    public void init() throws InstantiationException,
    IllegalAccessException, SecurityException,
    IllegalArgumentException, NoSuchFieldException {
        if(valueClass.getComponentType() != null){ // it is a sequence
            Object a = Array.newInstance(valueClass.getComponentType(), 0);
            values.set(corbaHolder, a);
        }
        else {
            D d = (D)valueClass.newInstance();
            setDataValue(d);
        }
    }

    public static void main(String[] args) {

        StringSeqHolder strH = new StringSeqHolder(
                new String[] { "0", "1", "2" });
        DataSeqHolder<String, StringSeqHolder> holder = null;

        try {
            holder = new DataSeqHolder<String, StringSeqHolder>(String[].class, strH);
            for (String s : holder.seqValue()) {
                System.out.println("Elément : " + s);
            }

            strH = new StringSeqHolder(new String[] { "3", "4", "5" });
            ArrayList<String> a = new ArrayList<String>();
            a.add("3");
            a.add("4");

            holder.setDataValueSeq_o(a.toArray());

            System.out.println(" ------------- ");

            for (String s : holder.seqValue()) {
                System.out.println("Elément : " + s);
            }

            System.out.println(" ------------- ");
            holder.initSeq(String[].class);
            if (holder.seqValue().length == 0) {
                System.out.println("SeqHolder initialisé");
            } else {
                for (String s : holder.seqValue()) {
                    System.out.println("Elément : " + s);
                }
            }
            StringHolder sh = new StringHolder();
            DataSeqHolder<String, StringHolder> holder2 = new DataSeqHolder<String, StringHolder>(
                    sh, String.class);

            System.out.println("holder2.value=" + holder2.value());
            holder2.setDataValue("Bye");
            System.out.println("holder2.value=" + holder2.value());
            System.out.println("StringHolder.value=" + sh.value);

            holder2.init(String.class);
            System.out.println("holder2.value=" + holder2.value());
            System.out.println("StringHolder.value=" + sh.value);

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

}
