/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.qosenablers.dds;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlEnum;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.xml.sax.SAXException;

/**
 *
 */
public class QoSProfileUnmarshaller {

    //public static String qosProfileSchemaFile = "/home/nawel/devel/MOSIC/trunk/CCMContainer/java/cdmw/ccm/qosenablers/dds/DDS_QoSProfile.xsd";
    public InputStream qosProfileSchemaFile = this.getClass().getClassLoader().getResourceAsStream("cdmw/ccm/qosenablers/dds/DDS_QoSProfile.xsd");

    private Schema qosProfileSchema;
    private Unmarshaller qosProfUnmarshaller;

    public QoSProfileUnmarshaller() throws JAXBException, SAXException{
        JAXBContext context = JAXBContext.newInstance(DDS_CCM.class,
                QosProfile.class,
                DataReaderQos.class,
                DataWriterQos.class,
                TopicQos.class,
                DomainParticipantQos.class,
                PublisherQos.class,
                SubscriberQos.class);

        qosProfUnmarshaller = context.createUnmarshaller();
        String schemaLang = "http://www.w3.org/2001/XMLSchema";
        SchemaFactory factory = SchemaFactory.newInstance(schemaLang);
        qosProfileSchema = factory.newSchema(new StreamSource(qosProfileSchemaFile));
        //qosProfUnmarshaller.setSchema(qosProfileSchema);
    }

    public QosProfile[] getQosProfiles(String xmlfile ) throws JAXBException, ParserConfigurationException, SAXException, IOException{

/*        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        DocumentBuilder db = dbf.newDocumentBuilder();
        Document doc = db.parse(new File( xmlfile));
        Element  e = doc.getDocumentElement();
        System.out.println("ROOT: " + e.getTagName());*/

        //JAXBElement<DDS_CCM> foo = qosProfUnmarshaller.unmarshal(doc, DDS_CCM.class);
        //DDS_CCM global = (DDS_CCM) qosProfUnmarshaller.unmarshal(doc);
        //DDS_CCM global = foo.getValue();
        //if(global==null) System.out.println("null JAXB Element");

        DDS_CCM global = (DDS_CCM)qosProfUnmarshaller.unmarshal(new File(xmlfile));

        /*JAXBElement<DDS_CCM> foo = qosProfUnmarshaller.unmarshal(new StreamSource(new File(xmlfile)), DDS_CCM.class);
        DDS_CCM global = foo.getValue();
        if(global==null) System.out.println("null JAXB Element");*/
        return global.qos_profile.toArray(new QosProfile[0]);
    }

    public QosProfile[] getQosProfiles(Reader xmlfile ) throws JAXBException{

        DDS_CCM global = (DDS_CCM)qosProfUnmarshaller.unmarshal(xmlfile);
        return global.qos_profile.toArray(new QosProfile[0]);
    }

    public boolean validate(String xmlfile) throws SAXException, IOException{
        Validator validator = qosProfileSchema.newValidator();
        validator.validate(new StreamSource(xmlfile));
        return true;
    }

    public boolean validate(Reader xmlfile) throws SAXException, IOException{
        Validator validator = qosProfileSchema.newValidator();
        validator.validate(new StreamSource(xmlfile));
        return true;
    }

    public static void main(String[] args) throws JAXBException, Exception{

        QoSProfileUnmarshaller m = new QoSProfileUnmarshaller();

        /*boolean res= m.validate("/home/nawel/devel/MOSIC/trunk/CCMContainer/java/cdmw/ccm/qosenablers/dds/DDS_DefaultQoS.xml");
        if(res==true) System.out.println("XML Document validation OK :-)");
        else System.out.println("XML Document validation KO :-(");*/

        QosProfile profile = m.getQosProfiles("/home/nawel/devel/MOSIC/trunk/CCMContainer/java/cdmw/ccm/qosenablers/dds/DDS_DefaultQoS.xml")[0];

        System.out.println(profile.name);
        System.out.println(profile.base_name);
        System.out.println(profile.datareader_qos.durability.kind.name());
        System.out.println(profile.datareader_qos.durability.toDDSPolicy().kind.value());
        System.out.println(profile.datareader_qos.deadline.period.nanosec);
        System.out.println(profile.datareader_qos.deadline.period.sec);
        System.out.println(profile.datareader_qos.latency_budget.duration.nanosec);
        System.out.println(profile.datareader_qos.latency_budget.duration.nanosec);
        System.out.println(profile.datareader_qos.liveliness.kind.name());
        System.out.println(profile.datareader_qos.liveliness.lease_duration.nanosec);
        System.out.println(profile.datareader_qos.liveliness.lease_duration.sec);
        System.out.println(profile.datareader_qos.reliability.kind.name());
        System.out.println(profile.datareader_qos.reliability.max_blocking_time.nanosec);
        System.out.println(profile.datareader_qos.reliability.max_blocking_time.sec);
        System.out.println(profile.datareader_qos.destination_order.kind.name());
        System.out.println(profile.datareader_qos.history.kind.name());
        System.out.println(profile.datareader_qos.history.depth);
        System.out.println(profile.datareader_qos.resource_limits.max_instances);
        System.out.println(profile.datareader_qos.resource_limits.max_samples);
        System.out.println(profile.datareader_qos.resource_limits.max_samples_per_instance);
        System.out.println(profile.datareader_qos.user_data.value);
        System.out.println(profile.datareader_qos.ownership.kind.name());
        System.out.println(profile.datareader_qos.time_based_filter.minimum_separation.nanosec);
        System.out.println(profile.datareader_qos.time_based_filter.minimum_separation.sec);
        System.out.println(profile.datareader_qos.reader_data_lifecycle.autopurge_disposed_samples_delay.nanosec);
        System.out.println(profile.datareader_qos.reader_data_lifecycle.autopurge_nowriter_samples_delay.nanosec);

    }

}

@XmlRootElement(name="dds_ccm")
@XmlType(name="", propOrder = {
    "qos_profile"
})
class DDS_CCM {
    @XmlElement(name="qos_profile")
    List<QosProfile> qos_profile;
}

@XmlRootElement(name="qos_profile")
@XmlType(name="qosProfile")
class QosProfile {

  public DataReaderQos datareader_qos ;
  public DataWriterQos datawriter_qos;
  public TopicQos topic_qos;
  public DomainParticipantQos domainparticipant_qos;
  public PublisherQos publisher_qos;
  public SubscriberQos subscriber_qos;

  @XmlAttribute()
  public String name;

  public String base_name;

}
@XmlRootElement(name="datareader_qos")
class DataReaderQos
{
  public DurabilityQosPolicy durability ;
  public DDS.DeadlineQosPolicy deadline ;
  public DDS.LatencyBudgetQosPolicy latency_budget ;
  public LivelinessQosPolicy liveliness ;
  public ReliabilityQosPolicy reliability ;
  public DestinationOrderQosPolicy destination_order ;
  public HistoryQosPolicy history ;
  public DDS.ResourceLimitsQosPolicy resource_limits ;
  public DDS.UserDataQosPolicy user_data ;
  public OwnershipQosPolicy ownership ;
  public DDS.TimeBasedFilterQosPolicy time_based_filter ;
  public DDS.ReaderDataLifecycleQosPolicy reader_data_lifecycle ;

  @XmlAttribute()
  public String name;

  public String base_name;

  public String topic_filter;


  public DDS.DataReaderQos toDDSQos(){

      return new DDS.DataReaderQos(durability!=null?durability.toDDSPolicy():null,
                                   deadline,
                                   latency_budget,
                                   liveliness!=null?liveliness.toDDSPolicy():null,
                                   reliability!=null?reliability.toDDSPolicy():null,
                                   destination_order!=null?destination_order.toDDSPolicy():null,
                                   history!=null?history.toDDSPolicy():null,
                                   resource_limits,
                                   user_data,
                                   ownership!=null?ownership.toDDSPolicy():null,
                                  time_based_filter, reader_data_lifecycle
                                  /*, ownership!=null?ownership.toDDSPolicy():null*/);
  }
}

@XmlRootElement(name="datawriter_qos")
class DataWriterQos
{
  public DurabilityQosPolicy durability ;
  public DDS.DeadlineQosPolicy deadline ;
  public DDS.LatencyBudgetQosPolicy latency_budget ;
  public LivelinessQosPolicy liveliness ;
  public ReliabilityQosPolicy reliability ;
  public DestinationOrderQosPolicy destination_order ;
  public HistoryQosPolicy history ;
  public DDS.ResourceLimitsQosPolicy resource_limits ;
  public DDS.TransportPriorityQosPolicy transport_priority ;
  public DDS.LifespanQosPolicy lifespan ;
  public DDS.UserDataQosPolicy user_data ;
  public OwnershipQosPolicy ownership ;
  public DDS.OwnershipStrengthQosPolicy ownership_strength ;
  public DDS.WriterDataLifecycleQosPolicy writer_data_lifecycle ;

  @XmlAttribute()
  public String name;

  public String base_name;

  public String topic_filter;


  public DDS.DataWriterQos toDDSQos(){
      return new DDS.DataWriterQos(durability!=null?durability.toDDSPolicy():null,
                                   deadline,
                                   latency_budget,
                                   liveliness!=null?liveliness.toDDSPolicy():null,
                                   reliability!=null?reliability.toDDSPolicy():null,
                                   destination_order!=null?destination_order.toDDSPolicy():null,
                                   history!=null?history.toDDSPolicy():null,
                                  resource_limits,
                                  transport_priority,
                                  lifespan, user_data,
                                  ownership!=null?ownership.toDDSPolicy():null,
                                  ownership_strength, writer_data_lifecycle
                                  /*, ownership!=null?ownership.toDDSPolicy():null*/);
  }
}

@XmlRootElement(name="domainparticipant_qos")
class DomainParticipantQos
{
  public DDS.UserDataQosPolicy user_data ;
  public DDS.EntityFactoryQosPolicy entity_factory ;
  //public SchedulingQosPolicy watchdog_scheduling ;
  //public SchedulingQosPolicy listener_scheduling ;

  @XmlAttribute()
  public String name;

  public String base_name;

  public String topic_filter;


  public DDS.DomainParticipantQos toDDSQos(){
      return new DDS.DomainParticipantQos(user_data, entity_factory,
              null, null);
  }
}

@XmlRootElement(name="publisher_qos")
class PublisherQos
{
  public PresentationQosPolicy presentation ;
  public DDS.PartitionQosPolicy partition ;
  public DDS.GroupDataQosPolicy group_data ;
  public DDS.EntityFactoryQosPolicy entity_factory ;

  public DDS.PublisherQos toDDSQos(){
      return new DDS.PublisherQos(presentation!=null?presentation.toDDSPolicy():null,
              partition, group_data, entity_factory);
  }
}

@XmlRootElement(name="subscriber_qos")
class SubscriberQos
{
  public PresentationQosPolicy presentation ;
  public DDS.PartitionQosPolicy partition ;
  public DDS.GroupDataQosPolicy group_data ;
  public DDS.EntityFactoryQosPolicy entity_factory ;

  @XmlAttribute()
  public String name;

  public String base_name;

  public String topic_filter;

  public DDS.SubscriberQos toDDSQos(){
      return new DDS.SubscriberQos(presentation!=null?presentation.toDDSPolicy():null,
              partition, group_data, entity_factory);
  }
}

@XmlRootElement(name="topic_qos")
class TopicQos
{
  public DDS.TopicDataQosPolicy topic_data ;
  public DurabilityQosPolicy durability ;
  public DurabilityServiceQosPolicy durability_service ;
  public DDS.DeadlineQosPolicy deadline ;
  public DDS.LatencyBudgetQosPolicy latency_budget ;
  public LivelinessQosPolicy liveliness ;
  public ReliabilityQosPolicy reliability ;
  public DestinationOrderQosPolicy destination_order ;
  public HistoryQosPolicy history ;
  public DDS.ResourceLimitsQosPolicy resource_limits ;
  public DDS.TransportPriorityQosPolicy transport_priority ;
  public DDS.LifespanQosPolicy lifespan ;
  public OwnershipQosPolicy ownership ;

  @XmlAttribute()
  public String name;

  public String base_name;

  public String topic_filter;

  public DDS.TopicQos toDDSQos(){
      return new DDS.TopicQos(topic_data,
              durability!=null?durability.toDDSPolicy():null,
              durability_service!=null?durability_service.toDDSPolicy():null,
              deadline,
              latency_budget,
              liveliness!=null?liveliness.toDDSPolicy():null,
              reliability!=null?reliability.toDDSPolicy():null,
              destination_order!=null?destination_order.toDDSPolicy():null,
              history!=null?history.toDDSPolicy():null,
              resource_limits,
              transport_priority,
              lifespan,
              ownership!=null?ownership.toDDSPolicy():null);
  }

}

class DurabilityServiceQosPolicy
{
  public DDS.Duration_t service_cleanup_delay ;
  public HistoryQosPolicyKind history_kind ;
  public int history_depth ;
  public int max_samples ;
  public int max_instances ;
  public int max_samples_per_instance ;

  public DDS.DurabilityServiceQosPolicy toDDSPolicy(){

      return new DDS.DurabilityServiceQosPolicy(service_cleanup_delay,
              history_kind.toDDSKind(), history_depth, max_samples,
              max_instances, max_samples_per_instance);

  }
}

class DurabilityQosPolicy
{
  public DurabilityQosPolicyKind kind;

  public DDS.DurabilityQosPolicy toDDSPolicy(){

      return new DDS.DurabilityQosPolicy(kind.toDDSKind());
  }
}

@XmlEnum
enum DurabilityQosPolicyKind
{
    VOLATILE_DURABILITY_QOS, TRANSIENT_LOCAL_DURABILITY_QOS,
    TRANSIENT_DURABILITY_QOS, PERSISTENT_DURABILITY_QOS;

    public DDS.DurabilityQosPolicyKind toDDSKind(){
        switch(this){
        case VOLATILE_DURABILITY_QOS :
            return DDS.DurabilityQosPolicyKind.VOLATILE_DURABILITY_QOS;
        case TRANSIENT_LOCAL_DURABILITY_QOS :
            return DDS.DurabilityQosPolicyKind.TRANSIENT_LOCAL_DURABILITY_QOS;
        case TRANSIENT_DURABILITY_QOS :
            return DDS.DurabilityQosPolicyKind.TRANSIENT_DURABILITY_QOS;
        case PERSISTENT_DURABILITY_QOS :
            return DDS.DurabilityQosPolicyKind.PERSISTENT_DURABILITY_QOS;
         default: return null;
        }
    }
}

class LivelinessQosPolicy
{
  public LivelinessQosPolicyKind kind ;
  public DDS.Duration_t lease_duration ;

  public DDS.LivelinessQosPolicy toDDSPolicy(){
      return new DDS.LivelinessQosPolicy(kind.toDDSKind(), lease_duration);
  }

}

@XmlEnum
enum LivelinessQosPolicyKind {
    AUTOMATIC_LIVELINESS_QOS,MANUAL_BY_PARTICIPANT_LIVELINESS_QOS,
    MANUAL_BY_TOPIC_LIVELINESS_QOS;

    public DDS.LivelinessQosPolicyKind toDDSKind(){
        switch(this){
        case AUTOMATIC_LIVELINESS_QOS:
            return DDS.LivelinessQosPolicyKind.AUTOMATIC_LIVELINESS_QOS;
        case MANUAL_BY_PARTICIPANT_LIVELINESS_QOS:
            return DDS.LivelinessQosPolicyKind.MANUAL_BY_PARTICIPANT_LIVELINESS_QOS;
        case MANUAL_BY_TOPIC_LIVELINESS_QOS:
            return DDS.LivelinessQosPolicyKind.MANUAL_BY_PARTICIPANT_LIVELINESS_QOS;
        default: return null;
        }
    }
}

class ReliabilityQosPolicy
{
  public ReliabilityQosPolicyKind kind ;
  public DDS.Duration_t max_blocking_time ;

  public DDS.ReliabilityQosPolicy toDDSPolicy(){
      return new DDS.ReliabilityQosPolicy(kind.toDDSKind(), max_blocking_time);
  }
}

@XmlEnum
enum ReliabilityQosPolicyKind{
    BEST_EFFORT_RELIABILITY_QOS,RELIABLE_RELIABILITY_QOS;

    public DDS.ReliabilityQosPolicyKind toDDSKind(){
        switch(this){
        case BEST_EFFORT_RELIABILITY_QOS:
            return DDS.ReliabilityQosPolicyKind.BEST_EFFORT_RELIABILITY_QOS ;
        case RELIABLE_RELIABILITY_QOS:
            return DDS.ReliabilityQosPolicyKind.RELIABLE_RELIABILITY_QOS;
         default: return null;
        }
    }
}

class DestinationOrderQosPolicy
{
  public DestinationOrderQosPolicyKind kind;

  public DDS.DestinationOrderQosPolicy toDDSPolicy(){
      return new DDS.DestinationOrderQosPolicy(kind.toDDSKind());
  }
}

@XmlEnum
enum DestinationOrderQosPolicyKind {
    BY_RECEPTION_TIMESTAMP_DESTINATIONORDER_QOS,
    BY_SOURCE_TIMESTAMP_DESTINATIONORDER_QOS;

    public DDS.DestinationOrderQosPolicyKind toDDSKind(){
        switch(this){
        case BY_RECEPTION_TIMESTAMP_DESTINATIONORDER_QOS:
            return DDS.DestinationOrderQosPolicyKind.BY_RECEPTION_TIMESTAMP_DESTINATIONORDER_QOS;
        case BY_SOURCE_TIMESTAMP_DESTINATIONORDER_QOS:
            return DDS.DestinationOrderQosPolicyKind.BY_SOURCE_TIMESTAMP_DESTINATIONORDER_QOS;
            default: return null;
        }
    }
}

class HistoryQosPolicy
{
  public HistoryQosPolicyKind kind;
  public int depth ;

  public DDS.HistoryQosPolicy toDDSPolicy(){
      return new DDS.HistoryQosPolicy(kind.toDDSKind(), depth);
  }

}

@XmlEnum
enum  HistoryQosPolicyKind{
    KEEP_ALL_HISTORY_QOS, KEEP_LAST_HISTORY_QOS;

    public DDS.HistoryQosPolicyKind toDDSKind(){
        switch(this){
        case KEEP_ALL_HISTORY_QOS:
            return DDS.HistoryQosPolicyKind.KEEP_ALL_HISTORY_QOS;
        case KEEP_LAST_HISTORY_QOS:
            return DDS.HistoryQosPolicyKind.KEEP_LAST_HISTORY_QOS;
            default: return null;
        }
    }
}

class SchedulingQosPolicy
{
  public SchedulingClassQosPolicy scheduling_class ;
  public SchedulingPriorityQosPolicy scheduling_priority_kind ;
  public int scheduling_priority;

  public DDS.SchedulingQosPolicy toDDSPolicy(){
      return new DDS.SchedulingQosPolicy(scheduling_class.toDDSPolicy(),
              scheduling_priority_kind.toDDSPolicy(),
              scheduling_priority);
  }
}

class SchedulingClassQosPolicy{
    public SchedulingClassQosPolicyKind kind;

    public DDS.SchedulingClassQosPolicy toDDSPolicy(){
        return new DDS.SchedulingClassQosPolicy(kind.toDDSKind());
    }
}

@XmlEnum
enum SchedulingClassQosPolicyKind {
    SCHEDULE_DEFAULT, SCHEDULE_REALTIME, SCHEDULE_TIMESHARING;

    public DDS.SchedulingClassQosPolicyKind toDDSKind(){
        switch(this){
        case SCHEDULE_DEFAULT:
            return DDS.SchedulingClassQosPolicyKind.SCHEDULE_DEFAULT ;
        case SCHEDULE_REALTIME:
            return DDS.SchedulingClassQosPolicyKind.SCHEDULE_REALTIME ;
        case SCHEDULE_TIMESHARING:
            return DDS.SchedulingClassQosPolicyKind.SCHEDULE_TIMESHARING ;
            default: return null;
        }
    }
}

class SchedulingPriorityQosPolicy
{
  public SchedulingPriorityQosPolicyKind kind ;

  public DDS.SchedulingPriorityQosPolicy toDDSPolicy(){
      return new DDS.SchedulingPriorityQosPolicy(kind.toDDSKind());
  }
}

@XmlEnum
enum SchedulingPriorityQosPolicyKind {
    PRIORITY_ABSOLUTE, PRIORITY_RELATIVE;

    public DDS.SchedulingPriorityQosPolicyKind toDDSKind(){
        switch(this){
        case PRIORITY_ABSOLUTE:
            return DDS.SchedulingPriorityQosPolicyKind.PRIORITY_ABSOLUTE ;
        case PRIORITY_RELATIVE:
            return DDS.SchedulingPriorityQosPolicyKind.PRIORITY_RELATIVE ;
            default: return null;
        }
    }
}

class PresentationQosPolicy
{
  public PresentationQosPolicyAccessScopeKind access_scope;
  public boolean coherent_access;
  public boolean ordered_access;

  public DDS.PresentationQosPolicy toDDSPolicy(){
      return new DDS.PresentationQosPolicy(access_scope.toDDSKind(),
              coherent_access, ordered_access);
  }
}

@XmlEnum
enum PresentationQosPolicyAccessScopeKind {
    GROUP_PRESENTATION_QOS, INSTANCE_PRESENTATION_QOS,
    TOPIC_PRESENTATION_QOS;

    public DDS.PresentationQosPolicyAccessScopeKind toDDSKind(){
        switch(this){
        case GROUP_PRESENTATION_QOS:
            return DDS.PresentationQosPolicyAccessScopeKind.GROUP_PRESENTATION_QOS ;
        case INSTANCE_PRESENTATION_QOS:
            return DDS.PresentationQosPolicyAccessScopeKind.INSTANCE_PRESENTATION_QOS ;
        case TOPIC_PRESENTATION_QOS:
            return DDS.PresentationQosPolicyAccessScopeKind.TOPIC_PRESENTATION_QOS ;
            default: return null;
        }
    }
}

class OwnershipQosPolicy
{
  public OwnershipQosPolicyKind kind;

  public DDS.OwnershipQosPolicy toDDSPolicy(){
      return new DDS.OwnershipQosPolicy(kind.toDDSKind());
  }
}

@XmlEnum
enum OwnershipQosPolicyKind {
    EXCLUSIVE_OWNERSHIP_QOS, SHARED_OWNERSHIP_QOS;

    public DDS.OwnershipQosPolicyKind toDDSKind(){
      switch(this){
      case EXCLUSIVE_OWNERSHIP_QOS:
          return DDS.OwnershipQosPolicyKind.EXCLUSIVE_OWNERSHIP_QOS;
      case SHARED_OWNERSHIP_QOS:
          return DDS.OwnershipQosPolicyKind.SHARED_OWNERSHIP_QOS;
          default: return null;
      }
    }
}
