/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
//package cdmw.loggingservice;
package cdmw.ccm.qosenablers.logging;

import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION;
import static cdmw.logging.Logger.CDMW_DEBUG;

import java.util.HashMap;
import java.util.Map;

import org.omg.CORBA.Object;
import org.omg.Components.CCMException;
import org.omg.Components.ConfigValuesHolder;
import org.omg.Components.Cookie;
import org.omg.Components.ExtensionContext;
import org.omg.Components.SessionContext;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;

import cdmw.ccm.common.CCMLoggers;

import com.thalesgroup.CdmwLogging.Logger;
import com.thalesgroup.CdmwLogging.LoggerFactory;
import com.thalesgroup.CdmwCcmPlugins.NamedValue;
import com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistration;
import com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistrationHelper;


/**
 * Logging QoSEnabler
 */
public class LoggingServiceComponentImpl
    extends cdmw.ccm.qosenablers.common.QOSEnablerBase
    implements com.thalesgroup.CdmwCcmPlugins.CCM_CdmwQosEnabler, com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor
{

    private static final String FACILITY = "LoggingQoSEnabler";
    
    // context
    ExtensionContext context;
     
    // Logger used by this QoSEnabler
    Logger enablerLogger = CreateLoggerFactory(FACILITY).get_logger(FACILITY);
    
    // need to track cookies as they are returned by install_service_reference
    private Map<String, org.omg.Components.Cookie> instanceId2cookieMap =  new HashMap<String, Cookie>();
    
    // status of this QoSEnabler
    int status = com.thalesgroup.CdmwCcmPlugins.UNKNOWN_STATUS.value;

    /**
     * Component attribute : settings
     */
    private NamedValue[] m_service_settings = null;
    
    /**
     * Component attribute : statistics
     */
    private NamedValue[] m_service_statistics = null;

    
    private void setStatusTo(int status) {
        synchronized (this) {
            this.status = status;            
        }        
    }    
    

    public void ccm_activate() throws CCMException {
        this.setStatusTo(com.thalesgroup.CdmwCcmPlugins.STARTING_STATUS.value);

   
    }
    

    public void ccm_passivate() throws CCMException {
        this.setStatusTo(com.thalesgroup.CdmwCcmPlugins.STOPPED_STATUS.value);
    }


    public void ccm_remove() throws CCMException {
        this.setStatusTo(com.thalesgroup.CdmwCcmPlugins.STOPPED_STATUS.value);
    }


    public void set_session_context(SessionContext ctx) throws CCMException {
        this.setStatusTo(com.thalesgroup.CdmwCcmPlugins.STARTING_STATUS.value);    
    }


    public void set_extension_context(ExtensionContext ctx) throws CCMException {
        this.setStatusTo(com.thalesgroup.CdmwCcmPlugins.STARTING_STATUS.value);            
        this.context = ctx;   
                
        try {
            cdmw.logging.Logger.CDMW_DEBUG(this.enablerLogger, "Resolving LifeCycleInterceptorRegistrationService");

            org.omg.CORBA.Object obj = context
                .resolve_service_reference(cdmw.ccm.container.LifeCycleInterceptorRegistration.LifeCycleInterceptorRegistrationServiceId); 
         
            LifeCycleInterceptorRegistration lifeCycleInterceptorRegistration =  
                LifeCycleInterceptorRegistrationHelper.narrow(obj);

            cdmw.logging.Logger.CDMW_DEBUG(this.enablerLogger, "Registering LifeCycleInterceptor");
            lifeCycleInterceptorRegistration.register_component_lifecycle_interceptor(this); 
        } catch (Exception e) {
            e.printStackTrace();
        }
    
        // preinstall 2 Logggers for testing    
        this.after_component_create(cdmw.ossupport.OS.getHostname() + "/CCMCOPITestApplication/CCMCOPITestProcess#CLIENT_COMPONENT_NAME1", null);
        this.after_component_create(cdmw.ossupport.OS.getHostname() + "/CCMCOPITestApplication/CCMCOPITestProcess#CLIENT_COMPONENT_NAME2", null);

        this.setStatusTo(com.thalesgroup.CdmwCcmPlugins.RUNNING_STATUS.value);     
                
        try {
            cdmw.logging.Logger.CDMW_DEBUG(this.enablerLogger, "Resolving LifeCycleInterceptorRegistrationService");

            org.omg.CORBA.Object obj = context
                .resolve_service_reference(cdmw.ccm.container.LifeCycleInterceptorRegistration.LifeCycleInterceptorRegistrationServiceId); 
         
            LifeCycleInterceptorRegistration lifeCycleInterceptorRegistration =  
                LifeCycleInterceptorRegistrationHelper.narrow(obj);

            cdmw.logging.Logger.CDMW_DEBUG(this.enablerLogger, "Registering LifeCycleInterceptor");
            lifeCycleInterceptorRegistration.register_component_lifecycle_interceptor(this); 
        } catch (Exception e) {
            e.printStackTrace();
    }

        // preinstall 2 Logggers for testing    
        this.after_component_create(cdmw.ossupport.OS.getHostname() + "/CCMCOPITestApplication/CCMCOPITestProcess#CLIENT_COMPONENT_NAME1", null);
        this.after_component_create(cdmw.ossupport.OS.getHostname() + "/CCMCOPITestApplication/CCMCOPITestProcess#CLIENT_COMPONENT_NAME2", null);
    
        this.setStatusTo(com.thalesgroup.CdmwCcmPlugins.RUNNING_STATUS.value);     
    }

    
    public static String LOGGINGFACTORY_NAME =  "LoggerFactory";

    // enforces naming convention
    public static String loggerFactoryIdForCompId(String id) {
        if(id == null)
            throw new IllegalArgumentException();
        
        return id + "@" + LOGGINGFACTORY_NAME;                 
    }

    /**
     * Returns a LoggerFactory for the specified component name.
     * All Logger created by this factory will use this component name.
     *
     * @param componentName the component name
     * @return the LoggerFactory
     */
    public static LoggerFactory CreateLoggerFactory(String componentName)
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), 
                          "componentName="+componentName);
        return new LoggerFactoryImpl(componentName);
    }
    

    public void componentInstanceTearDown(String id) {        

    }

    public String get_name() {    
        return this.FACILITY;
    }

    public int get_status() {        
        synchronized (this) {
            return status;
        }        
    }

    public void after_component_activate(String id) {
        // TODO Auto-generated method stub
        
    }

    public void after_component_connect(String id) {
        // TODO Auto-generated method stub
        
    }

    public void after_component_create(String id, Object ref) {
        if(id == null)
            throw new IllegalArgumentException();    

        com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory =
            CreateLoggerFactory(id);

        String factoryId = loggerFactoryIdForCompId(id);

        synchronized(this)
            {
                // does a factory with factoryId exist?
                if(this.instanceId2cookieMap.get(factoryId) != null) {
                    cdmw.logging.Logger.CDMW_WARN(this.enablerLogger, "LoggerFactory " + factoryId + " exists already.");
                } else { 
                    try {                    
                        cdmw.logging.Logger.CDMW_DEBUG(this.enablerLogger, "Installing LoggerFactory under " + factoryId);
                        org.omg.Components.Cookie ck = 
                            this.context.install_service_reference(factoryId, loggerFactory);

                        this.instanceId2cookieMap.put(factoryId, ck);

                    } catch (CCMException e) {                    
                        cdmw.logging.Logger.CDMW_ERROR(this.enablerLogger, "Failed to install LoggerFactory " + factoryId, e);                
                        e.printStackTrace();                                
                    }    
                }
            }        
    }

    public void after_component_passivate(String id) {
        // TODO Auto-generated method stub
        
    }

    public void after_component_remove(String id) {
        if(id == null)
            throw new IllegalArgumentException();
        
        String factoryId = loggerFactoryIdForCompId(id);
        
        synchronized(this)
            {
                org.omg.Components.Cookie ck = this.instanceId2cookieMap.get(factoryId);

                if(ck != null) {                    
                    try {
                        CDMW_DEBUG(this.enablerLogger, "Uninstalling LoggerFactory " + factoryId);
                        this.context.uninstall_service_reference(ck);                    
                    } catch (CCMException e) {
                        cdmw.logging.Logger.CDMW_ERROR(this.enablerLogger, "Failed to uninstall LoggerFactory " + factoryId, e);
                        e.printStackTrace();                    
                    } finally {
                        this.instanceId2cookieMap.remove(factoryId);
                    }
                } else {
                    cdmw.logging.Logger.CDMW_WARN(this.enablerLogger, "LoggerFactory " + factoryId + " not found.");                
                }
            }        
    }

    public void before_component_activate(String id) {
        // TODO Auto-generated method stub
        
    }

    public void before_component_connect(String id, String port, Object ref) {
        // TODO Auto-generated method stub
        
    }

    public void before_component_create(String id,
                                        ConfigValuesHolder configuration) {
        // TODO Auto-generated method stub
        
    }

    public void before_component_passivate(String id) {
        // TODO Auto-generated method stub
        
    }

    public void before_component_remove(String id) {
        // TODO Auto-generated method stub
        
    }


    public void destroy() {
        // TODO Auto-generated method stub
        
    }


    public String name() {
        // TODO Auto-generated method stub
        return null;
    }


    public short priority() {
        // TODO Auto-generated method stub
        return 0;
    }


    public IntegrationPoint registration_info() {
        // TODO Auto-generated method stub
        return null;
    }


    public void set_slot_id(int arg0) {
        // TODO Auto-generated method stub
        
    }        


    public NamedValue[] get_settings() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"settings=["+null+"]");
        return m_service_settings;
    }

    public NamedValue[] get_statistics() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"statistics=["+null+"]");
        return m_service_statistics;
    }

    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_settings:1.0
    //
    public NamedValue[] service_settings() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"");
        return m_service_settings;
    }


    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_settings:1.0
    //
    public void service_settings(NamedValue[] arg) {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        m_service_settings = new NamedValue[arg.length];
        System.arraycopy(arg, 0, m_service_settings, 0, arg.length);
        
        for(int idx=0 ; idx < m_service_settings.length ; idx++) {
            System.out.println("Setting ["+idx+"] : [" 
                               + m_service_settings[idx].name + "] -> ["
                               + m_service_settings[idx].value + "]");                       
        }
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_statistics:1.0
    //
    public NamedValue[] service_statistics() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"");
        return m_service_statistics;
    }


    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_statistics:1.0
    //
    public void service_statistics(NamedValue[] arg) {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        m_service_statistics = new NamedValue[arg.length];
        System.arraycopy(arg, 0, m_service_statistics, 0, arg.length);

        for(int idx=0 ; idx < m_service_statistics.length ; idx++) {
            System.out.println("Statistic ["+idx+"] : [" 
                               + m_service_statistics[idx].name + "] -> ["
                               + m_service_statistics[idx].value + "]");                       
        }
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }

}

