/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.qosenablers.requestfollowup;

import org.omg.CORBA.OctetSeqHelper;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Any;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItem;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItemSeqHelper;

import org.omg.PortableInterceptor.Current;
import org.omg.PortableInterceptor.CurrentHelper;
import org.omg.Components.Cookie;
import org.omg.Components.CCMException;
import org.omg.Components.SessionContext;
import org.omg.Components.ExtensionContext;
import org.omg.Components.ContainerPortableInterceptor.InvalidRegistration;

import cdmw.ccm.qosenablers.common.QOSEnablerBase;
import com.thalesgroup.CdmwCcmPlugins.CCM_CdmwQosEnabler;
import com.thalesgroup.Cdmw_CdmwCcmPlugins.CCM_CdmwQosEnabler_ExtensionContext;
import com.thalesgroup.Cdmw_CdmwCcmPlugins.CdmwPluginHelper;
import com.thalesgroup.CdmwCcmPlugins.NamedValue;

import cdmw.logging.Logger;
import cdmw.commonsvcs.requestfollowup.NDCLogger;
import cdmw.commonsvcs.requestfollowup.NDCFactory;
import cdmw.commonsvcs.requestfollowup.NDCServiceInit;

import static cdmw.logging.FunctionLogger.*;
import static cdmw.logging.Logger.*;


/**
 * This class implements the QOSEnablerBase class
 */
public class ExtendedNDCExecutorImpl
    extends QOSEnablerBase
    implements CCM_CdmwQosEnabler {

    private short index = 0;

    /**
     * Internal logging, initialised with a temporary logger that will be replaced in init method
     */
    // private Logger m_logger = Logger.Get_Logger(cdmw.commonsvcs.requestfollowup.NDCLogger.NDC_LOGGER_NAME);
    private Logger m_logger = null;

    /**
     * Extension context
     */
    private CCM_CdmwQosEnabler_ExtensionContext m_ext_ctx = null;

    /**
     * Servant Extended COPI
     */
    private NDCServantContainerInterceptorImpl m_servantCOPI = null;

    /**
     * Servant Cookie
     */
    private Cookie m_servantCookie = null;

    /**
     * Stub Extended COPI
     */
    private NDCStubContainerInterceptorImpl m_stubCOPI = null;

    /**
     * Stub cookie
     */
    private Cookie m_stubCookie = null;

    /**
     * Status of the Executor
     */
    private int m_status = com.thalesgroup.CdmwCcmPlugins.UNKNOWN_STATUS.value;

    /**
     * Cookie of the service
     */
    private Cookie m_RFUCookie;

    /**
     * PICurrent
     */
    private Current m_PICurrent = null;

    /**
     * Name of the service Request Follow Up
     */
    public static final String RFU_SERVICE_NAME = "RequestFollowUp_SERVICE";

    /**
     * Default request follow up format
     */
    private static final int CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST = 1; // Log all NDC of the list
    // public final static String REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST_PROPERTY_NAME = "Cdmw.RequestFollowUp.LogFullNDCList";
    
    private static final int CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_NDC_OPERATION = 1; // Log all NDC of the list
    // public final static String REQUEST_FOLLOW_UP_LOG_NDC_OPERATION_PROPERTY_NAME = "Cdmw.RequestFollowUp.LogNDCOperation";

    /**
     * Component attribute : settings
     */
    private NamedValue[] m_service_settings = null;

    /**
     * Component attribute : statistics
     */
    private NamedValue[] m_service_statistics = null;




    public ExtendedNDCExecutorImpl() {
        m_logger = Logger.Get_Logger(cdmw.commonsvcs.requestfollowup.NDCLogger.NDC_LOGGER_NAME);
        CDMW_LOG_FUNCTION(m_logger, "");
        CDMW_LOG_FUNCTION_RETURN(m_logger);
    }

    /**
     * Remove the component
     *
     */
    //
    // IDL:org.omg/Components/ExtensionComponent/ccm_remove:1.0
    //
    public void ccm_remove()
        throws CCMException {
        System.out.println("ExtendedNDCExecutorImpl.ccm_remove called");
        CDMW_LOG_FUNCTION(m_logger,"");

        // Unregister interceptors
        try {
            m_ext_ctx.get_servant_interceptor_registration().unregister_servant_interceptor(m_servantCookie);
            m_servantCookie = null;
            m_ext_ctx.get_stub_interceptor_registration().unregister_stub_interceptor(m_stubCookie);
            m_stubCookie = null;
        }
        catch(InvalidRegistration ire) {
            CDMW_ERROR(m_logger, "Exception raised when unregeistering interceptors: " + ire);
            CDMW_LOG_FUNCTION_EXCEPTION(m_logger, ire);
        }

        m_status = com.thalesgroup.CdmwCcmPlugins.STOPPED_STATUS.value;

        CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    /**
     * Passivate the component
     *
     */
    //
    // IDL:org.omg/Components/SessionComponent/ccm_passivate:1.0
    //
    public void ccm_passivate()
        throws CCMException {
        System.out.println("ExtendedNDCExecutorImpl.ccm_remove called");

        CDMW_LOG_FUNCTION(m_logger,"");

        m_status = com.thalesgroup.CdmwCcmPlugins.STOPPING_STATUS.value;

        //         // Uninstall the service
        //         m_ext_ctx.uninstall_service_reference(m_RFUCookie);

        CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    /**
     * Activate the component
     *
     */
    //
    // IDL:org.omg/Components/SessionComponent/ccm_activate:1.0
    //
    public void ccm_activate()
        throws CCMException {
        System.out.println("ExtendedNDCExecutorImpl.ccm_activate called");

        CDMW_LOG_FUNCTION(m_logger,"");

        this.init();

        //         // Install the service
        //         m_RFUCookie =
        //             m_ext_ctx.install_service_reference(RFU_SERVICE_NAME,
        //                                                 CdmwPluginHelper.narrow(this));

        m_status = com.thalesgroup.CdmwCcmPlugins.RUNNING_STATUS.value;

        CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    /**
     * Set the session context
     *
     * @param ctx the session context
     *
     */
    //
    // IDL:org.omg/Components/SessionComponent/set_session_context:1.0
    //
    public void set_session_context(SessionContext ctx)
        throws CCMException {
        System.out.println("ExtendedNDCExecutorImpl.set_session_context called");
        CDMW_LOG_FUNCTION(m_logger,"");
        m_status = com.thalesgroup.CdmwCcmPlugins.STARTED_STATUS.value;

        CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    /**
     * Set the extension context
     *
     * @param ctx the extension context
     *
     */
    //
    // IDL:org.omg/Components/ExtensionComponent/set_extension_context:1.0
    //
    public void set_extension_context(ExtensionContext ctx)
        throws CCMException {
        System.out.println("ExtendedNDCExecutorImpl.set_extension_context called");
        
        m_status = com.thalesgroup.CdmwCcmPlugins.STARTING_STATUS.value;
        m_ext_ctx = 
            com.thalesgroup.Cdmw_CdmwCcmPlugins.
            CCM_CdmwQosEnabler_ExtensionContextHelper.narrow(ctx);

        //         CDMW_LOG_FUNCTION(m_logger,"");

        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    /**
     * Get the name
     *
     * @return the name of the component
     *
     */
    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/get_name:1.0
    //
    public java.lang.String get_name() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"name=["+RFU_SERVICE_NAME+"]");
        return RFU_SERVICE_NAME;
    }


    public int get_status() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"status=["+m_status+"]");
        return m_status;
    }


    public NamedValue[] get_settings() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"settings=["+null+"]");
        return null;
    }


    public NamedValue[] get_statistics() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"statistics=["+null+"]");
        return null;
    }

    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_settings:1.0
    //
    public NamedValue[] service_settings() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"");
        System.out.println("ExtendedNDCExecutorImpl.service_settings (get) called");
        return m_service_settings;
    }


    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_settings:1.0
    //
    public void service_settings(NamedValue[] arg) {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        System.out.println("ExtendedNDCExecutorImpl.service_settings (set) called");
        m_service_settings = new NamedValue[arg.length];
        System.arraycopy(arg, 0, m_service_settings, 0, arg.length);
        
        for(int idx=0 ; idx < m_service_settings.length ; idx++) {
            System.out.println("Setting ["+idx+"] : [" 
                               + m_service_settings[idx].name + "] -> ["
                               + m_service_settings[idx].value + "]");                       
        }
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_statistics:1.0
    //
    public NamedValue[] service_statistics() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"");
        System.out.println("ExtendedNDCExecutorImpl.service_statistics (get) called");
        return m_service_statistics;
    }


    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_statistics:1.0
    //
    public void service_statistics(NamedValue[] arg) {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        System.out.println("ExtendedNDCExecutorImpl.service_statistics (set) called");
        m_service_statistics = new NamedValue[arg.length];
        System.arraycopy(arg, 0, m_service_statistics, 0, arg.length);

        for(int idx=0 ; idx < m_service_statistics.length ; idx++) {
            System.out.println("Statistic ["+idx+"] : [" 
                               + m_service_statistics[idx].name + "] -> ["
                               + m_service_statistics[idx].value + "]");                       
        }
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }

    /**
     * Initialize the RFU Service
     */
    private void init() 
        throws CCMException {

        CDMW_LOG_FUNCTION(m_logger,"");


        // Get request follow up format properties
        CDMW_DEBUG(m_logger,"Get request follow up format properties");
        int logFullNDCList = CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST;
        int logNDCOperation = CDMW_DEFAULT_REQUEST_FOLLOW_UP_LOG_NDC_OPERATION;

        try {
            if(m_service_settings != null) {
                for(int idx=0 ; idx < m_service_settings.length ; idx++) {
                    if((m_service_settings[idx].name).compareTo(NDCServiceInit.REQUEST_FOLLOW_UP_LOG_FULL_NDC_LIST_PROPERTY_NAME) == 0) {
                        logFullNDCList = Integer.parseInt(m_service_settings[idx].value);
                    }
                    if((m_service_settings[idx].name).compareTo(NDCServiceInit.REQUEST_FOLLOW_UP_LOG_NDC_OPERATION_PROPERTY_NAME) == 0) {
                        logNDCOperation = Integer.parseInt(m_service_settings[idx].value);
                    }
                }
            }

            if ( (logFullNDCList >= 0) && (logNDCOperation >= 0) ) {
                String fullStr = "TRUE", operationStr = "TRUE"; 
                if(logFullNDCList == 0)
                    fullStr = "FALSE";
                if(logNDCOperation == 0)
                    operationStr = "FALSE";
               
                CDMW_INFO(m_logger, "FULL NDC LIST ["+fullStr+"]");
                CDMW_INFO(m_logger, "NDC OPERATION ["+operationStr+"]");

                NDCLogger.initNDCLogger(logFullNDCList, logNDCOperation);                
                NDCFactory.initNDCFactory();
                m_logger = NDCLogger.getInstance().getLogger();

            } else {
                CDMW_WARN(m_logger,
                          "Invalid Request Follow Up format:" +
                          " logFullNDCList=" + logFullNDCList + 
                          ", logNDCOperation=" + logNDCOperation + 
                          ". RequestTimeoutORBInitializer will not be registered!");
            }
        } catch (NumberFormatException e) {
            CDMW_WARN(m_logger,
                      "Invalid Request Follow Up format:" +
                      " logFullNDCList=" + logFullNDCList + 
                      ", logNDCOperation=" + logNDCOperation + 
                      ". RequestTimeoutORBInitializer will not be registered!");
        }

        // Get the PICurrent
        System.out.println("Get the PICurrent");
        try {
            m_PICurrent = 
                CurrentHelper.
                narrow(m_ext_ctx.resolve_service_reference("PICurrent"));
        }
        catch(CCMException ccme) {
            CDMW_ERROR(m_logger, "Exception raised when getting the PICurrent: " + ccme);
            CDMW_LOG_FUNCTION_EXCEPTION(m_logger, ccme);
            throw ccme;
        }

        
        // Create and register interceptors
        CDMW_DEBUG(m_logger, "Create and register interceptors");
        try {
            CDMW_DEBUG(m_logger, "Creating");
            m_servantCOPI = new NDCServantContainerInterceptorImpl(m_PICurrent);
            m_stubCOPI = new NDCStubContainerInterceptorImpl(m_PICurrent);
            CDMW_DEBUG(m_logger, "Registering");
            m_servantCookie = m_ext_ctx.get_servant_interceptor_registration().register_servant_interceptor(m_servantCOPI);
            m_stubCookie = m_ext_ctx.get_stub_interceptor_registration().register_stub_interceptor(m_stubCOPI);
        }
        catch(CCMException ccme) {
            CDMW_ERROR(m_logger, "Exception raised when creating and registering interceptors: " + ccme);
            CDMW_LOG_FUNCTION_EXCEPTION(m_logger, ccme);
            throw ccme;
        }

        CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }
}
