/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include "COPITester_impl.hpp"
#include "ChangeMsgOnServantRecvReqCOPI_impl.hpp"
#include "ServerInfoConsumer_impl.hpp"
#include "TestCopi_Event_impl.hpp"
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>


namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {


ChangeMsgOnServantRecvReqCOPI_impl::ChangeMsgOnServantRecvReqCOPI_impl
(
 const std::string & name,
 OsSupport::Condition * cond,
 OsSupport::Mutex * lock
 ) : m_copi_name(name), m_cond(cond), m_lock(lock), m_failed(true)
{};


// Implementation skeleton destructor
ChangeMsgOnServantRecvReqCOPI_impl::~ChangeMsgOnServantRecvReqCOPI_impl (void)
{
}

void ChangeMsgOnServantRecvReqCOPI_impl::servant_receive_request (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    try {
    
        ::CORBA::String_var op = info->operation();
    
        std::cout << get_header() << "servant_receive_request called with op [" << op << "]"  << std::endl;
        
    
        if(0 == strcmp("echo", op.in())) {
            std::cout << "trying to modifying arg of echo" << std::endl;
            ::Dynamic::ParameterList_var param_list =
                  info->arguments();
    
            bool found = false;        
    
            const char * expected = "dynamischer diskord";
    
            for(::CORBA::ULong i = 0; i < param_list->length(); ++i) {
                if(::CORBA::tk_string == param_list[i].argument.type()->kind()) {
                    const char * arg;
                    param_list[i].argument >>= arg;                
                    std::cout << "received arg=[" << arg << "] expected was arg is [" << expected << "]" << std::endl;
    
                    if(0 == strcmp(arg, expected)) {
                        ::CORBA::Any any;
                        any <<= "dynamic discord";
                        param_list[i].argument = any; 
                    }
    
                    found = true;
                    break;
                }            
            }
    
            info->arguments(param_list.in());
        } else if(0 == strcmp("push", op.in())) {
            std::cout << "trying to modify event" << std::endl;
            ::Dynamic::ParameterList_var param_list =
                  info->arguments();
    
    
            if(0 == param_list->length()) {
              // read write request false
              std::cout << "received param_list of 0, returning, read write request false?" << std::endl;
     
              return;
            }
            
    
            CPPUNIT_ASSERT(1 == param_list->length());
            ::CORBA::ULong zero_idx = 0;
    
            const ::CORBA::Any * event_any = 0;
            param_list[zero_idx].argument >>= event_any;
    
            
            if( event_any && TestComponents::_tc_ServerInfo->equal(event_any->type()) ) {
                ::CORBA::ULong expected = 23;
                TestComponents::ServerInfo * arg = 0;
                if(!(*event_any >>= arg)) {
                    CPPUNIT_ASSERT(false);
                }
    
                std::cout << "ChangeMsgOnServantRecvReqCOPI_impl::servant_receive_request [" << arg->data() << "]" << std::endl;
    
                CPPUNIT_ASSERT(expected == arg->data());
                
                ::CORBA::Any any;
                TestComponents::ServerInfo_var si =
                    new ServerInfo_impl();
    
                std::cout << "ChangeMsgOnServantRecvReqCOPI_impl::servant_receive_request changing [" 
                          << arg->data() << "] into [5]" << std::endl;
    
                si->data(5);
    
                any <<= si.in();
                ::CORBA::Any hull;
                hull <<= any;
                param_list[zero_idx].argument = hull;            
                info->arguments(param_list.in());
            }
        }
        
    } catch (const CORBA::BAD_CONTEXT& ex) {
        // info->arguments() raised BAD_CONTEXT because of Disable_read_write_requests mode
        // Check this mode is set.
        CPPUNIT_ASSERT(!COPIServiceInit::Is_COPI_requests_read_write());
        std::cout << "BAD_CONTEXT exception was expected." << std::endl;
    }
}

void ChangeMsgOnServantRecvReqCOPI_impl::servant_send_reply (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException
  )
{
    ::CORBA::String_var op = info->operation();

    std::cout << get_header() << "servant_send_reply called with op [" << op << "]"  << std::endl;

    if(0 == strcmp("push", op.in())) {
        std::cout << "trying to modify event" << std::endl;
        ::Dynamic::ParameterList_var param_list =
              info->arguments();


        if(0 == param_list->length()) {
          // read write request false
          std::cout << "received param_list of 0, returning, read write request false?" << std::endl;
 
          return;
        }
 
        

        CPPUNIT_ASSERT(1 == param_list->length());
        ::CORBA::ULong zero_idx = 0;

        const ::CORBA::Any * event_any = 0;
        param_list[zero_idx].argument >>= event_any;

        if( event_any && true == TestComponents::_tc_ServerInfo->equal(event_any->type()) ) {
            ::CORBA::ULong expected = 5;
            TestComponents::ServerInfo * arg = 0;
            if(!(*event_any >>= arg)) {
                CPPUNIT_ASSERT(false);
            }

            std::cout << "ChangeMsgOnServantRecvReqCOPI_impl::servant_send_reply [" << arg->data() << "]" << std::endl;

            CPPUNIT_ASSERT(expected == arg->data());
            
            ::CORBA::Any any;
            TestComponents::ServerInfo_var si =
                new ServerInfo_impl();

            std::cout << "ChangeMsgOnServantRecvReqCOPI_impl::servant_send_reply changing [" 
                      << arg->data() << "] into [36]" << std::endl;

            si->data(36);

            any <<= si.in();

            ::CORBA::Any hull;
            hull <<= any;

            param_list[zero_idx].argument = hull;            
            info->arguments(param_list.in());
            
            {
                CDMW_MUTEX_GUARD(*m_lock);
                m_failed = false;
            }
            
            m_cond->broadcast();
        }
    }
}

void ChangeMsgOnServantRecvReqCOPI_impl::servant_send_exception (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    ::std::cout << "::servant_send_exception called!" << ::std::endl;
}

void ChangeMsgOnServantRecvReqCOPI_impl::servant_send_other (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
}

char * ChangeMsgOnServantRecvReqCOPI_impl::name ()
  throw (
    ::CORBA::SystemException
  )
{
    return CORBA::string_dup(m_copi_name.c_str());
}

::CORBA::UShort ChangeMsgOnServantRecvReqCOPI_impl::priority (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

::Components::ContainerPortableInterceptor::IntegrationPoint *
ChangeMsgOnServantRecvReqCOPI_impl::registration_info (
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

void ChangeMsgOnServantRecvReqCOPI_impl::destroy (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  // Add your implementation here
}

void ChangeMsgOnServantRecvReqCOPI_impl::set_slot_id (
    ::PortableInterceptor::SlotId slot_id
  )
  throw (
    ::CORBA::SystemException
  )
{
}


std::string ChangeMsgOnServantRecvReqCOPI_impl::get_header() {
    std::stringstream sstr;
    sstr << "####E#X#T#E#N#D#E#D#### " 
         << COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), "%a, %d %b %Y %H:%M:%S %z") 
         << "\t" << m_copi_name << "\t";
    return  sstr.str();
}


} // End namespace CIF

} // End namespace CCM

} // End namespce COPI
