/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tao/DynamicInterface/Request.h"
#include <iostream>
#include <sstream>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include "ChangeMsgOnStubSendReqCOPI_impl.hpp"
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include "ServerInfoConsumer_impl.hpp"
#include "TestCopi_Event_impl.hpp"
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>


namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {


ChangeMsgOnStubSendReqCOPI_impl::ChangeMsgOnStubSendReqCOPI_impl
(
 std::string name,
 OsSupport::Condition * cond,
 OsSupport::Mutex * lock
 ) : m_copi_name(name),  m_cond(cond), m_lock(lock), m_failed(true)
{
};


// Implementation skeleton destructor
ChangeMsgOnStubSendReqCOPI_impl::~ChangeMsgOnStubSendReqCOPI_impl (void)
{
}


void ChangeMsgOnStubSendReqCOPI_impl::stub_send_request (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    try {                        
        std::cout << get_header() << "stub_send_request called ..." << std::endl;
    
        ::CORBA::String_var op = info->operation();
    
        if(0 == strcmp("echo", op.in())) {
            std::cout << "trying to modifying arg of echo" << std::endl;
            ::Dynamic::ParameterList_var param_list =
                  info->arguments();
    
            bool found = false;        
            const char * expected = "discorde dynamique";
    
            for(::CORBA::ULong i = 0; i < param_list->length(); ++i) {
                if(::CORBA::tk_string == param_list[i].argument.type()->kind()) {
                    const char * arg;
                    param_list[i].argument >>= arg;
    
                    std::cout << "received arg=[" << arg << "] expected was arg is [" << expected << "]" << std::endl;
                    if(0 == strcmp(arg, expected)) {
                        ::CORBA::Any any;
                        any <<= "dynamischer diskord";
                        param_list[i].argument = any; 
                    }
    
                    found = true;
                    break;
                }            
            }
    
            info->arguments(param_list.in());
        } else if(0 == strcmp("push", op.in())) {
            std::cout << "trying to modify event" << std::endl;
            ::Dynamic::ParameterList_var param_list =
                  info->arguments();
            
    
            CPPUNIT_ASSERT(1 == param_list->length());
            ::CORBA::ULong zero_idx = 0;
    
            const ::CORBA::Any * any_event = 0;
            param_list[zero_idx].argument >>= any_event;
            
    
            if(any_event && true == TestComponents::_tc_ServerInfo->equal(any_event->type()) ) {
                //if( true == TestComponents::_tc_ServerInfo->equal(param_list[zero_idx].argument.type()) ) {
                ::CORBA::ULong expected = 9;
                TestComponents::ServerInfo * arg = 0;
                if(!(*any_event >>= arg)) {
                    CPPUNIT_ASSERT(false);
                }
    
                std::cout << "ChangeMsgOnStubSendReqCOPI_impl::stub_send_request [" << arg->data() << "]" << std::endl;
    
                CPPUNIT_ASSERT(expected == arg->data());
                
                ::CORBA::Any any;
                TestComponents::ServerInfo_var si =
                    new ServerInfo_impl();
    
                std::cout << "ChangeMsgOnStubSendReqCOPI_impl::stub_send_request [" 
                          << arg->data() << "] into [23]" << std::endl;
    
                si->data(23);
    
                any <<= si.in();
    
                ::CORBA::Any hull;
                hull <<= any;
                
                param_list[zero_idx].argument = hull;            
                info->arguments(param_list.in());
            }
        }
    } catch (const CORBA::BAD_CONTEXT& ex) {
        // info->arguments() raised BAD_CONTEXT because of Disable_read_write_requests mode
        // Check this mode is set.
        CPPUNIT_ASSERT(!COPIServiceInit::Is_COPI_requests_read_write());
        std::cout << "BAD_CONTEXT exception was expected." << std::endl;
    }

}


void ChangeMsgOnStubSendReqCOPI_impl::stub_receive_reply (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException
  )
{
    std::cout << get_header() << "stub_receive_reply called ..." << std::endl;

    ::CORBA::String_var op = info->operation();

    if(0 == strcmp("push", op.in())) {
        std::cout << "trying to modify event" << std::endl;
        ::Dynamic::ParameterList_var param_list =
              info->arguments();
        

        CPPUNIT_ASSERT(1 == param_list->length());
        ::CORBA::ULong zero_idx = 0;

        const ::CORBA::Any * any_event = 0;
        param_list[zero_idx].argument >>= any_event;
        

        if(any_event && true == TestComponents::_tc_ServerInfo->equal(any_event->type()) ) {
            //if( true == TestComponents::_tc_ServerInfo->equal(param_list[zero_idx].argument.type()) ) {
            ::CORBA::ULong expected = 23;
            TestComponents::ServerInfo * arg = 0;
            if(!(*any_event >>= arg)) {
                CPPUNIT_ASSERT(false);
            }

            std::cout << "ChangeMsgOnStubSendReqCOPI_impl::stub_receive_reply [" << arg->data() << "]" << std::endl;

            CPPUNIT_ASSERT(expected == arg->data());
            
            ::CORBA::Any any;
            TestComponents::ServerInfo_var si =
                new ServerInfo_impl();

            std::cout << "ChangeMsgOnStubSendReqCOPI_impl::stub_receive_reply [" 
                      << arg->data() << "] into [42]" << std::endl;

            si->data(42);

            any <<= si.in();

            ::CORBA::Any hull;
            hull <<= any;
            
            param_list[zero_idx].argument = hull;            
            info->arguments(param_list.in());

            // client will never get aware of this, events are event,
            // i.e. meant being asynchronous, no answer this time

            {
                CDMW_MUTEX_GUARD(*m_lock);
                m_failed = false;
            }

            m_cond->broadcast();
        }
    }



}

void ChangeMsgOnStubSendReqCOPI_impl::stub_receive_exception (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    std::cout << "ChangeMsgOnStubSendReqCOPI_impl::stub_receive_exception called" << std::endl;
}

void ChangeMsgOnStubSendReqCOPI_impl::stub_receive_other (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
}

char * ChangeMsgOnStubSendReqCOPI_impl::name (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return CORBA::string_dup(m_copi_name.c_str());
}

::CORBA::UShort ChangeMsgOnStubSendReqCOPI_impl::priority (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

::Components::ContainerPortableInterceptor::IntegrationPoint * ChangeMsgOnStubSendReqCOPI_impl::registration_info (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

void ChangeMsgOnStubSendReqCOPI_impl::destroy (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  // Add your implementation here
}

void ChangeMsgOnStubSendReqCOPI_impl::set_slot_id (
    ::PortableInterceptor::SlotId slot_id
  )
  throw (
    ::CORBA::SystemException
  )
{
}

std::string ChangeMsgOnStubSendReqCOPI_impl::get_header() {
    std::stringstream sstr;
    sstr << "####E#X#T#E#N#D#E#D#### " 
         << COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), "%a, %d %b %Y %H:%M:%S %z") 
         << "\t" << m_copi_name << "\t";
    return  sstr.str();
}


} // End namespace CIF

} // End namespace CCM

} // End namespce COPI
