/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



///// FILE TO BE GENERATED

#include <Foundation/logging/FunctionLogger.hpp>   
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include "FacetHello_impl.hpp"
#include "EchoServantRequest_impl.hpp"


   
namespace Cdmw {

namespace CCM {

namespace CIF {


   // Forward declaration
   class CCMObject_impl;

   
} // end of namespace Cdmw

} // end of namespace CCM

} // end of namespace CIF

namespace Cdmw {

namespace CCM {

namespace CIF {

namespace CdmwHelloModule {


    FacetHello_impl::FacetHello_impl
    (
     Components::ExecutorLocator_ptr exec_locator, 
     const std::string& name,
     Context * ctx,
     CORBA::Object_ptr           comp_ref,
     PortableServer::Servant     component,
     CdmwLogging::Logger_ptr     logger,
     const std::string& comp_id
     ) : 
         LifeCycleObjectBase(ctx),
         Hello_impl(exec_locator, name, ctx, comp_ref, component, logger),
         m_exec_loc(exec_locator),
         m_facet_name(name),
         m_context(ctx),
         m_component_identity(comp_id),
         m_logger(CdmwLogging::Logger::_duplicate(logger))
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "");
    }


    FacetHello_impl::~FacetHello_impl() throw() 
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "");
    }

    // override echo in order to call onto
    // the extended copies
    void
    FacetHello_impl::echo
    (                 
     const char * msg
     )
        throw (                              
               CORBA::SystemException
               )
      {
          CDMW_LOG_FUNCTION(m_logger.in(), "");

          try
              {
                  CORBA::Object_var obj =
                      m_exec_loc->obtain_executor(m_facet_name.c_str());

                  if (CORBA::is_nil(obj.in()))
                      {
                          CDMW_DEBUG(m_logger.in(), 
                                     "Executor for facet " << m_facet_name<< " is null");

                          ::CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, CORBA::COMPLETED_NO);
                          CDMW_LOG_FUNCTION_EXCEPTION(ex);
                          throw ex;
                      }

                  HelloModule::CCM_Hello_var facet_exec;

                  try {                      
                      facet_exec = HelloModule::CCM_Hello::_narrow(obj.in());
                      
                  }
                  catch (::CORBA::BAD_PARAM & e)
                      {
                  
                          CDMW_ERROR(m_logger.in(), 
                                     "The facet object has wrong type " << e);

                          ::CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, CORBA::COMPLETED_NO);
                          CDMW_LOG_FUNCTION_EXCEPTION(ex);                       
                          throw ex;
                      }
                  

                  std::string feature_name =
                      this->m_facet_name;

                  ::CORBA::Object_ptr target =
                        this->m_exec_loc;


                  EchoServantRequest_impl request                      
                      (
                       feature_name.c_str(),
                       m_component_identity.c_str(),
                       target,
                       m_context
                       );

                  if (true == request.is_read_write()){   
                      request.add_in_args_msg(msg);     
                  }

                  request.pre_invoke();
       
                  /**
                   * The operation parameters
                   */
                  ::CORBA::String_var _msg = msg;

                  if (request.is_read_write()) {
                      _msg = request.get_in_args_msg();                      
                  }
        
               
                  try {

                      ::CORBA::Any_var the_exception
                            = request.the_exception();
                                            
                      if(true == request.m_proceed_call && 
                         ::CORBA::tk_null == the_exception->type()->kind()) 
                          { 
                              facet_exec->echo(_msg.in());
                          }
                  } 
                  catch(::CORBA::SystemException & se){
                      request.add_system_exception(se);
                  } 
                  // CODEGEN:
                  // catch(UserEx & ex) {
                  //    add exception to request
                  //    ::CORBA::Any any;
                  //    any <<= ex;
                  //    request->add_exception(any);
                  // }

                  request.post_invoke();
                  this->m_exec_loc->release_executor(obj.in());              

                  request.throw_exception();

                  // CODEGEN
                  /*
                  if (true == request->is_read_write) {
                  // handle result and out args
                  // __result = request->result_value();
                  }


                  return __result;
                  */

                          

              } catch (const Components::CCMException& e ) {
                  CDMW_ERROR(m_logger.in(), 
                             "CCMException received: "<< e);

                  CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
                  
                  CDMW_LOG_FUNCTION_EXCEPTION(ex);
                  throw ex;
              }                  
      }
   

                
} // end of namespace Cdmw

} // end of namespace CCM

} // end of namespace CIF

} // end of namespace CdmwTestComponents


