/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <string.h>
#include "LifeCycleInterceptorTest.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/idllib/CdmwDeployment.stub.hpp"

namespace Cdmw
{
namespace CCM
{
namespace COPI
{

LifeCycleInterceptorTest::LifeCycleInterceptorTest(std::string name) :
    m_name(name)
{

}

LifeCycleInterceptorTest::LifeCycleInterceptorTest(std::string name, CORBA::UShort priority) :
    m_name(name), m_priority(priority)
{
}

LifeCycleInterceptorTest::~LifeCycleInterceptorTest()
{
}

void LifeCycleInterceptorTest::after_component_activate(const char* compId)
        throw (CORBA::SystemException)
{
    CDMW_ASSERT(0 != compId);
    std::cout << m_name << ": after_component_activate of component " << compId << " called ..."
            << std::endl;
}

void LifeCycleInterceptorTest::after_component_connect(const char* compId)
        throw (CORBA::SystemException)
{
    CDMW_ASSERT((0 != compId));
    std::cout << m_name << ": after_component_connect of component " << compId << " called ..."
            << std::endl;
}

void LifeCycleInterceptorTest::after_component_create(const char* compId, CORBA::Object_ptr ref)
        throw (CORBA::SystemException)
{
    CDMW_ASSERT(((0 != compId) && !CORBA::is_nil(ref)));
    std::cout << m_name << ": after_component_create of component " << compId << " called ..."
            << std::endl;
}

void LifeCycleInterceptorTest::after_component_passivate(const char* compId)
        throw (CORBA::SystemException)
{
    CDMW_ASSERT((0 != compId));
    std::cout << m_name << ": after_component_passivate of component " << compId << " called ..."
            << std::endl;
}

void LifeCycleInterceptorTest::after_component_remove(const char* compId)
        throw (CORBA::SystemException)
{
    CDMW_ASSERT((0 != compId));
    std::cout << m_name << ": after_component_remove of component " << compId << " called ..."
            << std::endl;
}

void LifeCycleInterceptorTest::before_component_activate(const char* compId)
        throw (CORBA::SystemException)
{
    CDMW_ASSERT((0 != compId));
    std::cout << m_name << ": before_component_activate of component " << compId << " called ..."
            << std::endl;
}

void LifeCycleInterceptorTest::before_component_connect(const char* compId, const char* port,
        CORBA::Object_ptr ref) throw (CORBA::SystemException)
{
    CDMW_ASSERT((0 != compId));
    std::cout << m_name << ": before_component_connect of component " << compId << " called ..."
            << std::endl;
}

void LifeCycleInterceptorTest::before_component_create(const char* compId,
        Components::ConfigValues& configValues) throw (CORBA::SystemException)
{
    CDMW_ASSERT((0 != compId));
    std::cout << m_name << ": before_component_create of component " << compId << " called ..."
            << std::endl;
    std::cout << "Testing the modification of the Client component config values ..." << std::endl;

    if (m_priority == USHRT_MAX && strstr(compId, "NAME1") != 0)
    {
        for (CORBA::ULong i = 0; i < configValues.length(); i++)
        {
            if (!strcmp(CdmwDeployment::COMPONENT_NAME, configValues[i]->name()))
            {
                std::cout << "Modifying the Client Component NAME ..." << std::endl;
                CORBA::Any name;
                name <<= "CLIENT_COMPONENT_TO_REMOVE";
                configValues[i]->value(name);
                break;
            }
        }
    }
//  FIXME useless? NEW_CONFIG_VALUE_NAME is never used anywhere
//    if (m_priority == USHRT_MAX && strstr(compId, "SERVER") != 0)
//    {
//        Components::ConfigValues new_cv(configValues->length() + 1);
//        CORBA::Any val;
//        val <<= "";
//        new_cv[0] = new Container::ConfigValue_impl("NEW_CONFIG_VALUE_NAME", val);
//
//
//        std ::cout << "Adding a new ConfigValue to the Server Component ..." << std::endl;
//    }


}

void LifeCycleInterceptorTest::before_component_passivate(const char* compId)
        throw (CORBA::SystemException)
{
    CDMW_ASSERT((0 != compId));
    std::cout << m_name << ": before_component_passivate of component " << compId << " called ..."
            << std::endl;
}

void LifeCycleInterceptorTest::before_component_remove(const char* compId)
        throw (CORBA::SystemException)
{
    CDMW_ASSERT(((0 != compId) && strstr(compId, "CLIENT_COMPONENT_TO_REMOVE") != 0));
    std::cout << m_name << ": before_component_remove of component " << compId << " called ..."
            << std::endl;
}

void LifeCycleInterceptorTest::destroy() throw (::CORBA::SystemException)
{
}

char* LifeCycleInterceptorTest::name() throw (::CORBA::SystemException)
{
    return strdup(m_name.c_str());
}

CORBA::UShort LifeCycleInterceptorTest::priority() throw (::CORBA::SystemException)
{
    return m_priority;
}

Components::ContainerPortableInterceptor::IntegrationPoint*
LifeCycleInterceptorTest::registration_info() throw (::CORBA::SystemException)
{
    return 0;
}

void LifeCycleInterceptorTest::set_slot_id(::PortableInterceptor::SlotId)
        throw (::CORBA::SystemException)
{
}

} //namespace Cdmw
} //namespace CCM
} //namespace COPI

