/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include "LoggerClientCOPI_impl.hpp"
#include "COPITester_impl.hpp"
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {

LoggerClientCOPI_impl::LoggerClientCOPI_impl
(
 std::string name, 
 Cdmw::CCM::COPI::COPITester_impl * copi_tester
 ) : m_copi_name(name), m_priority(0), m_port(""), m_operation(""), m_tester(copi_tester), m_ip(0)
{
};

LoggerClientCOPI_impl::LoggerClientCOPI_impl
(
 std::string name, 
 short p, 
 std::string port, 
 std::string operation
 ) : m_copi_name(name), m_priority(p), m_port(port), m_operation(operation), m_tester(0) 
{
    m_ip = new ::Components::ContainerPortableInterceptor::IntegrationPoint();
    m_ip->port = CORBA::string_dup(m_port.c_str());
    m_ip->operation = CORBA::string_dup(m_operation.c_str());
};
 

// Implementation skeleton destructor
LoggerClientCOPI_impl::~LoggerClientCOPI_impl (void)
{
    delete m_ip;
}

void LoggerClientCOPI_impl::send_request (
    ::Components::ContainerPortableInterceptor::ContainerClientRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    CPPUNIT_ASSERT(this->m_slot_id != 0);  

    ::CORBA::OctetSeq_var origin_id = info->origin_id();
    ::CORBA::OctetSeq_var target_id = info->target_id();
    ::CORBA::String_var name = info->name();

    // TODO if debug...
    ::CORBA::String_var origin_id_str =
        origin_id->length() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(origin_id.in()) : ::CORBA::string_dup("NA");

    ::CORBA::String_var target_id_str =
        target_id->length() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(target_id.in()) : ::CORBA::string_dup("NA");

    ::CORBA::String_var empty_name = CORBA::string_dup("");

    if (0 == strcmp(empty_name.in(), name.in())) {
        name = CORBA::string_dup("NA");
    }

   std::cout  << "Component "
                 << origin_id_str << " has sent a request to component "
                 << target_id_str << " on its port " << std::endl;
  
    std::cout << get_header() 
              << "invoking send_request of request " 
              << info->request_info()->request_id()
              << " ..." << std::endl;

    std::stringstream msgstr;
    
    msgstr << get_header()
           << "The operation "
           << ::CORBA::String_var(info->request_info()->operation()) 
           << " will be sent from component "
           << origin_id_str
           << " to component "
           << target_id_str;

    std::cout << msgstr.str() << std::endl;
      

    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->request_info()->operation();

        m_tester->notify_operation
            (
             "send_request", 
             m_copi_name, 
             timestamp, 
             op.in()
             );
    }
}

void LoggerClientCOPI_impl::send_poll (
    ::Components::ContainerPortableInterceptor::ContainerClientRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException
  )
{
    std::cout << get_header() 
              << "invoking send_poll of request "
              << info->request_info()->request_id()
              << " ..." << std::endl;
}

void LoggerClientCOPI_impl::receive_reply (
    ::Components::ContainerPortableInterceptor::ContainerClientRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException
  )
{
    ::CORBA::OctetSeq_var target_id = info->target_id();

    ::CORBA::String_var target_id_str =
        target_id->length() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(target_id.in()) : ::CORBA::string_dup("NA");

    std::cout << get_header()
              << "invoking receive_reply of request "
              << info->request_info()->request_id() 
              << " ..." << std::endl;

    std::stringstream msg;
    msg << "The operation "
        << ::CORBA::String_var(info->request_info()->operation())
        << "has received a reply from component " 
        <<  target_id_str;
   
    std::cout << get_header()
              << msg.str() << std::endl;
       
    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->request_info()->operation();

        m_tester->notify_operation
            (
             "receive_reply",
             m_copi_name,
             timestamp,
             op.in()
             );
    }
}

void LoggerClientCOPI_impl::receive_exception (
    ::Components::ContainerPortableInterceptor::ContainerClientRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    std::cout << get_header()
              << "invoking receive_exception of request "
              << info->request_info()->request_id()
              << " ..." << std::endl;
    
    if(0 != m_tester) { 
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();    

        ::CORBA::String_var op = info->request_info()->operation();

        m_tester->notify_operation
            (
             "receive_exception", 
             m_copi_name,
             timestamp,
             op.in()
             );
    }
}

void LoggerClientCOPI_impl::receive_other (
    ::Components::ContainerPortableInterceptor::ContainerClientRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    std::cout << get_header() 
              << "invoking receive_other of request "
              << info->request_info()->request_id() 
              << " ..." << std::endl;
        
    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();    
        ::CORBA::String_var op = info->request_info()->operation();
        m_tester->notify_operation
            (
             "receive_other",
             m_copi_name,
             timestamp,
             op.in()
             );
    }
}

char * LoggerClientCOPI_impl::name (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return CORBA::string_dup(this->m_copi_name.c_str());
}

::CORBA::UShort LoggerClientCOPI_impl::priority (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  return this->m_priority;
}

::Components::ContainerPortableInterceptor::IntegrationPoint *
LoggerClientCOPI_impl::registration_info ()
  throw (
    ::CORBA::SystemException
  )
{
    // Add your implementation here
    return m_ip;
}

void LoggerClientCOPI_impl::destroy (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  // Add your implementation here
}

void LoggerClientCOPI_impl::set_slot_id (
    ::PortableInterceptor::SlotId slot_id
  )
  throw (
    ::CORBA::SystemException
  )
{
    this->m_slot_id = slot_id;
}


std::string LoggerClientCOPI_impl::get_header() {
    std::stringstream sstr;
    sstr << "@B@A@S@I@C@@@C@O@P@I@ " 
         << COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), "%a, %d %b %Y %H:%M:%S %z") 
         << "\t" << m_copi_name << "\t";
    return  sstr.str();
}

} // End namespace CIF

} // End namespace CCM

} // End namespce COPI
