/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include "COPITester_impl.hpp"
#include "LoggerServantCOPI_impl.hpp"

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {


LoggerServantCOPI_impl::LoggerServantCOPI_impl
(
 const std::string & name, 
 Cdmw::CCM::COPI::COPITester_impl * copi_tester
 ) : m_copi_name(name), m_tester(0)
{};


// Implementation skeleton destructor
LoggerServantCOPI_impl::~LoggerServantCOPI_impl (void)
{
}

void LoggerServantCOPI_impl::servant_receive_request (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    CPPUNIT_ASSERT(this->m_slot_id != 0);

    std::string origin_id =
        info->origin_id() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*(info->origin_id())) : "NA";
    std::string target_id =
        info->target_id() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*(info->target_id())) : "NA";

    
    std::cout << get_header()
              << "servant_receive_request called ..."
              << std::endl;

    std::stringstream msgstr;

    msgstr << "The operation " 
           << info->operation() << " has been received by component " 
           << target_id << " from component "
           << origin_id;

    std::cout << get_header()
              << msgstr.str()
              << std::endl;
        
    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->operation();

        m_tester->notify_operation
            (
             "servant_receive_request",
             m_copi_name,
             timestamp,
             op.in()
             );
    }
}

void LoggerServantCOPI_impl::servant_send_reply (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException
  )
{
    std::cout << get_header()
              << "servant_send_reply called ..."
              << std::endl;

    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->operation();

        m_tester->notify_operation
            (
             "servant_send_reply",
             m_copi_name,
             timestamp,
             op.in()
             );
    }        
}

void LoggerServantCOPI_impl::servant_send_exception (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    std::cout << get_header() << "servant_send_exception called ..." << std::endl;

    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->operation();

        m_tester->notify_operation
            (
             "servant_send_exception",
             m_copi_name,
             timestamp,
             op.in()
             );
    }                
}

void LoggerServantCOPI_impl::servant_send_other (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    std::cout << get_header()
              << "servant_send_other called ..."
              << std::endl;

    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->operation();

        m_tester->notify_operation
            (
             "servant_send_other",
             m_copi_name,
             timestamp,
             op.in()
             );
    }                       
}

char * LoggerServantCOPI_impl::name ()
  throw (
    ::CORBA::SystemException
  )
{
    return CORBA::string_dup(m_copi_name.c_str());
}

::CORBA::UShort LoggerServantCOPI_impl::priority (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

::Components::ContainerPortableInterceptor::IntegrationPoint *
LoggerServantCOPI_impl::registration_info (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

void LoggerServantCOPI_impl::destroy (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  // Add your implementation here
}

void LoggerServantCOPI_impl::set_slot_id (
    ::PortableInterceptor::SlotId slot_id
  )
  throw (
    ::CORBA::SystemException
  )
{
    m_slot_id = slot_id;
}


std::string LoggerServantCOPI_impl::get_header() {
    std::stringstream sstr;
    sstr << "####E#X#T#E#N#D#E#D#### " 
         << COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), "%a, %d %b %Y %H:%M:%S %z") 
         << "\t" << m_copi_name << "\t";
    return  sstr.str();
}


} // End namespace CIF

} // End namespace CCM

} // End namespce COPI
