/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/idllib/Components.skel.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include "COPITester_impl.hpp"

#ifndef CDMW_CCM_LOGGERSERVERCOPI_IMPL_HPP
#define CDMW_CCM_LOGGERSERVERCOPI_IMPL_HPP

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {

class  LoggerServerCOPI_impl
  : public virtual Components::ContainerPortableInterceptor::ServerContainerInterceptor,
    public virtual TAO_Local_RefCounted_Object
{
public:
    // Constructor 

    LoggerServerCOPI_impl(
                          const std::string & name, 
                          Cdmw::CCM::COPI::COPITester_impl * copi_tester
                          );


    LoggerServerCOPI_impl
    (
     const std::string & name, 
     const short & p, 
     const std::string & port,
     const std::string & operation
     );

  
    // Destructor 
    virtual ~LoggerServerCOPI_impl (void);
  
    virtual
    void receive_request_service_contexts (
                                           ::Components::ContainerPortableInterceptor::ContainerServerRequestInfo_ptr csi
                                           )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               );
  
    virtual
    void receive_request (
                          ::Components::ContainerPortableInterceptor::ContainerServerRequestInfo_ptr info
                          )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               );
  
    virtual
    void send_reply (
                     ::Components::ContainerPortableInterceptor::ContainerServerRequestInfo_ptr info
                     )
        throw (
               ::CORBA::SystemException
               );
  
    virtual
    void send_exception (
                         ::Components::ContainerPortableInterceptor::ContainerServerRequestInfo_ptr info
                         )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               );
  
    virtual
    void send_other (
                     ::Components::ContainerPortableInterceptor::ContainerServerRequestInfo_ptr info
                     )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               );
  
    virtual
    char * name (
      
    )
        throw (
               ::CORBA::SystemException
               );
  
    virtual
    ::CORBA::UShort priority (
      
    )
        throw (
               ::CORBA::SystemException
               );
  
    virtual
    ::Components::ContainerPortableInterceptor::IntegrationPoint * registration_info (
      
    )
        throw (
               ::CORBA::SystemException
               );
  
    virtual
    void destroy (
      
    )
        throw (
               ::CORBA::SystemException
               );
  
    virtual
    void set_slot_id (
                      ::PortableInterceptor::SlotId slot_id
                      )
        throw (
               ::CORBA::SystemException
               );

    friend
    std::ostream& operator<<(std::ostream& output, const LoggerServerCOPI_impl & ci) {
        output << ci.m_copi_name << "-" 
               << ci.m_priority << "-" 
               << ci.m_ip->port << "/" 
               << ci.m_ip->operation;
        return output;
    }


private:
    LoggerServerCOPI_impl& operator=(const LoggerServerCOPI_impl & other);
    LoggerServerCOPI_impl(const LoggerServerCOPI_impl & other);

    std::string get_header();     
    std::string m_copi_name;

    const short m_priority;
    const std::string m_port;
    const std::string m_operation;
    int m_slot_id;  
    COPITester_impl * m_tester;
    ::Components::ContainerPortableInterceptor::IntegrationPoint * m_ip;
};


} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

#endif // CDMW_CCM_LOGGERSERVERCOPI_IMPL_HPP

