/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/PortableInterceptor.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.stub.hpp>
#include <Foundation/common/NotFoundException.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <Foundation/common/AlreadyExistException.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <map>

#include "MyCCM2Context_impl.hpp"


namespace Cdmw {

namespace CCM {

namespace CIF {


MyCCM2Context_impl::MyCCM2Context_impl 
(
 ::CORBA::ORB_ptr orb,
 ::PortableServer::POA_ptr poa,
 ::CdmwCcmCopi::COPICurrent_ptr copi_current
)
    : m_orb(::CORBA::ORB::_duplicate(orb)),
    m_poa(::PortableServer::POA::_duplicate(poa)),
    m_copi_current(::CdmwCcmCopi::COPICurrent::_duplicate(copi_current))
{    
}

// Implementation skeleton destructor
MyCCM2Context_impl::~MyCCM2Context_impl (void)
{
}

char * MyCCM2Context_impl::server_id (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

::CosEventChannelAdmin::EventChannel_ptr MyCCM2Context_impl::obtain_event_channel_for_emitter (
    ::Components::PortDescription * port
  )
  throw (
    ::CORBA::SystemException,
    ::CdmwCcmContainer::NoChannel
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

::CosEventChannelAdmin::EventChannel_ptr MyCCM2Context_impl::create_event_channel_for_publisher (
    ::Components::PortDescription * port
  )
  throw (
    ::CORBA::SystemException,
    ::Components::CreateFailure
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

::CORBA::Object_ptr MyCCM2Context_impl::resolve_initial_references (
    const char * identifier
  )
  throw (
    ::CORBA::SystemException,
    ::CdmwCcmContainer::CCM2Context::InvalidName
  )
{
    if (0 == strcmp(identifier, "PICurrent")) {

        ::CORBA::Object_var obj =
              m_orb->resolve_initial_references("PICurrent");
            
        ::PortableInterceptor::Current_var pi_current =
              ::PortableInterceptor::Current::_narrow(obj.in());

        return pi_current._retn();
          
    } else if (0 == strcmp(identifier, ::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID.c_str())) {
        return (::CdmwCcmCopi::COPICurrent::_duplicate(m_copi_current));
    } else {
        std::cout << "ERROR: MyContext_impl just supports PICurrent and " 
                  << ::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID << " as args" << std::endl;
        throw CORBA::BAD_PARAM();
    }
}

::CORBA::Policy_ptr MyCCM2Context_impl::create_policy (
    ::CORBA::PolicyType type,
    const ::CORBA::Any & value
  )
  throw (
    ::CORBA::SystemException,
    ::CORBA::PolicyError
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

::CORBA::OctetSeq * MyCCM2Context_impl::extract_oid_from_ref (
    ::CORBA::Object_ptr objref
  )
  throw (
    ::CORBA::SystemException,
    ::Components::BadComponentReference
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

::Components::Principal_ptr MyCCM2Context_impl::get_caller_principal (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

::Components::CCMHome_ptr MyCCM2Context_impl::get_CCM_home (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

::CORBA::Boolean MyCCM2Context_impl::get_rollback_only (
    
  )
  throw (
    ::CORBA::SystemException,
    ::Components::IllegalState
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

::Components::Transaction::UserTransaction_ptr MyCCM2Context_impl::get_user_transaction (
    
  )
  throw (
    ::CORBA::SystemException,
    ::Components::IllegalState
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

::CORBA::Boolean MyCCM2Context_impl::is_caller_in_role (
    const char * role
  )
  throw (
    ::CORBA::SystemException
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

void MyCCM2Context_impl::set_rollback_only (
    
  )
  throw (
    ::CORBA::SystemException,
    ::Components::IllegalState
  )
{
  throw ::CORBA::NO_IMPLEMENT();
}

::CORBA::Object_ptr MyCCM2Context_impl::resolve_service_reference (
    const char * service_id
  )
  throw (
    ::CORBA::SystemException,
    ::Components::CCMException
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

::Components::HomeRegistration_ptr MyCCM2Context_impl::get_home_registration (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

void MyCCM2Context_impl::req_passivate (
    
  )
  throw (
    ::CORBA::SystemException,
    ::Components::PolicyMismatch
  )
{
  throw ::CORBA::NO_IMPLEMENT();
}

::Components::CatalogBase_ptr MyCCM2Context_impl::get_persistence (
    const char * catalog_type_id
  )
  throw (
    ::CORBA::SystemException,
    ::Components::PersistenceNotAvailable
  )
{
  throw ::CORBA::NO_IMPLEMENT(); return 0;
}

} // namespace CIF 
} // end namespace Cdmw 
} // end namespace CCM 

