/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_CIF_MYCCM2CONTEXTIMPL_HPP
#define INCL_CCM_CIF_MYCCM2CONTEXTIMPL_HPP

#include <Foundation/common/Assert.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.stub.hpp>
#include <Foundation/common/NotFoundException.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <Foundation/common/AlreadyExistException.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>
#include <CCMContainer/ccmcif/Context.hpp>
#include <CCMContainer/ccmcopi/COPICurrent_impl.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <map>


namespace Cdmw {

  namespace CCM {

    namespace CIF {


class  MyCCM2Context_impl
  : public virtual CdmwCcmContainer::CCM2Context,
    public virtual TAO_Local_RefCounted_Object
{
public:
  // Constructor 
  MyCCM2Context_impl 
  (
   ::CORBA::ORB_ptr orb,
   ::PortableServer::POA_ptr poa,
   ::CdmwCcmCopi::COPICurrent_ptr copi_current
   );

  
  // Destructor 
  virtual ~MyCCM2Context_impl (void);
  
  virtual
  char * server_id (
      
    )
    throw (
      ::CORBA::SystemException
    );
  
  virtual
  ::CosEventChannelAdmin::EventChannel_ptr obtain_event_channel_for_emitter (
      ::Components::PortDescription * port
    )
    throw (
      ::CORBA::SystemException,
      ::CdmwCcmContainer::NoChannel
    );
  
  virtual
  ::CosEventChannelAdmin::EventChannel_ptr create_event_channel_for_publisher (
      ::Components::PortDescription * port
    )
    throw (
      ::CORBA::SystemException,
      ::Components::CreateFailure
    );
  
  virtual
  ::CORBA::Object_ptr resolve_initial_references (
      const char * identifier
    )
    throw (
      ::CORBA::SystemException,
      ::CdmwCcmContainer::CCM2Context::InvalidName
    );
  
  virtual
  ::CORBA::Policy_ptr create_policy (
      ::CORBA::PolicyType type,
      const ::CORBA::Any & value
    )
    throw (
      ::CORBA::SystemException,
      ::CORBA::PolicyError
    );
  
  virtual
  ::CORBA::OctetSeq * extract_oid_from_ref (
      ::CORBA::Object_ptr objref
    )
    throw (
      ::CORBA::SystemException,
      ::Components::BadComponentReference
    );
  
  virtual
  ::Components::Principal_ptr get_caller_principal (
      
    )
    throw (
      ::CORBA::SystemException
    );
  
  virtual
  ::Components::CCMHome_ptr get_CCM_home (
      
    )
    throw (
      ::CORBA::SystemException
    );
  
  virtual
  ::CORBA::Boolean get_rollback_only (
      
    )
    throw (
      ::CORBA::SystemException,
      ::Components::IllegalState
    );
  
  virtual
  ::Components::Transaction::UserTransaction_ptr get_user_transaction (
      
    )
    throw (
      ::CORBA::SystemException,
      ::Components::IllegalState
    );
  
  virtual
  ::CORBA::Boolean is_caller_in_role (
      const char * role
    )
    throw (
      ::CORBA::SystemException
    );
  
  virtual
  void set_rollback_only (
      
    )
    throw (
      ::CORBA::SystemException,
      ::Components::IllegalState
    );
  
  virtual
  ::CORBA::Object_ptr resolve_service_reference (
      const char * service_id
    )
    throw (
      ::CORBA::SystemException,
      ::Components::CCMException
    );
  
  virtual
  ::Components::HomeRegistration_ptr get_home_registration (
      
    )
    throw (
      ::CORBA::SystemException
    );
  
  virtual
  void req_passivate (
      
    )
    throw (
      ::CORBA::SystemException,
      ::Components::PolicyMismatch
    );
  
  virtual
  ::Components::CatalogBase_ptr get_persistence (
      const char * catalog_type_id
    )
    throw (
      ::CORBA::SystemException,
      ::Components::PersistenceNotAvailable
    );

    private:
        ::CORBA::ORB_var m_orb;
        ::PortableServer::POA_var m_poa;
        ::CdmwCcmCopi::COPICurrent_var m_copi_current;

};

  
    } // namespace CIF 
  } // end namespace Cdmw 
} // end namespace CCM 


#endif // INCL_CCM_CIF_MYCCM2CONTEXTIMPL_HPP
