/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/common/Assert.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.stub.hpp>
#include <Foundation/common/NotFoundException.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <Foundation/common/AlreadyExistException.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcopi/COPICurrent_impl.hpp>
#include <map>

#include "MyContext_impl.hpp"


namespace Cdmw {

namespace CCM {

namespace CIF {

    
    MyContext_impl::MyContext_impl
    (
     ::PortableServer::POA_ptr poa,
     ::CdmwCcmContainer::CCM2Context_ptr ctx
     ) 
        : Context(ctx), 
          m_poa(::PortableServer::POA::_duplicate(poa)),
          m_ccm2_ctx(::CdmwCcmContainer::CCM2Context::_duplicate(ctx))
          {}

    
   

    MyContext_impl::~MyContext_impl() 
    {
        // empty
    }

      
    //
    // IDL:thalesgroup.com/CdmwCcmContainer/CCM2MyContext_impl/resolve_initial_references:1.0
    //
    CORBA::Object_ptr MyContext_impl::resolve_initial_references(const char* identifier)
        throw (CdmwCcmContainer::CCM2Context::InvalidName,
               CORBA::SystemException)
    {
        return m_ccm2_ctx->resolve_initial_references(identifier);
    }
        


    CORBA::Object_ptr MyContext_impl::create_ref(const std::string& rep_id) 
        throw (CORBA::SystemException)
    {
        throw ::CORBA::NO_IMPLEMENT();
        return 0;
    }
          

    CORBA::Object_ptr MyContext_impl::create_ref_from_oid_str
    (
     const std::string& oid,
     const std::string& rep_id
     ) 
        throw (CORBA::SystemException)
    {
        throw ::CORBA::NO_IMPLEMENT();
        return 0;
    }
        
    char * MyContext_impl::get_oid_str_from_ref(const CORBA::Object_ptr comp_ref) 
          throw (CORBA::SystemException)
    {
        std::cout << ".......Ups get_oid_str_from_ref not implemented" << std::endl;            
        throw ::CORBA::NO_IMPLEMENT();
        return 0;
    }


    char *
    MyContext_impl::extract_oid_str_from_ref(const CORBA::Object_ptr comp_ref)
        throw (CORBA::SystemException)
    {
  /*      std::cout << "**WARN: overriding extract_oid_str_from_ref" << std::endl;
        if(true)
            return ::CORBA::string_dup("Hello");
*/

        try {
            PortableServer::ObjectId_var oid = m_poa->reference_to_id(comp_ref);


            const CORBA::ULong seqmax = oid->maximum();
            const CORBA::ULong seqlen = oid->length();

            CORBA::OctetSeq_var oseq =
                new CORBA::OctetSeq(seqmax, seqlen,
                                    oid->get_buffer(true),
                                    true);

            ::CORBA::String_var comp_oid_str =
                  ::Cdmw::CCM::Common::OctetSeq_to_string(oseq.in());

            return comp_oid_str._retn();

        } catch (::PortableServer::POA::WrongAdapter & wa) {
            std::cout << "WrongAdapter during extract_oid_str_from_ref " << wa << std::endl;
            throw;       
        } catch (CORBA::SystemException & se) {
            std::cout << "SysExec during extract_oid_str_from_refi " << se << std::endl;
            throw;
        } catch (...) {
            std::cout << "Exc during extract_oid_str_from_ref" << std::endl;
            throw;
        }


        return 0;
    }

    Components::PrimaryKeyBase* MyContext_impl::get_primary_key() 
        throw(Components::NoKeyAvailable, CORBA::SystemException) 
    {
        throw ::CORBA::NO_IMPLEMENT();
        return 0;
    }

  
    void MyContext_impl::req_passivate()
        throw (CORBA::SystemException)
    {
        throw ::CORBA::NO_IMPLEMENT();
    }

   
 
    CORBA::Object_ptr MyContext_impl::get_CCM_object()
        throw (CORBA::SystemException)
    {
        throw ::CORBA::NO_IMPLEMENT();
    }           

} // namespace CIF 
} // end namespace Cdmw 
} // end namespace CCM 


