/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_CIF_MYCONTEXT_HPP
#define INCL_CCM_CIF_MYCONTEXT_HPP

#include <Foundation/common/Assert.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.stub.hpp>
#include <Foundation/common/NotFoundException.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <Foundation/common/AlreadyExistException.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>
#include <CCMContainer/ccmcif/Context.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <map>


namespace Cdmw {

  namespace CCM {

    namespace CIF {
   
      /*
       * This class is the base class for ccmcif context embedding specific CCM 
       * context.
       */
      class MyContext_impl : public virtual Cdmw::CCM::CIF::Context
      {
      public:

        MyContext_impl
        (
         ::PortableServer::POA_ptr poa,
         ::CdmwCcmContainer::CCM2Context_ptr ctx
         ); 
        

          /**
         * Purpose:
         * <p> Destructor.
         */
        virtual ~MyContext_impl(); 

        
      
        //
        // IDL:thalesgroup.com/CdmwCcmContainer/CCM2MyContext_impl/resolve_initial_references:1.0
        //
        virtual CORBA::Object_ptr resolve_initial_references(const char* identifier)
          throw (CdmwCcmContainer::CCM2Context::InvalidName,
                 CORBA::SystemException);


        virtual CORBA::Object_ptr create_ref(const std::string& rep_id) 
          throw (CORBA::SystemException);

        virtual CORBA::Object_ptr create_ref_from_oid_str(
                                                          const std::string& oid,
                                                          const std::string& rep_id)
          throw (CORBA::SystemException);

        virtual char* get_oid_str_from_ref(const CORBA::Object_ptr comp_ref) 
            throw (CORBA::SystemException);

        virtual char * extract_oid_str_from_ref(const CORBA::Object_ptr comp_ref)
          throw (CORBA::SystemException);

   
        
        virtual Components::PrimaryKeyBase* get_primary_key() 
            throw(Components::NoKeyAvailable, CORBA::SystemException);
   

        virtual void req_passivate()
          throw (CORBA::SystemException);
   
 
        virtual CORBA::Object_ptr get_CCM_object()
          throw (CORBA::SystemException);
 
          
      private:
        MyContext_impl(const MyContext_impl&);
        void operator=(const MyContext_impl&);

          ::PortableServer::POA_var m_poa;
          ::CdmwCcmContainer::CCM2Context_var m_ccm2_ctx;

      }; // end class MyContext_impl

    } // namespace CIF 
  } // end namespace Cdmw 
} // end namespace CCM 

#endif // INCL_CCM_CIF_MYCONTEXT_HPP
