/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "tao/CodecFactory/CodecFactory.h"
#include "tao/Codeset/Codeset.h"
#include "TestCopi_BasicCopi_impl.hpp"
#include "EmptyCOPIs.hpp"
#include "LoggerClientCOPI_impl.hpp"
#include "LoggerServerCOPI_impl.hpp"
#include "CCMContainer/ccmcopi/COPIRegistration_impl.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.skel.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <sstream>
#include "Hello_impl.hpp"
#include "CCMContainer/ccmcopi/COPICurrent_impl.hpp"
#include "tao/PI/PICurrent.h"

namespace Cdmw {

namespace CCM {

namespace COPI {

class Server_thread : public Cdmw::OsSupport::Thread
{
public:
    Server_thread(CORBA::ORB_ptr orb)
        : m_orb(orb),  m_is_stopped(false), m_server_ready(false)
    {
        m_cond = new OsSupport::Condition(m_lock);
    }

    virtual ~Server_thread() throw() {}      
      
    virtual void run() throw()
    {
        
        CORBA::Object_var poa_object =
            m_orb->resolve_initial_references ("RootPOA");

        PortableServer::POA_var poa =
            PortableServer::POA::_narrow (poa_object.in ());


        PortableServer::POAManager_var poa_manager =
            poa->the_POAManager ();

        poa_manager->activate ();

        HelloModule::Hello_impl servant; 
        m_echoServer = servant._this();

        {
            CDMW_MUTEX_GUARD(m_lock); 
            m_server_ready = true;            
        }

        m_cond->broadcast();

        m_orb->run();
    }

    void stop_thread() {        
        CDMW_MUTEX_GUARD(m_lock);
        m_orb->destroy();
        this->m_is_stopped = true;       
        delete m_cond;

    }

    HelloModule::Hello_ptr get_echoServer() {
        CDMW_MUTEX_GUARD(m_lock);
        while(m_server_ready == false) {
            m_cond->wait();
        }

        return m_echoServer;
    }
   
private:
    HelloModule::Hello_ptr m_echoServer; 
    CORBA::ORB_ptr m_orb;
    bool m_is_stopped;
    bool m_server_ready;
    OsSupport::Mutex m_lock;
    OsSupport::Condition * m_cond;

    PortableServer::POA_ptr m_poa;
};



/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestCopi_BasicCopi_impl );



TestCopi_BasicCopi_impl::TestCopi_BasicCopi_impl ()
    throw()
{
    std::cout << "TestCopi_BasicCopi_impl: Enter in TestCopi_BasicCopi_impl()." << std::endl;
}


TestCopi_BasicCopi_impl::~TestCopi_BasicCopi_impl()
    throw()
{
    // does nothing
}


TestCopi_BasicCopi_impl::TestCopi_BasicCopi_impl(const TestCopi_BasicCopi_impl& rfu)
   throw()
{
   // does nothing
}



void TestCopi_BasicCopi_impl::test_BasicCopi_impl()
{
    TEST_INFO("Testing COPIUtils_impl");
    bool failed = false;

    std::cout << "*************************************************************" << std::endl
              << "       BasicCopi Test" << std::endl
              << "*************************************************************" << std::endl;

    CPPUNIT_ASSERT(::Cdmw::CCM::COPI::COPIServiceInit::Is_initialized() == true);
    CORBA::ORB_ptr orb =
        Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();

    ::CdmwCcmCopi::COPICurrent_ptr copi_current =
          ::CdmwCcmCopi::COPICurrent::_narrow
          (orb->resolve_initial_references(::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID.c_str()));

    // activate a Seravant
    Server_thread * thr = new Server_thread(orb);
    /*
    std::auto_ptr<Server_thread> 
        thr(new Server_thread(orb));
    */
    thr->start();
    HelloModule::Hello_ptr echoServer =
        thr->get_echoServer();

    COPITester_impl * copi_tester =
        new COPITester_impl
        (
         copi_current->get_client_interceptor_registration(),
         copi_current->get_server_interceptor_registration(),
         copi_current->get_stub_interceptor_registration(),
         copi_current->get_servant_interceptor_registration()
         );

    /////////////////////////////////////////////////////////////////////////
    // interception ordinary client
    /////////////////////////////////////////////////////////////////////////

    std::cout << "@@@@@@@@@ Testing BasicCopy - interception ordinary client / server call" << std ::endl;    
    // COPIs must not interfer with ordinary client server calls, i.e. when there is no
    // orgin id target id and so the call must succeed

    copi_tester->register_interceptor("CLIENT_IC_0", ::TestService::CLIENT);
    copi_tester->register_interceptor("CLIENT_IC_1", ::TestService::CLIENT);
    copi_tester->register_interceptor("CLIENT_IC_2", ::TestService::CLIENT);
    copi_tester->register_interceptor("CLIENT_IC_3", ::TestService::CLIENT);
        
    copi_tester->register_interceptor("SERVER_IC_0", ::TestService::SERVER);
    copi_tester->register_interceptor("SERVER_IC_1", ::TestService::SERVER);
    copi_tester->register_interceptor("SERVER_IC_2", ::TestService::SERVER);
    copi_tester->register_interceptor("SERVER_IC_3", ::TestService::SERVER);

    copi_tester->start_listening ();

    
    try {
        echoServer->echo("eris");
    } catch (CORBA::SystemException & ex) {       
        std::cout << "Caught during echoServer->doEcho(\"eris\"): " << ex << std::endl;
        failed = true;
    }

    copi_tester->end_listening ();
    copi_tester->test_interception_stacks ();

    copi_tester->unregister_interceptor("CLIENT_IC_0");
    copi_tester->unregister_interceptor("CLIENT_IC_1");
    copi_tester->unregister_interceptor("CLIENT_IC_2");
    copi_tester->unregister_interceptor("CLIENT_IC_3");

    copi_tester->unregister_interceptor("SERVER_IC_0");
    copi_tester->unregister_interceptor("SERVER_IC_1");
    copi_tester->unregister_interceptor("SERVER_IC_2");
    copi_tester->unregister_interceptor("SERVER_IC_3");



    /////////////////////////////////////////////////////////////////////////
    // simulating call between CCM components
    /////////////////////////////////////////////////////////////////////////
    std::cout << "@@@@@@@@@ Testing BasicCopy - simulating call between CCM components" << std ::endl;    

    // 


    // set originId / target_id

    // Access copi current in to obtain the right slot ids ,
    // which is created within the ORBInitilizer
    // further down the line copi current is used to obtain a reference to the 
    // COPI registration which in turn is needed in order to install basic copis
              

    ::PortableInterceptor::SlotId origin_id_slot_id =
          copi_current->get_slot_id (COPIServiceInit::M_origin_ID_SlotNAME.c_str());

    ::PortableInterceptor::SlotId target_id_slot_id =
          copi_current->get_slot_id (COPIServiceInit::M_target_ID_SlotNAME.c_str());

    ::PortableInterceptor::SlotId origin_feature_name_slot_id =
          copi_current->get_slot_id (COPIServiceInit::M_originFeatureName_SlotNAME.c_str());

    ::PortableInterceptor::SlotId custom_item_slot_id =
          copi_current->get_slot_id (COPIServiceInit::M_originCustomSlotItemSeq_SlotNAME.c_str());


    // create the origin_id / target_id values
    CORBA::OctetSeq * origin_id =
        ::Cdmw::CCM::Common::string_to_OctetSeq("my_origin_id");

    CORBA::OctetSeq * target_id =
        ::Cdmw::CCM::Common::string_to_OctetSeq("my_target_id");


   
    // set origin_id / target_id within PICurrent
    CORBA::Object_var obj = orb->resolve_initial_references("PICurrent");
    ::PortableInterceptor::Current_var pi_current
        = PortableInterceptor::Current::_narrow(obj.in());

    CORBA::Any origin_id_slot_data;
    origin_id_slot_data <<= origin_id;

    CORBA::Any target_id_slot_data;
    target_id_slot_data <<= target_id;

    CORBA::Any origin_feature_name_slot_data;
    origin_feature_name_slot_data <<= "";


    // custom slot item
    ::Components::ContainerPortableInterceptor::CustomSlotItem csi0;
    //::Components::ContainerPortableInterceptor::CustomSlotItem csi1;

    csi0.identifier = CORBA::string_dup("eris");


    CORBA::Any a1;
    a1 <<= "eris_value";

    csi0.content = a1;

    ::Components::ContainerPortableInterceptor::CustomSlotItemSeq csi_seq;
    csi_seq.length(1);
    csi_seq[0] = csi0;

    CORBA::Any custom_item_slot_data;
    custom_item_slot_data <<= csi_seq;

    pi_current->set_slot(origin_id_slot_id, origin_id_slot_data);
    pi_current->set_slot(target_id_slot_id, target_id_slot_data);
    pi_current->set_slot(origin_feature_name_slot_id, origin_feature_name_slot_data);
    pi_current->set_slot(custom_item_slot_id, custom_item_slot_data);


    copi_tester->register_interceptor("CLIENT_IC_0", ::TestService::CLIENT);
    copi_tester->register_interceptor("CLIENT_IC_1", ::TestService::CLIENT);
    copi_tester->register_interceptor("CLIENT_IC_2", ::TestService::CLIENT);
    copi_tester->register_interceptor("CLIENT_IC_3", ::TestService::CLIENT);
        
    copi_tester->register_interceptor("SERVER_IC_0", ::TestService::SERVER);
    copi_tester->register_interceptor("SERVER_IC_1", ::TestService::SERVER);
    copi_tester->register_interceptor("SERVER_IC_2", ::TestService::SERVER);
    copi_tester->register_interceptor("SERVER_IC_3", ::TestService::SERVER);

    copi_tester->start_listening ();

    
    try {
        echoServer->echo("eris");
    } catch (CORBA::SystemException & ex) {       
        std::cout << "Caught during echoServer->doEcho(\"eris\"): " << ex << std::endl;
        failed = true;
    }

    copi_tester->end_listening ();
    copi_tester->test_interception_stacks ();

    copi_tester->unregister_interceptor("CLIENT_IC_0");
    copi_tester->unregister_interceptor("CLIENT_IC_1");
    copi_tester->unregister_interceptor("CLIENT_IC_2");
    copi_tester->unregister_interceptor("CLIENT_IC_3");

    copi_tester->unregister_interceptor("SERVER_IC_0");
    copi_tester->unregister_interceptor("SERVER_IC_1");
    copi_tester->unregister_interceptor("SERVER_IC_2");
    copi_tester->unregister_interceptor("SERVER_IC_3");
    

    std::cout << "sleep 2s" << std::endl;
    int timescale = Cdmw::TestUtils::get_timescale();
    Cdmw::OsSupport::OS::sleep(timescale*2000);
    std::cout << "stopping thread" << std::endl;

    thr->stop_thread();
    thr->join();

    // clean up
    copi_tester->start_listening ();
        
    copi_tester->_remove_ref();

    if(failed)
        CPPUNIT_ASSERT(0);

    
    std::cout << "*************************************************************" << std::endl
              << "       End BasicCopi Test" << std::endl
              << "*************************************************************" << std::endl;        
}


        
} // End namespace COPI
} // End namespace CCM
} // End namespace Cdmw


