/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "tao/AnyTypeCode/Any_SystemException.h"
#include "tao/DynamicInterface/DII_CORBA_methods.h"
#include "tao/AnyTypeCode/ExceptionA.h"
#include "tao/AnyTypeCode/SystemExceptionA.h"
#include "tao/AnyTypeCode/AnyTypeCode_methods.h"
#include "tao/DynamicInterface/Request.h"
#include "tao/DynamicInterface/Unknown_User_Exception.h"



#include "tao/DynamicInterface/Request.h"
#include "tao/CodecFactory/CodecFactory.h"
#include "tao/Codeset/Codeset.h"
#include "TestCopi_Event_impl.hpp"
#include "EmptyCOPIs.hpp"
#include "LoggerClientCOPI_impl.hpp"
#include "LoggerServerCOPI_impl.hpp"
#include "CCMContainer/ccmcopi/COPIRegistration_impl.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.skel.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "CCMContainer/ccmcontainer/ServiceRegistry.hpp"
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <sstream>
#include "CCMContainer/ccmcopi/COPICurrent_impl.hpp"
#include "tao/PI/PICurrent.h"
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>


#include "Hello_impl.hpp"
#include "MyExecuterLocator_impl.hpp"
#include "MyContext_impl.hpp"
#include "MyCCM2Context_impl.hpp"


#include <CCMContainer/ccmcif/CCMUtils.hpp>
#include "ServerInfoConsumer_impl.hpp"
#include "ChangeMsgOnStubSendReqCOPI_impl.hpp"
#include "ChangeMsgOnServantRecvReqCOPI_impl.hpp"
#include "CCM_Server_SessionContext_impl.hpp"
#include "MyCCMObject_impl.hpp"

namespace Cdmw {

namespace CCM {

namespace COPI {



class Server_thread : public Cdmw::OsSupport::Thread
{
public:
    Server_thread(CORBA::ORB_ptr orb, ::PortableServer::POA_ptr poa)
        : 
        m_orb(::CORBA::ORB::_duplicate(orb)),
        m_poa(::PortableServer::POA::_duplicate(poa)),
        m_is_stopped(false),
        m_server_ready(false),
        m_cond(0),
        m_exec_locator(0),
        m_ccm2_ctx(0),
        m_ctx(0),
        m_servant(0)        
    {
        m_cond = new OsSupport::Condition(m_lock);

        m_exec_locator
            = new MyExecuterLocator_impl();

        ::CdmwCcmCopi::COPICurrent_var copi_current =
              ::CdmwCcmCopi::COPICurrent::_narrow
              (m_orb->resolve_initial_references(::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID.c_str()));

        m_ccm2_ctx
            = new ::Cdmw::CCM::CIF::MyCCM2Context_impl
            (
             m_orb,
             m_poa.in(), 
             copi_current.in()
             );

        m_ctx
            = new ::Cdmw::CCM::CIF::MyContext_impl
            (
             m_poa.in(), 
             m_ccm2_ctx
             );
                                
        CdmwLogging::LoggerFactory_var logger_factory =
            Cdmw::CCM::CIF::CCMUtils::Create_LoggerFactory("MyLoggerFactory");

        m_logger =
            logger_factory->get_logger("MyLogger");

        // Need ) a dummy servant which is the 
        // component, that ctor needs smth. to 
        // call add ref on...
        HelloModule::Hello_impl * servant 
            = new HelloModule::Hello_impl();

        m_servant = 
            new Cdmw::CCM::CIF::CdmwTestComponents::ServerInfoConsumer_impl
            (
             m_exec_locator,
             "ServerInfoConsumer_sink_name",
             "ServerInfoConsumer_component_identity",
             m_ctx, 
             servant,
             m_logger
             );       
       
    }

    virtual ~Server_thread() throw() 
    {
        try {
            delete m_cond;
            m_cond = 0;
            delete m_ccm2_ctx;
            m_ccm2_ctx = 0;
            m_servant->_remove_ref();
        }
        catch(...) {
            CPPUNIT_ASSERT(false);
        }
    }      
      
    virtual void run() throw()
    {       
        
        PortableServer::POAManager_var poa_manager =
            m_poa->the_POAManager ();
        poa_manager->activate ();

        try {

            // activate Facet, feed dummied up facilities
            

            PortableServer::IdAssignmentPolicy_var assignment_policy =
                m_poa->create_id_assignment_policy(PortableServer::USER_ID);

            CORBA::PolicyList policy_list;
            policy_list.length(1);
            policy_list[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignment_policy);

            ::PortableServer::POA_var consumer_poa =
                  m_poa->create_POA
                  (
                   "ConsumerPOA",
                   poa_manager.in(),
                   policy_list
                   );


            ::PortableServer::ObjectId_var user_oid =
                  ::PortableServer::string_to_ObjectId("ServerInfoConsumer");
                    
            consumer_poa->activate_object_with_id
                (
                 user_oid.in(),
                 m_servant
                 );

            ::CORBA::Object_var obj =
                  consumer_poa->id_to_reference(user_oid.in());

            m_consumer = 
                ::Cdmw_TestComponents::ServerInfoConsumer::_narrow(obj);

                                         
            {
                CDMW_MUTEX_GUARD(m_lock); 
                m_server_ready = true;            
            }
            
        }
        catch (::CORBA::SystemException & sy) {
            std::cout << "Caught SystemException during facet activation: " << sy << std::endl;
            CPPUNIT_ASSERT(false);
        }

        m_cond->broadcast();
        m_orb->run();
    }

    void stop_thread() {        
        CDMW_MUTEX_GUARD(m_lock);
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(m_orb);
        m_orb->destroy();
        this->m_is_stopped = true;       
    }

    
    Cdmw_TestComponents::ServerInfoConsumer_ptr get_ref() {
        CDMW_MUTEX_GUARD(m_lock);
        while(m_server_ready == false) {
            m_cond->wait();
        }

        return Cdmw_TestComponents::ServerInfoConsumer::_duplicate(m_consumer);
    }
    
   
private:
    Cdmw_TestComponents::ServerInfoConsumer_ptr m_consumer;
    CORBA::ORB_var m_orb;
    PortableServer::POA_var m_poa;
    bool m_is_stopped;
    bool m_server_ready;
    OsSupport::Mutex m_lock;
    OsSupport::Condition * m_cond;

    MyExecuterLocator_impl * m_exec_locator;
    ::Cdmw::CCM::CIF::MyCCM2Context_impl * m_ccm2_ctx;
    ::Cdmw::CCM::CIF::MyContext_impl * m_ctx;
    CdmwLogging::Logger_var m_logger;
    Cdmw::CCM::CIF::CdmwTestComponents::ServerInfoConsumer_impl * m_servant;

};




/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestCopi_Event_impl );



TestCopi_Event_impl::TestCopi_Event_impl ()
    throw()
{
    std::cout << "TestCopi_Event_impl: Enter in TestCopi_Event_impl()." << std::endl;
}


TestCopi_Event_impl::~TestCopi_Event_impl()
    throw()
{
    // does nothing
}


TestCopi_Event_impl::TestCopi_Event_impl(const TestCopi_Event_impl& rfu)
   throw()
{
   // does nothing
}


void myRaiser() {
    throw ::CORBA::BAD_PARAM();
}


::CORBA::SystemException * get_sysex() 
{
    ::CORBA::SystemException * heaped_sysex = 0;

    try {

      throw ::CORBA::BAD_PARAM();

    } catch (::CORBA::SystemException & se) {
        
        ::CORBA::Exception * ex = se._tao_duplicate();
        heaped_sysex 
            = ::CORBA::SystemException::_downcast(ex);
    }


    return heaped_sysex;
}


void TestCopi_Event_impl::test0()
{
    TEST_INFO("Testing ExtendedCopis Event Interception");
    bool failed = false;

    std::cout << "*************************************************************" << std::endl
              << "       ExtendedCopi Test" << std::endl
              << "*************************************************************" << std::endl;

    CPPUNIT_ASSERT(::Cdmw::CCM::COPI::COPIServiceInit::Is_initialized() == true);
    CORBA::ORB_var orb =
        Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();

    PortableServer::POA_var poa =
        Cdmw::TestUtils::CORBATestManager::instance()->get_POA();

    // register valuetype factory

    
    TestComponents::ServerInfo_init * si_factory =
        new TestComponents::ServerInfo_init();
        
    TestComponents::Activated_init * ai_factory =
        new TestComponents::Activated_init();
    
    orb->register_value_factory 
        (
         si_factory->tao_repository_id (),
         si_factory
         );

    orb->register_value_factory 
        (
         ai_factory->tao_repository_id (),
         ai_factory
         );
    
    si_factory->_remove_ref (); // release ownership
    ai_factory->_remove_ref (); // release ownership

    ::CdmwCcmCopi::COPICurrent_var copi_current =
          ::CdmwCcmCopi::COPICurrent::_narrow
          (orb->resolve_initial_references(::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID.c_str()));


    OsSupport::Mutex lock0;
    OsSupport::Condition ev_cond0(lock0);

    OsSupport::Mutex lock1;
    OsSupport::Condition ev_cond1(lock1);



    ChangeMsgOnStubSendReqCOPI_impl * stub_copi1 =
          new ChangeMsgOnStubSendReqCOPI_impl("stub1", &ev_cond0, &lock0);
    
    ChangeMsgOnServantRecvReqCOPI_impl * servant_copi1 =
        new ChangeMsgOnServantRecvReqCOPI_impl("servant1", &ev_cond1, &lock1);

    ::Components::Cookie_var ck1 =
          copi_current->get_stub_interceptor_registration()->register_stub_interceptor(stub_copi1);

    ::Components::Cookie_var ck2 =
          copi_current->get_servant_interceptor_registration()->register_servant_interceptor(servant_copi1);

    Server_thread * thr = new Server_thread(orb.in(), poa.in());

    thr->start();


    COPITester_impl * copi_tester =
        new COPITester_impl
        (
         copi_current->get_client_interceptor_registration(),
         copi_current->get_server_interceptor_registration(),
         copi_current->get_stub_interceptor_registration(),
         copi_current->get_servant_interceptor_registration()
         );


    try {

        /////////////////////////////////////////////////////////////////////////
        // call with Disable_read_write_requests 
        /////////////////////////////////////////////////////////////////////////

        std::cout << std::endl;
        std::cout << "@@@@@@@@@ Testing extended copies - facet call with Disable_read_write_requests()" 
                  << std ::endl << std ::endl;    

        ::Cdmw::CCM::COPI::COPIServiceInit::COPIServiceInit::Disable_read_write_requests();

        Cdmw::OrbSupport::CodecBase::Init(orb);

        Cdmw_TestComponents::ServerInfoConsumer_var consumer =
            thr->get_ref();

        
        ::Cdmw::CCM::CIF::MyCCM2Context_impl * ccm2_ctx
              = new ::Cdmw::CCM::CIF::MyCCM2Context_impl
              (
               orb.in(),
               poa, // ?? facet poa 
               copi_current.in()
               );

              
        ::Cdmw::CCM::CIF::MyContext_impl * ctx
              = new ::Cdmw::CCM::CIF::MyContext_impl
              (
               // thr->m_facet_poa.in(), 
               poa,
               ccm2_ctx
               );

                     
        CdmwLogging::LoggerFactory_var logger_factory =
        Cdmw::CCM::CIF::CCMUtils::Create_LoggerFactory("MyLoggerFactory");
        
        CdmwLogging::Logger_var logger =
            logger_factory->get_logger("MyLogger");
             
 
        ::Cdmw::CCM::CIF::CCMObject_impl * ccm_obj =
            new MyCCMObject_impl
            (
             "MyComponentId",
             ctx,
             "MyRepComponentId",
             0,
             0,
             logger_factory.in(),
             consumer.in()
             );



        ccm_obj->get_proxy_consumer("hatchi");

            
        Components::CCMContext_var ctw = ccm2_ctx;
        
        ::Cdmw::CCM::CIF::CdmwTestComponents::CCM_Server_SessionContext_impl * session_context =
              new ::Cdmw::CCM::CIF::CdmwTestComponents::CCM_Server_SessionContext_impl
              (
               ctw.in(),
               ccm_obj,
               logger_factory.in()
               );

                           
        TestComponents::ServerInfo_var si =
            new ServerInfo_impl();
        si->data(9);


        ::CORBA::Any event_any;
        event_any <<= si.in();

        std::cout << "pushing event..." << std::endl;              
        session_context->push_info(si.in());


        /////////////////////////////////////////////////////////////////////////
        // call with Enable_read_write_requests 
        /////////////////////////////////////////////////////////////////////////

        std::cout << std::endl;
        std::cout << "@@@@@@@@@ Testing extended copies - event interception with  Enable_read_write_requests()"
                  << std ::endl << std ::endl;
        ::Cdmw::CCM::COPI::COPIServiceInit::COPIServiceInit::Enable_read_write_requests();


        std::cout << "pushing event... waiting for correct interception" << std::endl;              
        session_context->push_info(si.in());

       

                                       
        // wait for stub_receive_reply to finish
        {
            CDMW_MUTEX_GUARD(lock0);
            while(stub_copi1->m_failed) {
                ev_cond0.wait();
            }
        }
        
        {
            // wait for servant_send_reply to 
            CDMW_MUTEX_GUARD(lock1);
            while(servant_copi1->m_failed) {
                ev_cond1.wait();
            }
        }


        copi_current->get_stub_interceptor_registration()->unregister_stub_interceptor(ck1.in());

        copi_current->get_servant_interceptor_registration()->unregister_servant_interceptor(ck2.in());

        delete stub_copi1;
        delete servant_copi1;
        session_context->_remove_ref();
        ccm_obj->_remove_ref();
        delete ctx;
        ccm2_ctx->_remove_ref();
        ccm2_ctx->_remove_ref();
                
               
    } catch (::CORBA::SystemException & ex) {
        
        ::std::cout << "Caught: " << ex << ::std::endl;
        failed = true;
    } catch (...) {
        ::std::cout << "Caught unknown exception" << ::std::endl;
        failed = true;
    }
    

    thr->stop_thread();
    thr->join();
        
    copi_tester->_remove_ref();

    

    CPPUNIT_ASSERT(failed == false);

    std::cout << "*************************************************************" << std::endl
              << "       End ExtendedCopis Event Interception Test"              << std::endl
              << "*************************************************************" << std::endl;        
}


        
} // End namespace COPI
} // End namespace CCM
} // End namespace Cdmw


