/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tao/DynamicInterface/Request.h"
#include "tao/CodecFactory/CodecFactory.h"
#include "tao/Codeset/Codeset.h"
#include "TestCopi_ExtendedCopi_impl.hpp"
#include "EmptyCOPIs.hpp"
#include "LoggerClientCOPI_impl.hpp"
#include "LoggerServerCOPI_impl.hpp"
#include "CCMContainer/ccmcopi/COPIRegistration_impl.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.skel.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "CCMContainer/ccmcontainer/ServiceRegistry_impl.hpp"
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <sstream>
#include "EchoServer_impl.hpp"
#include "basiccopitest.skel.hpp"
#include "CCMContainer/ccmcopi/COPICurrent_impl.hpp"
#include "tao/PI/PICurrent.h"
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>

#include "Cdmw_TestComponents_cif.skel.hpp"
#include "generated/testccmcopi/Hello_impl.hpp"
#include "MyExecuterLocator_impl.hpp"
#include "FacetHello_impl.hpp" 
#include "MyContext_impl.hpp"
#include "MyCCM2Context_impl.hpp"
#include "ReceptacleHello_impl.hpp"

#include <CCMContainer/ccmcif/CCMUtils.hpp>

namespace Cdmw {

namespace CCM {

namespace COPI {


class Server_thread : public Cdmw::OsSupport::Thread
{
public:
    Server_thread(CORBA::ORB_ptr orb, ::PortableServer::POA_ptr poa)
        : 
        m_orb(::CORBA::ORB::_duplicate(orb)),
        m_poa(::PortableServer::POA::_duplicate(poa)),
        m_is_stopped(false),
        m_server_ready(false),
        m_cond(0),
        m_exec_locator(0),
        m_ccm2_ctx(0),
        m_ctx(0),
        m_servant(0)        
    {
        m_cond = new OsSupport::Condition(m_lock);

        m_exec_locator
            = new MyExecuterLocator_impl();

        ::CdmwCcmCopi::COPICurrent_var copi_current =
              ::CdmwCcmCopi::COPICurrent::_narrow
              (m_orb->resolve_initial_references(::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID.c_str()));

        m_ccm2_ctx
            = new ::Cdmw::CCM::CIF::MyCCM2Context_impl
            (
             m_orb,
             m_poa.in(), 
             copi_current.in()
             );

        m_ctx
            = new ::Cdmw::CCM::CIF::MyContext_impl
            (
             m_poa.in(), 
             m_ccm2_ctx
             );
                                
        CdmwLogging::LoggerFactory_var logger_factory =
            Cdmw::CCM::CIF::CCMUtils::Create_LoggerFactory("MyLoggerFactory");

        m_logger =
            logger_factory->get_logger("MyLogger");
                     
        m_servant 
            = new Cdmw::CCM::CIF::CdmwTestComponents::FacetHello_impl
            (
             m_exec_locator, 
             "Hello",
             m_ctx,
             // CORBA::Object_ptr           comp_ref, 
             0,
             //PortableServer::Servant     component,
             0,
             //CdmwLogging::Logger_ptr     logger,
             m_logger,
                
             "Hello_id"
             );
    }

    virtual ~Server_thread() throw() 
    {
        try {
            delete m_cond;
            m_cond = 0;
            delete m_ccm2_ctx;
            m_ccm2_ctx = 0;
        }
        catch(...) {
            CPPUNIT_ASSERT(false);
        }
    }      
      
    virtual void run() throw()
    {       
        
        PortableServer::POAManager_var poa_manager =
            m_poa->the_POAManager ();
        poa_manager->activate ();

        try {

            // activate Facet, feed dummied up facilities

            PortableServer::IdAssignmentPolicy_var assignment_policy =
                m_poa->create_id_assignment_policy(PortableServer::USER_ID);

            CORBA::PolicyList policy_list;
            policy_list.length(1);
            policy_list[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignment_policy);

            ::PortableServer::POA_var facet_poa =
                  m_poa->create_POA
                  (
                   ::CORBA::string_dup("HelloFacetPOA"),
                   poa_manager.in(),
                   policy_list
                   );


            ::PortableServer::ObjectId_var user_oid =
                  ::PortableServer::string_to_ObjectId("comp0/comp1");
                    
            facet_poa->activate_object_with_id
                (
                 user_oid.in(),
                 m_servant
                 );

            ::CORBA::Object_var obj =
                  facet_poa->id_to_reference(user_oid.in());

            m_facet = 
                ::Cdmw_HelloModule::Hello::_narrow(obj);

           
            // m_facet = servant->_this();


            m_facet_poa = facet_poa;

            std::cout << m_orb->object_to_string(m_facet) << std::endl;
            
            {
                CDMW_MUTEX_GUARD(m_lock); 
                m_server_ready = true;            
            }
        }
        catch (::CORBA::SystemException & sy) {
            std::cout << "Caught SystemException during facet activation: " << sy << std::endl;
            CPPUNIT_ASSERT(false);
        }

        m_cond->broadcast();
        m_orb->run();
    }

    void stop_thread() {        
        CDMW_MUTEX_GUARD(m_lock);
        m_orb->destroy();
        this->m_is_stopped = true;       
    }

    ::Cdmw_HelloModule::Hello_ptr get_ref() {
        CDMW_MUTEX_GUARD(m_lock);
        while(m_server_ready == false) {
            m_cond->wait();
        }

        return ::Cdmw_HelloModule::Hello::_duplicate(m_facet);
    }

    const char * get_received_value() {
        CPPUNIT_ASSERT(m_exec_locator);

        HelloModule::HelloExecuter_impl * executer = 
            m_exec_locator->get_executer();

        if(executer) {
            return executer->get_received_value();
        } else {
            return 0;
        }
    }
   
private:
    ::Cdmw_HelloModule::Hello_ptr m_facet;
    CORBA::ORB_var m_orb;
    PortableServer::POA_var m_poa;
    bool m_is_stopped;
    bool m_server_ready;
    OsSupport::Mutex m_lock;
    OsSupport::Condition * m_cond;

    MyExecuterLocator_impl * m_exec_locator;
    ::Cdmw::CCM::CIF::MyCCM2Context_impl * m_ccm2_ctx;
    ::Cdmw::CCM::CIF::MyContext_impl * m_ctx;
    CdmwLogging::Logger_var m_logger;
    Cdmw::CCM::CIF::CdmwTestComponents::FacetHello_impl * m_servant;

public:    
    PortableServer::POA_var m_facet_poa;
};




/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestCopi_ExtendedCopi_impl );



TestCopi_ExtendedCopi_impl::TestCopi_ExtendedCopi_impl ()
    throw()
{
    std::cout << "TestCopi_ExtendedCopi_impl: Enter in TestCopi_ExtendedCopi_impl()." << std::endl;
}


TestCopi_ExtendedCopi_impl::~TestCopi_ExtendedCopi_impl()
    throw()
{
    // does nothing
}


TestCopi_ExtendedCopi_impl::TestCopi_ExtendedCopi_impl(const TestCopi_ExtendedCopi_impl& rfu)
   throw()
{
   // does nothing
}


void myRaiser() {
    throw ::CORBA::BAD_PARAM();
}



void TestCopi_ExtendedCopi_impl::test1()
{
    TEST_INFO("Server for extended copi test");

    std::cout << "*************************************************************" << std::endl
              << "       ExtendedCopi Test" << std::endl
              << "*************************************************************" << std::endl;

    CPPUNIT_ASSERT(::Cdmw::CCM::COPI::COPIServiceInit::Is_initialized() == true);
    CORBA::ORB_var orb =
        Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();

    PortableServer::POA_var poa =
        Cdmw::TestUtils::CORBATestManager::instance()->get_POA();


    ::CdmwCcmCopi::COPICurrent_var copi_current =
          ::CdmwCcmCopi::COPICurrent::_narrow
          (orb->resolve_initial_references(::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID.c_str()));

    // activate a Servant, i.e. the facet
    std::auto_ptr<Server_thread> 
        thr(new Server_thread(orb.in(), poa.in()));
    thr->start();

    // run forever
    thr->join();


    std::cout << "*************************************************************" << std::endl
              << "       End ExtendedCopi Test" << std::endl
              << "*************************************************************" << std::endl;        
}


        
} // End namespace COPI
} // End namespace CCM
} // End namespace Cdmw


