/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tao/DynamicInterface/Request.h"
#include "tao/CodecFactory/CodecFactory.h"
#include "tao/Codeset/Codeset.h"
#include "TestCopi_ExtendedCopi_impl.hpp"
#include "EmptyCOPIs.hpp"
#include "LoggerClientCOPI_impl.hpp"
#include "LoggerServerCOPI_impl.hpp"
#include "CCMContainer/ccmcopi/COPIRegistration_impl.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.skel.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "CCMContainer/ccmcontainer/ServiceRegistry.hpp"
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <sstream>
#include "CCMContainer/ccmcopi/COPICurrent_impl.hpp"
#include "tao/PI/PICurrent.h"
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>

#include "Cdmw_TestComponents_cif.skel.hpp"
#include "generated/testccmcopi/Hello_impl.hpp"
#include "MyExecuterLocator_impl.hpp"
#include "FacetHello_impl.hpp" 
#include "MyContext_impl.hpp"
#include "MyCCM2Context_impl.hpp"
#include "ReceptacleHello_impl.hpp"

#include <CCMContainer/ccmcif/CCMUtils.hpp>
#include "Hello_impl.hpp"
#include "ChangeMsgOnStubSendReqCOPI_impl.hpp"
#include "ChangeMsgOnServantRecvReqCOPI_impl.hpp"

namespace Cdmw {

namespace CCM {

namespace COPI {


class Server_thread : public Cdmw::OsSupport::Thread
{
public:
    Server_thread(CORBA::ORB_ptr orb, ::PortableServer::POA_ptr poa)
        : 
        m_orb(::CORBA::ORB::_duplicate(orb)),
        m_poa(::PortableServer::POA::_duplicate(poa)),
        m_is_stopped(false),
        m_server_ready(false),
        m_cond(0),
        m_exec_locator(0),
        m_ccm2_ctx(0),
        m_ctx(0),
        m_servant(0)        
    {
        m_cond = new OsSupport::Condition(m_lock);

        m_exec_locator
            = new MyExecuterLocator_impl();

        ::CdmwCcmCopi::COPICurrent_var copi_current =
              ::CdmwCcmCopi::COPICurrent::_narrow
              (m_orb->resolve_initial_references(::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID.c_str()));

        m_ccm2_ctx
            = new ::Cdmw::CCM::CIF::MyCCM2Context_impl
            (
             m_orb,
             m_poa.in(), 
             copi_current.in()
             );

        m_ctx
            = new ::Cdmw::CCM::CIF::MyContext_impl
            (
             m_poa.in(), 
             m_ccm2_ctx
             );
                                
        CdmwLogging::LoggerFactory_var logger_factory =
            Cdmw::CCM::CIF::CCMUtils::Create_LoggerFactory("MyLoggerFactory");

        m_logger =
            logger_factory->get_logger("MyLogger");
        
        // need a dummy servant which is the 
        // component, that ctor needs smth. to 
        // call add ref on...
        HelloModule::Hello_impl * servant 
            = new HelloModule::Hello_impl();

        m_servant 
            = new Cdmw::CCM::CIF::CdmwHelloModule::FacetHello_impl
            (
             m_exec_locator, 
             "Hello",
             m_ctx,
             // CORBA::Object_ptr           comp_ref, 
             0,
             //PortableServer::Servant     component,
             servant,
             //CdmwLogging::Logger_ptr     logger,
             m_logger,
                
             "Hello_id"
             );
    }

    virtual ~Server_thread() throw() 
    {
        try {
            delete m_cond;
            m_cond = 0;
            delete m_ccm2_ctx;
            m_ccm2_ctx = 0;
            m_servant->_remove_ref();
        }
        catch(...) {
            CPPUNIT_ASSERT(false);
        }
    }      
      
    virtual void run() throw()
    {       
        
        PortableServer::POAManager_var poa_manager =
            m_poa->the_POAManager ();
        poa_manager->activate ();

        try {

            // activate Facet, feed dummied up facilities

            PortableServer::IdAssignmentPolicy_var assignment_policy =
                m_poa->create_id_assignment_policy(PortableServer::USER_ID);

            CORBA::PolicyList policy_list;
            policy_list.length(1);
            policy_list[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignment_policy);

            ::PortableServer::POA_var facet_poa =
                  m_poa->create_POA
                  (
                   "HelloFacetPOA",
                   poa_manager.in(),
                   policy_list
                   );


            ::PortableServer::ObjectId_var user_oid =
                  ::PortableServer::string_to_ObjectId("comp0/comp1");
                    
            facet_poa->activate_object_with_id
                (
                 user_oid.in(),
                 m_servant
                 );

            ::CORBA::Object_var obj =
                  facet_poa->id_to_reference(user_oid.in());

            m_facet = 
                ::Cdmw_HelloModule::Hello::_narrow(obj);
          

            m_facet_poa = facet_poa;
            
            {
                CDMW_MUTEX_GUARD(m_lock); 
                m_server_ready = true;            
            }
        }
        catch (::CORBA::SystemException & sy) {
            std::cout << "Caught SystemException during facet activation: " << sy << std::endl;
            CPPUNIT_ASSERT(false);
        }

        m_cond->broadcast();
        m_orb->run();
    }

    void stop_thread() {        
        CDMW_MUTEX_GUARD(m_lock);
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(m_orb);
        m_orb->destroy();
        this->m_is_stopped = true;       
    }

    ::Cdmw_HelloModule::Hello_ptr get_ref() {
        CDMW_MUTEX_GUARD(m_lock);
        while(m_server_ready == false) {
            m_cond->wait();
        }

        return ::Cdmw_HelloModule::Hello::_duplicate(m_facet);
    }

    HelloModule::HelloExecuter_impl *
    get_executer() {
        return m_exec_locator->get_executer();
    }

    const char * get_received_value() {
        CPPUNIT_ASSERT(m_exec_locator);

        HelloModule::HelloExecuter_impl * executer = 
            m_exec_locator->get_executer();

        if(executer) {
            return executer->get_received_value();
        } else {
            return 0;
        }
    }



   
private:
    ::Cdmw_HelloModule::Hello_ptr m_facet;
    CORBA::ORB_var m_orb;
    PortableServer::POA_var m_poa;
    bool m_is_stopped;
    bool m_server_ready;
    OsSupport::Mutex m_lock;
    OsSupport::Condition * m_cond;

    MyExecuterLocator_impl * m_exec_locator;
    ::Cdmw::CCM::CIF::MyCCM2Context_impl * m_ccm2_ctx;
    ::Cdmw::CCM::CIF::MyContext_impl * m_ctx;
    CdmwLogging::Logger_var m_logger;
    Cdmw::CCM::CIF::CdmwHelloModule::FacetHello_impl * m_servant;

public:    
    PortableServer::POA_var m_facet_poa;
};




/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestCopi_ExtendedCopi_impl );



TestCopi_ExtendedCopi_impl::TestCopi_ExtendedCopi_impl ()
    throw()
{
    std::cout << "TestCopi_ExtendedCopi_impl: Enter in TestCopi_ExtendedCopi_impl()." << std::endl;
}


TestCopi_ExtendedCopi_impl::~TestCopi_ExtendedCopi_impl()
    throw()
{
    // does nothing
}


TestCopi_ExtendedCopi_impl::TestCopi_ExtendedCopi_impl(const TestCopi_ExtendedCopi_impl& rfu)
   throw()
{
   // does nothing
}


void myRaiser() {
    throw ::CORBA::BAD_PARAM();
}


::CORBA::SystemException * get_sysex() 
{
    ::CORBA::SystemException * heaped_sysex = 0;

    try {

      throw ::CORBA::BAD_PARAM();

    } catch (::CORBA::SystemException & se) {
        
        ::CORBA::Exception * ex = se._tao_duplicate();
        heaped_sysex 
            = ::CORBA::SystemException::_downcast(ex);
    }


    return heaped_sysex;
}

/**
 * Test the _tao_duplicate() which is used 
 * by the Extended COPI implementation.
 */
void TestCopi_ExtendedCopi_impl::test0() {

    ::CORBA::SystemException * se =
          get_sysex();

    std::cout << std::endl;
    std::cout << "*************************************************************" << std::endl
              << "      _tao_duplicate() Test" << std::endl
              << "*************************************************************" << std::endl;

    bool passed = false;

    try {
        se->_raise();
        CPPUNIT_ASSERT(false);
    } catch (::CORBA::BAD_PARAM & bp) {
        passed = true;
    } catch (...) {
        CPPUNIT_ASSERT(false);
    }
   
    delete se;

    CPPUNIT_ASSERT(true == passed);

    std::cout << "*************************************************************" << std::endl
              << "       End _tao_duplicate() Test" << std::endl
              << "*************************************************************" << std::endl;        
}


/**
 * Scenario: Test of stub/servant interception through extended copis with activated as well
 * as deactiavted read write request. 
 *
 * Steps:
 *  Activate a Facet object. 
 *  Pass facet reference to the corresponding recaptacle object.
 *  Call onto recaptacle with deactiaved read write
 *  Register Stub Interceptor which will change op arg 
 *  Register Servant Interceptor which will change op arg 
 *  Call onto recaptacle with activated read write
 *  
 *  NOTE: TAO 1.5 has problems with dynamic invocation when server / client
 *  are process collocated. Run this test using -ORBCollocation no
 */
void TestCopi_ExtendedCopi_impl::test1()
{
    TEST_INFO("Testing ExtendedCopis");
    bool failed = false;

    std::cout << "*************************************************************" << std::endl
              << "       ExtendedCopi Test" << std::endl
              << "*************************************************************" << std::endl;

    CPPUNIT_ASSERT(::Cdmw::CCM::COPI::COPIServiceInit::Is_initialized() == true);
    CORBA::ORB_var orb =
        Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();

    PortableServer::POA_var poa =
        Cdmw::TestUtils::CORBATestManager::instance()->get_POA();


    ::CdmwCcmCopi::COPICurrent_var copi_current =
          ::CdmwCcmCopi::COPICurrent::_narrow
          (orb->resolve_initial_references(::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID.c_str()));

    // activate a Servant, i.e. the facet

    Server_thread * thr =
        new Server_thread(orb.in(), poa.in());

    thr->start();


    COPITester_impl * copi_tester =
        new COPITester_impl
        (
         copi_current->get_client_interceptor_registration(),
         copi_current->get_server_interceptor_registration(),
         copi_current->get_stub_interceptor_registration(),
         copi_current->get_servant_interceptor_registration()
         );

   
    // obtain reference to facet 
    ::Cdmw_HelloModule::Hello_var hello =  thr->get_ref();


    ::CORBA::String_var msg
          = ::CORBA::string_dup("eris");


    try {

        /////////////////////////////////////////////////////////////////////////
        // call with Disable_read_write_requests 
        /////////////////////////////////////////////////////////////////////////

        std::cout << std::endl;
        std::cout << "@@@@@@@@@ Testing extended copies - facet call with Disable_read_write_requests()" << std ::endl;    

        ::Cdmw::CCM::COPI::COPIServiceInit::COPIServiceInit::Disable_read_write_requests();


        ::CdmwCcmCopi::COPICurrent_var copi_current =
              ::CdmwCcmCopi::COPICurrent::_narrow
              (orb->resolve_initial_references(::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID.c_str()));


        ::Cdmw::CCM::CIF::MyCCM2Context_impl * ccm2_ctx
              = new ::Cdmw::CCM::CIF::MyCCM2Context_impl
              (
               orb.in(),
               poa, // ?? facet poa 
               copi_current.in()
               );



        ::Cdmw::CCM::CIF::MyContext_impl * ctx
              = new ::Cdmw::CCM::CIF::MyContext_impl
              (
               // thr->m_facet_poa.in(), 
               poa,
               ccm2_ctx
               );

        CdmwLogging::LoggerFactory_var logger_factory =
            Cdmw::CCM::CIF::CCMUtils::Create_LoggerFactory("MyLoggerFactory");
        
        CdmwLogging::Logger_var logger =
            logger_factory->get_logger("MyLogger");
        
        
        // call hello on the Facet using the reseptacle 
        // -> Reseptacle will be calling stub copies, facet Servant copis, respectivelly
        ::Cdmw::CCM::CIF::CdmwHelloModule::ReceptacleHello_impl * receptacle_hello =
              new ::Cdmw::CCM::CIF::CdmwHelloModule::ReceptacleHello_impl
              (
               "ReceptacleHello",
               "ReceptacleHello_component_id",
               hello,
               ctx,
               logger.in()
               );


        receptacle_hello->echo(msg.in());

        // value received by executer which was
        // eventually invoked by the facet
        const char * received_value =
            thr->get_received_value();

        CPPUNIT_ASSERT(0 != received_value);
        CPPUNIT_ASSERT(0 == strcmp(received_value, msg.in()));


        /////////////////////////////////////////////////////////////////////////
        // call with Enable_read_write_requests 
        /////////////////////////////////////////////////////////////////////////

        std::cout << std::endl;
        std::cout << "@@@@@@@@@ Testing extended copies - facet call with Enable_read_write_requests()" << std ::endl;    
        ::Cdmw::CCM::COPI::COPIServiceInit::COPIServiceInit::Enable_read_write_requests();

        
        ::CORBA::String_var dyn_msg =
              ::CORBA::string_dup("discorde dynamique");

        // install stub copi which should chnage the arg

        ChangeMsgOnStubSendReqCOPI_impl * stub_copi1 =
            new ChangeMsgOnStubSendReqCOPI_impl("stub1", 0, 0);

        ChangeMsgOnServantRecvReqCOPI_impl * servant_copi1 =
            new ChangeMsgOnServantRecvReqCOPI_impl("servant1", 0, 0);


        ::Components::Cookie_var stub_ck =
              copi_current->get_stub_interceptor_registration()->register_stub_interceptor(stub_copi1);
        ::Components::Cookie_var servant_ck =
              copi_current->get_servant_interceptor_registration()->register_servant_interceptor(servant_copi1);

        receptacle_hello->echo(dyn_msg.in());

        // value received by executer which was
        // eventually invoked by the facet
         received_value =
            thr->get_received_value();

        // COPI should have changed the message, i.e. translated it :)
        CPPUNIT_ASSERT(0 != received_value);
        CPPUNIT_ASSERT(0 == strcmp(received_value, "dynamic discord"));


        std::cout << std::endl;
        std::cout << "@@@@@@@@@ Testing extended copies - facet call with Disable_read_write_requests() - Executer throws BAD_OPERATION" << std ::endl;    
        ::Cdmw::CCM::COPI::COPIServiceInit::COPIServiceInit::Disable_read_write_requests();

        thr->get_executer()->throw_on_echo(true);
        bool ex_raised = false;

        try {
            receptacle_hello->echo(dyn_msg.in());
        } catch (::CORBA::BAD_OPERATION & bo) {
            ex_raised = true;
        }
        
        CPPUNIT_ASSERT(ex_raised == true);

        std::cout << std::endl;
        std::cout << "@@@@@@@@@ Testing extended copies - facet call with Enable_read_write_requests() - Executer throws BAD_OPERATION" << std ::endl;    
        ::Cdmw::CCM::COPI::COPIServiceInit::COPIServiceInit::Enable_read_write_requests();

        thr->get_executer()->throw_on_echo(true);
        ex_raised = false;

        try {
            receptacle_hello->echo(dyn_msg.in());
        } catch (::CORBA::BAD_OPERATION & bo) {
            ex_raised = true;
        }
        
        CPPUNIT_ASSERT(ex_raised == true);


        copi_current->get_stub_interceptor_registration()->unregister_stub_interceptor(stub_ck.in());
        copi_current->get_servant_interceptor_registration()->unregister_servant_interceptor(servant_ck.in());



        thr->get_executer()->throw_on_echo(false);
        
        delete receptacle_hello;
        delete stub_copi1;
        delete servant_copi1;
        delete ctx;
        delete ccm2_ctx;

        
               
    } catch (::CORBA::SystemException & ex) {
        
        ::std::cout << "Caught: " << ex << ::std::endl;
        failed = true;
    } catch (...) {
        ::std::cout << "Caught unknown exception" << ::std::endl;
        failed = true;
    }
    

    thr->stop_thread();
    thr->join();
        
    copi_tester->_remove_ref();

    CPPUNIT_ASSERT(failed == false);



    std::cout << "*************************************************************" << std::endl
              << "       End ExtendedCopi Test" << std::endl
              << "*************************************************************" << std::endl;        
}


        
} // End namespace COPI
} // End namespace CCM
} // End namespace Cdmw


