/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "tao/CodecFactory/CodecFactory.h"
#include "tao/Codeset/Codeset.h"
#include "TestCopi_impl.hpp"
#include "EmptyCOPIs.hpp"
#include "LoggerClientCOPI_impl.hpp"
#include "CCMContainer/ccmcopi/COPIRegistration_impl.hpp"
#include "CCMContainer/ccmcopi/COPIUtils_impl.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.skel.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "CCMContainer/ccmcontainer/ServiceRegistry.hpp"
#include <Foundation/commonsvcs/time/Util.hpp>
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <sstream>

namespace Cdmw {

namespace CCM {

namespace COPI {


/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestCopi_impl );

const char * tf = "%a, %d %b %Y %H:%M:%S %z";

class COPIs_usage_simulation_thread : public Cdmw::OsSupport::Thread
{
public:
    COPIs_usage_simulation_thread(::CdmwCcmCopi::COPIRegistrar_ptr copiRegistration,
                                  OsSupport::Condition * wait_for_addref)
        : m_copiRegistration(copiRegistration), m_wait_for_addref(wait_for_addref), m_is_stopped(false)
    {}

    virtual ~COPIs_usage_simulation_thread() throw() {}      
      
    virtual void run() throw()
    {
        ::CdmwCcmCopi::ClientContainerInterceptorSeq_var interceptors;
        int timescale = Cdmw::TestUtils::get_timescale();
        const unsigned int DELAY = 2000 ;
        bool is_stopped = true;


        do {

            {
                CDMW_MUTEX_GUARD(m_lock);          
                is_stopped = m_is_stopped;
            }

            
            ::Components::ContainerPortableInterceptor::IntegrationPoint ip;
            ip.port = ::CORBA::string_dup("");
            ip.operation = ::CORBA::string_dup("");           
            interceptors = m_copiRegistration->getClientInterceptorByIPOrderedByPriority(ip);
                
            // the COPIs are used in a request processing
            for (unsigned int i = 0; i < interceptors->length(); ++i) {
                m_copiRegistration->addClientCOPIRef((*interceptors)[i]);
                std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]" 
                          <<  "\t Interceptor " << (i + 1) << " use count ++" << std::endl;

                m_copiRegistration->addClientCOPIRef((*interceptors)[i]);
                std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]" 
                          <<  "\t Interceptor " << (i + 1) << " use count ++" << std::endl;

                m_copiRegistration->addClientCOPIRef((*interceptors)[i]);
                std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]" 
                          <<  "\t Interceptor " << (i + 1) << " use count ++" << std::endl;
            }
           
            // wake up main thread
            m_wait_for_addref->broadcast();

            std::cout << std::endl;
              

            for (int i = interceptors->length() - 1; i > -1; --i) {
                m_copiRegistration->removeClientCOPIRef((*interceptors)[i]);
                std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                          <<  "\t Interceptor " << (i + 1) << " use count --" << std::endl;
                Cdmw::OsSupport::OS::sleep(timescale * DELAY);
            }

            interceptors = m_copiRegistration->getClientInterceptorByIPOrderedByPriority(ip);

                
            for (int i = interceptors->length() - 1; i > -1; --i) {
                m_copiRegistration->removeClientCOPIRef((*interceptors)[i]);
                std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                          <<  "\t Interceptor " << (i + 1) << " use count --" << std::endl;
                Cdmw::OsSupport::OS::sleep(timescale * DELAY);                                    
            }


            interceptors = m_copiRegistration->getClientInterceptorByIPOrderedByPriority(ip);
                
            for (int i = interceptors->length() - 1; i > -1; --i) {
                m_copiRegistration->removeClientCOPIRef((*interceptors)[i]);
                std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                          <<  "\t Interceptor " << (i + 1) << " use count --" << std::endl;
                Cdmw::OsSupport::OS::sleep(timescale * DELAY);
            }
               
            // the COPIs are no more used in a request processing
        } while (is_stopped == false);

    }

    void stop_thread() {        
        CDMW_MUTEX_GUARD(m_lock);
        this->m_is_stopped = true;
    }
   
private:
    ::CdmwCcmCopi::COPIRegistrar_ptr m_copiRegistration;
    OsSupport::Condition * m_wait_for_addref;

    bool m_is_stopped;
    OsSupport::Mutex m_lock;


};




TestCopi_impl::TestCopi_impl ()
    throw()
{
    std::cout << "TestCopi_impl: Enter in TestCopi_impl()." << std::endl;
}


TestCopi_impl::~TestCopi_impl()
    throw()
{
    // does nothing
}


TestCopi_impl::TestCopi_impl(const TestCopi_impl& rfu)
   throw()
{
   // does nothing
}

// checks whether a given sequence of copies has exactly the same elements as
// the vector of copis
bool copiseq_contains_copis(::CdmwCcmCopi::ClientContainerInterceptorSeq * interceptors,
                            std::vector< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr > & copiVec) {


    std::map< Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr, unsigned int >
        expectedOcc;

    for(unsigned int i = 0; i < interceptors->length(); ++i) {
        expectedOcc[(*interceptors)[i]] = 0;
    }  
       
    for(std::vector< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr >::iterator it = copiVec.begin();
        it != copiVec.end();
        ++it) {        
        expectedOcc[*it]++;
    }
        
    unsigned int i = 0;  
    for(std::map< Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr, unsigned int >::iterator it = expectedOcc.begin(); 
        it != expectedOcc.end(); ++i, ++it) {            
        if(it->second != 1)
            return false;
    } 

    if(i != copiVec.size())
        return false;


    return true;
}


void TestCopi_impl::test_COPIUtils_impl()
{
    TEST_INFO("Testing COPIUtils_impl");

    std::cout << "*************************************************************" << std::endl
              << "       COPIUtils_impl Test" << std::endl
              << "*************************************************************" << std::endl;


    std::cout << "@@@@@@@@@ Testing Copi_to_service_context()" << std ::endl;    

    // create a COPIServiceContext
    ::Components ::ContainerPortableInterceptor::COPIServiceContext * copi_context =
          new ::Components ::ContainerPortableInterceptor::COPIServiceContext();

    ::Components::ContainerPortableInterceptor::CustomSlotItem csi0;
    //::Components::ContainerPortableInterceptor::CustomSlotItem csi1;

    csi0.identifier = CORBA::string_dup("eris");
    CORBA::String_var str1 = CORBA::string_dup("eris_value");

    CORBA::Any a1;
    a1 <<= str1;

    csi0.content = a1;

    ::Components::ContainerPortableInterceptor::CustomSlotItemSeq csi_seq;
    csi_seq.length(1);
    csi_seq[0] = csi0;

    CORBA::OctetSeq origin_id;
    origin_id.length(1);
    origin_id[0] = 2;

    CORBA::OctetSeq target_id;
    target_id.length(1);
    target_id[0] = 3;
    
    copi_context->origin_id = origin_id;
    copi_context->target_id = target_id;
    copi_context->slot_info = csi_seq;

    IOP::Encoding_1_2 encoding_1_2;
    encoding_1_2.format = IOP::ENCODING_CDR_ENCAPS;
    encoding_1_2.major_version = 1;
    encoding_1_2.minor_version = 2;
    encoding_1_2.char_codeset = 0x00010001U;
    encoding_1_2.wchar_codeset = 0x00010109U;
    
    char ** o_argv = 0;
    int o_argc = 0;

    CORBA::ORB_var orb =
        CORBA::ORB_init (o_argc,
                         o_argv,
                         "my_orb"
                         );

    CORBA::Object_var obj =
        orb->resolve_initial_references ("CodecFactory");

    IOP::CodecFactory_var codec_factory =
        IOP::CodecFactory::_narrow (obj.in ());
    
    IOP::Codec_var codec_1_2 =
        codec_factory->create_codec_with_codesets (encoding_1_2);
    
    Cdmw::CCM::COPI::COPIUtils_impl::Set_codec(codec_1_2.in());

    
    // convert copi_context into service_context
    ::IOP::ServiceContext_var service_context =
          COPIUtils_impl::Copi_to_service_context(copi_context);

    // check whether service_context is sane
    CPPUNIT_ASSERT(service_context != 0);    
    CPPUNIT_ASSERT(service_context->context_id == COPIServiceInit::CDMW_COPI_SERVICE_CONTEXT_ID);
    CPPUNIT_ASSERT(service_context->context_data.length() > 0);
    

    std::cout << "@@@@@@@@@ Testing Service_to_copi_context()" << std ::endl;

    // convert :::IOP::ServiceContext back to copi service context
    ::Components::ContainerPortableInterceptor::COPIServiceContext_var csc_ptr =
         COPIUtils_impl::Service_to_copi_context(service_context);


    CPPUNIT_ASSERT(csc_ptr != 0);    
    CPPUNIT_ASSERT(csc_ptr->origin_id.length() == 1);
    CPPUNIT_ASSERT((int)csc_ptr->origin_id[0] == 2);        
    CPPUNIT_ASSERT(csc_ptr->target_id.length() == 1);
    CPPUNIT_ASSERT((int)csc_ptr->target_id[0] == 3);

    CPPUNIT_ASSERT(csc_ptr->slot_info.length() == 1);
    CPPUNIT_ASSERT(strcmp(csc_ptr->slot_info[0].identifier, "eris") == 0);
    {
        const char * str;
        csc_ptr->slot_info[0].content >>= str;
        CPPUNIT_ASSERT(strcmp("eris_value", str) == 0);
    }
    
    std::cout << "*************************************************************" << std::endl
              << "       End COPIUtils_impl Test" << std::endl
              << "*************************************************************" << std::endl;        
}


void TestCopi_impl::test_ServiceRegistry()
{
    TEST_INFO("Testing ServiceRegistry");

    std::cout << "*************************************************************" << std::endl
              << "       ServiceRegistry Test" << std::endl
              << "*************************************************************" << std::endl;


    CdmwCcmContainer::ServiceRegistry_ptr service_registry =
        Cdmw::CCM::Container::ServiceRegistry::get_instance();
    
    // look up smth. what is not registered
    CORBA::Object_ptr obj = service_registry->lookup("will not find this");
    CPPUNIT_ASSERT(CORBA::is_nil(obj) == true);    

    ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_var 
          s1 = new LoggerClientCOPI_impl("1", 0),
          s2 = new LoggerClientCOPI_impl("2", 0),
          s3 = new LoggerClientCOPI_impl("3", 0),
          s4 = new LoggerClientCOPI_impl("4", 0);


    // bind an object
    std::cout << "@@@@@@@@@ Testing bind()" << std::endl;
    service_registry->bind("s1", s1);
    
    // bind smth. under an id where smth. has
    // has been bound already
    bool exRaised = false;
    try {
        service_registry->bind("s1", s1);
    } catch (Components::CCMException & ex) {
        exRaised = true;
    }
    CPPUNIT_ASSERT(exRaised == true);   

    // check whether bind succeeded
    obj = service_registry->lookup("s1");
    CPPUNIT_ASSERT(obj == s1);

    // check rebind
    std::cout << "@@@@@@@@@ Testing rebind()" << std ::endl;
    service_registry->rebind("s1", s2);
    service_registry->rebind("s1", s2);
    obj = service_registry->lookup("s1");
    CPPUNIT_ASSERT(obj == s2);

    // check unbind
    std::cout << "@@@@@@@@@ Testing unbind()" << std ::endl;
    service_registry->unbind("s1");
    obj = service_registry->lookup("s1");
    CPPUNIT_ASSERT(CORBA::is_nil(obj) == true);

    // check multiple bounds
    std::cout << "@@@@@@@@@ Testing multiple bind()s" << std ::endl;
    service_registry->bind("s1", s1);
    service_registry->bind("s2", s2);
    service_registry->bind("s3", s3);
    service_registry->bind("s4", s4);
    obj = service_registry->lookup("s1");
    CPPUNIT_ASSERT(obj == s1);
    obj = service_registry->lookup("s2");
    CPPUNIT_ASSERT(obj == s2);
    obj = service_registry->lookup("s3");
    CPPUNIT_ASSERT(obj == s3);
    obj = service_registry->lookup("s4");
    CPPUNIT_ASSERT(obj == s4);

    // occurence counter
    std::map<std::string, unsigned short> occC; 
    occC["s1"] = 0;
    occC["s2"] = 0;
    occC["s3"] = 0;
    occC["s4"] = 0;

    // Check list 
    std::cout << "@@@@@@@@@ Testing list()" << std ::endl;
    ::CdmwCcmContainer::ServicesIds_var serviceIds =
          service_registry->list();
    CPPUNIT_ASSERT(serviceIds->length() == 4);
    for(unsigned int i = 0; i<serviceIds->length(); ++i) {
        
        occC[ (*serviceIds)[i].in() ]++;
    }

    int counter = 0;
    for(std::map<std::string, unsigned short>::iterator it = occC.begin();
        it != occC.end();
        ++it, ++counter) {
        CPPUNIT_ASSERT( occC[it->first] == 1);
    }

    CPPUNIT_ASSERT(counter == 4);
    
   
    //check clear
    std::cout << "@@@@@@@@@ Testing clear()" << std ::endl;
    service_registry->clear();

    serviceIds =
          service_registry->list();
    CPPUNIT_ASSERT(serviceIds->length() == 0); 
    

    std::cout << "@@@@@@@@@ Testing instance_destroy()" << std ::endl;
    
    Cdmw::CCM::Container::ServiceRegistry::destroy_instance();    
    Cdmw::CCM::Container::ServiceRegistry::destroy_instance();    



    std::cout << "*************************************************************" << std::endl
              << "       End COPIRegistration_impl Test" << std::endl
              << "*************************************************************" << std::endl;        
}

void TestCopi_impl::test_COPIRegistration_impl()
{
    std::cout << std::endl << std::endl << std::endl;
    std::cout << "TestCopi_impl: Enter in TestCopi_imPL::Test_COPIRegistration_impl()." << std::endl;
   
    TEST_INFO("Testing COPIRegistration_impl");

    std::cout << "*************************************************************" << std::endl
              << "       COPIRegistration_impl Test" << std::endl
              << "*************************************************************" << std::endl;

    try {
        COPIRegistration_impl::Initialise(2,3);
            
        ::CdmwCcmCopi::COPIRegistrar_var copiRegistration = 
            COPIRegistration_impl::Get_Instance();
    
        LoggerClientCOPI_impl * clientCOPI1 = new LoggerClientCOPI_impl("1", 0);
        LoggerClientCOPI_impl * clientCOPI2 = new LoggerClientCOPI_impl("2", 0);
        LoggerClientCOPI_impl * clientCOPI3 = new LoggerClientCOPI_impl("3", 0);
        LoggerClientCOPI_impl * clientCOPI4 = new LoggerClientCOPI_impl("4", 0);

        char * ips [][2]  = {{"",""}, 
                             {"port1",""},
                             {"", "op1"},
                             {"port1","op1"},
                             {"port2","op2"},
                             {"port3",""},
                             {"", "op3"}};
        
     
        std::vector< LoggerClientCOPI_impl * > 
            clientCOPIs(56);



        std::vector< ::Components::Cookie * > cks(clientCOPIs.size());

        unsigned int k = 0;
        for(unsigned int i = 0; i < (sizeof(ips) / (2 * sizeof(char *))); i++){
            for(unsigned short j=0 ; j<4 ; j++){
                {
                    std::stringstream ostr1;
                    ostr1 << 5+j;
                    
                    clientCOPIs[k] = 
                        new LoggerClientCOPI_impl(ostr1.str(),j, ips[i][0], ips[i][1] );
                    
                    /*
                    std::cout << "clientCOPIs[" 
                              << k << "]" 
                              << " " 
                              << ips[i][0] 
                              << "/"  
                              << ips[i][1] 
                              << std::endl;
                    */
                    k++;
                }

                {
                    std::stringstream ostr1;
                    ostr1 << 5+j;

                    clientCOPIs[k] = 
                        new LoggerClientCOPI_impl(ostr1.str(),j, ips[i][0], ips[i][1] );
                    /*
                    std::cout << "clientCOPIs[" 
                              << k << "]" 
                              << " " 
                              << ips[i][0] 
                              << "/"  
                              << ips[i][1] 
                              << std::endl;
                    */

                    k++;
                }
            }

                       
        }


        std::cout << "@@@@@@@@@ Testing client COPIs registration" << std::endl;        
        ::Components::Cookie_var ck1 =
              copiRegistration->register_client_interceptor(clientCOPI1);
    
        CPPUNIT_ASSERT(ck1 != 0);
        bool exRaised = false;

        try{
            std::cout << "@@@@@@@@@ Testing duplicate COPIs registration" << std::endl;
            copiRegistration->register_client_interceptor(
                    clientCOPI1);
        } catch(CORBA::BAD_PARAM & bp){
            exRaised = true;
        }

        CPPUNIT_ASSERT(exRaised == true);

        
        ::Components::Cookie_var ck2 = copiRegistration->register_client_interceptor(
                clientCOPI2);
        CPPUNIT_ASSERT(ck2 != 0);
        ::Components::Cookie_var ck3 = copiRegistration->register_client_interceptor(
                clientCOPI3);
        CPPUNIT_ASSERT(ck3 != 0);
        ::Components::Cookie_var ck4 = copiRegistration->register_client_interceptor(
                clientCOPI4);
        CPPUNIT_ASSERT(ck4 != 0);
        {
            ::Components::ContainerPortableInterceptor::IntegrationPoint ip;
            ip.port = ::CORBA::string_dup("");
            ip.operation = ::CORBA::string_dup("");           

            ip.port = "";
            ip.operation = "";


            ::CdmwCcmCopi::ClientContainerInterceptorSeq_var
                  interceptors = copiRegistration->getClientInterceptorByIPOrderedByPriority(ip);

            CPPUNIT_ASSERT(interceptors->length() == 4);

            // make sure that we got the right interceptors back 
            // TODO: sync JavaUT
        
            std::vector< ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr >
                copiVec;

            copiVec.push_back(clientCOPI1);
            copiVec.push_back(clientCOPI2);
            copiVec.push_back(clientCOPI3);
            copiVec.push_back(clientCOPI4);
                
            CPPUNIT_ASSERT( copiseq_contains_copis(interceptors, copiVec) == true ); 

            // feed 0
            exRaised = false;
            try{
                std::cout << "@@@@@@@@@ Testing unregistration of 0" << std::endl;
                copiRegistration->unregister_client_interceptor(0);
            } catch(CORBA::BAD_PARAM & bp){
                exRaised = true;
            }
            CPPUNIT_ASSERT(exRaised == true);

            // feed cookie of wrong type

            exRaised = false;
            try{
                std::cout << "@@@@@@@@@ Testing unregistration using cookie of wrong type" << std::endl;
            
                ::Components::Cookie_var badCookie =
                      Cdmw::CCM::Common::Cookie_impl::create_cookie();            
                copiRegistration->unregister_client_interceptor( badCookie );
            
            } catch(CORBA::BAD_PARAM & bp){
                exRaised = true;
            }
            CPPUNIT_ASSERT(exRaised == true);


            std::cout << "@@@@@@@@@ Testing client COPIs unregistration" << std::endl;

            // TODO: the usage thread stuff seems to be pointless since the main thread
            // will be waiting on the first unregister, unregister will return when a

            OsSupport::Mutex mtx;
            mtx.lock();
            OsSupport::Condition wait_for_addref(mtx);
            COPIs_usage_simulation_thread * thr =
                // std::auto_ptr<COPIs_usage_simulation_thread> 
                /* thr( */ new COPIs_usage_simulation_thread(copiRegistration, &wait_for_addref) /*)*/;
            thr->start();

            wait_for_addref.wait();

            int timescale = Cdmw::TestUtils::get_timescale();
            Cdmw::OsSupport::OS::sleep(timescale*1000);

       

            std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                      << "\t Interceptor 4 will be unregistered" << std::endl;
            copiVec.clear();              
            copiVec.push_back(clientCOPI1);
            copiVec.push_back(clientCOPI2);
            copiVec.push_back(clientCOPI3);       
            ::Components::ContainerPortableInterceptor::ContainerInterceptor_ptr
                  rck = copiRegistration->unregister_client_interceptor(ck4);
            CPPUNIT_ASSERT( rck == clientCOPI4 );
            
            interceptors = copiRegistration->getClientInterceptorByIPOrderedByPriority(ip);

            CPPUNIT_ASSERT( copiseq_contains_copis(interceptors, copiVec) == true ); 
            std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                      << "\t Interceptor 4 has been unregistered" << std::endl;


            std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                      << "\t Interceptor 3 will be unregistered" << std::endl;
            copiVec.clear();
            copiVec.push_back(clientCOPI1);
            copiVec.push_back(clientCOPI2);
            rck = copiRegistration->unregister_client_interceptor(ck3);
            CPPUNIT_ASSERT( rck == clientCOPI3 );

            interceptors = copiRegistration->getClientInterceptorByIPOrderedByPriority(ip);

            CPPUNIT_ASSERT( copiseq_contains_copis(interceptors, copiVec) == true ); 
            std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                      << "\t Interceptor 3 has been unregistered" << std::endl;

            std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                      << "\t Interceptor 2 will be unregistered" << std::endl;
            copiVec.clear();
            copiVec.push_back(clientCOPI1);
            rck = copiRegistration->unregister_client_interceptor(ck2);
            CPPUNIT_ASSERT( rck == clientCOPI2 );

            interceptors = copiRegistration->getClientInterceptorByIPOrderedByPriority(ip);

            CPPUNIT_ASSERT( copiseq_contains_copis(interceptors, copiVec) == true ); 
            std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                      << "\t Interceptor 2 has been unregistered" << std::endl;


            std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                      << "\t Interceptor 1 will be unregistered" << std::endl;
            copiVec.clear();
            rck = copiRegistration->unregister_client_interceptor(ck1);
            CPPUNIT_ASSERT( rck == clientCOPI1 );

            interceptors = copiRegistration->getClientInterceptorByIPOrderedByPriority(ip);

            CPPUNIT_ASSERT( copiseq_contains_copis(interceptors, copiVec) == true ); 
            std::cout << "["<< COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), tf) << "]"
                      << "\t Interceptor 1 has been unregistered" << std::endl;

            thr->stop_thread();
            thr->join();
      
            std::cout << "@@@@@@@@@ Testing COPIs ordering by priority" << std::endl;
            for(unsigned int i=0; i < clientCOPIs.size(); ++i) {
                cks[i] = copiRegistration->register_client_interceptor
                    (dynamic_cast < ::Components::ContainerPortableInterceptor::ClientContainerInterceptor_ptr> ( clientCOPIs[i] ));
            }        

        
            for(unsigned int i = 0; i < (sizeof(ips) / (2 * sizeof(char *))); i++){

                ::Components::ContainerPortableInterceptor::IntegrationPoint ip;
                ip.port = CORBA::string_dup(ips[i][0]);
                ip.operation = CORBA::string_dup(ips[i][1]);     

                interceptors = 
                    copiRegistration->getClientInterceptorByIPOrderedByPriority(ip);
                // TODO: check whether we're getting the right number of COPIs back!
                // std::cout << "for: " << ips[i][0] << "/" << ips[i][1] << " got " << interceptors->length() << std::endl;
                unsigned long priority = 232323 ;
                for(unsigned int j=0; j < interceptors->length(); ++j){                
                    CPPUNIT_ASSERT( (*interceptors)[j]->priority() <= priority);
                    priority = (*interceptors)[j]->priority();
                }
            
            }

 
            

            // copi sequence goes out of scope
        }

       
            // unregister
            for(unsigned int i=0; i < cks.size(); ++i) {
                copiRegistration->unregister_client_interceptor
                    ( cks[i] );
            } 
            
            // delete copis
            for(unsigned int i=0; i < clientCOPIs.size(); ++i) {
                delete clientCOPIs[i];
            };


            // delete cookies
            for(unsigned int i=0; i < cks.size(); ++i) {
                ::Components::Cookie_var v = cks[i];
            };

        

        delete clientCOPI1;
        delete clientCOPI2;
        delete clientCOPI3;
        delete clientCOPI4;



        std::cout << "*************************************************************" << std::endl
                  << "       End COPIRegistration_impl Test" << std::endl
                  << "*************************************************************" << std::endl;
        
              
    } catch (const CORBA::Exception & ex) {
        std::cerr << "Caught CORBA Exception:" << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
        CPPUNIT_ASSERT(false);
    }
}
        
} // End namespace COPI
} // End namespace CCM
} // End namespace Cdmw


