/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testlci/CCM_Client_impl.hpp"
#include "testlci/Event_impl.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/Assert.hpp"


#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>
#include <fstream>

namespace 
{

// Helper function for CORBA::WString creation    
CORBA::WChar* make_wstring(const char* str)
{
    if (str == 0)
        return 0;
    
    CORBA::ULong length = strlen(str);
    CORBA::WString_var ret = CORBA::wstring_alloc(length);
    for (CORBA::ULong i=0; i < length; i++)
    {
        ret[i] = (CORBA::WChar) str[i];
    }
    ret[length] = (CORBA::WChar) '\0';
    
    return ret._retn();
};
    
} // End of anonymous namespace



//
// IDL:thalesgroup.com/BasicDemo/CCM_Client:1.0
//
BasicDemo::CCM_Client_impl::CCM_Client_impl()
    : m_name(""),
      m_session_context(BasicDemo::CCM_Client_Context::_nil())
{
}

BasicDemo::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
BasicDemo::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "Set_session_context called!" << std::endl;
   
    m_session_context = ::BasicDemo::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
BasicDemo::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    // get_context() performs a duplicate
    ::BasicDemo::CCM_Client_Context_var ctx = get_context();
    Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", ctx.in());
    // get_text() returns a CORBA::String
    CORBA::String_var txt = get_text();
    Cdmw::TestUtils::CORBATestManager::instance()->set_property ("text",txt.in());
    ::CommonDemo::Registration_var register_server;
    register_server = m_session_context->get_connection_to_register();
    CORBA::String_var client_name = register_server->_cxx_register();
    set_name(client_name.in());




    Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
    runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_ccm_client_activate").makeTest() );
    runner.start();
    runner.join();

    sleep(5);
    
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
BasicDemo::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
BasicDemo::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "Ccm_remove called!" << std::endl;
}


//
// IDL:thalesgroup.com/BasicDemo/CCM_Client_Executor/name:1.0
//
char*
BasicDemo::CCM_Client_impl::name()
    throw(CORBA::SystemException)
{
    char* name = CORBA::string_dup(m_name.in());
    return name;
}

//
// IDL:thalesgroup.com/BasicDemo/CCM_Client_Executor/event:1.0
//
BasicDemo::Event* 
BasicDemo::CCM_Client_impl::event()
    throw(CORBA::SystemException)
{
    BasicDemo::Event* event = new BasicDemo::Event_impl();
    event->text(m_text.in());

    return event;
}

void 
BasicDemo::CCM_Client_impl::event(BasicDemo::Event* event)
    throw(CORBA::SystemException)
{
    m_text = CORBA::string_dup(event->text());
}

