/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <string.h>
#include "LifeCycleInterceptorComponentTest.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/idllib/CdmwDeployment.stub.hpp"
#include "CCMContainer/ccmcontainer/ConfigValue_impl.hpp"
#include <Foundation/testutils/CORBATestManager.hpp>

namespace Cdmw
{
namespace CCM
{
namespace COPI
{

LifeCycleInterceptorComponentTest::LifeCycleInterceptorComponentTest(std::string name) :
    m_name(name),
    m_priority(0)
{

}

LifeCycleInterceptorComponentTest::LifeCycleInterceptorComponentTest(std::string name, CORBA::UShort priority) :
    m_name(name), m_priority(priority)
{
}

LifeCycleInterceptorComponentTest::~LifeCycleInterceptorComponentTest()
{
}

void LifeCycleInterceptorComponentTest::after_component_activate(const char* compId)
        throw (CORBA::SystemException)
{
    CPPUNIT_ASSERT(0 != compId);
    TEST_INFO(m_name << ": after_component_activate of component " << compId << " called ...");
}

void LifeCycleInterceptorComponentTest::after_component_connect(const char* compId)
        throw (CORBA::SystemException)
{
    CPPUNIT_ASSERT((0 != compId));
    TEST_INFO(m_name << ": after_component_connect of component " << compId << " called ...");
}

void LifeCycleInterceptorComponentTest::after_component_create(const char* compId, CORBA::Object_ptr ref)
        throw (CORBA::SystemException)
{
    CPPUNIT_ASSERT(((0 != compId) && !CORBA::is_nil(ref)));
    TEST_INFO( m_name << ": after_component_create of component " << compId << " called ...");
}

void LifeCycleInterceptorComponentTest::after_component_passivate(const char* compId)
        throw (CORBA::SystemException)
{
    CPPUNIT_ASSERT((0 != compId));
    TEST_INFO( m_name << ": after_component_passivate of component " << compId << " called ...");
}

void LifeCycleInterceptorComponentTest::after_component_remove(const char* compId)
        throw (CORBA::SystemException)
{
    CPPUNIT_ASSERT((0 != compId));
    TEST_INFO(m_name << ": after_component_remove of component " << compId << " called ...");
}

void LifeCycleInterceptorComponentTest::before_component_activate(const char* compId)
        throw (CORBA::SystemException)
{
    CPPUNIT_ASSERT((0 != compId));
    TEST_INFO(m_name << ": before_component_activate of component " << compId << " called ...");
}

void LifeCycleInterceptorComponentTest::before_component_connect(const char* compId, const char* port,
        CORBA::Object_ptr ref) throw (CORBA::SystemException)
{
    CPPUNIT_ASSERT((0 != compId));
    TEST_INFO(m_name << ": before_component_connect of component " << compId << " called ...");
}

void LifeCycleInterceptorComponentTest::before_component_create(const char* compId,
        Components::ConfigValues& configValues) throw (CORBA::SystemException)
{
    CPPUNIT_ASSERT((0 != compId));
    TEST_INFO(m_name << ": before_component_create of component " << compId << " called ...");

    for (CORBA::ULong i = 0; i < configValues.length(); i++)
    {
        if (strcmp("ADD_A_NEW_CV", configValues[i]->name()) == 0)
        {
            const char* comp_name = 0;
            configValues[i]->value() >>= comp_name;
            if (strcmp(comp_name, "TRUE") == 0)
            {
                std::cout << "Modifying the Server Component NAME ..." << std::endl;
                CORBA::Any name;
                name <<= "FALSE";
                configValues[i]->value(name);

                int len = configValues.length();
                configValues.length(len + 1);
                CORBA::Any value;
                value <<= "MyServerComponent";
                configValues[len] = 
                    new Cdmw::CCM::Container::ConfigValue_impl(
                    CdmwDeployment::COMPONENT_NAME,
                    value);
            }
            break;
        }
    }

    //check that COMPONENT_NAME hasn't been added twice (by two interceptor)
    int COMPONENT_NAME_count = 0;
    int ADD_A_NEW_CV_count = 0;
    for (CORBA::ULong i = 0; i < configValues.length(); i++)
    {
        if (strcmp("ADD_A_NEW_CV", configValues[i]->name()) == 0)
            ADD_A_NEW_CV_count++;
        else if  (strcmp(CdmwDeployment::COMPONENT_NAME, configValues[i]->name()) == 0)
            COMPONENT_NAME_count++;
    }    
    if (ADD_A_NEW_CV_count != 0)
        CPPUNIT_ASSERT(COMPONENT_NAME_count == 1);
}

void LifeCycleInterceptorComponentTest::before_component_passivate(const char* compId)
        throw (CORBA::SystemException)
{
    CPPUNIT_ASSERT((0 != compId));
    TEST_INFO(m_name << ": before_component_passivate of component " << compId << " called ...");
}

void LifeCycleInterceptorComponentTest::before_component_remove(const char* compId)
        throw (CORBA::SystemException)
{
    CPPUNIT_ASSERT((0 != compId));
    TEST_INFO(m_name << ": before_component_remove of component " << compId << " called ...");
}

void LifeCycleInterceptorComponentTest::destroy() throw (::CORBA::SystemException)
{
}

char* LifeCycleInterceptorComponentTest::name() throw (::CORBA::SystemException)
{
    return strdup(m_name.c_str());
}

CORBA::UShort LifeCycleInterceptorComponentTest::priority() throw (::CORBA::SystemException)
{
    return m_priority;
}

Components::ContainerPortableInterceptor::IntegrationPoint*
LifeCycleInterceptorComponentTest::registration_info() throw (::CORBA::SystemException)
{
    return 0;
}

void LifeCycleInterceptorComponentTest::set_slot_id(::PortableInterceptor::SlotId)
        throw (::CORBA::SystemException)
{
}

} //namespace Cdmw
} //namespace CCM
} //namespace COPI

