/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef LIFECYCLEINTERCEPTORTEST_HPP_
#define LIFECYCLEINTERCEPTORTEST_HPP_

#include <string>
#include <CCMContainer/idllib/CdmwCcmContainer.stub.hpp>
#include <Foundation/idllib/Components.stub.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>

namespace Cdmw
{
namespace CCM
{
namespace COPI
{

class LifeCycleInterceptorComponentTest;
typedef LifeCycleInterceptorComponentTest* LifeCycleInterceptorComponentTest_ptr;

/**
 *
 */
class LifeCycleInterceptorComponentTest: public CdmwCcmContainer::ComponentLifeCycleInterceptor,
                                public Cdmw::OrbSupport::RefCountLocalObject
{
public:

    explicit LifeCycleInterceptorComponentTest(std::string name);

    LifeCycleInterceptorComponentTest(std::string name, CORBA::UShort priority);

protected:
    virtual ~LifeCycleInterceptorComponentTest();

public:
    virtual void after_component_activate(const char* id) throw (CORBA::SystemException);
    virtual void after_component_connect(const char* id) throw (CORBA::SystemException);
    virtual void after_component_create(const char* id, CORBA::Object_ptr ref)
            throw (CORBA::SystemException);
    virtual void after_component_passivate(const char* arg0) throw (CORBA::SystemException);
    virtual void after_component_remove(const char* id) throw (CORBA::SystemException);

    virtual void before_component_activate(const char* id) throw (CORBA::SystemException);
    virtual void before_component_connect(const char* id, const char* port, CORBA::Object_ptr ref)
            throw (CORBA::SystemException);
    virtual void before_component_create(const char* id, Components::ConfigValues& configuration)
            throw (CORBA::SystemException);
    virtual void before_component_passivate(const char* id) throw (CORBA::SystemException);
    virtual void before_component_remove(const char* id) throw (CORBA::SystemException);

    void destroy() throw (::CORBA::SystemException);
    char* name() throw (::CORBA::SystemException);
    CORBA::UShort priority() throw (::CORBA::SystemException);

    Components::ContainerPortableInterceptor::IntegrationPoint* registration_info()
            throw (::CORBA::SystemException);

    void set_slot_id(::PortableInterceptor::SlotId slot_id) throw (::CORBA::SystemException);

private:
    std::string m_name;
    CORBA::UShort m_priority;
//    CORBA::Object_ptr m_orb;

};

} //namespace Cdmw
} //namespace CCM
} //namespace COPI

#endif /* LIFECYCLEINTERCEPTORTEST_HPP_ */
