/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "ClientComponent_impl.hpp"


#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/Assert.hpp"
#include <Foundation/osthreads/Condition.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>

#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>
#include <fstream>
#include <sstream>

#include "testqosenabler/UserTestService.stub.hpp"
#include "EventDataExchanger_impl.hpp"

namespace 
{
} // End of anonymous namespace

namespace TestComponents 
{
    class COPI_registration_thread : public Cdmw::OsSupport::Thread
    {
    public:
        COPI_registration_thread
        (
         ::TestService::COPITester_ptr tester,
         Cdmw::OsSupport::Condition * wait_for_unregister
         )
            : m_tester(::TestService::COPITester::_duplicate(tester)), 
              m_wait_for_unregister(wait_for_unregister),
              m_unregister(false)
        {}

        virtual ~COPI_registration_thread() throw() {}      
      
        virtual void run() throw()
        {
            for(int i=0; i<3; i++) {
                std::ostringstream out;
                out << "ClientCOPI" << i;
                m_tester->register_interceptor(out.str().c_str(), ::TestService::CLIENT);
            }

            for(int i=0; i<4; i++) {
                std::ostringstream out;
                out << "ServantCOPI" << i;
                m_tester->register_interceptor(out.str().c_str(), ::TestService::SERVANT);
            }

            for(int i=0; i<1; i++) {
                std::ostringstream out;
                out << "ServerCOPI" << i;
                m_tester->register_interceptor(out.str().c_str(), ::TestService::SERVER);
            }

            for(int i=0; i<2; i++) {
                std::ostringstream out;
                out << "StubCOPI" << i;
                m_tester->register_interceptor(out.str().c_str(), ::TestService::STUB);
            }

            try {
                
                bool temp = false;
                do {
                    ::std::cout << "@@@@@@@@@@@@@@ waiting for COPIs Unregistration request ..." << ::std::endl;
                    m_wait_for_unregister->wait();
                    ::std::cout << "@@@@@@@@@@@@@@ COPIs Unregistration will start ..." << ::std::endl;
                    {
                        CDMW_MUTEX_GUARD(m_lock);
                        temp = m_unregister;
                    }
                } while (false == temp);



                for (int i = 0; i < 4; i++) {
                    std::ostringstream out;
                    out << "ServantCOPI" << i;
                    m_tester->unregister_interceptor(out.str().c_str());
                }
            
                unsigned int timescale = 1;
                ::Cdmw::OsSupport::OS::sleep(timescale * 100);
            
                for (int i = 0; i < 2; i++) {
                    std::ostringstream out;
                    out << "StubCOPI" << i;                
                    m_tester->unregister_interceptor(out.str().c_str());
                }
            
                ::Cdmw::OsSupport::OS::sleep(timescale * 100);
            
                for (int i = 0; i < 3; i++) {
                    std::ostringstream out;
                    out << "ClientCOPI" << i;
                    m_tester->unregister_interceptor(out.str().c_str());
                }
            
                ::Cdmw::OsSupport::OS::sleep(timescale * 100);
            
                for (int i = 0; i < 1; i++) {
                    std::ostringstream out;
                    out << "ServerCOPI" << i;
                    m_tester->unregister_interceptor(out.str().c_str());
                }

            } catch (::Components::ContainerPortableInterceptor::InvalidRegistration & ir) {
                ::std::cout << __FILE__ << "(" << __LINE__ << "): "  
                            << " caught InvalidRegistration: " 
                            << ir << ::std::endl;
                throw;
            } catch (...) {
                ::std::cout << __FILE__ << "(" << __LINE__ << "): "  
                            << " caught unknown exception: "; 
                throw;
            }
        }
        

        void unregister() { 
            CDMW_MUTEX_GUARD(m_lock);
            this->m_unregister = true;
        }
   
    private:
        ::TestService::COPITester_var m_tester;
        ::Cdmw::OsSupport::Condition * m_wait_for_unregister;

        bool m_unregister;
        ::Cdmw::OsSupport::Mutex m_lock;
    };
    

    CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_client_component_impl, "test_client_component_impl");

    class test_client_component_impl : public CppUnit::TestFixture
    {
        TestComponents::ClientComponent_impl * m_client_component;

    public:
        test_client_component_impl() 
        {        
            CORBA::Object_var obj =
                Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_component");
            ::UserTestComponents::User_CCM_Client_var user_ccm_client =
                  ::UserTestComponents::User_CCM_Client::_narrow(obj.in());
            m_client_component = 
                reinterpret_cast< ::TestComponents::ClientComponent_impl * > (user_ccm_client.in());
        };


        void do_deprecated_tests()
        {
            // set number of requested successfull tests
            //      set_nbOfRequestedTestOK (4);
            
            // This was the orignial test
            // left here for pure nostalgy
    
            TEST_INFO("********************************************");
            TEST_INFO("            ccm_activate thread start");
            TEST_INFO("********************************************");
            try {
                ::std::cout << "[Client] ccm_activate called!" << ::std::endl;
                ::HelloModule::Hello_var hello = 
                      m_client_component->m_session_context->get_connection_to_echo();

                ::CORBA::String_var string_to_echo = ::CORBA::string_dup("eris");
                ::std::cout << "[Client] calling echo with [" << string_to_echo << "]"  << ::std::endl;

                hello->echo(string_to_echo.in());
                ::std::cout << "[Client] echo called" << ::std::endl;

                ::TestComponents::FacetTest_var facet_test = 
                      m_client_component->m_session_context->get_connection_to_server();
    
  
                ::std::cout << "[Client] calling makeOperation" << ::std::endl;
                CORBA::Long result = 0;
                CORBA::Long operand1 = 23;
                CORBA::Long operand2 = 24;
                CORBA::Long remainder = 0;

                result =
                    facet_test->makeOperation
                    (
                     TestComponents::ADD,
                     operand1,
                     operand2,
                     remainder
                     );

                ::std::cout << "[Client] makeOperation called" 
                            <<  "|" 
                            << operand1 << "|" 
                            << operand2 << "|" 
                            << remainder << "|" 
                            << result << ::std::endl;

                if(result == 36 && remainder == 5) {
                    ::std::cout << "MILESTONE1 (perl script will search for this string)" << ::std::endl;
                }
        

            } catch (...) {
                ::std::cout << "[ERROR]"
                            << __FILE__ << "(" << __LINE__ 
                            << ") caught unknown exception in TestComponents::ClientComponent_impl::ccm_activate"
                            << ::std::endl;

                throw;
            }


        }


        void do_tests()
        {
            
            ::std::cout << "*************************************************************" << ::std::endl;
            ::std::cout << "       ClientComponentImpl.class Test" << ::std::endl;
            ::std::cout << "*************************************************************" << ::std::endl;

            ::TestService::Logger_var logger;

            try {
                // getting the context
                CORBA::Object_var obj =
                    Cdmw::TestUtils::CORBATestManager::instance()->get_object("ClientSessionContext");
                ::TestComponents::CCM_Client_Context_var context =
                      ::TestComponents::CCM_Client_Context::_narrow(obj);

                obj =
                    ::Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_component");
                ::UserTestComponents::User_CCM_Client_var user_ccm_client =
                      ::UserTestComponents::User_CCM_Client::_narrow(obj.in());
                m_client_component =
                    reinterpret_cast< ::TestComponents::ClientComponent_impl * > (user_ccm_client.in());
                //
                CPPUNIT_ASSERT(false == ::CORBA::is_nil(context.in()));
                obj =
                    context->resolve_service_reference("LOGGING_SERVICE");
                logger =
                    ::TestService::Logger::_narrow(obj);

                CPPUNIT_ASSERT(false == ::CORBA::is_nil(logger.in()));
                logger->LOG_MESSAGE("Succeeded to get the LOGGING_SERVICE reference ...");

                obj =
                    context->resolve_service_reference("COPITester_SERVICE");
                ::TestService::COPITester_var tester =
                      ::TestService::COPITester::_narrow(obj);
                CPPUNIT_ASSERT(false == ::CORBA::is_nil(tester.in()));
                logger->LOG_MESSAGE("Succeeded to get the COPITester_SERVICE reference ...");

                ::HelloModule::Hello_var hello = 
                      context->get_connection_to_echo();
                CPPUNIT_ASSERT(false == ::CORBA::is_nil(tester.in()));

                logger->LOG_MESSAGE("Succeeded to get connection to echo facet :-)");

                ::TestComponents::FacetTest_var facet = 
                      context->get_connection_to_server();
                CPPUNIT_ASSERT(false == ::CORBA::is_nil(facet.in()));
                
                logger->LOG_MESSAGE("Succeeded to get connection to FacetTest facet :-)");

                // this thread allows parallel registration
                ::Cdmw::OsSupport::Mutex mtx;
                mtx.lock();
                ::Cdmw::OsSupport::Condition cond(mtx);
                COPI_registration_thread t(tester, &cond);
                t.start();

                tester->start_listening();

                ::std::cout << "@@@@@@@@@@@@@@ Testing Disabled ReadWrite requests" << ::std::endl;
                do_tests(logger.in(), hello.in(), facet.in());

                tester->end_listening();
                tester->test_interception_stacks();

                tester->start_listening();

                ::std::cout << "@@@@@@@@@@@@@@ Testing Enabled ReadWrite requests" << std::endl;

                Cdmw::CCM::COPI::COPIServiceInit::Enable_read_write_requests();
                CPPUNIT_ASSERT(true == Cdmw::CCM::COPI::COPIServiceInit::Is_COPI_requests_read_write());

                // notify the Registrator thread to start to unregister the COPIs
                t.unregister();
                ::std::cout << "@@@@@@@@@@@@@@ COPIs Unregistration requested ..." << ::std::endl;
                cond.broadcast();

                // ** BUG!!!
/*
                sleep(5);

                do_tests(logger.in(), hello.in(), facet.in());

                tester->end_listening();
                tester->test_interception_stacks();
*/

                t.join();

            } catch (...) {
                ::std::cout << __FILE__ << "(" << __LINE__ << "): "  
                            << " caught unknown exception: "; 
                                
                CPPUNIT_ASSERT(0);
            }

            ::std::cout << "*************************************************************" << ::std::endl;
            ::std::cout << "       End of ClientComponentImpl.class Test" << ::std::endl;
            ::std::cout << "*************************************************************" << ::std::endl;
        }
        
        void do_tests
        (
         ::TestService::Logger_ptr logger,
         ::HelloModule::Hello_ptr hello,
         ::TestComponents::FacetTest_ptr facet
         ) {

            ::std::cout << "********************************" << ::std::endl;
            ::std::cout << "Testing Operations interception" << ::std::endl;
            ::std::cout << "********************************" << ::std::endl;

            logger->LOG_MESSAGE("echoing messages ...");

            for(int i=0; i<5;i++) {
                std::ostringstream out;
                out << "Message no " << i;               
                hello->echo(out.str().c_str());
            }

            ::std::cout << "********************************" << ::std::endl;
            ::std::cout << "Testing Exception raising" << ::std::endl;
            ::std::cout << "********************************" << ::std::endl;
            bool got_the_right_ex = false;
            try{
                facet->raiseException(true, "Testing an exceptional behaviour");      
                CPPUNIT_ASSERT(0);
            }catch(TestComponents::ExceptionTest & et){
                got_the_right_ex = true;
                logger->LOG_MESSAGE("Succeded to get the right exception :-)");
            }catch(::CORBA::SystemException & ex){
                CPPUNIT_ASSERT(0);
            }catch (...) {
                CPPUNIT_ASSERT(0);
            }

            CPPUNIT_ASSERT(true == got_the_right_ex);

            ::std::cout << "********************************" << ::std::endl;            
            ::std::cout << "Testing Return and Out parameters" << ::std::endl;
            ::std::cout << "********************************" << ::std::endl;


            try {
                ::CORBA::Long rem;
                ::CORBA::Long res = facet->makeOperation(::TestComponents::DIV, 100, 3, rem);
                std::ostringstream out;
                out << "100 DIV 3 = " << res << ", remainder = " << rem;
                logger->LOG_MESSAGE(out.str().c_str());
                //CPPUNIT_ASSERT(res==33 && rem==1);
                //logger->LOG_MESSAGE("Succeded to get right return and out parameters :-)");

            } catch (::TestComponents::ExceptionTest & e) {
                ::std::cout << __FILE__ << "(" << __LINE__ << "): "  
                            << ": " 
                            << e << ::std::endl;
                CPPUNIT_ASSERT(0);
            }

        }


        CPPUNIT_TEST_SUITE( test_client_component_impl );
        //        CPPUNIT_TEST( do_deprecated_tests );
        CPPUNIT_TEST( do_tests );
        CPPUNIT_TEST_SUITE_END();

    };
};



//
// IDL:thalesgroup.com/BasicDemo/CCM_Client:1.0
//
TestComponents::ClientComponent_impl::ClientComponent_impl()
    : m_name(""),
      m_session_context(TestComponents::CCM_Client_Context::_nil())
{
    m_activation_thread =
        new ClientComponentActivationThread(this);
}

TestComponents::ClientComponent_impl::~ClientComponent_impl()
{
    std::cout << "ClientComponent_impl::~ClientComponent_impl called!" << std::endl;
}


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
TestComponents::ClientComponent_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "Set_session_context called!" << std::endl;
   
    m_session_context = ::TestComponents::CCM_Client_Context::_narrow(ctx); 

    ::Cdmw::TestUtils::CORBATestManager::instance()->add ("ClientSessionContext", this->m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
TestComponents::ClientComponent_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    m_activation_thread->start();
    m_activation_thread->join();
}



//
// Ccm_passivate function : it makes stop the component (call by the container)
//

void
TestComponents::ClientComponent_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
TestComponents::ClientComponent_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "Ccm_remove called!" << std::endl;
}


void
TestComponents::ClientComponent_impl::push_server_is_activated(TestComponents::Activated * event)
    throw (CORBA::SystemException)
{
    ::std::cout << "::ClientComponent_impl::push_server_is_activated called" << ::std::endl;
}

void
TestComponents::ClientComponent_impl::push_server_info(TestComponents::ServerInfo * event)
    throw (CORBA::SystemException)
{
    ::std::cout << "::ClientComponent_impl::push_server_info called: " << event->data() << ::std::endl;
    ::Cdmw::CCM::CIF::EventDataExchanger_impl::Instance()->set_data(event->data());
    
    /*
    if(9 == event->data()) {
        ::std::cout << "MILESTONE2 (perl script will search for this string)" << ::std::endl;
    }
    */
    
}




