/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef ___QOSENABLERTEST_CLIENT_COMPONENT_IMPL_HPP__
#define ___QOSENABLERTEST_CLIENT_COMPONENT_IMPL_HPP__

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/RefCountLocalObject.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "generated/ComponentServer/testqosenabler/UserTestComponents.skel.hpp"

namespace TestComponents
{

// forward declaration
class ClientComponent_impl;
class ClientComponentActivationThread;
class test_client_component_impl;


//
// IDL:thalesgroup.com/TestComponents/CCM_Client:1.0
//
class ClientComponent_impl : virtual public ::UserTestComponents::User_CCM_Client,
                        virtual public Cdmw::OrbSupport::RefCountLocalObject
{
    friend class TestComponents::test_client_component_impl;

    ClientComponent_impl(const ClientComponent_impl&);
    void operator=(const ClientComponent_impl&);

     CORBA::String_var                   m_name;
     CORBA::String_var                   m_text;
    ::TestComponents::CCM_Client_Context_var m_session_context;
    bool                                 m_test_result;

     
public:

    ClientComponent_impl();
    ~ClientComponent_impl();

    void set_session_context(Components::SessionContext_ptr ctx)
                 throw(CORBA::SystemException);
    void ccm_activate()
                 throw(CORBA::SystemException);
    void ccm_passivate()
                 throw(CORBA::SystemException);
    void ccm_remove()
                 throw(CORBA::SystemException);

    virtual void push_server_is_activated(TestComponents::Activated*)
        throw (CORBA::SystemException);
    virtual void push_server_info(TestComponents::ServerInfo*)
        throw (CORBA::SystemException);

private:
    ClientComponentActivationThread * m_activation_thread;


};

class ClientComponentActivationThread : public Cdmw::OsSupport::Thread 
{
public:
    ClientComponentActivationThread(::TestComponents::ClientComponent_impl * client_component)
         : m_client_component(client_component)
     {
     }

    ~ClientComponentActivationThread() throw() {};

protected:
    void
    run() throw()
     {
         Cdmw::TestUtils::CORBATestManager::instance()->add("client_component", static_cast< ::UserTestComponents::User_CCM_Client_ptr > (m_client_component));
         Cdmw::TestUtils::ThreadedTestRunner runner(Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
         runner.addTest( CppUnit::TestFactoryRegistry::getRegistry("test_client_component_impl").makeTest() );
         runner.start();
         runner.join();
     }
    
private:
    
    ClientComponentActivationThread();
    ClientComponentActivationThread(const ClientComponentActivationThread&);
    ClientComponentActivationThread& operator = (const ClientComponentActivationThread&);
    
    TestComponents::ClientComponent_impl * m_client_component;
};


}; // End of namespace TestComponents


#endif // ___QOSENABLERTEST_CLIENT_COMPONENT_IMPL_HPP__

