/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <string>
#include <string.h>
#include <vector>
#include <map>

#include "ExtensionContainerTest.hpp"
#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

#include <CCMContainer/ccmcontainer/Container_impl.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <CCMContainer/ccmcontainer/HomeAllocatorRegistry.hpp>
#include <CCMContainer/ccmcontainer/HomesServantActivator.hpp>

#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcommon/CCMSystemProperties.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include "testqosenabler/TestService_cif.skel.hpp"
#include "testqosenabler/TestComponents_cif.skel.hpp"
#include <Foundation/idllib/CdmwDeployment.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>

#include "testqosenabler/Cdmw_TestService_cif.skel.hpp"
#include "EventModifierCOPITestServantCOPI_impl.hpp"

#include "ServerInfoFactory_impl.hpp"

namespace
{


}; // End anonymous namespace

namespace Cdmw
{
namespace CCM
{
namespace CIF
{


/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/


EventModifierCOPITestServantCOPI_impl::EventModifierCOPITestServantCOPI_impl()
{
};


EventModifierCOPITestServantCOPI_impl::~EventModifierCOPITestServantCOPI_impl()
    throw()
{
    // does nothing
}

void EventModifierCOPITestServantCOPI_impl::do_modify_event_data
(
 ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
 ::CORBA::ULong new_value
 )
{
    ::CORBA::String_var op = info->operation();
    
    if(0 == strcmp("push", op.in())) {
    
        bool modified = false;
        ::Dynamic::ParameterList_var params =
              info->arguments();

        CPPUNIT_ASSERT(params->length() == 1);
        
        ::std::cout << "Modifying event data..." << ::std::endl;

        ::CORBA::Any hull
              = params[0L].argument;

        if(::TestComponents::_tc_ServerInfo->equal(hull.type())) {
            ::CORBA::Any any;
            ::TestComponents::ServerInfo_impl * server_info_impl =
                new ::TestComponents::ServerInfo_impl();
            server_info_impl->data(new_value);
            any <<= server_info_impl;
            /*
            ::CORBA::Any _hull;
            _hull <<= any;
            */
            params[0L].argument= any;
            modified = true;
        } else { CPPUNIT_ASSERT(0); } 


        if(modified == true) {            
            info->arguments(params);
            
            ::std::cout << "Successfully modified event data..." << ::std::endl;
                
        } else {
            CPPUNIT_ASSERT(0);
        }
    } else {
        CPPUNIT_ASSERT(0);
    }        

}

::CORBA::ULong 
EventModifierCOPITestServantCOPI_impl::get_event_data
(
 ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info
 ) 
{
    try {
        ::Dynamic::ParameterList_var params =
              info->arguments();
        
        CPPUNIT_ASSERT(params->length() == 1);                

        ::CORBA::Any hull
              = params[0L].argument;

        if(::TestComponents::_tc_ServerInfo->equal(hull.type())) {
            ::TestComponents::ServerInfo * server_info_impl = 0;
            hull >>= server_info_impl;
            ::CORBA::ULong data =
                  server_info_impl->data();
            return data;
        } else { CPPUNIT_ASSERT(0); }


    } catch (...) {
        CPPUNIT_ASSERT(0);
    }    

    return 0;
}


void EventModifierCOPITestServantCOPI_impl::servant_receive_request (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{

    try {
        // ReadWriteRequests is disabled via COPIServiceInit.disableReadWriteRequests()    
        if(test_case_id == -1) {                 
            ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "): " << "bad test case" << ::std::endl;
            CPPUNIT_ASSERT(0);
        } else if(test_case_id == 0) {
            // set event data to 2323

            do_modify_event_data(info, 2323);
        } else if (test_case_id == 3) {
            ::CORBA::ULong data =
                  get_event_data(info);
            if(data != 1) {
                CPPUNIT_ASSERT(0);
            }
            do_modify_event_data(info, ++data); 
        }

    } catch (...) {
        ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "): " << "Caught exception" << ::std::endl;
        CPPUNIT_ASSERT(0);
    }                    

}


void EventModifierCOPITestServantCOPI_impl::servant_send_exception (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    CPPUNIT_ASSERT(0);
}

void EventModifierCOPITestServantCOPI_impl::servant_send_other (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    CPPUNIT_ASSERT(0);    
}



void EventModifierCOPITestServantCOPI_impl::servant_send_reply (
    ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException
  )
{
    try {
        // ReadWriteRequests is disabled via COPIServiceInit.disableReadWriteRequests()    
        if(test_case_id == -1) {
            CPPUNIT_ASSERT(0);
        } else if(test_case_id == 2) {
            do_modify_event_data(info, 8);
        } else if (test_case_id == 3) {
            ::CORBA::ULong data =
                  get_event_data(info);
            if(data != 2) CPPUNIT_ASSERT(0);
            do_modify_event_data(info, ++data);
        }

    } catch (...) {
        CPPUNIT_ASSERT(0);
    }
}



char * EventModifierCOPITestServantCOPI_impl::name ()
  throw (
    ::CORBA::SystemException
  )
{
    return CORBA::string_dup(m_copi_name.c_str());
}

::CORBA::UShort EventModifierCOPITestServantCOPI_impl::priority (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

::Components::ContainerPortableInterceptor::IntegrationPoint *
EventModifierCOPITestServantCOPI_impl::registration_info (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

void EventModifierCOPITestServantCOPI_impl::destroy (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  // Add your implementation here
}

void EventModifierCOPITestServantCOPI_impl::set_slot_id (
    ::PortableInterceptor::SlotId slot_id
  )
  throw (
    ::CORBA::SystemException
  )
{
    m_slot_id = slot_id;
}


std::string EventModifierCOPITestServantCOPI_impl::get_header() {
    std::stringstream sstr;
    sstr << "####E#X#T#E#N#D#E#D#### " 
         << ::Cdmw::CCM::COPI::COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), "%a, %d %b %Y %H:%M:%S %z") 
         << "\t" << m_copi_name << "\t";
    return  sstr.str();
}

              
}; // End namespace CIF
}; // End namespace CCM
}; // End namespace Cdmw

