/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <string>
#include <string.h>
#include <vector>
#include <map>

#include "ExtensionContainerTest.hpp"
#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

#include <CCMContainer/ccmcontainer/Container_impl.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <CCMContainer/ccmcontainer/HomeAllocatorRegistry.hpp>
#include <CCMContainer/ccmcontainer/HomesServantActivator.hpp>

#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcommon/CCMSystemProperties.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include "testqosenabler/TestService_cif.skel.hpp"
#include "testqosenabler/TestComponents_cif.skel.hpp"
#include <Foundation/idllib/CdmwDeployment.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>

#include "ServerInfoFactory_impl.hpp"
#include "testqosenabler/Cdmw_TestService_cif.skel.hpp"
#include "EventModifierCOPITestStubCOPI_impl.hpp"


namespace
{


}; // End anonymous namespace

namespace Cdmw
{
namespace CCM
{
namespace CIF
{


/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

// static data


EventModifierCOPITestStubCOPI_impl::EventModifierCOPITestStubCOPI_impl()
{};


EventModifierCOPITestStubCOPI_impl::~EventModifierCOPITestStubCOPI_impl()
    throw()
{
    // does nothing
}

void EventModifierCOPITestStubCOPI_impl::do_modify_event_data
(
 ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
 ::CORBA::ULong new_value
 )
{

    ::CORBA::String_var op = info->operation();
    
    if(0 == strcmp("push", op.in())) {
    
        bool modified = false;
        ::Dynamic::ParameterList_var params =
              info->arguments();

        CPPUNIT_ASSERT(params->length() == 1);
         
        ::std::cout << "Modifying event data..." << ::std::endl;

        if (::TestComponents::_tc_ServerInfo->equal(params[0L].argument.type())) 
        {
            ::CORBA::Any any;
            ::TestComponents::ServerInfo_impl * server_info_impl =
                new ::TestComponents::ServerInfo_impl();
                     
            server_info_impl->data(new_value);
         
            any <<= server_info_impl;
            params[0L].argument=any;
            modified = true;
        } else { 
            CPPUNIT_ASSERT(0); 
        } 
        
/*        ::CORBA::Any hull
              = params[0L].argument;

        // Bug: getting the event inside an any inside an any, 
        ::std::cout << "PART_NOT_EXECUTED: unexpected nesting, making test proceed " << __FILE__ << "(" << __LINE__ << ")" << ::std::endl;
        const ::CORBA::Any * event_any = 0; 
        hull >>= event_any;
         
        if(::TestComponents::_tc_ServerInfo->equal(event_any->type())) 
        {
                     
            ::CORBA::Any any;
            ::TestComponents::ServerInfo_impl * server_info_impl =
                new ::TestComponents::ServerInfo_impl();
                     
            server_info_impl->data(new_value);
         
            any <<= server_info_impl;
                     
            ::CORBA::Any _hull;
            _hull <<= any;
            params[0L].argument= _hull;                    
         
            modified = true;
        } else { 
            CPPUNIT_ASSERT(0); 
        } 
*/

        if(modified == true) {            
                     
            info->arguments(params);
         
            ::std::cout << "Successfully modified event data..." << ::std::endl;
                
        } else {
            CPPUNIT_ASSERT(0);
        }
    } else {
        CPPUNIT_ASSERT(0);
    }        

}

::CORBA::ULong 
EventModifierCOPITestStubCOPI_impl::get_event_data(::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info) 
{
    try {
        ::Dynamic::ParameterList_var params =
              info->arguments();
        
        CPPUNIT_ASSERT(params->length() == 1);    
        
        if(::TestComponents::_tc_ServerInfo->equal(params[0L].argument.type())) {
            
            ::TestComponents::ServerInfo * server_info_impl = 0;
            params[0L].argument >>= server_info_impl;
            ::CORBA::ULong data =
                  server_info_impl->data();
            return data;
            
        } else { CPPUNIT_ASSERT(0); }
        
        
/*
        ::CORBA::Any hull
              = params[0L].argument;        

        // Bug: nesting not expected!
        ::std::cout << "PART_NOT_EXECUTED: unexpected nesting, making test proceed " << __FILE__ << "(" << __LINE__ << ")" << ::std::endl;
        const ::CORBA::Any * event_any = 0;
        hull >>= event_any;
        


        if(::TestComponents::_tc_ServerInfo->equal(event_any->type())) {

            ::TestComponents::ServerInfo * server_info_impl = 0;
            (*event_any) >>= server_info_impl;
            ::CORBA::ULong data =
                  server_info_impl->data();
            return data;
        } else { CPPUNIT_ASSERT(0); }
*/

    } catch (...) {
        CPPUNIT_ASSERT(0);
    }    

    return 0;
}



void EventModifierCOPITestStubCOPI_impl::stub_receive_exception (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    ::std::cout << get_header()
                << "invoking stub_receive_exception of request ..."
                << std::endl;

    CPPUNIT_ASSERT(0);
}

void EventModifierCOPITestStubCOPI_impl::stub_receive_other (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    ::std::cout << get_header() 
                << "invoking stub_receive_other of request ..." 
                << std::endl;
    CPPUNIT_ASSERT(0);
}

void EventModifierCOPITestStubCOPI_impl::stub_receive_reply (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException
  )
{
    ::CORBA::ULong value = 0;

    ::Dynamic::ParameterList_var params =
          info->arguments();

    if(1 == params->length()) {
        for(::CORBA::ULong i=0; i<params->length(); i++) {
            ::CORBA::Any hull =
                  params[i].argument;

            ::TestComponents::ServerInfo * server_info = 0;
            hull >>= server_info;
            if(0 != server_info) {
                value = server_info->data();
            } else {
                ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "): " << "event extraction failed" << ::std::endl;
                CPPUNIT_ASSERT(0);
            }
             
        }
    }


    if(test_case_id == -1) {
        ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "): " << "bad test case" << ::std::endl;
        CPPUNIT_ASSERT(0);
    } else if(test_case_id == 0) { 
        if(value != 2224) CPPUNIT_ASSERT(0);
    } else if(test_case_id == 1) {
        if(value != 55) CPPUNIT_ASSERT(0);
        // event modified by stub copi, i.e. server will send reply with the modification                                      
    } else if(test_case_id == 2) {
        // event modified by the server copi, but after treply was sent            
        if(value != 2224) CPPUNIT_ASSERT(0);
    } else if (test_case_id == 3) {
        ::CORBA::ULong data = get_event_data(info);
        if(data != 1) CPPUNIT_ASSERT(0);
        do_modify_event_data(info, ++data);  
    } else {
        ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "): " << "bad test case" << ::std::endl;
        CPPUNIT_ASSERT(0);
    }    
}


void EventModifierCOPITestStubCOPI_impl::stub_send_request (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{        

    if(test_case_id == -1) { 
        ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "): " << "bad test case" << ::std::endl;
        CPPUNIT_ASSERT(0);
    } 
    else if(test_case_id == 1) {
        do_modify_event_data(info, 55);             
    } else if (test_case_id == 3) {
        ::CORBA::ULong data = get_event_data(info);
        if(data != 0) CPPUNIT_ASSERT(0);
        do_modify_event_data(info, ++data);
    }
}


char * EventModifierCOPITestStubCOPI_impl::name ()
  throw (
    ::CORBA::SystemException
  )
{
    return CORBA::string_dup(m_copi_name.c_str());
}

::CORBA::UShort EventModifierCOPITestStubCOPI_impl::priority (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

::Components::ContainerPortableInterceptor::IntegrationPoint *
EventModifierCOPITestStubCOPI_impl::registration_info (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

void EventModifierCOPITestStubCOPI_impl::destroy (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  // Add your implementation here
}

void EventModifierCOPITestStubCOPI_impl::set_slot_id (
    ::PortableInterceptor::SlotId slot_id
  )
  throw (
    ::CORBA::SystemException
  )
{
    m_slot_id = slot_id;
}


std::string EventModifierCOPITestStubCOPI_impl::get_header() {
    std::stringstream sstr;
    sstr << "####E#X#T#E#N#D#E#D#### " 
         << ::Cdmw::CCM::COPI::COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), "%a, %d %b %Y %H:%M:%S %z") 
         << "\t" << m_copi_name << "\t";
    return  sstr.str();
}

              
}; // End namespace CIF
}; // End namespace CCM
}; // End namespace Cdmw

