/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <string>
#include <string.h>
#include <vector>
#include <map>

#include "ExtensionContainerTest.hpp"
#include <Foundation/ossupport/OS.hpp>
#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

#include <CCMContainer/ccmcontainer/Container_impl.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <CCMContainer/ccmcontainer/HomeAllocatorRegistry.hpp>
#include <CCMContainer/ccmcontainer/HomesServantActivator.hpp>

#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcommon/CCMSystemProperties.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include "testqosenabler/TestService_cif.skel.hpp"
#include "testqosenabler/TestComponents_cif.skel.hpp"
#include <Foundation/idllib/CdmwDeployment.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>

#include "testqosenabler/Cdmw_TestService_cif.skel.hpp"
#include "ServerInfoFactory_impl.hpp"
#include "EventModifierCOPITest_impl.hpp"
#include "EventDataExchanger_impl.hpp"
#include "EventModifierCOPITestStubCOPI_impl.hpp"
#include "EventModifierCOPITestServantCOPI_impl.hpp"

namespace
{


}; // End anonymous namespace

namespace Cdmw
{
namespace CCM
{
namespace CIF
{


/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

// static data


CPPUNIT_TEST_SUITE_REGISTRATION( EventModifierCOPITest_impl );

EventModifierCOPITest_impl::EventModifierCOPITest_impl()
{};


EventModifierCOPITest_impl::~EventModifierCOPITest_impl()
    throw()
{
    // does nothing
}

void EventModifierCOPITest_impl::do_tests()
{

    ::std::cout << "*************************************************************" << ::std::endl;
    ::std::cout << "       EventModifierCOPITest_impl.class Test" << ::std::endl;
    ::std::cout << "*************************************************************" << ::std::endl;
    
    
    try {
                       
        ::CORBA::Object_var obj = 
              ::Cdmw::TestUtils::CORBATestManager::instance()->get_object("LoggerServiceContext");

        
        ::Cdmw_TestService::CCM_LoggerService_ExtensionContext_var qos_enabler_context = 
              ::Cdmw_TestService::CCM_LoggerService_ExtensionContext::_narrow(obj.in());
        
        CPPUNIT_ASSERT(false == ::CORBA::is_nil(qos_enabler_context.in()));
        
        obj = 
            ::Cdmw::TestUtils::CORBATestManager::instance()->get_object("ClientSessionContext");
       
        ::TestComponents::CCM_Client_Context_var client_context =
              ::TestComponents::CCM_Client_Context::_narrow(obj.in());
        
        CPPUNIT_ASSERT(false == ::CORBA::is_nil(client_context.in()));


        obj = 
            ::Cdmw::TestUtils::CORBATestManager::instance()->get_object("ServerSessionContext");
       
        m_server_context =
            ::TestComponents::CCM_Server_Context::_narrow(obj.in());
        
        CPPUNIT_ASSERT(false == ::CORBA::is_nil(m_server_context.in()));

        EventModifierCOPITestStubCOPI_impl * stub_copi =
            new EventModifierCOPITestStubCOPI_impl();

        EventModifierCOPITestServantCOPI_impl * servant_copi =
            new EventModifierCOPITestServantCOPI_impl();

        ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_var stub_interceptor_registration =
              qos_enabler_context->get_stub_interceptor_registration();

        ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_var servant_interceptor_registration =
              qos_enabler_context->get_servant_interceptor_registration();
        
        ::Components::Cookie_var stub_copi_cookie =
              stub_interceptor_registration->register_stub_interceptor(stub_copi);
        
        ::Components::Cookie_var servant_copi_cookie =
              servant_interceptor_registration->register_servant_interceptor(servant_copi);

        

        
        ::std::cout << "************************************************" << ::std::endl;
        ::std::cout << " Testing Event Data modification by a SERVANT COPI" << ::std::endl;
        ::std::cout << "***********************************************" << ::std::endl;
                        
        /* 
        // Event data should not have been modified 
               
        COPIServiceInit.disableReadWriteRequests();        
        assertTrue(COPIServiceInit.isCOPIRequestsReadWrite() == false);
            
        test_modify_event(2224);
        assertTrue(checkReceivedEvent(2224));
/        */
            
            

        Cdmw::CCM::COPI::COPIServiceInit::Enable_read_write_requests(); 
        CPPUNIT_ASSERT(true == Cdmw::CCM::COPI::COPIServiceInit::Is_COPI_requests_read_write());


        // test mod via servant_receive_request
        // Event data should have been modified by COPI
        test_case_id = 0;
        test_modify_event(2224);
        CPPUNIT_ASSERT(check_received_event(2323));


        // test mod via stub_send_request
        // Event data should have been modified by COPI
        test_case_id = 1;
        test_modify_event(2224);
        CPPUNIT_ASSERT(check_received_event(55));
            
        // test mod via servant_send_reply
        // At the level of stub_receive_reply a modification should be observed                                                    
        test_case_id = 2;
        test_modify_event(2224);
        CPPUNIT_ASSERT(check_received_event(2224)); // need to sync here, next test case must not be affected! 
                                                        
        // tracer bullet / increment data on each interception, check calling order

        test_case_id = 3; 
        test_modify_event(0);
        CPPUNIT_ASSERT(check_received_event(2));

                                                        
            
        stub_interceptor_registration->unregister_stub_interceptor(stub_copi_cookie);
        servant_interceptor_registration->unregister_servant_interceptor(servant_copi_cookie);   
     
    }
    catch(...) {
        ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << ")" << ::std::endl;
        CPPUNIT_ASSERT(0);
    }



    ::std::cout << "*************************************************************" << ::std::endl;
    ::std::cout << "       End of EventModifierCOPITestImpl.class Test" << ::std::endl;
    ::std::cout << "*************************************************************" << ::std::endl;        
}

bool EventModifierCOPITest_impl::check_received_event(::CORBA::ULong data)
{
    // ClientComponent is supposed to set the event data
    for(int i = 0; i<10; ++i) {
        ::CORBA::ULong _data = EventDataExchanger_impl::Instance()->get_data();            
        if(_data == data) {
            return true;
        }
        ::std::cout << "Waiting for event sink to set event data value of " 
                    << data << " ..." << ::std::endl;;
        ::Cdmw::OsSupport::OS::sleep(500);
    }
    
    ::std::cout << "Giving up waiting for event sink to set event data value of " 
                << data << " ..." << ::std::endl;
    return false;
}

void EventModifierCOPITest_impl::test_modify_event(::CORBA::ULong data)
{
        TestComponents::ServerInfo * si =
            new TestComponents::ServerInfo_impl();
        si->data(data);
        ::std::cout << "pushing " << si->data() << ::std::endl;;
        
        this->m_server_context->push_info(si);
}


              
}; // End namespace CIF
}; // End namespace CCM
}; // End namespace Cdmw

