/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <string>
#include <string.h>
#include <vector>
#include <map>

#include "ExtensionContainerTest.hpp"
#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

#include <CCMContainer/ccmcontainer/Container_impl.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <CCMContainer/ccmcontainer/HomeAllocatorRegistry.hpp>
#include <CCMContainer/ccmcontainer/HomesServantActivator.hpp>


#include <CCMContainer/ccmcommon/CCMSystemProperties.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include "testqosenabler/TestService_cif.skel.hpp"
#include "testqosenabler/TestComponents_cif.skel.hpp"
#include <Foundation/idllib/CdmwDeployment.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>

#include <testqosenabler/generated/ComponentServer/testqosenabler/ExtensionLoggerServiceHome_impl.hpp>
#include <testqosenabler/generated/ComponentServer/testqosenabler/SessionClientHome_impl.hpp>
#include <testqosenabler/generated/ComponentServer/testqosenabler/SessionServerHome_impl.hpp>

#include <testqosenabler/LoggerServiceComponentHome_impl.hpp>



namespace
{


}; // End anonymous namespace

namespace Cdmw
{
namespace CCM
{
namespace CIF
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

CPPUNIT_TEST_SUITE_REGISTRATION( ExtensionContainerTest );
ExtensionContainerTest::ExtensionContainerTest()
   throw(CORBA::SystemException)
{

    m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
    try
    {
        CORBA::Object_var obj = 
            m_orb->string_to_object(Cdmw::TestUtils::CORBATestManager::instance()->get_property ("containerIOR").c_str());

        if (CORBA::is_nil(obj.in()))
        {
            TEST_INFO("TEST FAILED : failed to acces the container ior property");
            CPPUNIT_ASSERT(false);
        }
        else
        {
            m_container = Components::Deployment::Container::_narrow(obj.in());
        }

        std::string typechecking =
            Cdmw::OsSupport::SystemProperties::Get_Property(
                  Cdmw::CCM::TYPECHECKING_PROPERTY_NAME,
                  "off");
        if (typechecking == "on")
            m_is_typechecking_on = true;
        else
            m_is_typechecking_on = false;
        // get the naming context
        obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object
              ("namingContext");

        m_nc 
            = CosNaming::NamingContext::_narrow(obj.in());

    }
    catch (const Cdmw::TestUtils::CORBATestManager::InvalidObject& )
    {
        TEST_INFO("TEST FAILED : InvalidObject raised during test initialisation!");
        CPPUNIT_ASSERT(false);
    }
    catch (const CORBA::SystemException & e)
    {
        TEST_INFO("TEST FAILED : system exception raised during test initialisation!");
        throw;
    }
    catch (...)
    {
        TEST_INFO("TEST FAILED : unexpected exception raised during test initialisation!");
        CPPUNIT_ASSERT(false);
    }
}


ExtensionContainerTest::~ExtensionContainerTest()
    throw()
{
    // does nothing
}


ExtensionContainerTest::ExtensionContainerTest(const ExtensionContainerTest& rhs)
    throw()
    :m_orb(CORBA::ORB::_duplicate(rhs.m_orb)),
     m_container(Components::Deployment::Container::_duplicate(rhs.m_container)),
     m_nc(CosNaming::NamingContext::_duplicate(rhs.m_nc))
{
    // does nothing
}
        
ExtensionContainerTest&
ExtensionContainerTest::operator=(const ExtensionContainerTest& rhs)
    throw()
{
    if (this != &rhs) {

        m_orb = CORBA::ORB::_duplicate(rhs.m_orb.in());
        m_container = Components::Deployment::Container::_duplicate(rhs.m_container.in());
        m_nc = rhs.m_nc;
    }
    return *this;
}

void ExtensionContainerTest::do_tests()
{
    ::Cdmw::CCM::COPI::COPIServiceInit::Enable_read_write_requests();

    // deploy the QoSEnabler
                

    // Register HomeAllocator
    typedef Cdmw::CCM::Container::HomeAllocator<Cdmw::CCM::CIF::CdmwTestService::ExtensionLoggerServiceHome_impl> SessionLoggerService_Allocator;

    Cdmw::CCM::Container::HomeAllocatorRegistry::Register
        (
         "Cdmw.CCM.CIF.CdmwTestService.ExtensionLoggerServiceHome_impl",
         &SessionLoggerService_Allocator::TheAllocator
         );


    TEST_INFO("Testing The Extension Container ...");


    //Create an extension container
    /*
    TEST_INBFO("Creating a container instance ...");
    Components::ConfigValues cfg_compserver(0);
    cfg_compserver.length(0);

    org.omg.Components.Deployment.Container theContainer = null;
    try {
        theContainer = componentServer.create_container(cfgCompserver);
    } catch (org.omg.Components.CreateFailure cf) {
        fail(cf.getMessage());
    } catch (org.omg.Components.Deployment.InvalidConfiguration ic) {
        fail(ic.getMessage());
    }
    */

    // Testing theContainer.install_home() ...
    TEST_INFO("******************************************************");
    TEST_INFO("************* QoSEnabler Deployment  *****************");
    TEST_INFO("******************************************************");
    TEST_INFO("Installing an Extension Home ...");

    try {
        // Create a set of ConfigValues.


        const char *  ENTRY_PT = "create_LoggerServiceComponentHome";
        const char * UUID_home1 = "LoggerServiceHome";


        Components::ConfigValues_var config_values = 
            create_config_values
            (
             CdmwDeployment::EXTENSION,
             CdmwDeployment::COMPONENT_LIFETIME,
             CdmwDeployment::MULTITHREAD,
             "IDL:thalesgroup.com/TestService/LoggerServiceHome:1.0",
             "Cdmw.CCM.CIF.CdmwTestService.ExtensionLoggerServiceHome_impl",
             0
             );

             

        TEST_INFO("Creating " <<  UUID_home1<<  " home...");

        Components::CCMHome_var home = m_container->install_home
            (
             UUID_home1,
             ENTRY_PT,
             config_values
             );


        Components::KeylessCCMHome_var keyless_server_home 
            = Components::KeylessCCMHome::_narrow(home.in());

        if (CORBA::is_nil(keyless_server_home.in()))
            {
                TEST_INFO("ERROR: Created home is not a keyless home!");
                CPPUNIT_ASSERT(false);
            }
     
        // Creating the QoS Enabler component
        TestService::LoggerService_var comp1;
        TEST_INFO("Creating the QoSEnabler component using create_component()...");

        Components::CCMObject_var ccm_obj =
            keyless_server_home->create_component();

        comp1 = TestService::LoggerService::_narrow(ccm_obj);

        CPPUNIT_ASSERT(CORBA::is_nil(comp1) == false);
        comp1->configuration_complete();            

    } catch (::CORBA::SystemException & se) {
        std::cout << "Caught " << se << std::endl;
        CPPUNIT_ASSERT(0);
    } catch (...) {
        std::cout << "Caught unknown exception" << std::endl;
        CPPUNIT_ASSERT(0);
    }    



    ::std::cout << "******************************************************" << ::std::endl;
    ::std::cout << "************* Application Deployment  ****************" << ::std::endl;
    ::std::cout << "******************************************************" << ::std::endl;


    try {

        // Register HomeAllocators
        typedef Cdmw::CCM::Container::HomeAllocator<Cdmw::CCM::CIF::CdmwTestComponents::SessionClientHome_impl> ClientHome_Allocator;
        typedef Cdmw::CCM::Container::HomeAllocator<Cdmw::CCM::CIF::CdmwTestComponents::SessionServerHome_impl> ServerHome_Allocator;

        Cdmw::CCM::Container::HomeAllocatorRegistry::Register
            (
             "Cdmw.CCM.CIF.CdmwTestComponents.SessionClientHome_impl",
             &ClientHome_Allocator::TheAllocator
             );
        
        Cdmw::CCM::Container::HomeAllocatorRegistry::Register
            (
             "Cdmw.CCM.CIF.CdmwTestComponents.SessionServerHome_impl",
             &ServerHome_Allocator::TheAllocator
             );

        ::std::cout << "Installing the Server component Home ..." << ::std::endl;

        const char * ENTRY_PT = "create_ServerComponentHome";
        const char * UUID_home1 = "ServerHome";

        ////////// INSTALLING HOMES //////////

        CORBA::ULong lg = 2 ;
        CdmwDeployment::ValuetypeFactoryDescriptionSeq_var seq = 
            new CdmwDeployment::ValuetypeFactoryDescriptionSeq();
        seq->length(lg);
        {
            CdmwDeployment::ValuetypeFactoryDescription desc;
            desc.repid = ::CORBA::string_dup("IDL:acme.com/TestComponents/Activated:1.0");
            desc.factory_entrypoint = ::CORBA::string_dup("create_activated_event_factory");
            (*seq)[0] = desc;

            desc.repid = ::CORBA::string_dup("IDL:acme.com/TestComponents/ServerInfo:1.0");
            desc.factory_entrypoint = ::CORBA::string_dup("create_server_info_factory");
            (*seq)[1] = desc;            
        }


        Components::ConfigValues_var cfg = 
            create_config_values
            (
             CdmwDeployment::EXTENSION,
             CdmwDeployment::COMPONENT_LIFETIME,
             CdmwDeployment::MULTITHREAD,
             "IDL:thalesgroup.com/TestComponents/ServerHome:1.0",
             "Cdmw.CCM.CIF.CdmwTestComponents.SessionServerHome_impl",
             1
             );

        TEST_INFO("Creating " <<  UUID_home1<<  " home...");

        ::Components::CCMHome_var home = m_container->install_home
            (
             UUID_home1,
             ENTRY_PT,
             cfg
             );

        ::TestComponents::ServerHome_var server_home =
            TestComponents::ServerHome::_narrow(home);

        CPPUNIT_ASSERT(CORBA::is_nil(server_home) == false);

               
        ::std::cout << "Installing the Client component Home ..." << ::std::endl;

        const char * ENTRY_PT_CL = "create_ClientComponentHome";
        const char * UUID_home1_CL = "ClientHome";


        cfg = 
            create_config_values
            (
             CdmwDeployment::EXTENSION,
             CdmwDeployment::COMPONENT_LIFETIME,
             CdmwDeployment::MULTITHREAD,
             "IDL:thalesgroup.com/TestComponents/ClientHome:1.0",
             "Cdmw.CCM.CIF.CdmwTestComponents.SessionClientHome_impl",
             1
             );



        TEST_INFO("Creating " << UUID_home1_CL <<  " home...");

        home = m_container->install_home
            (
             UUID_home1_CL,
             ENTRY_PT_CL,
             cfg
             );

        TestComponents::ClientHome_var client_home =
            TestComponents::ClientHome::_narrow(home);

        CPPUNIT_ASSERT(CORBA::is_nil(client_home) == false);


        ////////// CREATING COMPONENTS //////////
     

        ::std::cout << "Creation a Server Component instance ..." << ::std::endl;

        Components::ConfigValues_var config_values =
            new Components::ConfigValues();
        config_values->length(1);
        ::CORBA::Any value;
        value <<= ::CORBA::string_dup("SERVER_COMPONENT_NAME");
        config_values[0L] = 
            new Container::ConfigValue_impl
            (
             CdmwDeployment::COMPONENT_NAME,
             value
             );

        ::CdmwCcmCif::KeylessCCMHome_var cdmw_keyless_home = 
              ::CdmwCcmCif::KeylessCCMHome::_narrow(server_home);

        Components::CCMObject_var comp = 
            cdmw_keyless_home->create_component_with_config_values(config_values);
            
        ::TestComponents::Server_var server_comp =
              ::TestComponents::Server::_narrow(comp);

        ::std::cout << "Creation a Client Component instance 1 ..." << ::std::endl;
        config_values =
            new Components::ConfigValues();
        config_values->length(1);

        value <<= ::CORBA::string_dup("CLIENT_COMPONENT_NAME1");
        config_values[0L] = 
            new Container::ConfigValue_impl
            (
             ::CdmwDeployment::COMPONENT_NAME, 
             value
             );

        
        
        cdmw_keyless_home = 
            ::CdmwCcmCif::KeylessCCMHome::_narrow(client_home);
        comp = cdmw_keyless_home->create_component_with_config_values(config_values);
        

        
            
        ::TestComponents::Client_var client_comp_1 =
              ::TestComponents::Client::_narrow(comp);

        ::std::cout << "Creation a Client Component instance 2 ..." << ::std::endl;
        config_values =
            new Components::ConfigValues();
        config_values->length(1);

        value <<= ::CORBA::string_dup("CLIENT_COMPONENT_NAME2");
        config_values[0L] = 
            new Container::ConfigValue_impl
            (
             ::CdmwDeployment::COMPONENT_NAME, 
             value
             );

        comp = cdmw_keyless_home->create_component_with_config_values(config_values);
            
        ::TestComponents::Client_var client_comp_2 =
              ::TestComponents::Client::_narrow(comp);

            
        ::std::cout << "Removing the Client Component instance 1 ..." << ::std::endl;
        client_home->remove_component(client_comp_1);            
        

        
        client_comp_2 =
              ::TestComponents::Client::_narrow(comp);
        

        

        ::std::cout << "Connecting Client2 to Server ..." << ::std::endl;

        Components::NameList name_list;
        name_list.length(1);
        name_list[0] = ::CORBA::string_dup("echo");

        Components::FacetDescriptions_var facet_descriptions =
            server_comp->get_named_facets(name_list);
        CPPUNIT_ASSERT(facet_descriptions->length() == 1);
        ::CORBA::Object_ptr obj = facet_descriptions[0L]->facet_ref();

        ::HelloModule::Hello_ptr hello_facet
            = ::HelloModule::Hello::_narrow(obj);
            
        name_list[0] = ::CORBA::string_dup("for_clients");

        facet_descriptions =
            server_comp->get_named_facets(name_list);
        CPPUNIT_ASSERT(facet_descriptions->length() == 1);
        obj = facet_descriptions[0L]->facet_ref();
        ::TestComponents::FacetTest_ptr facet_test =
              ::TestComponents::FacetTest::_narrow(obj);


        //facet_test = 0;
        //hello_facet = 0;
        
          
        
        client_comp_2->connect_to_echo(hello_facet);
        client_comp_2->connect_to_server(facet_test);
        
            
        ::std::cout << "Connecting the Server event sources to the Client sink ..." << ::std::endl;
        
        
        server_comp->connect_i_am_activated(client_comp_2->get_consumer_server_is_activated());
        server_comp->subscribe_info(client_comp_2->get_consumer_server_info());
        
            
        ::std::cout << "configuration_complete ..." << ::std::endl;
        server_comp->configuration_complete();
        client_comp_2->configuration_complete();


        
    } catch (::CORBA::SystemException & se) {
        std::cout << "Caught " << se << std::endl;
        CPPUNIT_ASSERT(0);
    } catch (...) {
        std::cout << "Caught unknown exception" << std::endl;
        CPPUNIT_ASSERT(0);
    }    

        
}

Components::ConfigValues * ExtensionContainerTest::create_config_values (
            CdmwDeployment::ComponentKindValue component_kind,
            CdmwDeployment::ServantLifetimeValue servant_lifetime,
            CdmwDeployment::ThreadingPolicyValue threading_policy,
            const char * repid, 
            const char * classname,
            int do_seq
) 
{

    int nb_values = (do_seq != 0 ? 6:5);
    CdmwDeployment::ValuetypeFactoryDescriptionSeq seq_copied;


    Components::ConfigValues * cfg = new Components::ConfigValues();
    cfg->length(nb_values);
    int idx = 0;
    ::CORBA::Any value;

    // COMPONENT_KIND
    value <<= component_kind;
    (*cfg)[idx++] 
        = new Container::ConfigValue_impl(CdmwDeployment::COMPONENT_KIND, 
                                          value);
        
    // HOME_REPOSITORY_ID
    value <<= repid;
    (*cfg)[idx++]
        = new Container::ConfigValue_impl(CdmwDeployment::HOME_REPOSITORY_ID, 
                                          value);

    // HOME_SERVANT_CLASSNAME
    value <<= classname;
    (*cfg)[idx++]
        = new Container::ConfigValue_impl(CdmwDeployment::HOME_SERVANT_CLASSNAME, 
                                          value);


    // SERVANT_LIFETIME
    value <<= servant_lifetime;
    (*cfg)[idx++]
        = new Container::ConfigValue_impl(CdmwDeployment::SERVANT_LIFETIME, 
                                          value);

    // THREADING_POLICY
    value <<= threading_policy;
    (*cfg)[idx++]
        = new Container::ConfigValue_impl(CdmwDeployment::THREADING_POLICY, 
                                          value);
        
    //VALUETYPE_FACTORY_DEPENDENCIES
    if(do_seq != 0) {
        CORBA::ULong lg = 2 ;
        CdmwDeployment::ValuetypeFactoryDescriptionSeq * seq = 
            new CdmwDeployment::ValuetypeFactoryDescriptionSeq();
        seq->length(lg);
        {
            CdmwDeployment::ValuetypeFactoryDescription * desc =
                new CdmwDeployment::ValuetypeFactoryDescription();
            desc->repid = ::CORBA::string_dup("IDL:acme.com/TestComponents/Activated:1.0");
            desc->factory_entrypoint = ::CORBA::string_dup("create_activated_event_factory");
            (*seq)[0] = (*desc);

            desc =
                new CdmwDeployment::ValuetypeFactoryDescription();

            desc->repid = ::CORBA::string_dup("IDL:acme.com/TestComponents/ServerInfo:1.0");
            desc->factory_entrypoint = ::CORBA::string_dup("create_server_info_factory");
            (*seq)[1] = (*desc);            
        }



        value <<= seq;
        (*cfg)[idx] = 
            new Container::ConfigValue_impl
            (
             CdmwDeployment::VALUETYPE_FACTORY_DEPENDENCIES,
             value
             );
    }

    return cfg;
}

        
}; // End namespace CIF
}; // End namespace CCM
}; // End namespace Cdmw

