/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include "COPITester_impl.hpp"
#include "LoggerServerCOPI_impl.hpp"

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {

LoggerServerCOPI_impl::LoggerServerCOPI_impl
(
 const std::string & name, 
 Cdmw::CCM::COPI::COPITester_impl * copi_tester
 ) : m_copi_name(name), m_priority(0), m_port(""), m_operation(""), m_slot_id(0), m_tester(copi_tester), m_ip(0)
{};

LoggerServerCOPI_impl::LoggerServerCOPI_impl
(
 const std::string & name, 
 const short & p, 
 const std::string & port,
 const std::string & operation
 ) : m_copi_name(name), m_priority(p), m_port(port), m_operation(operation), m_tester(0)
{
    m_ip = new ::Components::ContainerPortableInterceptor::IntegrationPoint();
    m_ip->port = CORBA::string_dup(m_port.c_str());
    m_ip->operation = CORBA::string_dup(m_operation.c_str());
};
 

// Implementation skeleton destructor
LoggerServerCOPI_impl::~LoggerServerCOPI_impl (void)
{
  delete m_ip;
}

void LoggerServerCOPI_impl::receive_request_service_contexts (
    ::Components::ContainerPortableInterceptor::ContainerServerRequestInfo_ptr csi
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    std::cout << get_header()
              << "invoking receive_request_service_contexts of request "
              << csi->request_info()->request_id()
              << " ..." << std::endl;
       
    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = csi->request_info()->operation();

        m_tester->notify_operation
            (
             "receive_request_service_contexts",
             m_copi_name,
             timestamp,
             op.in()
             );   
    }
}

void LoggerServerCOPI_impl::receive_request (
    ::Components::ContainerPortableInterceptor::ContainerServerRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    CPPUNIT_ASSERT(this->m_slot_id != 0);        

    ::CORBA::OctetSeq_var origin_id = info->origin_id();
    ::CORBA::OctetSeq_var target_id = info->target_id();

    ::CORBA::String_var origin_id_str =
        origin_id->length() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(origin_id.in()) : ::CORBA::string_dup("NA");

    ::CORBA::String_var target_id_str =
        target_id->length() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(target_id.in()) : ::CORBA::string_dup("NA");

    std::cout << get_header()
              << "invoking receive_request of request "
              << info->request_info()->request_id() 
              << " ..." << std::endl;

    std::stringstream msgstr;
    msgstr << get_header()
           << "The operation " 
           << ::CORBA::String_var(info->request_info()->operation())
           << " has been received by component " 
           << target_id_str 
           << " from component "
           << origin_id_str;

    std::cout << msgstr.str() << std::endl;

    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->request_info()->operation();            
        m_tester->notify_operation
            (
             "receive_request", m_copi_name,
             timestamp,
             op.in()
             );
    }
}

void LoggerServerCOPI_impl::send_reply (
    ::Components::ContainerPortableInterceptor::ContainerServerRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException
  )
{

    ::CORBA::OctetSeq_var target_id = info->target_id();

    ::CORBA::String_var target_id_str =
       0 != target_id->length() ? ::Cdmw::CCM::Common::OctetSeq_to_string(target_id.in()) : ::CORBA::string_dup("NA");

    std::cout << get_header()
              << "invoking send_reply of request "
              << info->request_info()->request_id()
              << " ..." << std::endl;

    std::stringstream msg;
    
    msg << "The reply to the operation " 
        << ::CORBA::String_var(info->request_info()->operation())
        << " will be sent by the component " 
        << target_id_str;

    std::cout << msg.str() << std::endl;

    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        m_tester->notify_operation
            (
             "send_reply",
             m_copi_name,
             timestamp,
             info->request_info()->operation()
             );
    }      
}

void LoggerServerCOPI_impl::send_exception (
    ::Components::ContainerPortableInterceptor::ContainerServerRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    std::cout << get_header()
              << "invoking send_exception of request "
              << info->request_info()->request_id()
              << " ..." << std::endl;
    
    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->request_info()->operation();
    
        m_tester->notify_operation
            (
             "send_exception", 
             m_copi_name,
             timestamp,
             op.in()
             );
    }
}

void LoggerServerCOPI_impl::send_other (
    ::Components::ContainerPortableInterceptor::ContainerServerRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    std::cout << get_header() 
              << "invoking send_other of request "
              << info->request_info()->request_id() 
              << " ..." << std::endl;

    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->request_info()->operation();

        m_tester->notify_operation
            (
             "send_other",
             m_copi_name,
             timestamp,
             op.in()
             );
    }
}

char * LoggerServerCOPI_impl::name ()
  throw (
    ::CORBA::SystemException
  )
{
    return ::CORBA::string_dup(this->m_copi_name.c_str());
}

::CORBA::UShort LoggerServerCOPI_impl::priority (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return this->m_priority; 
}

::Components::ContainerPortableInterceptor::IntegrationPoint * LoggerServerCOPI_impl::registration_info (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return this->m_ip;
}

void LoggerServerCOPI_impl::destroy ()
  throw (
    ::CORBA::SystemException
  )
{
  // Add your implementation here
}

void LoggerServerCOPI_impl::set_slot_id (
    ::PortableInterceptor::SlotId slot_id
  )
  throw (
    ::CORBA::SystemException
  )
{
    this->m_slot_id = slot_id;
}


std::string LoggerServerCOPI_impl::get_header() {
    std::stringstream sstr;
    sstr << "@B@A@S@I@C@@@C@O@P@I@ " 
         << COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), "%a, %d %b %Y %H:%M:%S %z") 
         << "\t" << m_copi_name << "\t";
    return  sstr.str();
}




} // End namespace CIF

} // End namespace CCM

} // End namespce COPI
