/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "LoggerServiceComponent_impl.hpp"

#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/Assert.hpp"


#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>
#include <fstream>
#include <sstream>
#include "LoggerClientCOPI_impl.hpp"

namespace 
{
    
} // End of anonymous namespace

// initialise static data
::Cdmw::CCM::COPI::COPITester_impl * TestService::LoggerServiceComponent_impl::M_copi_tester = 0;

TestService::LoggerServiceComponent_impl::LoggerServiceComponent_impl()
    : m_name(""),
      m_context(::Cdmw_TestService::CCM_LoggerService_ExtensionContext::_nil())
{
}

TestService::LoggerServiceComponent_impl::~LoggerServiceComponent_impl()
{
    std::cout << "LoggerServiceComponent_impl::~LoggerServiceComponent_impl called!" << std::endl;
}


void
TestService::LoggerServiceComponent_impl::set_extension_context(Components::ExtensionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "Set_extension_context called!" << std::endl;
   
    m_context = ::Cdmw_TestService::CCM_LoggerService_ExtensionContext::_narrow(ctx);
    ::Cdmw::TestUtils::CORBATestManager::instance()->add ("LoggerServiceContext", this->m_context.in());

    ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_ptr registration =
          m_context->get_client_interceptor_registration();


    Cdmw::CCM::COPI::LoggerClientCOPI_impl * my_copi =
        new Cdmw::CCM::COPI::LoggerClientCOPI_impl("my_name", 0);

    registration->register_client_interceptor(my_copi);

    if (0 == M_copi_tester) {
        M_copi_tester =
            new ::Cdmw::CCM::COPI::COPITester_impl(m_context);
    }


    this->LOG_MESSAGE("installing a LOGGING_SERVICE service reference ...");
    m_context->install_service_reference
        (
         "LOGGING_SERVICE", 
         ::TestService::CCM_Logger::_duplicate(this)
         );

    this->LOG_MESSAGE("installing the COPITester service reference ...");
    // this service allows to check the correct COPIs states change
    m_context->install_service_reference
        (
         "COPITester_SERVICE", 
         M_copi_tester
         );

    // registering ComponentlifecycleInterceptor
    /* 
       // LC stuff here
    this->LOG_MESSAGE("Getting the LifeCycleInterceptorRegistrationService reference  ...");
    ::CORBA::Object_ptr obj =
          m_context->resolve_service_reference(Cdmw::CCM::Container::M_LifeCycleInterceptorRegistration.LifeCycleInterceptorRegistrationServiceId);


    CPPUNIT_ASSERT(false == ::CORBA::Object::_is_nil(obj));

    LifeCycleInterceptorRegistration_var registration =
        LifeCycleInterceptorRegistration::_narrow(obj);
    CPPUNIT_ASSERT(false == ::CORBA::Object::_is_nil(registration));

    this->LOG_MESSAGE("Registering Component LifecycleInterceptor ...");

    LifeCycleInterceptorTest_impl * clci = 0;
    LifeCycleInterceptorTest_impl * clci_to_remove = O;


    int nb = 5, idx_to_rem = 2;
    Cookie[] cookies = new Cookie[nb];
        
    cookies[0] = registration
        .register_component_lifecycle_interceptor(
                                                  new LifeCycleInterceptorTest("ComponentLifecycleInterceptor0", Short.MAX_VALUE));
    for (int i = 1; i < nb; i++) {
        clci = new LifeCycleInterceptorTest("ComponentLifecycleInterceptor" + i);
        cookies[i] = registration
            .register_component_lifecycle_interceptor(clci);
        assertTrue(cookies[i] != null);
        if (i == idx_to_rem)
            clci_to_remove = clci;
    }
    try {
        this.LOG_MESSAGE("Unregistering Component LifecycleInterceptor ...");
        clci = (LifeCycleInterceptorTest) registration
            .unregister_component_lifecycle_interceptor(cookies[idx_to_rem]);
        assertTrue(clci.equals(clci_to_remove));
    } catch (Exception ex) {
        ex.printStackTrace();
    }
    */

    // ***
}

void 
TestService::LoggerServiceComponent_impl::ccm_remove()
      throw(CORBA::SystemException)
{
    std::cout << "TestService::LoggerServiceComponent_impl::ccm_remove called!" << std::endl;
}


TestService::CCM_Logger * TestService::LoggerServiceComponent_impl::get_logger()
    throw(CORBA::SystemException) {
    std::cout << "TestService::LoggerServiceComponent_impl::get_logger()  called!" << std::endl;

    return TestService::CCM_Logger::_duplicate(this);
}

void 
TestService::LoggerServiceComponent_impl::LOG_MESSAGE(const char* msg) 
    throw(CORBA::SystemException) {

    Cdmw::OsSupport::OS::Timeval when =
        Cdmw::OsSupport::OS::get_time();


    std::cout << "-----------------------" << ::std::endl
              << "received    : " << "[TIME]" << ::std::endl 
              << "message : " << msg << ::std::endl
              << "-----------------------" << ::std::endl;
}



