/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tao/DynamicInterface/Request.h"
#include <iostream>
#include <sstream>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include "LoggerStubCOPI_impl.hpp"
#include "COPITester_impl.hpp"
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>

namespace {


} // End anonymous space


namespace Cdmw {

namespace CCM {

namespace COPI {


LoggerStubCOPI_impl::LoggerStubCOPI_impl
(
 std::string name, 
 Cdmw::CCM::COPI::COPITester_impl * copi_tester
 ) : m_copi_name(name), m_slot_id(0), m_tester(copi_tester)
{
};


// Implementation skeleton destructor
LoggerStubCOPI_impl::~LoggerStubCOPI_impl (void)
{
}

void LoggerStubCOPI_impl::stub_send_request (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    CPPUNIT_ASSERT(this->m_slot_id != 0);

    std::string origin_id =
        info->origin_id() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*(info->origin_id())) : "NA";
    std::string target_id =
        info->target_id() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*(info->target_id())) : "NA";
                        
    std::cout << get_header() << "stub_send_request called ..." << std::endl;

    std::stringstream msgstr;

    msgstr << "The operation " 
           << info->operation() << " will be sent from component " 
           << origin_id << " to component "
           << target_id;

    std::cout << get_header() 
              << msgstr.str()
              << std::endl; 

    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->operation();

        m_tester->notify_operation
            (
             "stub_send_request", 
             m_copi_name, 
             timestamp, 
             op.in()
             );
    }          
}


void LoggerStubCOPI_impl::stub_receive_reply (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException
  )
{
    std::cout << get_header() << "stub_receive_reply called ..." << std::endl;

    std::string origin_id =
        info->origin_id() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*(info->origin_id())) : "NA";
    std::string target_id =
        info->target_id() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*(info->target_id())) : "NA";
               
    std::cout << get_header() << "stub_receive_reply called ..." << std::endl;

    std::stringstream msgstr;

    msgstr << "The operation " 
           << info->operation() << " reply has been received by component " 
           << target_id << " from component "
           << origin_id;

    std::cout << get_header() 
              << msgstr.str() 
              << std::endl;       

    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->operation();

        m_tester->notify_operation
            (
             "stub_receive_reply", 
             m_copi_name, 
             timestamp, 
             op.in()
             );
    }                
}

void LoggerStubCOPI_impl::stub_receive_exception (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    std::cout << get_header() << "stub_receive_exception called ..." << std::endl;
               
    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();
        ::CORBA::String_var op = info->operation();
        m_tester->notify_operation
            (
             "stub_receive_exception", 
             m_copi_name, 
             timestamp, 
             op.in()
             );
    }                
}

void LoggerStubCOPI_impl::stub_receive_other (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    std::cout << get_header() << "stub_receive_other called ..." << std::endl;

    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();
        ::CORBA::String_var op = info->operation();
        m_tester->notify_operation
            (
             "stub_receive_other", 
             m_copi_name, 
             timestamp, 
             op.in()
             );
    }                       
}

char * LoggerStubCOPI_impl::name (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return CORBA::string_dup(m_copi_name.c_str());
}

::CORBA::UShort LoggerStubCOPI_impl::priority (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

::Components::ContainerPortableInterceptor::IntegrationPoint * LoggerStubCOPI_impl::registration_info (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

void LoggerStubCOPI_impl::destroy (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  // Add your implementation here
}

void LoggerStubCOPI_impl::set_slot_id (
    ::PortableInterceptor::SlotId slot_id
  )
  throw (
    ::CORBA::SystemException
  )
{
    m_slot_id = slot_id;
}

std::string LoggerStubCOPI_impl::get_header() {
    std::stringstream sstr;
    sstr << "####E#X#T#E#N#D#E#D#### " 
         << COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), "%a, %d %b %Y %H:%M:%S %z") 
         << "\t" << m_copi_name << "\t";
    return  sstr.str();
}


} // End namespace CIF

} // End namespace CCM

} // End namespce COPI
