/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <string>
#include <string.h>
#include <vector>
#include <map>

#include "ExtensionContainerTest.hpp"
#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

#include <CCMContainer/ccmcontainer/Container_impl.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include <CCMContainer/ccmcontainer/HomeAllocatorRegistry.hpp>
#include <CCMContainer/ccmcontainer/HomesServantActivator.hpp>

#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcommon/CCMSystemProperties.hpp>
#include <CCMContainer/ccmcontainer/ConfigValue_impl.hpp>
#include "testqosenabler/TestService_cif.skel.hpp"
#include "testqosenabler/TestComponents_cif.skel.hpp"
#include <Foundation/idllib/CdmwDeployment.skel.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>

#include "testqosenabler/Cdmw_TestService_cif.skel.hpp"
#include "ParamModifierStubCOPITest_impl.hpp"



namespace
{


}; // End anonymous namespace

namespace Cdmw
{
namespace CCM
{
namespace CIF
{

/**
*Implementation notes: [if no pertinent write none ]
*<p>
*Portability issues: [if no pertinent write none ]
*<p>
*/

// static data

int ParamModifierStubCOPITest_impl::M_test_case_id = -1;
::CORBA::ORB_var ParamModifierStubCOPITest_impl::M_orb = 0;

CPPUNIT_TEST_SUITE_REGISTRATION( ParamModifierStubCOPITest_impl );

ParamModifierStubCOPITest_impl::ParamModifierStubCOPITest_impl()
{};


ParamModifierStubCOPITest_impl::ParamModifierStubCOPITest_impl
(
 const ::std::string name
 )
   throw (CORBA::SystemException)
    : m_copi_name(name),
      m_tester(0)
{
}

ParamModifierStubCOPITest_impl::ParamModifierStubCOPITest_impl
    (
     const ::std::string name,
     ::Cdmw::CCM::COPI::COPITester_impl * tester
    )
    throw(CORBA::SystemException)    
        : m_copi_name(name),
          m_tester(tester)
{
}


ParamModifierStubCOPITest_impl::ParamModifierStubCOPITest_impl
(
 const ::std::string name,
 TestService::COPITester_ptr tester
 ) 
   throw (CORBA::SystemException)
    : m_copi_name(name),
      m_tester(0)
{
    // this is kind of ugly...
    m_tester = dynamic_cast < ::Cdmw::CCM::COPI::COPITester_impl * > (tester);
}


ParamModifierStubCOPITest_impl::~ParamModifierStubCOPITest_impl()
    throw()
{
    // does nothing
}

void ParamModifierStubCOPITest_impl::stub_receive_exception (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    ::std::cout << get_header()
                << "invoking stub_receive_exception of request ..."
                << std::endl;

    /*
    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->operation();

        m_tester->notify_operation
            (
             "stub_receive_exception",
             m_copi_name,
             timestamp,
             op.in()
             );
    } 
    */       
    
    switch(M_test_case_id){
    case MODIFY_EXCEPTION: modify_exception(info, proceed_call);
        break;
    }
}

void ParamModifierStubCOPITest_impl::stub_receive_other (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{
    ::std::cout << get_header() 
                << "invoking stub_receive_other of request ..." 
                << std::endl;

    /*
    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->operation();

        m_tester->notify_operation
            (
             "stub_receive_other",
             m_copi_name,
             timestamp,
             op.in()
             );
    }
    */                       
}

void ParamModifierStubCOPITest_impl::stub_receive_reply (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException
  )
{
    ::std::cout << get_header()
                << "invoking stub_receive_reply of request ..."
                << ::std::endl;

    /*
    if(0 != m_tester) {
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->operation();

        m_tester->notify_operation
            (
             "stub_receive_reply",
             m_copi_name,
             timestamp,
             op.in()
             );
    } 
    */

    switch(M_test_case_id){
    case MODIFY_OUT_RET: modify_out_return_param(info, proceed_call);
        break;
    }
}

void ParamModifierStubCOPITest_impl::stub_send_request (
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
    ::CORBA::Boolean_out proceed_call
  )
  throw (
    ::CORBA::SystemException,
    ::PortableInterceptor::ForwardRequest
  )
{        
    CPPUNIT_ASSERT(this->m_slot_id != 0);

    std::string origin_id =
        info->origin_id() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*(info->origin_id())) : "NA";
    std::string target_id =
        info->target_id() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*(info->target_id())) : "NA";

    
    std::cout << get_header()
              << "invoking stub_send_request of request ..."
              << std::endl;

    std::stringstream msgstr;

    msgstr << "The operation " 
           << info->operation() << " has been received by component " 
           << target_id << " from component "
           << origin_id;

    std::cout << get_header()
              << msgstr.str()
              << std::endl;

    /*
    if(0 != m_tester) {        
        Cdmw::OsSupport::OS::Timeval timestamp =
            Cdmw::OsSupport::OS::get_time();

        ::CORBA::String_var op = info->operation();

        m_tester->notify_operation
            (
             "stub_send_request",
             m_copi_name,
             timestamp,
             op.in()
             );
    }
    */

                
    switch(M_test_case_id){
    case MODIFY_IN_PARAM: modify_in_param(info, proceed_call);
        break;
    case DO_NOT_PROCEED_CALL : do_not_proceed_call(info, proceed_call);
        break;
    case THROW_EXCEPTION : throw_exception(info, proceed_call);
        break;
    }      
}

void ParamModifierStubCOPITest_impl::destroy (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
  // Add your implementation here
}


char * ParamModifierStubCOPITest_impl::name ()
  throw (
    ::CORBA::SystemException
  )
{
    return CORBA::string_dup(m_copi_name.c_str());
}

::CORBA::UShort ParamModifierStubCOPITest_impl::priority (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

::Components::ContainerPortableInterceptor::IntegrationPoint *
ParamModifierStubCOPITest_impl::registration_info (
    
  )
  throw (
    ::CORBA::SystemException
  )
{
    return 0;
}

void ParamModifierStubCOPITest_impl::set_slot_id (
    ::PortableInterceptor::SlotId slot_id
  )
  throw (
    ::CORBA::SystemException
  )
{
    m_slot_id = slot_id;
}


std::string ParamModifierStubCOPITest_impl::get_header() {
    std::stringstream sstr;
    sstr << "####E#X#T#E#N#D#E#D#### " 
         << ::Cdmw::CCM::COPI::COPIUtils_impl::Timestamp_to_string(Cdmw::OsSupport::OS::get_time(), "%a, %d %b %Y %H:%M:%S %z") 
         << "\t" << m_copi_name << "\t";
    return  sstr.str();
}


void ParamModifierStubCOPITest_impl::do_tests()
{

    ::std::cout << "*************************************************************" << ::std::endl;
    ::std::cout << "       ParamModifierStubCOPIImpl.class Test" << ::std::endl;
    ::std::cout << "*************************************************************" << ::std::endl;
  

    Cdmw::CCM::COPI::COPIServiceInit::Enable_read_write_requests();
    
    CPPUNIT_ASSERT(true == Cdmw::CCM::COPI::COPIServiceInit::Is_COPI_requests_read_write());
    
    
    try{
        
        M_orb =
            ::Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();
        
        
        ::CORBA::Object_var obj = 
              ::Cdmw::TestUtils::CORBATestManager::instance()->get_object("LoggerServiceContext");

        
        ::Cdmw_TestService::CCM_LoggerService_ExtensionContext_var qos_enabler_context = 
              ::Cdmw_TestService::CCM_LoggerService_ExtensionContext::_narrow(obj.in());
        
        CPPUNIT_ASSERT(false == ::CORBA::is_nil(qos_enabler_context.in()));
        
        obj = 
            ::Cdmw::TestUtils::CORBATestManager::instance()->get_object("ClientSessionContext");
       
        ::TestComponents::CCM_Client_Context_var client_context =
              ::TestComponents::CCM_Client_Context::_narrow(obj.in());
        
        CPPUNIT_ASSERT(false == ::CORBA::is_nil(client_context.in()));

        obj =
            client_context->resolve_service_reference("COPITester_SERVICE");

        ::TestService::COPITester_var tester =
              ::TestService::COPITester::_narrow(obj.in());
        
        CPPUNIT_ASSERT(false == ::CORBA::is_nil(tester.in()));
        
        obj =
            client_context->resolve_service_reference("LOGGING_SERVICE");

        m_logger = ::TestService::Logger::_narrow(obj.in());
        
        CPPUNIT_ASSERT(false == ::CORBA::is_nil(m_logger.in()));
        
        
        m_facet =
            client_context->get_connection_to_server();
                
        ParamModifierStubCOPITest_impl * copi 
            = new ParamModifierStubCOPITest_impl("ParamModifierStubCOPI", tester.in());

        ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_var stub_interceptor_registration =
              qos_enabler_context->get_stub_interceptor_registration();
        
        ::Components::Cookie_var stub_copi_cookie =
              stub_interceptor_registration->register_stub_interceptor(copi);
        
        tester->register_interceptor("Dummy COPI", ::TestService::STUB);
        
        tester->start_listening();
        
        ::std::cout << "*******************************************" << ::std::endl;
        ::std::cout << " Testing IN param modification by a STUB COPI" << ::std::endl;
        ::std::cout << "*******************************************" << ::std::endl;        
        M_test_case_id++;        
        test_modify_in_param();
        
        
        ::std::cout << "************************************************" << ::std::endl;
        ::std::cout << " Testing OUT and RETURN param modification by a STUB COPI" << ::std::endl;
        ::std::cout << "************************************************" << ::std::endl;
        M_test_case_id++;
        test_modify_out_return_param();
        
        ::std::cout << "************************************************" << ::std::endl;
        ::std::cout << " Testing throwing an exception by a STUB COPI" << ::std::endl;
        ::std::cout << "************************************************" << ::std::endl;
        M_test_case_id++;
        test_throw_exception();
        
        ::std::cout << "************************************************" << ::std::endl;
        ::std::cout << " Testing Exception modification by a STUB COPI" << ::std::endl;
        ::std::cout << "************************************************" << ::std::endl;
        M_test_case_id++;
        test_modify_exception();

        /* Bug: this blocks and produced crashes */
        ::std::cout << "************************************************" << ::std::endl;
        ::std::cout << " Testing Call blocking by a STUB COPI" << ::std::endl;
        ::std::cout << "***********************************************" << ::std::endl;
        M_test_case_id++;
        test_do_not_proceed_call();
        
        // make perl script recognise this test as failure,
        //::std::cout << "PART_NOT_EXECUTED: Testing Call blocking by a STUB COPI" << __FILE__ << "(" << __LINE__ << ")" << ::std::endl;
        
        tester->end_listening();
        tester->test_interception_stacks();
                       
        stub_interceptor_registration->unregister_stub_interceptor(stub_copi_cookie.in());
        tester->unregister_interceptor("Dummy COPI");
        
    }
    catch(...){
        ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "):  Caught unknown exception." << ::std::endl;
        CPPUNIT_ASSERT(0);
    }
        
    ::std::cout << "*************************************************************" << ::std::endl;
    ::std::cout << "       End of ParamModifierStubCOPIImpl.class Test" << ::std::endl;
    ::std::cout << "*************************************************************" << ::std::endl;        
}


void ParamModifierStubCOPITest_impl::test_modify_in_param()
{
    try {
        ::CORBA::Long rem = 0;
        
        ::CORBA::Long res =
              m_facet->makeOperation(::TestComponents::DIV, 100, 3, rem);
        std::stringstream msgstr;
        
        msgstr << "100 DIV 3 = "
               << res 
               << ", remainder = "
               << rem;
        
        m_logger->LOG_MESSAGE(msgstr.str().c_str());
        // the in params has been changed by the COPI
        
        CPPUNIT_ASSERT(res==1003 && rem==0);
        
        m_logger->LOG_MESSAGE("Succeded to get right return and out parameters :-) (test_modify_in_param)");
            
    } catch (...) {
        ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "):  Caught unknown exception." << ::std::endl;
        CPPUNIT_ASSERT(0);
    }

}


void ParamModifierStubCOPITest_impl::modify_in_param
(
 ::Components::ContainerPortableInterceptor::ContainerRequestInfo_ptr info,  
 ::CORBA::Boolean & proceed_call
 )
{

    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr stub_info =
          dynamic_cast< ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr > (info);

    ::CORBA::String_var op = stub_info->operation();
              
    if(0 == strcmp("makeOperation", op.in())) {
        // change the operator of makeOperation to ADD
        // change the first long arg to 1000

        ::std::cout << get_header() 
                    << "Modifying the Operator and the first operand" << ::std::endl;

        ::Dynamic::ParameterList_var params =
              stub_info->arguments();

        ::CORBA::ULong i = 0, found = 0;

        while(i<params->length() && found!=2 ){

            if (::TestComponents::_tc_ArithOperator->equal(params[i].argument.type())) {
                ::CORBA::Any any;
                any <<= ::TestComponents::ADD;
                params[i].argument= any;
                found++;
            }
            if(::CORBA::tk_long == params[i].argument.type()->kind()) {
                ::CORBA::Any any;
                ::CORBA::Long value = 1000;
                any <<= value;
                params[i].argument = any;
                found++;
            }
            i++;
        }
        stub_info->arguments(params.in());
        if(found==2) ::std::cout << get_header() << "Succeeded to modify IN params of operation makeOperation" << ::std::endl;
    }
}


void ParamModifierStubCOPITest_impl::modify_out_return_param
(
 ::Components::ContainerPortableInterceptor::ContainerRequestInfo_ptr info, 
 ::CORBA::Boolean & proceed_call
 ) 
{
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr stub_info =
          dynamic_cast< ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr > (info);
    
    ::CORBA::String_var op = stub_info->operation();

    if(0 == strcmp("makeOperation", op.in())) {
        ::std::cout << get_header() << "Modifying the user return value" << ::std::endl;
        ::CORBA::Any_var any =
              stub_info->result();
        ::CORBA::Long value = 197556;
        any <<= value;
        stub_info->result(any.in());
    }    
} 


void ParamModifierStubCOPITest_impl::test_modify_out_return_param()
{
    try {
        ::CORBA::Long rem = 0;
        ::CORBA::Long res =
              m_facet->makeOperation(::TestComponents::DIV, 100, 3, rem);
        ::std::stringstream msg;
        msg << "100 DIV 3 = "
            << res
            << ", remainder = "
            << rem;
        
        m_logger->LOG_MESSAGE(msg.str().c_str());
        CPPUNIT_ASSERT(res==197556 && rem==1);
        m_logger->LOG_MESSAGE("Succeded to get right return and out parameters :-) (test_modify_out_return_param)");
            
    } catch (...) {
        ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "):  Caught unknown exception." << ::std::endl;
        CPPUNIT_ASSERT(0);
    }        
}

void ParamModifierStubCOPITest_impl::throw_exception
(
 ::Components::ContainerPortableInterceptor::ContainerRequestInfo_ptr info, 
 ::CORBA::Boolean & proceed_call
 )
{
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr stub_info =
          dynamic_cast< ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr > (info);

    ::CORBA::String_var op = stub_info->operation();

    if(0 == strcmp("makeOperation", op.in())) {
        ::std::cout << get_header() 
                    << "Throwing ExceptionTest exception" << ::std::endl;
        ::CORBA::Any a;


        ::TestComponents::ExceptionTest ex;
        // No chance to get NO_IMPLEMENT inserted, works fine for java though
        //::CORBA::NO_IMPLEMENT ex;

        a <<= ex;
        stub_info->the_exception(a);
        //throw ex;
    }
}


void ParamModifierStubCOPITest_impl::test_throw_exception()
{
    bool exception_ok = false;
    try{
        ::CORBA::Long rem = 0;
        
        ::CORBA::Long res =
              m_facet->makeOperation(::TestComponents::DIV, 100, 3, rem);
        
        res = 0;
            
    } catch(::TestComponents::ExceptionTest & ni){        
        exception_ok = true;
        m_logger->LOG_MESSAGE("Succeded to get the right exception");
    } catch (...) {
        
        ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "):  Caught unknown exception." << ::std::endl;
        CPPUNIT_ASSERT(0);
    }        

    
    CPPUNIT_ASSERT(true == exception_ok);
    
}

           
void ParamModifierStubCOPITest_impl::modify_exception
(
 ::Components::ContainerPortableInterceptor::ContainerRequestInfo_ptr info, 
 ::CORBA::Boolean & proceed_call      
 )
{
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr stub_info =
          dynamic_cast< ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr > (info);
  
    ::CORBA::String_var op = stub_info->operation();

    if(0 == strcmp("raiseException", op.in())) {
        ::CORBA::Any_var a =
              stub_info->the_exception();

        CPPUNIT_ASSERT(true == ::TestComponents::_tc_ExceptionTest->equal(a->type()));
        
        ::TestComponents::ExceptionTest * ex = 0;
        a.in() >>= ex;
        CPPUNIT_ASSERT(0 != ex);

        //modifying the exception reason
        ::std::stringstream msg;
        msg << "The stub COPI '"
            << m_copi_name
            << "' has modified the reason of that exception";
                    
        ex->reason = ::CORBA::string_dup(msg.str().c_str());

        CORBA::Any a2;
        a2 <<= (*ex);
        stub_info->the_exception(a2);
    }
}

void ParamModifierStubCOPITest_impl::test_modify_exception()
{
    bool exception_ok = false; 

    try{

        m_facet->raiseException(true, "Testing exception raising");
        
    }catch (::TestComponents::ExceptionTest & e) {
        
        exception_ok = true; 
        ::std::stringstream msg;
        msg << "Succeded to get the right exception(2) : "
            << e.reason;        
        m_logger->LOG_MESSAGE(msg.str().c_str());
    } catch (...) {
                
        ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "):  Caught unknown exception." << ::std::endl;
        CPPUNIT_ASSERT(0);
    }
       
    
    CPPUNIT_ASSERT(true == exception_ok);
    
}

void ParamModifierStubCOPITest_impl::do_not_proceed_call
(
 ::Components::ContainerPortableInterceptor::ContainerRequestInfo_ptr info, 
 ::CORBA::Boolean & proceed_call      
 )
{
    ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr stub_info =
          dynamic_cast< ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr > (info);
    
    ::CORBA::String_var op = stub_info->operation();
    
    if(0 == strcmp("makeOperation", op.in())) {
        ::std::cout << get_header() << "Replying in stead of the user code" << ::std::endl;
        ::CORBA::Any_var any =
              stub_info->result();
        ::CORBA::Long value = 197556;
        any <<= value;
        stub_info->result(any.in());
        proceed_call = false;
    }        
}

void ParamModifierStubCOPITest_impl::test_do_not_proceed_call()
{
        try {
            // Note: the java test tests rem for 0.
            // But in gere rem contains garbage, what does not surprise because 
            // this is an out param.
            // For java this param is 0, I guess this has smth. to do
            // with the fact that ints in java are auto initialised to 0

            ::CORBA::Long rem = 0;
            ::CORBA::Long res =
                  m_facet->makeOperation(::TestComponents::DIV, 100, 3, rem);
            ::std::stringstream msg;
            msg << "100 DIV 3 = "
                << res;
            m_logger->LOG_MESSAGE(msg.str().c_str());

            CPPUNIT_ASSERT(res==197556);
            m_logger->LOG_MESSAGE("Succeded to get right return and out parameters :-) (test_do_not_proceed_call)");
            
        } catch (...) {
            ::std::cerr << "**ERROR: " << __FILE__ << "(" << __LINE__ << "):  Caught unknown exception." << ::std::endl;
            CPPUNIT_ASSERT(0);
        }
}
       
}; // End namespace CIF
}; // End namespace CCM
}; // End namespace Cdmw

