/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "ServerComponent_impl.hpp"


#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/common/Assert.hpp"


#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/TextOutputter.h>
#include <fstream>

#include "ServerInfoFactory_impl.hpp"

namespace 
{
    
} // End of anonymous namespace



//
// IDL:thalesgroup.com/BasicDemo/CCM_Client:1.0
//
TestComponents::ServerComponent_impl::ServerComponent_impl()
    : m_name(""),
      m_session_context(TestComponents::CCM_Server_Context::_nil())
{
}

TestComponents::ServerComponent_impl::~ServerComponent_impl()
{
    std::cout << "ServerComponent_impl::~ServerComponent_impl called!" << std::endl;
}


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
TestComponents::ServerComponent_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "Set_session_context called!" << std::endl;
   
    m_session_context = ::TestComponents::CCM_Server_Context::_narrow(ctx); 

    ::Cdmw::TestUtils::CORBATestManager::instance()->add ("ServerSessionContext", this->m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
TestComponents::ServerComponent_impl::ccm_activate()
   throw(CORBA::SystemException)
{    

    TestComponents::ServerInfo * si =
        new TestComponents::ServerInfo_impl();
    si->data(9);
    
    std::cout << "[Server] pushing ServerInfo event" << std::endl;
    m_session_context->push_info(si);
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
TestComponents::ServerComponent_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
TestComponents::ServerComponent_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "Ccm_remove called!" << std::endl;
}

TestComponents::CCM_FacetTest* 
TestComponents::ServerComponent_impl::get_for_clients()
    throw(CORBA::SystemException) 
{
    std::cout << "get_for_clients called!" << std::endl;
    

    return TestComponents::CCM_FacetTest::_duplicate(this);
}

    
HelloModule::CCM_Hello*
TestComponents::ServerComponent_impl::get_echo()
        throw(CORBA::SystemException)
{
    std::cout << "get_echo called!" << std::endl;
    
    return HelloModule::CCM_Hello::_duplicate(this);
}



void
TestComponents::ServerComponent_impl::raiseException
(
 CORBA::Boolean _do,  
 const char* reason
 )
throw (CORBA::SystemException, TestComponents::ExceptionTest)
{
    ::std::cout << "ServerComponent : request to raise an exception" << ::std::endl;
    if (_do == true) {
        TestComponents::ExceptionTest exceptionTest(reason);
        ::std::cout << "ServerComponent : throwing ExceptionTest" << ::std::endl;
        throw exceptionTest;
    }
}

CORBA::Long
TestComponents::ServerComponent_impl::makeOperation
(
 TestComponents::ArithOperator oper,
 CORBA::Long operand1,
 CORBA::Long operand2,
 CORBA::Long& remainder
 )
throw (CORBA::SystemException, TestComponents::ExceptionTest)
{
    ::CORBA::Long result = 0;
    remainder = 0;

    ::std::cout << "ServerComponent : request to process an operation" << ::std::endl;
    switch (oper){
    case ::TestComponents::ADD: 
        {
            result = operand1 + operand2;
            break;
        }
    case ::TestComponents::DIV:
        {
            if(operand2 == 0) {
                ExceptionTest ex;
                ex.reason = ::CORBA::string_dup("Division By Zero");
                throw ex;
            }
            result = operand1 / operand2;
            remainder = operand1 % operand2;
            break;
        }            
    case ::TestComponents::MULTI: 
        {
            result = operand1 * operand2;
            break;
        }        
    case ::TestComponents::SUB: 
        {
            result = operand1 - operand2;
            break;
        }
    default : 
        {
            ::TestComponents::ExceptionTest ex;
            ex.reason = ::CORBA::string_dup("Unsupported operation");
            throw ex;
        }

    }
        
    return result;
}

void 
TestComponents::ServerComponent_impl::echo
(
 const char* msg
 )
throw (CORBA::SystemException)
{
    std::cout << "echo called! [" << msg << "]"<< std::endl;
    if(0 == strcmp("eris", msg)) 
      ::std::cout << "MILESTONE3" << std::endl;
  
}








