/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "MakeOperationServantRequest_impl.hpp"


namespace {


} // End anonymous space



namespace Cdmw {
    
namespace CCM {

namespace CIF {

namespace CdmwTestComponents {



MakeOperationServantRequest_impl::MakeOperationServantRequest_impl 
(
 const char *  receptacle_name, 
 const char *  component_id,
 ::CORBA::Object_ptr facet_object,
 ::Cdmw::CCM::CIF::Context * ctx  
 ) :
    ::Cdmw::CCM::COPI::ContainerExtendedCopisRequest_impl<Cdmw::CCM::COPI::ContainerServantRequestInfo_impl>(ctx),
    ContainerServantRequest_impl(ctx), m_index(-1)
{
    initialise(receptacle_name, component_id, facet_object);
}

MakeOperationServantRequest_impl::~MakeOperationServantRequest_impl() 
{
    // empty
}

char * MakeOperationServantRequest_impl::operation() {
    return ::CORBA::string_dup("makeOperation");
}



void MakeOperationServantRequest_impl::add_in_args_oper(::TestComponents::ArithOperator oper) {
    ::CORBA::Any any;
    any <<= oper;
    this->add_in_args(any);
    m_arguments_indices.insert(::std::make_pair("oper", ++m_index));
}
        
::TestComponents::ArithOperator MakeOperationServantRequest_impl::get_in_args_oper() {   
    ::std::map< ::std::string, int >::iterator it =
          m_arguments_indices.find("oper");
    
    ::TestComponents::ArithOperator oper;

    if(it != m_arguments_indices.end()) {
        ::CORBA::Any_var any =
              get_arg(it->second);            

        if(!(any.in() >>= oper)) {
            CDMW_ASSERT(0);
        };
        
    } else {            
        CDMW_ASSERT(0);
    }

    return oper;
}
                                            
void MakeOperationServantRequest_impl::add_in_args_operand1(::CORBA::Long operand1) {
    ::CORBA::Any any;
    any <<= operand1;
    this->add_in_args(any);
    m_arguments_indices.insert(::std::make_pair("operand1", ++m_index));
}
         
::CORBA::Long MakeOperationServantRequest_impl::get_in_args_operand1() {
    ::std::map< ::std::string, int >::iterator it =
          m_arguments_indices.find("operand1");

    ::CORBA::Long operand1 = 0;

    if(it != m_arguments_indices.end()) {
        ::CORBA::Any_var any =
              get_arg(it->second);            

        if(!(any.in() >>= operand1)) {
            CDMW_ASSERT(0);
        };
                                   
    } else {            
        CDMW_ASSERT(0);
    }

    return operand1;
}

void MakeOperationServantRequest_impl::add_in_args_operand2(::CORBA::Long operand2) {
    ::CORBA::Any any;
    any <<= operand2;
    this->add_in_args(any);
    m_arguments_indices.insert(::std::make_pair("operand2", ++m_index));
}

::CORBA::Long MakeOperationServantRequest_impl::get_in_args_operand2() {
    ::std::map< ::std::string, int >::iterator it =
          m_arguments_indices.find("operand2");

    ::CORBA::Long operand2;

    if(it != m_arguments_indices.end()) {
        ::CORBA::Any_var any =
              get_arg(it->second);            

        if(!(any.in() >>= operand2)) {
            CDMW_ASSERT(0);
        };
                                   
    } else {            
        CDMW_ASSERT(0);
    }

    return operand2;
}


void MakeOperationServantRequest_impl::add_out_args_remainder(::CORBA::Long remainder) {
    ::CORBA::Any any;
    any <<= remainder;
    this->add_out_args(any);
    m_arguments_indices["remainder"] =  ++m_index;
    // m_arguments_indices.insert(::std::make_pair("remainder", ++m_index));
}

::CORBA::Long MakeOperationServantRequest_impl::get_out_args_remainder() {
        ::std::map< ::std::string, int >::iterator it =
          m_arguments_indices.find("remainder");

    ::CORBA::Long remainder;

    if(it != m_arguments_indices.end()) {
        ::CORBA::Any_var any =
              get_arg(it->second);            

        if(!(any.in() >>= remainder)) {
            CDMW_ASSERT(0);
        };
                                   
    } else {            
        CDMW_ASSERT(0);
    }

    return remainder;
}
             
void MakeOperationServantRequest_impl::add_result_value(::CORBA::Long value) {
    ::CORBA::Any any;
    any <<= value;
    this->add_result(any);
}
               
::CORBA::Long MakeOperationServantRequest_impl::result_value() {
    ::CORBA::Any_var res = this->result();
    ::CORBA::Long result;
    if(!(res.in() >>= result)) {
        CDMW_ASSERT(0);
    };

    return result;
}
                 
void MakeOperationServantRequest_impl::add_exception_ExceptionTest (::TestComponents::ExceptionTest ex) {
    ::CORBA::Any ex_any;
    ex_any <<= ex;
    this->add_exception(ex_any);
}

void MakeOperationServantRequest_impl::add_exception_ExceptionTest_tc () {
    this->add_exception_tc(::TestComponents::_tc_ExceptionTest);
}

void 
MakeOperationServantRequest_impl::throw_exception()
    throw (::CORBA::SystemException, ::TestComponents::ExceptionTest) {

    m_request_info->throw_system_exception();
    

    ::CORBA::Any_var ex_any = this->the_exception();

    if (TestComponents::_tc_ExceptionTest->equal(ex_any->type())) {
        ::TestComponents::ExceptionTest * ex = 0;
        if(!(ex_any.in() >>= ex)) {
            CDMW_ASSERT(0);
        }
        throw ex;
    }
}

} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

}

