/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "RaiseExceptionServantRequest_impl.hpp"


namespace {


} // End anonymous space



namespace Cdmw {
    
namespace CCM {

namespace CIF {

namespace CdmwTestComponents {



RaiseExceptionServantRequest_impl::RaiseExceptionServantRequest_impl 
(
 const char *  receptacle_name, 
 const char *  component_id,
 ::CORBA::Object_ptr facet_object,
 ::Cdmw::CCM::CIF::Context * ctx  
 ) :
    ::Cdmw::CCM::COPI::ContainerExtendedCopisRequest_impl<Cdmw::CCM::COPI::ContainerServantRequestInfo_impl>(ctx),
    ContainerServantRequest_impl(ctx), m_index(-1)
{
    initialise(receptacle_name, component_id, facet_object);
}

RaiseExceptionServantRequest_impl::~RaiseExceptionServantRequest_impl() 
{
    // empty
}

char * RaiseExceptionServantRequest_impl::operation() {
    return ::CORBA::string_dup("raiseException");
}

// ***
void RaiseExceptionServantRequest_impl::add_in_args_raise(::CORBA::Boolean raise) {
    ::CORBA::Any any;
    any <<= ::CORBA::Any::from_boolean(raise);
    this->add_in_args(any);
    m_arguments_indices.insert(::std::make_pair("raise", ++m_index));
}
         
bool RaiseExceptionServantRequest_impl::get_in_args_raise() {
    ::std::map< ::std::string, int >::iterator it =
          m_arguments_indices.find("raise");

    ::CORBA::Boolean raise;

    if(it != m_arguments_indices.end()) {
        ::CORBA::Any_var any =
              get_arg(it->second);            

        if(!(any.in() >>= ::CORBA::Any::to_boolean(raise))) {
            CDMW_ASSERT(0);
        };
                                   
    } else {            
        CDMW_ASSERT(0);
    }

    return raise;
}

void RaiseExceptionServantRequest_impl::add_in_args_reason(const char * reason) {
    ::CORBA::Any any;
    any <<= reason;
    this->add_in_args(any);
    m_arguments_indices.insert(::std::make_pair("reason", ++m_index));
}

         
char * RaiseExceptionServantRequest_impl::get_in_args_reason() {
    ::std::map< ::std::string, int >::iterator it =
          m_arguments_indices.find("reason");

    const char * reason = 0;

    if(it != m_arguments_indices.end()) {
        ::CORBA::Any_var any =
              get_arg(it->second);            

        if(!(any.in() >>= reason)) {
            CDMW_ASSERT(0);
        };
                                   
    } else {            
        CDMW_ASSERT(0);
    }

    return ::CORBA::string_dup(reason);
}

void RaiseExceptionServantRequest_impl::add_exception_ExceptionTest 
(
 ::TestComponents::ExceptionTest ex
 ) {
    ::CORBA::Any ex_any;
    ex_any <<= ex;
    this->add_exception(ex_any);
}
              
void RaiseExceptionServantRequest_impl::add_exception_ExceptionTest_tc () {
    this->add_exception_tc(::TestComponents::_tc_ExceptionTest);
}
                             
void 
RaiseExceptionServantRequest_impl::throw_exception()
    throw (::CORBA::SystemException, ::TestComponents::ExceptionTest) 
{
    m_request_info->throw_system_exception();
    
    ::CORBA::Any_var ex_any = this->the_exception();

    if (TestComponents::_tc_ExceptionTest->equal(ex_any->type())) {
        ::TestComponents::ExceptionTest * ex = 0;
        if(!(ex_any.in() >>= ex)) {
            CDMW_ASSERT(0);
        }
        throw ex;
    }
}

// ****


} // End namespace CIF

} // End namespace CCM

} // End namespce COPI

}

