/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



///// FILE TO BE GENERATED


#include <Foundation/logging/FunctionLogger.hpp>   
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include "ReceptacleFacetTest_impl.hpp"
#include "RaiseExceptionStubRequest_impl.hpp"
#include "MakeOperationStubRequest_impl.hpp"

   

namespace Cdmw {

namespace CCM {

namespace CIF {

namespace CdmwTestComponents {




   ReceptacleFacetTest_impl::ReceptacleFacetTest_impl
    (
     const ::std::string & receptacle_name,
     const ::std::string & component_id,
     ::CORBA::Object_ptr facet_object,
     ::Cdmw::CCM::CIF::Context * context,
     CdmwLogging::Logger_ptr     logger
     ) 
         : m_receptacle_name(receptacle_name),
           m_component_identity(component_id),
           m_facet_object(facet_object),
           m_context(context),
           m_logger(logger)
           
    {
        CDMW_LOG_FUNCTION(m_logger, "");
    };
    
    ReceptacleFacetTest_impl::~ReceptacleFacetTest_impl() throw() 
    {
        CDMW_LOG_FUNCTION(m_logger, "");
        // empty
    }


    void ReceptacleFacetTest_impl::raiseException 
    (
     ::CORBA::Boolean raise,
     const char * reason
     )
        throw (
               ::CORBA::SystemException,
               ::TestComponents::ExceptionTest
               )
    {
        CDMW_LOG_FUNCTION(m_logger, "");

                
        const std::string &  feature_name = this->m_receptacle_name;
        ::CORBA::Object_var target = 
              ::CORBA::Object::_duplicate(this->m_facet_object);

        ::TestComponents::FacetTest_var facet_object;

              
        RaiseExceptionStubRequest_impl request
            (
             feature_name.c_str(), 
             m_component_identity.c_str(), 
             target.in(), 
             m_context
             );
       
        if (request.is_read_write()){   
            request.add_in_args_raise(raise);
            request.add_in_args_reason(reason);
            request.add_exception_ExceptionTest_tc();

            request.pre_invoke();
            request.invoke();
            request.post_invoke();

            ::CORBA::Any_var ex =
                  request.the_exception();

            if(::CORBA::tk_null != ex->type()->kind()) {
                if(ex->type()->equal(::TestComponents::_tc_ExceptionTest)) {
                    ::TestComponents::ExceptionTest * user_exception = 0;
                    if(!(ex.in() >>= user_exception)) {
                        CDMW_ASSERT(0);
                    }
                    
                    throw *user_exception;
                } else {
                    throw request.the_systemException();
                }
            }                                       
       
        } else { 
            request.pre_invoke();
            try {
                facet_object =
                    ::TestComponents::FacetTest::_narrow (target);

            } catch (::CORBA::BAD_PARAM & e) {
                CDMW_ERROR(m_logger, 
                           "The facet object has wrong type");
                ::CORBA::OBJECT_NOT_EXIST ex
                      (
                       ::Cdmw::OrbSupport::OBJECT_NOT_EXIST, 
                       ::CORBA::COMPLETED_NO
                       );
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
               
            try{  
                facet_object->raiseException
                    (
                     raise, 
                     reason
                     );
            } 
            catch(::CORBA::SystemException & se){
                request.add_system_exception(se);
            }
            catch(::TestComponents::ExceptionTest & e) {
                request.add_exception_ExceptionTest(e);
            }

      
            request.post_invoke();
            request.throw_exception();
         
        } //end of else
                             
    }
  

    ::CORBA::Long ReceptacleFacetTest_impl::makeOperation 
    (
     ::TestComponents::ArithOperator oper,
     ::CORBA::Long operand1,
     ::CORBA::Long operand2,
     ::CORBA::Long_out remainder
     )
        throw (
               ::CORBA::SystemException,
               ::TestComponents::ExceptionTest
               )
    {
        CDMW_LOG_FUNCTION(m_logger, "");
        const ::std::string feature_name =
            this->m_receptacle_name;

        ::CORBA::Object_var target =
              this->m_facet_object;

        ::TestComponents::FacetTest_var
              facet_object;
              
        ::CORBA::Long __result =0;
        MakeOperationStubRequest_impl request
            (
             feature_name.c_str(),
             m_component_identity.c_str(),
             target.in(),
             m_context
             );
       
        if (request.is_read_write()){   
            request.add_in_args_oper(oper);
            request.add_in_args_operand1(operand1);
            request.add_in_args_operand2(operand2);
            std::cout << "xxx request.add_out_args_remainder(remainder); " << remainder << std::endl;
            request.add_out_args_remainder(remainder);
            request.add_exception_ExceptionTest_tc();

            request.add_result_value(__result);
       
            request.pre_invoke();
            request.invoke();
            request.post_invoke();

            ::CORBA::Any_var ex =
                  request.the_exception();
       
            if(::CORBA::tk_null != ex->type()->kind()) {
                if(ex->type()->equal(::TestComponents::_tc_ExceptionTest)) {
                    ::TestComponents::ExceptionTest * user_exception = 0;
                    if(!(ex.in() >>= user_exception)) {
                        CDMW_ASSERT(0);
                    }
                    
                    throw *user_exception;
                } else {
                    throw request.the_systemException();
                }
            }                                       
            
                         
            __result = request.result_value();
            remainder = request.get_out_args_remainder();

       
        } else { 
            request.pre_invoke();
            try {
                facet_object = ::TestComponents::FacetTest::_narrow (target);

            } catch (::CORBA::BAD_PARAM & e) {
                CDMW_ERROR(m_logger, 
                           "The facet object has wrong type");

                ::CORBA::OBJECT_NOT_EXIST ex
                      (
                       ::Cdmw::OrbSupport::OBJECT_NOT_EXIST, 
                       ::CORBA::COMPLETED_NO
                       );
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
               
            try{  
                __result =facet_object->makeOperation
                    (
                     oper,
                     operand1,
                     operand2,
                     remainder
                     );
            } 
            catch(::CORBA::SystemException & se){
                request.add_system_exception(se);
            }
            catch(::TestComponents::ExceptionTest & e) {
                request.add_exception_ExceptionTest(e);
            }

      
            request.post_invoke();
            request.throw_exception();
         
        } //end of else
        
        
        return __result;
        
           
    
    }
                 
                
} // end of namespace Cdmw

} // end of namespace CCM

} // end of namespace CIF

} // end of namespace CdmwTestComponents


