/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



///// FILE TO BE GENERATED


#include <Foundation/logging/FunctionLogger.hpp>   
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include "ReceptacleHello_impl.hpp"
#include "EchoStubRequest_impl.hpp"

   

namespace Cdmw {

namespace CCM {

namespace CIF {

namespace CdmwTestComponents {




   ReceptacleHello_impl::ReceptacleHello_impl
    (
     const ::std::string & receptacle_name,
     const ::std::string & component_id,
     ::CORBA::Object_ptr facet_object,
     ::Cdmw::CCM::CIF::Context * context,
     CdmwLogging::Logger_ptr     logger
     ) 
         : m_receptacle_name(receptacle_name),
           m_component_identity(component_id),
           m_facet_object(facet_object),
           m_context(context),
           m_logger(logger)
           
    {
        CDMW_LOG_FUNCTION(m_logger, "");

    };
    
    ReceptacleHello_impl::~ReceptacleHello_impl() throw() 
    {
        // empty
          CDMW_LOG_FUNCTION(m_logger, "");

    }

                 
         
    void ReceptacleHello_impl::echo
    (                 
     const char * msg
     ) throw (CORBA::SystemException)
    {               
        CDMW_LOG_FUNCTION(m_logger, "");


        const std::string feature_name = this->m_receptacle_name;
        ::CORBA::Object_ptr target = m_facet_object;
        ::HelloModule::Hello_var facet_object;

              
        EchoStubRequest_impl request
            (            
             feature_name.c_str(),
             m_component_identity.c_str(),
             target, 
             m_context
             );
       
        if (true == request.is_read_write()) {   
            request.add_in_args_msg(msg);

            request.pre_invoke();
            request.invoke();
            request.post_invoke();

            request.throw_exception();
                              
        } else { 
            request.pre_invoke();
            try {
                facet_object = ::HelloModule::Hello::_narrow (target);

            } catch (::CORBA::BAD_PARAM & e) {
                CDMW_ERROR(m_logger, 
                           "The facet object has wrong type");

                ::CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
               
            try{  
                facet_object->echo
                    (
                     msg
                     );
            } 
            catch(::CORBA::SystemException & se){
                request.add_system_exception(se);
                request.post_invoke();
                throw;
            }


            request.post_invoke();
            request.throw_exception();
                                 
        } //end of else
           
    }
                
} // end of namespace Cdmw

} // end of namespace CCM

} // end of namespace CIF

} // end of namespace CdmwTestComponents


