/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import org.junit.Test;
import org.omg.Components.CCMException;
import org.omg.Components.ConfigValue;
import org.omg.Components.SessionContext;
import org.omg.Components.Deployment.ComponentServer;
import org.omg.Components.Deployment.ComponentServerHelper;

import com.thalesgroup.CdmwCcmContainer.ApplicationElementContextService;
import com.thalesgroup.CdmwCcmContainer.ApplicationElementContextServiceHelper;
import com.thalesgroup.CdmwCcmContainer.ComponentElementContext;
import com.thalesgroup.CdmwCcmContainer.ComponentServerContext;

import cdmw.ccm.cif.ComponentIdentity;
import cdmw.ccm.container.ApplicationElementsContextService;
import cdmw.testutils.CORBATestManager;
import static org.junit.Assert.*;

/**
 *
 */
public class ApplicationElementsContextServiceTest {


    @Test
    public void doTests() {
        System.out
                .println("*************************************************************\n"
                        + "       applicationElementsContextServiceTest.class Test\n"
                        + "*************************************************************");

        try {
            SessionContext context = org.omg.Components.SessionContextHelper
                    .narrow(CORBATestManager.instance().getLocalProperty(
                            "ClientSessionContext"));
            assertTrue(context != null);

            org.omg.CORBA.Object obj = context
                    .resolve_service_reference(ApplicationElementsContextService.ApplicationElementsContextServiceId);
            assertTrue(obj != null);
            ApplicationElementContextService svc = ApplicationElementContextServiceHelper.narrow(obj);
            ComponentServerContext cs_ctx = svc.get_component_server_context();
            System.out.println("Getting the ComponentServer context : ");
            assertTrue(cs_ctx.ref() != null);
            ComponentServer cs = ComponentServerHelper.narrow(cs_ctx.ref());
            assertTrue(cs !=null);
            System.out.println("===> the CS ref is not null");
            System.out.println("===> the CS id : "+cs_ctx.id());
            System.out.println("===> the CS config vlaues : "+configValuesToString(cs_ctx.configuration()));

            try{
                ComponentElementContext cts = svc.find_component_context_by_id("");
            }
            catch(CCMException ex){
                assertTrue("Expected exception was not thrown !", true);
            }
            // NOTE : To take care to give an INS name
            //        cdmw.commonsvcs.naming.NamingInterface.toString method can be used
            System.out.println("Getting the Client Component Context : ");
            String name = ComponentIdentity.toString("CLIENT_COMPONENT_NAME2",cs_ctx.id());
            ComponentElementContext cts = svc.find_component_context_by_id
                    (name);
            assertTrue(cts != null);
            assertTrue(cts.ref() !=null);
            System.out.println("===> the Client Component ref is not null");
            System.out.println("===> the Client Component id : "+cts.id());
            System.out.println("===> its config vlaues : "+configValuesToString(cts.configuration()));

            System.out.println("Getting the Server Component Context : ");
            name = ComponentIdentity.toString("SERVER_COMPONENT_NAME",cs_ctx.id());
            cts = svc.find_component_context_by_id
                    (name);
            assertTrue(cts != null);
            assertTrue(cts.ref() !=null);
            System.out.println("===> the Server Component ref is not null");
            System.out.println("===> the Server Component id : "+cts.id());
            System.out.println("===> its config vlaues : "+configValuesToString(cts.configuration()));



        } catch (Exception ex) {
            ex.printStackTrace();
            fail();
        }

        System.out
        .println("*************************************************************\n"
                + "       End of applicationElementsContextServiceTest.class Test\n"
                + "*************************************************************");

    }

    String configValuesToString(ConfigValue[] config){
        String str ="";

        if(config==null) return str;

        for(ConfigValue c : config){
            str = str + "/" + c.name ;
        }

        return str;

    }

}

