/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import static org.junit.Assert.*;

import java.lang.reflect.Array;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.junit.Before;
import org.junit.Test;
import org.omg.Components.Cookie;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.ContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.Components.ContainerPortableInterceptor.InvalidRegistration;

import cdmw.ccm.ccmcopi.COPIUtils;
import cdmw.orbsupport.ORBSupport;
import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;



/**
 * a wrapper class of the tested class cdmw.ccm.ccmcopi.COPIRegistration
 */
class COPIRegistration extends cdmw.ccm.ccmcopi.COPIRegistration{
    
    private static COPIRegistration theInstance= null;
    
    private COPIRegistration(){
        super(0,0);
    }
    
    public static COPIRegistration instance(){
        if(theInstance == null)
            theInstance = new COPIRegistration();
        return theInstance;
    }
}


public class COPIRegistrationTest {
    
    public static void main(String[] args){
        
        cdmw.logging.LogManager.Init(args);
        
        CORBATestManager testManager = CORBATestManager.instance();
        testManager.init("COPI REGISTRATION TEST", null, null);
        testManager.addTest(COPIRegistrationTest.class);
        testManager.runTests();
        testManager.printSummary();
    }

    @Before
    public void init() throws InvalidObject {
    }

    @Test
    public void doTests() {
        
        System.out.println("*************************************************************\n"+
                "       COPIRegistrationTest.class Test\n"+
                "*************************************************************");
        
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

        ClientContainerInterceptor clientCOPI1 = new LoggerClientCOPIImpl("1"), 
        clientCOPI2 = new LoggerClientCOPIImpl("2"), 
        clientCOPI3 = new LoggerClientCOPIImpl("3"), 
        clientCOPI4 = new LoggerClientCOPIImpl("4");
        
        String[][] ips = new String[][]{{"",""}, 
                                        {"port1",""},
                                        {"", "op1"},
                                        {"port1","op1"},
                                        {"port2","op2"},
                                        {"port3",""},
                                        {"", "op3"}};
        
        ClientContainerInterceptor[] clientCOPIs = new ClientContainerInterceptor[56];
        Cookie[] cks = new Cookie[clientCOPIs.length];
               
        int k = 0;
        for(int i = 0; i<ips.length; i++){
            for(short j=0 ; j<4 ; j++){
                clientCOPIs[k] = new LoggerClientCOPIImpl(Integer.toString(5+k),j, ips[i][0], ips[i][1] );
                k++;
                clientCOPIs[k] = new LoggerClientCOPIImpl(Integer.toString(5+k),j, ips[i][0], ips[i][1] );
                k++;
            }
        }

        System.out.println("@@@@@@@@@ Testing client COPIs registration");
        Cookie ck1 = COPIRegistration.instance().register_client_interceptor(
                clientCOPI1);
        assertTrue(ck1 != null);
        try{
            System.out.println("@@@@@@@@@ Testing duplicate COPIs registration");
            COPIRegistration.instance().register_client_interceptor(
                    clientCOPI1);
        }catch(org.omg.CORBA.BAD_PARAM bp){
            assertTrue(true);
            System.out.println("Exception raised: "+ORBSupport.exceptionToString(bp));
        }
        Cookie ck2 = COPIRegistration.instance().register_client_interceptor(
                clientCOPI2);
        assertTrue(ck2 != null);
        Cookie ck3 = COPIRegistration.instance().register_client_interceptor(
                clientCOPI3);
        assertTrue(ck3 != null);
        Cookie ck4 = COPIRegistration.instance().register_client_interceptor(
                clientCOPI4);
        assertTrue(ck4 != null);

        ClientContainerInterceptor[] interceptors = COPIRegistration.instance()
                .getAllClientContainerInterceptors();
        assertTrue("Unexpected registered interceptors number: " + interceptors.length, interceptors.length == 4);
        
        COPIsUsageSimulationThread t = new COPIsUsageSimulationThread();
        t.start();
        
        try {
            OS.sleep(10000);
            System.out.println("@@@@@@@@@ Testing client COPIs unregistration");
            String time = sdf.format(new Date(System.currentTimeMillis()));
            System.out.println(time + "\t Interceptor 3 will be unregistered");
            ClientContainerInterceptor copi = COPIRegistration.instance()
                    .unregister_client_interceptor(ck3);
            assertTrue(copi != null && copi.equals(clientCOPI3));
            time = sdf.format(new Date(System.currentTimeMillis()));
            System.out.println(time + "\t Interceptor 3 has been unregistered");
            time = sdf.format(new Date(System.currentTimeMillis()));
            System.out
                    .println(time
                            + "\t Interceptor 3 will be unregistered again (expected exception)");
            COPIRegistration.instance().unregister_client_interceptor(ck3);

        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InvalidRegistration e) {
            assertTrue(true);
        }

        interceptors = COPIRegistration.instance()
                .getAllClientContainerInterceptors();
        assertTrue(interceptors.length == 3);
        System.out.println("3 client interceptors left");
        try {
            System.out.println(sdf.format(new Date(System.currentTimeMillis()))
                    + "\t Interceptor 4 will be unregistered");
            ClientContainerInterceptor copi = COPIRegistration.instance()
                    .unregister_client_interceptor(ck4);
            assertTrue(copi != null && copi.equals(clientCOPI4));
            System.out.println(new Date(System.currentTimeMillis())
                    + "\t Interceptor 4 has been unregistered");

            interceptors = COPIRegistration.instance()
                    .getAllClientContainerInterceptors();
            assertTrue(interceptors.length == 2);
            System.out.println("2 client interceptors left");

            System.out.println(sdf.format(new Date(System.currentTimeMillis()))
                    + "\t Interceptor 2 will be unregistered");
            copi = COPIRegistration.instance().unregister_client_interceptor(
                    ck2);
            assertTrue(copi != null && copi.equals(clientCOPI2));
            System.out.println(sdf.format(new Date(System.currentTimeMillis()))
                    + "\t Interceptor 2 has been unregistered");

            interceptors = COPIRegistration.instance()
                    .getAllClientContainerInterceptors();
            assertTrue(interceptors.length == 1);
            System.out.println("1 client interceptor left");

            System.out.println(sdf.format(new Date(System.currentTimeMillis()))
                    + "\t Interceptor 1 will be unregistered");
            copi = COPIRegistration.instance().unregister_client_interceptor(
                    ck1);
            assertTrue(copi != null && copi.equals(clientCOPI1));
            System.out.println(sdf.format(new Date(System.currentTimeMillis()))
                    + "\t Interceptor 1 has been unregistered");

            interceptors = COPIRegistration.instance()
                    .getAllClientContainerInterceptors();
            assertTrue(interceptors.length == 0);
            System.out.println("No client interceptor left");
            t.stopThread();
            
            System.out.println("@@@@@@@@@ Testing COPIs ordering by priority");
            for(int i=0;i < clientCOPIs.length; i++){
                cks[i] = COPIRegistration.instance().register_client_interceptor(clientCOPIs[i]);
            }
            interceptors = COPIRegistration.instance().getAllClientContainerInterceptors();
            assertTrue("Unexpected number of registered COPIs : "+interceptors.length,
                    interceptors.length == clientCOPIs.length);
            
            for(int i = 0; i<ips.length; i++){
                interceptors = COPIRegistration.instance().getClientInterceptorByIPOrderedByPriority(new IntegrationPoint(ips[i][0], ips[i][1]));
                short priority = Short.MAX_VALUE;
                for(int j =0; j<interceptors.length;j++){
                    assertTrue(interceptors[j].priority() <= priority);
                    priority = interceptors[j].priority();
                }

            }
           
            System.out.println("@@@@@@@@@ Testing addClientCOPIRef operation call " +
                        "when a COPI is unregistered");
            interceptors = COPIRegistration.instance().getAllClientContainerInterceptors();
            System.out.println("There are "+interceptors.length +" client interceptors registered");

            ClientContainerInterceptor missing = 
                COPIRegistration.instance().unregister_client_interceptor(cks[2]);
            
            //This loop is extracted from the Client/ServerRequestInterceptorIterator classes
            for (int i = 0; i < interceptors.length; i++) {
                if(COPIRegistration.instance().addClientCOPIRef(interceptors[i]) != 0) {
                    assertTrue("Unexpected missing copi n°"+i, missing.equals(interceptors[i]));
                    interceptors = COPIUtils.removeArrayElement(interceptors, i);
                    i--;
                }
            }
            assertEquals(interceptors.length,COPIRegistration.instance().getAllClientContainerInterceptors().length);
            for (int i = 0; i < interceptors.length; i++) {
                assertTrue(COPIRegistration.instance().removeClientCOPIRef(interceptors[i]) == 0);
            }
            for (int i = 0; i < cks.length; i++) {
                if(i!=2) COPIRegistration.instance().unregister_client_interceptor(cks[i]);
            }
            
        } catch (InvalidRegistration e) {
            e.printStackTrace();
            fail();
        }
        
        System.out.println("*************************************************************\n"+
                "       End COPIRegistrationTest.class Test\n"+
                "*************************************************************");
    }

}

class COPIsUsageSimulationThread extends Thread {

    ClientContainerInterceptor[] interceptors;
    boolean isStopped = false;
 

    public void run() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

        try {
            while (!isStopped) {
                interceptors = COPIRegistration.instance().getAllClientContainerInterceptors();
                
                // the COPIs are used in a request processing
                for (int i = 0; i < interceptors.length; i++) {
                    COPIRegistration.instance().addClientCOPIRef(interceptors[i]);
                    String time = sdf.format(new Date(System
                            .currentTimeMillis()));
                    System.out.println(time + "\t Interceptor " + (i + 1)
                            + " use count ++");
                    COPIRegistration.instance().addClientCOPIRef(interceptors[i]);
                    time = sdf.format(new Date(System
                            .currentTimeMillis()));
                    System.out.println(time + "\t Interceptor " + ((short)i + 1)
                            + " use count ++");
                    COPIRegistration.instance().addClientCOPIRef(interceptors[i]);
                    time = sdf.format(new Date(System
                            .currentTimeMillis()));
                    System.out.println(time + "\t Interceptor " + (i + 1)
                            + " use count ++");
                    
                    OS.sleep(2000);
                }

                for (int i = interceptors.length - 1; i > -1; i--) {
                    COPIRegistration.instance().removeClientCOPIRef(interceptors[i]);
                    String time = sdf.format(new Date(System
                            .currentTimeMillis()));
                    System.out.println(time + "\t Interceptor " + (i + 1)
                            + " use count --");
                    OS.sleep(2000);
                }
                interceptors = COPIRegistration.instance().getAllClientContainerInterceptors();
                
                for (int i = interceptors.length - 1; i > -1; i--) {
                    COPIRegistration.instance().removeClientCOPIRef(interceptors[i]);
                    String time = sdf.format(new Date(System
                            .currentTimeMillis()));
                    System.out.println(time + "\t Interceptor " + (i + 1)
                            + " use count --");
                    OS.sleep(2000);
                }
                
                interceptors = COPIRegistration.instance().getAllClientContainerInterceptors();
                System.out.println(sdf.format(new Date(System
                        .currentTimeMillis())) +"\t"+interceptors.length+" Interceptors left");
                
                for (int i = interceptors.length - 1; i > -1; i--) {
                    COPIRegistration.instance().removeClientCOPIRef(interceptors[i]);
                    String time = sdf.format(new Date(System
                            .currentTimeMillis()));
                    System.out.println(time + "\t Interceptor " + (i + 1)
                            + " use count --");
                    OS.sleep(2000);
                }
             // the COPIs are no more used in a request processing
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }
    
    public void stopThread(){
        isStopped = true;
    }
}
