/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;
import org.omg.Components.Cookie;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptorRegistration;

import com.thalesgroup.CdmwCcmCopi.COPICurrent;
import com.thalesgroup.CdmwCcmCopi.COPICurrentHelper;
import com.acme.HelloModule.Hello;

import cdmw.ccm.ccmcopi.COPIRegistration;
import cdmw.ccm.ccmcopi.COPIServiceInit;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;

/**
 *
 */
public class MainTest {
    
    @Before
    public void init() throws InvalidObject {

    }
    
    @Test
    public void doTests() {
        
        org.omg.CORBA.ORB orb = null;

        try {
            String[] args = new String[0]; 
            //init COPI
            COPIServiceInit.init(args);
            assertTrue(COPIServiceInit.isInitialized());
            
            // Initialize the ORB
            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(3);

            orb = ORBSupport.ORBInit(args, orbStrategies);

            //
            // Get the root POA
            //
            org.omg.CORBA.Object obj =
                orb.resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA =
                org.omg.PortableServer.POAHelper.narrow(obj);

            //
            // Run implementation (Activate the POA manager)
            //
            rootPOA.the_POAManager().activate();
            
            // get the COPICurrent to retreive the registration interfaces
            COPICurrent copiCurrent = COPICurrentHelper.narrow(orb.resolve_initial_references("COPICurrent"));
            ClientContainerInterceptorRegistration cci = copiCurrent.get_client_interceptor_registration();
            assertTrue(cci != null);
            
            //register a Client Container interceptor
            ClientContainerInterceptor basic_client_copi=new LoggerClientCOPIImpl("");
            Cookie ck = cci.register_client_interceptor(basic_client_copi);
            
            //create the CORBA Object
            HelloImpl impl=new HelloImpl();
            com.acme.HelloModule.Hello hello_obj=impl._this(orb);
            
            String ior = orb.object_to_string(hello_obj);
            java.io.FileWriter fw = new java.io.FileWriter("Hello.ior");
            fw.write(ior);
            fw.close();
            
            ClientThread client=new ClientThread(orb);
            client.start();
            
            //
            // Run the ORB
            //
            orb.run();
            
            ClientContainerInterceptor registered_copi = cci.unregister_client_interceptor(ck);
            assertTrue(basic_client_copi._is_equivalent(registered_copi));

            
        } catch (Exception e) {
            e.printStackTrace();
        }

        if(orb != null) {
            try {
                //cleanup
                
                orb.destroy();
            } catch(Exception e) {
                e.printStackTrace();
            }
        }

    }

}

class ClientThread extends Thread{
    
    org.omg.CORBA.ORB orb = null;
    
    public ClientThread(org.omg.CORBA.ORB orb){
        this.orb=orb;
    }
    
    public void run(){
        try{
        //waiting for the server orb running
        cdmw.ossupport.OS.sleep(5000);
        
        org.omg.CORBA.Object obj = orb.string_to_object("file://Hello.ior");
       
        com.acme.HelloModule.Hello server = com.acme.HelloModule.HelloHelper.narrow(obj);
        server.echo("Hello,");
        server.echo("Client Component speaking");
        server.echo("How are you ?");
        
        server.echo("bla bla bla ...");
        server.echo("Bye");
        
        orb.shutdown(false);
        
        }
        catch(Exception ex){
            ex.printStackTrace();
        }
        
    }
    
}
