/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testloggingqosenabler;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Before;
import org.junit.Test;
import org.omg.Components.CCMObject;

import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescription;
import com.thalesgroup.CdmwDeployment.ValuetypeFactoryDescriptionSeqHelper;

import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ccm.container.HomeAllocator;
import cdmw.ccm.container.HomeAllocatorRegistry;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;

import com.thalesgroup.CdmwCcmPlugins.NamedValue;

/**
 *
 */
public class ExtensionContainerTest {
    private org.omg.CORBA.ORB orb;

    private org.omg.PortableServer.POA poa;
    
    
    

    @Before
    public void init() throws InvalidObject {
        CORBATestManager testManager = CORBATestManager.instance();

        this.orb = testManager.getORB();
        this.poa = testManager.getPOA();
    }

    @Test
    public void doTests() {

        int timescale = TestManager.getTimescale();
        
        
        HomeAllocator allocator = null;

        // Register HomeAllocator
        
        allocator= new HomeAllocator(
                "Cdmw.CCM.CIF.CdmwTestComponents.SessionClientHome_impl");
        HomeAllocatorRegistry.register(
                "Cdmw.CCM.CIF.CdmwTestComponents.SessionClientHome_impl", allocator);
        allocator= new HomeAllocator(
                "Cdmw.CCM.CIF.CdmwCdmwCcmPlugins.ExtensionCdmwQosEnablerHome_impl");
        HomeAllocatorRegistry.register(
                "Cdmw.CCM.CIF.CdmwCdmwCcmPlugins.ExtensionCdmwQosEnablerHome_impl", allocator);


        System.out.println("Testing The Extension Container ...");

        org.omg.Components.Deployment.ComponentServer componentServer = null;
        System.out.println("get the reference of the component_server");
        try {
            org.omg.CORBA.Object obj = orb
                    .string_to_object("file://componentServer.ior");
            componentServer = org.omg.Components.Deployment.ComponentServerHelper
                    .narrow(obj);

        } catch (org.omg.CORBA.BAD_PARAM e) {
            fail("Error while retrieving component_server reference: "
                    + e.getMessage());
        }

        //Create an extension container
        System.out.println("Creating a container instance ...");
        org.omg.Components.ConfigValue[] cfgCompserver = new org.omg.Components.ConfigValue[0];
        org.omg.Components.Deployment.Container theContainer = null;
        try {
            theContainer = componentServer.create_container(cfgCompserver);
        } catch (org.omg.Components.CreateFailure cf) {
            fail(cf.getMessage());
        } catch (org.omg.Components.Deployment.InvalidConfiguration ic) {
            fail(ic.getMessage());
        }

        System.out
                .println("******************************************************");
        System.out
                .println("************* Deploying LoggingService QoS  *****************");
        System.out
                .println("******************************************************");
        System.out.println("Installing an Extension Home ...");
        String ENTRY_PT = "cdmw.ccm.qosenablers.logging.LoggingServiceComponentHomeImpl.createHomeImpl";
        String UUID_home1 = "LoggingServiceHome";

        com.thalesgroup.CdmwCcmPlugins.CdmwQosEnablerHome cdmwQosEnablerHome = null;
        try {
            // Create a set of ConfigValues.
            org.omg.Components.ConfigValue[] cfg = createConfigValues(
                    com.thalesgroup.CdmwDeployment.ComponentKindValue.EXTENSION,
                    com.thalesgroup.CdmwDeployment.ServantLifetimeValue.COMPONENT_LIFETIME,
                    com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD,
                    "IDL:thalesgroup.com/CdmwCcmPlugins/CdmwQosEnablerHome:1.0",
                    "Cdmw.CCM.CIF.CdmwCdmwCcmPlugins.ExtensionCdmwQosEnablerHome_impl", null);
            System.out.println("Creating " + UUID_home1 + " home...");
            org.omg.Components.CCMHome home = theContainer.install_home(
                    UUID_home1, ENTRY_PT, cfg);

            // logginghome = LoggingQosEnabler.LoggingServiceHomeHelper.narrow(home);
            
            cdmwQosEnablerHome = com.thalesgroup.CdmwCcmPlugins.CdmwQosEnablerHomeHelper.narrow(home);
            
            assertTrue(cdmwQosEnablerHome != null);
            
            // Creating the QoS Enabler component
            com.thalesgroup.CdmwCcmPlugins.CdmwQosEnabler comp1 = null;
            System.out
                    .println("Creating the QoSEnabler component using create()...");

            org.omg.Components.CCMObject cmp = cdmwQosEnablerHome.create();
            // comp1 = LoggingQosEnabler.LoggingServiceHelper.narrow(cmp);
            comp1 = com.thalesgroup.CdmwCcmPlugins.CdmwQosEnablerHelper.narrow(cmp);
            assertTrue(comp1 != null);
            comp1.remove();
            
            org.omg.Components.ConfigValue[] configValues =
                new org.omg.Components.ConfigValue[1];
            org.omg.CORBA.Any value = orb.create_any();
            value.insert_string("LOGGER_COMPONENT_NAME");
            configValues[0] = 
                new cdmw.ccm.container.ConfigValueImpl(
                      com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value, 
                      value);
            System.out
            .println("Creating the QoSEnabler component using create_component_with_config_values()...");
            com.thalesgroup.CdmwCcmCif.KeylessCCMHome cdmwKeylessHome = 
                com.thalesgroup.CdmwCcmCif.KeylessCCMHomeHelper.narrow(home);
            cmp = cdmwKeylessHome.create_component_with_config_values(configValues);
            comp1 = com.thalesgroup.CdmwCcmPlugins.CdmwQosEnablerHelper.narrow(cmp);
            assertTrue(comp1 != null);
            
            // TODO Initialize attributes
            NamedValue[] service_settings = new NamedValue[1];
            service_settings[0] = new NamedValue();
            service_settings[0].name="setting1";
            service_settings[0].value="value1";
            comp1.service_settings(service_settings);

            NamedValue[] service_statistics = new NamedValue[1];
            service_statistics[0] = new NamedValue();
            service_statistics[0].name="statistic1";
            service_statistics[0].value="value1";
            comp1.service_statistics(service_statistics);

            comp1.configuration_complete();
        } catch (Exception ex) {
            ex.printStackTrace();
            fail("ex = " + ex.toString());
        }



        System.out
                .println("******************************************************");
        System.out
                .println("************* Application Deployment  ****************");
        System.out
                .println("******************************************************");
        
      /* 
        ValuetypeFactoryDescription[] valueTypeFactorySeq = new ValuetypeFactoryDescription[2];
        ValuetypeFactoryDescription desc = new ValuetypeFactoryDescription("IDL:acme.com/TestComponents/Activated:1.0",
            "cdmw.ccm.testloggingqosenabler.ActivatedEventFactoryImpl.createActivatedFactory");
        valueTypeFactorySeq[0] = desc;
        desc = new ValuetypeFactoryDescription("IDL:acme.com/TestComponents/ServerInfo:1.0",
                "cdmw.ccm.testloggingqosenabler.ServerInfoFactoryImpl.createServerInfoFactory");
        valueTypeFactorySeq[1] = desc;
*/
        
        try {
            System.out.println("Installing the Client component Home ...");
            ENTRY_PT = "cdmw.ccm.testloggingqosenabler.ClientComponentHomeImpl.createHomeImpl";
            UUID_home1 = "ClientHome";
            com.acme.TestComponents.ClientHome clienthome = null;

            // Create a set of ConfigValues.
/*            org.omg.Components.ConfigValue[] cfg = createConfigValues(
                    com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION,
                    com.thalesgroup.CdmwDeployment.ServantLifetimeValue.COMPONENT_LIFETIME,
                    com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD,
                    "IDL:thalesgroup.com/TestComponents/ClientHome:1.0",
                    "Cdmw.CCM.CIF.CdmwTestComponents.SessionClientHome_impl", valueTypeFactorySeq);
*/

            org.omg.Components.ConfigValue[] cfg = createConfigValues(
                    com.thalesgroup.CdmwDeployment.ComponentKindValue.SESSION,
                    com.thalesgroup.CdmwDeployment.ServantLifetimeValue.COMPONENT_LIFETIME,
                    com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD,
                    "IDL:thalesgroup.com/TestComponents/ClientHome:1.0",
                    "Cdmw.CCM.CIF.CdmwTestComponents.SessionClientHome_impl", null);

 
            System.out.println("Creating " + UUID_home1 + " home...");
            org.omg.Components.CCMHome home = theContainer.install_home(
                    UUID_home1, ENTRY_PT, cfg);
            clienthome = com.acme.TestComponents.ClientHomeHelper
                .narrow(home);
            assertTrue(clienthome != null);

           System.out.println("Creation a Client Component instance 1 ...");
           org.omg.Components.ConfigValue[] configValues = new org.omg.Components.ConfigValue[1];
            org.omg.CORBA.Any value = orb.create_any();
            value.insert_string("CLIENT_COMPONENT_NAME1");
            configValues[0] = 
                new cdmw.ccm.container.ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value, 
                        value);
            com.thalesgroup.CdmwCcmCif.KeylessCCMHome cdmwKeylessHome = 
                com.thalesgroup.CdmwCcmCif.KeylessCCMHomeHelper.narrow(clienthome);
            CCMObject comp = cdmwKeylessHome.create_component_with_config_values(configValues);

            com.acme.TestComponents.Client clientComp1 = com.acme.TestComponents.ClientHelper.narrow(comp);

            System.out.println("Creation a Client Component instance 2 ...");
            configValues =
                new org.omg.Components.ConfigValue[1];
            value = orb.create_any();
            value.insert_string("CLIENT_COMPONENT_NAME2");
            configValues[0] = 
                new cdmw.ccm.container.ConfigValueImpl(
                        com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value, 
                        value);

            comp = cdmwKeylessHome.create_component_with_config_values(configValues);
            
            com.acme.TestComponents.Client clientComp2 = com.acme.TestComponents.ClientHelper.narrow(comp);

            // System.out.println("Removing the Client Component instance 1 ...");
            // clienthome.remove_component(clientComp1);


            System.out.println("configuration_completeE ...");
            clientComp1.configuration_complete();
            clientComp2.configuration_complete();

        } catch (Exception ex) {
            fail("ex = " + ex.toString());
        }
        
    }

    private org.omg.Components.ConfigValue[] createConfigValues(
            com.thalesgroup.CdmwDeployment.ComponentKindValue componentKind,
            com.thalesgroup.CdmwDeployment.ServantLifetimeValue servantLifetime,
            com.thalesgroup.CdmwDeployment.ThreadingPolicyValue threadingPolicy,
            String repid, String classname, ValuetypeFactoryDescription[] seq) {

        int nbValues = (seq != null?6:5);
        org.omg.Components.ConfigValue[] cfg = new org.omg.Components.ConfigValue[nbValues];
        int idx = 0;
        org.omg.CORBA.Any value;

        // COMPONENT_KIND
        value = orb.create_any();
        com.thalesgroup.CdmwDeployment.ComponentKindValueHelper.insert(value,
                componentKind);
        cfg[idx++] = new ConfigValueImpl(
                com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, value);

        // HOME_REPOSITORY_ID
        value = orb.create_any();
        value.insert_string(repid);
        cfg[idx++] = new ConfigValueImpl(
                com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value, value);

        // HOME_SERVANT_CLASSNAME
        value = orb.create_any();
        value.insert_string(classname);
        cfg[idx++] = new ConfigValueImpl(
                com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                value);

        // SERVANT_LIFETIME
        value = orb.create_any();
        com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper.insert(value,
                servantLifetime);
        cfg[idx++] = new ConfigValueImpl(
                com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value, value);

        // THREADING_POLICY
        value = orb.create_any();
        com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper.insert(value,
                threadingPolicy);
        cfg[idx++] = new ConfigValueImpl(
                com.thalesgroup.CdmwDeployment.THREADING_POLICY.value, value);
        
        //VALUETYPE_FACTORY_DEPENDENCIES
        if(seq != null) {
            value = orb.create_any();
            ValuetypeFactoryDescriptionSeqHelper.insert(value, seq);
            cfg[idx] = new ConfigValueImpl(
            com.thalesgroup.CdmwDeployment.VALUETYPE_FACTORY_DEPENDENCIES.value, value);
        }

        return cfg;

    }
}
