/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testloggingqosenabler;

import org.junit.Assert;

import cdmw.ccm.ccmcopi.COPIServiceInit;
import cdmw.ccm.cif.PushConsumerImpl;
import cdmw.ccm.common.CCMSystemProperties;
import cdmw.common.Locations;
import cdmw.common.Options;
import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.namingandrepository.RepositoryInterface;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.ORBUtils;
import cdmw.orbsupport.StrategyList;
import cdmw.ossupport.OS;
import cdmw.ossupport.ThreadedProcess;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;

/**
 *
 */
public class MainCCMTest {

    public static final String NAMING_SERVICE_PORT_OPTION = "--NamePort";
    public static final String REPOSITORY_PORT_OPTION = "--RepPort";
    public static final int POA_THREAD_POOL_SIZE = 10;

    public static com.thalesgroup.CdmwPlatformMngt.ProcessCallback getProcessCallback(
            org.omg.CORBA.ORB orb) throws org.omg.CORBA.TRANSIENT {

        Assert.assertTrue(orb != null);

        org.omg.CORBA.Object obj = null;
        //
        // Get naming service
        //
        try {
            obj = orb.resolve_initial_references("NameService");
        } catch (org.omg.CORBA.ORBPackage.InvalidName in) {
            System.err.println("Can't resolve `NameService'");
            throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        if (obj == null) {
            System.err.println("`NameService' is a nil object reference");
            throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        org.omg.CosNaming.NamingContextExt nc = org.omg.CosNaming.NamingContextExtHelper
                .narrow(obj);

        if (nc == null) {
            System.err
                    .println("`NameService' is not a NamingContextExt object reference");
            throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        String PROCESS_CALLBACK_BINDING_NAME = "CDMW.I/process-callback.simulated";

        // Get reference to the ProcessCallback object
        com.thalesgroup.CdmwPlatformMngt.ProcessCallback callback = null;
        try {
            NamingInterface ni = new NamingInterface(nc);
            org.omg.CORBA.Object obj__ = ni
                    .resolve(PROCESS_CALLBACK_BINDING_NAME);
            callback = com.thalesgroup.CdmwPlatformMngt.ProcessCallbackHelper
                    .narrow(obj__);
            if (callback == null) {
                System.err
                        .println("Couldn't retrieve reference of the ProcessCallback");
                // TODO: Update minor code
                throw new org.omg.CORBA.TRANSIENT(
                        ExceptionMinorCodes.TRANSIENTCannotReachApplicationAgent,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
        } catch (Exception e) {
            System.err
                    .println("Couldn't retrieve reference of the ProcessCallback");
            // TODO: Update minor code
            throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachApplicationAgent,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        return callback;
    }

    public static void initRepository(org.omg.CORBA.ORB orb) throws Exception {

        //
        // Get naming service
        //
        org.omg.CORBA.Object obj = null;

        try {
            obj = orb.resolve_initial_references("NameService");
        } catch (org.omg.CORBA.ORBPackage.InvalidName in) {
            System.err.println("Can't resolve `NameService'");
            throw in;
        }

        if (obj == null) {
            System.err.println("`NameService' is a nil object reference");
            throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        org.omg.CosNaming.NamingContextExt nc = null;
        try {
            nc = org.omg.CosNaming.NamingContextExtHelper.narrow(obj);
        } catch (Exception e) {
            System.err.println("Obj is not a NamingContextExt");
            throw e;
        }

        if (nc == null) {
            System.err
                    .println("`NameService' is not a NamingContextExt object reference");
            throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        String REPOSITORY_NAME = "CDMW.I/repository.simulated";

        // Get reference to the repository interface
        com.thalesgroup.CdmwNamingAndRepository.Repository rep = null;
        try {
            NamingInterface ni = new NamingInterface(nc);
            org.omg.CORBA.Object obj__ = ni.resolve(REPOSITORY_NAME);

            rep = com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper
                    .narrow(obj__);
            if (rep == null) {
                System.err
                        .println("Couldn't retrieve reference of the Repository");
                throw new org.omg.CORBA.TRANSIENT(
                        ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("Couldn't retrieve reference of the Repository");
            throw new org.omg.CORBA.TRANSIENT(
                    ExceptionMinorCodes.TRANSIENTCannotReachRepository,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        try {
            RepositoryInterface.init(Locations.CDMW_SERVICES_NAME_DOMAIN, rep);
        } catch (Exception e) {
            System.err.println("Couldn't init Repository");
            throw e;
        }
    }

    public static void main(String[] args) {

        org.omg.CORBA.ORB orb = null;
        int timescale = TestManager.getTimescale();

        if (args.length >= 1) {
            if (args[0].equals("--help")) {
                System.out
                        .println("Usage:\n"
                                + "java cdmw.ccm.testccmcopi.MainCCMTest options\n"
                                + "\n"
                                + "Options:\n"
                                + "--help                     Display this information.\n"
                                + "\n"
                                + "[--NamePort=<nbr>          Port of Name Service.\n"
                                + "[--RepPort=<nbr>           Port of Name and Repository.\n");
                return;
            }
        }

        try {

            // get Name Service port
            String nameServicePort = OS.getOptionValue(args,
                    NAMING_SERVICE_PORT_OPTION);

            if (nameServicePort.equals("no")) {
                nameServicePort = "5000";
            }

            // get Repository port
            String repositoryPort = OS.getOptionValue(args,
                    REPOSITORY_PORT_OPTION);

            if (repositoryPort.equals("no")) {
                repositoryPort = "5005";
            }

            // Initialize the ORB
            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            String s = ORBUtils.NAMING_PORT_OPTION + "corbaloc::localhost:"
                    + nameServicePort + "/NameService";
            String[] serverArgs = s.split(" ");

            // init COPI
            COPIServiceInit.init(args);

            orb = ORBSupport.ORBInit(serverArgs, orbStrategies);
            cdmw.logging.LogManager.Init(args);
            // PCR-0049
            cdmw.orbsupport.Codec.init(orb);

            System.out.println("Starting Name Service on port "
                    + nameServicePort + "...");

            ThreadedProcess naming = ORBUtils
                    .startNamingService(nameServicePort);
            OS.sleep(timescale * 3000);

            // Starting the Repository Simulator :
            String namingCorbaloc = " -ORBInitRef NameService=corbaloc::localhost:"
                    + nameServicePort + "/NameService";

            String simRepositoryArgs = " -ORBEndpoint iiop://localhost: "
                    + repositoryPort + namingCorbaloc + " -DOM "
                    + Locations.CDMW_SERVICES_NAME_DOMAIN + " -DOM "
                    + Locations.CDMW_EVENT_SERVICE_NAME_DOMAIN
                    + " -DOM CDMW/SERVICES/EVENT/FACTORIES"
                    + " -DOM CDMW/SERVICES/EVENT/EVENT_CHANNELS"
                    + " -DOM CDMW/SERVICES/EVENT/TYPED_EVENT_CHANNELS"
                    + " -ROOT " + Locations.ADMIN_ROOT_CONTEXT_ID;

            System.out
                    .println("Starting Simulated Repository with the following arguments:");
            System.out.println(simRepositoryArgs);

            ThreadedProcess repository = OS
                    .createProcess("cdmw_simulated_repository "
                            + simRepositoryArgs);
            OS.sleep(timescale * 10000);
            System.out.println("Simulated Repository started");

            // init repository
            initRepository(orb);
            OS.sleep(timescale * 3000);
            System.out.println("Simrepository initialised");

            // Getting callback IOR
            com.thalesgroup.CdmwPlatformMngt.ProcessCallback callback = getProcessCallback(orb);
            String callbackIOR = orb.object_to_string(callback);

            // Get the root POA
            //
            org.omg.CORBA.Object obj = orb
                    .resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA = org.omg.PortableServer.POAHelper
                    .narrow(obj);

            //
            // Run implementation (Activate the POA manager)
            //
            System.out.println("Run implementation (Activate the POA manager)");
            rootPOA.the_POAManager().activate();

            OS.sleep(timescale * 1000);
            // Starting the EventChannelManager
            String profileManagerService = "cdmw_event_channel_manager";
            String managerArgs = " " + " " + Options.CALLBACK_IOR_OPTION + "="
                    + callbackIOR + " " + Options.PROCESS_PORT_OPTION
                    + "=15327" + " " + Options.LOCALISATION_SERVICE_OPTION
                    + "=15328" + " " + Options.PROCESS_XML_FILE_OPTION
                    + "=./EventChannelManager.xml"
                    + " --ProfileXMLFile=EventChannelConf1.xml" + " "
                    + Options.VALIDATE_XML_OPTION ;//+ " --CdmwLogLevel=ALL -ORBDebugLevel 10 --CdmwLogFile=EM.log";

            System.out
                    .println("Starting EventChannelManager with the following parameters: ");
            System.out.println(managerArgs);
            ThreadedProcess profileManager = OS
                    .createProcess(profileManagerService + managerArgs);
            OS.sleep(timescale * 20000);
            System.out.println("EventChannelManager started");

            // EventChannelManager Proc URL is CDMW Test Application/PROC_000
            com.thalesgroup.CdmwPlatformMngt.ProcessDelegate proc = null;
            String procURL = "corbaname::localhost:"
                    + nameServicePort
                    + "#CDMW.I/ProcessCallback.simulated/CDMW Test Application/PROC_000";
            obj = orb.string_to_object(procURL);

            if (obj != null) {
                proc = com.thalesgroup.CdmwPlatformMngt.ProcessDelegateHelper
                        .narrow(obj);
            } else {
                System.exit(-1);
            }

            cdmw.tools.ProcessAdmin admin = new cdmw.tools.ProcessAdmin(orb,
                    proc);

            // send initialise order
            java.io.BufferedReader commands = new java.io.BufferedReader(
                    new java.io.StringReader("init LAST 2\n exit \n"));
            admin.run(commands, System.out);
            OS.sleep(timescale * 5000);

            // send run order
            commands = new java.io.BufferedReader(new java.io.StringReader(
                    "run\n exit \n"));
            admin.run(commands, System.out);
            OS.sleep(timescale * 3000);

            java.lang.String isTypeCheckingOnStr = 
                System.getProperty(CCMSystemProperties.TYPECHECKING, "off");
            System.out.println(CCMSystemProperties.TYPECHECKING
                               + " set to " + isTypeCheckingOnStr);
            
            // declare Value factory to the orb
            org.omg.CORBA.portable.ValueFactory vf;
            vf = new cdmw.ccm.container.ConfigValueFactoryImpl();
            ((org.omg.CORBA_2_3.ORB) orb).register_value_factory(
                    "IDL:omg.org/Components/ConfigValue:1.0", vf);
            vf = new cdmw.ccm.cif.FacetDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB) orb).register_value_factory(
                    "IDL:omg.org/Components/FacetDescription:1.0", vf);
            vf = new cdmw.ccm.common.CookieFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/Cookie:1.0", vf);

            // Creating the Component Installation
            MyComponentInstallation cmpInst = new MyComponentInstallation(
                    rootPOA);
            com.thalesgroup.CdmwDeployment.ComponentInstallation componentInstallation = cmpInst
                    ._this(orb);
            // Creating the Component Server
            MyComponentServer cmpSvr = new MyComponentServer(orb, rootPOA,
                    componentInstallation);
            org.omg.Components.Deployment.ComponentServer componentServer = cmpSvr
                    ._this(orb);
            System.out.println("Activating the Component Server ...");
            String componentServerIOR = orb.object_to_string(componentServer);
            java.io.FileWriter fw = new java.io.FileWriter(
                    "componentServer.ior");
            fw.write(componentServerIOR);
            fw.close();

            TestThread tt = new TestThread(orb, rootPOA);
            tt.start();
            orb.run();

            if (orb != null) {
                try {
                    // send stop order
                    commands = new java.io.BufferedReader(
                        new java.io.StringReader("stop\n exit \n"));
                    System.out.println("Stopping EventChannelManager...");
                    // admin.run(commands, System.out);
                    if (profileManager != null) { OS.killProcess(profileManager);}
                    RepositoryInterface.finish();
                    OS.sleep(timescale * 2000);
                    orb.destroy();
                } catch(Exception ex) {
                    System.err.println(ex.toString());
                } 
            }    
            
            // And we kill launched process
            try {
                if (naming != null) {
                    OS.killProcess(naming);
                }
                if (repository != null) {
                    OS.killProcess(repository);
                }
                if (profileManager != null) {
                    OS.killProcess(profileManager);
                }
                OS.sleep(timescale * 2000);
            } catch (InterruptedException ie) {
                System.err.println(ie.toString());
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}

class TestThread extends Thread {

    private org.omg.CORBA.ORB orb;

    private org.omg.PortableServer.POA poa;

    public TestThread(org.omg.CORBA.ORB orb, org.omg.PortableServer.POA poa) {
        this.orb = orb;
        this.poa = poa;
    }

    public void run() {

        try {
            OS.sleep(TestManager.getTimescale() * 2000);
        } catch (InterruptedException ie) {
        }
      
        CORBATestManager testManager = CORBATestManager.instance();
 
        
        testManager.init("LoggingQoSEnabler Test suite", this.orb, this.poa);
                
        testManager.addTest(ExtensionContainerTest.class);
        testManager.addTest(ClientComponentImpl.class);
        testManager.runTests();
        testManager.printSummary();

        orb.shutdown(false);

    }

}
