package cdmw.ccm.testqosenabler_dds4ccm;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;
import org.omg.CCM_DDS.BadParameter;
import org.omg.CCM_DDS.InternalError;
import org.omg.CCM_DDS.NonExistent;
import org.omg.CCM_DDS.QueryFilter;
import org.omg.CCM_DDS.ReadInfoHolder;
import org.omg.CCM_DDS.ReadInfoSeqHolder;
import org.omg.DDS.Duration_t;

import com.acme.Hello.Donnee;

import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;

public class GetTest {

    @Test
    public void doTests() {
        
        System.out
        .println("******************************************************");
        System.out
        .println("***************** DDS_Get Fragment *******************");
        System.out
        .println("******************************************************");
        
        CORBATestManager testManager = (CORBATestManager)TestManager.instance();

        com.acme.Hello.Donnee_Writer writer
        = (com.acme.Hello.Donnee_Writer)testManager.getLocalProperty(
            "cdmw.ccm.testqosenabler_dds4ccm.p_Write_data");
        
        com.acme.Hello.Donnee_Getter getter
        = (com.acme.Hello.Donnee_Getter)testManager.getLocalProperty(
            "cdmw.ccm.testqosenabler_dds4ccm.p_Get_data");
        
        com.acme.Hello.DonneeSeqHolder instances = new com.acme.Hello.DonneeSeqHolder();
        ReadInfoSeqHolder infos = new ReadInfoSeqHolder();
        ReadInfoHolder info = new ReadInfoHolder();
        boolean status = false;
        
        Duration_t timeout = new Duration_t();
        timeout.nanosec = 200000000;     
        getter.time_out(timeout);
        
        com.acme.Hello.Donnee an_instance = new com.acme.Hello.Donnee();
        
        try {
            an_instance.ID = 1;
            an_instance.msg = "testU";
            writer.write(an_instance);
            
            an_instance.ID = 2;
            an_instance.msg = "testU2";
            writer.write(an_instance);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Writer Error ! Error code = " + e.error_code);
        }
        
        System.out.println("Test get_all");
        try {
            status = getter.get_all(instances, infos);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Getter Error ! Error code = " + e.error_code);
        }
        
        for(int i=0; i < instances.value.length; i++) {
            System.out.println("ID = " + instances.value[i].ID + " | MSG = " + instances.value[i].msg);
        }
        assertTrue("Error on get_all", instances.value.length == 2);
        assertTrue("Error on get_all", (instances.value[0].ID == 1) && instances.value[0].msg.equals("testU"));
        assertTrue("Error on get_all", (instances.value[1].ID == 2) && instances.value[1].msg.equals("testU2"));
        System.out.println("get_all OK");
        
        
        try {
            an_instance = new Donnee(1,"testU");
            writer.write(an_instance);
            
            an_instance = new Donnee(2,"testU2");
            writer.write(an_instance);
            writer.write(an_instance);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Writer Error ! Error code = " + e.error_code);
        }
        
        System.out.println("Test get_all_history");
        try {
            getter.get_all_history(instances, infos);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Getter Error ! Error code = " + e.error_code);
        }
        for(int i=0; i < instances.value.length; i++) {
            System.out.println("ID = " + instances.value[i].ID + " | MSG = " + instances.value[i].msg);
        }
        
        assertTrue("Error on get_all_history", instances.value.length == 3);
        assertTrue("Error on get_all_history", (instances.value[0].ID == 1) && instances.value[0].msg.equals("testU"));
        assertTrue("Error on get_all_history", (instances.value[1].ID == 2) && instances.value[1].msg.equals("testU2"));
        assertTrue("Error on get_all_history", (instances.value[2].ID == 2) && instances.value[2].msg.equals("testU2"));
        System.out.println("get_all_history OK");
        
        
        try {
            an_instance = new Donnee(1,"testU");
            writer.write(an_instance);
            
            an_instance = new Donnee(2,"testU2");
            writer.write(an_instance);
            writer.write(an_instance);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Writer Error ! Error code = " + e.error_code);
        }
        
        
        
        com.acme.Hello.DonneeHolder an_instanceholder = new com.acme.Hello.DonneeHolder();
        an_instanceholder.value = new Donnee();
        an_instanceholder.value.ID = 2;
        
        System.out.println("Test get_one");
        try {
            getter.get_one(an_instanceholder, info);
        } catch (NonExistent e) {
            fail("Getter Error !");
            e.printStackTrace();
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Getter Error ! Error code = " + e.error_code);
        }
        
        assertTrue("Error on get_one", an_instanceholder.value.msg.equals("testU2"));
        System.out.println("get_one OK");
                
        try {
            an_instance = new Donnee(1,"testU");
            
            an_instance = new Donnee(2,"testU2");
            writer.write(an_instance);
            writer.write(an_instance);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Writer Error ! Error code = " + e.error_code);
        }
        
        
        an_instance.ID = 2;
        
        System.out.println("Test get_one_history");
        try {
            getter.get_one_history(an_instance, instances, infos);
        } catch (NonExistent e) {
            fail("Getter Error !");
            e.printStackTrace();
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Getter Error ! Error code = " + e.error_code);
        }
        
        for(int i=0; i < instances.value.length; i++) {
            System.out.println("ID = " + instances.value[i].ID + " | MSG = " + instances.value[i].msg);
        }
        
        assertTrue("Error on get_all_history", instances.value.length == 2);
        assertTrue("Error on get_all_history", (instances.value[0].ID == 2) && instances.value[0].msg.equals("testU2"));
        assertTrue("Error on get_all_history", (instances.value[1].ID == 2) && instances.value[1].msg.equals("testU2"));
    
        System.out.println("get_one_history OK");
        
        
        System.out.println("Test get_next");
        try {
            getter.get_next(an_instanceholder, info);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Getter Error ! Error code = " + e.error_code);
        }
        
        assertTrue("Error on get_next", an_instanceholder.value.ID == 1);
        System.out.println("get_next OK");
        
        
        // ---------------- TIMEOUT ------------------
        System.out.println("TIMEOUT TEST");
        
        try {
            status = getter.get_all(instances, infos);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Getter Error ! Error code = " + e.error_code);
        }
        assertTrue("Error on get_all timeout", !status);
        System.out.println("get_all timeout OK");
        
        try {
            status = getter.get_all_history(instances, infos);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Getter Error ! Error code = " + e.error_code);
        }        
        assertTrue("Error on get_all timeout", !status);
        System.out.println("get_all_history timeout OK");
        
        
        try {
            status = getter.get_next(an_instanceholder, info);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Getter Error ! Error code = " + e.error_code);
        }        
        assertTrue("Error on get_next timeout", !status);
        System.out.println("get_next timeout OK");
        

        try {
            status = getter.get_one(an_instanceholder, info);
        } catch (NonExistent e) {
            fail("Getter Error !");
            e.printStackTrace();
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Getter Error ! Error code = " + e.error_code);
        }        
        assertTrue("Error on get_one timeout", !status);
        System.out.println("get_one timeout OK");
        
        
        
        try {
            status = getter.get_one_history(an_instance, instances, infos);
        } catch (NonExistent e) {
            e.printStackTrace();
            fail("Getter Error !");
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Getter Error ! Error code = " + e.error_code);
        }        
        assertTrue("Error on get_one_history timeout", !status);
        System.out.println("get_one_history timeout OK");
        
        
        
        
        
        
        
        QueryFilter filter = new QueryFilter();
        filter.query = "ID = %0";
        filter.query_parameters = new String[1];   
        filter.query_parameters[0] = "" + 2;
        
        try {
            getter.set_filter(filter);
        } catch (BadParameter e) {
            fail("Error on set_filter");
            e.printStackTrace();
        }
        
        try {
            an_instance = new Donnee(1,"testU");
            writer.write(an_instance);
            
            an_instance = new Donnee(2,"testU2");
            writer.write(an_instance);
            writer.write(an_instance);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Writer Error ! Error code = " + e.error_code);
        }
        
        try {
            getter.get_all_history(instances, infos);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Reader Error ! Error code = " + e.error_code);
        }
        
        
        assertTrue("Error on get_all_history with filter", instances.value.length == 2);
        assertTrue("Error on get_all_history with filter", (instances.value[0].ID == 2) && instances.value[0].msg.equals("testU2"));
        assertTrue("Error on get_all_history with filter", (instances.value[1].ID == 2) && instances.value[1].msg.equals("testU2"));
        
        
        filter.query = "";
        
        try {
            getter.set_filter(filter);
        } catch (BadParameter e) {            
            e.printStackTrace();
            fail("Error on set_filter");
        }
        
        
        try {            
  
            an_instance = new Donnee(2,"testU2");
            writer.write(an_instance);
            writer.write(an_instance);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Writer Error ! Error code = " + e.error_code);
        }

        
        try {
            getter.get_all_history(instances, infos);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Reader Error ! Error code = " + e.error_code);
        }
        
        assertTrue("Error on get_all_history", instances.value.length == 3);
        assertTrue("Error on get_all_history", (instances.value[0].ID == 1) && instances.value[0].msg.equals("testU"));
        assertTrue("Error on get_all_history", (instances.value[1].ID == 2) && instances.value[1].msg.equals("testU2"));
        assertTrue("Error on get_all_history", (instances.value[2].ID == 2) && instances.value[2].msg.equals("testU2"));
         
        System.out.println("filter OK");
        System.out.println("Getter OK");
        
        
         
    }
    


}
