/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testqosenabler_dds4ccm;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;

/**
 *
 */
public class MainTest {
    public final static int POA_THREAD_POOL_SIZE = 5;

    public static void main(String[] args) {

        org.omg.CORBA.ORB orb = null;

        try {
            // Initialize the ORB
            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

            orb = ORBSupport.ORBInit(args, orbStrategies);
            cdmw.logging.LogManager.Init(args);

            // declare Value factory to the orb
            org.omg.CORBA.portable.ValueFactory vf;
            vf = new cdmw.ccm.container.ConfigValueFactoryImpl();
            ((org.omg.CORBA_2_3.ORB) orb).register_value_factory(
                    "IDL:omg.org/Components/ConfigValue:1.0", vf);
            vf = new cdmw.ccm.cif.FacetDescriptionFactoryImpl();
            ((org.omg.CORBA_2_3.ORB) orb).register_value_factory(
                    "IDL:omg.org/Components/FacetDescription:1.0", vf);
            vf = new cdmw.ccm.common.CookieFactoryImpl();
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
                "IDL:omg.org/Components/Cookie:1.0", vf);

            //CdmwInterface.cdmwInit(orb, args);

            //
            // Get the root POA
            //
            org.omg.CORBA.Object obj = orb
                    .resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA = org.omg.PortableServer.POAHelper
                    .narrow(obj);

            //
            // Run implementation (Activate the POA manager)
            //
            rootPOA.the_POAManager().activate();

            String server_option = cdmw.ossupport.OS.getOptionValue(args,
                    "--server");
            /*if (server_option.equals("no")) {
                TestThread thr = new TestThread(orb, rootPOA);
                thr.start();
                orb.run();
                thr.join();
            } else {
                // Creating the Component Installtion
                MyComponentInstallation cmpInst = new MyComponentInstallation(
                        rootPOA);
                com.thalesgroup.CdmwDeployment.ComponentInstallation componentInstallation = cmpInst
                        ._this(orb);
                // Creating the Component Server
                MyComponentServer cmpSvr = new MyComponentServer(orb, rootPOA,
                        componentInstallation);
                org.omg.Components.Deployment.ComponentServer componentServer = cmpSvr
                        ._this(orb);

                String componentServerIOR = orb
                        .object_to_string(componentServer);
                java.io.FileWriter fw = new java.io.FileWriter(
                        "componentServer.ior");
                fw.write(componentServerIOR);
                fw.close();

                orb.run();
            }*/

            MyComponentInstallation cmpInst = new MyComponentInstallation(
                    rootPOA);
            com.thalesgroup.CdmwDeployment.ComponentInstallation componentInstallation = cmpInst
                    ._this(orb);
            // Creating the Component Server
            MyComponentServer cmpSvr = new MyComponentServer(orb, rootPOA,
                    componentInstallation);
            org.omg.Components.Deployment.ComponentServer componentServer = cmpSvr._this(orb);

            TestThread thr = new TestThread(orb, rootPOA, componentServer);
            thr.start();
            orb.run();
            thr.join();

        } catch (Exception e) {
            e.printStackTrace();
        }

        if (orb != null) {
            try {
                //CdmwInterface.cdmwCleanup(orb);
                orb.destroy();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        System.out.println("THE END ....");

    }

    public static class TestThread extends Thread {

        private org.omg.CORBA.ORB orb;

        private org.omg.PortableServer.POA poa;
        org.omg.Components.Deployment.ComponentServer cs;

        public TestThread(org.omg.CORBA.ORB orb, org.omg.PortableServer.POA poa) {
            this.orb = orb;
            this.poa = poa;
        }

        public TestThread(org.omg.CORBA.ORB orb, org.omg.PortableServer.POA poa,
                org.omg.Components.Deployment.ComponentServer cs){
             this.orb = orb;
             this.poa = poa;
             this.cs = cs;
        }

        public void run() {

            try {
                OS.sleep(TestManager.getTimescale() * 2000);
            } catch (InterruptedException ie) {
            }

            CORBATestManager testManager = CORBATestManager.instance();

            testManager.init("DDS4CCM QoSEnabler TEST CASES", this.orb,
                    this.poa);
            testManager.setLocalProperty("ComponentServer", this.cs);
            testManager.addTest(DeploymentTest.class);
            testManager.addTest(WriteTest.class);
            testManager.addTest(ReadTest.class);
            testManager.addTest(GetTest.class);
            testManager.addTest(RawListenTest.class);
            testManager.addTest(UpdateTest.class);
            testManager.addTest(StateListenTest.class);
            testManager.addTest(TearDownTest.class);
            testManager.runTests();
            testManager.printSummary();

            orb.shutdown(false);

        }

    }

}

class MyComponentInstallation extends
        com.thalesgroup.CdmwDeployment.ComponentInstallationPOA {

    private org.omg.PortableServer.POA poa;

    public MyComponentInstallation(org.omg.PortableServer.POA poa) {
        this.poa = poa;
    }

    public org.omg.PortableServer.POA _default_POA() {
        return this.poa;
    }

    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/install:1.0
    //
    public void install(String implUUID, String component_loc)
            throws org.omg.Components.Deployment.InvalidLocation,
            org.omg.Components.Deployment.InstallationFailure {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/replace:1.0
    //
    public void replace(String implUUID, String component_loc)
            throws org.omg.Components.Deployment.InvalidLocation,
            org.omg.Components.Deployment.InstallationFailure {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/remove:1.0
    //
    public void remove(String implUUID)
            throws org.omg.Components.Deployment.UnknownImplId,
            org.omg.Components.RemoveFailure {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/get_implementation:1.0
    //
    public String get_implementation(String implUUID)
            throws org.omg.Components.Deployment.UnknownImplId,
            org.omg.Components.Deployment.InstallationFailure {
        return "";
    }

    //
    // IDL:thalesgroup.com/CdmwDeployment/ComponentInstallation/get_valuetypefactory_location:1.0
    //
    public String get_valuetypefactory_location(String implUUID, String repid)
            throws org.omg.Components.Deployment.UnknownImplId,
            com.thalesgroup.CdmwDeployment.UnknownValuetype,
            org.omg.Components.Deployment.InstallationFailure {
        return "";
    }

}
