package cdmw.ccm.testqosenabler_dds4ccm;

import static org.junit.Assert.fail;
import static org.junit.Assert.assertTrue;

import org.junit.Test;
import org.omg.CCM_DDS.BadParameter;
import org.omg.CCM_DDS.InternalError;
import org.omg.CCM_DDS.NonExistent;
import org.omg.CCM_DDS.QueryFilter;
import org.omg.CCM_DDS.ReadInfoHolder;
import org.omg.CCM_DDS.ReadInfoSeqHolder;

import com.acme.Hello.Donnee;
import com.acme.Hello.DonneeSeqHolder;

import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;

public class ReadTest {

    @Test
    public void doTests() {

        System.out
        .println("******************************************************");
        System.out
        .println("***************** DDS_Read Fragment ******************");
        System.out
        .println("******************************************************");

        CORBATestManager testManager = (CORBATestManager)TestManager.instance();

        com.acme.Hello.Donnee_Writer writer
        = (com.acme.Hello.Donnee_Writer)testManager.getLocalProperty(
            "cdmw.ccm.testqosenabler_dds4ccm.p_Write_data");

        com.acme.Hello.Donnee_Reader reader
        = (com.acme.Hello.Donnee_Reader)testManager.getLocalProperty(
            "cdmw.ccm.testqosenabler_dds4ccm.p_RawListen_data");

        com.acme.Hello.DonneeSeqHolder instances = new com.acme.Hello.DonneeSeqHolder();
        ReadInfoSeqHolder infos = new ReadInfoSeqHolder();
        ReadInfoHolder info = new ReadInfoHolder();

        com.acme.Hello.Donnee an_instance = new com.acme.Hello.Donnee();
        an_instance.ID = 2;
        an_instance.msg = "testU2";

        try {
            writer.write(an_instance);
            writer.write(an_instance);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Writer Error ! Error code = " + e.error_code);
        }

        System.out.println("Test read_all");
        try {
            reader.read_all(instances, infos);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Reader Error ! Error code = " + e.error_code);
        }

        for(int i=0; i < instances.value.length; i++) {
            System.out.println("ID = " + instances.value[i].ID + " | MSG = " + instances.value[i].msg);
        }

        assertTrue("Error on read_all", instances.value.length == 2);
        assertTrue("Error on read_all", (instances.value[0].ID == 1) && instances.value[0].msg.equals("testU"));
        assertTrue("Error on read_all", (instances.value[1].ID == 2) && instances.value[1].msg.equals("testU2"));
        System.out.println("read_all OK");

        System.out.println("Test read_all_history");
        try {
            reader.read_all_history(instances, infos);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Reader Error ! Error code = " + e.error_code);
        }

        for(int i=0; i < instances.value.length; i++) {
            System.out.println("ID = " + instances.value[i].ID + " | MSG = " + instances.value[i].msg);
        }


        assertTrue("Error on read_all_history", instances.value.length == 3);
        assertTrue("Error on read_all_history", (instances.value[0].ID == 1) && instances.value[0].msg.equals("testU"));
        assertTrue("Error on read_all_history", (instances.value[1].ID == 2) && instances.value[1].msg.equals("testU2"));
        assertTrue("Error on read_all_history", (instances.value[2].ID == 2) && instances.value[2].msg.equals("testU2"));
        System.out.println("read_all_history OK");


        com.acme.Hello.DonneeHolder an_instanceholder = new com.acme.Hello.DonneeHolder();
        an_instanceholder.value = new Donnee();
        an_instanceholder.value.ID = 2;

        System.out.println("Test read_one");
        try {
            try {
                reader.read_one(an_instanceholder, info);
            } catch (NonExistent e) {
                e.printStackTrace();
            }
        } catch (InternalError e) {
            e.printStackTrace();
            System.out.println(e.toString());
            fail("Reader Error ! Error code = " + e.error_code);
        }

        System.out.println("ID = " + an_instanceholder.value.ID + " | MSG = " + an_instanceholder.value.msg);

        assertTrue("Error on read_one", (an_instanceholder.value.ID == 2) && an_instanceholder.value.msg.equals("testU2"));
        assertTrue("Error on read_one", an_instanceholder.value.msg.equals("testU2"));
        System.out.println("read_one OK");

        an_instance.ID = 2;
        instances = new DonneeSeqHolder();

        System.out.println("Test read_one_history");
        try {
            try {
                reader.read_one_history(an_instance, instances, infos);
            } catch (NonExistent e) {
                e.printStackTrace();
            }
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Reader Error ! Error code = " + e.error_code);
        }

        for(int i=0; i < instances.value.length; i++) {
            System.out.println("ID = " + instances.value[i].ID + " | MSG = " + instances.value[i].msg);
        }

        assertTrue("Error on read_one_history", instances.value.length == 2);
        assertTrue("Error on read_one_history", (instances.value[0].ID == 2) && instances.value[0].msg.equals("testU2"));
        assertTrue("Error on read_one_history", (instances.value[1].ID == 2) && instances.value[1].msg.equals("testU2"));
        System.out.println("read_one_history OK");

        QueryFilter filter = new QueryFilter();
        filter.query = "ID = %0";
        filter.query_parameters = new String[1];
        filter.query_parameters[0] = "" + 2;

        try {
            reader.set_filter(filter);
        } catch (BadParameter e) {
            fail("Error on set_filter");
            e.printStackTrace();
        }

        try {
            reader.read_all_history(instances, infos);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Reader Error ! Error code = " + e.error_code);
        }


        assertTrue("Error on read_all_history with filter", instances.value.length == 2);
        assertTrue("Error on read_all_history with filter", (instances.value[0].ID == 2) && instances.value[0].msg.equals("testU2"));
        assertTrue("Error on read_all_history with filter", (instances.value[1].ID == 2) && instances.value[1].msg.equals("testU2"));


        filter.query = "";

        try {
            reader.set_filter(filter);
        } catch (BadParameter e) {
            e.printStackTrace();
            fail("Error on set_filter");
        }


        try {
            reader.read_all_history(instances, infos);
        } catch (InternalError e) {
            e.printStackTrace();
            fail("Reader Error ! Error code = " + e.error_code);
        }

        assertTrue("Error on read_all_history", instances.value.length == 3);
        assertTrue("Error on read_all_history", (instances.value[0].ID == 1) && instances.value[0].msg.equals("testU"));
        assertTrue("Error on read_all_history", (instances.value[1].ID == 2) && instances.value[1].msg.equals("testU2"));
        assertTrue("Error on read_all_history", (instances.value[2].ID == 2) && instances.value[2].msg.equals("testU2"));

        System.out.println("filter OK");
        System.out.println("Reader OK");

    }

}
