/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testqosenabler_recording;

import org.junit.Before;
import org.junit.Test;

import com.thalesgroup.CdmwCcmPlugins.CdmwQosEnabler;
import com.thalesgroup.CdmwCcmPlugins.CdmwQosEnablerHome;
import com.thalesgroup.CdmwCcmPlugins.CdmwQosEnablerHomeHelper;
import com.thalesgroup.CdmwCcmPlugins.NamedValue;

import cdmw.ccm.container.ConfigValueImpl;
import cdmw.ccm.container.HomeAllocator;
import cdmw.ccm.container.HomeAllocatorRegistry;
import cdmw.ccm.qosenablers.recording.RecordingServiceComponentImpl;
import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.TestManager;
import static org.junit.Assert.*;

/**
 *
 */
public class TestRecordingQoSEnabler {
    private org.omg.CORBA.ORB orb;
    private org.omg.Components.Deployment.Container container;

    public static final String QOS_RECORDING_HOME_REP_ID =
        "IDL:thalesgroup.com/CdmwCcmPlugins/CdmwQosEnablerHome:1.0";
    public static final String QOS_RECORDING_HOME_SERVANT_NAME =
        "Cdmw.CCM.CIF.CdmwCdmwCcmPlugins.ExtensionCdmwQosEnablerHome_impl";
    public static final String QOS_RECORDING_HOME_UID = "RecordingQoSEnablerHome_1";
    public static final String QOS_RECORDING_HOME_ENTRY_POINT =
        "cdmw.ccm.qosenablers.recording.RecordingServiceHomeImpl.createHomeImpl";

    @Before
    public void init() throws Exception {
        CORBATestManager testManager = (CORBATestManager) TestManager
                .getLocalInstance();

        String containerIOR = (String) testManager
                .getLocalProperty("cdmw.ccm.testqosenabler_recording.CONTAINER_IOR");

        this.orb = testManager.getORB();

        org.omg.CORBA.Object obj = this.orb.string_to_object(containerIOR);
        assertTrue(
                "TEST FAILED : exception raised during test initialisation!",
                obj != null);

        this.container = org.omg.Components.Deployment.ContainerHelper
                .narrow(obj);

        try {
            OS.sleep(TestManager.getTimescale() * 3000);
        } catch (InterruptedException ie) {
        }
    }

    @Test
    public void doTests() {
        org.omg.Components.CCMHome home = null;
        CdmwQosEnablerHome recordingHome;
        CdmwQosEnabler recordingComp;

        System.out.println("Deploying the Recording QoSEnabler ...");

        try {
            HomeAllocator allocator = new HomeAllocator(
                "Cdmw.CCM.CIF.CdmwCdmwCcmPlugins.ExtensionCdmwQosEnablerHome_impl");
            HomeAllocatorRegistry.register(
                "Cdmw.CCM.CIF.CdmwCdmwCcmPlugins.ExtensionCdmwQosEnablerHome_impl",
                allocator);

            org.omg.Components.ConfigValue[] configValues = new org.omg.Components.ConfigValue[5];

            org.omg.CORBA.Any value = orb.create_any();
            com.thalesgroup.CdmwDeployment.ComponentKindValueHelper
                    .insert(
                            value,
                            com.thalesgroup.CdmwDeployment.ComponentKindValue.EXTENSION);
            configValues[0] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.COMPONENT_KIND.value, value);

            value = orb.create_any();
            com.thalesgroup.CdmwDeployment.ServantLifetimeValueHelper
                    .insert(
                            value,
                            com.thalesgroup.CdmwDeployment.ServantLifetimeValue.CONTAINER_LIFETIME);
            configValues[1] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.SERVANT_LIFETIME.value,
                    value);

            value = orb.create_any();
            com.thalesgroup.CdmwDeployment.ThreadingPolicyValueHelper
                    .insert(
                            value,
                            com.thalesgroup.CdmwDeployment.ThreadingPolicyValue.MULTITHREAD);
            configValues[2] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.THREADING_POLICY.value,
                    value);

            value = orb.create_any();
            value.insert_string(QOS_RECORDING_HOME_REP_ID);
            configValues[3] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_REPOSITORY_ID.value,
                    value);

            value = orb.create_any();
            value.insert_string(QOS_RECORDING_HOME_SERVANT_NAME);
            configValues[4] = new ConfigValueImpl(
                    com.thalesgroup.CdmwDeployment.HOME_SERVANT_CLASSNAME.value,
                    value);

            System.out.println("Installing the Home...");
            home = this.container.install_home(QOS_RECORDING_HOME_UID,
                    QOS_RECORDING_HOME_ENTRY_POINT, configValues);
            assertNotNull(home);

        } catch (org.omg.CORBA.BAD_PARAM bp) {
            fail(bp.toString());
        } catch (org.omg.Components.Deployment.InstallationFailure f) {
            fail("Unexpected exception raised: "
                    + "Components::Deployment::InstallationFailure " + f.reason);
        } catch (Exception e) {
            fail("Unexpected exception raised: " + e.toString());
        }

        System.out.println("Create a the Recording component using create() operation");
        try {
            recordingHome = CdmwQosEnablerHomeHelper.narrow(home);
            assertTrue("Home is not a CdmwQosEnablerHome!",
                    recordingHome != null);

            recordingComp = recordingHome.create();
            assertTrue("ERROR: component ref null!", recordingComp != null);

            NamedValue[] props = new NamedValue[1];
            props[0] = new NamedValue(
                    RecordingServiceComponentImpl.PROPERTY_RECORD_USING_COMPONENT_NAME, "true");

            recordingComp.service_settings(props);
            recordingComp.service_statistics(props);

            System.out.println("Calling configuration_complete on the recording component ...");
            recordingComp.configuration_complete();

        } catch (Exception se) {
            fail("Unexpected exception raised: " + se.toString());
        }

    }

}
