/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


package cdmw.cdmwinit.testtraceinit;

import cdmw.testutils.CORBATestManager;

import java.io.*;
import java.util.*;

import org.omg.CORBA.*;
import org.omg.CORBA.Object;
import org.omg.PortableServer.*;

import cdmw.common.*;
import cdmw.logging.*;
import static cdmw.logging.Logger.*;
import cdmw.namingandrepository.*;
import cdmw.orbsupport.*;
import cdmw.ossupport.*;
import cdmw.platformmngt.*;
import cdmw.trace.*;

import com.thalesgroup.CdmwNamingAndRepository.*;

public class MainTest {
    private static final int POA_THREAD_POOL_SIZE = 5;


    public static void main(String[] args) {
        String applicationName = "";
        int retCode = 0;
        
        StrategyList strategy = new StrategyList();
        strategy.addORBThreaded();
        try
        {
            strategy.addPOAThreadPool(MainTest.POA_THREAD_POOL_SIZE);
        }
        catch (BadParameterException e)
        {
            e.printStackTrace();
        }
        
        ORB orb = null;

        try
        {
            // initialise the orb    
            orb = ORBSupport.ORBInit(args, strategy);
            Object obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);
            POAManager poaMng = rootPOA.the_POAManager();
            poaMng.activate();

            // The logger
            Logger logger = Logger.Get_Logger("TEST", "TEST");

            CORBATestManager testManager = CORBATestManager.instance();
        
            testManager.init("cdmw.cdmwinit.test.InitConfigurationTest", orb, rootPOA);
        
            testManager.addTest(InitConfigurationTest.class);
        
            testManager.runTests();
            testManager.printSummary();

            if (orb != null) {
                //orb->shutdown(false) is done by the ProcessBehaviour
                ORBSupport.ORBCleanup(orb);
                orb.destroy();
            }
            
        }
        catch (SystemException e)
        {  
            System.err.println( "FAILURE : CORBA exception raised :" + e.getMessage());
            e.printStackTrace();
            retCode = -1;
        }
        catch(java.lang.Exception e)
        {
            System.err.println("FAILURE : Exception raised" + "\r\n" + e.getMessage());
            e.printStackTrace();
            retCode = -1;
        }

        System.exit(retCode);
    }

}

