/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Std include files
#include <iostream>
#include <sstream>
#include <string.h>

#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>

#include "testinterrogatingsimple/TestInterrogatingSimpleMonitoring.hpp"

using namespace std;

int main (int argc, char* argv[])
{
    
    std::string timeOn;
    std::string timeOnOpt =
      Cdmw::OsSupport::OS::get_option_value(argc, argv, "--timeOn");

    if (timeOnOpt == "no")
    {
      cout << "Interrogating Simple Monitoring will run during 500 ms" << std::endl;
      timeOn="500";
    }
    else if (timeOnOpt == "keypressed")
    {
      cout << "Interrogating Simple Monitoring will run until key pressed + enter" << std::endl;
      cout << "When you have finish to read the message, press key + enter" << std::endl;
      char dummy;
      cin >> dummy;
      timeOn="0";
    }
    else
    {
      timeOn=timeOnOpt;
      cout << "Interrogating Simple Monitoring will run during "
           << timeOn << " milliseconds" << std::endl;
    }

    Cdmw::OrbSupport::StrategyList strategyList;
    strategyList.add_OrbThreaded(); // Mandatory in OrbSupport::ORB_init
    strategyList.add_multicast();
    // Initialises the ORB
    CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList, "main_orb");

    // Get the root POA
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

    Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
    Cdmw::TestUtils::CORBATestManager::instance()->set_property ("keypressed",timeOn);
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
    
    return Cdmw::TestUtils::CORBATestManager::instance()->was_successful();
}

