/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

package cdmw.logging;

import com.thalesgroup.CdmwLogging.LogCollector;
import com.thalesgroup.CdmwLogging.LogCollectorHelper;
import com.thalesgroup.CdmwLogging.LogHeader;
import com.thalesgroup.CdmwLogging.LogContext;
import com.thalesgroup.CdmwLogging.StructuredLog;

import java.lang.String;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;


/**
 * This class enables to delegate logging to a remote or collocated LogCollector.
 */
public class LogManagerCollectorDelegate implements LogManagerDelegate
{

    private LogCollector refCollector;
    
    /**
     * Constructor.
     * @param refCollector reference to the remote CORBA LogCollector
     */
    public LogManagerCollectorDelegate(LogCollector refCollector) {
        cdmw.common.Assert.check(refCollector != null);
        this.refCollector = refCollector;
    }

    /**
     * Send a message to the LogCollector.
     *
     * @param context the log context.
     * @param header the log header.
     * @param msg the message.
     *
     * @return true if successful.
     */
    public boolean addLog(LogContext context, LogHeader header, String msg) {
        
        boolean result = true;
        
        // Bug traces using JacORB with interceptors or FT (PCR-1719)
        if ( ( ((header.level & LogLevel.ORB_TRACE) != 0)
            && ((header.level & LogLevel.DEBUG_9) != 0)
            && (header.user_domain.equals("jacorb.orb.interceptors")) )
          || ( ((header.level & LogLevel.ORB_TRACE) != 0)
            && ((header.level & (LogLevel.INFO|LogLevel.DEBUG_9)) != 0)
            && (header.user_domain.equals("jacorb.ft.client")) ) ) {

            // write time with second precision in buffer
            SimpleDateFormat tf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss:SSS");
            GregorianCalendar calendar = new GregorianCalendar();
            calendar.setTime( new Date((long) header.time_stamping.seconds * 1000
                                            + header.time_stamping.microseconds / 1000) );

            String buffer  =  tf.format(calendar.getTime()) + ":000 " + " " + LogLevel.LevelToString(header.level)
                              + " [" + header.component_name + "]" + " "+ header.user_domain + " [tid =" + header.thread_id + "] "
                              + "(" + header.file_name + ":" + header.file_line + ")"
                              + " [" + context.ndc + "] " + "\t *** " + msg;
            System.out.println(buffer);
            return true;
        }
        
        try {
            StructuredLog log = new StructuredLog(context, header, msg);
            result = this.refCollector.add_log(log);
        }
        catch (java.lang.Exception e) {
            System.out.println("Cannot log message: " + e.getMessage());
            result = false;
        }

        return result;
    }

}
