/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.logging.test.testlogmanager;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import cdmw.common.BadParameterException;
import cdmw.logging.*;

import com.thalesgroup.CdmwLogging.LogCollectorPOA;
import com.thalesgroup.CdmwLogging.StructuredLog;

public class LogCollectorImpl extends LogCollectorPOA {

    private BufferedWriter ofstream;

    public LogCollectorImpl(String fileName) {
        try {
            this.ofstream = new BufferedWriter(new FileWriter(fileName, false));
        }
        catch (IOException ioe) {
            System.err.println("Couldn't create log file " + fileName + ": " + ioe.getMessage());
        }
    }

    public boolean add_log(StructuredLog log) {
        boolean result = true;
        try
        {
            String buffer =
                log.ctx.application_name + "-" + log.ctx.process_name + "-"
                + log.header.component_name + "-" + log.header.user_domain + "-"
                + LogLevel.LevelToString(log.header.level) + ":" + log.message;

            synchronized (this) {
                if (this.ofstream != null) {
                    this.ofstream.write(buffer);
                    this.ofstream.newLine();
                    this.ofstream.flush();
                }
                else {
                    System.out.println(buffer);
                }
            }
        }
        catch (IOException e) {
            System.out.println("Cannot log message: " + e.getMessage());
            result = false;
        }

        return result;
    }

}
