# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================
eval '(exit $?0)' && eval 'exec perl -S $0 ${1+"$@"}'
    & eval 'exec perl -S $0 $argv:q'
    if 0;

# This is a Perl script that tests LoadBalancing/testlbinit

use lib "$ENV{ACE_ROOT}/bin";
use lib "$ENV{CDMW_HOME}/Build/testautomation";
use File::Basename;
use Cwd;
use PerlACE::Run_Test;
use TestUtils;

#setting common variables
$nbtestable = 1;
$max_wait_time = 180;
$test_exec = "cdmw_test_lb_init";
$test_id = "U-LDB-0003";
$runtest_path = "runtest/".$test_id."/";
$XML_results = $test_exec."_test_results.xml";
@needed_files = (cwd()."/CdmwTestLBInitGroupCreator_conf.xml", 
                 cwd()."/test_server_init", 
		 cwd()."/test_lb_init", 
		 cwd()."/test_lb_init_with_time_refresh", 
		 cwd()."/test_override_lb_policy_default", 
		 cwd()."/test_override_lb_policy_force", 
                 $ENV{CDMW_HOME}."/share/CdmwLBGroupManager_conf.xml",
                 $ENV{CDMW_HOME}."/share/CdmwLBGroupManager.dtd",
                 $ENV{CDMW_HOME}."/share/CdmwLoadBalancingGroupCreator.dtd",
                 $ENV{CDMW_HOME}."/bin/cdmw_lb_group_manager");
@files_to_move = ($XML_results);
@processes_to_kill = ($test_exec, "test_server_init", "test_lb_init", "test_lb_init_with_time_refresh", 
                      "test_override_lb_policy_default", "test_override_lb_policy_force", "output_redirection.sh");

#creating folders and removing old test results
TestUtils::prepareDirectoryStructureForUnitTest($test_id);

#creating soft links for needed files
symlink cwd()."/".$test_exec, $runtest_path.$test_exec;
foreach $from_file (@needed_files)
{
    $to_file = basename $from_file;
    symlink $from_file, $runtest_path.$to_file;
}

PerlACE::add_lib_path(cwd()."/".$runtest_path);
chdir $runtest_path;
 
# replace environment variables in linked files
$replace_cmd = "$ENV{CDMW_HOME}/Build/scripts/replace_environment.sh";
$replace_dir = ".";
if (PerlACE::is_vxworks_test()) 
         {$test = new PerlACE::ProcessVX ($replace_cmd, $replace_dir); }
else     {$test = new PerlACE::Process   ($replace_cmd, $replace_dir); }
$status = $test->SpawnWaitKill(10);


$status = 0;
#create and run test process
$redirect = "$ENV{CDMW_HOME}/Build/testautomation/output_redirection.sh";
$cmnd = "$test_exec $test_args 2>&1 | tee results/output.txt" ;
if (PerlACE::is_vxworks_test()) 
         {$test = new PerlACE::ProcessVX ($redirect, $cmnd); }
else     {$test = new PerlACE::Process   ($redirect, $cmnd); }
$status = $test->SpawnWaitKill($max_wait_time);

#move temprorary and log files to results dir
foreach $file (@files_to_move)
{
    TestUtils::moveFileToResults($file);
}

#open file to write result of test
open (RES, '>>results/result.txt');

#check returned value of test
if ($status != 0)
{
    print RES "ERROR: test returned $status\n";
    $status = 1;
}
else
{
    $status = 0;
}

#check results in XML file
if ($status == 0)
{    
    $checkXML_result = TestUtils::checkXML("results/".$XML_results);
    if ($checkXML_result ne "")
    {
        print RES $checkXML_result;
        $status = 1;
    }
}

#check existence of output.txt
if (!-e "results/output.txt")
{
    print RES "ERROR: output.txt file does not exist\n";
    $status = 1;
}

#check results in test output
if ($status == 0)
{
    if (TestUtils::checkForMessage("results/output.txt", "TESTABLE_OK") != $nbtestable)
    {
        print RES "ERROR: Incorrect count of TESTABLE_OK in log of test\n";
        $status = 1;
    }
}

#check for errors in test output
if ($status == 0)
{
    if (TestUtils::checkForErrorMessages("results/output.txt") > 0)
    {
        print RES "ERROR: There is an error in log of test\n";
	$status = 1;
    }
}

#print test global result
if ($status == 0)
{
    print RES "Test OK\n";    
}
else
{
    print RES "Test FAILED\n";    
}
close(RES);

#kill processes
foreach $process (@processes_to_kill)
{
    system("killall -q -s KILL $process");
}

exit $status;
