/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.acme.Hello;

import org.omg.CCM_DDS.CCM_ListenerControl;
import org.omg.CORBA.Object;
import org.omg.Components.CCMException;
import org.omg.Components.SessionContext;

import com.acme.Hello.DonneeDataReader;
import com.acme.Hello.DonneeDataReaderHelper;
import com.acme.Hello.DonneeDataWriter;
import com.acme.Hello.DonneeTypeSupport;


import cdmw.ccm.qosenablers.dds.DDS_TopicBaseFragmentImpl;
import cdmw.ccm.qosenablers.dds.ErrorHandler;
import cdmw.ccm.qosenablers.dds.ListenerControlImpl;

import DDS.*;

public class Donnee_MyConnector_DDS_RawListenExecutorImpl extends
        DDS_TopicBaseFragmentImpl implements
        CCM_Donnee_MyConnector_DDS_RawListen, org.omg.Components.SessionComponent {

    /* Type-specific DDS entities */
    DonneeTypeSupport dataTS;
    String DonneeTypeName;

    // --------------- attributs --------------

    private CCM_Donnee_MyConnector_DDS_RawListen_Context sessionContext;

    private CCM_Donnee_Reader donneeReaderPort;
    private Donnee_RawListener donneeListenerPort;
    protected CCM_ListenerControl listenerControl;

    public Donnee_MyConnector_DDS_RawListenExecutorImpl() {
        //this("", null, "DefaultTOPIC", null);
        super();
        this.listenerControl = new ListenerControlImpl();
        dataTS = new DonneeTypeSupport();
        super.topic_name("DEMO_TOPIC");
    }

    public Donnee_MyConnector_DDS_RawListenExecutorImpl(String domain_id,
            String qos_profile, String topic_name, String[] key_fields) {
        super(domain_id, qos_profile, topic_name, key_fields);
        this.listenerControl = new ListenerControlImpl();
        dataTS = new DonneeTypeSupport();
    }

    public CCM_Donnee_Reader get_p_RawListen_data() {
        return this.donneeReaderPort;
    }

    public CCM_ListenerControl get_p_RawListen_control() {
        return this.listenerControl;
    }

    public void ccm_activate() throws CCMException{
        try {
            super.setupDDSInfrastructure(false,true);
            // getting the typed datareader
            DonneeDataReader donneeDR = DonneeDataReaderHelper.narrow(reader);
            ErrorHandler.checkHandle(donneeDR,
                    "DDS.DonneeDataReaderHelper.narrow");

            //To create the facet executor
            this.donneeReaderPort = new Donnee_ReaderFacetImpl(donneeDR);

            // To get the component listener
            this.donneeListenerPort = this.sessionContext.get_connection_p_RawListen_listener();

            if(donneeListenerPort == null) logger.fine("Failed to get the component RawListen facet !");


            // To set the DDS listener
            DonneeDataReaderListenerImpl listener = new DonneeDataReaderListenerImpl(
                    donneeListenerPort, listenerControl, donneeReaderPort);
            int status = donneeDR.set_listener(listener,
                    DDS.DATA_AVAILABLE_STATUS.value);
            ErrorHandler.checkStatus(status, "set_listener");

        } catch (Exception e) {
            System.out.println("Unexpected exception raised!\n" + e.toString());
        }

    }

    public void ccm_remove() throws CCMException{
        super.teardownDDSInfrastructure(false,true);
    }

    public void ccm_passivate() throws CCMException{

    }

    public void set_session_context(SessionContext arg0) throws CCMException{
        sessionContext = CCM_Donnee_MyConnector_DDS_RawListen_ContextHelper
                .narrow((Object) arg0);
    }

    protected TypeSupport getDataTypeSupport() {
        return dataTS;
    }

}
