/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.acme.Hello;

import org.omg.CCM_DDS.DDSEntityKind;
import org.omg.CCM_DDS.InternalError;
import org.omg.CCM_DDS.NoSuchDDSEntity;
import org.omg.CORBA.Object;
import org.omg.Components.CCMException;
import org.omg.Components.EnterpriseComponent;
import org.omg.Components.SessionContext;

import cdmw.ccm.qosenablers.dds.DDS_TopicBaseFragmentImpl;
import cdmw.ccm.qosenablers.dds.ErrorHandler;

import com.acme.Hello.CCM_Donnee_Writer;
import com.acme.Hello.Donnee;
import com.acme.Hello.DonneeDataWriter;
import com.acme.Hello.DonneeDataWriterHelper;
import com.acme.Hello.DonneeTypeSupport;

import DDS.*;

public class Donnee_MyConnector_DDS_WriteExecutorImpl extends
        DDS_TopicBaseFragmentImpl implements
        CCM_Donnee_Writer, CCM_Donnee_MyConnector_DDS_Write, org.omg.Components.SessionComponent {


    /* Type-specific DDS entities */
    DonneeTypeSupport dataTS;
    DonneeDataWriter talker;


    public Donnee_MyConnector_DDS_WriteExecutorImpl(){
        dataTS = new DonneeTypeSupport();
        ErrorHandler.checkHandle(dataTS, "new DonneeTypeSupport");
        super.topic_name("DEMO_TOPIC");
    }

    public CCM_Donnee_Writer get_p_Write_data() {
        return (CCM_Donnee_Writer)this;
    }

    public void ccm_activate() throws CCMException{
        System.out.println("Donnee_MyConnector_DDS_WriteExecutorImpl.ccm_activate called ...");
        try {
            super.setupDDSInfrastructure(true,false);

            /* Narrow the abstract parent into its typed representative. */
            talker = DonneeDataWriterHelper.narrow(this.writer);
            ErrorHandler.checkHandle(talker,
                    "Donnee.DonneeDataWriterHelper.narrow");

        } catch (Exception e) {
            System.out.println("Unexpected exception raised!\n" + e.toString());
        }

    }

    public void ccm_remove() throws CCMException{

        try {
            /* Remove the DataWriters */
            int status = this.publisher.delete_datawriter(talker);
            ErrorHandler.checkStatus(status,
                    "DDS.Publisher.delete_datawriter (talker)");
            super.teardownDDSInfrastructure(true,false);

        } catch (Exception e) {
            System.out.println("Unexpected exception raised!\n" + e.toString());
        }

    }

    public void ccm_passivate() throws CCMException{

    }

    public void set_session_context(SessionContext arg0) throws CCMException{
    }

    @Override
    protected TypeSupport getDataTypeSupport() {
        return dataTS;
    }


    /**
     * The Writer facet implementation
     *
    /**/

    /**
     *
     * @param an_instance
     * @throws InternalError
     */
    public void write(Donnee an_instance) throws InternalError {
        long userHandle = talker.register_instance(an_instance);
        int status = talker.write(an_instance, userHandle);
        if (status != RETCODE_OK.value && status != RETCODE_NO_DATA.value) {
            InternalError e = new InternalError();
            e.error_code = status;
            throw e;
        }
    }

    public org.omg.CORBA.Object getDDSEntity(DDSEntityKind kind) throws NoSuchDDSEntity{
        return null;
    }

}