/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package test;

import DDS.*;
import com.acme.Hello.*;
import cdmw.ccm.qosenablers.dds.ErrorHandler;

public class Writer {

    public static void main(String[] args) {

        /* Generic DDS entities */
        DomainParticipantFactory dpf;
        DomainParticipant participant;
        Topic DonneeTopic;
        Publisher donneePublisher;
        DataWriter parentWriter;

        /* DDS Identifiers */
        String domain = "";
        long userHandle;
        int status;

        /* Type-specific DDS entities */
        DonneeTypeSupport donneeTS;
        DonneeDataWriter talker;

        /* Sample definitions */
        Donnee msg = new Donnee();

        /* Others */
        int ownID = 1;
        String DonneeTypeName;

        /* Options: Chatter [ownID [name]] */
        if (args.length > 0) {
            ownID = Integer.parseInt(args[0]);
        }
        try {
            /*
             * Create a DomainParticipantFactory and a DomainParticipant (using
             * Default QoS settings.
             */
            dpf = DomainParticipantFactory.get_instance();
            ErrorHandler.checkHandle(dpf,
                    "DDS.DomainParticipantFactory.get_instance");

            participant = dpf.create_participant(domain,
                    PARTICIPANT_QOS_DEFAULT.value, null, ANY_STATUS.value);
            ErrorHandler.checkHandle(participant,
                    "DDS.DomainParticipantFactory.create_participant");

            /* Register the required datatype for Donnee. */
            donneeTS = new DonneeTypeSupport();
            ErrorHandler.checkHandle(donneeTS, "new DonneeTypeSupport");
            DonneeTypeName = donneeTS.get_type_name();
            status = donneeTS.register_type(participant, DonneeTypeName);
            ErrorHandler.checkStatus(status,
                    "Donnee.DonneeTypeSupport.register_type");

            /* Use the changed policy when defining the Donnee topic */
            DonneeTopic = participant.create_topic("donnee_topic",
                    DonneeTypeName, TOPIC_QOS_DEFAULT.value, null,
                    ANY_STATUS.value);
            ErrorHandler.checkHandle(DonneeTopic,
                    "DDS.DomainParticipant.create_topic (Donnee)");

            /* Create a Publisher for the Donneeter application. */
            donneePublisher = participant.create_publisher(
                    PUBLISHER_QOS_DEFAULT.value, null, ANY_STATUS.value);
            ErrorHandler.checkHandle(donneePublisher,
                    "DDS.DomainParticipant.create_publisher");

            /*
             * Create a DataWriter for the Donnee Topic (using the appropriate
             * QoS).
             */
            parentWriter = donneePublisher.create_datawriter(DonneeTopic,
                    DATAWRITER_QOS_USE_TOPIC_QOS.value, null, ANY_STATUS.value);
            ErrorHandler.checkHandle(parentWriter,
                    "DDS.Publisher.create_datawriter (Donnee)");

            /* Narrow the abstract parent into its typed representative. */
            talker = DonneeDataWriterHelper.narrow(parentWriter);
            ErrorHandler.checkHandle(talker,
                    "Donnee.DonneeDataWriterHelper.narrow");

            /* Initialize the Donnee messages. */
            msg.ID = ownID;
            msg.msg = "][_, ([]) ][_,";

            System.out.println("Writing message: \"" + msg.msg + "\"");

            /*
             * Register a Donnee message for this user (pre-allocating resources
             * for it!!)
             */
            userHandle = talker.register_instance(msg);

            /* Write a message using the pre-generated instance handle. */
            status = talker.write(msg, userHandle);
            ErrorHandler.checkStatus(status, "Donnee.DonneeDataWriter.write");

            try {
                Thread.sleep(100); /* do not run so fast! */
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

            /*
             * Leave the room by disposing and unregistering the message
             * instance
             */
            status = talker.dispose(msg, userHandle);
            ErrorHandler.checkStatus(status, "Donnee.DonneeDataWriter.dispose");
            status = talker.unregister_instance(msg, userHandle);
            ErrorHandler.checkStatus(status,
                    "Donnee.DonneeDataWriter.unregister_instance");

            /* Remove the DataWriters */
            status = donneePublisher.delete_datawriter(talker);
            ErrorHandler.checkStatus(status,
                    "DDS.Publisher.delete_datawriter (talker)");

            /* Remove the Publisher. */
            status = participant.delete_publisher(donneePublisher);
            ErrorHandler.checkStatus(status,
                    "DDS.DomainParticipant.delete_publisher");

            status = participant.delete_topic(DonneeTopic);
            ErrorHandler.checkStatus(status,
                    "DDS.DomainParticipant.delete_topic (DonneeTopic)");

            /* Remove the DomainParticipant. */
            status = dpf.delete_participant(participant);
            ErrorHandler.checkStatus(status,
                    "DDS.DomainParticipantFactory.delete_participant");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
