/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2010. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


package cdmw.qosenablers4ccm;

import java.io.File;
import java.util.Vector;
import java.util.StringTokenizer;
import java.io.IOException;

import cdmw.testutils.Testable;
import cdmw.common.Options;
import cdmw.ossupport.OS;
import cdmw.ossupport.FileUtilities;
import cdmw.ossupport.ThreadedProcess;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.ExceptionMinorCodes;

import cdmw.ccm.container.HomeAllocator;
import cdmw.ccm.container.HomeAllocatorRegistry;
import cdmw.ccm.componentserver.ComponentServerImpl;
import com.thalesgroup.CdmwDeployment.ComponentKindValue;
import com.thalesgroup.CdmwDeployment.ComponentKindValueHelper;
import com.thalesgroup.CdmwPlatformMngt.ProcessCallback;
import com.thalesgroup.CdmwPlatformMngt.ProcessCallbackHelper;
import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.namingandrepository.RepositoryInterface;
import cdmw.common.Locations;
import cdmw.orbsupport.ORBUtils;
import cdmw.cdmwinit.CdmwInterface;
import cdmw.ossupport.ThreadedProcess;

public class RecordingDaemonLauncher {


    public static final int POA_THREAD_POOL_SIZE = 8;
    public static final int SUCCESS = 0;
    public static final int FAILURE = 1;



    public static void main(String[] args) {
        int status = SUCCESS;

        org.omg.CORBA.ORB orb = null;
        int timescale = Testable.getTimescale();
        ThreadedProcess idDaemon = null;

        try {

            StrategyList orbStrategies = new StrategyList();
            orbStrategies.addORBThreaded();
            orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
            String nameServicePort = "21869";

            String s = ORBUtils.NAMING_PORT_OPTION + "corbaloc::localhost:"
                + nameServicePort + "/NameService ";
            s += Options.SYSTEM_PORT_OPTION;
            s += "=";
            s += nameServicePort;
            s +=" --CdmwLogLevel=DEFAULT+DEBUG+CALL_TRACE";
            String[] serverArgs = s.split(" ");
            orb = ORBSupport.ORBInit(serverArgs, orbStrategies);


            // Get the root POA
            org.omg.CORBA.Object obj =
                orb.resolve_initial_references("RootPOA");
            org.omg.PortableServer.POA rootPOA =
                org.omg.PortableServer.POAHelper.narrow(obj);

            // Run implementation (Activate the POA manager)
            rootPOA.the_POAManager().activate();



                // Start RecordingServer emulator
            RecordingServerEmulatorThread server = new RecordingServerEmulatorThread(orb, args);

            server.start();
            System.out.println("waiting 2 sec..");
            OS.sleep(timescale * 2000);

            {
                // Starting the RecordingDaemon
                String recordingDaemon = "cdmw_recordingdaemon";

                String namingCorbaloc =
                    " -ORBInitRef NameService=corbaloc:iiop:localhost:" + nameServicePort + "/NameService";

                String recordingArgs ="";
                recordingArgs += " --debug_file=./recording.log";
                recordingArgs += " --CdmwProcessPort=59875";
                recordingArgs += " --rebind_to_ns" + namingCorbaloc;
                recordingArgs += " -DInitial_TCPPorts_List=6072";
                recordingArgs += " --recorder_url=";
                recordingArgs += server.getIor();

                System.out.println("start recording daemon");
                System.out.println(recordingDaemon + recordingArgs);

                idDaemon =  OS.createProcess(recordingDaemon + recordingArgs);
            }


            orb.run();

        } catch(Exception e) {
            System.err.println(e.toString());
            status = FAILURE;
        }

        if (orb != null) {
            try {
                orb.destroy();
            } catch(Exception e) {
                System.err.println(e.toString());
                status = FAILURE;
            }
        }

        OS.killProcess(idDaemon);

        // And we kill launched process
        // for the simulated repository, launch process is a script shell
        // which ended when process are launched: no kill of this process
        // to stop sim repository, the stop_simu.sh script shall be called
        try {
            cdmw.tools.SimuUtils.stopSimu(System.out);
            OS.sleep(timescale * 2000);
        } catch (InterruptedException ie) {
            ie.printStackTrace();
            status = FAILURE;
        }

        if (status == FAILURE) {
            System.out.println("Main test failed.");
        } else {
            System.out.println("Main test succeeded.");
        }
        System.exit(0);
    }

}

