/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_Get_Pulse;

import java.util.logging.Logger;

import org.omg.CCM_DDS.InternalError;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.SessionContext;

import com.prismtech.mosic.radar.CCM_PulseEventConnector_Pulse_DDS_Get_pull_consumer;
import com.prismtech.mosic.radar.CCM_Pulse_Getter;
import com.prismtech.mosic.radar.PulseDataReader;
import com.prismtech.mosic.radar.PulseDataReaderHelper;
import com.prismtech.mosic.radar.PulseTypeSupport;

import DDS.TypeSupport;
import cdmw.ccm.qosenablers.dds.DDS4CCMLoggers;
import cdmw.ccm.qosenablers.dds.DDS_TopicBaseFragmentImpl;
import cdmw.ccm.qosenablers.dds.ErrorHandler;


public class DDS_Get_PulseExecutorImpl extends DDS_TopicBaseFragmentImpl
implements CCM_PulseEventConnector_Pulse_DDS_Get_pull_consumer,
org.omg.Components.SessionComponent
{
    Logger logger = DDS4CCMLoggers.getDDS_GetLogger();

    PulseDataReader pulseReader;
    PulseTypeSupport dataTS;

    CCM_Pulse_Getter data;

    // ------------- attributs ----------------

    public CCM_Pulse_Getter get_pull_consumer_data() {
        return data;
    }

    public DDS_Get_PulseExecutorImpl() {
        super();
        dataTS = new PulseTypeSupport();
        try{
            ErrorHandler.checkHandle(dataTS, "new PulseTypeSupport");
        } catch (InternalError e) {
            logger.severe("Failed to get the data type support object ! " + e.getMessage());
            logger.throwing(this.getClass().getName(), "DDS_Get_MotorPositionExecutorImpl", e);
            e.printStackTrace();
        }
    }

    public void ccm_activate() throws CCMException{
        System.out
        .println("DDS_Get_PulseExecutorImpl.ccm_activate called ...");
        try {
            super.setupDDSInfrastructure(false,true);
            pulseReader = PulseDataReaderHelper.narrow(super.reader);
            ErrorHandler.checkHandle(pulseReader,
                    "DDS.PulseDataReaderHelper.narrow");

            this.data = new Getter_PulseImpl(pulseReader);

        } catch (Exception e) {
            logger.severe("Unexpected exception raise ! " + e.getMessage());
            logger.throwing(this.getClass().getName(), "ccm_activate", e);
            System.out.println("Unexpected exception raised!\n" + e.toString());
        }
    }

    public void ccm_remove() throws CCMException{
        try{
            super.teardownDDSInfrastructure(false,true);
    } catch (InternalError e) {
        e.printStackTrace();
        logger.throwing(this.getClass().getName(), "ccm_remove", e);
        throw new CCMException("Failed to teardown the DDS infrastructure of fragment" +
                    "Pulse_MyConnector_DDS_GetExecutorImpl", CCMExceptionReason.REMOVE_ERROR);
    }
    }

    public void ccm_passivate() throws CCMException{

    }

    public void set_session_context(SessionContext arg0) throws CCMException{
    }

    @Override
    protected TypeSupport getDataTypeSupport() {
        return dataTS;
    }

}
