/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_Get_TargetDistance;

import static cdmw.ccm.qosenablers.dds.DataUtil.initData;

import org.omg.CCM_DDS.InternalError;
import org.omg.CCM_DDS.NonExistent;
import org.omg.CCM_DDS.QueryFilter;
import org.omg.CCM_DDS.ReadInfoHolder;
import org.omg.CCM_DDS.ReadInfoSeqHolder;

import DDS.LENGTH_UNLIMITED;
import DDS.SampleInfoSeqHolder;

import com.prismtech.mosic.radar.CCM_TargetDistance_Getter;
import com.prismtech.mosic.radar.TargetDistance;
import com.prismtech.mosic.radar.TargetDistanceDataReader;
import com.prismtech.mosic.radar.TargetDistanceHolder;
import com.prismtech.mosic.radar.TargetDistanceSeqHolder;


import cdmw.ccm.qosenablers.dds.DDS_GetterImpl;
import cdmw.ccm.qosenablers.dds.DataSeqHolder;

public class Getter_TargetDistanceImpl extends DDS_GetterImpl implements CCM_TargetDistance_Getter {

    private TargetDistanceDataReader reader;

    public Getter_TargetDistanceImpl(TargetDistanceDataReader targetDistanceReader) {
        super(targetDistanceReader);
        this.reader = targetDistanceReader;
    }


    /**
     * returns for all the instances their last sample
     */
    public boolean get_all(TargetDistanceSeqHolder instances, ReadInfoSeqHolder infos)
    throws InternalError {
        Exception exp = null;

        try{
            try{
                initData(new DataSeqHolder<TargetDistance, TargetDistanceSeqHolder>(TargetDistance[].class, instances), infos, null);
                super.waitData();
            }catch(InternalError e){
                if(e.error_code == DDS.RETCODE_TIMEOUT.value) return false;
                else throw e;
            }

            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();
            int status = reader.read_w_condition(instances, infoSeq,
                    LENGTH_UNLIMITED.value, this.userQuery);

            return getAll(new DataSeqHolder<TargetDistance, TargetDistanceSeqHolder>(TargetDistance[].class, instances), infos,
                    infoSeq.value, status);

        }catch(Exception e){
            exp = e;
        }

        throw new InternalError("Operation Failed: "+ exp.getMessage(), DDS.RETCODE_ERROR.value, 0);
    }

    /**
     * returns all the samples of all instances
     * @param instances
     * @param infos
     * @return
     * @throws InternalError
     */
    public boolean get_all_history(TargetDistanceSeqHolder instances,
            ReadInfoSeqHolder infos) throws InternalError {
        Exception exp = null;

        try{
            try{
                initData(new DataSeqHolder<TargetDistance, TargetDistanceSeqHolder>(TargetDistance[].class, instances), infos, null);
                waitData();
            }catch(InternalError e){
                if(e.error_code == DDS.RETCODE_TIMEOUT.value) return false;
                else throw e;
            }
            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();
            int status = reader.read_w_condition(instances, infoSeq,
                    LENGTH_UNLIMITED.value, this.userQuery);

            return getAllHistory(new DataSeqHolder<TargetDistance, TargetDistanceSeqHolder>(TargetDistance[].class, instances), infos,
                    infoSeq.value, status);
        }catch(Exception e){
            exp = e;
        }

        throw new InternalError("Operation Failed: "+ exp.getMessage(), DDS.RETCODE_ERROR.value, 0);

    }

    /**
     * returns each read sample one by one
     * @param an_instance
     * @param info
     * @return
     * @throws InternalError
     */
    public boolean get_next(TargetDistanceHolder an_instance, ReadInfoHolder info)
    throws InternalError {
        Exception exp = null;

        try{
            try{
                initData(new DataSeqHolder<TargetDistance, TargetDistanceHolder>(an_instance, TargetDistance.class), null, info);
                waitData();
            }catch(InternalError e){
                if(e.error_code == DDS.RETCODE_TIMEOUT.value) return false;
                else throw e;
            }
            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();
            TargetDistanceSeqHolder instance = new TargetDistanceSeqHolder();
            int status = reader.read_w_condition(instance, infoSeq, 1, this.userQuery);

            //FIXME:do we keep this
            if (infoSeq.value[0].valid_data) {
                an_instance.value = instance.value[0];
            }
            return getNext(info, infoSeq.value, status);
        }catch(Exception e){
            exp = e;
        }

        throw new InternalError("Operation Failed: "+ exp.getMessage(), DDS.RETCODE_ERROR.value, 0);

    }

    /**
     * returns the last sample of the given instance
     * NOTA: userQuery is ignored because read_instance_with_condition is not implemented by Splice ...
     */
    public boolean get_one(TargetDistanceHolder an_instance, ReadInfoHolder info)
    throws NonExistent, InternalError {
        Exception exp = null;

        try{
            try{
                //initData(new DataSeqHolder<TargetDistance, TargetDistanceHolder>(an_instance, TargetDistance.class), null, info);
                waitData();
            }catch(InternalError e){
                if(e.error_code == DDS.RETCODE_TIMEOUT.value) return false;
                else throw e;
            }

            long handle = reader.lookup_instance(an_instance.value);
            if (handle == DDS.HANDLE_NIL.value) {
                throw new NonExistent();
            }
            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();
            TargetDistanceSeqHolder instances = new TargetDistanceSeqHolder();
            int status = reader.read_instance(instances, infoSeq, LENGTH_UNLIMITED.value,
                    handle,
                    DDS.NOT_READ_SAMPLE_STATE.value,
                    DDS.NEW_VIEW_STATE.value | DDS.NOT_NEW_VIEW_STATE.value,
                    DDS.ALIVE_INSTANCE_STATE.value | DDS.NOT_ALIVE_INSTANCE_STATE.value);

            if (infoSeq.value[infoSeq.value.length - 1].valid_data) {
                an_instance.value = instances.value[infoSeq.value.length - 1];
            }
            return getOne(info, infoSeq.value, status);
        }catch(Exception e){
            exp = e;
        }

        throw new InternalError("Operation Failed: "+ exp.getMessage(), DDS.RETCODE_ERROR.value, 0);

     }

    /**
     * returns all the samples for the given instance
     * NOTA: userQuery ignored
     */
    public boolean get_one_history(TargetDistance an_instance,
            TargetDistanceSeqHolder instances, ReadInfoSeqHolder infos)
    throws NonExistent, InternalError {
        Exception exp = null;

        try{
            try{
                initData(new DataSeqHolder<TargetDistance, TargetDistanceSeqHolder>(TargetDistance[].class, instances), infos, null);
                waitData();
            }catch(InternalError e){
                if(e.error_code == DDS.RETCODE_TIMEOUT.value) return false;
                else throw e;
            }

            long handle = reader.lookup_instance(an_instance);

            if (handle == DDS.HANDLE_NIL.value) {
                throw new NonExistent();
            }
            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();
            int status = reader.read_instance(instances, infoSeq,
                    LENGTH_UNLIMITED.value, handle,
                    DDS.NOT_READ_SAMPLE_STATE.value, DDS.NEW_VIEW_STATE.value
                    | DDS.NOT_NEW_VIEW_STATE.value,
                    DDS.ALIVE_INSTANCE_STATE.value
                    | DDS.NOT_ALIVE_INSTANCE_STATE.value);

            return getOneHistory(infos, infoSeq.value, status);
        }catch(Exception e){
            exp = e;
        }

        throw new InternalError("Operation Failed: "+ exp.getMessage(), DDS.RETCODE_ERROR.value, 0);

    }



}
