/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_RawListen_Pulse;

import java.util.logging.Logger;

import org.omg.CCM_DDS.CCM_ListenerControl;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.SessionContext;

import DDS.TypeSupport;

import com.prismtech.mosic.radar.CCM_PulseEventConnector_Pulse_DDS_RawListen_push_consumer;
import com.prismtech.mosic.radar.CCM_PulseEventConnector_Pulse_DDS_RawListen_push_consumer_Context;
import com.prismtech.mosic.radar.CCM_PulseEventConnector_Pulse_DDS_RawListen_push_consumer_ContextHelper;
import com.prismtech.mosic.radar.CCM_Pulse_Reader;
import com.prismtech.mosic.radar.CCM_Pulse_Reader;
import com.prismtech.mosic.radar.PulseDataReader;
import com.prismtech.mosic.radar.PulseDataReaderHelper;
import com.prismtech.mosic.radar.PulseTypeSupport;
import com.prismtech.mosic.radar.Pulse_RawListener;

import cdmw.ccm.qosenablers.dds.DDS4CCMLoggers;
import cdmw.ccm.qosenablers.dds.DDS_TopicBaseFragmentImpl;
import cdmw.ccm.qosenablers.dds.ErrorHandler;
import cdmw.ccm.qosenablers.dds.ListenerControlImpl;

public class DDS_RawListen_PulseExecutorImpl extends DDS_TopicBaseFragmentImpl
        implements CCM_PulseEventConnector_Pulse_DDS_RawListen_push_consumer,
        org.omg.Components.SessionComponent {

    private Logger logger = DDS4CCMLoggers.getDDS_RawListenLogger();

    private PulseTypeSupport dataTS;
    private CCM_PulseEventConnector_Pulse_DDS_RawListen_push_consumer_Context sessionContext;
    private CCM_Pulse_Reader pulseReaderPort;
    private Pulse_RawListener pulseListenerPort;
    protected CCM_ListenerControl listenerControl;

    public DDS_RawListen_PulseExecutorImpl() {
        // this("", null, "DefaultTOPIC", null);
        super();
        this.listenerControl = new ListenerControlImpl();
        dataTS = new PulseTypeSupport();
    }

    public DDS_RawListen_PulseExecutorImpl(String domain_id,
            String qos_profile, String topic_name, String[] key_fields) {
        super(domain_id, qos_profile, topic_name, key_fields);
        this.listenerControl = new ListenerControlImpl();
        dataTS = new PulseTypeSupport();
    }

    public CCM_Pulse_Reader get_push_consumer_data() {
        return this.pulseReaderPort;
    }

    public CCM_ListenerControl get_push_consumer_control() {
        return this.listenerControl;
    }

    public void ccm_activate() throws CCMException {
        logger.info("DDS_RawListen_PulseExecutorImpl.ccm_activate called ...");
        try {
            super.setupDDSInfrastructure(false, true);
            // getting the typed datareader
            PulseDataReader pulseDR = PulseDataReaderHelper.narrow(reader);
            ErrorHandler.checkHandle(pulseDR,
                    "DDS.PulseDataReaderHelper.narrow");

            // To create the facet executor
            this.pulseReaderPort = new Reader_PulseImpl(pulseDR);

            // To get the component listener
            this.pulseListenerPort = this.sessionContext
                    .get_connection_push_consumer_listener();

            if (pulseListenerPort == null)
                logger.fine("Failed to get the component RawListen facet !");

            // To set the DDS listener
            PulseDataReaderListenerImpl listener = new PulseDataReaderListenerImpl(
                    pulseListenerPort, listenerControl, pulseReaderPort);
            int status = pulseDR.set_listener(listener,
                    DDS.DATA_AVAILABLE_STATUS.value);
            ErrorHandler.checkStatus(status, "set_listener");

        } catch (Exception e) {
            logger.severe("Unexpected exception raise ! " + e.getMessage());
            logger.throwing(this.getClass().getName(), "ccm_activate", e);
            System.out.println("Unexpected exception raised!\n" + e.toString());
        }

    }

    public void ccm_remove() throws CCMException {
        try {
            super.teardownDDSInfrastructure(false, true);
        } catch (org.omg.CCM_DDS.InternalError e) {
            e.printStackTrace();
            logger.throwing(this.getClass().getName(), "ccm_remove", e);
            throw new CCMException(
                    "Failed to teardown the DDS infrastructure of fragment"
                            + "DDS_RawListen_PulseExecutorImpl",
                    CCMExceptionReason.REMOVE_ERROR);
        }
    }

    public void ccm_passivate() throws CCMException {

    }

    public void set_session_context(SessionContext ctx) throws CCMException {
        sessionContext = CCM_PulseEventConnector_Pulse_DDS_RawListen_push_consumer_ContextHelper
                .narrow(ctx);
    }

    protected TypeSupport getDataTypeSupport() {
        return dataTS;
    }

}
