/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_RawListen_Pulse;

import static cdmw.ccm.qosenablers.dds.DataUtil.fillReadInfo;

import java.util.logging.Logger;

import org.omg.CCM_DDS.CCM_ListenerControl;
import org.omg.CCM_DDS.ReadInfo;

import com.prismtech.mosic.radar.CCM_Pulse_Reader;
import com.prismtech.mosic.radar.Pulse;
import com.prismtech.mosic.radar.PulseDataReader;
import com.prismtech.mosic.radar.PulseDataReaderHelper;
import com.prismtech.mosic.radar.PulseSeqHolder;
import com.prismtech.mosic.radar.Pulse_RawListener;

import DDS.DataReader;
import DDS.RETCODE_OK;
import DDS.SampleInfoSeqHolder;
import cdmw.ccm.qosenablers.dds.DDS4CCMLoggers;
import cdmw.ccm.qosenablers.dds.DDS_ReaderImpl;
import cdmw.ccm.qosenablers.dds.DataReaderListenerImpl;

public class PulseDataReaderListenerImpl extends DataReaderListenerImpl {

    private Logger logger = DDS4CCMLoggers.getDDS_RawListenLogger();
    private Pulse_RawListener listenerReceptacle;

    public PulseDataReaderListenerImpl(Pulse_RawListener rawListen_listener,
            CCM_ListenerControl rawListen_control,
            CCM_Pulse_Reader rawListen_data) {
        super((DDS_ReaderImpl)rawListen_data, rawListen_control);
        this.listenerReceptacle = rawListen_listener;
    }

    public void on_data_available(DataReader dr) {
        if(!listenEnabler.enabled() || this.listenerReceptacle == null) return;

        PulseDataReader pulseReader = PulseDataReaderHelper.narrow(dr);

        PulseSeqHolder data = new PulseSeqHolder();
        SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();

//        int status = pulseReader.read_w_condition(data, infoSeq, 1, getReadCondition());
        int status = pulseReader.read(data, infoSeq, 1, DDS.NOT_READ_SAMPLE_STATE.value,
                DDS.NEW_VIEW_STATE.value
                | DDS.NOT_NEW_VIEW_STATE.value,
        DDS.ALIVE_INSTANCE_STATE.value
                | DDS.NOT_ALIVE_INSTANCE_STATE.value);

        if (status == RETCODE_OK.value) {
            if (data.value.length != 1) {
                logger.warning("Size mismatch : " + data.value.length);
                System.out.println("Size mismatch : " + data.value.length);
            }

            ReadInfo[] info = new ReadInfo[]{new ReadInfo()};
            fillReadInfo(infoSeq.value[0], info[0], 0);

            if (infoSeq.value[0].valid_data) {
                Pulse an_instance = data.value[0];
                this.listenerReceptacle.on_data(an_instance, info[0]);
            }

        }

    }


}
