/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_RawListen_Pulse;

import static cdmw.ccm.qosenablers.dds.DataUtil.initData;

import java.util.logging.Logger;

import org.omg.CCM_DDS.InternalError;
import org.omg.CCM_DDS.NonExistent;
import org.omg.CCM_DDS.ReadInfoHolder;
import org.omg.CCM_DDS.ReadInfoSeqHolder;

import DDS.LENGTH_UNLIMITED;
import DDS.SampleInfoSeqHolder;
import cdmw.ccm.qosenablers.dds.DDS4CCMLoggers;
import cdmw.ccm.qosenablers.dds.DDS_ReaderImpl;
import cdmw.ccm.qosenablers.dds.DataSeqHolder;

import com.prismtech.mosic.radar.CCM_Pulse_Reader;
import com.prismtech.mosic.radar.Pulse;
import com.prismtech.mosic.radar.PulseDataReader;
import com.prismtech.mosic.radar.PulseHolder;
import com.prismtech.mosic.radar.PulseSeqHolder;

public class Reader_PulseImpl extends DDS_ReaderImpl implements
        CCM_Pulse_Reader {

    private Logger logger = DDS4CCMLoggers.getDDS_RawListenLogger();
    PulseDataReader reader;

    public Reader_PulseImpl(PulseDataReader reader) {
        super(reader);
        this.reader = reader;
    }

    /**
     * returns for each living instance, its last sample ordered by instance
     * first and then by sample
     *
     * @param instances
     * @param infos
     * @throws InternalError
     */
    public void read_all(PulseSeqHolder instances, ReadInfoSeqHolder infos)
            throws InternalError {

        try {

            initData(new DataSeqHolder<Pulse, PulseSeqHolder>(Pulse[].class,
                    instances), infos, null);

            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();
            int status = reader.read_w_condition(instances, infoSeq,
                    LENGTH_UNLIMITED.value, this.userQuery);

            readAll(new DataSeqHolder<Pulse, PulseSeqHolder>(Pulse[].class,
                    instances), infos, infoSeq.value, status);

        } catch (Exception e) {
            logger.throwing(this.getClass().getName(), "read_all", e);
            throw new InternalError("Operation Failed: " + e.getMessage(),
                    DDS.RETCODE_ERROR.value, 0);
        }

    }

    /**
     * read_all_history returns all the samples of all instances ordered by
     * instance first and then by sample
     *
     * @param instances
     * @param infos
     * @throws InternalError
     */
    public void read_all_history(PulseSeqHolder instances,
            ReadInfoSeqHolder infos) throws InternalError {


        try {
            initData(new DataSeqHolder<Pulse, PulseSeqHolder>(Pulse[].class,
                    instances), infos, null);

            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();
            int status = reader.read_w_condition(instances, infoSeq,
                    LENGTH_UNLIMITED.value, this.userQuery);

            readAllHistory(new DataSeqHolder<Pulse, PulseSeqHolder>(
                    Pulse[].class, instances), infos, infoSeq.value, status);

        } catch (Exception e) {
            logger.throwing(this.getClass().getName(), "read_all_history", e);
            throw new InternalError("Operation Failed: " + e.getMessage(),
                    DDS.RETCODE_ERROR.value, 0);
        }

    }

    /**
     * returns the last sample of the given instance NOTA: userQuery is ignored
     * because read_instance_with_condition is not implemented by Splice ...
     */
    public void read_one(PulseHolder an_instance, ReadInfoHolder info)
            throws NonExistent, InternalError {

        try {

            long handle = reader.lookup_instance(an_instance.value);
            if (handle == DDS.HANDLE_NIL.value) {
                throw new NonExistent();
            }
            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();
            PulseSeqHolder instances = new PulseSeqHolder();
            int status = reader.read_instance(instances, infoSeq,
                    LENGTH_UNLIMITED.value, handle,
                    DDS.NOT_READ_SAMPLE_STATE.value
                            | DDS.READ_SAMPLE_STATE.value,
                    DDS.NEW_VIEW_STATE.value | DDS.NOT_NEW_VIEW_STATE.value,
                    DDS.ALIVE_INSTANCE_STATE.value);

            if (infoSeq.value[infoSeq.value.length - 1].valid_data) {
                an_instance.value = instances.value[infoSeq.value.length - 1];
            }

            readOne(info, infoSeq.value, status);

        } catch (Exception e) {
            logger.throwing(this.getClass().getName(), "read_one", e);
            throw new InternalError("Operation Failed: " + e.getMessage(),
                    DDS.RETCODE_ERROR.value, 0);
        }

    }

    /**
     * returns all the samples for the given instance NOTA: userQuery ignored
     */
    public void read_one_history(Pulse an_instance, PulseSeqHolder instances,
            ReadInfoSeqHolder infos) throws NonExistent, InternalError {

        try {

            long handle = reader.lookup_instance(an_instance);

            if (handle == DDS.HANDLE_NIL.value) {
                throw new NonExistent();
            }
            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();
            int status = reader.read_instance(instances, infoSeq,
                    DDS.LENGTH_UNLIMITED.value, handle, DDS.READ_SAMPLE_STATE.value
                            | DDS.NOT_READ_SAMPLE_STATE.value,
                    DDS.NEW_VIEW_STATE.value | DDS.NOT_NEW_VIEW_STATE.value,
                    DDS.ALIVE_INSTANCE_STATE.value);

            readOneHistory(infos, infoSeq.value, status);

        } catch (Exception e) {
            logger.throwing(this.getClass().getName(), "read_one_history", e);
            throw new InternalError("Operation Failed: " + e.getMessage(),
                    DDS.RETCODE_ERROR.value, 0);
        }

    }

}
