/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_Read_MotorPosition;

import java.util.logging.Logger;

import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.SessionContext;

import DDS.TypeSupport;
import cdmw.ccm.qosenablers.dds.DDS4CCMLoggers;
import cdmw.ccm.qosenablers.dds.DDS_TopicBaseFragmentImpl;
import cdmw.ccm.qosenablers.dds.ErrorHandler;

import com.prismtech.mosic.radar.CCM_MotorPositionStateConnector_MotorPosition_DDS_Read_passive_observer;
import com.prismtech.mosic.radar.CCM_MotorPosition_Reader;
import com.prismtech.mosic.radar.MotorPositionDataReader;
import com.prismtech.mosic.radar.MotorPositionDataReaderHelper;
import com.prismtech.mosic.radar.MotorPositionTypeSupport;

public class DDS_Read_MotorPositionExecutorImpl extends
        DDS_TopicBaseFragmentImpl implements
        CCM_MotorPositionStateConnector_MotorPosition_DDS_Read_passive_observer,
        org.omg.Components.SessionComponent {

    private java.util.logging.Logger logger = DDS4CCMLoggers.getDDS_ReadLogger();

    MotorPositionDataReader motorPositionReader;
    MotorPositionTypeSupport dataTS;

    CCM_MotorPosition_Reader data;

    // ------------- attributs ----------------

    public CCM_MotorPosition_Reader get_passive_observer_data() {
        return data;
    }

    public DDS_Read_MotorPositionExecutorImpl() {
        super();
        dataTS = new MotorPositionTypeSupport();
    }

    public void ccm_activate() throws CCMException {
        logger.info("DDS_Read_MotorPositionExecutorImpl.ccm_activate called ...");
        try {
            super.setupDDSInfrastructure(false, true);

            motorPositionReader = MotorPositionDataReaderHelper
                    .narrow(super.reader);
            ErrorHandler.checkHandle(motorPositionReader,
                    "DDS.MotorPositionDataReaderHelper.narrow");

            this.data = new Reader_MotorPositionImpl(motorPositionReader);

        } catch (Exception e) {
            logger.severe("Unexpected exception raise ! " + e.getMessage());
            logger.throwing(this.getClass().getName(), "ccm_activate", e);
            System.out.println("Unexpected exception raised!\n" + e.toString());
        }
    }

    public void ccm_remove() throws CCMException {
        try{
            super.teardownDDSInfrastructure(false, true);
        } catch (org.omg.CCM_DDS.InternalError e) {
            e.printStackTrace();
            logger.throwing(this.getClass().getName(), "ccm_remove", e);
            throw new CCMException(
                    "Failed to teardown the DDS infrastructure of fragment"
                            + "DDS_Read_MotorPositionExecutorImpl",
                    CCMExceptionReason.REMOVE_ERROR);
        }
    }

    public void ccm_passivate() throws CCMException {

    }

    public void set_session_context(SessionContext arg0) throws CCMException {
    }

    @Override
    protected TypeSupport getDataTypeSupport() {
        return dataTS;
    }

}
