/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_StateListen_MotorPosition;

import static cdmw.ccm.qosenablers.dds.DataUtil.fillReadInfo;

import java.util.Hashtable;

import org.omg.CCM_DDS.CCM_ListenerControl;
import org.omg.CCM_DDS.InstanceStatus;
import org.omg.CCM_DDS.ReadInfo;

import com.prismtech.mosic.radar.CCM_MotorPosition_Reader;
import com.prismtech.mosic.radar.MotorPosition;
import com.prismtech.mosic.radar.MotorPositionDataReader;
import com.prismtech.mosic.radar.MotorPositionDataReaderHelper;
import com.prismtech.mosic.radar.MotorPositionSeqHolder;
import com.prismtech.mosic.radar.MotorPosition_StateListener;

import DDS.DataReader;
import DDS.RETCODE_NO_DATA;
import DDS.RETCODE_OK;
import DDS.SampleInfoSeqHolder;
import cdmw.ccm.qosenablers.dds.DDS4CCMLoggers;
import cdmw.ccm.qosenablers.dds.DDS_ReaderImpl;
import cdmw.ccm.qosenablers.dds.DataReaderListenerImpl;

public class MotorPositionDataReaderListenerImpl extends DataReaderListenerImpl {

    private java.util.logging.Logger logger = DDS4CCMLoggers
            .getDDS_StateListenLogger();

    private MotorPosition_StateListener listenerReceptacle;
    private final int INSTANCE_ALIVE = 1;
    private final int INSTANCE_DELETED = 0;
    private Hashtable<Integer, Integer> historic = new Hashtable<Integer, Integer>();

    public MotorPositionDataReaderListenerImpl(
            MotorPosition_StateListener stateListen_listener,
            CCM_ListenerControl StateListen_control,
            CCM_MotorPosition_Reader StateListen_data) {
        super((DDS_ReaderImpl) StateListen_data, StateListen_control);
        this.listenerReceptacle = stateListen_listener;
    }

    public void on_data_available(DataReader dr) {

        if (!listenEnabler.enabled() || this.listenerReceptacle == null)
            return;

        MotorPositionDataReader motorPositionReader = MotorPositionDataReaderHelper
                .narrow(dr);

        MotorPositionSeqHolder data = new MotorPositionSeqHolder();
        SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();
        /* filter not actived */
        // int status = motorPositionReader.read_w_condition(data, infoSeq, 1,
        // getReadCondition());
        int status = motorPositionReader.read(data, infoSeq,
                DDS.LENGTH_UNLIMITED.value, DDS.NOT_READ_SAMPLE_STATE.value,
                DDS.NEW_VIEW_STATE.value | DDS.NOT_NEW_VIEW_STATE.value,
                DDS.ALIVE_INSTANCE_STATE.value
                        | DDS.NOT_ALIVE_INSTANCE_STATE.value);

        if (status == RETCODE_OK.value) {

            if (data.value.length == 1) {

                ReadInfo[] info = new ReadInfo[] { new ReadInfo() };
                fillReadInfo(infoSeq.value[0], info[0], 0);

                if (infoSeq.value[0].valid_data) {

                    MotorPosition an_instance = data.value[0];
                    org.omg.DDS.Time_t tmp = new org.omg.DDS.Time_t(
                            infoSeq.value[0].source_timestamp.sec,
                            infoSeq.value[0].source_timestamp.nanosec);

                    if (info[0].instance_status == InstanceStatus.INSTANCE_CREATED) {
                        historic.put(an_instance.angle, INSTANCE_ALIVE);
                        this.listenerReceptacle.on_creation(an_instance, tmp);
                    }

                    if (info[0].instance_status == InstanceStatus.INSTANCE_UPDATED) {
                        historic.put(an_instance.angle, INSTANCE_ALIVE);
                        this.listenerReceptacle.on_update(an_instance, tmp);
                    }

                    if (info[0].instance_status == InstanceStatus.INSTANCE_DELETED) {
                        historic.put(an_instance.angle, INSTANCE_DELETED);
                        this.listenerReceptacle.on_deletion(an_instance, tmp);
                    }

                }

            } else {

                ReadInfo[] info = new ReadInfo[data.value.length];

                for (int i = 0; i < infoSeq.value.length; i++) {

                    if (infoSeq.value[i].sample_rank == 0) {

                        if (infoSeq.value[i].valid_data) {

                            info[i] = new ReadInfo();
                            fillReadInfo(infoSeq.value[i], info[i],
                                    infoSeq.value[i].sample_rank);

                            MotorPosition an_instance = data.value[i];
                            org.omg.DDS.Time_t tmp = new org.omg.DDS.Time_t(
                                    infoSeq.value[i].source_timestamp.sec,
                                    infoSeq.value[i].source_timestamp.nanosec);

                            if (info[i].instance_status == InstanceStatus.INSTANCE_CREATED) {
                                historic.put(an_instance.angle, INSTANCE_ALIVE);
                                this.listenerReceptacle.on_creation(
                                        an_instance, tmp);
                            }

                            if (info[i].instance_status == InstanceStatus.INSTANCE_UPDATED) {
                                historic.put(an_instance.angle, INSTANCE_ALIVE);
                                this.listenerReceptacle.on_update(an_instance,
                                        tmp);
                            }

                            if (info[i].instance_status == InstanceStatus.INSTANCE_DELETED) {
                                historic.put(an_instance.angle,
                                        INSTANCE_DELETED);
                                this.listenerReceptacle.on_deletion(
                                        an_instance, tmp);
                            }

                        }

                    } else {
                        i = i - 1 + infoSeq.value[i].sample_rank;
                    }
                }

            }

        }

        if (status == RETCODE_NO_DATA.value) {

            status = motorPositionReader.read(data, infoSeq,
                    DDS.LENGTH_UNLIMITED.value, DDS.ANY_SAMPLE_STATE.value,
                    DDS.NEW_VIEW_STATE.value | DDS.NOT_NEW_VIEW_STATE.value,
                    DDS.NOT_ALIVE_DISPOSED_INSTANCE_STATE.value);

            for (int i = 0; i < infoSeq.value.length; i++) {
                if (infoSeq.value[i].sample_rank == 0) {
                    if (historic.get(data.value[i].angle) != null) {
                        if (historic.get(data.value[i].angle) == this.INSTANCE_ALIVE) {
                            org.omg.DDS.Time_t tmp = new org.omg.DDS.Time_t(
                                    infoSeq.value[i].source_timestamp.sec,
                                    infoSeq.value[i].source_timestamp.nanosec);
                            this.listenerReceptacle.on_deletion(data.value[i],
                                    tmp);
                            historic.put(data.value[i].angle, INSTANCE_DELETED);
                        }
                    }
                } else {
                    i = i - 1 + infoSeq.value[i].sample_rank;
                }
            }

        }

    }

}
