/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_StateListen_TargetDistance;

import org.omg.CCM_DDS.CCM_ListenerControl;
import org.omg.CORBA.Object;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.SessionContext;

import DDS.TypeSupport;
import cdmw.ccm.qosenablers.dds.DDS4CCMLoggers;
import cdmw.ccm.qosenablers.dds.DDS_TopicBaseFragmentImpl;
import cdmw.ccm.qosenablers.dds.ErrorHandler;
import cdmw.ccm.qosenablers.dds.ListenerControlImpl;

import com.prismtech.mosic.radar.CCM_TargetDistanceStateConnector_TargetDistance_DDS_StateListen_push_observer;
import com.prismtech.mosic.radar.CCM_TargetDistanceStateConnector_TargetDistance_DDS_StateListen_push_observer_Context;
import com.prismtech.mosic.radar.CCM_TargetDistanceStateConnector_TargetDistance_DDS_StateListen_push_observer_ContextHelper;
import com.prismtech.mosic.radar.CCM_TargetDistance_Reader;
import com.prismtech.mosic.radar.TargetDistanceDataReader;
import com.prismtech.mosic.radar.TargetDistanceDataReaderHelper;
import com.prismtech.mosic.radar.TargetDistanceTypeSupport;
import com.prismtech.mosic.radar.TargetDistance_StateListener;
import com.prismtech.mosic.radar.ddsfragments.DDS_Read_TargetDistance.Reader_TargetDistanceImpl;

public class DDS_StateListen_TargetDistanceExecutorImpl extends
        DDS_TopicBaseFragmentImpl implements
        CCM_TargetDistanceStateConnector_TargetDistance_DDS_StateListen_push_observer,
        org.omg.Components.SessionComponent {

    private java.util.logging.Logger logger = DDS4CCMLoggers.getDDS_StateListenLogger();

    /* Type-specific DDS entities */
    TargetDistanceTypeSupport dataTS;

    private CCM_TargetDistanceStateConnector_TargetDistance_DDS_StateListen_push_observer_Context sessionContext;

    private CCM_TargetDistance_Reader targetDistanceReaderPort;
    private TargetDistance_StateListener targetDistanceListenerPort;
    protected CCM_ListenerControl listenerControl;

    public DDS_StateListen_TargetDistanceExecutorImpl() {
        // this("", null, "DefaultTOPIC", null);
        super();
        this.listenerControl = new ListenerControlImpl();
        dataTS = new TargetDistanceTypeSupport();
    }

    public DDS_StateListen_TargetDistanceExecutorImpl(String domain_id,
            String qos_profile, String topic_name, String[] key_fields) {
        super(domain_id, qos_profile, topic_name, key_fields);
        this.listenerControl = new ListenerControlImpl();
        dataTS = new TargetDistanceTypeSupport();
    }

    public CCM_TargetDistance_Reader get_push_observer_data() {
        return this.targetDistanceReaderPort;
    }

    public CCM_ListenerControl get_push_observer_control() {
        return this.listenerControl;
    }

    public void ccm_activate() throws CCMException {
        try {
            super.setupDDSInfrastructure(false, true);
            // getting the typed datareader
            TargetDistanceDataReader targetDistanceDR = TargetDistanceDataReaderHelper
                    .narrow(reader);
            ErrorHandler.checkHandle(targetDistanceDR,
                    "DDS.TargetDistanceDataReaderHelper.narrow");

            // To create the facet executor
            this.targetDistanceReaderPort = new Reader_TargetDistanceImpl(
                    targetDistanceDR);

            // To get the component listener
            this.targetDistanceListenerPort = this.sessionContext
                    .get_connection_push_observer_listener();

            if (targetDistanceListenerPort == null)
                logger.fine("Failed to get the component StateListen facet !");

            // To set the DDS listener
            TargetDistanceDataReaderListenerImpl listener = new TargetDistanceDataReaderListenerImpl(
                    targetDistanceListenerPort, listenerControl,
                    targetDistanceReaderPort);
            int status = targetDistanceDR.set_listener(listener,
                    DDS.DATA_AVAILABLE_STATUS.value);
            ErrorHandler.checkStatus(status, "set_listener");

        } catch (Exception e) {
            logger.severe("Unexpected exception raise ! " + e.getMessage());
            logger.throwing(this.getClass().getName(), "ccm_activate", e);
            System.out.println("Unexpected exception raised!\n" + e.toString());
        }

    }

    public void ccm_remove() throws CCMException {
        try{
            super.teardownDDSInfrastructure(false, true);
        } catch (org.omg.CCM_DDS.InternalError e) {
            e.printStackTrace();
            logger.throwing(this.getClass().getName(), "ccm_remove", e);
            throw new CCMException(
                    "Failed to teardown the DDS infrastructure of fragment"
                            + "DDS_StateListen_TargetDistanceExecutorImpl",
                    CCMExceptionReason.REMOVE_ERROR);
        }
    }

    public void ccm_passivate() throws CCMException {

    }

    public void set_session_context(SessionContext ctx) throws CCMException {
        sessionContext = CCM_TargetDistanceStateConnector_TargetDistance_DDS_StateListen_push_observer_ContextHelper
                .narrow(ctx);
    }

    protected TypeSupport getDataTypeSupport() {
        return dataTS;
    }


}
