/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_Update_MotorPosition;

import org.omg.CORBA.Object;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.SessionContext;

import DDS.TypeSupport;
import cdmw.ccm.qosenablers.dds.DDS4CCMLoggers;
import cdmw.ccm.qosenablers.dds.DDS_TopicBaseFragmentImpl;
import cdmw.ccm.qosenablers.dds.ErrorHandler;

import com.prismtech.mosic.radar.CCM_MotorPositionStateConnector_MotorPosition_DDS_Update_observable;
import com.prismtech.mosic.radar.CCM_MotorPositionStateConnector_MotorPosition_DDS_Update_observable_Context;
import com.prismtech.mosic.radar.CCM_MotorPositionStateConnector_MotorPosition_DDS_Update_observable_ContextHelper;
import com.prismtech.mosic.radar.CCM_MotorPosition_Updater;
import com.prismtech.mosic.radar.MotorPositionDataReader;
import com.prismtech.mosic.radar.MotorPositionDataReaderHelper;
import com.prismtech.mosic.radar.MotorPositionDataWriter;
import com.prismtech.mosic.radar.MotorPositionDataWriterHelper;
import com.prismtech.mosic.radar.MotorPositionTypeSupport;

public class DDS_Update_MotorPositionExecutorImpl extends
        DDS_TopicBaseFragmentImpl implements
        CCM_MotorPositionStateConnector_MotorPosition_DDS_Update_observable,
        org.omg.Components.SessionComponent {

    private java.util.logging.Logger logger = DDS4CCMLoggers.getDDS_UpdateLogger();

    private MotorPositionTypeSupport dataTS;
    private CCM_MotorPositionStateConnector_MotorPosition_DDS_Update_observable_Context sessionContext;
    private CCM_MotorPosition_Updater target_distance_updater;

    public DDS_Update_MotorPositionExecutorImpl() {
        super();
        dataTS = new MotorPositionTypeSupport();
    }

    public DDS_Update_MotorPositionExecutorImpl(String domain_id,
            String qos_profile, String topic_name, String[] key_fields) {
        super(domain_id, qos_profile, topic_name, key_fields);
        dataTS = new MotorPositionTypeSupport();
    }

    public CCM_MotorPosition_Updater get_observable_data() {
        return this.target_distance_updater;
    }

    protected TypeSupport getDataTypeSupport() {
        return dataTS;
    }

    public void ccm_activate() throws CCMException {
        System.out
                .println("DDS_Update_MotorPositionExecutorImpl.ccm_activate called ...");
        try {
            super.setupDDSInfrastructure(true, true);

            MotorPositionDataReader motorPositionDR = MotorPositionDataReaderHelper
                    .narrow(reader);
            ErrorHandler.checkHandle(motorPositionDR,
                    "DDS.MotorPositionDataReaderHelper.narrow");

            /* Narrow the abstract parent into its typed representative. */
            MotorPositionDataWriter talker = MotorPositionDataWriterHelper
                    .narrow(writer);
            ErrorHandler.checkHandle(talker,
                    "!!!!MotorPosition.MotorPositionDataWriterHelper.narrow");

            this.target_distance_updater = new Updater_MotorPositionImpl(
                    talker, motorPositionDR);

        } catch (Exception e) {
            logger.severe("Unexpected exception raise ! " + e.getMessage());
            logger.throwing(this.getClass().getName(), "ccm_activate", e);
            System.out.println("Unexpected exception raised!\n" + e.toString());
        }
    }

    public void ccm_remove() throws CCMException {

        try {
            /* Remove the DataWriters */
            int status = this.publisher.delete_datawriter(writer);
            ErrorHandler.checkStatus(status,
                    "DDS.Publisher.delete_datawriter (talker)");
            super.teardownDDSInfrastructure(true, true);

        } catch (org.omg.CCM_DDS.InternalError e) {
            e.printStackTrace();
            logger.throwing(this.getClass().getName(), "ccm_remove", e);
            throw new CCMException(
                    "Failed to teardown the DDS infrastructure of fragment"
                            + "DDS_Update_MotorPositionExecutorImpl",
                    CCMExceptionReason.REMOVE_ERROR);
        }

    }

    public void ccm_passivate() throws CCMException {
        // TODO Auto-generated method stub
    }

    public void set_session_context(SessionContext ctx) throws CCMException {
        sessionContext = CCM_MotorPositionStateConnector_MotorPosition_DDS_Update_observable_ContextHelper
                .narrow(ctx);
    }

}
