/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_Write_Pulse;

import org.omg.CCM_DDS.DDSEntityKind;
import org.omg.CCM_DDS.InternalError;
import org.omg.CCM_DDS.NoSuchDDSEntity;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.SessionContext;

import DDS.RETCODE_OK;
import DDS.TypeSupport;
import cdmw.ccm.qosenablers.dds.DDS4CCMLoggers;
import cdmw.ccm.qosenablers.dds.DDS_TopicBaseFragmentImpl;
import cdmw.ccm.qosenablers.dds.ErrorHandler;

import com.prismtech.mosic.radar.CCM_PulseEventConnector_Pulse_DDS_Write_supplier;
import com.prismtech.mosic.radar.CCM_Pulse_Writer;
import com.prismtech.mosic.radar.Pulse;
import com.prismtech.mosic.radar.PulseDataWriter;
import com.prismtech.mosic.radar.PulseDataWriterHelper;
import com.prismtech.mosic.radar.PulseTypeSupport;

public class DDS_Write_PulseExecutorImpl extends DDS_TopicBaseFragmentImpl
        implements CCM_Pulse_Writer,
        CCM_PulseEventConnector_Pulse_DDS_Write_supplier,
        org.omg.Components.SessionComponent {

    private java.util.logging.Logger logger = DDS4CCMLoggers
            .getDDS_WriteLogger();
    PulseTypeSupport dataTS;
    PulseDataWriter talker;

    public DDS_Write_PulseExecutorImpl() {
        dataTS = new PulseTypeSupport();
    }

    public CCM_Pulse_Writer get_supplier_data() {
        return this;
    }

    public void ccm_activate() throws CCMException {
        logger
                .info("DDS_Write_PulseExecutorImpl.ccm_activate called ...");
        try {
            super.setupDDSInfrastructure(true, false);
            talker = PulseDataWriterHelper.narrow(this.writer);
            ErrorHandler.checkHandle(talker,
                    "Pulse.PulseDataWriterHelper.narrow");

        } catch (Exception e) {
            logger.severe("Unexpected exception raise ! " + e.getMessage());
            logger.throwing(this.getClass().getName(), "ccm_activate", e);
            System.out.println("Unexpected exception raised!\n" + e.toString());
        }

    }

    public void ccm_remove() throws CCMException {

        try {
            int status = this.publisher.delete_datawriter(talker);
            ErrorHandler.checkStatus(status,
                    "DDS.Publisher.delete_datawriter (talker)");
            super.teardownDDSInfrastructure(true, false);

        } catch (Exception e) {
            e.printStackTrace();
            logger.throwing(this.getClass().getName(), "ccm_remove", e);
            throw new CCMException(
                    "Failed to teardown the DDS infrastructure of fragment"
                            + "DDS_Write_PulseExecutorImpl",
                    CCMExceptionReason.REMOVE_ERROR);
        }

    }

    public void ccm_passivate() throws CCMException {

    }

    public void set_session_context(SessionContext arg0) throws CCMException {
    }

    @Override
    protected TypeSupport getDataTypeSupport() {
        return dataTS;
    }

    /**
     * The Writer facet implementation
     *
     * /
     **/

    /**
     *
     * @param an_instance
     * @throws InternalError
     */
    public void write(Pulse an_instance) throws InternalError {
        long userHandle = talker.register_instance(an_instance);
        int status = talker.write(an_instance, userHandle);
        if (status != RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = status;
            logger.warning("Failed to write the instance ! Error "
                    + ErrorHandler.getErrorName(status));
            throw e;
        }
    }

    public org.omg.CORBA.Object getDDSEntity(DDSEntityKind kind)
            throws NoSuchDDSEntity {
        return null;
    }

}
