#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================
CDM_RESULT_DIR=`dirname ${0}`/results
if [  ! -d ${CDM_RESULT_DIR} ] ; then
  echo "it's necessary to create the ${CDM_RESULT_DIR} directory"
fi


export SCRIPTNAME=`basename $0`

HOST1=${CDMW_HOSTNAME_1}
HOST2=${CDMW_HOSTNAME_2}
HOST3=${CDMW_HOSTNAME_3}
HOST4=${CDMW_HOSTNAME_4}
HOST5=${CDMW_HOSTNAME_5}
    
# horizontal size of xterm (nb of columns)
HORIZONTAL_SIZE=81
# vertical size of xterm (nb of lignes)
VERTICAL_SIZE=8
# vertical space between xterm (pixels ?)
VERTICAL_SPACE=130



#------------------------------------------------------------------------------#
#                                print_usage                                   #
#------------------------------------------------------------------------------#
# display the usage of this script
#------------------------------------------------------------------------------#
print_usage()
{
    echo " "
    echo "usage: $SCRIPTNAME [--help] <user_login> [<install_valid_path>]"
    echo " "
    echo "    <user_login>         : login to be used for xterms 1, 2, 5, 6 & 9 "
    echo "                           (xterms 3, 4 & 8 will be logged as root for Daemons)"
    echo "    --help               : display this help"
    echo " "
    exit 0
}


print_root_error()
{
    echo " "
    echo "Error: you should be root to run $SCRIPTNAME !"
    exit -1
}

print_user_error()
{
    echo " "
    echo "Error: user $1 is inexistant !"
    exit -1
}

print_no_valid_path_error()
{
    echo " "
    echo "Error: qualification directory path should be specified !"
    exit -1
}

print_valid_path_warning()
{
    echo " "
    echo "Warning: validation directory path $1 doesn't exist on local host !"
    echo " "
}

#################
parse_arguments()
{
    # test is this script is started as root
    if [ `whoami` != root ]; then
        print_root_error
    fi

    if [ "$1" == "" ]; then
        print_usage
    fi
    if [ "$1" == "--help" ]; then
        print_usage
    fi
    
    USERLOGIN=$1
    TEST_USER=`id $USERLOGIN 2>/dev/null`
    if [ "$TEST_USER" = "" ]; then 
        print_user_error $USERLOGIN
    fi
    
    VALID_PATH=${CDMW_HOME}

    if [ "$VALID_PATH" = "" ]; then
        print_usage
    fi
    if [ ! -d $VALID_PATH ]; then
        print_valid_path_warning $VALID_PATH
    fi

}

# find xterm
if [ -f /usr/bin/xterm ]; then
    TERM=/usr/bin/xterm
elif [ -f /usr/X11R6/bin/xterm ]; then 
    TERM=/usr/X11R6/bin/xterm
elif [ -f USER/usr/X/bin/xterm ]; then
    TERM=/usr/X/bin/xterm
else
    TERM=xterm
fi


# TERM COMMAND
export TERM="$TERM +ls -sb -sl 10000"




#################################
# start a xterm as specified user
#   arg1 = user login
#   arg2 = hostname
#   arg3 = xterm title
#   arg4 = xterm geometry
#   following args = commands to be executed on host before xterm start
#################################
start_xterm_as_user ()
{
    USER=$1
    HOST=$2
    TITLE=$3
    GEO=$4
    shift; shift; shift; shift
    
    if [ "${HOST}" != "NONE" ]; then
      # set xterm color according to host
      case $HOST in
          r2d2 )
              # light grey
              XTERM_COLOR=rgb:ea/ea/ea ;;
          solo )
              # yellow
              XTERM_COLOR=rgb:ff/ff/cd ;;
          leia )
              # fushia
              XTERM_COLOR=rgb:ff/f0/ff ;;
          ackbar )
              # purple
              XTERM_COLOR=rgb:e2/d9/ff ;;
          boba )
              # orange
              XTERM_COLOR=rgb:ff/f0/b7 ;;
          calrissian )
              # green
              XTERM_COLOR=rgb:e6/ff/e6 ;;
          vador )
              # dark grey
              XTERM_COLOR=rgb:c0/c0/c0 ;;
          padmee )
              # pink
              XTERM_COLOR=rgb:fff/e1/e1 ;;
          c3po )
              # beige
              XTERM_COLOR=rgb:ef/d3/ad ;;
          *)
              # white
              XTERM_COLOR=rgb:ff/ff/ff ;;
      esac
      case ${USER} in
        root)
          TERM_PRE_COMMAND="cd $VALID_PATH/bin; ./set_root.sh; . ./set_env.sh; cd ../share; export CDM_USER=${USERLOGIN}; export NIS_HOST=${NIS_HOST}; export HOST1=${HOST1};export HOST2=${HOST2};export HOST3=${HOST3};export HOST4=${HOST4};export HOST5=${HOST5}; export CDM_RESULT_DIR=${CDM_RESULT_DIR}"
        ;;
        *)
          TERM_PRE_COMMAND="cd $VALID_PATH/bin; . ./set_env.sh; cd ../share; export CDM_USER=${USERLOGIN}; export NIS_HOST=${NIS_HOST};export HOST1=${HOST1};export HOST2=${HOST2};export HOST3=${HOST3};export HOST4=${HOST4};export HOST5=${HOST5}; export CDM_RESULT_DIR=${CDM_RESULT_DIR}"
        ;;
      esac
        
      echo "${TITLE}_on_${HOST}@${USER}"
      ssh -l $USER -X -f $HOST "${TERM_PRE_COMMAND}; unset PROMPT_COMMAND; export PS1=${TITLE}_on_${HOST}@${USER}\>\ ; $TERM -bg $XTERM_COLOR -geometry $GEO -title $TITLE -l " 
      sleep 2
    fi
}

##########
###MAIN###
##########
# parse arguments
    TERM1_HOST=NONE
    TERM2_HOST=NONE
    TERM3_HOST=NONE
    TERM4_HOST=NONE
    TERM5_HOST=NONE
    TERM6_HOST=NONE
    TERM7_HOST=NONE
    TERM8_HOST=NONE
    TERM9_HOST=NONE
    TERM10_HOST=NONE
    TERM11_HOST=NONE
    TERM1_USER=NONE
    TERM2_USER=NONE
    TERM3_USER=NONE
    TERM4_USER=NONE
    TERM5_USER=NONE
    TERM6_USER=NONE
    TERM7_USER=NONE
    TERM8_USER=NONE
    TERM9_USER=NONE
    TERM10_USER=NONE
    TERM11_USER=NONE
parse_arguments $*


#ssh keys
echo "exporting ssh keys"
if [ ! -f ~/.ssh/id_rsa.pub ]; then 
    ssh-keygen -t rsa -f ~/.ssh/id_rsa -N ""
fi

cat ~/.ssh/id_rsa.pub | ssh root@$HOST1 "cat - >> ~/.ssh/authorized_keys"
cat ~/.ssh/id_rsa.pub | ssh root@$HOST2 "cat - >> ~/.ssh/authorized_keys"
cat ~/.ssh/id_rsa.pub | ssh root@$HOST3 "cat - >> ~/.ssh/authorized_keys"
cat ~/.ssh/id_rsa.pub | ssh root@$HOST4 "cat - >> ~/.ssh/authorized_keys"
cat ~/.ssh/id_rsa.pub | ssh root@$HOST5 "cat - >> ~/.ssh/authorized_keys"

cat ~/.ssh/id_rsa.pub | ssh $USERLOGIN@$HOST1 "cat - >> ~/.ssh/authorized_keys"
cat ~/.ssh/id_rsa.pub | ssh $USERLOGIN@$HOST2 "cat - >> ~/.ssh/authorized_keys"
cat ~/.ssh/id_rsa.pub | ssh $USERLOGIN@$HOST3 "cat - >> ~/.ssh/authorized_keys"
cat ~/.ssh/id_rsa.pub | ssh $USERLOGIN@$HOST4 "cat - >> ~/.ssh/authorized_keys"
cat ~/.ssh/id_rsa.pub | ssh $USERLOGIN@$HOST5 "cat - >> ~/.ssh/authorized_keys"


ssh -X $USERLOGIN@$HOST1 "cd ${VALID_PATH};if [ -d LOGS ]; then mv LOGS LOGS_`date +%Y_%m_%d-%H_%M`; fi; mkdir -p LOGS/${HOST1} LOGS/${HOST2} LOGS/${HOST3} LOGS/${HOST4} LOGS/${HOST5}"

TERM1_HOST=$HOST1
TERM2_HOST=$HOST1
TERM3_HOST=$HOST2    
TERM4_HOST=$HOST1

TERM5_HOST=$HOST1
TERM5_USER=${USERLOGIN}
TERM5_TITLE="05___Platform_Supervision"

TERM6_HOST=$HOST2
TERM6_USER=$USERLOGIN
TERM6_TITLE="06___Deployment_Tool"

TERM7_HOST=$HOST3
TERM7_USER=root
TERM7_TITLE="07___Test_results_HOST_3"

TERM8_HOST=$HOST1
TERM8_TITLE="08___Primary_FT_Manager"
TERM8_USER=${USERLOGIN}

TERM9_HOST=$HOST2
TERM9_TITLE="09___Backup_FT_Manager"

TERM10_HOST=$HOST4
TERM10_USER=root
TERM10_TITLE="10___Test_results_HOST_4"

TERM11_HOST=$HOST5
TERM11_USER=root
TERM11_TITLE="11___Test_results_HOST_5"


# TERM1: platform admin
TERM1_USER=$USERLOGIN
TERM1_TITLE="01___Platform_Admin"
TERM1_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}+0+0"
start_xterm_as_user $TERM1_USER $TERM1_HOST $TERM1_TITLE $TERM1_GEO 

# TERM2: platform observer
TERM2_USER=$USERLOGIN
TERM2_TITLE="02___Platform_Observer"
TERM2_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+0"
start_xterm_as_user $TERM2_USER $TERM2_HOST $TERM2_TITLE $TERM2_GEO 

# TERM3: daemon on HOST2
TERM3_USER=root
TERM3_TITLE="03___Test_results_HOST_2"
TERM3_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+$VERTICAL_SPACE"
start_xterm_as_user $TERM3_USER $TERM3_HOST $TERM3_TITLE $TERM3_GEO 

# TERM4: daemon on HOST1
TERM4_USER=root
TERM4_TITLE="04___Test_results_HOST_1"
TERM4_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}+0+$VERTICAL_SPACE"
start_xterm_as_user $TERM4_USER $TERM4_HOST $TERM4_TITLE $TERM4_GEO 

# TERM5: platform supervision
TERM5_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}+0+$((VERTICAL_SPACE * 2))"
start_xterm_as_user $TERM5_USER $TERM5_HOST $TERM5_TITLE $TERM5_GEO 

# TERM6: deployment
TERM6_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+$((VERTICAL_SPACE * 2))"
start_xterm_as_user $TERM6_USER $TERM6_HOST $TERM6_TITLE $TERM6_GEO 

# TERM7: test results on HOST3
TERM7_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+$((VERTICAL_SPACE * 3))"
start_xterm_as_user $TERM7_USER $TERM7_HOST $TERM7_TITLE $TERM7_GEO 

# TERM8:
TERM8_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}+0+$((VERTICAL_SPACE * 4))"
start_xterm_as_user $TERM8_USER $TERM8_HOST $TERM8_TITLE $TERM8_GEO 

# TERM9: fault tolerance manager
TERM9_USER=$USERLOGIN
TERM9_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+$((VERTICAL_SPACE * 4))"
start_xterm_as_user $TERM9_USER $TERM9_HOST $TERM9_TITLE $TERM9_GEO 

# TERM10: test results on HOST4
TERM10_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}+0+$((VERTICAL_SPACE * 5))"
start_xterm_as_user $TERM10_USER ${TERM10_HOST} $TERM10_TITLE $TERM10_GEO $TERM10_PRE_COMMAND

# TERM11: test results on HOST4
TERM11_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+$((VERTICAL_SPACE * 5))"
start_xterm_as_user $TERM11_USER ${TERM11_HOST} $TERM11_TITLE $TERM11_GEO $TERM11_PRE_COMMAND

