/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007.
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_PLATFORMMNGT_HOSTPROCESSINIT_HPP
#define INCL_PLATFORMMNGT_HOSTPROCESSINIT_HPP

#include "idllib/CdmwPlatformMngtProcess.stub.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp>
#include "platformapplicationlibrary/Process_impl.hpp"
#include <iostream>
#include <vector>
#include <map>

namespace Cdmw
{
namespace PlatformMngt
{

class Process_impl;

using namespace std;

// vector of ProcessHostInfo in relation with a host
typedef std::vector<ProcessHostInfo> ProcessInfos;
typedef std::map<std::string,Cdmw::PlatformMngt::ProcessInfos*> MapHostProcess;

typedef MapHostProcess::iterator MapIterator;

class CommandKind;

class HostProcessInitSequencer
{
public:
    static HostProcessInitSequencer* get_instance();

    void print();

    void set_run_entry(const std::string& host_name, ProcessHostInfo proc_info);

    void remove_run_entry(const std::string& host_name, ProcessHostInfo proc_info);

    void set_init_entry(const std::string& host_name, ProcessHostInfo proc_info);

    void remove_init_entry(const std::string& host_name, ProcessHostInfo proc_info);

    void initialise_host_processes(CdmwPlatformMngt::RestartWarmth restart_warmth, const std::string& host_name)
    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException);

    void run_host_processes(CdmwPlatformMngt::RestartWarmth restart_warmth, const std::string& host_name)
    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException);

    void stop_host_processes(const std::string& host_name);


private:
    HostProcessInitSequencer();

    void set_entry(const std::string& host_name, ProcessHostInfo proc_info, int m_select);

    void remove_entry(const std::string& host_name, ProcessHostInfo proc_info, int m_select);

    void perform_command(CommandKind* command_kind, CdmwPlatformMngt::RestartWarmth restart_warmth,
            const std::string& host_name, int m_select)
    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException);

    MapHostProcess m_host_process_map[2];

};

} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_HOSTINITPROCESS_HPP

