/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>
#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>
#include "daemoncontroller/DaemonController_impl.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemonController.skel.hpp"

extern std::string BASE_EXIT_ACTION;
using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;    

namespace
{
    const std::string DAEMON_PORT_OPTION = "--daemonport-number";
    const std::string BEXITACTION = "--exit-action";
    const std::string NAME_DAEMON_CORBALOC = "CdmwPlatformMngtDaemon";
    const std::string FILE_LOG = "/tmp/daemoncontrollerlog.txt";
    const int POA_THREAD_POOL_SIZE = 5;
}

//write in log file
extern void writeRowInLog(std::string value);
 
//Find the platforl daemon controller in file /tmp/CdmwDaemon_<port>/ __CDMW_DAEMON__.pid
int findDaemonPid(std::string daemonport){
    int daemon_pid = -1;
    
    std::string file_name = "/tmp/CdmwDaemon_" + daemonport + "/__CDMW_DAEMON__.pid";
    writeRowInLog(file_name);
    //std::cout << file_name;
    // import the object reference from the file
    std::ifstream fin; 
    fin.open(file_name.c_str());
    
    writeRowInLog("waiting platform daemon.... ");
    while(!fin.is_open()) {
        sleep(1);
        fin.open(file_name.c_str());
    }
    std::ifstream is(file_name.c_str());
    std::cout<< "platform daemon find!!! " << std::endl;
    writeRowInLog("platform daemon find!!! ");
    std::string objref_str;
    is >> objref_str;
    daemon_pid = atoi(objref_str.c_str());
    return daemon_pid;
}   

class DaemonChecker : public Cdmw::OsSupport::Thread
{
    public:
        int localDaemonPid;
        CdmwPlatformMngt::DaemonController_var localdm;
        std::string localplatform_daemon_port;
        CORBA::ORB_var localorb;
        /**
        * Purpose:
        * <p> Constructor
        * 
        *@param DaemonPid
        *
        */ 
        DaemonChecker::DaemonChecker (CdmwPlatformMngt::DaemonController_var dm, std::string platform_daemon_port,
        CORBA::ORB_var orb){
        localDaemonPid = 0;
        localdm = dm;
        localplatform_daemon_port = platform_daemon_port;
        localorb = orb;
    };
        /**
        * Purpose:
        * <p> Destructor
        * 
        */  
        DaemonChecker::~DaemonChecker()
                throw(){};

        void DaemonChecker::run() throw(){
        while(true){
            int localDaemonPid = findDaemonPid(localplatform_daemon_port);
            sleep(2000);     
            try {
                writeRowInLog("Register daemon controller in daemon's local services with name DaemonController");
                std::string daemon_corbaloc =  "corbaloc::localhost:"+ localplatform_daemon_port + "/" + NAME_DAEMON_CORBALOC;
                CORBA::Object_var obj_daemon = localorb->string_to_object(daemon_corbaloc.c_str());
                CdmwPlatformMngt::Daemon_var daemon = CdmwPlatformMngt::Daemon::_narrow(obj_daemon.in());
                daemon->register_local_service("DaemonController", localdm.in());
                CORBA::Object_var obv = daemon->resolve_local_service("DaemonController");
                writeRowInLog("Start controller");
            }   
            catch (const CORBA::BAD_PARAM& e) {
                std::cerr << "Local service DaemonController is unreachable";     
                writeRowInLog("Local service DaemonController is unreachable");
            }
            catch (const CORBA::SystemException& e) {
                // not yet started        
            }  
            int res = 0;
            if(localDaemonPid != -1){
                while(res == 0){
                    res = kill(localDaemonPid, 0);
                    //std::cout << "Result of kill: " << res << std::endl;
                    sleep(1000);
                }
            std::cout << "Platform daemon dies" << std::endl;
            writeRowInLog("Platform daemon dies");
            }
            else{
                std::cout << "Invalid pid process" << std::endl;
                writeRowInLog("Invalid pid process");
            }
            localdm->notify(0);
        }
    }
};

static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " options\n"
       << "\n"
       << "Options:\n"
       << "--help                                   Display this information.\n"
       << "[--daemonport-number=<number>]           The port number of the platform daemon.\n"
       << "[--CdmwLocalisationService=<number>]     The port number of the Controller.\n"
       << std:: endl;
}
    
int main(int argc, char* argv[])
{   
    writeRowInLog("---------START-------");   
    // the return code 
    int ret_code = 0;
    StrategyList strategy;
    strategy.add_OrbThreaded();
    strategy.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
    CORBA::ORB_var orb;

    //
    // help argument
    //
    if (argc > 1)
    {
        if(strcmp(argv[1], "--help") == 0)
        {
            usage(std::cout, argv[0]);
            return ret_code;
        }
    }
    if (argc == 1)
    {
        usage(std::cout, argv[0]);
        return ret_code;   
    }
   
    // set localisation_port
    char** ORB_init_argv = 0;
    int ORB_init_argc=argc;
    
    
    std::string localiaction = OS::get_option_value (argc, argv, Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION);
    if (localiaction == "yes" || localiaction == "no")
    {
        std::string str_opt( "" );
        for (int i=0; i<argc; i++) str_opt += std::string( argv[i] ) + " ";
        str_opt += std::string(Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION );
        str_opt += "=" ;
        str_opt += "5051";
        std::cout << "Parametri: "<< str_opt << std::endl;
        ORB_init_argc += 1;
        ORB_init_argv = Cdmw::Common::String::to_char_array(
        Cdmw::Common::String::to_strings( str_opt ) );
    }
    else{
        std::string str_opt( "" );
        for (int i=0; i<argc; i++) str_opt += std::string( argv[i] ) + " ";
        std::cout << "Parametri: "<< str_opt << std::endl;
        ORB_init_argv = argv;
    }   
    std::string platform_daemon_port;
    try
    {
        platform_daemon_port = OS::get_option_value (argc, argv, DAEMON_PORT_OPTION);
        if (platform_daemon_port == "yes" || platform_daemon_port == "no")
        {
            std::cout << "bad value for " << DAEMON_PORT_OPTION.c_str() << std::endl;
            return 0;
        }
        std::string bea = OS::get_option_value (argc, argv, BEXITACTION);
        if (!(bea == "yes" || bea == "no"))
        {
            BASE_EXIT_ACTION = bea;
        }
    }      
    catch(...)
    {
        return 0;
    }
    orb =  Cdmw::OrbSupport::OrbSupport::ORB_init(ORB_init_argc, ORB_init_argv, strategy); 
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");;
    PortableServer::POA_var poa = PortableServer::POA::_narrow(obj.in());
    PortableServer::POAManager_var poaMgr = poa->the_POAManager();
    poaMgr->activate();
    Cdmw::DaemonController_impl*  daemoncontroller = new Cdmw::DaemonController_impl();
    CdmwPlatformMngt::DaemonController_var dm = daemoncontroller->_this();    
    OrbSupport::OrbSupport::bind_object_to_corbaloc(orb.in(), "daemon_controller", dm.in() );    
    orb->register_initial_reference("DaemonController", dm.in()); 
    DaemonChecker dmc(dm,platform_daemon_port, orb);
    dmc.start();   
    orb->run();
    return ret_code;

}






