/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

#include <vector>

#include "platformadminlibrary/CcmComponentParser.hpp"
#include "platformadminlibrary/EntityParser.hpp"

namespace Cdmw {         // Begin namespace Cdmw
namespace PlatformAdmin { // Begin namespace PlatformAdmin

CcmComponentContainerParser::CcmComponentContainerParser(
    const DOM_Node& entityContainerParser )
: ConfigElementParser( entityContainerParser ),
  EntityContainerParser( entityContainerParser)
{
}

unsigned int CcmComponentContainerParser::get_ccmcomponent_count() const
{
    return get_child_count( "Ccm-component" );
}

CcmComponentParser CcmComponentContainerParser::get_ccmcomponent(
    unsigned int entityIndex ) const
{
    DOM_Node ccmcomponent_Parser(
        get_child_element( "Ccm-component", entityIndex ) );

    return CcmComponentParser( ccmcomponent_Parser );
}

CcmComponentParser::CcmComponentParser( const DOM_Node& entityParser )
: ConfigElementParser( entityParser ),
  EntityContainerParser( entityParser)
{
}

std::string CcmComponentParser::name() const
{
    return get_attribute_value( "Name" );
}

std::string CcmComponentParser::deployment_plan() const
{
    return get_attribute_value( "Deployment-plan" );    
}

std::string CcmComponentParser::redundancy_group_name() const
{
    return get_attribute_value( "Redundancy-group-name");   
}


CdmwPlatformMngt::ElementType CcmComponentParser::redundancy_group_type() const
{
    CdmwPlatformMngt::ElementType group_type = CdmwPlatformMngt::COMPONENT_TYPE;
    
    if ( "FT_GROUP_TYPE" == get_attribute_value( "Redundancy-group-type") )
    {
        group_type = CdmwPlatformMngt::FT_GROUP_TYPE;
    }
    else if ( "LB_GROUP_TYPE" == get_attribute_value( "Redundancy-group-type"))
    {
        group_type = CdmwPlatformMngt::LB_GROUP_TYPE;
    }
    else if ( "NONE" == get_attribute_value( "Redundancy-group-type") )
    {
        // If the component does not belong to any group, the group type is set to COMPONENT_TYPE
        group_type = CdmwPlatformMngt::COMPONENT_TYPE;
    }
    return group_type;
}

PropertyParser CcmComponentParser::get_property(unsigned int propertyIndex) const
{
    DOM_Node property_parser(get_child_element("Property", propertyIndex ));
    return PropertyParser(property_parser);
}

                                                                                                                 
unsigned int CcmComponentParser::get_property_count() const
{
     return get_child_count("Property");
}


} // End namespace PlatformAdmin
} // End namespace Cdmw

