/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007.
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "platformapplicationlibrary/HostProcessInitSequencer.hpp"
#include "platformapplicationlibrary/ManagedProcess_impl.hpp"
#include <cstdlib>
#include <iostream>
#include <vector>
#include <map>

using namespace std;

namespace Cdmw
{
namespace PlatformMngt
{


class CommandKind
{
private:
     virtual void execute_command(Process_impl* proc_impl) = 0;

public:
     CommandKind(){};

     CdmwPlatformMngtBase::StartupKind_var get_startup_kind();

    virtual void execute(Process_impl* proc_impl, CdmwPlatformMngt::RestartWarmth restart_warmth,
            KindOfStart kind_of_start)
    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
     {
        if ((DEFAULT == kind_of_start) ||
            ((ADDED == kind_of_start) && (CdmwPlatformMngt::HOT_RESTART == restart_warmth)) ||
            ((DEFAULT_REMOVED == kind_of_start) && (CdmwPlatformMngt::COLD_RESTART == restart_warmth)))
     {
         execute_command(proc_impl);
     }
        //else
        //{
     //std::cout<<"*** OUTSIDE IF OF kind_of_start in execute, kind_of_start="<<kind_of_start<<std::endl;
        //}
     };
};

CdmwPlatformMngtBase::StartupKind_var CommandKind::get_startup_kind() {
    CdmwPlatformMngtBase::StartupKind_var startupKind = new CdmwPlatformMngtBase::StartupKind();
    startupKind->state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
    startupKind->startup_mode = 0;
    startupKind->specified_state_id = CORBA::string_dup("");

    return startupKind;
}

class RunCommandKind: public CommandKind
{
public:
    virtual void execute_command(Process_impl* proc_impl)
    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        if (0 != dynamic_cast<ManagedProcess_impl*> (proc_impl))
        {
            proc_impl->run();
        }
    }
};


class InitCommandKind: public CommandKind
{
public:
    virtual void execute_command(Process_impl* proc_impl)
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        proc_impl->initialise(get_startup_kind());
    }
};


void HostProcessInitSequencer::set_init_entry(const std::string& host_name, ProcessHostInfo proc_info)
{
    set_entry(host_name, proc_info, 0);
}


void HostProcessInitSequencer::set_run_entry(const std::string& host_name, ProcessHostInfo proc_info)
{
    set_entry(host_name, proc_info, 1);
}


HostProcessInitSequencer* HostProcessInitSequencer::get_instance()
{
    static HostProcessInitSequencer host_proc_host_seq;
    return &host_proc_host_seq;
}

HostProcessInitSequencer::HostProcessInitSequencer()
{
}

void HostProcessInitSequencer::remove_init_entry(const std::string& host_name, ProcessHostInfo proc_info)
{
    remove_entry(host_name, proc_info, 0);
}

void HostProcessInitSequencer::remove_run_entry(const std::string& host_name, ProcessHostInfo proc_info)
{
    remove_entry(host_name, proc_info, 1);
}

void HostProcessInitSequencer::set_entry(const std::string& host_name, ProcessHostInfo proc_info, int m_select)
{
    MapIterator map_it = m_host_process_map[m_select].find(host_name);
    //check if an host_name entry is present,
    // if not create a vector
    if (map_it == m_host_process_map[m_select].end())
    {
        Cdmw::PlatformMngt::ProcessInfos* vect_host_proc = new ProcessInfos(1);
        (*vect_host_proc)[0] = proc_info;
        m_host_process_map[m_select].insert(MapHostProcess::value_type(host_name, vect_host_proc));
    }
    else //update the vector in map
    {
        // check if the entry is not present in the vector
        int size_vector = map_it->second->size();
        bool found = false;
        for (int i=0; i< size_vector; i++)
        {
            if ( (*map_it->second)[i].process_name == proc_info.process_name)
            {
                found = true;
                //substitute the entry (to update ProcessHostInfo.kind_of_start)
                (*map_it->second)[i] = proc_info;
                break;
            }
        }
        if (!found)
        {
            map_it->second -> push_back(proc_info);
        }
    }
}

void HostProcessInitSequencer::remove_entry(const std::string& host_name, ProcessHostInfo proc_info, int m_select)
{
    MapIterator map_it = m_host_process_map[m_select].find(host_name);
    //check if an host_name entry is present,
    // if not create a vector
    if (map_it != m_host_process_map[m_select].end())
    {
        // check if the entry is not present in the vector
        int size_vector = map_it->second->size();
        for (int i=0; i< size_vector; i++)
        {
            if (( (*map_it->second)[i].process_name == proc_info.process_name) &&
                  ( ADDED == (*map_it->second)[i].kind_of_start))
            {
                for (int j=i;j<(size_vector-1);j++)
                {
                    (*map_it->second)[j] = (*map_it->second)[j+1];
                }
                map_it->second->resize(size_vector - 1);
            }

            if (( (*map_it->second)[i].process_name == proc_info.process_name) &&
                  ( DEFAULT == (*map_it->second)[i].kind_of_start))
            {
                (*map_it->second)[i] = proc_info;
            }

            if (map_it->second->size() == 0)
            {
                m_host_process_map[m_select].erase(host_name);
            }
        }
    }
}

void HostProcessInitSequencer::initialise_host_processes(CdmwPlatformMngt::RestartWarmth restart_warmth,
        const std::string& host_name)
throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
{
    InitCommandKind* init_cmd = new InitCommandKind;
    perform_command(init_cmd, restart_warmth, host_name,0);
};

void HostProcessInitSequencer::run_host_processes(CdmwPlatformMngt::RestartWarmth restart_warmth,
        const std::string& host_name)
throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
{
    RunCommandKind* run_cmd = new RunCommandKind;
    perform_command(run_cmd, restart_warmth, host_name,1);
};

void HostProcessInitSequencer::perform_command(CommandKind* command_kind,
        CdmwPlatformMngt::RestartWarmth restart_warmth,const std::string& host_name, int m_select)
throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
{

    MapIterator map_it = m_host_process_map[m_select].find(host_name);
    //check if an host_name entry is present,
    // if not create a vector

    // check if the entry is not present in the vector
    int size_vector = map_it->second->size();
    for (int i=0; i< size_vector; i++)
    {
        try
        {
            (*map_it->second)[i].process_impl->set_requested_restart_warmth(restart_warmth);
            command_kind->execute((*map_it->second)[i].process_impl, restart_warmth, (*map_it->second)[i].kind_of_start);
        }
        catch(const CdmwPlatformMngt::IncompatibleStatus& ex)
        {
        }
    }
}

void HostProcessInitSequencer::print()
{

 for (int select = 0; select<= 1; select++)
 {
// MapIterator map_it = m_host_process_map[select].begin();
    //check if an host_name entry is present,
    // if not create a vector
 std::cerr << "MAP N:" << select + 1<< std::endl;
        for (MapIterator map_it = m_host_process_map[select].begin();
             map_it != m_host_process_map[select].end(); map_it++)
  {
    std::cerr << " key =" << map_it->first << std::endl;
   int size_vector = map_it->second->size();
   for (int i=0; i< size_vector; i++)
        {
             std::cerr << "      |____ value process name "<<
             (*map_it->second)[i].process_name<< "  kind_of_start " <<(*map_it->second)[i].kind_of_start << endl;
            }
        }
    }
}

void HostProcessInitSequencer::stop_host_processes(const std::string& host_name)
{
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "in stop_host_processes " << host_name);
    MapIterator map_it = m_host_process_map[0].find(host_name);
    int size_vector = map_it->second->size();
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "Numero di processi da stoppare: " << size_vector);
    for (int i=size_vector-1; i>= 0; i--)
    {
         CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "Stopping process: " << i);
         CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
            "Stopping process: " << (*map_it->second)[i].process_name << " on host: "
             << host_name);
         (*map_it->second)[i].process_impl->stop_and_wait(false);
    }
}

} // End namespace PlatformMngt
} // End namespace Cdmw

