/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include "testperiodicnotifier/FakeApplication.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

FakeApplication::FakeApplication()
{   

    //fake system elements
    m_entity = new FakeEntity();
    m_lb_group = new FakeLBGroup();
    m_ft_group = new FakeFTGroup();
    m_proc = new FakeProcess();

    m_attr_value = 2 ;
   
}


void FakeApplication::get_attribute(const char * attr_name, CORBA::Any_out ret_attr , CdmwPlatformMngt::Owner flag)  
throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
{ 
    TEST_INFO("FakeApplication::get_attribute"<<std::endl);
    
    if (!strcmp(attr_name, "ApplicationAttribute"))
    {
        TEST_INFO("FakeApplication::get_attribute - HostAttribute"<<std::endl);
        CORBA::Any_var tempany = new CORBA::Any;
        tempany.inout () <<= m_attr_value;
        ret_attr = tempany._retn ();
    }
    else if (!strcmp(attr_name, "AttributeNotFound"))
    {
        TEST_INFO("FakeApplication::get_attribute - AttributeNotFound"<<std::endl);
        throw CdmwPlatformMngt::AttributeNotFound();
    }
    else if (!strcmp(attr_name, "AttributeException"))
    {
        TEST_INFO("FakeApplication::get_attribute - AttributeException"<<std::endl);
        CDMW_THROW2 (Cdmw::Exception, Cdmw::Exception::UNKNOWN,
                     "FakeApplication::get_attribute - AttributeException");

    }
}

CdmwPlatformMngt::Entity_ptr FakeApplication::get_entity (const char* entity_name)
throw( CdmwPlatformMngt::EntityNotFound,CORBA::SystemException )
{
    TEST_INFO("FakeApplication::get_entity"<<std::endl);
    if (!strcmp(entity_name, "ENTNotFound"))
    {
        TEST_INFO("FakeApplication::get_entity - EntityNotFound"<<std::endl);
        throw CdmwPlatformMngt::EntityNotFound();
    }
    return m_entity;
} 
  

CdmwPlatformMngt::Process_ptr FakeApplication::get_process(const char* proc_name,const char* host_name)
throw (CdmwPlatformMngt::ProcessNotFound,CORBA::SystemException)
{
    TEST_INFO("FakeApplication::get_process"<<std::endl);
    if (!strcmp(proc_name, "PROCNotFound") || !strcmp(host_name, "HOSTNotFound"))
    {
        TEST_INFO("FakeApplication::get_process - ProcessNotFound"<<std::endl);
        throw CdmwPlatformMngt::ProcessNotFound();
    }
    return m_proc;
}

CdmwPlatformMngt::LBGroup_ptr FakeApplication::get_lb_group(const char* group_name)
throw (CdmwPlatformMngt::GroupNotFound,CORBA::SystemException)
{
    TEST_INFO("FakeApplication::get_lb_group"<<std::endl);
    if (!strcmp(group_name, "LBGRNotFound"))
    {
        TEST_INFO("FakeApplication::get_lb_group - LBGRNotFound"<<std::endl);
        throw CdmwPlatformMngt::GroupNotFound();
    }
    return m_lb_group;
}

CdmwPlatformMngt::FTGroup_ptr FakeApplication::get_ft_group(const char* group_name)
throw (CdmwPlatformMngt::GroupNotFound,CORBA::SystemException)
{
    TEST_INFO("FakeApplication::get_ft_group"<<std::endl);
    if (!strcmp(group_name, "FTGRNotFound"))
    {
        TEST_INFO("FakeApplication::get_ft_group - FTGRNotFound"<<std::endl);
        throw CdmwPlatformMngt::GroupNotFound();
    }
    return m_ft_group;
}

            
}
}









