/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include "testperiodicnotifier/FakeSystem.hpp"
#include "testperiodicnotifier/FakeHost.hpp"
#include "testperiodicnotifier/FakeEntity.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

FakeSystem::FakeSystem()
{ 
    //fake system elements
    m_appl = new FakeApplication();   
    m_entity = new FakeEntity();  
    m_host = new FakeHost();
    
    m_attr_value = 1 ;
}

FakeSystem::~FakeSystem()
{
    
}


void FakeSystem::get_attribute(const char * attr_name, CORBA::Any_out ret_attr , CdmwPlatformMngt::Owner flag)  
throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
{ 
    if (!strcmp(attr_name, "SystemAttribute"))
    {
        TEST_INFO("FakeSystem::get_attribute"<<std::endl);
        CORBA::Any_var tempany = new CORBA::Any;
        tempany.inout () <<= m_attr_value;
        ret_attr = tempany._retn ();
    }
    else if (!strcmp(attr_name, "AttributeNotFound"))
    {
        TEST_INFO("FakeSystem::get_attribute - AttributeNotFound"<<std::endl);
        throw CdmwPlatformMngt::AttributeNotFound();
    }
    else if (!strcmp(attr_name, "AttributeException"))
    {
        TEST_INFO("FakeSystem::get_attribute - AttributeException"<<std::endl);
        CDMW_THROW2 (Cdmw::Exception, Cdmw::Exception::UNKNOWN,
                     "FakeSystem::get_attribute - AttributeException");
    }
}

CdmwPlatformMngt::Entity_ptr FakeSystem::get_entity (const char* entity_name)
throw( CdmwPlatformMngt::EntityNotFound,CORBA::SystemException )
{
    TEST_INFO("FakeSystem::get_entity"<<std::endl);
    if (!strcmp(entity_name, "ENTNotFound"))
    {
    TEST_INFO("FakeSystem::get_entity - ENTNotFound"<<std::endl);
    throw CdmwPlatformMngt::EntityNotFound();
    }
    return m_entity;
} 
  

CdmwPlatformMngt::Host_ptr FakeSystem::get_host(const char* host_name)
throw (CdmwPlatformMngt::HostNotFound,CORBA::SystemException)
{
    TEST_INFO("FakeSystem::get_host"<<std::endl);
    if (!strcmp(host_name, "HOSTNotFound"))
    {
    TEST_INFO("FakeSystem::get_entity - HOSTNotFound"<<std::endl);
        throw CdmwPlatformMngt::HostNotFound();
    }
    return m_host;
}

CdmwPlatformMngt::Application_ptr FakeSystem::get_application (const char* application_name)
throw (CdmwPlatformMngt::ApplicationNotFound,CORBA::SystemException)
{
    TEST_INFO("FakeSystem::get_application"<<std::endl);
    if (!strcmp(application_name, "APPLNotFound"))
    {
    TEST_INFO("FakeSystem::get_application - APPLNotFound"<<std::endl);
        throw CdmwPlatformMngt::ApplicationNotFound();
    }
    return m_appl;
}

}
}









