/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

/* @author Fabrizio Lorenna <f.lorenna@vitrociset.it> */

#include <iostream>
#include <sstream>
#include <limits>

#include "test_components_and_groups_attributes/TestComponentsAndGroupsAttributes.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>
#include "tao/AnyTypeCode/TypeCode.h"


using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestComponentsAndGroupsAttributes);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestComponentsAndGroupsAttributes::TestComponentsAndGroupsAttributes ()
    {

    }


    TestComponentsAndGroupsAttributes::~TestComponentsAndGroupsAttributes ()
    {

    }
    bool TestComponentsAndGroupsAttributes::m_is_one_shot_initialized = false;

    CdmwPlatformMngt::System_var TestComponentsAndGroupsAttributes::mysystem;
    CdmwPlatformMngt::Process_var TestComponentsAndGroupsAttributes::myprocess;
    CdmwPlatformMngt::Application_var TestComponentsAndGroupsAttributes::app;
    CdmwPlatformMngt::CcmComponent_var TestComponentsAndGroupsAttributes::comp;

    void TestComponentsAndGroupsAttributes::setUp ()
    {

      TestComponentsAndGroupsAttributes::one_shot_init ();
    }

    void TestComponentsAndGroupsAttributes::one_shot_init ()
    {

      if (!m_is_one_shot_initialized)
        {
          m_is_one_shot_initialized = true;

          try
          {
            CORBA::ORB_var orb;
            orb =
              CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::
                                      instance ()->get_ORB ());

            PortableServer::POA_var poa =
              PortableServer::POA::_duplicate (Cdmw::TestUtils::
                                               CORBATestManager::instance ()->
                                               get_POA ());

            PortableServer::POAManager_var poaMgr =
              PortableServer::POAManager::_duplicate (poa->the_POAManager ());
            poaMgr->activate ();


            std::string fault_manager_corbaloc =
              "corbaloc::localhost:4555/fault_manager";

            CORBA::Object_var fault_manager_obj =
              orb->string_to_object (fault_manager_corbaloc.c_str ());
            if (CORBA::is_nil (fault_manager_obj.in ()))
              {
                CDMW_THROW2 (Cdmw::Exception, Cdmw::Exception::FATAL,
                             "Invalid fault manager corbaloc");
              }

            CdmwFT::FTManager_var fault_manager;
            try
            {
              fault_manager =
                CdmwFT::FTManager::_duplicate (::CdmwFT::FTManager::
                                               _narrow (fault_manager_obj.
                                                        in ()));
            }

            catch (const CORBA::SystemException & ex)
            {
              std::
                cerr << "Exception while contacting FTManager: " << ex << "\n"
                << fault_manager_corbaloc << " is probably invalid" << std::
                endl;
            }

            CdmwFT::ReplicationManager_var replication_manager =
              fault_manager->get_replication_manager ();

            // get from the replication manager the system accessor group
            // reference
            // from its group name
            ::FT::Name group_name;
            group_name.length (1);
            group_name[0].id =
            Cdmw::PlatformMngt::Configuration::
              Get_SystemAccessor_group_name ().c_str ();

            CORBA::Object_var system_accessor_group_obj =
              replication_manager->get_object_group_ref_from_name (group_name);

            CdmwFTSystMngt::SystemAccessor_var system_accessor =
            CdmwFTSystMngt::SystemAccessor::
              _narrow (system_accessor_group_obj.in ());


            Cdmw::PlatformMngt::Proxy::SystemProxy * theSystemProxyHandle =
              NULL;
            const char *proxyName = "SystemMngtProxy";


            /* create lookup service */
            auto_ptr < Cdmw::PlatformMngt::Proxy::AccessorLookupService >
              lookupService (new Cdmw::PlatformMngt::Proxy::
                             DefaultAccessorLookupService (system_accessor.
                                                           in ()));

            theSystemProxyHandle = Cdmw::PlatformMngt::Proxy::InitUtils::create_platformmngt_proxy (orb.in (), poa.in (), proxyName, lookupService.get ());     /* proxy takes over responsibility for lookup service */
            lookupService.release ();


            mysystem = theSystemProxyHandle->getReference ();


            if (CORBA::is_nil (mysystem.in ()))
              {
                exit (-1);
              }

            /*from System, use a method to retrieve an application */
            app = mysystem->get_application ("HELLO_WORLD");

            /*from the application, use a method to retrieve a Process */

            /*retrieve the current hostname */
            char *hostname = getenv ("HOSTNAME");

            myprocess = app->get_process ("P12", hostname);

          }
          catch (const Cdmw::Exception & ex)
          {
            std::cerr << ex.what () << std::endl;
            exit (-1);
          }
          catch (const CORBA::Exception & ex)
          {
            std::cerr << ex << std::endl;
            exit (-1);
          }
          catch (...)
          {
            std::cerr << "unexpected exception" << std::endl;
            exit (-1);
          }
        }
    }



    void TestComponentsAndGroupsAttributes::get_component_properties()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::CcmComponent_var comp2 = 
              myprocess->get_component("FABCOMPONENT2");


      //get user properties ...
      CORBA::Any_var prop_value1;
      comp2->get_attribute("COMP_PROP_SHORT", prop_value1,
         CdmwPlatformMngt::USER_DEFINED );

      CORBA::Any_var prop_value2;
      comp2->get_attribute("COMP_PROP_FLOAT", prop_value2,
         CdmwPlatformMngt::USER_DEFINED );

      CORBA::Any_var prop_value3;
      comp2->get_attribute("COMP_PROP_STR", prop_value3,
         CdmwPlatformMngt::USER_DEFINED );

      //get pre-defined properties ...
      CORBA::Any_var prop_value4;
      comp2->get_attribute("ReplicationRole", prop_value4,
         CdmwPlatformMngt::PRE_DEFINED );

      CORBA::Any_var prop_value5;
      comp2->get_attribute("DefaultRestartWarmth", prop_value5,
         CdmwPlatformMngt::PRE_DEFINED );

      CORBA::Any_var prop_value6;
      comp2->get_attribute("RequestedRestartWarmth", prop_value6,
         CdmwPlatformMngt::PRE_DEFINED );

      //see number of properties ...

      CORBA::ULong num_props = comp2->get_number_of_properties();

      CORBA::TypeCode_var tc1 = prop_value1->type();
      CORBA::TypeCode_var tc2 = prop_value2->type();
      CORBA::TypeCode_var tc3 = prop_value3->type();

      CORBA::TypeCode_var tc4 = prop_value4->type();
      CORBA::TypeCode_var tc5 = prop_value5->type();
      CORBA::TypeCode_var tc6 = prop_value6->type();


      CORBA::Short value1;
      (*prop_value1) >>= value1 ;

      CORBA::Float value2;
      (*prop_value2) >>= value2 ;


      const char* value3_tmp;
      prop_value3 >>= value3_tmp;


      std::string value3( value3_tmp ) ;

      CdmwPlatformMngt::RedundancyEltState value4;
      (*prop_value4) >>= value4;

      CdmwPlatformMngt::RestartWarmth value5;
      (*prop_value5) >>= value5;

      CdmwPlatformMngt::RestartWarmth value6;
      (*prop_value6) >>= value6;


      if ( (num_props == 6) && 
       (tc1->kind() == CORBA::tk_short) &&
       (tc2->kind() == CORBA::tk_float) &&
       (tc3->kind() == CORBA::tk_string) &&
       (tc4->equal(CdmwPlatformMngt::_tc_RedundancyEltState)) &&
       (tc5->equal(CdmwPlatformMngt::_tc_RestartWarmth)) &&
       (tc6->equal(CdmwPlatformMngt::_tc_RestartWarmth)) &&
       (value1 == 5) &&
       (value2 == ( (CORBA::Float) 2.4) ) &&
       (value3.compare("MYSTRING") == 0)  &&
       (value4 == CdmwPlatformMngt::REDSTATE_NORG) &&
       (value5 == CdmwPlatformMngt::HOT_RESTART) &&
       (value6 == CdmwPlatformMngt::HOT_RESTART) )
        {
    //ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 



    void TestComponentsAndGroupsAttributes::set_and_get_component_properties()
    {
    CORBA::String_var info ;

      try
      {

      CdmwPlatformMngt::CcmComponent_var comp2 = 
              myprocess->get_component("FABCOMPONENT2");

      CORBA::Any_var prop_value2 = new CORBA::Any() ;
      CORBA::Any_var prop_value3 = new CORBA::Any() ;
      CORBA::Any_var prop_value4 = new CORBA::Any() ;


      CdmwPlatformMngt::RestartWarmth value2;
      value2 = CdmwPlatformMngt::COLD_RESTART ;
      (*prop_value2) <<= value2;
      comp2->set_attribute("DefaultRestartWarmth", (*prop_value2),
         CdmwPlatformMngt::PRE_DEFINED ); 


      CORBA::Short value4 = 32 ;
      (*prop_value4) <<= value4;
      comp2->set_attribute("COMP_PROP_SHORT", (*prop_value4),
         CdmwPlatformMngt::USER_DEFINED ); 


      //get the values ...


      CORBA::Any_var prop_value6;
      comp2->get_attribute("DefaultRestartWarmth", prop_value6,
         CdmwPlatformMngt::PRE_DEFINED );
      CdmwPlatformMngt::RestartWarmth value6;
      (*prop_value6) >>= value6;


      CORBA::Any_var prop_value8;
      comp2->get_attribute("COMP_PROP_SHORT", prop_value8,
         CdmwPlatformMngt::USER_DEFINED );
      CORBA::Short value8;
      (*prop_value8) >>= value8 ;

      CORBA::TypeCode_var tc6 = prop_value6->type();
      CORBA::TypeCode_var tc8 = prop_value8->type();

      //see number of properties ...

      CORBA::ULong num_props = comp2->get_number_of_properties();

      if ( (num_props == 6) && 
       (tc6->equal(CdmwPlatformMngt::_tc_RestartWarmth) ) &&
       (tc8->kind() == CORBA::tk_short) &&
       (value6 == CdmwPlatformMngt::COLD_RESTART ) &&
       (value8 == 32 )  )
        {
    //ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 



    void TestComponentsAndGroupsAttributes::get_ftgroup_properties()
    {
      CdmwPlatformMngt::FTGroup_var ftg ;

      try
        {
         ftg = app->get_ft_group("FABFTGROUP1");

         //get the properties ...
     CORBA::Any_var prop_value1;
         ftg->get_attribute("FTG_PROP_SHORT", prop_value1,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value2;
         ftg->get_attribute("FTG_PROP_FLOAT", prop_value2,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value3;
         ftg->get_attribute("FTG_PROP_STR", prop_value3,
          CdmwPlatformMngt::USER_DEFINED );
 
         CORBA::Any_var prop_value4;
         ftg->get_attribute("GroupSize", prop_value4,
          CdmwPlatformMngt::PRE_DEFINED );

         //get number of properties ...
         CORBA::ULong num_props = ftg->get_number_of_properties();

         CORBA::TypeCode_var tc1 = prop_value1->type();
         CORBA::TypeCode_var tc2 = prop_value2->type();
         CORBA::TypeCode_var tc3 = prop_value3->type();
         CORBA::TypeCode_var tc4 = prop_value4->type();

         CORBA::Short value1; 
         (*prop_value1) >>= value1 ;

         CORBA::Float value2; 
         (*prop_value2) >>= value2 ;

         const char* value3_tmp;
         prop_value3 >>= value3_tmp;
         std::string value3( value3_tmp ) ;


         CORBA::Long value4; 
         (*prop_value4) >>= value4 ;


     if ( (num_props = 4) &&
              ( value1 == 10) &&
              (value2 == ((CORBA::Float) 11.1) ) &&
              ( value3.compare("MYSTRING_FT") == 0) &&
              (value4 == 0) &&
              (tc1->kind() == CORBA::tk_short) &&
              (tc2->kind() == CORBA::tk_float) &&
              (tc3->kind() == CORBA::tk_string) &&
              (tc4->kind() == CORBA::tk_long ))
                  CPPUNIT_ASSERT(1); //ok
         else
                  CPPUNIT_ASSERT(0); //not ok


        }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }


    }



    void TestComponentsAndGroupsAttributes::set_and_get_ftgroup_properties()
    {
    CORBA::String_var info ;

    CdmwPlatformMngt::FTGroup_var ftg ;
    CORBA::Any_var prop_value1 = new CORBA::Any() ;
    CORBA::Any_var prop_value2 = new CORBA::Any() ;
    CORBA::Any_var prop_value1_get;
    CORBA::Short newvalue1; 

    try
     {
      ftg = app->get_ft_group("FABFTGROUP1");



      CORBA::Short value1 = 23 ;
      (*prop_value1) <<= value1 ;
      ftg->set_attribute("FTG_PROP_SHORT", (*prop_value1),
         CdmwPlatformMngt::USER_DEFINED );


      ftg->get_attribute("FTG_PROP_SHORT", prop_value1_get,
         CdmwPlatformMngt::USER_DEFINED );

      (*prop_value1_get) >>= newvalue1 ;
      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }


      try
      {
      CORBA::TypeCode_var tc1_get = prop_value1_get->type();

      //see number of properties ...

      CORBA::ULong num_props = ftg->get_number_of_properties();

      if ( (num_props == 4) && 
       (tc1_get->kind() == CORBA::tk_short) &&
           ( newvalue1 == 23 ) ) 
        {
    //test ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 



    void TestComponentsAndGroupsAttributes::get_lbgroup_properties()
    {
      CdmwPlatformMngt::LBGroup_var lbg ;

      try
        {
         lbg = app->get_lb_group("FABLBGROUP3");

         //get the properties ...
         CORBA::Any_var prop_value1;
         lbg->get_attribute("LBG_PROP_SHORT", prop_value1,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value2;
         lbg->get_attribute("LBG_PROP_FLOAT", prop_value2,
          CdmwPlatformMngt::USER_DEFINED );

         CORBA::Any_var prop_value3;
         lbg->get_attribute("LBG_PROP_STR", prop_value3,
          CdmwPlatformMngt::USER_DEFINED );
 
         CORBA::Any_var prop_value4;
         lbg->get_attribute("GroupSize", prop_value4,
          CdmwPlatformMngt::PRE_DEFINED );

         //get number of properties ...
         CORBA::ULong num_props = lbg->get_number_of_properties();

         CORBA::TypeCode_var tc1 = prop_value1->type();
         CORBA::TypeCode_var tc2 = prop_value2->type();
         CORBA::TypeCode_var tc3 = prop_value3->type();
         CORBA::TypeCode_var tc4 = prop_value4->type();

         CORBA::Short value1; 
         (*prop_value1) >>= value1 ;

         CORBA::Float value2; 
         (*prop_value2) >>= value2 ;

         const char* value3_tmp;
         prop_value3 >>= value3_tmp;
         std::string value3( value3_tmp ) ;


         CORBA::Long value4; 
         (*prop_value4) >>= value4 ;


     if ( (num_props = 4) &&
              ( value1 == 13) &&
              (value2 == ((CORBA::Float) 14.4) ) &&
              ( value3.compare("MYSTRING_LB") == 0) &&
              (value4 == 0) &&
              (tc1->kind() == CORBA::tk_short) &&
              (tc2->kind() == CORBA::tk_float) &&
              (tc3->kind() == CORBA::tk_string) &&
              (tc4->kind() == CORBA::tk_long ))
                  CPPUNIT_ASSERT(1); //ok
         else
                  CPPUNIT_ASSERT(0); //not ok


        }
      catch(...)
        {
        //error
        CPPUNIT_ASSERT(0);
    }


    }


    void TestComponentsAndGroupsAttributes::set_and_get_lbgroup_properties()
    {
    CORBA::String_var info ;

    CdmwPlatformMngt::LBGroup_var lbg ;
    CORBA::Any_var prop_value1 = new CORBA::Any() ;
    CORBA::Any_var prop_value1_get;
    CORBA::Short newvalue1;

    try
     {
      lbg = app->get_lb_group("FABLBGROUP3");


      CORBA::Short value1 = 23 ;
      (*prop_value1) <<= value1 ;
      lbg->set_attribute("LBG_PROP_SHORT", (*prop_value1),
         CdmwPlatformMngt::USER_DEFINED );


      lbg->get_attribute("LBG_PROP_SHORT", prop_value1_get,
         CdmwPlatformMngt::USER_DEFINED );

      (*prop_value1_get) >>= newvalue1 ;
      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }


      try
      {
      CORBA::TypeCode_var tc1_get = prop_value1_get->type();

      //see number of properties ...

      CORBA::ULong num_props = lbg->get_number_of_properties();

      if ( (num_props == 4) && 
       (tc1_get->kind() == CORBA::tk_short) &&
           ( newvalue1 == 23) ) 
        {
    //test ok
        CPPUNIT_ASSERT (1);
    }
      else
        CPPUNIT_ASSERT (0); //test not ok 

      }
      catch (...)
      {
    //error
        CPPUNIT_ASSERT (0);
      }

    } 

  }                             /* End namespace PlatformMngt */
}                               /* End namespace Cdmw */
