/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2007. All rights reserved.
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */
 
#include "clockservice/FTExecutorActivationHandler.hpp"

Cdmw::clock::FTExecutorActivationHandler::FTExecutorActivationHandler(const std::string&            name,
                                                                      Cdmw::clock::FTExecutor_impl* executor_impl)
    : m_name(name),
      m_is_active(false),
      m_executor_impl(executor_impl)
{
    m_executor_impl->_add_ref();
}

Cdmw::clock::FTExecutorActivationHandler::~FTExecutorActivationHandler()
{
    m_executor_impl->_remove_ref();
}

void
Cdmw::clock::FTExecutorActivationHandler::activate()
    throw (CORBA::SystemException)
{
    m_is_active = true;
    m_executor_impl->activate();
}

void
Cdmw::clock::FTExecutorActivationHandler::deactivate()
    throw (CORBA::SystemException)
{
    m_is_active = false;
}

bool
Cdmw::clock::FTExecutorActivationHandler::is_active()
{
    return m_is_active;
}

Cdmw::clock::FTExecutor_impl*
Cdmw::clock::FTExecutorActivationHandler::getServant()
{
    return m_executor_impl;
}
