/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2007. All rights reserved.
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */

#include "clockservice/FTExecutor_impl.hpp"
#include "clockservice/StateTransferConfig.hpp"
#include "clockservice/FTController_impl.hpp"

Cdmw::clock::FTExecutor_impl::FTExecutor_impl(ACE_Reactor* reactor)
    : Cdmw::clock::Executor_impl(reactor),
      m_controller_id(0),
      m_controller_data_storage_home(0)
{
    try
    {
        m_controller_data_storage_home = new Cdmw::clock::ControllerDataStorageHome(CONTROLLER_DATA_IDENTIFIER);
    }
    catch (const CORBA::SystemException & e)
    {
        std::cerr << "FTExecutor_impl::FTExecutor_impl: ERROR: CORBA::SystemException exception raised: " << e << std::endl;
        throw;
    }
}

Cdmw::clock::FTExecutor_impl::~FTExecutor_impl()
{
    delete m_controller_data_storage_home;
}

::CosClockService::PeriodicExecution::Controller_ptr
Cdmw::clock::FTExecutor_impl::enable_periodic_execution(CosClockService::PeriodicExecution::Periodic_ptr on)
    throw (CORBA::SystemException)
{
    if (CORBA::is_nil(on))
    {
        std::cerr << "FTExecutor_impl::enable_periodic_execution: invalid periodic" << std::endl;
        throw CORBA::BAD_PARAM();
    }

    // Create FT controller status data into data storage.
    try
    {
        std::cout << "FTExecutor_impl::enable_periodic_execution: creating FT controller data into data storage" << std::endl;
        Cdmw::clock::ControllerData data;
        ControllerDataStorageObject ds_obj = m_controller_data_storage_home->create(m_controller_id, data);
    }
    catch (...)
    {
        std::cout << "FTExecutor_impl::enable_periodic_execution: unable to perform FT controller status transfer" << std::endl;
    }

    // Create new FT controller instance.
    std::cout << "FTExecutor_impl::enable_periodic_execution: creating a new FT controller" << std::endl;
    Cdmw::clock::FTController_impl* ctrl = new Cdmw::clock::FTController_impl(m_controller_id,
                                                                              *m_controller_data_storage_home,
                                                                              *this,
                                                                              on,
                                                                              this->get_reactor_lock_free());
    m_controller_id++;

    // Return a reference to the newly created FT controller.
    CosClockService::PeriodicExecution::Controller_var ctrl_var = ctrl->_this();
    return ctrl_var._retn();
}

void
Cdmw::clock::FTExecutor_impl::activate()
{
    std::cout << "############################################################" << std::endl;
    std::cout << ">>>>>>>>>>>>>>>>>> Activating FTExecutor Replica"             << std::endl;

    Cdmw::clock::FTController_impl* ctrl = 0;
    try
    {
        for (;; m_controller_id++)
        {
            //Retrieve storage object data from data storage home related to a specified object id
            Cdmw::clock::ControllerData_var data_var;
            Cdmw::clock::FTController_impl::retrieve_status_lock_free(m_controller_id,
                                                                      *m_controller_data_storage_home,
                                                                      data_var);

            // Create new replica of FT controller.
            ctrl = new Cdmw::clock::FTController_impl(m_controller_id,
                                                      *m_controller_data_storage_home,
                                                      *this,
                                                      CosClockService::PeriodicExecution::Periodic::_nil(),
                                                      this->get_reactor_lock_free());

            // Activate new replica of FT controller.
            ctrl->restore_object_lock_free(data_var.in());
        }
    }
    catch (const Cdmw::FT::NotFoundException& e)
    {
    }

    std::cout << ">>>>>>>>>>>>>>>>>> Completed FTExecutor Replica Activation"   << std::endl;
    std::cout << "############################################################" << std::endl;
}

