/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2007. All rights reserved.
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */

#ifndef CARDAMOM_FT_EXECUTOR_IMPL_HPP
#define CARDAMOM_FT_EXECUTOR_IMPL_HPP
/// @cond JUMP_FILE
// -- Cdmw Time Includes --
#include <Time/clockservice/ClockStateDataStore.hpp>
#include "clockservice/Clock_impl.hpp"
#include "clockservice/Executor_impl.hpp"

extern "C++"
{
    namespace Cdmw
    {
        namespace clock
        {
            class FTExecutorActivationHandler;
        } /* clock */
    } /* Cdmw */
} /* extern "C++" */

namespace Cdmw
{
    namespace clock
    {
        class FTExecutor_impl;
    } /* clock */
} /* Cdmw */

/**
 * @brief This class provides a fault-tolerant implementation of the
 * <code>Clock</code> and <code>Executor</code> interface.
 *
 * @author Angelo Corsaro <acorsaro@selex-si.com>
 * @author Marco Macellari <mmacellari@progesi.it>
 */
class Cdmw::clock::FTExecutor_impl : public Cdmw::clock::Executor_impl  
{
    public:

        // Friend class declaration.
        friend class ::Cdmw::clock::FTExecutorActivationHandler;

    public:

        /**
         * @brief Create an <code>FTExecutor_impl</code>.
         *
         * @param reactor the reactor that will be used by this instance.
         */
        FTExecutor_impl(ACE_Reactor* reactor);

        /**
         * Destructor.
         */
        virtual ~FTExecutor_impl();

    public:

        // -- CORBA methods from the Executor Interface --

        /**
         * @brief Register an instance of the Periodic interface for periodic execution.
         * 
         * @return the <code>Controller</code> to be used to control 
         *         the execution of the <code>Periodic</code> instance.
         */
        virtual ::CosClockService::PeriodicExecution::Controller_ptr
            enable_periodic_execution(CosClockService::PeriodicExecution::Periodic_ptr on)
            throw (CORBA::SystemException);

    protected:

        /**
         * @brief This method is called by the framework to activate the clock.
         */ 
        virtual void activate();

    private:

        FTExecutor_impl(const FTExecutor_impl&);
        FTExecutor_impl& operator=(const FTExecutor_impl&);

    private:

        Cdmw::clock::Ident                      m_controller_id;
        Cdmw::clock::ControllerDataStorageHome* m_controller_data_storage_home; 
};
/// @endcond

#endif /* CARDAMOM_FT_EXECUTOR_IMPL_HPP */

