#!/bin/bash
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

# handle input parameters
SCRIPT=`basename $0`
if [[ $# -ne 1 ]] ; then
    echo "usage: $SCRIPT <test program name>"
    echo "where: <test program name> is the program name of the test to be run"
    echo "notes: required local environment variables are:"
    echo "       - TESTS_REPORTS_SCRIPTS_ROOT pointing to the local absolute path for test reports logs."
    echo "       - TESTS_REPORTS_LOGS_ROOT    pointing to the local absolute path for test reports logs."
    echo "       - CDMW_HOME                  pointing to the local cardamom installation root."
    echo "       - VALGRIND_2_4_1_BIN_ROOT    pointing to the local bin directory of valgrind-2.4.1 installation."
    exit 1
fi

# include global macros script
source $TESTS_REPORTS_SCRIPTS_ROOT/tests_reports-global_macros.bash

# run the test and handle 
EXIT_CODE=0
$VALGRIND_2_4_1_BIN_ROOT/valgrind --tool=memcheck --leak-check=full --trace-children=yes --log-file=$TESTS_REPORTS_LOGS_ROOT/valgrind-2.4.1.memo-check.$1.single-process.log $CDMW_HOME/bin/$1 2>&1 | tee $TESTS_REPORTS_LOGS_ROOT/valgrind-2.4.1.memo-check.$1.single-process.out
if [[ $? -ne 0 ]] ; then EXIT_CODE=1 ; fi
mv $TESTS_REPORTS_LOGS_ROOT/valgrind-2.4.1.memo-check.$1.single-process.log.* $TESTS_REPORTS_LOGS_ROOT/valgrind-2.4.1.memo-check.$1.single-process.log 2>&1 > /dev/null
if [[ $? -ne 0 ]] ; then EXIT_CODE=1 ; fi
exit $EXIT_CODE

