#!/bin/bash
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

# handle input parameters
SCRIPT=`basename $0`
if [[ $# -ne 1 ]] ; then
    echo "usage: $SCRIPT <test program name>"
    echo "where: <test program name> is the program name of the test to be run"
    echo "notes: required local environment variables are:"
    echo "       - TESTS_REPORTS_SCRIPTS_ROOT pointing to the local absolute path for test reports logs."
    echo "       - TESTS_REPORTS_LOGS_ROOT    pointing to the local absolute path for test reports logs."
    echo "       - CDMW_HOME                  pointing to the local cardamom installation root."
    echo "       - VALGRIND_3_1_0_BIN_ROOT    pointing to the local bin directory of valgrind-3.1.0 installation."
    echo "       - TMG_SERVER                 (from tests_reports-global_macros.bash) containing the name of the server host."
    echo "       - TMG_PORT                   (from tests_reports-global_macros.bash) containing the communication port."
    exit 1
fi

# include global macros script
source $TESTS_REPORTS_SCRIPTS_ROOT/tests_reports-global_macros.bash

# run the test and handle log files
EXIT_CODE=0
$VALGRIND_3_1_0_BIN_ROOT/valgrind --tool=memcheck --leak-check=full --trace-children=yes --log-file=$TESTS_REPORTS_LOGS_ROOT/valgrind-3.1.0.memo-check.$1.client-process.log $CDMW_HOME/bin/$1 --client -ORBInitRef ClockService=corbaloc::$TMG_SERVER:$TMG_PORT/ClockService -ORBInitRef LocalClock=corbaloc::$TMG_SERVER:$TMG_PORT/LocalClock -ORBInitRef PolicyControllerUpdateAdmin=corbaloc::$TMG_SERVER:$TMG_PORT/PolicyControllerUpdateAdmin 2>&1 | tee $TESTS_REPORTS_LOGS_ROOT/valgrind-3.1.0.memo-check.$1.client-process.out
if [[ $? -ne 0 ]] ; then EXIT_CODE=1 ; fi
mv $TESTS_REPORTS_LOGS_ROOT/valgrind-3.1.0.memo-check.$1.client-process.log.* $TESTS_REPORTS_LOGS_ROOT/valgrind-3.1.0.memo-check.$1.client-process.log 2>&1 > /dev/null
if [[ $? -ne 0 ]] ; then EXIT_CODE=1 ; fi
exit $EXIT_CODE

