/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2007. All rights reserved.
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */
   
package cdmw.clock.test;
import cdmw.testutils.Testable;
import cdmw.ossupport.Timeval;
import cdmw.clock.*;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.TimeZone;


public class ImportExportTimevalTest extends Testable
{
    private final int       NUMOFTESTS = 6;


    private final String dateSample    = "Fri Sep 08 18:07:41 CEST 2006";
    private final long dateInTimeT     = 133770244610000000L;
    public ImportExportTimevalTest()
    {
      this.name = "ImportExportTimevalTest";
      this.out = System.out;
    }

    public boolean checkTimevalToTimeT(Timeval dateInTimeval) 
    {

         long utcTime = Time.timeval2TimeT(dateInTimeval);
         return ( (long) utcTime == dateInTimeT ) ;
    }
    
    public boolean checkTimeTtoTimeval(Timeval dateInTimeval) 
    {
         Timeval tv = Time.TimeT2timeval(dateInTimeT); 
         return (Util.compare(tv,dateInTimeval) == 0);
    }

    public boolean PCR_1224() 
    {
        cdmw.clock.cdmwTime res = new cdmw.clock.cdmwTime();
        // Java Epoch
        res.tm_year = 70;
        res.tm_mon = 0;
        res.tm_mday = 1;
        res.tm_hour = 0;
        res.tm_min = 0;
        res.tm_sec = 0;
        res.tm_msec = 0;

        cdmw.ossupport.Timeval res2 = cdmw.clock.Util.cdmwTime2timeval(res);
        String date1 = res2.toDate().toString();
        long timevalue = cdmw.clock.Time.timeval2TimeT(res2);
        cdmw.ossupport.Timeval value3 = cdmw.clock.Time.TimeT2timeval(timevalue);
        String date2 = value3.toDate().toString();
        if (!date1.equalsIgnoreCase(date2)) 
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }
    
    public boolean PCR_1223(){
        cdmw.clock.cdmwTime res = new cdmw.clock.cdmwTime();
        res.tm_year = 90;
        res.tm_mon = 0;
        res.tm_mday = 1;
        res.tm_hour = 15;
        res.tm_min = 30;
        res.tm_sec = 0;
        res.tm_msec = 0;
        cdmw.ossupport.Timeval res2 = cdmw.clock.Util.cdmwTime2timeval(res);
        String date1 = res2.toDate().toString();
        res = cdmw.clock.Util.timeval2cdmwTime(res2);
        res2 = cdmw.clock.Util.cdmwTime2timeval(res);
        String date2 = res2.toDate().toString();
        if (!date1.equalsIgnoreCase(date2)) 
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
        
    }

    public boolean createUtcT()
    {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        int offset =  cal.get( java.util.Calendar.ZONE_OFFSET );   // zone difference
            
              
        long inaccuracy = 10000L;
        short tdf = ( short ) ( offset / 60000 );  // millis to minutes
        short inacchi = (short) ((inaccuracy >> 32) &  0xffffL);
        int inacclo = (int) (inaccuracy & 0xffffffffL);

        org.omg.TimeBase.UtcT utct = Time.createUtcT(dateInTimeT);
        if ( (utct.time == dateInTimeT) &&
             (utct.inacclo == inacclo) &&
             (utct.inacchi == inacchi) &&
             (utct.tdf == tdf) )
            return Macro.TEST_OK;
        return Macro.TEST_NOK;
    }

    public boolean toStringTest()
    {
            final String dateSampleInGMT = "2006-09-08 16:07:41.00+0000";
        
            org.omg.TimeBase.UtcT utct = Time.createUtcT(dateInTimeT);
            SimpleDateFormat df = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss.SSZ");
            df.setTimeZone(TimeZone.getTimeZone("GMT")); // refers to GMT
            String dateInTimeT_GMT 
                = df.format( new java.util.Date(Time.TimeT2timeval(utct.time).getMilliseconds()) );
           
            if (dateInTimeT_GMT.compareTo(dateSampleInGMT) == 0)
                return Macro.TEST_OK;
            return  Macro.TEST_NOK;
    }
    

    public void doTests()
    {
       setNbOfRequestedTestOK(NUMOFTESTS);
      try 
      {
        SimpleDateFormat df = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy");

        // Fri Sep 08 18:07:41 CEST 2006
        // TimeT = 133770244610300000L
        Date d = df.parse(dateSample) ;

        Timeval dateInTimeval = new Timeval(d.getTime());
        check(checkTimevalToTimeT (dateInTimeval));
        check(checkTimeTtoTimeval (dateInTimeval));
        check(createUtcT());
        check(toStringTest());
        check(PCR_1224());
        check(PCR_1223());
      }
      catch (ParseException pe)
      {
      }

    }

}
