/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#ifndef INCL_TRACE_XMLTRACEPRINTER_HPP
#define INCL_TRACE_XMLTRACEPRINTER_HPP

#include <iostream>
#include "tracecollector/TracePrinter.hpp"

namespace Cdmw
{
/**
 * Namespace for CDMW Trace types and data.
 */
   namespace Trace
   {
      /**
      *Purpose:
      *<p>    This class, whose interface is defined in TracePrinter,
      *       is used to print, in Log4j XML format, the messages 
      *       comming from any of the Trace Producers.
      *
      */
      class XMLTracePrinter : public TracePrinter
      {
            // Log4j XML Format
            static const std::string XML_END_OF_TRACE_FILE;
            static const std::string XML_TRACE_HEADER1;
            static const std::string XML_TRACE_HEADER2;
            static const std::string XML_TRACE_HEADER3;
            static const std::string XML_TRACE_ERR_MESG1;
            static const std::string XML_TRACE_ERR_MESG2;
      
         public:
            /**
            * Purpose:
            * <p> Default public Constructor
            *@param time_mode Can be either Cdmw::OsSupport::OS::LOCAL_TIME or
            *                 Cdmw::OsSupport::OS::GMT_TIME
            *@exception
            */
            XMLTracePrinter(OsSupport::OS::TimeMode time_mode) throw(): TracePrinter(time_mode) {}
            /**
            * Purpose:
            * <p> Destructor
            * 
            *@param
            *@exception
            */
            virtual ~XMLTracePrinter() throw() {}
            /**
            * Purpose:
            * <p> prints the body of the message, in Log4j XML format
            * 
            *@param os std::ostream to write to
            *@param Data Data containing the information to read from,
            *            like the message's header, the timestamp, etc.
            *@exception
            */
            virtual void print_message(std::ostream & os, const TraceCircularBuffer::TraceMessageRaw & Data) const throw();
            /**
            * Purpose:
            * <p> Prints an error message
            * 
            *@param os std::ostream to write to
            *@param errMesg Error message to write
            *@exception
            */
            virtual void print_error_message (std::ostream & os, const std::string& errMesg) const throw();
            /**
            * Purpose:
            * <p> Prints the header of the message in the circular file referenced by file_number
            * 
            *@param os std::ostream to write to
            *@param file_number Index of the circular file to write to
            *@exception
            */            
            virtual void print_header (std::ostream & os, int file_number) const throw();
            /**
            * Purpose:
            * <p> Prints the end of the trace
            * 
            *@param os std::ostream to write to
            *@exception
            */
            virtual void print_end_of_trace (std::ostream & os) const throw();
      };
    
   }
}
#endif /*INCL_TRACE_XMLTRACEPRINTER_HPP*/
