/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include <cstdlib>
#include <sstream>
#include <iostream>
#include <fstream>
#include <Foundation/common/Locations.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "testlogcollector/TestLogCollector.hpp"
#include "TraceAndPerf/tracecollector/CdmwTraceLoggingCollector.stub.hpp"
#include <Foundation/ossupport/OS.hpp>
#include "testlogcollector/SimulatedGlobalTraceCollector.stub.hpp"
#include <Foundation/commonsvcs/naming/NamingUtil.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Foundation/common/Locations.hpp>

namespace
{
  const std::string DEF_REPOSITORY_PORT         = "5021";
  const std::string DEF_NAME_SERVICE_PORT       = "5011";
  const std::string GLOBAL_COLLECTOR_NAME       = "global_collector_1";
}

namespace Cdmw
{
namespace Trace
{

/**
*Implementation notes:
*<p>   none
*Portability issues: 
*<p>   none
*/

CPPUNIT_TEST_SUITE_REGISTRATION( TestLogCollector );

TestLogCollector::TestLogCollector()
    throw()
            
{
       m_system_port = Cdmw::TestUtils::CORBATestManager::instance()->get_property("system_port");
       std::string process_id = Cdmw::TestUtils::CORBATestManager::instance()->get_property("local_trace_collector_id");
       std::istringstream in_stream(process_id);
       in_stream >> m_local_trace_collector_id;
       

}


TestLogCollector::~TestLogCollector()
    throw()
{
    // does nothing
}

TestLogCollector&
TestLogCollector::operator=(const TestLogCollector& rhs)
    throw()
{
    // does nothing
    return *this;
}


void TestLogCollector::test_log_collector()
{
    TEST_INFO("Test Log Collector library");

    int timescale = Cdmw::TestUtils::get_timescale();

    std::string namesrv_corbaloc = "corbaloc:iiop:localhost:" +
        m_system_port + "/NameService";
    
    CORBA::ORB_var orb =
      Cdmw::TestUtils::CORBATestManager::instance()->get_ORB();
    CORBA::Object_var obj = orb->string_to_object(namesrv_corbaloc.c_str()) ;
    CosNaming::NamingContext_var nc = 
      CosNaming::NamingContext::_narrow(obj.in());
    Cdmw::CommonSvcs::Naming::NamingInterface ni (nc.in());

    CORBA::Object_var log_collector_obj = 
      ni.resolve(Cdmw::Common::Locations::DEFAULT_LOG_COLLECTOR_ID);

    CdmwLogging::LogCollector_var log_collector =
      CdmwLogging::LogCollector::_narrow(log_collector_obj.in());

    TEST_INFO("Log collector resolved");

    CdmwLogging::StructuredLog log;

    log.ctx.process_id = 1;
    log.ctx.application_name = "Application";
    log.ctx.process_name = "Process";
    log.ctx.ndc = "NDC";
    log.header.thread_id = 2;
    log.ctx.host_name = "Hostname";
    log.header.component_name = "ComponentName";
    log.header.user_domain = "Domain";
    log.header.level = 3;
    log.header.file_line = 4;
    log.header.file_name = "File";
    log.header.time_stamping.seconds = 5;
    log.header.time_stamping. microseconds = 6;     
    log.message = "Message";

    // Status of this unit test
    CORBA::Boolean status = false;

    log_collector->add_log(log);
    
    TEST_INFO("After add log");

     //
    // Recover the reference of SIM Collector to know its status
    //
    std::string sim_repository_corbaloc= "corbaloc:iiop:localhost:" +
      DEF_REPOSITORY_PORT + "/SimRepository";
    
    obj = orb->string_to_object(sim_repository_corbaloc.c_str()) ;
    CdmwNamingAndRepository::Repository_var repository_ref = CdmwNamingAndRepository::Repository::_narrow(obj.in());

    TEST_INFO("Sim Repository resolved");

    CosNaming::NamingContext_var nc_root_context =
      repository_ref->resolve_root_context(
                                           CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
    using namespace Cdmw::CommonSvcs::Naming;
    // naming interface of default root context
    NamingInterface ni_root(nc_root_context.in());
    
    CosNaming::NamingContext_var nc_collector_dom =
      CosNaming::NamingContext::_nil();
    
    std::string collector_domain_str(Cdmw::Common::Locations::CDMW_SERVICES_NAME_DOMAIN);
    collector_domain_str += "/";
    collector_domain_str += Cdmw::Common::Locations::TRACE_COLLECTORS_NAME_DOMAIN;
    
    // get the global collector domain name.
    typedef NamingUtil<CosNaming::NamingContext> Util;
    nc_collector_dom = Util::resolve_name(ni_root,
                                          collector_domain_str);

    // naming interface of global collector domain
    NamingInterface ni_collector(nc_collector_dom.in());
    CORBA::Object_var global_collector_obj = ni_collector.resolve(GLOBAL_COLLECTOR_NAME.c_str());

    CdmwTrace::SimulatedGlobalTraceCollector_var sim_collector = CdmwTrace::SimulatedGlobalTraceCollector::_narrow(global_collector_obj.in());

    TEST_INFO("Simulated Global Collector resolved");

   // Wait that SIM Collector recovers the log
    Cdmw::OsSupport::OS::sleep(timescale * 10000);

    // Get status
    status = sim_collector->get_status();

    // Test the hardiness of the Log Collector
    // All the add_log must be successful and no exception is
    // expected.
//     try {
//       for(int i = 0; i < 30000; i++) {
//         CORBA::Boolean loop_status;
//         loop_status = log_collector->add_log(log);
//         if (loop_status == false) {
//           status = loop_status;
//         }
//       }
//     } catch (...) {
//       std::cerr << "Unexpected exception" << std::endl;
//       status = false;
//     }

//     // Wait enough before a check
//     Cdmw::OsSupport::OS::sleep(timescale * 10000);
//     std::cout << "Check status SIM Collector" << std::endl;
//     status = sim_collector->get_status();
//     std::cout << "status SIM Collector = " << status << std::endl;

    CPPUNIT_ASSERT(status);

}

void TestLogCollector::do_tests()      
{
    test_log_collector();
    //kill log collector and restart the log collector
    
}
        
}; // End namespace Trace
}; // End namespace Cdmw

