/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#ifndef INCL_TRACE_TESTLOGCOLLECTOR_HPP 
#define INCL_TRACE_TESTLOGCOLLECTOR_HPP 

#include <string>

#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/ossupport/OS.hpp>

namespace Cdmw
{
namespace Trace 
{

/**
*Purpose:
*<p>   This class tests the Performance Log file written by the Probe Collector
*      when Performance service is clean up.
*
*Features:
*<p>   none
*
*@see 
*@link 
*
*/
class TestLogCollector : public CppUnit::TestFixture
{

    public:

        /**
        * Purpose:
        * <p> Constructor
        * 
        *@param
        *@exception
        */ 
        TestLogCollector()
                throw();

        /**
        * Purpose:
        * <p>  Copy constructor 
        *
        *@param rhs A TestLogCollector object to assign to current object.
        *@exception
        */ 
        TestLogCollector(const TestLogCollector& rhs)
                throw();


        /**
        * Purpose:
        * <p> Destructor
        * 
        */ 
        virtual 
        ~TestLogCollector()
                throw();


        /**
        * Purpose:
        * <p> Assignement operator
        * 
        *@param rhs A TestPerformanceFile object to assign to current object.
        *@return A reference to the current object.
        *@exception
        */ 
        TestLogCollector&
        operator=(const TestLogCollector& rhs)
                throw();

    protected:

        virtual void do_tests();

    private:
        void test_log_collector();
        
//         std::string m_logFileName1;
//         std::string m_logFileName2;
        
//         bool m_logCreationSuccess;
        
//         bool m_horizontalFormat;

        std::string m_system_port;
        
       // Process ID of the Local Trace Collector
        Cdmw::OsSupport::OS::ProcessId m_local_trace_collector_id;

CPPUNIT_TEST_SUITE( TestLogCollector );
CPPUNIT_TEST( do_tests );
CPPUNIT_TEST_SUITE_END();

}; // End class TestLogCollector 

}; // End namespace Trace
}; // End namespace Cdmw
#endif // INCL_TRACE_TESTLOGCOLLECTOR_HPP

