/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.LocalObject;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptorRegistrationHelper;
import org.omg.Components.ContainerPortableInterceptor.ServantContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.ServantContainerInterceptorRegistrationHelper;
import org.omg.Components.ContainerPortableInterceptor.ServerContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.ServerContainerInterceptorRegistrationHelper;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptorRegistration;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptorRegistrationHelper;
import org.omg.PortableInterceptor.InvalidSlot;

import com.thalesgroup.CdmwCcmCopi.COPICurrent;
import com.thalesgroup.CdmwCcmCopi.RegisteredCOPICollection;
import com.thalesgroup.CdmwCcmCopi.RegisteredCOPICollectionHelper;

/**
 * Current object that provides support to a multi-ORB environment.
 * It is created in the ORBInitializer and used to provide data 
 * related to the current ORB-specific COPIs.
 */

public class COPICurrentImpl extends LocalObject implements COPICurrent {

    public static final String COPI_CURRENT_ID = "COPICurrent";
    /**
     * The needed slots identifiers
     */
    int Origin_ID_SlotID, Target_ID_SlotID, OriginFeatureName_SlotID,
            TargetFeatureName_SlotID, OriginCustomSlotItemSeq_SlotID,
            TargetCustomSlotItemSeq_SlotID;

    /**
     * The registration interface of COPIs
     */
    COPIRegistration ContainerInterceptorRegistration;

    
    public COPICurrentImpl(int origin_ID_SlotID, int target_ID_SlotID,
            int originFeatureName_SlotID, int targetFeatureName_SlotID,
            int originCustomSlotItemSeq_SlotID,
            int targetCustomSlotItemSeq_SlotID, COPIRegistration reg) {

        Origin_ID_SlotID = origin_ID_SlotID;
        Target_ID_SlotID = target_ID_SlotID;
        OriginFeatureName_SlotID = originFeatureName_SlotID;
        TargetFeatureName_SlotID = targetFeatureName_SlotID;
        OriginCustomSlotItemSeq_SlotID = originCustomSlotItemSeq_SlotID;
        TargetCustomSlotItemSeq_SlotID = targetCustomSlotItemSeq_SlotID;
        ContainerInterceptorRegistration = reg;
    }

    public int get_slot_id(String slot_name) throws InvalidSlot {

        if (COPIServiceInit.Origin_ID_SlotNAME.equals(slot_name))
            return Origin_ID_SlotID;

        if (COPIServiceInit.Target_ID_SlotNAME.equals(slot_name))
            return Target_ID_SlotID;

        if (COPIServiceInit.OriginFeatureName_SlotNAME.equals(slot_name))
            return OriginFeatureName_SlotID;

        if (COPIServiceInit.TargetFeatureName_SlotNAME.equals(slot_name))
            return TargetFeatureName_SlotID;

        if (COPIServiceInit.OriginCustomSlotItemSeq_SlotNAME.equals(slot_name))
            return OriginCustomSlotItemSeq_SlotID;

        if (COPIServiceInit.TargetCustomSlotItemSeq_SlotNAME.equals(slot_name))
            return TargetCustomSlotItemSeq_SlotID;

        throw new InvalidSlot(
                "Invalid slot name. Only Origin_ID_SlotNAME, Target_ID_SlotNAME, OriginFeatureName_SlotNAME,"
                        + " TargetFeatureName_SlotNAME, OriginCustomSlotItemSeq_SlotNAME, TargetCustomSlotItemSeq_SlotNAME"
                        + " are allocated");
    }

    public ClientContainerInterceptorRegistration get_client_interceptor_registration()
            throws CCMException {
        ClientContainerInterceptorRegistration clientCOPIRegistration = null;
        try {
            clientCOPIRegistration = ClientContainerInterceptorRegistrationHelper
                    .narrow(ContainerInterceptorRegistration);
        } catch (BAD_PARAM bp) {
            throw new CCMException(
                    "Enable to get the client registration interface",
                    CCMExceptionReason.REGISTRATION_ERROR);
        }

        if (clientCOPIRegistration == null)
            throw new CCMException(
                    "Enable to get the client registration interface",
                    CCMExceptionReason.REGISTRATION_ERROR);

        return clientCOPIRegistration;
    }

    public ServerContainerInterceptorRegistration get_server_interceptor_registration()
            throws CCMException {
        ServerContainerInterceptorRegistration serverCOPIRegistration = null;
        try {
            serverCOPIRegistration = ServerContainerInterceptorRegistrationHelper
                    .narrow(ContainerInterceptorRegistration);
        } catch (BAD_PARAM bp) {
            throw new CCMException(
                    "Enable to get the server registration interface",
                    CCMExceptionReason.REGISTRATION_ERROR);
        }

        if (serverCOPIRegistration == null)
            throw new CCMException(
                    "Enable to get the server registration interface",
                    CCMExceptionReason.REGISTRATION_ERROR);

        return serverCOPIRegistration;

    }

    public StubContainerInterceptorRegistration get_stub_interceptor_registration()
            throws CCMException {
        StubContainerInterceptorRegistration stubCOPIRegistration = null;
        try {
            stubCOPIRegistration = StubContainerInterceptorRegistrationHelper
                    .narrow(ContainerInterceptorRegistration);
        } catch (BAD_PARAM bp) {
            throw new CCMException(
                    "Enable to get the stub registration interface",
                    CCMExceptionReason.REGISTRATION_ERROR);
        }

        if (stubCOPIRegistration == null)
            throw new CCMException(
                    "Enable to get the stub registration interface",
                    CCMExceptionReason.REGISTRATION_ERROR);

        return stubCOPIRegistration;
    }

    public ServantContainerInterceptorRegistration get_servant_interceptor_registration()
            throws CCMException {
        ServantContainerInterceptorRegistration servantCOPIRegistration = null;
        try {
            servantCOPIRegistration = ServantContainerInterceptorRegistrationHelper
                    .narrow(ContainerInterceptorRegistration);
        } catch (BAD_PARAM bp) {
            throw new CCMException(
                    "Enable to get the servant registration interface",
                    CCMExceptionReason.REGISTRATION_ERROR);
        }

        if (servantCOPIRegistration == null)
            throw new CCMException(
                    "Enable to get the servant registration interface",
                    CCMExceptionReason.REGISTRATION_ERROR);

        return servantCOPIRegistration;
    }
    
    public RegisteredCOPICollection get_registered_copi_collection(){        
        
      return RegisteredCOPICollectionHelper.narrow(ContainerInterceptorRegistration);
        
    }
}
