/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import static cdmw.logging.Logger.CDMW_DEBUG;
import static cdmw.logging.Logger.CDMW_ERROR;

import org.omg.CORBA.LocalObject;
import org.omg.IOP.Codec;
import org.omg.IOP.CodecFactoryPackage.UnknownEncoding;
import org.omg.PortableInterceptor.Current;
import org.omg.PortableInterceptor.CurrentHelper;
import org.omg.PortableInterceptor.ORBInitInfo;
import org.omg.PortableInterceptor.ORBInitializer;
import org.omg.PortableInterceptor.ORBInitInfoPackage.DuplicateName;
import org.omg.PortableInterceptor.ORBInitInfoPackage.InvalidName;

import com.thalesgroup.CdmwCcmCopi.RegisteredCOPICollection;

import cdmw.ccm.common.CCMLoggers;

/**
 *
 */
public class COPIMainORBInitializer extends LocalObject implements ORBInitializer {
    
    cdmw.logging.Logger logger = CCMLoggers.GetCcmCopiLogger();

 

    public void post_init(ORBInitInfo info) {
        //
        // allocate needed slot_ids
        //
        CDMW_DEBUG(logger, "post_init => allocate needed slot_ids");
        int OriginID = info.allocate_slot_id(),
            TargetID = info.allocate_slot_id(),
            OriginFeatureNameID = info.allocate_slot_id(),
            TargetFeatureNameID = info.allocate_slot_id(),
            OriginCustomSlotItemSeqID = info.allocate_slot_id(),
            TargetCustomSlotItemSeqID = info.allocate_slot_id();
                
        try{
            
            // creating a Codec for service context encoding
            // setting the format = ENCODING_CDR_ENCAPS, major_version=1 and minor_version=2
            // TODO : to make the version setting dynamic and not hard-coded
            org.omg.IOP.Encoding enc = new org.omg.IOP.Encoding(org.omg.IOP.ENCODING_CDR_ENCAPS.value,Byte.parseByte("1"),Byte.parseByte("2")); 
            Codec codec = info.codec_factory().create_codec(enc);
            //setting the Codec for some util methods
            COPIUtils.setCodec(codec);
            
            //creating the registration interface
            COPIRegistration registrationInterface = new COPIRegistration(OriginCustomSlotItemSeqID, TargetCustomSlotItemSeqID);
            
          // create and register the COPICurrent object
            com.thalesgroup.CdmwCcmCopi.COPICurrent copiCurrent = new COPICurrentImpl(OriginID, TargetID, 
                    OriginFeatureNameID, TargetFeatureNameID,
                    OriginCustomSlotItemSeqID, TargetCustomSlotItemSeqID, registrationInterface);
            
            info.register_initial_reference(COPICurrentImpl.COPI_CURRENT_ID, copiCurrent);
            
         // registering the main client and server interceptors
            ClientRequestInterceptorIterator cri = new ClientRequestInterceptorIterator(OriginID,
                    TargetID, OriginFeatureNameID, OriginCustomSlotItemSeqID, (RegisteredCOPICollection)registrationInterface);
            ServerRequestInterceptorIerator sri = new ServerRequestInterceptorIerator(OriginID,
                    TargetID, TargetFeatureNameID, TargetCustomSlotItemSeqID, (RegisteredCOPICollection)registrationInterface);
            info.add_client_request_interceptor(cri);
            info.add_server_request_interceptor(sri);
            
        }
        catch(InvalidName in){
            CDMW_ERROR(logger,"Failed to get PICurrent reference");
            in.printStackTrace();
        }
        catch(DuplicateName ex){
            ex.printStackTrace();
        }
        catch(UnknownEncoding ue){
            CDMW_ERROR(logger,"Unsupported Encoding format");
            ue.printStackTrace();
        }
                
    }

 
    public void pre_init(ORBInitInfo info) {
        // TODO Auto-generated method stub

    }
}
