/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.container;

import java.util.TreeMap;

import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.LocalObject;
import org.omg.Components.Cookie;
import org.omg.Components.ContainerPortableInterceptor.ContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.InvalidRegistration;

import cdmw.ccm.ccmcopi.COPICookieImpl;
import cdmw.ccm.common.CCMLoggers;
import cdmw.ccm.common.CookieImpl;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.orbsupport.ExceptionStrings;
import cdmw.orbsupport.ORBSupport;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor;
import com.thalesgroup.CdmwCcmContainer.HomeLifeCycleInterceptor;

/**
 * Singleton class that allows to register and unregister LifeCycle interceptors
 */
public class LifeCycleInterceptorRegistration extends LocalObject implements
com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistration {
    
    public static String LifeCycleInterceptorRegistrationServiceId ="CDMW_LifeCycleInterceptorRegistrationService";
    
    private static LifeCycleInterceptorRegistration theInstance;
    
    private TreeMap<Cookie, ComponentLifeCycleInterceptor> componentLCIMap = new TreeMap<Cookie, ComponentLifeCycleInterceptor>();
    private TreeMap<Cookie,HomeLifeCycleInterceptor> homeLCIMap = new TreeMap<Cookie, HomeLifeCycleInterceptor>();

    protected cdmw.logging.Logger logger = CCMLoggers.GetCcmcontainerLogger();
    
    
    private LifeCycleInterceptorRegistration(){
    }
    
    public static LifeCycleInterceptorRegistration getInstance(){
        if (theInstance==null) theInstance = new LifeCycleInterceptorRegistration();
        return theInstance;
    }

    public Cookie register_component_lifecycle_interceptor(
            ComponentLifeCycleInterceptor clci) {
        CDMW_LOG_FUNCTION(logger, "");
        Cookie ck = null;

        synchronized (componentLCIMap) {
            if (componentLCIMap.containsValue(clci)) {
                org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                        "COPI Already Registered !!");
                try {
                    int minor_code = ExceptionMinorCodes.BAD_PARAMCOPIAlreadyExist;
                    String message = ExceptionStrings.BAD_PARAM_reasons[minor_code
                            & ExceptionMinorCodes.MINOR_CODE_MASK];
                    ex = (org.omg.CORBA.BAD_PARAM) ORBSupport
                            .createSystemException("org.omg.CORBA.BAD_PARAM",
                                    minor_code,
                                    ExceptionStrings.BAD_PARAM_reasons,
                                    CompletionStatus.COMPLETED_YES);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }

            ck = new COPICookieImpl(clci);
            componentLCIMap.put(ck, clci);
        }

        CDMW_LOG_FUNCTION_RETURN(logger);
        return ck;
    }


    public Cookie register_home_lifecycle_interceptor(
            HomeLifeCycleInterceptor hlci) {
        CDMW_LOG_FUNCTION(logger, "");
        Cookie ck = null;

        synchronized (homeLCIMap) {
            if (homeLCIMap.containsValue(hlci)) {
                org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                        "COPI Already Registered !!");
                try {
                    int minor_code = ExceptionMinorCodes.BAD_PARAMCOPIAlreadyExist;
                    String message = ExceptionStrings.BAD_PARAM_reasons[minor_code
                            & ExceptionMinorCodes.MINOR_CODE_MASK];
                    ex = (org.omg.CORBA.BAD_PARAM) ORBSupport
                            .createSystemException("org.omg.CORBA.BAD_PARAM",
                                    minor_code,
                                    ExceptionStrings.BAD_PARAM_reasons,
                                    CompletionStatus.COMPLETED_YES);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }

            ck = new COPICookieImpl(hlci);
            homeLCIMap.put(ck, hlci);
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
        return ck;
    }

 
    public ComponentLifeCycleInterceptor unregister_component_lifecycle_interceptor(
            Cookie ck) throws InvalidRegistration {
        CDMW_LOG_FUNCTION(logger, "");

        synchronized (componentLCIMap) {
            if (!componentLCIMap.containsKey(ck)) {
                throw new InvalidRegistration(
                        "Unable to find component lifecycle interceptor associated to this cookie");
            }
            CDMW_LOG_FUNCTION_RETURN(logger);
            return componentLCIMap.remove(ck);
        }

    }


    public HomeLifeCycleInterceptor unregister_home_lifecycle_interceptor(Cookie ck)
            throws InvalidRegistration {
        CDMW_LOG_FUNCTION(logger, "");

        synchronized (homeLCIMap) {
            if (!homeLCIMap.containsKey(ck)) {
                throw new InvalidRegistration(
                        "Unable to find home lifecycle interceptor associated to this cookie");
            }
            CDMW_LOG_FUNCTION_RETURN(logger);
            return homeLCIMap.remove(ck);
        }

    }
    
    public ComponentLifeCycleInterceptor[] listComponentLifeCycleInterceptor(){
        synchronized (componentLCIMap) {
            return componentLCIMap.values().toArray(new ComponentLifeCycleInterceptor[0]); 
        }
    }
    
    public HomeLifeCycleInterceptor[] listHomeLifeCycleInterceptor(){
        synchronized (homeLCIMap) {
            return homeLCIMap.values().toArray(new HomeLifeCycleInterceptor[0]); 
        }
    }
}
