/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.container;

import org.apache.avalon.framework.service.ServiceManager;
import org.omg.CORBA.Object;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;

/**
 * A simple registry for the container services references
 */
public class ServiceRegistry extends org.omg.CORBA.LocalObject
        implements com.thalesgroup.CdmwCcmContainer.ServiceRegistry{


    static private ServiceRegistry theInstance;
    String name;

    protected java.util.Map<String, org.omg.CORBA.Object> servicesMap = new java.util.HashMap<String, org.omg.CORBA.Object>();


    public static ServiceRegistry getInstance(){
        if (theInstance == null) theInstance = new ServiceRegistry("GlobalRegistry");
        return theInstance;
    }

    public ServiceRegistry(String name) {
        this.name = name;
    }


    synchronized public void bind(String service_id, Object service_ref) throws CCMException {
        if(servicesMap.containsKey(service_id))
            throw new CCMException(CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
        servicesMap.put(service_id, service_ref);

    }

    synchronized public String[] list() {
        return servicesMap.keySet().toArray(new String[0]);
    }

    synchronized public Object lookup(String service_id) throws CCMException {
        if(!servicesMap.containsKey(service_id))
            throw new CCMException(CCMExceptionReason.OBJECT_NOT_FOUND);
        return servicesMap.get(service_id);
    }

    synchronized public void rebind(String service_id, Object service_ref) throws CCMException {
        if(!servicesMap.containsKey(service_id))
            throw new CCMException(CCMExceptionReason.OBJECT_NOT_FOUND);
        servicesMap.put(service_id, service_ref);

    }

    synchronized public Object unbind(String service_id) throws CCMException {
        if(!servicesMap.containsKey(service_id))
            throw new CCMException(CCMExceptionReason.FIND_ERROR);
        return servicesMap.remove(service_id);
    }

    synchronized public void clear(){
        servicesMap.clear();
    }





}
