/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.qosenablers.dds;

import java.util.logging.Logger;

import org.omg.CCM_DDS.BadParameter;
import org.omg.CCM_DDS.DDSEntityKind;
import org.omg.CCM_DDS.InternalError;
import org.omg.CCM_DDS.NoSuchDDSEntity;
import org.omg.CCM_DDS.ReadInfoHolder;
import org.omg.CCM_DDS.ReadInfoSeqHolder;

import DDS.DataReader;
import DDS.ReadCondition;
import DDS.SampleInfo;

import org.omg.CCM_DDS.QueryFilter;

import org.omg.CORBA.LocalObject;
import org.omg.DDS.Duration_t;
import org.omg.DDS.RETCODE_OK;

/**
 *
 */
public class DDS_ReaderImpl extends LocalObject{

    Logger logger = Logger.getLogger("cdmw.ccm.qosenablers.dds.DDS_ReaderImpl");

    protected ReadCondition readCond;
    protected ReadCondition userQuery;
    protected QueryFilter filter;
    protected Duration_t time_out = new Duration_t();
    protected DataReader reader;


    public DDS_ReaderImpl(DataReader reader) {
        this.reader = reader;

        readCond = reader.create_readcondition(DDS.READ_SAMPLE_STATE.value |
                DDS.NOT_READ_SAMPLE_STATE.value, DDS.NEW_VIEW_STATE.value  |
                DDS.NOT_NEW_VIEW_STATE.value, DDS.ALIVE_INSTANCE_STATE.value);
        //TODO: To review this case
        if(readCond == null) {
                System.out.println("Error on create_readcondition");
                System.exit(-1);
        }

        userQuery = readCond;

}


    /**
     * Attributes operations
     */
    synchronized public QueryFilter filter() {
        return this.filter;
    }

    synchronized public void set_filter(QueryFilter arg) throws BadParameter {
        this.filter = arg;
        if ("".equals(arg.query)) {
            userQuery = readCond;
            return;
        }
        userQuery = reader.create_querycondition(
                DDS.READ_SAMPLE_STATE.value |
                DDS.NOT_READ_SAMPLE_STATE.value, DDS.NEW_VIEW_STATE.value
                        | DDS.NOT_NEW_VIEW_STATE.value,
                DDS.ALIVE_INSTANCE_STATE.value , filter.query,
                filter.query_parameters);
        if (userQuery == null) {
            throw new BadParameter();
        }

    }

    public org.omg.CORBA.Object getDDSEntity(DDSEntityKind kind) throws NoSuchDDSEntity{
        return null;
    }

    /**
     * prepares the data
     *
     * @param <D>
     *            the data type
     * @param <H>
     *            the data sequence corba holder type
     * @param instances
     *            the read instances
     * @param infos
     * @param samples
     * @param readStatus
     * @return
     * @throws InternalError
     */
    public <D, H> void readAll(DataSeqHolder<D, H> instances,
            ReadInfoSeqHolder infos, SampleInfo[] samples, int readStatus)
            throws InternalError {

        DataUtil.getData(instances, infos, samples, readStatus, true);
    }

    /**
     *
     * @param <D>
     * @param <H>
     * @param instances
     * @param infos
     * @param samples
     * @param readStatus
     * @return
     * @throws InternalError
     */
    public <D, H> void readAllHistory(DataSeqHolder<D, H> instances,
            ReadInfoSeqHolder infos, SampleInfo[] samples, int readStatus)
            throws InternalError {

        DataUtil.getData(instances, infos, samples, readStatus, false);
    }

    /**
     *
     * @param infos
     * @param samples
     * @param readStatus
     * @return
     * @throws InternalError
     */
    public void readNext(ReadInfoHolder infos, SampleInfo[] samples,
            int readStatus) throws InternalError {

        if (samples.length != 1) {
            InternalError e = new InternalError();
            e.error_code = DDS.RETCODE_ERROR.value;
            e.index = 0;
            throw e;
        }

        DataUtil.getData(infos, samples[0], readStatus);
    }

    public void readOne(ReadInfoHolder infos, SampleInfo[] samples,
            int readStatus) throws InternalError {

        if (readStatus != RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = readStatus;
            e.index = 0;
            throw e;
        }

        for (SampleInfo sample : samples) {
            if (sample.sample_rank == 0) {
                DataUtil.getData(infos, sample, readStatus);
            }
        }

    }

    public void readOneHistory(ReadInfoSeqHolder infos, SampleInfo[] samples,
            int readStatus) throws InternalError {

        DataUtil.getData(infos, samples, readStatus, false);

    }



}
