/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.qosenablers.dds;

import java.util.ArrayList;
import java.util.List;

import org.omg.CCM_DDS.AccessStatus;
import org.omg.CCM_DDS.InstanceStatus;
import org.omg.CCM_DDS.InternalError;
import org.omg.CCM_DDS.ReadInfo;
import org.omg.CCM_DDS.ReadInfoHolder;
import org.omg.CCM_DDS.ReadInfoSeqHolder;
import org.omg.DDS.RETCODE_OK;

import DDS.ConditionSeqHolder;
import DDS.ReadCondition;
import DDS.SampleInfo;
import DDS.WaitSet;

/**
 * @param <H>
 *
 */
public class DataUtil extends org.omg.CORBA.LocalObject {


    public static void fillReadInfo(SampleInfo sampleInfo, ReadInfo readInfo, int instance_rank) {

        //setting the access_status
        if (sampleInfo.sample_state == DDS.NOT_READ_SAMPLE_STATE.value) {
            readInfo.access_status = AccessStatus.FRESH_INFO;
        } else {
            readInfo.access_status = AccessStatus.ALREADY_SEEN;
        }

        //setting the instance_state
        if(sampleInfo.view_state == DDS.NEW_VIEW_STATE.value &&
           sampleInfo.instance_state == DDS.ALIVE_INSTANCE_STATE.value &&
           sampleInfo.valid_data == true){
            readInfo.instance_status = InstanceStatus.INSTANCE_CREATED;
        }

        if(((sampleInfo.view_state == DDS.NOT_NEW_VIEW_STATE.value &&
                sampleInfo.instance_state == DDS.ALIVE_INSTANCE_STATE.value) ||
                sampleInfo.instance_state == DDS.NOT_ALIVE_NO_WRITERS_INSTANCE_STATE.value) &&
                sampleInfo.valid_data == true){
                 readInfo.instance_status = InstanceStatus.INSTANCE_UPDATED;
        }

        if (sampleInfo.instance_state == DDS.NOT_ALIVE_DISPOSED_INSTANCE_STATE.value) {
            readInfo.instance_status = InstanceStatus.INSTANCE_DELETED;
        }

        readInfo.instance_rank = instance_rank;
        readInfo.timestamp = new org.omg.DDS.Time_t(
                sampleInfo.source_timestamp.sec,
                sampleInfo.source_timestamp.nanosec);

    }


    /**
     *
     * @param <D>
     * @param <H>
     * @param instances
     *            only the valid instances data
     * @param infos
     * @param samples
     * @param readStatus
     * @param lastSample
     * @return
     * @throws InternalError
     */
    public static <D, H> boolean getData(DataSeqHolder<D, H> instances,
            ReadInfoSeqHolder infos, SampleInfo[] samples, int readStatus,
            boolean lastSample) throws InternalError {

        if (readStatus != RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = readStatus;
            e.index = 0;
            throw e;
        }

        List<ReadInfo> readInfoList = new ArrayList<ReadInfo>();
        List<D> dataValues = new ArrayList<D>();
        try {
            for (int i = 0; i < samples.length; i++) {
                if (lastSample) {
                    if (samples[i].sample_rank == 0 && samples[i].valid_data) {
                        ReadInfo ri = new ReadInfo();
                        fillReadInfo(samples[i], ri, readInfoList.size()+1);
                        readInfoList.add(ri);
                        dataValues.add(instances.seqValue()[i]);
                    }

                } else {
                    if (samples[i].valid_data) {
                        ReadInfo ri = new ReadInfo();
                        fillReadInfo(samples[i], ri, readInfoList.size()+1);
                        readInfoList.add(ri);
                        dataValues.add(instances.seqValue()[i]);
                    }
                }
            }

            infos.value = readInfoList.toArray(new ReadInfo[0]);
            instances.setDataValueSeq_o(dataValues.toArray());

        } catch (Exception e) {
            throw new InternalError("Failed to insert the data values : "+ e.getMessage(),
                    DDS.RETCODE_ERROR.value, 0);
        }

        return true;
    }

    public static boolean getData(ReadInfoSeqHolder infos, SampleInfo[] samples,
            int readStatus, boolean lastSample) throws InternalError {

        if (readStatus != RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = readStatus;
            e.index = 0;
            throw e;
        }

        List<ReadInfo> readInfoList = new ArrayList<ReadInfo>();

        for (int i = 0; i < samples.length; i++) {
            if (lastSample) {
                if (samples[i].sample_rank == 0) {
                    ReadInfo ri = new ReadInfo();
                    fillReadInfo(samples[i], ri, readInfoList.size()+1);
                    readInfoList.add(ri);
                }
            } else {
                ReadInfo ri = new ReadInfo();
                fillReadInfo(samples[i], ri, readInfoList.size()+1);
                readInfoList.add(ri);
            }
        }

        infos.value = readInfoList.toArray(new ReadInfo[0]);
        return true;
    }

    public static boolean getData(ReadInfoHolder infos, SampleInfo sample,
            int readStatus) throws InternalError {

        if (readStatus != RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = readStatus;
            e.index = 0;
            throw e;
        }

        ReadInfo ri = new ReadInfo();
        fillReadInfo(sample, ri, 0);

        infos.value = ri;
        return true;
    }

    public static <D, H> boolean getData(DataSeqHolder<D, H> instances, ReadInfoHolder infos,
            SampleInfo sample, int readStatus) throws InternalError {

        if (readStatus != RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = readStatus;
            e.index = 0;
            throw e;
        }

        if(sample.valid_data){
            ReadInfo ri = new ReadInfo();
            fillReadInfo(sample, ri, 0);
            infos.value = ri;
        } else {
          throw new InternalError("No valid data found", DDS.RETCODE_NO_DATA.value, 0);
        }

        return true;
    }

    public static void waitData( WaitSet waits, ReadCondition readCond,
            DDS.Duration_t timeout ) throws InternalError {
        ConditionSeqHolder condList = new ConditionSeqHolder();

        int status = waits._wait(condList, timeout);

        if (status != RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = status;
            e.index = 0;
            throw e;
        }

        if (condList.value[0] != readCond) {
            InternalError e = new InternalError();
            e.error_code = DDS.RETCODE_ERROR.value;
            e.index = 0;
            throw e;
        }

    }

    public static <D, H> void initData(DataSeqHolder<D, H> instances,
            ReadInfoSeqHolder rish, ReadInfoHolder rih )
                throws IllegalArgumentException, SecurityException,
                IllegalAccessException, InstantiationException, NoSuchFieldException{

        if(instances!=null){
            instances.init();
        }
        initReadInfo(rish, rih);
    }

    public static <D, H> void initSeqData(Class<D[]> valueClass, DataSeqHolder<D, H> instances,
            ReadInfoSeqHolder rish, ReadInfoHolder rih )
                throws IllegalArgumentException, SecurityException,
                IllegalAccessException, InstantiationException, NoSuchFieldException{

        if(instances!=null){
            instances.initSeq(valueClass);
        }
        initReadInfo(rish, rih);

    }

    public static <D, H> void initData(Class<D> valueClass, DataSeqHolder<D, H> instances,
            ReadInfoSeqHolder rish, ReadInfoHolder rih )
                throws IllegalArgumentException, SecurityException,
                IllegalAccessException, InstantiationException, NoSuchFieldException{

        if(instances!=null){
            instances.init(valueClass);
        }
        initReadInfo(rish, rih);

    }

    public static void initReadInfo(ReadInfoSeqHolder rish, ReadInfoHolder rih){
        if(rish!=null){
            rish.value = new ReadInfo[0];
        }
        if(rih!=null){
            rih.value = new ReadInfo();
        }
    }



}
