/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.SortedMap;
import java.util.TreeMap;

import static org.junit.Assert.*;
import org.omg.CORBA.LocalObject;
import org.omg.Components.CCMException;
import org.omg.Components.Cookie;
import org.omg.Components.ExtensionContext;
import org.omg.Components.ContainerPortableInterceptor.InvalidRegistration;

import com.acme.TestService.COPITester;
import com.acme.TestService.COPIType;

/**
 *
 */
public class COPITesterImpl extends LocalObject implements COPITester {

    ExtensionContext context;
    
    HashMap<String, COPIInfo> registeredCOPIs = new HashMap<String, COPIInfo>();
    
    boolean started;
    
    class COPIInfo{
        Cookie ck;
        COPIType type;
        int used;
        String name;
        
        
        public COPIInfo(Cookie ck, COPIType type, String name) {
            this.ck = ck;
            this.type = type;
            this.name = name;
        }
        
        synchronized public void push(){
            used++;
        }
        
        synchronized public void pop(){
           used--; 
        }
        
    }
    
    class InterceptionPoint{
        String COPIname,
               interceptedOp,
               interceptionPoint;
        Date timestamp;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        
        public InterceptionPoint(String iname, String interceptedOp,
                String interceptionPoint, Date timestamp) {
            COPIname = iname;
            this.interceptedOp = interceptedOp;
            this.interceptionPoint = interceptionPoint;
            this.timestamp = timestamp;
        }
        
        public String toString(){
            String message = sdf.format(timestamp) +"\t"+
                COPIname +"."+interceptionPoint+ "\t"+
                interceptedOp;
            return message;
        }
        
    }
    
    //SortedMap<Date, InterceptionPoint> histo = new TreeMap<Date, InterceptionPoint>();
    SortedMap<Integer, InterceptionPoint> histo = new TreeMap<Integer, InterceptionPoint>();
    
    public COPITesterImpl(ExtensionContext context) {
        this.context = context;
    }
    

    public void register_interceptor(String name, COPIType type) {
        Cookie ck =null;
        COPIInfo info = null;
        
        try{
            switch(type.value()){
            case COPIType._CLIENT :
                ck = context.get_client_interceptor_registration().register_client_interceptor(new LoggerClientCOPIImpl(name, this));
                break;
            case COPIType._SERVER :
                ck = context.get_server_interceptor_registration().register_server_interceptor(new LoggerServerCOPIImpl(name, this));
                break;
            case COPIType._SERVANT :
                ck = context.get_servant_interceptor_registration().register_servant_interceptor(new LoggerServantCOPIImpl(name, this));
                break;
            case COPIType._STUB :
                ck = context.get_stub_interceptor_registration().register_stub_interceptor(new LoggerStubCOPIImpl(name, this));
                break;
                default:       
            }
            this.registeredCOPIs.put(name, new COPIInfo(ck,type, name));
            System.out.println("==================================");
            System.out.println("Interceptor "+name+" REGISTERED");
            System.out.println("==================================");
        }
        catch(CCMException ccme){
            ccme.printStackTrace();
        }
        

    }

    public synchronized void test_interception_stacks() {

        System.out.println("****************************************");
        System.out.println("****** COPIs Testing report *************");
        System.out.println("****************************************");
        
        if(histo.isEmpty()) {
            System.out.println("No interception points registered. May be the COPITester_SERVICE is not started !");
            return;
        }
        System.out.println("Checking that every COPI processed an ending interception point ....");
        for(COPIInfo info:registeredCOPIs.values()){
            System.out.println(info.name +"\t"+(info.used==0?"OK":"Not OK"));
        }
        
       System.out.println("Checking state changes of every COPI ...");
       InterceptionPoint[] histoTab = histo.values().toArray(new InterceptionPoint[]{});
       
      /* for(int i=0; i < histoTab.length; i++){
           System.out.println(histoTab[i].toString());
       }
       System.out.println("**************************************************"); */
       
       int clientBasicStep = 0,
           serverBasicStep = 0,
           stubExtendedStep = 0,
           servantExtendedStep = 0;
       
       for(int i=0; i < histoTab.length; ){
               
           while(i < histoTab.length && "stub_send_request".equals(histoTab[i].interceptionPoint)){
               stubExtendedStep++;
               System.out.println(histoTab[i].toString());
               i++;
           }
           
           
           while(i < histoTab.length && "send_request".equals(histoTab[i].interceptionPoint)){
               clientBasicStep++;
               System.out.println(histoTab[i].toString());
               i++;
           }
           
           while(i < histoTab.length && "receive_request_service_contexts".equals(histoTab[i].interceptionPoint)){
               serverBasicStep++;
               System.out.println(histoTab[i].toString());i++;
           }
           
           while(i < histoTab.length && ("receive_request".equals(histoTab[i].interceptionPoint))){
               
               System.out.println(histoTab[i].toString());i++;
           }
           
           while(i < histoTab.length && ("servant_receive_request".equals(histoTab[i].interceptionPoint))){
               servantExtendedStep++;
               System.out.println(histoTab[i].toString());i++;
           }
           
           
           while(i < histoTab.length && (histoTab[i].interceptionPoint.startsWith("servant_send_"))){
               assertTrue("Unexpected interception point: "+histoTab[i].toString(),servantExtendedStep != 0);
               servantExtendedStep--;
               System.out.println(histoTab[i].toString());i++;
           }
           
           while(i < histoTab.length && (histoTab[i].interceptionPoint.startsWith("send_reply") || histoTab[i].interceptionPoint.startsWith("send_exception") ||
                   histoTab[i].interceptionPoint.startsWith("send_other"))){
               assertTrue("Unexpected interception point: "+histoTab[i].toString(),serverBasicStep != 0);
               serverBasicStep--;
               System.out.println(histoTab[i].toString());i++;
           }
           
           while(i < histoTab.length && (histoTab[i].interceptionPoint.startsWith("receive_reply") || histoTab[i].interceptionPoint.startsWith("receive_exception") ||
                   histoTab[i].interceptionPoint.startsWith("receive_other"))){
               assertTrue("Unexpected interception point: "+histoTab[i].toString(),clientBasicStep != 0);
               clientBasicStep--;
               System.out.println(histoTab[i].toString());i++;
           }
           
           while(i < histoTab.length && (histoTab[i].interceptionPoint.startsWith("stub_receive_"))){
               assertTrue("Unexpected interception point: "+histoTab[i].toString(),stubExtendedStep!=0);
               stubExtendedStep --;
               System.out.println(histoTab[i].toString());i++;
           }
           
         //(*ExtendedStep=1) expresses the case where one of the servant/stub COPIs raise an exception or blocks the call
           assertTrue("Illegal Stub COPI state transitions", stubExtendedStep == 0 || stubExtendedStep == 1);
           assertTrue("Illegal Servant COPI state transitions", servantExtendedStep == 0 || servantExtendedStep == 1); 
           servantExtendedStep = 0;
           stubExtendedStep = 0;
           assertTrue("Illegal Client COPI state transitions",clientBasicStep==0);
           assertTrue("Illegal Server COPI state transitions", serverBasicStep==0);
           
           
       }
       
       
       
       System.out.println("Checking state changes of every COPI ... OK :-)");

    }

    public void unregister_interceptor(String name) throws InvalidRegistration {
        COPIInfo info = registeredCOPIs.get(name);

        if(info==null) throw new org.omg.Components.ContainerPortableInterceptor.InvalidRegistration();
        try{
        switch(info.type.value()){
        case COPIType._CLIENT :
            context.get_client_interceptor_registration().unregister_client_interceptor(info.ck);
            
            break;
        case COPIType._SERVER :
            context.get_server_interceptor_registration().unregister_server_interceptor(info.ck);
            break;
        case COPIType._SERVANT :
            context.get_servant_interceptor_registration().unregister_servant_interceptor(info.ck);
            break;
        case COPIType._STUB :
            context.get_stub_interceptor_registration().unregister_stub_interceptor(info.ck);
            break;
            default:       
        }
        System.out.println("==================================");
        System.out.println("Interceptor "+name+" UNREGISTERED");
        System.out.println("==================================");
        assertTrue(info.used == 0);
        registeredCOPIs.remove(name);
        }
        catch(CCMException ccme){
            ccme.printStackTrace();
        }

    }
    
    int i;
    
    public synchronized void notify_operation(String op_name, String copi_name, Date timestamp, String intercepted_op){
        
        if(!isStarted()) return;
        
        //assertTrue(!histo.containsKey(timestamp));
        
        histo.put(new Integer(i), new InterceptionPoint(copi_name, intercepted_op, op_name, timestamp)); 
        System.out.println("Operation "+op_name+" notified ! rank : "+i); i++;
        
        COPIInfo info = registeredCOPIs.get(copi_name);
        
        if(intercepted_op.contains("send_request")){
            info.push();
        }        
        
        if("receive_service_context".equals(intercepted_op)){
            info.push();
        }
        
        if(intercepted_op.contains("receive_request")){
            info.push();
        }
                
        if(intercepted_op.startsWith("servant_send_")){
            info.pop();
        }
        
        if(intercepted_op.startsWith("send_reply") | intercepted_op.startsWith("send_exception") |
                intercepted_op.startsWith("send_other")){
            info.pop();
        }
        
        if(intercepted_op.startsWith("receive_reply") | intercepted_op.startsWith("receive_exception") |
                intercepted_op.startsWith("receive_other")){
            
            info.pop();
        }
        
        if(intercepted_op.startsWith("stub_receive_")){
            info.pop();
        }
    }
    
    
    public synchronized void start_listening(){
        started = true;
        histo.clear();
        i = 0;
    }
    
    public synchronized void end_listening(){
        started = false;
    }
    
    private boolean isStarted(){
        return started;
    }
    
    

}


