/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import static org.junit.Assert.*;

import org.junit.Test;
import org.omg.CORBA.IntHolder;
import org.omg.CORBA.LocalObject;
import org.omg.Components.CCMException;
import org.omg.Components.SessionComponent;
import org.omg.Components.SessionContext;
import org.omg.Components.ContainerPortableInterceptor.InvalidRegistration;

import cdmw.ccm.ccmcopi.COPIServiceInit;
import cdmw.testutils.CORBATestManager;

import com.acme.HelloModule.Hello;

import com.acme.TestComponents.ArithOperator;
import com.acme.TestComponents.CCM_Client;
import com.acme.TestComponents.ExceptionTest;
import com.acme.TestComponents.FacetTest;
import com.acme.TestService.COPIType;

/**
 *
 */
public class ClientComponentImpl extends LocalObject implements CCM_Client,
        SessionComponent {

    com.acme.TestComponents.CCM_Client_Context context;


    public void ccm_activate() throws CCMException {
        System.out.println("ccm_activate called on the Client Component ...");
    }

    public void ccm_passivate() throws CCMException {
        System.out.println("ccm_passivate called on the Client Component ...");
    }

    public void ccm_remove() throws CCMException {
        System.out.println("ccm_remove called on the Client Component ...");
    }

    public void set_session_context(SessionContext ctx) throws CCMException {
        this.context=com.acme.TestComponents.CCM_Client_ContextHelper.narrow(ctx);
        CORBATestManager.instance().setLocalProperty ("ClientSessionContext", this.context);
    }

    public void push_server_info(com.acme.TestComponents.ServerInfo event) {
        System.out.println("Receiving a ServerInfo event holding the following data value :" + event.data);
        notify_sender();
        EventDataExchanger.instance().setData(event.data);
    }

    public void push_server_is_activated(com.acme.TestComponents.Activated event) {
        System.out.println("Receiving the Server activation event notification ...");
        notify_sender();
    }

    private void notify_sender(){
        ServerComponentImpl.eventReceived();
    }

    @Test
    public void doTests() {

        System.out.println("*************************************************************\n"+
                "       ClientComponentImpl.class Test\n"+
                "*************************************************************");

        com.acme.TestService.Logger logger = null;

        try {
            context = (com.acme.TestComponents.CCM_Client_Context)CORBATestManager.instance().
                        getLocalProperty("ClientSessionContext");
            assertTrue(context != null);
            org.omg.CORBA.Object obj = context
                    .resolve_service_reference("LOGGING_SERVICE");
            logger = com.acme.TestService.LoggerHelper.narrow(obj);
            assertTrue(logger != null);

            logger.LOG_MESSAGE("Succeeded to get the LOGGING_SERVICE reference ...");

            obj = context.resolve_service_reference("COPITester_SERVICE");
            com.acme.TestService.COPITester tester = com.acme.TestService.COPITesterHelper.narrow(obj);
            assertNotNull(tester);
            logger.LOG_MESSAGE("Succeeded to get the COPITester_SERVICE reference ...");

            Hello hello = context.get_connection_to_echo();
            logger.LOG_MESSAGE("Succeeded to get connection to echo facet :-)");

            FacetTest facet = context.get_connection_to_server();
            logger.LOG_MESSAGE("Succeeded to get connection to FacetTest facet :-)");

            // this thread allows parallel registration
            COPIRegistratorThread t = new COPIRegistratorThread(tester);
            t.start();

            tester.start_listening();

            System.out.println("@@@@@@@@@@@@@@ Testing Disabled ReadWrite requests");
            doTests(logger, hello, facet);

            tester.end_listening();
            tester.test_interception_stacks();

            tester.start_listening();

            System.out.println("@@@@@@@@@@@@@@ Testing Enabled ReadWrite requests");

            COPIServiceInit.enableReadWriteRequests();
            assertTrue(COPIServiceInit.isCOPIRequestsReadWrite());

            // notify the Registrator thread to start to unregister the COPIs
            t.unregister();
            synchronized(t){
                System.out.println("@@@@@@@@@@@@@@ COPIs Unregistration requested ...");
                t.notify();
            }

            doTests(logger, hello, facet);
            tester.end_listening();
            tester.test_interception_stacks();
            t.join();

        } catch (Exception ex) {
            ex.printStackTrace();
            fail();
        }

        System.out.println("*************************************************************\n"+
                "       End of ClientComponentImpl.class Test\n"+
                "*************************************************************");
    }

    public void doTests(com.acme.TestService.Logger logger, Hello hello, FacetTest facet){

        System.out.println("********************************");
        System.out.println("Testing Operations interception");
        System.out.println("********************************");


        logger.LOG_MESSAGE("echoing messages ...");

        for(int i=0; i<5;i++)
            hello.echo("Message no "+i);

        System.out.println("********************************");
        System.out.println("Testing Exception raising");
        System.out.println("********************************");
        boolean gotTheRightEx = false;        
        try{
           facet.raiseException(true, "Testing an exceptional behaviour");
           assertTrue(false);
        }catch(ExceptionTest et){
            gotTheRightEx = true;
            logger.LOG_MESSAGE("Succeded to get the right exception :-)");
        }catch(org.omg.CORBA.SystemException ex){
            assertTrue(false);
        }catch(Exception e){
            e.printStackTrace();
            assertTrue(false);
        }
        
        assertTrue(true == gotTheRightEx);

        System.out.println("********************************");
        System.out.println("Testing Return and Out parameters");
        System.out.println("********************************");

        try {
            IntHolder rem = new IntHolder();
            int res = facet.makeOperation(ArithOperator.DIV, 100, 3, rem);
            logger.LOG_MESSAGE("100 DIV 3 = "+ res +", remainder = "+ rem.value);
            assertTrue(res==33 && rem.value==1);
            logger.LOG_MESSAGE("Succeded to get right return and out parameters :-)");

        } catch (ExceptionTest e) {            
            e.printStackTrace();
            fail();
        }

    }


    class COPIRegistratorThread extends Thread{

        com.acme.TestService.COPITester tester;
        boolean unregister = false;

        public COPIRegistratorThread(com.acme.TestService.COPITester tester){
            this.tester = tester;
        }

        public void run(){
            for(int i=0; i<3; i++)
                tester.register_interceptor("ClientCOPI"+i, COPIType.CLIENT);

            for(int i=0; i<4; i++)
                tester.register_interceptor("ServantCOPI"+i, COPIType.SERVANT);

            for(int i=0; i<1; i++)
                tester.register_interceptor("ServerCOPI"+i, COPIType.SERVER);

            for(int i=0; i<2; i++)
                tester.register_interceptor("StubCOPI"+i, COPIType.STUB);

            try {
                // waiting for the unregistration notification
                synchronized(this){
                while(!unregister){
                    System.out.println("@@@@@@@@@@@@@@ waiting for COPIs Unregistration request ...");
                    this.wait();
                    System.out.println("@@@@@@@@@@@@@@ COPIs Unregistration will start ...");
                }
                }

                for (int i = 0; i < 4; i++)
                    tester.unregister_interceptor("ServantCOPI" + i);

                Thread.sleep(100);

                for (int i = 0; i < 2; i++)
                    tester.unregister_interceptor("StubCOPI" + i);

                Thread.sleep(100);

                for (int i = 0; i < 3; i++)
                    tester.unregister_interceptor("ClientCOPI" + i);

                Thread.sleep(100);

                for (int i = 0; i < 1; i++)
                    tester.unregister_interceptor("ServerCOPI" + i);
            } catch (InvalidRegistration ir) {
                ir.printStackTrace();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        public void unregister(){
            unregister = true;
        }

    }
}

