/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.testccmcopi;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;
import org.omg.CORBA.Any;
import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.LocalObject;
import org.omg.Components.Cookie;
import org.omg.Components.ContainerPortableInterceptor.ContainerServantRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.ContainerStubRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.Components.ContainerPortableInterceptor.ServantContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptor;
import org.omg.Dynamic.Parameter;
import org.omg.PortableInterceptor.ForwardRequest;

import cdmw.ccm.ccmcopi.COPIServiceInit;
import cdmw.testutils.CORBATestManager;

import com.acme.TestComponents.ServerInfoHelper;


public class EventModifierCOPITestImpl extends LocalObject implements StubContainerInterceptor, ServantContainerInterceptor {
    
    private String COPIname;    
    static org.omg.CORBA.ORB orb;         
    static int testCaseId = -1;
    private com.acme.TestComponents.CCM_Server_Context ServerContext;
    
    @Test
    public void doTests() {

        System.out.println("*************************************************************\n"+
                "       EventModifierCOPITestImpl.class Test\n"+
        "*************************************************************");
        try{
            orb = CORBATestManager.instance().getORB();

            com.thalesgroup.Cdmw_TestService.CCM_LoggerService_ExtensionContext QoSEnablerContext = 
                (com.thalesgroup.Cdmw_TestService.CCM_LoggerService_ExtensionContext) CORBATestManager
                .instance().getLocalProperty("LoggerServiceContext");
            assertTrue(QoSEnablerContext != null);

            com.acme.TestComponents.CCM_Client_Context ClientContext = (com.acme.TestComponents.CCM_Client_Context)CORBATestManager.instance().
            getLocalProperty("ClientSessionContext");
            assertTrue(ClientContext != null);

            ServerContext = (com.acme.TestComponents.CCM_Server_Context)CORBATestManager.instance().
            getLocalProperty("ServerSessionContext");
            assertTrue(ServerContext != null);                        

            EventModifierCOPITestImpl copi = new EventModifierCOPITestImpl();
            
            Cookie stubCOPICookie = QoSEnablerContext.get_stub_interceptor_registration().register_stub_interceptor(copi);
            assertTrue(stubCOPICookie != null);

            Cookie servantCOPICookie = QoSEnablerContext.get_servant_interceptor_registration().register_servant_interceptor(copi);        
            assertTrue(servantCOPICookie != null);
                        
            
            System.out.println("************************************************\n"+
                    " Testing Event Data modification by a SERVANT COPI\n"+
            "***********************************************");
            

            // Event data should not have been modified
            
            
            COPIServiceInit.disableReadWriteRequests();        
            assertTrue(COPIServiceInit.isCOPIRequestsReadWrite() == false);
            
            test_modify_event(2224);
            assertTrue(checkReceivedEvent(2224));            
            
            
            COPIServiceInit.enableReadWriteRequests();        
            assertTrue(COPIServiceInit.isCOPIRequestsReadWrite());                        

            // test mod via servant_receive_request
            // Event data should have been modified by COPI        
            testCaseId = 0;                        
            test_modify_event(2224);
            assertTrue(checkReceivedEvent(2323));

            // test mod via stub_send_request
            // Event data should have been modified by COPI
            testCaseId = 1;
            test_modify_event(2224);
            assertTrue(checkReceivedEvent(55));
            
            // test mod via servant_send_reply
            // At the level of stub_receive_reply a modification should be observed                                                    
            testCaseId = 2;
            test_modify_event(2224);
            assertTrue(checkReceivedEvent(2224)); // need to sync here, next test case must not be affected! 
                                                        
            // tracer bullet / increment data on each interception, check calling order
            testCaseId = 3; 
            test_modify_event(0);
            assertTrue(checkReceivedEvent(2));
                                                        
            
            QoSEnablerContext.get_stub_interceptor_registration().unregister_stub_interceptor(stubCOPICookie);
            QoSEnablerContext.get_servant_interceptor_registration().unregister_servant_interceptor(servantCOPICookie);

        }
        catch(Exception e){
            e.printStackTrace();
            fail();
        }

        System.out.println("*************************************************************\n"+
                "       End of EventModifierCOPITestImpl.class Test\n"+
        "*************************************************************");
    }

    // The receiver of an event, i.e. the even sink, should have set the event data on event reception
    private boolean checkReceivedEvent(int data) throws InterruptedException {
        // ClientComponent is supposed to set the event data
        for(int i = 0; i<10; ++i) {
            int _data = EventDataExchanger.instance().getData();            
            if(_data == data) {
                return true;
            }
            System.out.println("Waiting for event sink to set event data value of " + data + " ...");
            Thread.sleep(500);
        }
        
        System.out.println("Giving up waiting for event sink to set event data value of " + data + " ...");
        return false;
    }



    public void test_modify_event(int data) {
        ServerInfoImpl ev = new ServerInfoImpl();
        ev.data = data;
        System.out.println("pushing " + ev.data);
        
        this.ServerContext.push_info(ev);                
    }

    
    // modifies ContainerServantRequestInfo object
    protected void doModifyEventData(ContainerServantRequestInfo info, int newValue) {

        if( "push".equals(info.operation())) {
            boolean modified = false;
            Parameter[] params = info.arguments();

            if(params == null || params.length != 1) 
                throw new RuntimeException(); 
            System.out.println("Modifying event data...");

            if(ServerInfoHelper.type().equal(params[0].argument.type())) {
                Any any = orb.create_any();
                ServerInfoImpl serverInfoImpl = new ServerInfoImpl();
                serverInfoImpl.data = newValue;
                ServerInfoHelper.insert(any, serverInfoImpl);

                params[0].argument= any;
                modified = true;
            } else { throw new RuntimeException(); } 


            if(modified == true) {            
                info.arguments(params);
                System.out.println("Successfully modified event data...");
                
            } else {
                throw new RuntimeException();
            }
        } else {
            throw new RuntimeException();
        }    
    }
    
    // modifies ContainerStubRequestInfo object
    // NOTE: there is a certain asymmetry between ContainerStubRequestInfo and ContainerServantRequestInfo
    //       ContainerStubRequestInfo has the actual event any nested in another any
    protected void doModifyEventData(ContainerStubRequestInfo info, int newValue) {

        if( "push".equals(info.operation())) {
            boolean modified = false;
            Parameter[] params = info.arguments();

            if(params == null || params.length != 1) 
                throw new RuntimeException(); 
            System.out.println("Modifying event data...");


            Any hull = params[0].argument.extract_any();

            if(ServerInfoHelper.type().equal(hull.type())) {
                Any any = orb.create_any();
                ServerInfoImpl serverInfoImpl = new ServerInfoImpl();
                serverInfoImpl.data = newValue;
                ServerInfoHelper.insert(any, serverInfoImpl);

                Any _hull = orb.create_any();
                _hull.insert_any(any);
                params[0].argument= _hull;
                modified = true;
            } else { throw new RuntimeException(); } 


            if(modified == true) {            
                info.arguments(params);
                System.out.println("Successfully modified event data...");
                
            } else {
                throw new RuntimeException();
            }
        } else {
            throw new RuntimeException();
        }        
    }
    
        
    public int getEventData(ContainerServantRequestInfo info) {
        try {
            return ServerInfoHelper.extract(info.arguments()[0].argument).data;
        } catch (Exception e) {
            e.printStackTrace();
            fail();
        }
        
        throw new RuntimeException();
    }
    
    public int getEventData(ContainerStubRequestInfo info) {
        try {
            return ServerInfoHelper.extract(info.arguments()[0].argument.extract_any()).data;
        } catch (Exception e) {
            e.printStackTrace();
            
        }
        
        throw new RuntimeException();        
    }
    


    public void servant_receive_request(ContainerServantRequestInfo info,
            BooleanHolder proceed_call) throws ForwardRequest {        
        
        try {
            // ReadWriteRequests is disabled via COPIServiceInit.disableReadWriteRequests()    
            if(testCaseId == -1) {                 
                throw new RuntimeException();
            } else if(testCaseId == 0) {
                // set event data to 2323
                doModifyEventData(info, 2323);            
            } else if (testCaseId == 3) {
                int data = getEventData(info);
                if(data != 1)
                    throw new RuntimeException();
                doModifyEventData(info, ++data);            
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }                    
    }

    public void servant_send_exception(ContainerServantRequestInfo info,
            BooleanHolder proceed_call) throws ForwardRequest {        
        throw new RuntimeException();
        
    }

    public void servant_send_other(ContainerServantRequestInfo info)
            throws ForwardRequest {
        throw new RuntimeException();        
    }

    public void servant_send_reply(ContainerServantRequestInfo info,
            BooleanHolder proceed_call) {

        try {
            // ReadWriteRequests is disabled via COPIServiceInit.disableReadWriteRequests()    
            if(testCaseId == -1) { 
                throw new RuntimeException(); 
            } else if(testCaseId == 2) {
                doModifyEventData(info, 8);
            } else if (testCaseId == 3) {
                int data = getEventData(info);
                if(data != 2) throw new RuntimeException();                
                doModifyEventData(info, ++data);
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }

    }

    public void destroy() {
        // TODO Auto-generated method stub
        
    }

    public String name() {
        // TODO Auto-generated method stub
        return null;
    }

    public short priority() {
        // TODO Auto-generated method stub
        return 0;
    }

    public IntegrationPoint registration_info() {
        // TODO Auto-generated method stub
        return null;
    }

    public void set_slot_id(int slot_id) {                
    }

    public void stub_receive_exception(ContainerStubRequestInfo info,
            BooleanHolder proceed_call) throws ForwardRequest {
        throw new RuntimeException();
        
    }

    public void stub_receive_other(ContainerStubRequestInfo info)
            throws ForwardRequest {
        throw new RuntimeException();
        
    }

    public void stub_receive_reply(ContainerStubRequestInfo info,
            BooleanHolder proceed_call) {
        
        int value = -1;
        
        Parameter params[] = info.arguments();
        if(params != null) {
            for(int i=0; i<params.length; i++) {
                Any hull = params[i].argument.extract_any();
                value = ServerInfoHelper.extract(hull).data;
                                
            }
        } 
                
        if(testCaseId == -1) {
            throw new RuntimeException();
        } else if(testCaseId == 0) { 
            if(value != 2224) throw new RuntimeException();        
            // event modified by the server copi, but after treply was sent
        } else if(testCaseId == 1) {
            if(value != 55) throw new RuntimeException();        
            // event modified by stub copi, i.e. server will send reply with the modification                                      
        } else if(testCaseId == 2) {
            // event modified by the server copi, but after treply was sent            
            if(value != 2224) throw new RuntimeException();
        } else if (testCaseId == 3) {
            int data = getEventData(info);
            if(data != 1) throw new RuntimeException();;
            doModifyEventData(info, ++data);                    
        } else {
            throw new RuntimeException();
        }
    }
    
    
    public void stub_send_request(ContainerStubRequestInfo info,
            BooleanHolder proceed_call) throws ForwardRequest {

        //ReadWriteRequests is disabled via COPIServiceInit.disableReadWriteRequests()    
        if(testCaseId == -1) { 
            throw new RuntimeException(); 
        } 
        else if(testCaseId == 1) {
            doModifyEventData(info, 55);             
        } else if (testCaseId == 3) {
            int data = getEventData(info);
            if(data != 0) throw new RuntimeException();            
            doModifyEventData(info, ++data);                    
        }
    }
    

}
