/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import static org.junit.Assert.assertTrue;

import org.omg.CORBA.Any;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.Components.ConfigValue;
import org.omg.Components.ConfigValuesHolder;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;

import cdmw.ccm.container.ConfigValueFactoryImpl;
import cdmw.ccm.container.ConfigValueImpl;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;

import com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor;
/**
 *
 */
public class LifeCycleInterceptorTest extends org.omg.CORBA.LocalObject 
                        implements ComponentLifeCycleInterceptor {

 
    private String name ="";
    private short priority;
    private ORB orb; 
    
    public LifeCycleInterceptorTest(String name){
        this.name = name;
        
        try {
            orb = CORBATestManager.instance().getORB();
        } catch (InvalidObject e) {
            orb = ORB.init();
        }
    }
    
    public LifeCycleInterceptorTest(String name, short priority){
        this(name);
        this.priority = priority;
    }
    
    public void after_component_activate(String compId) {
        assertTrue(compId!=null);
        System.out.println(name + ": after_component_activate of component "+ compId +"called ...");        

    }


    public void after_component_connect(String compId) {
        assertTrue(compId!=null);
        System.out.println(name + ": after_component_connect of component "+ compId +"called ...");

    }

 
    public void after_component_create(String compId, Object ref) {
        assertTrue(compId!=null && ref!=null);
        System.out.println(name + ": after_component_create of component "+ compId +"called ...");

    }

 
    public void after_component_passivate(String compId) {
        assertTrue(compId!=null);
        System.out.println(name + ": after_component_passivate of component "+ compId +"called ...");

    }

 
    public void after_component_remove(String compId) {
        assertTrue(compId!=null);
        System.out.println(name + ": after_component_remove of component "+ compId +"called ...");

    }

  
    public void before_component_activate(String compId) {
        assertTrue(compId!=null);
        System.out.println(name + ": before_component_activate of component "+ compId +"called ...");

    }

 
    public void before_component_connect(String compId, String arg1, Object arg2) {
        assertTrue(compId!=null);
        System.out.println(name + ": before_component_connect of component "+ compId +"called ...");

    }

 
    public void before_component_create(String compId, ConfigValuesHolder configValues) {
        assertTrue(compId!=null && configValues!=null);
        System.out.println(name + ": before_component_create of component "+ compId +"called ...");
        System.out.println("Testing the modification of the Client component config values ...");
        if(this.priority==Short.MAX_VALUE && compId.indexOf("NAME1") >= 0) {
            ConfigValue[] c_values = configValues.value;
            if(c_values != null){
                for (ConfigValue c : c_values){
                    if(com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value.equals(c.name)) {
                        System.out.println("Modifying the Client Component NAME ...");
                        Any name=orb.create_any();
                        name.insert_string("CLIENT_COMPONENT_TO_REMOVE");
                        c.value = name;
                        break;
                    }
                }
            }
        }
        
        if(this.priority==Short.MAX_VALUE && compId.indexOf("SERVER") >= 0) {
            ConfigValue[] c_values = configValues.value;
            if(c_values != null){
                ConfigValue[] new_cv = new ConfigValue[c_values.length + 1];
                Any val = orb.create_any();
                val.insert_string("");
                ConfigValue c = new ConfigValueImpl("NEW_CONFIG_VALUE_NAME", val);
                new_cv[0] = c;
                System.arraycopy(c_values, 0, new_cv, 1, c_values.length);
                configValues.value = new_cv;
                System.out.println("Adding a new ConfigValue to the Server Component ...");
            }
        }
        

    }

 
    public void before_component_passivate(String compId) {
        assertTrue(compId!=null);
        System.out.println(name + ": before_component_passivate of component "+ compId +"called ...");

    }

 
    public void before_component_remove(String compId) {
        assertTrue(compId!=null && compId.indexOf("CLIENT_COMPONENT_TO_REMOVE")>=0);
        System.out.println(name + ": before_component_remove of component "+ compId +"called ...");

    }

 
    public void destroy() {
        // TODO Auto-generated method stub

    }

 
    public String name() {
        return name;
    }

  
    public short priority() {
        return this.priority;
    }

 
    public IntegrationPoint registration_info() {
        return null;
    }

 
    public void set_slot_id(int s_id) {

    }


}
