/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import java.text.SimpleDateFormat;
import java.util.Date;

import junit.framework.Assert;

import org.omg.CORBA.LocalObject;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.ContainerClientRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.PortableInterceptor.ForwardRequest;



/**
 *
 */
public class LoggerClientCOPIImpl extends LocalObject
            implements ClientContainerInterceptor {

    int slotId;
    
    private String COPIname;
    private short priority;
    private IntegrationPoint ip;
    private COPITesterImpl tester;
    
    
    public LoggerClientCOPIImpl(String name){
        this.COPIname = name;
    }

    public LoggerClientCOPIImpl(String name, COPITesterImpl tester){
        this.COPIname = name;
        this.tester = tester;
    }
    
    public LoggerClientCOPIImpl(String name, short p, String port, String operation){
        this.COPIname = name;
        this.priority = p;
        this.ip = new IntegrationPoint(port, operation);
        
    }
 
    public void send_request(ContainerClientRequestInfo info)
            throws ForwardRequest {
        // testing the selotId setting
        Assert.assertTrue(this.slotId!=0);
        String origin_id = info.origin_id()!=null ? new String(info.origin_id()):null;
        String target_id = info.target_id()!=null ? new String(info.target_id()):null;
        
        System.out.println(getHeader()+"invoking send_request of request "+info.request_info().request_id() +" ...");
        String msg ="The operation " +info.request_info().operation()+
                "will be sent from component " + origin_id +
                "to component "+target_id ;
        System.out.println(getHeader()+msg);
        
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("send_request", COPIname, timestamp, info.request_info().operation());
    }

 
    public String name() {
        return COPIname;
    }

    public void receive_exception(ContainerClientRequestInfo info)
            throws ForwardRequest {
        System.out.println(getHeader()+"invoking receive_exception of request "+info.request_info().request_id() +" ...");
        
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("receive_exception", COPIname, timestamp, info.request_info().operation());
    }

    public void receive_other(ContainerClientRequestInfo info)
            throws ForwardRequest {
        System.out.println(getHeader()+"invoking receive_other of request "+info.request_info().request_id() +" ...");
        
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("receive_other", COPIname, timestamp, info.request_info().operation());
    }

    public void receive_reply(ContainerClientRequestInfo info) {
        String target_id = info.target_id()!=null ? new String(info.target_id()):null;
        
        System.out.println(getHeader()+"invoking receive_reply of request "+info.request_info().request_id() +" ...");
        String msg ="The operation " +info.request_info().operation()+
            "has received a reply from component " + target_id;
        System.out.println(getHeader()+msg);
       
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("receive_reply", COPIname, timestamp, info.request_info().operation());

    }

    public void send_poll(ContainerClientRequestInfo info) {
        System.out.println(getHeader()+"invoking send_poll of request "+info.request_info().request_id() +" ...");
          
    }

    public void destroy() {
        // TODO Auto-generated method stub
        
    }

    public void set_slot_id(int slot_id) {
        this.slotId = slot_id;
    }
    
    private String getHeader(){
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm:ss");
        
        return ("@B@A@S@I@C@@@C@O@P@I@ "+sdf.format(new Date())+"\t"+name()+"\t");
    }

    public short priority() {
        return this.priority;
    }

    public IntegrationPoint registration_info() {
        return this.ip;
    }
    
    public String toString(){
       String s = name() +"-"+priority+"-"+ip.port+"/"+ip.operation; 
       return s;
    }

}
