/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import java.text.SimpleDateFormat;
import java.util.Date;

import junit.framework.Assert;

import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.LocalObject;
import org.omg.Components.ContainerPortableInterceptor.ContainerServantRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.Components.ContainerPortableInterceptor.ServantContainerInterceptor;
import org.omg.Dynamic.Parameter;
import org.omg.PortableInterceptor.ForwardRequest;

import cdmw.ccm.ccmcopi.COPIServiceInit;

/**
 *
 */
public class LoggerServantCOPIImpl extends LocalObject implements
        ServantContainerInterceptor {

    int slotId;
    
    private String COPIname;
    
    
    
    public LoggerServantCOPIImpl(String name){
        this.COPIname = name;
    }
    
    
    private COPITesterImpl tester;
    public LoggerServantCOPIImpl(String name, COPITesterImpl tester){
        this.COPIname = name;
        this.tester = tester;
    }
    
    
    public void servant_receive_request(ContainerServantRequestInfo info,
            BooleanHolder proceed_call) throws ForwardRequest {
        Assert.assertTrue(this.slotId!=0);
        String origin_id = (info.origin_id()!=null? new String(info.origin_id()):""),
        target_id = (info.target_id()!=null? new String(info.target_id()):"");
        
        System.out.println(getHeader()+"servant_receive_request called ...");
        String msg ="The operation " +info.operation()+
                " has been received by component " + target_id +
                " from component "+ origin_id;
        System.out.println(getHeader()+msg);
        
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("servant_receive_request", COPIname, timestamp, info.operation());

    }
 
    public void servant_send_exception(ContainerServantRequestInfo info,
            BooleanHolder proceed_call) throws ForwardRequest {
        System.out.println(getHeader()+"servant_send_exception called ...");
        
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("servant_send_exception", COPIname, timestamp, info.operation());

    }

    public void servant_send_other(ContainerServantRequestInfo info)
            throws ForwardRequest {
        System.out.println(getHeader()+"servant_send_other called ...");
        
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("servant_send_other", COPIname, timestamp, info.operation());

    }
 
    public void servant_send_reply(ContainerServantRequestInfo info,
            BooleanHolder proceed_call) {
        System.out.println(getHeader()+"servant_send_reply called ...");
        
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("servant_send_reply", COPIname, timestamp, info.operation());

    }
 
    public void destroy() {

    }

    public String name() {
        return COPIname;
    }
 
    public void set_slot_id(int slot_id) {
       this.slotId = slot_id;

    }
    
    private String getHeader(){
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm:ss");
        
        return ("####E#X#T#E#N#D#E#D#### "+sdf.format(new Date())+"\t"+name()+"\t");
    }


    public short priority() {
        // TODO Auto-generated method stub
        return 0;
    }


    public IntegrationPoint registration_info() {
        // TODO Auto-generated method stub
        return null;
    }
}
