/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import static org.junit.Assert.assertTrue;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.junit.Test;
import org.omg.CORBA.LocalObject;
import org.omg.Components.CCMException;
import org.omg.Components.Cookie;
import org.omg.Components.ExtensionComponent;
import org.omg.Components.ExtensionContext;
import org.omg.Components.SessionComponent;
import org.omg.Components.SessionContext;
import org.omg.Components.ContainerPortableInterceptor.InvalidRegistration;

import cdmw.testutils.CORBATestManager;

import com.acme.TestService.CCM_Logger;
import com.acme.TestService.CCM_LoggerService;
import com.acme.TestService.COPITester;
import com.acme.TestService.COPIType;
import com.acme.TestService.LoggerHelper;
import com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistration;
import com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistrationHelper;



/**
 *
 */
public class LoggerServiceComponentImpl extends LocalObject implements
        SessionComponent, ExtensionComponent, CCM_LoggerService, CCM_Logger{
    
    com.thalesgroup.Cdmw_TestService.CCM_LoggerService_ExtensionContext context;
    static COPITester copiTester;

 
    public void ccm_activate() throws CCMException {
        if (copiTester == null)
            copiTester = new COPITesterImpl(context);

        this.LOG_MESSAGE("installing a LOGGING_SERVICE service reference ...");
        context.install_service_reference("LOGGING_SERVICE", LoggerHelper
                .narrow(this));

        this.LOG_MESSAGE("installing the COPITester service reference ...");
        // this service allows to check the correct COPIs states change
        context.install_service_reference("COPITester_SERVICE", copiTester);

        // registering ComponentlifecycleInterceptor
        this.LOG_MESSAGE("Getting the LifeCycleInterceptorRegistrationService reference  ...");
        org.omg.CORBA.Object obj = context
                .resolve_service_reference(cdmw.ccm.container.LifeCycleInterceptorRegistration.LifeCycleInterceptorRegistrationServiceId);
        assertTrue(obj != null);
        LifeCycleInterceptorRegistration registration = LifeCycleInterceptorRegistrationHelper
                .narrow(obj);
        assertTrue(registration != null);

        this.LOG_MESSAGE("Registering Component LifecycleInterceptor ...");
        LifeCycleInterceptorTest clci = null, clci_to_remove = null;
        int nb = 5, idx_to_rem = 2;
        Cookie[] cookies = new Cookie[nb];
        
        cookies[0] = registration
        .register_component_lifecycle_interceptor(
                new LifeCycleInterceptorTest("ComponentLifecycleInterceptor0", Short.MAX_VALUE));
        for (int i = 1; i < nb; i++) {
            clci = new LifeCycleInterceptorTest("ComponentLifecycleInterceptor" + i);
            cookies[i] = registration
                    .register_component_lifecycle_interceptor(clci);
            assertTrue(cookies[i] != null);
            if (i == idx_to_rem)
                clci_to_remove = clci;
        }
        try {
            this.LOG_MESSAGE("Unregistering Component LifecycleInterceptor ...");
            clci = (LifeCycleInterceptorTest) registration
                    .unregister_component_lifecycle_interceptor(cookies[idx_to_rem]);
            assertTrue(clci.equals(clci_to_remove));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }


    public void ccm_passivate() throws CCMException {
        // TODO Auto-generated method stub

    }


    public void ccm_remove() throws CCMException {
        // TODO Auto-generated method stub

    }


    public void set_session_context(SessionContext ctx) throws CCMException {
        // TODO Auto-generated method stub
    }


    public void set_extension_context(ExtensionContext ctx) throws CCMException {
        this.context = com.thalesgroup.Cdmw_TestService.CCM_LoggerService_ExtensionContextHelper.narrow(ctx);
        CORBATestManager.instance().setLocalProperty ("LoggerServiceContext", this.context);
    }

    public void LOG_MESSAGE(String msg) {
     String when = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(new Date());
        String message ="-----------------------\n" +
                        "received    : "+ when +"\n"+
                        "message : "+msg +"\n"+
                        "-----------------------\n";
        System.out.println(message);   
    }
    
     
    Cookie[] stubCOPICookie;
    Cookie[] servantCOPICookie;
    
    
    
    @Test
    public void doTests() {
        System.out.println("*************************************************************\n"+
                "       LoggerServiceComponentImpl.class Test\n"+
                "*************************************************************");
        
        com.thalesgroup.Cdmw_TestService.CCM_LoggerService_ExtensionContext context = null;

        context = (com.thalesgroup.Cdmw_TestService.CCM_LoggerService_ExtensionContext) CORBATestManager
                .instance().getLocalProperty("LoggerServiceContext");
        assertTrue(context != null);

        String to_register = (String)CORBATestManager.instance().getLocalProperty(
                "REGISTER_COPI_POOL");
        
        if (!"undo".equals(to_register)) {
                stubCOPICookie = new Cookie[4];
                servantCOPICookie = new Cookie[4];
                
                for (int i = 0; i < 4; i++) {
                    copiTester.register_interceptor("StubCOPI"+i, COPIType.STUB);
                    copiTester.register_interceptor("ServantCOPI"+i, COPIType.SERVANT);
                }
              
                CORBATestManager.instance().setLocalProperty("REGISTER_COPI_POOL","undo");
            
        } else {
            try {
                stubCOPICookie = (Cookie[])CORBATestManager.instance().getLocalProperty("STUB_COPI_COOKIE_ARRAY");
                servantCOPICookie = (Cookie[])CORBATestManager.instance().getLocalProperty("SERVANT_COPI_COOKIE_ARRAY");
                
                for (int i = 0; i < 4; i++) {
                    copiTester.unregister_interceptor("StubCOPI"+i);
                    copiTester.unregister_interceptor("ServantCOPI"+i);
                }
            } catch (InvalidRegistration e) {
                e.printStackTrace();
            }
        }
        System.out.println("*************************************************************\n"+
                "       End of LoggerServiceComponentImpl.class Test\n"+
                "*************************************************************");
    }

    public CCM_Logger get_logger() {
        return this;
    }



}
