/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import static org.junit.Assert.*;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.text.SimpleDateFormat;
import java.util.Date;

import junit.framework.Assert;

import org.junit.Test;
import org.omg.CORBA.Any;
import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.IntHolder;
import org.omg.CORBA.LocalObject;
import org.omg.CORBA.NO_IMPLEMENT;
import org.omg.CORBA.TCKind;
import org.omg.Components.Cookie;
import org.omg.Components.ContainerPortableInterceptor.ContainerRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.ContainerServantRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.ContainerStubRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptor;
import org.omg.Dynamic.Parameter;
import org.omg.PortableInterceptor.ForwardRequest;

import cdmw.ccm.ccmcopi.COPIServiceInit;
import cdmw.testutils.CORBATestManager;

import com.acme.TestComponents.ArithOperator;
import com.acme.TestComponents.ArithOperatorHelper;
import com.acme.TestComponents.ExceptionTest;
import com.acme.TestComponents.ExceptionTestHelper;
import com.acme.TestComponents.FacetTest;
import com.acme.TestService.COPIType;

/**
 *
 */
public class ParamModifierStubCOPITestImpl extends LocalObject implements
        StubContainerInterceptor,
        ParamModifierCOPITest {

    int slotId;
    
    private String COPIname;
    
    static org.omg.CORBA.ORB orb; 
    
    static int testCaseId = -1;
    
    public ParamModifierStubCOPITestImpl(){
        this.COPIname = "UnamedCOPI"; 
    }
    
    
    public ParamModifierStubCOPITestImpl(String name){
        this.COPIname = name;
    }
    
    private COPITesterImpl tester;
    public ParamModifierStubCOPITestImpl(String name, COPITesterImpl tester){
        this.COPIname = name;
        this.tester = tester;
    }
    
    
    public void stub_receive_exception(ContainerStubRequestInfo info,
            BooleanHolder proceed_call) throws ForwardRequest {
        System.out.println(getHeader()+"invoking stub_receive_exception of request ...");
        
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("stub_receive_exception", COPIname, timestamp, info.operation());
        
        switch(testCaseId){
        case MODIFY_EXCEPTION: modify_exception(info, proceed_call);
            break;
        }
    }

 
    public void stub_receive_other(ContainerStubRequestInfo info)
            throws ForwardRequest {
        System.out.println(getHeader()+"invoking stub_receive_other of request ...");
        
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("stub_receive_other", COPIname, timestamp, info.operation());
    }

 
    public void stub_receive_reply(ContainerStubRequestInfo info,
            BooleanHolder proceed_call) {
        System.out.println(getHeader()+"invoking stub_receive_reply of request ...");
       
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("stub_receive_reply", COPIname, timestamp, info.operation());
        
        switch(testCaseId){
        case MODIFY_OUT_RET: modify_out_return_param(info, proceed_call);
            break;
        }
    }

 
    public void stub_send_request(ContainerStubRequestInfo info,
            BooleanHolder proceed_call) throws ForwardRequest {
        Assert.assertTrue(this.slotId!=0);
        
        String origin_id = (info.origin_id()!=null? new String(info.origin_id()):""),
        target_id = (info.target_id()!=null? new String(info.target_id()):"");

        System.out.println(getHeader()+"stub_send_request called ...");
        String msg ="The operation " +info.operation()+
                " will be sent from component " + origin_id +
                " to component " + target_id;
        System.out.println(getHeader()+msg); 
        
        Date timestamp = new Date(System.currentTimeMillis());
        if(tester != null) tester.notify_operation("stub_send_request", COPIname, timestamp, info.operation());
    
        switch(testCaseId){
        case MODIFY_IN_PARAM: modify_in_param(info, proceed_call);
            break;
        case DO_NOT_PROCEED_CALL : do_not_proceed_call(info, proceed_call);
            break;
        case THROW_EXCEPTION : throw_exception(info, proceed_call);
            break;
        }
    }

 
    public void destroy() {
        // TODO Auto-generated method stub

    }

 
    public String name() {
        return this.COPIname;
    }
    
    public short priority() {
        // TODO Auto-generated method stub
        return 0;
    }


    public IntegrationPoint registration_info() {
        // TODO Auto-generated method stub
        return null;
    }


 
    public void set_slot_id(int slot_id) {
        this.slotId = slot_id;
    }
    
    private String getHeader(){
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm:ss");
        
        return ("####E#X#T#E#N#D#E#D#### "+sdf.format(new Date())+"\t"+name()+"\t");
    }
    
    
    private com.acme.TestService.Logger logger;
    private FacetTest facet;
    
    
    @Test
    public void doTests() {
        
        System.out.println("*************************************************************\n"+
                "       ParamModifierStubCOPIImpl.class Test\n"+
                "*************************************************************");
        COPIServiceInit.enableReadWriteRequests();
        assertTrue(COPIServiceInit.isCOPIRequestsReadWrite());
        try{
        orb = CORBATestManager.instance().getORB();
        
        com.thalesgroup.Cdmw_TestService.CCM_LoggerService_ExtensionContext QoSEnablerContext = 
            (com.thalesgroup.Cdmw_TestService.CCM_LoggerService_ExtensionContext) CORBATestManager
                .instance().getLocalProperty("LoggerServiceContext");
        assertTrue(QoSEnablerContext != null);
        
        com.acme.TestComponents.CCM_Client_Context ClientContext = (com.acme.TestComponents.CCM_Client_Context)CORBATestManager.instance().
        getLocalProperty("ClientSessionContext");
        assertTrue(ClientContext != null);
        
        com.acme.TestComponents.CCM_Server_Context ServerContext = (com.acme.TestComponents.CCM_Server_Context)CORBATestManager.instance().
        getLocalProperty("ServerSessionContext");
        assertTrue(ServerContext != null);
        
        Object obj = ClientContext.resolve_service_reference("COPITester_SERVICE");
        com.acme.TestService.COPITester tester = com.acme.TestService.COPITesterHelper.narrow(obj);
        assertNotNull(tester);
        
        obj = ClientContext.resolve_service_reference("LOGGING_SERVICE");
        logger = com.acme.TestService.LoggerHelper.narrow(obj);
        assertTrue(logger != null);
        
        
        facet = ClientContext.get_connection_to_server();
                
        ParamModifierStubCOPITestImpl copi = new ParamModifierStubCOPITestImpl("ParamModifierStubCOPI", (COPITesterImpl)tester);
        
        Cookie stubCOPICookie = QoSEnablerContext.get_stub_interceptor_registration().register_stub_interceptor(copi);
        assertTrue(stubCOPICookie != null);
        tester.register_interceptor("Dummy COPI", COPIType.STUB); //registering a COPI after the modifier one
        
        tester.start_listening();
        
        System.out.println("*******************************************\n"+
                " Testing IN param modification by a STUB COPI\n"+
                "*******************************************");
        testCaseId++;
        test_modify_in_param();
        
        System.out.println("************************************************\n"+
                " Testing OUT and RETURN param modification by a STUB COPI\n"+
                "***********************************************");
        testCaseId++;
        test_modify_out_return_param();
        
        System.out.println("************************************************\n"+
                " Testing throwing an exception by a STUB COPI\n"+
        "***********************************************");
        testCaseId++;
        test_throw_exception();
        
        System.out.println("************************************************\n"+
                " Testing Exception modification by a STUB COPI\n"+
        "***********************************************");
        testCaseId++;
        test_modify_exception();
        
        System.out.println("************************************************\n"+
                " Testing Call blocking by a STUB COPI\n"+
        "***********************************************");
        testCaseId++;
        test_do_not_proceed_call();
        
        tester.end_listening();
        tester.test_interception_stacks();
        
        QoSEnablerContext.get_stub_interceptor_registration().unregister_stub_interceptor(stubCOPICookie);
        tester.unregister_interceptor("Dummy COPI");
        
        }
        catch(Exception e){
            e.printStackTrace();
            fail();
        }
        
        
        System.out.println("*************************************************************\n"+
                "       End of ParamModifierStubCOPIImpl.class Test\n"+
                "*************************************************************");
    }

    
    public void test_modify_in_param() {
        try {
            IntHolder rem = new IntHolder();
            int res = facet.makeOperation(ArithOperator.DIV, 100, 3, rem);
            logger.LOG_MESSAGE("100 DIV 3 = "+ res +", remainder = "+ rem.value);
            // the in params has been changed by the COPI
            assertTrue(res==1003 && rem.value==0);
            logger.LOG_MESSAGE("Succeded to get right return and out parameters :-)");
            
        } catch (ExceptionTest e) {
            e.printStackTrace();
        }

        
    }
    public void modify_in_param(ContainerRequestInfo info, BooleanHolder proceed_call) {
        
        ContainerStubRequestInfo Stub_info = (ContainerStubRequestInfo)info;
        if( "makeOperation".equals(Stub_info.operation())){
            System.out.println(getHeader()+"Modifying the Operator and the first operand");
            Parameter[] params = Stub_info.arguments();
            int i = 0, found = 0;
            while(i<params.length && found!=2 ){
                if(ArithOperatorHelper.type().equal(params[i].argument.type())){
                    Any any = orb.create_any();
                    ArithOperatorHelper.insert(any, ArithOperator.ADD);
                    params[i].argument= any;
                    found++;
                }
                if(TCKind.tk_long.equals(params[i].argument.type().kind())){
                    Any any = orb.create_any();
                    any.insert_long(1000);
                    params[i].argument = any;
                    found++;
                }
                i++;
            }
            Stub_info.arguments(params);
            if(found==2) System.out.println(getHeader() + "Succeeded to modify IN params of operation makeOperation");
        }
        
    }
    

    public void modify_out_return_param(ContainerRequestInfo info, BooleanHolder proceed_call) {
        
        ContainerStubRequestInfo Stub_info = (ContainerStubRequestInfo)info;
        if("makeOperation".equals(Stub_info.operation())){
            System.out.println(getHeader()+"Modifying the user return value");
            org.omg.CORBA.Any any = Stub_info.result();
            any.insert_long(197556);
            Stub_info.result(any);
        }
        
    }
    public void test_modify_out_return_param() {
        try {
            IntHolder rem = new IntHolder();
            int res = facet.makeOperation(ArithOperator.DIV, 100, 3, rem);
            logger.LOG_MESSAGE("100 DIV 3 = "+ res +", remainder = "+ rem.value);
            assertTrue(res==197556 && rem.value==1);
            logger.LOG_MESSAGE("Succeded to get right return and out parameters :-)");
            
        } catch (ExceptionTest e) {
            e.printStackTrace();
        }
        
    }
    

    public void throw_exception(ContainerRequestInfo info, BooleanHolder proceed_call) {
        ContainerStubRequestInfo Stub_info = (ContainerStubRequestInfo)info;
        if("makeOperation".equals(Stub_info.operation())){
            org.omg.CORBA.Any a = orb.create_any();
            org.omg.CORBA.NO_IMPLEMENT ex = new org.omg.CORBA.NO_IMPLEMENT("The Stub COPI has thrown this exception");
            org.omg.CORBA.NO_IMPLEMENTHelper.insert(a, ex);
            Stub_info.the_exception(a);
            throw(ex);
        }
        
    }
    public void test_throw_exception() {
        try{
            IntHolder rem = new IntHolder();
            int res = facet.makeOperation(ArithOperator.DIV, 100, 3, rem);
            
        }catch(NO_IMPLEMENT ni){
            assertTrue(true);
            logger.LOG_MESSAGE("Succeded to get the right exception : "+ ni.getMessage());
            ni.printStackTrace();
        } catch (ExceptionTest e) {
            assertTrue(false);
        }
        
    }
    
    
    public void modify_exception(ContainerRequestInfo info, BooleanHolder proceed_call) {
        ContainerStubRequestInfo stub_info = (ContainerStubRequestInfo)info;
        if("raiseException".equals(stub_info.operation())){
            org.omg.CORBA.Any a = stub_info.the_exception();
            assertTrue(ExceptionTestHelper.type().equal(a.type()));
            ExceptionTest ex = ExceptionTestHelper.extract(a);
            //modifying the exception reason
            ex.reason = "The stub COPI '"+name()+"' has modified the reason of that exception";
            ExceptionTestHelper.insert(a, ex);
            stub_info.the_exception(a);
        }
        
    }
    public void test_modify_exception() {
        try{
            facet.raiseException(true, "Testing exception raising");
            
        }catch (ExceptionTest e) {
            assertTrue(true);
            logger.LOG_MESSAGE("Succeded to get the right exception : "+ e.reason);
        }
        
    }


    public void do_not_proceed_call(ContainerRequestInfo info, BooleanHolder proceed_call){
        ContainerStubRequestInfo stub_info = (ContainerStubRequestInfo)info;
        if("makeOperation".equals(stub_info.operation())){
            System.out.println(getHeader()+"Replying in stead of the user code");
            org.omg.CORBA.Any any = stub_info.result();
            any.insert_long(197556);
            stub_info.result(any);
            proceed_call.value = false;
        }
        
    }
    public void test_do_not_proceed_call(){
        try {
            IntHolder rem = new IntHolder();
            int res = facet.makeOperation(ArithOperator.DIV, 100, 3, rem);
            logger.LOG_MESSAGE("100 DIV 3 = "+ res +", remainder = "+ rem.value);
            assertTrue(res==197556 && rem.value==0);
            logger.LOG_MESSAGE("Succeded to get right return and out parameters :-)");
            
        } catch (ExceptionTest e) {
            e.printStackTrace();
        }
    }


 


}
