/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testccmcopi;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import org.junit.Assert;
import org.junit.Test;
import org.omg.CORBA.LocalObject;
import org.omg.Components.CCMException;
import org.omg.Components.SessionComponent;
import org.omg.Components.SessionContext;

import cdmw.ccm.ccmcopi.COPIServiceInit;
import cdmw.ossupport.OS;
import cdmw.testutils.CORBATestManager;

import com.acme.HelloModule.CCM_Hello;

import com.acme.TestComponents.Activated;
import com.acme.TestComponents.ArithOperator;
import com.acme.TestComponents.CCM_FacetTest;
import com.acme.TestComponents.CCM_Server;
import com.acme.TestComponents.ExceptionTest;
import com.acme.TestComponents.ServerInfo;

/**
 *
 */
public class ServerComponentImpl extends LocalObject
            implements CCM_Server, CCM_Hello, CCM_FacetTest, SessionComponent {
    
    private static Integer eventsToReceive;

    com.acme.TestComponents.CCM_Server_Context context;
    
    
    public CCM_Hello get_echo() {
        return this;
    }

    public CCM_FacetTest get_for_clients() {
        return this;
    }
    
    public void ccm_activate() throws CCMException {
        System.out.println("ccm_activate called on the Server Component ...");
    }

    public void ccm_passivate() throws CCMException {
        System.out.println("ccm_passivate called on the Server Component ...");
    }

    public void ccm_remove() throws CCMException {
        System.out.println("ccm_remove called on the Server Component ...");
    }

    public void set_session_context(SessionContext ctx) throws CCMException {
        System.out.println("set_session_context called on the Server Component ...");
        this.context=com.acme.TestComponents.CCM_Server_ContextHelper.narrow(ctx);
        CORBATestManager.instance().setLocalProperty ("ServerSessionContext", this.context);
    }

    public void echo(String msg) {
        System.out.println("ServerComponent : request to echo the following message '"+msg+"'");
        
    }

    public void raiseException(boolean _do, String reason) throws ExceptionTest {
        System.out.println("ServerComponent : request to raise an exception");
        if (_do == true) throw new ExceptionTest(reason);
    }

    public int makeOperation(com.acme.TestComponents.ArithOperator oper,
            int operand1, int operand2, org.omg.CORBA.IntHolder remainder)
            throws com.acme.TestComponents.ExceptionTest {
        int result = 0;
        System.out.println("ServerComponent : request to process an operation");
        switch (oper.value()){
        case ArithOperator._ADD: result = operand1 + operand2;
            break;
        case ArithOperator._DIV:
            if(operand2 == 0) throw new ExceptionTest("Division By Zero");
            result = operand1 / operand2;
            remainder.value = operand1 % operand2;
            break;
            
        case ArithOperator._MULTI: result = operand1 * operand2;
            break;
        
        case ArithOperator._SUB: result = operand1 - operand2;
            break;
        default : throw new ExceptionTest("Unsupported operation");
        }
        
        return result;
    }

   synchronized public static void eventReceived(){
       eventsToReceive--;
   }
    
    @Test
    public void doTests() {
        
        System.out.println("*************************************************************\n"+
                "       ServerComponentImpl.class Test\n"+
                "*************************************************************");
        
        com.acme.TestService.Logger logger = null;
        
        try {
            context = (com.acme.TestComponents.CCM_Server_Context)CORBATestManager.instance().
                        getLocalProperty("ServerSessionContext");
            assertTrue(context != null);
            org.omg.CORBA.Object obj = context
                    .resolve_service_reference("LOGGING_SERVICE");
            logger = com.acme.TestService.LoggerHelper.narrow(obj);
            assertTrue(logger != null);
            logger.LOG_MESSAGE("Succeeded to get the LOGGING_SERVICE reference ...");
            
            eventsToReceive = 10;
            COPIServiceInit.enableReadWriteRequests();
            assertTrue(COPIServiceInit.isCOPIRequestsReadWrite());
            System.out.println("****************************************************");
            System.out.println("Testing events interception with enabled RW requests ");
            System.out.println("****************************************************");
            pushEvents();
            
            //waiting for the events to be received by the client to be sure
            // that they are received before the R/W mode is disabled (by the next steps)
            
            synchronized (this){
                while(eventsToReceive != 0)
                    OS.sleep(500);
            }
            
            eventsToReceive = 10;
            
            COPIServiceInit.disableReadWriteRequests();
            assertTrue(!COPIServiceInit.isCOPIRequestsReadWrite());
            System.out.println("****************************************************");
            System.out.println("Testing events interception with disabled RW requests ");
            System.out.println("****************************************************");
            pushEvents();
            
            //waiting for the events to be received before jumping to the next test !
            synchronized (this){
                while(eventsToReceive != 0)
                    OS.sleep(500);
            }
            
            
        }
        catch(Exception e){
            e.printStackTrace();
            Assert.fail();
        }
        
        System.out.println("*************************************************************\n"+
                "       End of ServerComponentImpl.class Test\n"+
                "*************************************************************");
    }
    
    private void pushEvents(){
        
        int eventsToSend = eventsToReceive;
        
        System.out.println("Pushing an Activated event ...");
        Activated activationEvent = new ActivatedImpl();
        context.push_i_am_activated(activationEvent);
        
        System.out.println("Pushing ServerInfo events ...");
        ServerInfo event = new ServerInfoImpl();
        for(int i = 0; i < eventsToSend-1 ; i++){
            event.data = i;
            context.push_info(event);
        } 
        
    }

}
