/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testloggingqosenabler;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;
import org.omg.CORBA.LocalObject;
import org.omg.Components.CCMException;
import org.omg.Components.SessionComponent;
import org.omg.Components.SessionContext;

import cdmw.testutils.CORBATestManager;

import com.acme.TestComponents.CCM_Client;
import com.thalesgroup.CdmwLogging.Logger;
import com.thalesgroup.CdmwLogging.LoggerFactory;
import com.thalesgroup.CdmwLogging.LoggerFactoryHelper;

/**
 *
 */
public class ClientComponentImpl extends LocalObject implements CCM_Client,
        SessionComponent {
    
    com.acme.TestComponents.CCM_Client_Context context;
    
    private static short instanceCounter = 0;


    public void ccm_activate() throws CCMException {
        System.out.println("ccm_activate called on the Client Component ..."); 
    }

    public void ccm_passivate() throws CCMException {
        System.out.println("ccm_passivate called on the Client Component ...");
    }

    public void ccm_remove() throws CCMException {
        System.out.println("ccm_remove called on the Client Component ...");
    }

    public void set_session_context(SessionContext ctx) throws CCMException {        
        this.context=com.acme.TestComponents.CCM_Client_ContextHelper.narrow(ctx);        
        CORBATestManager.instance().setLocalProperty ("ClientSessionContext" + instanceCounter, this.context);
        instanceCounter++;
    }
    
    
    @Test
    public void doTests() {
        
        System.out.println("*************************************************************\n"+
                "       ClientComponentImpl.class Test\n"+
                "*************************************************************");
        
        
        
        try {
            context = (com.acme.TestComponents.CCM_Client_Context)CORBATestManager.instance().
                        getLocalProperty("ClientSessionContext0");
            
            // just the dummy port stuff to ping the component
            assertTrue(context != null);
            
            
            // things are getting exciting here, resolve the LoggerFactory which must have been installed
            // by the QoSLoggingEnabler upfront, log then smth. 
            org.omg.CORBA.Object obj = context.resolve_service_reference("LoggerFactory");
            assertTrue(obj != null);
            LoggerFactory loggerFactory = LoggerFactoryHelper.narrow(obj);                        
            Logger ll = loggerFactory.get_logger("MyPrivateLoggerOffFactory0");
            assertTrue(ll != null);
            ll.debug("logging smth", "mylog.log", 1);
            
            
            context = (com.acme.TestComponents.CCM_Client_Context)CORBATestManager.instance().
            getLocalProperty("ClientSessionContext1");
            
            obj = context.resolve_service_reference("LoggerFactory");
            assertTrue(obj != null);
            loggerFactory = LoggerFactoryHelper.narrow(obj);                        
            ll = loggerFactory.get_logger("MyPrivateLoggerOffFactory1");
            assertTrue(ll != null);
            ll.debug("logging smth", "mylog.log", 1);


                                              
                                              
        } catch (Exception ex) {
            ex.printStackTrace();
            fail();
        }
        
        System.out.println("*************************************************************\n"+
                "       End of ClientComponentImpl.class Test\n"+
                "*************************************************************");
    }
    
   
 
    
}

