/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.testrequestfollowupqosenabler;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import org.junit.Test;
import org.omg.CORBA.LocalObject;
import org.omg.Components.CCMException;
import org.omg.Components.SessionComponent;
import org.omg.Components.SessionContext;

import cdmw.testutils.CORBATestManager;

import com.acme.TestRequestFollowUpQoSEnablerComponents.CCM_ChainLink;
import com.acme.TestRequestFollowUpQoSEnablerComponents.CCM_ChainLink_Context;
import com.acme.TestRequestFollowUpQoSEnablerComponents.CCM_ChainLink_ContextHelper;
import com.acme.TestRequestFollowUpQoSEnablerComponents.CCM_Echo;

import cdmw.logging.Logger;

/**
 *
 */
public class ChainLinkComponentImpl
    extends LocalObject
    implements CCM_ChainLink, CCM_Echo, SessionComponent {

    CCM_ChainLink_Context context = null;

    private static short instanceCounter = 0;

    private short instanceNr = 0;

    private Logger m_logger = null;


    public ChainLinkComponentImpl() {
        m_logger = Logger.Get_Logger("ChainLink");
        CDMW_LOG_FUNCTION(m_logger,"");
        System.out.println("ChainLinkComponentImpl constructor called");
        CDMW_LOG_FUNCTION_RETURN(m_logger);
    }

    public void ccm_activate() throws CCMException {
        CDMW_LOG_FUNCTION(m_logger,"");
        System.out.println("ChainLinkComponentImpl ccm_activate called");
        CDMW_LOG_FUNCTION_RETURN(m_logger);
    }

    public void ccm_passivate() throws CCMException {
        CDMW_LOG_FUNCTION(m_logger,"");
        System.out.println("ChainLinkComponentImpl ccm_passivate called");
        CDMW_LOG_FUNCTION_RETURN(m_logger);
    }

    public void ccm_remove() throws CCMException {
        CDMW_LOG_FUNCTION(m_logger,"");
        System.out.println("ChainLinkComponentImpl ccm_remove called");
        CDMW_LOG_FUNCTION_RETURN(m_logger);
    }

    public void set_session_context(SessionContext ctx) throws CCMException {
        CDMW_LOG_FUNCTION(m_logger,"");
        System.out.println("ChainLinkComponentImpl set_session_context called");
        this.context = CCM_ChainLink_ContextHelper.narrow(ctx);
        CORBATestManager.instance().setLocalProperty ("ctx" + instanceCounter, this.context);
        instanceCounter++;
        this.instanceNr = instanceCounter;
        CDMW_LOG_FUNCTION_RETURN(m_logger);
    }


    @Test
    public void doTests() {
        CDMW_LOG_FUNCTION(m_logger,"");

        System.out.println("*************************************************************\n"+
                "       ChainLinkComponentImpl.class Test\n"+
                "*************************************************************");

        try {
            if(this.instanceNr == 0) {

                int idx_max = 10;
                for (int idx = 1 ; idx <= idx_max ; idx++ ) {
                    System.out.println("Iteration ["+idx+"/"+idx_max+"] starts.");
                    CDMW_INFO(m_logger,"Iteration ["+idx+"/"+idx_max+"] starts.");
                    // set context for doEcho()
                    this.context = (CCM_ChainLink_Context) 
                        CORBATestManager.instance().getLocalProperty("ctx0");
                    this.doEcho(Integer.toString(this.instanceNr));
                    CDMW_INFO(m_logger,"Iteration ["+idx+"/"+idx_max+"] ends.");
                    System.out.println("Iteration ["+idx+"/"+idx_max+"] ends.");
                }
            }

        } catch (Exception ex) {
            ex.printStackTrace();
            fail();
        }

        System.out.println("*************************************************************\n"+
                "       End of ChainLinkComponentImpl.class Test\n"+
                "*************************************************************");
        CDMW_LOG_FUNCTION_RETURN(m_logger);
    }

    public CCM_Echo get_echoProvidesPort() {
        CDMW_LOG_FUNCTION(m_logger,"");
        CDMW_LOG_FUNCTION_RETURN(m_logger);
        return this;
    }

    public String doEcho(String msg) {
        CDMW_LOG_FUNCTION(m_logger,"");
        System.out.println(this.instanceNr + "+ doEcho() received: " + msg);
        if(msg.length() > 23) { // abort condition of recursion, note: msg.length() must not exceed the POA thread pool size!
            CDMW_LOG_FUNCTION_RETURN(m_logger);
            return msg;
        } else {
            msg += this.instanceNr;
            CDMW_LOG_FUNCTION_RETURN(m_logger);
            return this.context.get_connection_echoUsesPort().doEcho(msg);
        }
    }

}
