/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


package cdmw.qosenablers4ccm;

import cdmw.ossupport.OS;
import cdmw.logging.Logger;

public class CCMClientDanceImpl extends org.omg.CORBA.LocalObject
    implements com.acme.QOSEnablersDemo.ClientModule.CCM_ClientDance,
               org.omg.Components.SessionComponent {
    
    private String configuration = "";
    private String expectedServerHostname = "";
    private String hostnameCommandResult = "";

    private com.acme.QOSEnablersDemo.ClientModule.CCM_ClientDance_Context sessionContext = null;
    private com.acme.QOSEnablersDemo.FacetModule.CCM_ClientHostInfo clientHostInfo = new CCMClientHostInfoImpl();

    private Logger logger;
    
    public CCMClientDanceImpl() {
        this.logger = Logger.Get_Logger("ClientDance");
        System.out.println("CCMClientDanceImpl.CCMClientDanceImpl");
    }
    
    //
    // attribute configuration
    //
    public String configuration() {
        return this.configuration;
    }
    public void configuration(String configuration) {
        this.configuration = configuration;
    }

    //
    // attribute expectedServerHostname
    //
    public String expectedServerHostname() {
        return this.expectedServerHostname;
    }
    public void expectedServerHostname(String hostname) {
        this.expectedServerHostname = hostname;
    }
    
    //
    // facet clientHostInfo
    //
    public com.acme.QOSEnablersDemo.FacetModule.CCM_ClientHostInfo get_clientHostInfo() {
        return this.clientHostInfo;
    }
    
    //
    // consumer hostnameResults
    //
    public void push_hostnameResults(com.acme.QOSEnablersDemo.EventTypeModule.HostnameResult ev) {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : push_hostnameResults called with hostname " + ev.hostname, 
            new Throwable());
        this.hostnameCommandResult = ev.hostname;
    }
    
    //
    // supported interface TestableComponent
    // 
    public java.lang.String get_property(java.lang.String prop_name) {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : get_property called", 
            new Throwable());
        String property = "";
        
        if (prop_name.equals(com.acme.QOSEnablersDemo.TestFramework.PROPERTY_GET_HOSTNAME.value)) {
            Logger.CDMW_INFO(
                this.logger,
                configuration + " : PROPERTY_GET_HOSTNAME",
                new Throwable());
            property = expectedServerHostname;
        } else if (prop_name.equals(com.acme.QOSEnablersDemo.TestFramework.PROPERTY_GET_SERVER_HOSTNAME.value)) {
            Logger.CDMW_INFO(
                this.logger,
                configuration + " : PROPERTY_GET_SERVER_HOSTNAME",
                new Throwable());
            try {
                com.acme.QOSEnablersDemo.FacetModule.HostInfo hostInfo =
                    sessionContext.get_connection_serverHostInfo();
                    
                if (hostInfo == null) {
                    Logger.CDMW_ERROR(
                        this.logger,
                        configuration + " : hostInfo is null !!!",
                        new Throwable());
                    throw new org.omg.CORBA.BAD_PARAM();
                }
                
                property = hostInfo.get_hostname();
            } catch (Exception e) {
                e.printStackTrace();
                Logger.CDMW_ERROR(
                    this.logger,
                    configuration + " : Unexpected exception " + e,
                    new Throwable());
            }
        } else {
            Logger.CDMW_INFO(
                this.logger,
                configuration + " : Unknown property",
                new Throwable());
            property = "UnknownProperty";
        }
        
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : get_property returns " + property,
            new Throwable());
            
        return property;
    }

    
    public void reset() {

        try {
            // things are getting exciting here, resolve the LoggerFactory which must have been installed
            // by the QoSLoggingEnabler upfront, log then smth.

            org.omg.CORBA.Object obj =
                sessionContext.resolve_service_reference("LoggerFactory");
            com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory =
                com.thalesgroup.CdmwLogging.LoggerFactoryHelper.narrow(obj);
            com.thalesgroup.CdmwLogging.Logger loggerFromQoSEnabler =
                loggerFactory.get_logger("LoggerFromQoSEnabler");

            // log smth. into the Logger managed by the QoSEnabler
            loggerFromQoSEnabler.debug("going to log smth. into logger from QoSEnabler", "mylog.log", 1);


         } catch (Exception e)  {
            e.printStackTrace();
            throw new RuntimeException();
         } 


        Logger.CDMW_INFO(this.logger,configuration + " : reset called", new Throwable());
        this.hostnameCommandResult = "";
    }
    
    public boolean check(java.lang.String ref) {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : check called, ref[" + ref 
            + "] compare to [" + hostnameCommandResult + "]", 
            new Throwable());

        boolean checkResult = false;
        if (ref.equals(hostnameCommandResult)) {
            Logger.CDMW_INFO(
                this.logger,configuration + " : Server's hostname OK !", 
                new Throwable());
            checkResult = true;
        } else {
            Logger.CDMW_ERROR(
                this.logger,
                configuration + " : Server's hostname is not the one expected: " + expectedServerHostname, 
                new Throwable());
        }
        return checkResult;
    }
    
    public void do_test(int idx) {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : do_test called", 
            new Throwable());
        
        if (idx == 1) {
            com.acme.QOSEnablersDemo.EventTypeModule.Command command = 
                new CommandImpl();
            command.commandName = 
                com.acme.QOSEnablersDemo.EventTypeModule.COMMAND_GET_HOSTNAME.value;
            Logger.CDMW_INFO(
                this.logger,
                configuration + " : --<> emits command >-- " + command.commandName,
                new Throwable());
            sessionContext.push_commandEmits(command);
        } else if (idx == 2) {
            com.acme.QOSEnablersDemo.EventTypeModule.Command command = 
                new CommandImpl();
            command.commandName = 
                com.acme.QOSEnablersDemo.EventTypeModule.COMMAND_CHECK_HOSTNAME.value;
            Logger.CDMW_INFO(
                this.logger,
                configuration + " : --<> emits command >-- " + command.commandName,
                new Throwable());
            sessionContext.push_commandEmits(command);
        }
    }
    
    //
    // Session component specific operations
    //
    public void set_session_context(org.omg.Components.SessionContext ctx) {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : set_session_context called", 
            new Throwable());
        this.sessionContext = 
            com.acme.QOSEnablersDemo.ClientModule.CCM_ClientDance_ContextHelper.narrow(ctx);
    }
    
    public void ccm_activate() {


        Logger.CDMW_INFO(
            this.logger,
            configuration + " : ccm_activate called", 
            new Throwable());
    }
    
    public void ccm_passivate() {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : ccm_passivate called", 
            new Throwable());
    }
    
    public void ccm_remove() {
        Logger.CDMW_INFO(
            this.logger,
            configuration + " : ccm_remove called", 
            new Throwable());
    }
    
    //
    // additional operations
    //
    public com.acme.QOSEnablersDemo.ClientModule.CCM_ClientDance_Context getContext() {
        return this.sessionContext;
    }
    
    public String getHostnameCommandResult() {
        return this.hostnameCommandResult;
    }
}
