/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.qosenablers4ccm;

import cdmw.common.Assert;
import cdmw.testutils.Testable;

public class CCMTestDriverImpl extends org.omg.CORBA.LocalObject
    implements com.acme.QOSEnablersDemo.DriverModule.CCM_TestDriver,
               org.omg.Components.SessionComponent {

    //
    // This inner class provides an object to perform the test
    //
    private class CCMDanceTest extends Testable {
        CCMTestDriverImpl testDriver = null;

        public CCMDanceTest(CCMTestDriverImpl testDriver) {
            this.name = "Test qosenablers4ccm java";
            this.out = System.out;
            this.testDriver = testDriver;

            try {
                println("Sleep 5 sec");
                cdmw.ossupport.OS.sleep(5000);
            } catch (java.lang.InterruptedException e) {
                println("Unexpected exception" + e);
            }
        }
        
        public void doTests() {
            Assert.check(this.testDriver != null);
            try {
                println("Sleep 2 sec");
                cdmw.ossupport.OS.sleep(2000);
            } catch (java.lang.InterruptedException e) {
                println("Unexpected exception" + e);
            }
            boolean testOK = false;
            
            com.acme.QOSEnablersDemo.DriverModule.CCM_TestDriver_Context driverContext = 
                testDriver.getContext();
            Assert.check(driverContext != null);

            println("Get the testable components");
            
            com.acme.QOSEnablersDemo.TestFramework.TestableComponent serverComponent = null;
            com.acme.QOSEnablersDemo.TestFramework.TestableComponent client1Component = null;
            com.acme.QOSEnablersDemo.TestFramework.TestableComponent client2Component = null;
            
            try {
                serverComponent = driverContext.get_connection_server();
                client1Component = driverContext.get_connection_client1();
                client2Component = driverContext.get_connection_client2();
            } catch (Exception e){
                println("Unexpected exception " + e);
            }
            
            Assert.check(serverComponent != null);
            Assert.check(client1Component != null);
            Assert.check(client2Component != null);
        
            setNbOfRequestedTestOK(8);

            println("First test : primaryClient is Client1, secondaryClient is Client2");
            testOK = doTest(serverComponent, client1Component, client2Component);
            println("End of first test.");

            println("Second test : primaryClient is Client2, secondaryClient is Client1");
            testOK = doTest(serverComponent, client2Component, client1Component);
            println("End of second test.");

            if (testOK == true) {
                println("Demo test status is SUCCESS !");
            } else {
                println("Demo test status is FAILED !");
            }
        }
        
        private boolean doTest(
                               com.acme.QOSEnablersDemo.TestFramework.TestableComponent server,
                               com.acme.QOSEnablersDemo.TestFramework.TestableComponent primaryClient,
                               com.acme.QOSEnablersDemo.TestFramework.TestableComponent secondaryClient) {
            boolean testResult = false;
            
            try {
                println("Reseting components");
                server.reset();
                primaryClient.reset();
                secondaryClient.reset();
                
                String clt1Hostname = primaryClient.get_property(
                    com.acme.QOSEnablersDemo.TestFramework.PROPERTY_GET_HOSTNAME.value);
                
                String clt2Hostname = secondaryClient.get_property(
                    com.acme.QOSEnablersDemo.TestFramework.PROPERTY_GET_HOSTNAME.value);
                
                String srvHostname = primaryClient.get_property(
                    com.acme.QOSEnablersDemo.TestFramework.PROPERTY_GET_SERVER_HOSTNAME.value);
                
                println("Compares ["+clt1Hostname+"] to ["+srvHostname+"]");
                if (clt1Hostname.equals(srvHostname)) {
                    println("hostnames are identical");
                    testResult = true;
                } else {
                    println("ERROR : hostname mismatch");
                }
                check(testResult);
                
                testResult = false;
                println("Test primary client's sending getHostname command to the server");
                primaryClient.do_test(1);

                println("Waiting for result...");
                cdmw.ossupport.OS.sleep(2000);
                println("Check primaryClient's result");
                
                boolean bCheck = primaryClient.check(clt1Hostname);
                if (bCheck == true) {
                    println("hostnames are identical");
                    testResult = true;
                } else {
                    println("ERROR : hostname mismatch");
                    cdmw.ossupport.OS.sleep(10000);
                    System.exit(1);
                }
                check(testResult);
                
                testResult = false;
                println("Check secondaryClient's result");
        
                bCheck = secondaryClient.check(clt2Hostname);
                
                if (bCheck == true) {
                    println("hostnames are identical");
                    testResult = true;
                } else {
                    println("ERROR : hostname mismatch");
                }
                check(testResult);
                
                testResult = false;
                println("Test primary client's sending checkHostname command to the server");
                primaryClient.do_test(2);
                
                println("Waiting for result...");
                cdmw.ossupport.OS.sleep(4000);
                println("Check server's result");
        
                bCheck = server.check(clt1Hostname);
        
                if (bCheck == true) {
                    println("hostnames are identical");
                    testResult = true;
                } else {
                    println("ERROR : hostname mismatch");
                }
                check(testResult);

            } catch (Exception e){
                println("Unexpected exception " + e);
                e.printStackTrace();
            }          
            return testResult;
        }
    }
    
    private com.acme.QOSEnablersDemo.DriverModule.CCM_TestDriver_Context sessionContext = null;
    
    public CCMTestDriverImpl() {
        System.out.println("CCMTestDriverImpl.CCMTestDriverImpl");
    }
    
    //
    // Session component specific operations
    //
    public void set_session_context(org.omg.Components.SessionContext ctx) {
        System.out.println("CCMTestDriverImpl.set_session_context");
        this.sessionContext = 
            com.acme.QOSEnablersDemo.DriverModule.CCM_TestDriver_ContextHelper.narrow(ctx);
    }
    public void ccm_activate() {
        System.out.println("CCMTestDriverImpl.ccm_activate");
        final CCMTestDriverImpl thisCCMTestDriverImpl = this;
        Thread activationThread = new Thread(new Runnable() {
                public void run() {
                    CCMDanceTest test = new CCMDanceTest(thisCCMTestDriverImpl);
                    test.start();
                }
            });
        try {
            activationThread.start();
        } catch (IllegalThreadStateException e) {
            // shouldn't happen
            e.printStackTrace();
        }
    }
    
    public void ccm_passivate() {
        System.out.println("CCMTestDriverImpl.ccm_passivate");
    }
    
    public void ccm_remove() {
        System.out.println("CCMTestDriverImpl.ccm_remove");
    }
    
    //
    // additional operations
    //
    public com.acme.QOSEnablersDemo.DriverModule.CCM_TestDriver_Context getContext() {
        return this.sessionContext;
    }

    
}
