/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2009-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package com.prismtech.mosic.radar.ddsfragments.DDS_Update_TargetDistance;

import org.omg.CCM_DDS.AlreadyCreated;
import org.omg.CCM_DDS.DDSEntityKind;
import org.omg.CCM_DDS.InternalError;
import org.omg.CCM_DDS.NoSuchDDSEntity;
import org.omg.CCM_DDS.NonExistent;
import org.omg.CORBA.LocalObject;
import org.omg.CORBA.Object;


import DDS.ANY_SAMPLE_STATE;
import DDS.ANY_VIEW_STATE;
import DDS.LENGTH_UNLIMITED;
import DDS.SampleInfoSeqHolder;

import com.prismtech.mosic.radar.CCM_TargetDistance_Updater;
import com.prismtech.mosic.radar.TargetDistance;
import com.prismtech.mosic.radar.TargetDistanceDataReader;
import com.prismtech.mosic.radar.TargetDistanceDataWriter;
import com.prismtech.mosic.radar.TargetDistanceSeqHolder;

public class Updater_TargetDistanceImpl extends LocalObject implements CCM_TargetDistance_Updater {


    private TargetDistanceDataWriter writer;
    private TargetDistanceDataReader reader;
    private boolean is_lifecycle_checked = true;



    public Updater_TargetDistanceImpl(TargetDistanceDataWriter talker,
            TargetDistanceDataReader targetDistanceReader) {
        this.writer = talker;
        this.reader = targetDistanceReader;
    }


    public void create(TargetDistance an_instance) throws AlreadyCreated, InternalError {

        int status;

        if(this.is_lifecycle_checked) {

            TargetDistanceSeqHolder msgSeq = new TargetDistanceSeqHolder();
            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();

            long handle = reader.lookup_instance(an_instance);

            if(handle != DDS.HANDLE_NIL.value) {

                status = reader.read_instance(
                        msgSeq,
                        infoSeq,
                        LENGTH_UNLIMITED.value, handle,
                        ANY_SAMPLE_STATE.value,
                        ANY_VIEW_STATE.value,
                        DDS.ALIVE_INSTANCE_STATE.value | DDS.NOT_ALIVE_NO_WRITERS_INSTANCE_STATE.value);

                if(status == DDS.RETCODE_OK.value) {
                    throw new AlreadyCreated();
                } else if(status != DDS.RETCODE_NO_DATA.value) {
                    InternalError e = new InternalError();
                    e.error_code = status;
                    throw e;
                }
            }

        }

        status = writer.write(an_instance, DDS.HANDLE_NIL.value);

        if(status != DDS.RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = status;
            throw e;
        }



    }


    public void update(TargetDistance an_instance) throws NonExistent, InternalError {

        int status;

        if(this.is_lifecycle_checked) {

            TargetDistanceSeqHolder msgSeq = new TargetDistanceSeqHolder();
            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();

            long handle = reader.lookup_instance(an_instance);

            if(handle != DDS.HANDLE_NIL.value) {

                status = reader.read_instance(
                        msgSeq,
                        infoSeq,
                        LENGTH_UNLIMITED.value, handle,
                        ANY_SAMPLE_STATE.value,
                        ANY_VIEW_STATE.value,
                        DDS.ALIVE_INSTANCE_STATE.value  | DDS.NOT_ALIVE_NO_WRITERS_INSTANCE_STATE.value);

                if(status == DDS.RETCODE_NO_DATA.value) {
                    throw new NonExistent();
                } else if(status != DDS.RETCODE_OK.value) {
                    InternalError e = new InternalError();
                    e.error_code = status;
                    throw e;
                }

            } else {
                throw new NonExistent();
            }

        }

        status = writer.write(an_instance, DDS.HANDLE_NIL.value);

        if(status != DDS.RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = status;
            throw e;
        }

    }


    public void delete(TargetDistance an_instance) throws NonExistent, InternalError {

        int status;

        if(this.is_lifecycle_checked) {

            TargetDistanceSeqHolder msgSeq = new TargetDistanceSeqHolder();
            SampleInfoSeqHolder infoSeq = new SampleInfoSeqHolder();

            long handle = reader.lookup_instance(an_instance);

            if(handle != DDS.HANDLE_NIL.value) {

                status = reader.read_instance(
                        msgSeq,
                        infoSeq,
                        LENGTH_UNLIMITED.value, handle,
                        ANY_SAMPLE_STATE.value,
                        ANY_VIEW_STATE.value,
                        DDS.ALIVE_INSTANCE_STATE.value  | DDS.NOT_ALIVE_NO_WRITERS_INSTANCE_STATE.value);

                if(status == DDS.RETCODE_NO_DATA.value) {
                    throw new NonExistent();
                } else if(status != DDS.RETCODE_OK.value) {
                    InternalError e = new InternalError();
                    e.error_code = status;
                    throw e;
                }

            } else {
                throw new NonExistent();
            }

        }

        status = writer.dispose(an_instance, DDS.HANDLE_NIL.value);
        writer.unregister_instance(an_instance, DDS.HANDLE_NIL.value);

        if(status != DDS.RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = status;
            throw e;
        }

    }



    public boolean is_lifecycle_checked() {
        return this.is_lifecycle_checked;
    }


    public Object getDDSEntity(DDSEntityKind arg0) throws NoSuchDDSEntity {
        return null;
    }


}
